/* girsourcecomment.c generated by valac, the Vala compiler
 * generated from girsourcecomment.vala, do not modify */

/* sourcecomment.vala
 *
 * Copyright (C) 2011  Florian Brosch
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Florian Brosch <flo.brosch@gmail.com>
 */

#include "valadoc.h"
#include <valagee.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

#define _vala_map_unref0(var) ((var == NULL) ? NULL : (var = (vala_map_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))
#define _valadoc_api_source_comment_unref0(var) ((var == NULL) ? NULL : (var = (valadoc_api_source_comment_unref (var), NULL)))

struct _ValadocApiGirSourceCommentPrivate {
	ValaMap* parameters;
	gchar* _instance_param_name;
	ValadocApiSourceComment* _return_comment;
	ValadocApiSourceComment* _deprecated_comment;
	ValadocApiSourceComment* _version_comment;
	ValadocApiSourceComment* _stability_comment;
};

static gint ValadocApiGirSourceComment_private_offset;
static gpointer valadoc_api_gir_source_comment_parent_class = NULL;

static void valadoc_api_gir_source_comment_finalize (ValadocApiSourceComment * obj);
static GType valadoc_api_gir_source_comment_get_type_once (void);

static inline gpointer
valadoc_api_gir_source_comment_get_instance_private (ValadocApiGirSourceComment* self)
{
	return G_STRUCT_MEMBER_P (self, ValadocApiGirSourceComment_private_offset);
}

const gchar*
valadoc_api_gir_source_comment_get_instance_param_name (ValadocApiGirSourceComment* self)
{
	const gchar* result;
	const gchar* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_instance_param_name;
	result = _tmp0_;
	return result;
}

void
valadoc_api_gir_source_comment_set_instance_param_name (ValadocApiGirSourceComment* self,
                                                        const gchar* value)
{
	gchar* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp0_ = g_strdup (value);
	_g_free0 (self->priv->_instance_param_name);
	self->priv->_instance_param_name = _tmp0_;
}

ValadocApiSourceComment*
valadoc_api_gir_source_comment_get_return_comment (ValadocApiGirSourceComment* self)
{
	ValadocApiSourceComment* result;
	ValadocApiSourceComment* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_return_comment;
	result = _tmp0_;
	return result;
}

static gpointer
_valadoc_api_source_comment_ref0 (gpointer self)
{
	return self ? valadoc_api_source_comment_ref (self) : NULL;
}

void
valadoc_api_gir_source_comment_set_return_comment (ValadocApiGirSourceComment* self,
                                                   ValadocApiSourceComment* value)
{
	ValadocApiSourceComment* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp0_ = _valadoc_api_source_comment_ref0 (value);
	_valadoc_api_source_comment_unref0 (self->priv->_return_comment);
	self->priv->_return_comment = _tmp0_;
}

ValadocApiSourceComment*
valadoc_api_gir_source_comment_get_deprecated_comment (ValadocApiGirSourceComment* self)
{
	ValadocApiSourceComment* result;
	ValadocApiSourceComment* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_deprecated_comment;
	result = _tmp0_;
	return result;
}

void
valadoc_api_gir_source_comment_set_deprecated_comment (ValadocApiGirSourceComment* self,
                                                       ValadocApiSourceComment* value)
{
	ValadocApiSourceComment* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp0_ = _valadoc_api_source_comment_ref0 (value);
	_valadoc_api_source_comment_unref0 (self->priv->_deprecated_comment);
	self->priv->_deprecated_comment = _tmp0_;
}

ValadocApiSourceComment*
valadoc_api_gir_source_comment_get_version_comment (ValadocApiGirSourceComment* self)
{
	ValadocApiSourceComment* result;
	ValadocApiSourceComment* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_version_comment;
	result = _tmp0_;
	return result;
}

void
valadoc_api_gir_source_comment_set_version_comment (ValadocApiGirSourceComment* self,
                                                    ValadocApiSourceComment* value)
{
	ValadocApiSourceComment* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp0_ = _valadoc_api_source_comment_ref0 (value);
	_valadoc_api_source_comment_unref0 (self->priv->_version_comment);
	self->priv->_version_comment = _tmp0_;
}

ValadocApiSourceComment*
valadoc_api_gir_source_comment_get_stability_comment (ValadocApiGirSourceComment* self)
{
	ValadocApiSourceComment* result;
	ValadocApiSourceComment* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_stability_comment;
	result = _tmp0_;
	return result;
}

void
valadoc_api_gir_source_comment_set_stability_comment (ValadocApiGirSourceComment* self,
                                                      ValadocApiSourceComment* value)
{
	ValadocApiSourceComment* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp0_ = _valadoc_api_source_comment_ref0 (value);
	_valadoc_api_source_comment_unref0 (self->priv->_stability_comment);
	self->priv->_stability_comment = _tmp0_;
}

ValaMapIterator*
valadoc_api_gir_source_comment_parameter_iterator (ValadocApiGirSourceComment* self)
{
	ValaMap* _tmp0_;
	ValaMapIterator* _tmp1_;
	ValaMapIterator* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->parameters;
	_tmp1_ = vala_map_map_iterator (_tmp0_);
	result = _tmp1_;
	return result;
}

void
valadoc_api_gir_source_comment_add_parameter_content (ValadocApiGirSourceComment* self,
                                                      const gchar* param_name,
                                                      ValadocApiSourceComment* comment)
{
	ValaMap* _tmp0_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (param_name != NULL);
	g_return_if_fail (comment != NULL);
	_tmp0_ = self->priv->parameters;
	vala_map_set (_tmp0_, param_name, comment);
}

ValadocApiSourceComment*
valadoc_api_gir_source_comment_get_parameter_comment (ValadocApiGirSourceComment* self,
                                                      const gchar* param_name)
{
	ValaMap* _tmp0_;
	ValaMap* _tmp1_;
	gpointer _tmp2_;
	ValadocApiSourceComment* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (param_name != NULL, NULL);
	_tmp0_ = self->priv->parameters;
	if (_tmp0_ == NULL) {
		result = NULL;
		return result;
	}
	_tmp1_ = self->priv->parameters;
	_tmp2_ = vala_map_get (_tmp1_, param_name);
	result = (ValadocApiSourceComment*) _tmp2_;
	return result;
}

ValadocApiGirSourceComment*
valadoc_api_gir_source_comment_construct (GType object_type,
                                          const gchar* content,
                                          ValadocApiSourceFile* file,
                                          gint first_line,
                                          gint first_column,
                                          gint last_line,
                                          gint last_column)
{
	ValadocApiGirSourceComment* self = NULL;
	g_return_val_if_fail (content != NULL, NULL);
	g_return_val_if_fail (file != NULL, NULL);
	self = (ValadocApiGirSourceComment*) valadoc_api_source_comment_construct (object_type, content, file, first_line, first_column, last_line, last_column);
	return self;
}

ValadocApiGirSourceComment*
valadoc_api_gir_source_comment_new (const gchar* content,
                                    ValadocApiSourceFile* file,
                                    gint first_line,
                                    gint first_column,
                                    gint last_line,
                                    gint last_column)
{
	return valadoc_api_gir_source_comment_construct (VALADOC_API_TYPE_GIR_SOURCE_COMMENT, content, file, first_line, first_column, last_line, last_column);
}

static void
valadoc_api_gir_source_comment_class_init (ValadocApiGirSourceCommentClass * klass,
                                           gpointer klass_data)
{
	valadoc_api_gir_source_comment_parent_class = g_type_class_peek_parent (klass);
	((ValadocApiSourceCommentClass *) klass)->finalize = valadoc_api_gir_source_comment_finalize;
	g_type_class_adjust_private_offset (klass, &ValadocApiGirSourceComment_private_offset);
}

static void
valadoc_api_gir_source_comment_instance_init (ValadocApiGirSourceComment * self,
                                              gpointer klass)
{
	GHashFunc _tmp0_;
	GEqualFunc _tmp1_;
	GEqualFunc _tmp2_;
	ValaHashMap* _tmp3_;
	self->priv = valadoc_api_gir_source_comment_get_instance_private (self);
	_tmp0_ = g_str_hash;
	_tmp1_ = g_str_equal;
	_tmp2_ = g_direct_equal;
	_tmp3_ = vala_hash_map_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, VALADOC_API_TYPE_SOURCE_COMMENT, (GBoxedCopyFunc) valadoc_api_source_comment_ref, (GDestroyNotify) valadoc_api_source_comment_unref, _tmp0_, _tmp1_, _tmp2_);
	self->priv->parameters = (ValaMap*) _tmp3_;
}

static void
valadoc_api_gir_source_comment_finalize (ValadocApiSourceComment * obj)
{
	ValadocApiGirSourceComment * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, VALADOC_API_TYPE_GIR_SOURCE_COMMENT, ValadocApiGirSourceComment);
	_vala_map_unref0 (self->priv->parameters);
	_g_free0 (self->priv->_instance_param_name);
	_valadoc_api_source_comment_unref0 (self->priv->_return_comment);
	_valadoc_api_source_comment_unref0 (self->priv->_deprecated_comment);
	_valadoc_api_source_comment_unref0 (self->priv->_version_comment);
	_valadoc_api_source_comment_unref0 (self->priv->_stability_comment);
	VALADOC_API_SOURCE_COMMENT_CLASS (valadoc_api_gir_source_comment_parent_class)->finalize (obj);
}

/**
 * A documentation comment used by valadoc
 */
static GType
valadoc_api_gir_source_comment_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (ValadocApiGirSourceCommentClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valadoc_api_gir_source_comment_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValadocApiGirSourceComment), 0, (GInstanceInitFunc) valadoc_api_gir_source_comment_instance_init, NULL };
	GType valadoc_api_gir_source_comment_type_id;
	valadoc_api_gir_source_comment_type_id = g_type_register_static (VALADOC_API_TYPE_SOURCE_COMMENT, "ValadocApiGirSourceComment", &g_define_type_info, 0);
	ValadocApiGirSourceComment_private_offset = g_type_add_instance_private (valadoc_api_gir_source_comment_type_id, sizeof (ValadocApiGirSourceCommentPrivate));
	return valadoc_api_gir_source_comment_type_id;
}

GType
valadoc_api_gir_source_comment_get_type (void)
{
	static volatile gsize valadoc_api_gir_source_comment_type_id__once = 0;
	if (g_once_init_enter (&valadoc_api_gir_source_comment_type_id__once)) {
		GType valadoc_api_gir_source_comment_type_id;
		valadoc_api_gir_source_comment_type_id = valadoc_api_gir_source_comment_get_type_once ();
		g_once_init_leave (&valadoc_api_gir_source_comment_type_id__once, valadoc_api_gir_source_comment_type_id);
	}
	return valadoc_api_gir_source_comment_type_id__once;
}

