/* field.c generated by valac, the Vala compiler
 * generated from field.vala, do not modify */

/* field.vala
 *
 * Copyright (C) 2008-2011  Florian Brosch
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Florian Brosch <flo.brosch@gmail.com>
 */

#include "valadoc.h"
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <glib-object.h>
#include <vala.h>
#include <valacodegen.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

enum  {
	VALADOC_API_FIELD_0_PROPERTY,
	VALADOC_API_FIELD_FIELD_TYPE_PROPERTY,
	VALADOC_API_FIELD_IS_STATIC_PROPERTY,
	VALADOC_API_FIELD_IS_CLASS_PROPERTY,
	VALADOC_API_FIELD_IS_VOLATILE_PROPERTY,
	VALADOC_API_FIELD_NODE_TYPE_PROPERTY,
	VALADOC_API_FIELD_NUM_PROPERTIES
};
static GParamSpec* valadoc_api_field_properties[VALADOC_API_FIELD_NUM_PROPERTIES];
#define _g_free0(var) (var = (g_free (var), NULL))
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _valadoc_api_signature_builder_unref0(var) ((var == NULL) ? NULL : (var = (valadoc_api_signature_builder_unref (var), NULL)))

struct _ValadocApiFieldPrivate {
	gchar* cname;
	ValadocApiTypeReference* _field_type;
	gboolean _is_static;
	gboolean _is_class;
	gboolean _is_volatile;
};

static gint ValadocApiField_private_offset;
static gpointer valadoc_api_field_parent_class = NULL;

static void valadoc_api_field_set_is_static (ValadocApiField* self,
                                      gboolean value);
static void valadoc_api_field_set_is_class (ValadocApiField* self,
                                     gboolean value);
static void valadoc_api_field_set_is_volatile (ValadocApiField* self,
                                        gboolean value);
static ValadocContentInline* valadoc_api_field_real_build_signature (ValadocApiItem* base);
static void valadoc_api_field_real_accept (ValadocApiNode* base,
                                    ValadocApiVisitor* visitor);
static void valadoc_api_field_finalize (GObject * obj);
static GType valadoc_api_field_get_type_once (void);
static void _vala_valadoc_api_field_get_property (GObject * object,
                                           guint property_id,
                                           GValue * value,
                                           GParamSpec * pspec);
static void _vala_valadoc_api_field_set_property (GObject * object,
                                           guint property_id,
                                           const GValue * value,
                                           GParamSpec * pspec);

static inline gpointer
valadoc_api_field_get_instance_private (ValadocApiField* self)
{
	return G_STRUCT_MEMBER_P (self, ValadocApiField_private_offset);
}

ValadocApiField*
valadoc_api_field_construct (GType object_type,
                             ValadocApiNode* parent,
                             ValadocApiSourceFile* file,
                             const gchar* name,
                             ValaSymbolAccessibility accessibility,
                             ValadocApiSourceComment* comment,
                             ValaField* data)
{
	ValadocApiField * self = NULL;
	gboolean _tmp0_ = FALSE;
	ValaMemberBinding _tmp3_;
	ValaMemberBinding _tmp4_;
	gboolean _tmp5_;
	gboolean _tmp6_;
	gchar* _tmp7_;
	g_return_val_if_fail (parent != NULL, NULL);
	g_return_val_if_fail (file != NULL, NULL);
	g_return_val_if_fail (name != NULL, NULL);
	g_return_val_if_fail (data != NULL, NULL);
	self = (ValadocApiField*) valadoc_api_symbol_construct (object_type, parent, file, name, accessibility, comment, (ValaSymbol*) data);
	if (!VALADOC_API_IS_NAMESPACE (parent)) {
		ValaMemberBinding _tmp1_;
		ValaMemberBinding _tmp2_;
		_tmp1_ = vala_field_get_binding (data);
		_tmp2_ = _tmp1_;
		_tmp0_ = _tmp2_ == VALA_MEMBER_BINDING_STATIC;
	} else {
		_tmp0_ = FALSE;
	}
	valadoc_api_field_set_is_static (self, _tmp0_);
	_tmp3_ = vala_field_get_binding (data);
	_tmp4_ = _tmp3_;
	valadoc_api_field_set_is_class (self, _tmp4_ == VALA_MEMBER_BINDING_CLASS);
	_tmp5_ = vala_field_get_is_volatile (data);
	_tmp6_ = _tmp5_;
	valadoc_api_field_set_is_volatile (self, _tmp6_);
	_tmp7_ = vala_get_ccode_name ((ValaCodeNode*) data);
	_g_free0 (self->priv->cname);
	self->priv->cname = _tmp7_;
	return self;
}

ValadocApiField*
valadoc_api_field_new (ValadocApiNode* parent,
                       ValadocApiSourceFile* file,
                       const gchar* name,
                       ValaSymbolAccessibility accessibility,
                       ValadocApiSourceComment* comment,
                       ValaField* data)
{
	return valadoc_api_field_construct (VALADOC_API_TYPE_FIELD, parent, file, name, accessibility, comment, data);
}

/**
 * Returns the name of this field as it is used in C.
 */
gchar*
valadoc_api_field_get_cname (ValadocApiField* self)
{
	const gchar* _tmp0_;
	gchar* _tmp1_;
	gchar* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->cname;
	_tmp1_ = g_strdup (_tmp0_);
	result = _tmp1_;
	return result;
}

ValadocApiTypeReference*
valadoc_api_field_get_field_type (ValadocApiField* self)
{
	ValadocApiTypeReference* result;
	ValadocApiTypeReference* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_field_type;
	result = _tmp0_;
	return result;
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

void
valadoc_api_field_set_field_type (ValadocApiField* self,
                                  ValadocApiTypeReference* value)
{
	ValadocApiTypeReference* old_value;
	g_return_if_fail (self != NULL);
	old_value = valadoc_api_field_get_field_type (self);
	if (old_value != value) {
		ValadocApiTypeReference* _tmp0_;
		_tmp0_ = _g_object_ref0 (value);
		_g_object_unref0 (self->priv->_field_type);
		self->priv->_field_type = _tmp0_;
		g_object_notify_by_pspec ((GObject *) self, valadoc_api_field_properties[VALADOC_API_FIELD_FIELD_TYPE_PROPERTY]);
	}
}

gboolean
valadoc_api_field_get_is_static (ValadocApiField* self)
{
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = self->priv->_is_static;
	return result;
}

static void
valadoc_api_field_set_is_static (ValadocApiField* self,
                                 gboolean value)
{
	gboolean old_value;
	g_return_if_fail (self != NULL);
	old_value = valadoc_api_field_get_is_static (self);
	if (old_value != value) {
		self->priv->_is_static = value;
		g_object_notify_by_pspec ((GObject *) self, valadoc_api_field_properties[VALADOC_API_FIELD_IS_STATIC_PROPERTY]);
	}
}

gboolean
valadoc_api_field_get_is_class (ValadocApiField* self)
{
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = self->priv->_is_class;
	return result;
}

static void
valadoc_api_field_set_is_class (ValadocApiField* self,
                                gboolean value)
{
	gboolean old_value;
	g_return_if_fail (self != NULL);
	old_value = valadoc_api_field_get_is_class (self);
	if (old_value != value) {
		self->priv->_is_class = value;
		g_object_notify_by_pspec ((GObject *) self, valadoc_api_field_properties[VALADOC_API_FIELD_IS_CLASS_PROPERTY]);
	}
}

gboolean
valadoc_api_field_get_is_volatile (ValadocApiField* self)
{
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = self->priv->_is_volatile;
	return result;
}

static void
valadoc_api_field_set_is_volatile (ValadocApiField* self,
                                   gboolean value)
{
	gboolean old_value;
	g_return_if_fail (self != NULL);
	old_value = valadoc_api_field_get_is_volatile (self);
	if (old_value != value) {
		self->priv->_is_volatile = value;
		g_object_notify_by_pspec ((GObject *) self, valadoc_api_field_properties[VALADOC_API_FIELD_IS_VOLATILE_PROPERTY]);
	}
}

/**
 * {@inheritDoc}
 */
static ValadocContentInline*
valadoc_api_field_real_build_signature (ValadocApiItem* base)
{
	ValadocApiField * self;
	ValadocApiSignatureBuilder* signature = NULL;
	ValadocApiSignatureBuilder* _tmp0_;
	ValadocApiSignatureBuilder* _tmp1_;
	ValaSymbolAccessibility _tmp2_;
	ValaSymbolAccessibility _tmp3_;
	const gchar* _tmp4_;
	gboolean _tmp5_;
	gboolean _tmp9_;
	ValadocApiSignatureBuilder* _tmp11_;
	ValadocApiTypeReference* _tmp12_;
	ValadocContentInline* _tmp13_;
	ValadocContentInline* _tmp14_;
	ValadocApiSignatureBuilder* _tmp15_;
	ValadocApiSignatureBuilder* _tmp16_;
	ValadocContentRun* _tmp17_;
	ValadocContentInline* result;
	self = (ValadocApiField*) base;
	_tmp0_ = valadoc_api_signature_builder_new ();
	signature = _tmp0_;
	_tmp1_ = signature;
	_tmp2_ = valadoc_api_symbol_get_accessibility ((ValadocApiSymbol*) self);
	_tmp3_ = _tmp2_;
	_tmp4_ = vala_symbol_accessibility_to_string (_tmp3_);
	valadoc_api_signature_builder_append_keyword (_tmp1_, _tmp4_, TRUE);
	_tmp5_ = self->priv->_is_static;
	if (_tmp5_) {
		ValadocApiSignatureBuilder* _tmp6_;
		_tmp6_ = signature;
		valadoc_api_signature_builder_append_keyword (_tmp6_, "static", TRUE);
	} else {
		gboolean _tmp7_;
		_tmp7_ = self->priv->_is_class;
		if (_tmp7_) {
			ValadocApiSignatureBuilder* _tmp8_;
			_tmp8_ = signature;
			valadoc_api_signature_builder_append_keyword (_tmp8_, "class", TRUE);
		}
	}
	_tmp9_ = self->priv->_is_volatile;
	if (_tmp9_) {
		ValadocApiSignatureBuilder* _tmp10_;
		_tmp10_ = signature;
		valadoc_api_signature_builder_append_keyword (_tmp10_, "volatile", TRUE);
	}
	_tmp11_ = signature;
	_tmp12_ = self->priv->_field_type;
	_tmp13_ = valadoc_api_item_get_signature ((ValadocApiItem*) _tmp12_);
	_tmp14_ = _tmp13_;
	valadoc_api_signature_builder_append_content (_tmp11_, _tmp14_, TRUE);
	_tmp15_ = signature;
	valadoc_api_signature_builder_append_symbol (_tmp15_, (ValadocApiNode*) self, TRUE);
	_tmp16_ = signature;
	_tmp17_ = valadoc_api_signature_builder_get (_tmp16_);
	result = (ValadocContentInline*) _tmp17_;
	_valadoc_api_signature_builder_unref0 (signature);
	return result;
}

static ValadocApiNodeType
valadoc_api_field_real_get_node_type (ValadocApiNode* base)
{
	ValadocApiNodeType result;
	ValadocApiField* self;
	self = (ValadocApiField*) base;
	result = VALADOC_API_NODE_TYPE_FIELD;
	return result;
}

/**
 * {@inheritDoc}
 */
static void
valadoc_api_field_real_accept (ValadocApiNode* base,
                               ValadocApiVisitor* visitor)
{
	ValadocApiField * self;
	self = (ValadocApiField*) base;
	g_return_if_fail (visitor != NULL);
	valadoc_api_visitor_visit_field (visitor, self);
}

static void
valadoc_api_field_class_init (ValadocApiFieldClass * klass,
                              gpointer klass_data)
{
	valadoc_api_field_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &ValadocApiField_private_offset);
	((ValadocApiItemClass *) klass)->build_signature = (ValadocContentInline* (*) (ValadocApiItem*)) valadoc_api_field_real_build_signature;
	((ValadocApiNodeClass *) klass)->accept = (void (*) (ValadocApiNode*, ValadocApiVisitor*)) valadoc_api_field_real_accept;
	VALADOC_API_NODE_CLASS (klass)->get_node_type = (ValadocApiNodeType (*) (ValadocApiNode*)) valadoc_api_field_real_get_node_type;
	G_OBJECT_CLASS (klass)->get_property = _vala_valadoc_api_field_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_valadoc_api_field_set_property;
	G_OBJECT_CLASS (klass)->finalize = valadoc_api_field_finalize;
	/**
	 * The field type.
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALADOC_API_FIELD_FIELD_TYPE_PROPERTY, valadoc_api_field_properties[VALADOC_API_FIELD_FIELD_TYPE_PROPERTY] = g_param_spec_object ("field-type", "field-type", "field-type", VALADOC_API_TYPE_TYPEREFERENCE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
	/**
	 * Specifies whether the field is static.
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALADOC_API_FIELD_IS_STATIC_PROPERTY, valadoc_api_field_properties[VALADOC_API_FIELD_IS_STATIC_PROPERTY] = g_param_spec_boolean ("is-static", "is-static", "is-static", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	/**
	 * Specifies whether this field is a class field.
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALADOC_API_FIELD_IS_CLASS_PROPERTY, valadoc_api_field_properties[VALADOC_API_FIELD_IS_CLASS_PROPERTY] = g_param_spec_boolean ("is-class", "is-class", "is-class", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	/**
	 * Specifies whether the field is volatile.
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALADOC_API_FIELD_IS_VOLATILE_PROPERTY, valadoc_api_field_properties[VALADOC_API_FIELD_IS_VOLATILE_PROPERTY] = g_param_spec_boolean ("is-volatile", "is-volatile", "is-volatile", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	/**
	 * {@inheritDoc}
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALADOC_API_FIELD_NODE_TYPE_PROPERTY, valadoc_api_field_properties[VALADOC_API_FIELD_NODE_TYPE_PROPERTY] = g_param_spec_enum ("node-type", "node-type", "node-type", VALADOC_API_TYPE_NODE_TYPE, 0, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
}

static void
valadoc_api_field_instance_init (ValadocApiField * self,
                                 gpointer klass)
{
	self->priv = valadoc_api_field_get_instance_private (self);
}

static void
valadoc_api_field_finalize (GObject * obj)
{
	ValadocApiField * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, VALADOC_API_TYPE_FIELD, ValadocApiField);
	_g_free0 (self->priv->cname);
	_g_object_unref0 (self->priv->_field_type);
	G_OBJECT_CLASS (valadoc_api_field_parent_class)->finalize (obj);
}

/**
 * Represents a field.
 */
static GType
valadoc_api_field_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (ValadocApiFieldClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valadoc_api_field_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValadocApiField), 0, (GInstanceInitFunc) valadoc_api_field_instance_init, NULL };
	GType valadoc_api_field_type_id;
	valadoc_api_field_type_id = g_type_register_static (VALADOC_API_TYPE_SYMBOL, "ValadocApiField", &g_define_type_info, 0);
	ValadocApiField_private_offset = g_type_add_instance_private (valadoc_api_field_type_id, sizeof (ValadocApiFieldPrivate));
	return valadoc_api_field_type_id;
}

GType
valadoc_api_field_get_type (void)
{
	static volatile gsize valadoc_api_field_type_id__once = 0;
	if (g_once_init_enter (&valadoc_api_field_type_id__once)) {
		GType valadoc_api_field_type_id;
		valadoc_api_field_type_id = valadoc_api_field_get_type_once ();
		g_once_init_leave (&valadoc_api_field_type_id__once, valadoc_api_field_type_id);
	}
	return valadoc_api_field_type_id__once;
}

static void
_vala_valadoc_api_field_get_property (GObject * object,
                                      guint property_id,
                                      GValue * value,
                                      GParamSpec * pspec)
{
	ValadocApiField * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, VALADOC_API_TYPE_FIELD, ValadocApiField);
	switch (property_id) {
		case VALADOC_API_FIELD_FIELD_TYPE_PROPERTY:
		g_value_set_object (value, valadoc_api_field_get_field_type (self));
		break;
		case VALADOC_API_FIELD_IS_STATIC_PROPERTY:
		g_value_set_boolean (value, valadoc_api_field_get_is_static (self));
		break;
		case VALADOC_API_FIELD_IS_CLASS_PROPERTY:
		g_value_set_boolean (value, valadoc_api_field_get_is_class (self));
		break;
		case VALADOC_API_FIELD_IS_VOLATILE_PROPERTY:
		g_value_set_boolean (value, valadoc_api_field_get_is_volatile (self));
		break;
		case VALADOC_API_FIELD_NODE_TYPE_PROPERTY:
		g_value_set_enum (value, valadoc_api_node_get_node_type ((ValadocApiNode*) self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_valadoc_api_field_set_property (GObject * object,
                                      guint property_id,
                                      const GValue * value,
                                      GParamSpec * pspec)
{
	ValadocApiField * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, VALADOC_API_TYPE_FIELD, ValadocApiField);
	switch (property_id) {
		case VALADOC_API_FIELD_FIELD_TYPE_PROPERTY:
		valadoc_api_field_set_field_type (self, g_value_get_object (value));
		break;
		case VALADOC_API_FIELD_IS_STATIC_PROPERTY:
		valadoc_api_field_set_is_static (self, g_value_get_boolean (value));
		break;
		case VALADOC_API_FIELD_IS_CLASS_PROPERTY:
		valadoc_api_field_set_is_class (self, g_value_get_boolean (value));
		break;
		case VALADOC_API_FIELD_IS_VOLATILE_PROPERTY:
		valadoc_api_field_set_is_volatile (self, g_value_get_boolean (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

