/* manyrule.c generated by valac, the Vala compiler
 * generated from manyrule.vala, do not modify */

/* manyrule.vala
 *
 * Copyright (C) 2008-2009 Florian Brosch, Didier Villevalois
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Didier 'Ptitjes Villevalois <ptitjes@free.fr>
 */

#include "valadoc.h"
#include <glib-object.h>
#include <glib.h>
#include <stdlib.h>
#include <string.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

#define VALADOC_TYPE_MANY_RULE (valadoc_many_rule_get_type ())
#define VALADOC_MANY_RULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALADOC_TYPE_MANY_RULE, ValadocManyRule))
#define VALADOC_MANY_RULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALADOC_TYPE_MANY_RULE, ValadocManyRuleClass))
#define VALADOC_IS_MANY_RULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALADOC_TYPE_MANY_RULE))
#define VALADOC_IS_MANY_RULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALADOC_TYPE_MANY_RULE))
#define VALADOC_MANY_RULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALADOC_TYPE_MANY_RULE, ValadocManyRuleClass))

typedef struct _ValadocManyRule ValadocManyRule;
typedef struct _ValadocManyRuleClass ValadocManyRuleClass;
typedef struct _ValadocManyRulePrivate ValadocManyRulePrivate;
enum  {
	VALADOC_MANY_RULE_0_PROPERTY,
	VALADOC_MANY_RULE_NUM_PROPERTIES
};
static GParamSpec* valadoc_many_rule_properties[VALADOC_MANY_RULE_NUM_PROPERTIES];
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

#define VALADOC_MANY_RULE_TYPE_STATE (valadoc_many_rule_state_get_type ())
#define VALADOC_MANY_RULE_STATE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALADOC_MANY_RULE_TYPE_STATE, ValadocManyRuleState))
#define VALADOC_MANY_RULE_STATE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALADOC_MANY_RULE_TYPE_STATE, ValadocManyRuleStateClass))
#define VALADOC_MANY_RULE_IS_STATE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALADOC_MANY_RULE_TYPE_STATE))
#define VALADOC_MANY_RULE_IS_STATE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALADOC_MANY_RULE_TYPE_STATE))
#define VALADOC_MANY_RULE_STATE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALADOC_MANY_RULE_TYPE_STATE, ValadocManyRuleStateClass))

typedef struct _ValadocManyRuleState ValadocManyRuleState;
typedef struct _ValadocManyRuleStateClass ValadocManyRuleStateClass;
typedef struct _ValadocManyRuleStatePrivate ValadocManyRuleStatePrivate;
#define _g_free0(var) (var = (g_free (var), NULL))
enum  {
	VALADOC_MANY_RULE_STATE_0_PROPERTY,
	VALADOC_MANY_RULE_STATE_NUM_PROPERTIES
};
static GParamSpec* valadoc_many_rule_state_properties[VALADOC_MANY_RULE_STATE_NUM_PROPERTIES];

struct _ValadocManyRule {
	ValadocRule parent_instance;
	ValadocManyRulePrivate * priv;
};

struct _ValadocManyRuleClass {
	ValadocRuleClass parent_class;
};

struct _ValadocManyRulePrivate {
	GObject* _scheme;
};

struct _ValadocManyRuleState {
	GObject parent_instance;
	gboolean started;
	gboolean done_one;
	ValadocManyRuleStatePrivate * priv;
};

struct _ValadocManyRuleStateClass {
	GObjectClass parent_class;
};

static gint ValadocManyRule_private_offset;
static gpointer valadoc_many_rule_parent_class = NULL;
static gpointer valadoc_many_rule_state_parent_class = NULL;

 G_GNUC_INTERNAL GType valadoc_many_rule_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
 G_GNUC_INTERNAL ValadocManyRule* valadoc_many_rule_new (GObject* scheme);
 G_GNUC_INTERNAL ValadocManyRule* valadoc_many_rule_construct (GType object_type,
                                              GObject* scheme);
static gboolean valadoc_many_rule_real_is_optional (ValadocRule* base);
static gboolean valadoc_many_rule_real_starts_with_token (ValadocRule* base,
                                                   ValadocToken* token);
static gboolean valadoc_many_rule_real_accept_token (ValadocRule* base,
                                              ValadocToken* token,
                                              ValadocParserCallback* parser,
                                              ValadocRuleForward forward,
                                              GError** error);
static GType valadoc_many_rule_state_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
static ValadocManyRuleState* valadoc_many_rule_state_new (void);
static ValadocManyRuleState* valadoc_many_rule_state_construct (GType object_type);
static gboolean valadoc_many_rule_real_would_accept_token (ValadocRule* base,
                                                    ValadocToken* token,
                                                    GObject* state);
static gboolean valadoc_many_rule_real_would_reduce (ValadocRule* base,
                                              ValadocToken* token,
                                              GObject* rule_state);
static gchar* valadoc_many_rule_real_to_string (ValadocRule* base,
                                         GObject* rule_state);
static void valadoc_many_rule_state_finalize (GObject * obj);
static GType valadoc_many_rule_state_get_type_once (void);
static void valadoc_many_rule_finalize (GObject * obj);
static GType valadoc_many_rule_get_type_once (void);

static inline gpointer
valadoc_many_rule_get_instance_private (ValadocManyRule* self)
{
	return G_STRUCT_MEMBER_P (self, ValadocManyRule_private_offset);
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

 G_GNUC_INTERNAL ValadocManyRule*
valadoc_many_rule_construct (GType object_type,
                             GObject* scheme)
{
	ValadocManyRule * self = NULL;
	GObject* _tmp0_;
	g_return_val_if_fail (scheme != NULL, NULL);
	self = (ValadocManyRule*) valadoc_rule_construct (object_type);
	_tmp0_ = _g_object_ref0 (scheme);
	_g_object_unref0 (self->priv->_scheme);
	self->priv->_scheme = _tmp0_;
	return self;
}

 G_GNUC_INTERNAL ValadocManyRule*
valadoc_many_rule_new (GObject* scheme)
{
	return valadoc_many_rule_construct (VALADOC_TYPE_MANY_RULE, scheme);
}

static gboolean
valadoc_many_rule_real_is_optional (ValadocRule* base)
{
	ValadocManyRule * self;
	GObject* _tmp0_;
	gboolean result;
	self = (ValadocManyRule*) base;
	_tmp0_ = self->priv->_scheme;
	result = valadoc_rule_is_optional_rule ((ValadocRule*) self, _tmp0_);
	return result;
}

static gboolean
valadoc_many_rule_real_starts_with_token (ValadocRule* base,
                                          ValadocToken* token)
{
	ValadocManyRule * self;
	GObject* _tmp0_;
	gboolean result;
	self = (ValadocManyRule*) base;
	g_return_val_if_fail (token != NULL, FALSE);
	_tmp0_ = self->priv->_scheme;
	if (valadoc_rule_has_start_token ((ValadocRule*) self, _tmp0_, token)) {
		result = TRUE;
		return result;
	}
	result = FALSE;
	return result;
}

static gboolean
valadoc_many_rule_real_accept_token (ValadocRule* base,
                                     ValadocToken* token,
                                     ValadocParserCallback* parser,
                                     ValadocRuleForward forward,
                                     GError** error)
{
	ValadocManyRule * self;
	ValadocManyRuleState* state = NULL;
	GObject* _tmp0_;
	ValadocManyRuleState* _tmp1_;
	ValadocManyRuleState* _tmp2_;
	ValadocManyRuleState* _tmp5_;
	gboolean _tmp9_ = FALSE;
	ValadocManyRuleState* _tmp10_;
	gboolean handled = FALSE;
	gboolean _tmp15_ = FALSE;
	GObject* _tmp16_;
	gboolean _tmp17_ = FALSE;
	gboolean _tmp18_;
	GObject* _tmp24_;
	GError* _inner_error0_ = NULL;
	gboolean result;
	self = (ValadocManyRule*) base;
	g_return_val_if_fail (token != NULL, FALSE);
	g_return_val_if_fail (parser != NULL, FALSE);
	_tmp0_ = valadoc_parser_callback_get_rule_state (parser);
	_tmp1_ = VALADOC_MANY_RULE_IS_STATE (_tmp0_) ? ((ValadocManyRuleState*) _tmp0_) : NULL;
	if (_tmp1_ == NULL) {
		_g_object_unref0 (_tmp0_);
	}
	state = _tmp1_;
	_tmp2_ = state;
	if (_tmp2_ == NULL) {
		ValadocManyRuleState* _tmp3_;
		ValadocManyRuleState* _tmp4_;
		_tmp3_ = valadoc_many_rule_state_new ();
		_g_object_unref0 (state);
		state = _tmp3_;
		_tmp4_ = state;
		valadoc_parser_callback_set_rule_state (parser, (GObject*) _tmp4_);
	}
	_tmp5_ = state;
	if (!_tmp5_->started) {
		ValadocManyRuleState* _tmp8_;
		valadoc_rule_do_start ((ValadocRule*) self, parser, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALADOC_PARSER_ERROR) {
				gboolean _tmp6_ = FALSE;
				g_propagate_error (error, _inner_error0_);
				_g_object_unref0 (state);
				return _tmp6_;
			} else {
				gboolean _tmp7_ = FALSE;
				_g_object_unref0 (state);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return _tmp7_;
			}
		}
		_tmp8_ = state;
		_tmp8_->started = TRUE;
	}
	_tmp10_ = state;
	if (_tmp10_->done_one) {
		_tmp9_ = valadoc_parser_callback_would_parent_accept_token (parser, token);
	} else {
		_tmp9_ = FALSE;
	}
	if (_tmp9_) {
		valadoc_rule_do_reduce ((ValadocRule*) self, parser, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALADOC_PARSER_ERROR) {
				gboolean _tmp11_ = FALSE;
				g_propagate_error (error, _inner_error0_);
				_g_object_unref0 (state);
				return _tmp11_;
			} else {
				gboolean _tmp12_ = FALSE;
				_g_object_unref0 (state);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return _tmp12_;
			}
		}
		result = FALSE;
		_g_object_unref0 (state);
		return result;
	}
	if (valadoc_parser_callback_would_parent_reduce_to_rule (parser, token, (ValadocRule*) self)) {
		valadoc_rule_do_reduce ((ValadocRule*) self, parser, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALADOC_PARSER_ERROR) {
				gboolean _tmp13_ = FALSE;
				g_propagate_error (error, _inner_error0_);
				_g_object_unref0 (state);
				return _tmp13_;
			} else {
				gboolean _tmp14_ = FALSE;
				_g_object_unref0 (state);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return _tmp14_;
			}
		}
		result = FALSE;
		_g_object_unref0 (state);
		return result;
	}
	_tmp16_ = self->priv->_scheme;
	_tmp18_ = valadoc_rule_try_to_apply ((ValadocRule*) self, _tmp16_, token, parser, &_tmp17_, &_inner_error0_);
	handled = _tmp17_;
	_tmp15_ = _tmp18_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALADOC_PARSER_ERROR) {
			gboolean _tmp19_ = FALSE;
			g_propagate_error (error, _inner_error0_);
			_g_object_unref0 (state);
			return _tmp19_;
		} else {
			gboolean _tmp20_ = FALSE;
			_g_object_unref0 (state);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return _tmp20_;
		}
	}
	if (_tmp15_) {
		ValadocManyRuleState* _tmp21_;
		_tmp21_ = state;
		_tmp21_->done_one = TRUE;
		result = handled;
		_g_object_unref0 (state);
		return result;
	}
	if (valadoc_parser_callback_would_parent_accept_token (parser, token)) {
		valadoc_rule_do_reduce ((ValadocRule*) self, parser, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALADOC_PARSER_ERROR) {
				gboolean _tmp22_ = FALSE;
				g_propagate_error (error, _inner_error0_);
				_g_object_unref0 (state);
				return _tmp22_;
			} else {
				gboolean _tmp23_ = FALSE;
				_g_object_unref0 (state);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return _tmp23_;
			}
		}
		result = FALSE;
		_g_object_unref0 (state);
		return result;
	}
	_tmp24_ = self->priv->_scheme;
	if (VALADOC_IS_TOKEN_TYPE (_tmp24_)) {
		GObject* _tmp25_;
		const gchar* _tmp26_;
		gchar* _tmp27_;
		gchar* _tmp28_;
		_tmp25_ = self->priv->_scheme;
		_tmp26_ = valadoc_token_type_to_pretty_string (G_TYPE_CHECK_INSTANCE_CAST (_tmp25_, VALADOC_TYPE_TOKEN_TYPE, ValadocTokenType));
		_tmp27_ = g_strdup_printf ("expected %s", _tmp26_);
		_tmp28_ = _tmp27_;
		valadoc_parser_callback_error (parser, NULL, _tmp28_, &_inner_error0_);
		_g_free0 (_tmp28_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALADOC_PARSER_ERROR) {
				gboolean _tmp29_ = FALSE;
				g_propagate_error (error, _inner_error0_);
				_g_object_unref0 (state);
				return _tmp29_;
			} else {
				gboolean _tmp30_ = FALSE;
				_g_object_unref0 (state);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return _tmp30_;
			}
		}
	} else {
		valadoc_parser_callback_error (parser, token, "unexpected token", &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALADOC_PARSER_ERROR) {
				gboolean _tmp31_ = FALSE;
				g_propagate_error (error, _inner_error0_);
				_g_object_unref0 (state);
				return _tmp31_;
			} else {
				gboolean _tmp32_ = FALSE;
				_g_object_unref0 (state);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return _tmp32_;
			}
		}
	}
	g_assert_not_reached ();
}

static gboolean
valadoc_many_rule_real_would_accept_token (ValadocRule* base,
                                           ValadocToken* token,
                                           GObject* state)
{
	ValadocManyRule * self;
	GObject* _tmp0_;
	gboolean result;
	self = (ValadocManyRule*) base;
	g_return_val_if_fail (token != NULL, FALSE);
	_tmp0_ = self->priv->_scheme;
	if (valadoc_rule_has_start_token ((ValadocRule*) self, _tmp0_, token)) {
		result = TRUE;
		return result;
	}
	result = FALSE;
	return result;
}

static gboolean
valadoc_many_rule_real_would_reduce (ValadocRule* base,
                                     ValadocToken* token,
                                     GObject* rule_state)
{
	ValadocManyRule * self;
	ValadocManyRuleState* state = NULL;
	ValadocManyRuleState* _tmp0_;
	gboolean _tmp1_ = FALSE;
	ValadocManyRuleState* _tmp2_;
	gboolean result;
	self = (ValadocManyRule*) base;
	g_return_val_if_fail (token != NULL, FALSE);
	_tmp0_ = _g_object_ref0 (VALADOC_MANY_RULE_IS_STATE (rule_state) ? ((ValadocManyRuleState*) rule_state) : NULL);
	state = _tmp0_;
	_tmp2_ = state;
	if (_tmp2_->done_one) {
		_tmp1_ = TRUE;
	} else {
		GObject* _tmp3_;
		_tmp3_ = self->priv->_scheme;
		_tmp1_ = valadoc_rule_is_optional_rule ((ValadocRule*) self, _tmp3_);
	}
	result = _tmp1_;
	_g_object_unref0 (state);
	return result;
}

static gchar*
bool_to_string (gboolean self)
{
	gchar* result;
	if (self) {
		gchar* _tmp0_;
		_tmp0_ = g_strdup ("true");
		result = _tmp0_;
		return result;
	} else {
		gchar* _tmp1_;
		_tmp1_ = g_strdup ("false");
		result = _tmp1_;
		return result;
	}
}

static gchar*
valadoc_many_rule_real_to_string (ValadocRule* base,
                                  GObject* rule_state)
{
	ValadocManyRule * self;
	ValadocManyRuleState* state = NULL;
	ValadocManyRuleState* _tmp0_;
	ValadocManyRuleState* _tmp1_;
	const gchar* _tmp3_ = NULL;
	const gchar* _tmp4_;
	const gchar* _tmp5_;
	ValadocManyRuleState* _tmp8_;
	gchar* _tmp9_;
	gchar* _tmp10_;
	ValadocManyRuleState* _tmp11_;
	gchar* _tmp12_;
	gchar* _tmp13_;
	gchar* _tmp14_;
	gchar* _tmp15_;
	gchar* result;
	self = (ValadocManyRule*) base;
	_tmp0_ = _g_object_ref0 (VALADOC_MANY_RULE_IS_STATE (rule_state) ? ((ValadocManyRuleState*) rule_state) : NULL);
	state = _tmp0_;
	_tmp1_ = state;
	if (_tmp1_ == NULL) {
		ValadocManyRuleState* _tmp2_;
		_tmp2_ = valadoc_many_rule_state_new ();
		_g_object_unref0 (state);
		state = _tmp2_;
	}
	_tmp4_ = valadoc_rule_get_name ((ValadocRule*) self);
	_tmp5_ = _tmp4_;
	if (_tmp5_ != NULL) {
		const gchar* _tmp6_;
		const gchar* _tmp7_;
		_tmp6_ = valadoc_rule_get_name ((ValadocRule*) self);
		_tmp7_ = _tmp6_;
		_tmp3_ = _tmp7_;
	} else {
		_tmp3_ = " ";
	}
	_tmp8_ = state;
	_tmp9_ = bool_to_string (_tmp8_->started);
	_tmp10_ = _tmp9_;
	_tmp11_ = state;
	_tmp12_ = bool_to_string (_tmp11_->done_one);
	_tmp13_ = _tmp12_;
	_tmp14_ = g_strdup_printf ("%-15s%-15s(started=%s;done_one=%s)", _tmp3_, "[many]", _tmp10_, _tmp13_);
	_tmp15_ = _tmp14_;
	_g_free0 (_tmp13_);
	_g_free0 (_tmp10_);
	result = _tmp15_;
	_g_object_unref0 (state);
	return result;
}

static ValadocManyRuleState*
valadoc_many_rule_state_construct (GType object_type)
{
	ValadocManyRuleState * self = NULL;
	self = (ValadocManyRuleState*) g_object_new (object_type, NULL);
	return self;
}

static ValadocManyRuleState*
valadoc_many_rule_state_new (void)
{
	return valadoc_many_rule_state_construct (VALADOC_MANY_RULE_TYPE_STATE);
}

static void
valadoc_many_rule_state_class_init (ValadocManyRuleStateClass * klass,
                                    gpointer klass_data)
{
	valadoc_many_rule_state_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->finalize = valadoc_many_rule_state_finalize;
}

static void
valadoc_many_rule_state_instance_init (ValadocManyRuleState * self,
                                       gpointer klass)
{
	self->started = FALSE;
	self->done_one = FALSE;
}

static void
valadoc_many_rule_state_finalize (GObject * obj)
{
	ValadocManyRuleState * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, VALADOC_MANY_RULE_TYPE_STATE, ValadocManyRuleState);
	G_OBJECT_CLASS (valadoc_many_rule_state_parent_class)->finalize (obj);
}

static GType
valadoc_many_rule_state_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (ValadocManyRuleStateClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valadoc_many_rule_state_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValadocManyRuleState), 0, (GInstanceInitFunc) valadoc_many_rule_state_instance_init, NULL };
	GType valadoc_many_rule_state_type_id;
	valadoc_many_rule_state_type_id = g_type_register_static (G_TYPE_OBJECT, "ValadocManyRuleState", &g_define_type_info, 0);
	return valadoc_many_rule_state_type_id;
}

static GType
valadoc_many_rule_state_get_type (void)
{
	static volatile gsize valadoc_many_rule_state_type_id__once = 0;
	if (g_once_init_enter (&valadoc_many_rule_state_type_id__once)) {
		GType valadoc_many_rule_state_type_id;
		valadoc_many_rule_state_type_id = valadoc_many_rule_state_get_type_once ();
		g_once_init_leave (&valadoc_many_rule_state_type_id__once, valadoc_many_rule_state_type_id);
	}
	return valadoc_many_rule_state_type_id__once;
}

static void
valadoc_many_rule_class_init (ValadocManyRuleClass * klass,
                              gpointer klass_data)
{
	valadoc_many_rule_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &ValadocManyRule_private_offset);
	((ValadocRuleClass *) klass)->is_optional = (gboolean (*) (ValadocRule*)) valadoc_many_rule_real_is_optional;
	((ValadocRuleClass *) klass)->starts_with_token = (gboolean (*) (ValadocRule*, ValadocToken*)) valadoc_many_rule_real_starts_with_token;
	((ValadocRuleClass *) klass)->accept_token = (gboolean (*) (ValadocRule*, ValadocToken*, ValadocParserCallback*, ValadocRuleForward, GError**)) valadoc_many_rule_real_accept_token;
	((ValadocRuleClass *) klass)->would_accept_token = (gboolean (*) (ValadocRule*, ValadocToken*, GObject*)) valadoc_many_rule_real_would_accept_token;
	((ValadocRuleClass *) klass)->would_reduce = (gboolean (*) (ValadocRule*, ValadocToken*, GObject*)) valadoc_many_rule_real_would_reduce;
	((ValadocRuleClass *) klass)->to_string = (gchar* (*) (ValadocRule*, GObject*)) valadoc_many_rule_real_to_string;
	G_OBJECT_CLASS (klass)->finalize = valadoc_many_rule_finalize;
}

static void
valadoc_many_rule_instance_init (ValadocManyRule * self,
                                 gpointer klass)
{
	self->priv = valadoc_many_rule_get_instance_private (self);
}

static void
valadoc_many_rule_finalize (GObject * obj)
{
	ValadocManyRule * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, VALADOC_TYPE_MANY_RULE, ValadocManyRule);
	_g_object_unref0 (self->priv->_scheme);
	G_OBJECT_CLASS (valadoc_many_rule_parent_class)->finalize (obj);
}

static GType
valadoc_many_rule_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (ValadocManyRuleClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valadoc_many_rule_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValadocManyRule), 0, (GInstanceInitFunc) valadoc_many_rule_instance_init, NULL };
	GType valadoc_many_rule_type_id;
	valadoc_many_rule_type_id = g_type_register_static (VALADOC_TYPE_RULE, "ValadocManyRule", &g_define_type_info, 0);
	ValadocManyRule_private_offset = g_type_add_instance_private (valadoc_many_rule_type_id, sizeof (ValadocManyRulePrivate));
	return valadoc_many_rule_type_id;
}

 G_GNUC_INTERNAL GType
valadoc_many_rule_get_type (void)
{
	static volatile gsize valadoc_many_rule_type_id__once = 0;
	if (g_once_init_enter (&valadoc_many_rule_type_id__once)) {
		GType valadoc_many_rule_type_id;
		valadoc_many_rule_type_id = valadoc_many_rule_get_type_once ();
		g_once_init_leave (&valadoc_many_rule_type_id__once, valadoc_many_rule_type_id);
	}
	return valadoc_many_rule_type_id__once;
}

