/* valaccodedefine.c generated by valac, the Vala compiler
 * generated from valaccodedefine.vala, do not modify */

/* valaccodedefine.vala
 *
 * Copyright (C) 2018  Dr. Michael 'Mickey' Lauer
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Dr. Michael 'Mickey' Lauer <mickey@vanille-media.de>
 * 	Rico Tzschichholz <ricotz@ubuntu.com>
 */

#include "valaccode.h"
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <glib-object.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

#define _g_free0(var) (var = (g_free (var), NULL))
#define _vala_ccode_node_unref0(var) ((var == NULL) ? NULL : (var = (vala_ccode_node_unref (var), NULL)))

struct _ValaCCodeDefinePrivate {
	gchar* _name;
	gchar* _value;
	ValaCCodeExpression* _value_expression;
};

static gint ValaCCodeDefine_private_offset;
static gpointer vala_ccode_define_parent_class = NULL;

static void vala_ccode_define_real_write (ValaCCodeNode* base,
                                   ValaCCodeWriter* writer);
static void vala_ccode_define_finalize (ValaCCodeNode * obj);
static GType vala_ccode_define_get_type_once (void);

static inline gpointer
vala_ccode_define_get_instance_private (ValaCCodeDefine* self)
{
	return G_STRUCT_MEMBER_P (self, ValaCCodeDefine_private_offset);
}

const gchar*
vala_ccode_define_get_name (ValaCCodeDefine* self)
{
	const gchar* result;
	const gchar* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_name;
	result = _tmp0_;
	return result;
}

void
vala_ccode_define_set_name (ValaCCodeDefine* self,
                            const gchar* value)
{
	gchar* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp0_ = g_strdup (value);
	_g_free0 (self->priv->_name);
	self->priv->_name = _tmp0_;
}

const gchar*
vala_ccode_define_get_value (ValaCCodeDefine* self)
{
	const gchar* result;
	const gchar* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_value;
	result = _tmp0_;
	return result;
}

void
vala_ccode_define_set_value (ValaCCodeDefine* self,
                             const gchar* value)
{
	gchar* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp0_ = g_strdup (value);
	_g_free0 (self->priv->_value);
	self->priv->_value = _tmp0_;
}

ValaCCodeExpression*
vala_ccode_define_get_value_expression (ValaCCodeDefine* self)
{
	ValaCCodeExpression* result;
	ValaCCodeExpression* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_value_expression;
	result = _tmp0_;
	return result;
}

static gpointer
_vala_ccode_node_ref0 (gpointer self)
{
	return self ? vala_ccode_node_ref (self) : NULL;
}

void
vala_ccode_define_set_value_expression (ValaCCodeDefine* self,
                                        ValaCCodeExpression* value)
{
	ValaCCodeExpression* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp0_ = _vala_ccode_node_ref0 (value);
	_vala_ccode_node_unref0 (self->priv->_value_expression);
	self->priv->_value_expression = _tmp0_;
}

ValaCCodeDefine*
vala_ccode_define_construct (GType object_type,
                             const gchar* name,
                             const gchar* value)
{
	ValaCCodeDefine* self = NULL;
	g_return_val_if_fail (name != NULL, NULL);
	self = (ValaCCodeDefine*) vala_ccode_node_construct (object_type);
	vala_ccode_define_set_name (self, name);
	vala_ccode_define_set_value (self, value);
	return self;
}

ValaCCodeDefine*
vala_ccode_define_new (const gchar* name,
                       const gchar* value)
{
	return vala_ccode_define_construct (VALA_TYPE_CCODE_DEFINE, name, value);
}

ValaCCodeDefine*
vala_ccode_define_construct_with_expression (GType object_type,
                                             const gchar* name,
                                             ValaCCodeExpression* expression)
{
	ValaCCodeDefine* self = NULL;
	g_return_val_if_fail (name != NULL, NULL);
	g_return_val_if_fail (expression != NULL, NULL);
	self = (ValaCCodeDefine*) vala_ccode_node_construct (object_type);
	vala_ccode_define_set_name (self, name);
	vala_ccode_define_set_value_expression (self, expression);
	return self;
}

ValaCCodeDefine*
vala_ccode_define_new_with_expression (const gchar* name,
                                       ValaCCodeExpression* expression)
{
	return vala_ccode_define_construct_with_expression (VALA_TYPE_CCODE_DEFINE, name, expression);
}

static void
vala_ccode_define_real_write (ValaCCodeNode* base,
                              ValaCCodeWriter* writer)
{
	ValaCCodeDefine * self;
	const gchar* _tmp0_;
	const gchar* _tmp1_;
	self = (ValaCCodeDefine*) base;
	g_return_if_fail (writer != NULL);
	vala_ccode_writer_write_indent (writer, NULL);
	vala_ccode_writer_write_string (writer, "#define ");
	_tmp0_ = self->priv->_name;
	vala_ccode_writer_write_string (writer, _tmp0_);
	_tmp1_ = self->priv->_value;
	if (_tmp1_ != NULL) {
		const gchar* _tmp2_;
		vala_ccode_writer_write_string (writer, " ");
		_tmp2_ = self->priv->_value;
		vala_ccode_writer_write_string (writer, _tmp2_);
	} else {
		ValaCCodeExpression* _tmp3_;
		_tmp3_ = self->priv->_value_expression;
		if (_tmp3_ != NULL) {
			ValaCCodeExpression* _tmp4_;
			vala_ccode_writer_write_string (writer, " ");
			_tmp4_ = self->priv->_value_expression;
			vala_ccode_expression_write_inner (_tmp4_, writer);
		}
	}
	vala_ccode_writer_write_newline (writer);
}

static void
vala_ccode_define_class_init (ValaCCodeDefineClass * klass,
                              gpointer klass_data)
{
	vala_ccode_define_parent_class = g_type_class_peek_parent (klass);
	((ValaCCodeNodeClass *) klass)->finalize = vala_ccode_define_finalize;
	g_type_class_adjust_private_offset (klass, &ValaCCodeDefine_private_offset);
	((ValaCCodeNodeClass *) klass)->write = (void (*) (ValaCCodeNode*, ValaCCodeWriter*)) vala_ccode_define_real_write;
}

static void
vala_ccode_define_instance_init (ValaCCodeDefine * self,
                                 gpointer klass)
{
	self->priv = vala_ccode_define_get_instance_private (self);
}

static void
vala_ccode_define_finalize (ValaCCodeNode * obj)
{
	ValaCCodeDefine * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, VALA_TYPE_CCODE_DEFINE, ValaCCodeDefine);
	_g_free0 (self->priv->_name);
	_g_free0 (self->priv->_value);
	_vala_ccode_node_unref0 (self->priv->_value_expression);
	VALA_CCODE_NODE_CLASS (vala_ccode_define_parent_class)->finalize (obj);
}

/**
 * Represents a definition in the C code.
 */
static GType
vala_ccode_define_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (ValaCCodeDefineClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_ccode_define_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCCodeDefine), 0, (GInstanceInitFunc) vala_ccode_define_instance_init, NULL };
	GType vala_ccode_define_type_id;
	vala_ccode_define_type_id = g_type_register_static (VALA_TYPE_CCODE_NODE, "ValaCCodeDefine", &g_define_type_info, 0);
	ValaCCodeDefine_private_offset = g_type_add_instance_private (vala_ccode_define_type_id, sizeof (ValaCCodeDefinePrivate));
	return vala_ccode_define_type_id;
}

GType
vala_ccode_define_get_type (void)
{
	static volatile gsize vala_ccode_define_type_id__once = 0;
	if (g_once_init_enter (&vala_ccode_define_type_id__once)) {
		GType vala_ccode_define_type_id;
		vala_ccode_define_type_id = vala_ccode_define_get_type_once ();
		g_once_init_leave (&vala_ccode_define_type_id__once, vala_ccode_define_type_id);
	}
	return vala_ccode_define_type_id__once;
}

