/* ownership.c generated by valac, the Vala compiler
 * generated from ownership.vala, do not modify */

/* ownership.vala
 *
 * Copyright (C) 2011  Florian Brosch
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Florian Brosch <flo.brosch@gmail.com>
 */

#include "valadoc.h"

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

static GType
valadoc_api_ownership_get_type_once (void)
{
	static const GEnumValue values[] = {{VALADOC_API_OWNERSHIP_DEFAULT, "VALADOC_API_OWNERSHIP_DEFAULT", "default"}, {VALADOC_API_OWNERSHIP_UNOWNED, "VALADOC_API_OWNERSHIP_UNOWNED", "unowned"}, {VALADOC_API_OWNERSHIP_OWNED, "VALADOC_API_OWNERSHIP_OWNED", "owned"}, {VALADOC_API_OWNERSHIP_WEAK, "VALADOC_API_OWNERSHIP_WEAK", "weak"}, {0, NULL, NULL}};
	GType valadoc_api_ownership_type_id;
	valadoc_api_ownership_type_id = g_enum_register_static ("ValadocApiOwnership", values);
	return valadoc_api_ownership_type_id;
}

GType
valadoc_api_ownership_get_type (void)
{
	static volatile gsize valadoc_api_ownership_type_id__once = 0;
	if (g_once_init_enter (&valadoc_api_ownership_type_id__once)) {
		GType valadoc_api_ownership_type_id;
		valadoc_api_ownership_type_id = valadoc_api_ownership_get_type_once ();
		g_once_init_leave (&valadoc_api_ownership_type_id__once, valadoc_api_ownership_type_id);
	}
	return valadoc_api_ownership_type_id__once;
}

