/* inlinecontent.c generated by valac, the Vala compiler
 * generated from inlinecontent.vala, do not modify */

/* inlinecontent.vala
 *
 * Copyright (C) 2008-2009 Didier Villevalois
 * Copyright (C) 2008-2012 Florian Brosch
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Didier 'Ptitjes Villevalois <ptitjes@free.fr>
 */

#include "valadoc.h"
#include <valagee.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

enum  {
	VALADOC_CONTENT_INLINE_CONTENT_0_PROPERTY,
	VALADOC_CONTENT_INLINE_CONTENT_CONTENT_PROPERTY,
	VALADOC_CONTENT_INLINE_CONTENT_NUM_PROPERTIES
};
static GParamSpec* valadoc_content_inline_content_properties[VALADOC_CONTENT_INLINE_CONTENT_NUM_PROPERTIES];
#define _vala_iterable_unref0(var) ((var == NULL) ? NULL : (var = (vala_iterable_unref (var), NULL)))
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _vala_assert(expr, msg) if G_LIKELY (expr) ; else g_assertion_message_expr (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);
#define _vala_return_if_fail(expr, msg) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return; }
#define _vala_return_val_if_fail(expr, msg, val) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return val; }
#define _vala_warn_if_fail(expr, msg) if G_LIKELY (expr) ; else g_warn_message (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);

struct _ValadocContentInlineContentPrivate {
	ValaList* _content;
};

static gint ValadocContentInlineContent_private_offset;
static gpointer valadoc_content_inline_content_parent_class = NULL;

static void valadoc_content_inline_content_set_content (ValadocContentInlineContent* self,
                                                 ValaList* value);
 G_GNUC_INTERNAL ValadocContentInlineContent* valadoc_content_inline_content_construct (GType object_type);
static void valadoc_content_inline_content_real_check (ValadocContentContentElement* base,
                                                ValadocApiTree* api_root,
                                                ValadocApiNode* container,
                                                const gchar* file_path,
                                                ValadocErrorReporter* reporter,
                                                ValadocSettings* settings);
 G_GNUC_INTERNAL void valadoc_content_content_element_set_parent (ValadocContentContentElement* self,
                                                 ValadocContentContentElement* value);
static void valadoc_content_inline_content_real_accept_children (ValadocContentContentElement* base,
                                                          ValadocContentContentVisitor* visitor);
static gboolean valadoc_content_inline_content_real_is_empty (ValadocContentContentElement* base);
 G_GNUC_INTERNAL void valadoc_content_inline_content_replace_node (ValadocContentInlineContent* self,
                                                  ValadocContentInline* old,
                                                  ValadocContentInline* replacement);
static GObject * valadoc_content_inline_content_constructor (GType type,
                                                      guint n_construct_properties,
                                                      GObjectConstructParam * construct_properties);
static void valadoc_content_inline_content_finalize (GObject * obj);
static GType valadoc_content_inline_content_get_type_once (void);
static void _vala_valadoc_content_inline_content_get_property (GObject * object,
                                                        guint property_id,
                                                        GValue * value,
                                                        GParamSpec * pspec);
static void _vala_valadoc_content_inline_content_set_property (GObject * object,
                                                        guint property_id,
                                                        const GValue * value,
                                                        GParamSpec * pspec);

static inline gpointer
valadoc_content_inline_content_get_instance_private (ValadocContentInlineContent* self)
{
	return G_STRUCT_MEMBER_P (self, ValadocContentInlineContent_private_offset);
}

ValaList*
valadoc_content_inline_content_get_content (ValadocContentInlineContent* self)
{
	ValaList* result;
	ValaList* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_content;
	result = _tmp0_;
	return result;
}

static gpointer
_vala_iterable_ref0 (gpointer self)
{
	return self ? vala_iterable_ref (self) : NULL;
}

static void
valadoc_content_inline_content_set_content (ValadocContentInlineContent* self,
                                            ValaList* value)
{
	ValaList* old_value;
	g_return_if_fail (self != NULL);
	old_value = valadoc_content_inline_content_get_content (self);
	if (old_value != value) {
		ValaList* _tmp0_;
		_tmp0_ = _vala_iterable_ref0 (value);
		_vala_iterable_unref0 (self->priv->_content);
		self->priv->_content = _tmp0_;
		g_object_notify_by_pspec ((GObject *) self, valadoc_content_inline_content_properties[VALADOC_CONTENT_INLINE_CONTENT_CONTENT_PROPERTY]);
	}
}

 G_GNUC_INTERNAL ValadocContentInlineContent*
valadoc_content_inline_content_construct (GType object_type)
{
	ValadocContentInlineContent * self = NULL;
	self = (ValadocContentInlineContent*) valadoc_content_content_element_construct (object_type);
	return self;
}

static void
valadoc_content_inline_content_real_check (ValadocContentContentElement* base,
                                           ValadocApiTree* api_root,
                                           ValadocApiNode* container,
                                           const gchar* file_path,
                                           ValadocErrorReporter* reporter,
                                           ValadocSettings* settings)
{
	ValadocContentInlineContent * self;
	self = (ValadocContentInlineContent*) base;
	g_return_if_fail (api_root != NULL);
	g_return_if_fail (container != NULL);
	g_return_if_fail (file_path != NULL);
	g_return_if_fail (reporter != NULL);
	g_return_if_fail (settings != NULL);
	{
		ValaList* _element_list = NULL;
		ValaList* _tmp0_;
		gint _element_size = 0;
		ValaList* _tmp1_;
		gint _tmp2_;
		gint _tmp3_;
		gint _element_index = 0;
		_tmp0_ = self->priv->_content;
		_element_list = _tmp0_;
		_tmp1_ = _element_list;
		_tmp2_ = vala_collection_get_size ((ValaCollection*) _tmp1_);
		_tmp3_ = _tmp2_;
		_element_size = _tmp3_;
		_element_index = -1;
		while (TRUE) {
			gint _tmp4_;
			gint _tmp5_;
			ValadocContentInline* element = NULL;
			ValaList* _tmp6_;
			gpointer _tmp7_;
			ValadocContentInline* _tmp8_;
			ValadocContentInline* _tmp9_;
			_element_index = _element_index + 1;
			_tmp4_ = _element_index;
			_tmp5_ = _element_size;
			if (!(_tmp4_ < _tmp5_)) {
				break;
			}
			_tmp6_ = _element_list;
			_tmp7_ = vala_list_get (_tmp6_, _element_index);
			element = (ValadocContentInline*) _tmp7_;
			_tmp8_ = element;
			valadoc_content_content_element_set_parent ((ValadocContentContentElement*) _tmp8_, (ValadocContentContentElement*) self);
			_tmp9_ = element;
			valadoc_content_content_element_check ((ValadocContentContentElement*) _tmp9_, api_root, container, file_path, reporter, settings);
			_g_object_unref0 (element);
		}
	}
}

static void
valadoc_content_inline_content_real_accept_children (ValadocContentContentElement* base,
                                                     ValadocContentContentVisitor* visitor)
{
	ValadocContentInlineContent * self;
	self = (ValadocContentInlineContent*) base;
	g_return_if_fail (visitor != NULL);
	{
		ValaList* _element_list = NULL;
		ValaList* _tmp0_;
		gint _element_size = 0;
		ValaList* _tmp1_;
		gint _tmp2_;
		gint _tmp3_;
		gint _element_index = 0;
		_tmp0_ = self->priv->_content;
		_element_list = _tmp0_;
		_tmp1_ = _element_list;
		_tmp2_ = vala_collection_get_size ((ValaCollection*) _tmp1_);
		_tmp3_ = _tmp2_;
		_element_size = _tmp3_;
		_element_index = -1;
		while (TRUE) {
			gint _tmp4_;
			gint _tmp5_;
			ValadocContentInline* element = NULL;
			ValaList* _tmp6_;
			gpointer _tmp7_;
			ValadocContentInline* _tmp8_;
			_element_index = _element_index + 1;
			_tmp4_ = _element_index;
			_tmp5_ = _element_size;
			if (!(_tmp4_ < _tmp5_)) {
				break;
			}
			_tmp6_ = _element_list;
			_tmp7_ = vala_list_get (_tmp6_, _element_index);
			element = (ValadocContentInline*) _tmp7_;
			_tmp8_ = element;
			valadoc_content_content_element_accept ((ValadocContentContentElement*) _tmp8_, visitor);
			_g_object_unref0 (element);
		}
	}
}

static gboolean
valadoc_content_inline_content_real_is_empty (ValadocContentContentElement* base)
{
	ValadocContentInlineContent * self;
	gboolean result;
	self = (ValadocContentInlineContent*) base;
	{
		ValaList* _item_list = NULL;
		ValaList* _tmp0_;
		gint _item_size = 0;
		ValaList* _tmp1_;
		gint _tmp2_;
		gint _tmp3_;
		gint _item_index = 0;
		_tmp0_ = self->priv->_content;
		_item_list = _tmp0_;
		_tmp1_ = _item_list;
		_tmp2_ = vala_collection_get_size ((ValaCollection*) _tmp1_);
		_tmp3_ = _tmp2_;
		_item_size = _tmp3_;
		_item_index = -1;
		while (TRUE) {
			gint _tmp4_;
			gint _tmp5_;
			ValadocContentInline* item = NULL;
			ValaList* _tmp6_;
			gpointer _tmp7_;
			ValadocContentInline* _tmp8_;
			_item_index = _item_index + 1;
			_tmp4_ = _item_index;
			_tmp5_ = _item_size;
			if (!(_tmp4_ < _tmp5_)) {
				break;
			}
			_tmp6_ = _item_list;
			_tmp7_ = vala_list_get (_tmp6_, _item_index);
			item = (ValadocContentInline*) _tmp7_;
			_tmp8_ = item;
			if (!valadoc_content_content_element_is_empty ((ValadocContentContentElement*) _tmp8_)) {
				result = FALSE;
				_g_object_unref0 (item);
				return result;
			}
			_g_object_unref0 (item);
		}
	}
	result = TRUE;
	return result;
}

 G_GNUC_INTERNAL void
valadoc_content_inline_content_replace_node (ValadocContentInlineContent* self,
                                             ValadocContentInline* old,
                                             ValadocContentInline* replacement)
{
	gint index = 0;
	ValaList* _tmp0_;
	ValaList* _tmp1_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (old != NULL);
	g_return_if_fail (replacement != NULL);
	_tmp0_ = self->priv->_content;
	index = vala_list_index_of (_tmp0_, old);
	_vala_assert (index >= 0, "index >= 0");
	_tmp1_ = self->priv->_content;
	vala_list_set (_tmp1_, index, replacement);
}

static GObject *
valadoc_content_inline_content_constructor (GType type,
                                            guint n_construct_properties,
                                            GObjectConstructParam * construct_properties)
{
	GObject * obj;
	GObjectClass * parent_class;
	ValadocContentInlineContent * self;
	GEqualFunc _tmp0_;
	ValaArrayList* _tmp1_;
	ValaArrayList* _tmp2_;
	parent_class = G_OBJECT_CLASS (valadoc_content_inline_content_parent_class);
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, VALADOC_CONTENT_TYPE_INLINE_CONTENT, ValadocContentInlineContent);
	_tmp0_ = g_direct_equal;
	_tmp1_ = vala_array_list_new (VALADOC_CONTENT_TYPE_INLINE, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, _tmp0_);
	_tmp2_ = _tmp1_;
	valadoc_content_inline_content_set_content (self, (ValaList*) _tmp2_);
	_vala_iterable_unref0 (_tmp2_);
	return obj;
}

static void
valadoc_content_inline_content_class_init (ValadocContentInlineContentClass * klass,
                                           gpointer klass_data)
{
	valadoc_content_inline_content_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &ValadocContentInlineContent_private_offset);
	((ValadocContentContentElementClass *) klass)->check = (void (*) (ValadocContentContentElement*, ValadocApiTree*, ValadocApiNode*, const gchar*, ValadocErrorReporter*, ValadocSettings*)) valadoc_content_inline_content_real_check;
	((ValadocContentContentElementClass *) klass)->accept_children = (void (*) (ValadocContentContentElement*, ValadocContentContentVisitor*)) valadoc_content_inline_content_real_accept_children;
	((ValadocContentContentElementClass *) klass)->is_empty = (gboolean (*) (ValadocContentContentElement*)) valadoc_content_inline_content_real_is_empty;
	G_OBJECT_CLASS (klass)->get_property = _vala_valadoc_content_inline_content_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_valadoc_content_inline_content_set_property;
	G_OBJECT_CLASS (klass)->constructor = valadoc_content_inline_content_constructor;
	G_OBJECT_CLASS (klass)->finalize = valadoc_content_inline_content_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALADOC_CONTENT_INLINE_CONTENT_CONTENT_PROPERTY, valadoc_content_inline_content_properties[VALADOC_CONTENT_INLINE_CONTENT_CONTENT_PROPERTY] = vala_param_spec_iterable ("content", "content", "content", VALA_TYPE_LIST, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
}

static void
valadoc_content_inline_content_instance_init (ValadocContentInlineContent * self,
                                              gpointer klass)
{
	self->priv = valadoc_content_inline_content_get_instance_private (self);
}

static void
valadoc_content_inline_content_finalize (GObject * obj)
{
	ValadocContentInlineContent * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, VALADOC_CONTENT_TYPE_INLINE_CONTENT, ValadocContentInlineContent);
	_vala_iterable_unref0 (self->priv->_content);
	G_OBJECT_CLASS (valadoc_content_inline_content_parent_class)->finalize (obj);
}

static GType
valadoc_content_inline_content_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (ValadocContentInlineContentClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valadoc_content_inline_content_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValadocContentInlineContent), 0, (GInstanceInitFunc) valadoc_content_inline_content_instance_init, NULL };
	GType valadoc_content_inline_content_type_id;
	valadoc_content_inline_content_type_id = g_type_register_static (VALADOC_CONTENT_TYPE_CONTENT_ELEMENT, "ValadocContentInlineContent", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
	ValadocContentInlineContent_private_offset = g_type_add_instance_private (valadoc_content_inline_content_type_id, sizeof (ValadocContentInlineContentPrivate));
	return valadoc_content_inline_content_type_id;
}

GType
valadoc_content_inline_content_get_type (void)
{
	static volatile gsize valadoc_content_inline_content_type_id__once = 0;
	if (g_once_init_enter (&valadoc_content_inline_content_type_id__once)) {
		GType valadoc_content_inline_content_type_id;
		valadoc_content_inline_content_type_id = valadoc_content_inline_content_get_type_once ();
		g_once_init_leave (&valadoc_content_inline_content_type_id__once, valadoc_content_inline_content_type_id);
	}
	return valadoc_content_inline_content_type_id__once;
}

static void
_vala_valadoc_content_inline_content_get_property (GObject * object,
                                                   guint property_id,
                                                   GValue * value,
                                                   GParamSpec * pspec)
{
	ValadocContentInlineContent * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, VALADOC_CONTENT_TYPE_INLINE_CONTENT, ValadocContentInlineContent);
	switch (property_id) {
		case VALADOC_CONTENT_INLINE_CONTENT_CONTENT_PROPERTY:
		vala_value_set_iterable (value, valadoc_content_inline_content_get_content (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_valadoc_content_inline_content_set_property (GObject * object,
                                                   guint property_id,
                                                   const GValue * value,
                                                   GParamSpec * pspec)
{
	ValadocContentInlineContent * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, VALADOC_CONTENT_TYPE_INLINE_CONTENT, ValadocContentInlineContent);
	switch (property_id) {
		case VALADOC_CONTENT_INLINE_CONTENT_CONTENT_PROPERTY:
		valadoc_content_inline_content_set_content (self, vala_value_get_iterable (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

