/* valainvalidexpression.c generated by valac, the Vala compiler
 * generated from valainvalidexpression.vala, do not modify */

/* valainvalidexpression.vala
 *
 * Copyright (C) 2020  Rico Tzschichholz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Rico Tzschichholz <ricotz@ubuntu.com>
 */

#include "vala.h"
#include <glib.h>
#include <glib-object.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

static gpointer vala_invalid_expression_parent_class = NULL;

static gboolean vala_invalid_expression_real_is_pure (ValaExpression* base);
static gboolean vala_invalid_expression_real_check (ValaCodeNode* base,
                                             ValaCodeContext* context);
static GType vala_invalid_expression_get_type_once (void);

ValaInvalidExpression*
vala_invalid_expression_construct (GType object_type)
{
	ValaInvalidExpression* self = NULL;
	self = (ValaInvalidExpression*) vala_expression_construct (object_type);
	vala_code_node_set_error ((ValaCodeNode*) self, TRUE);
	return self;
}

ValaInvalidExpression*
vala_invalid_expression_new (void)
{
	return vala_invalid_expression_construct (VALA_TYPE_INVALID_EXPRESSION);
}

static gboolean
vala_invalid_expression_real_is_pure (ValaExpression* base)
{
	ValaInvalidExpression * self;
	gboolean result;
	self = (ValaInvalidExpression*) base;
	result = FALSE;
	return result;
}

static gboolean
vala_invalid_expression_real_check (ValaCodeNode* base,
                                    ValaCodeContext* context)
{
	ValaInvalidExpression * self;
	gboolean result;
	self = (ValaInvalidExpression*) base;
	g_return_val_if_fail (context != NULL, FALSE);
	result = FALSE;
	return result;
}

static void
vala_invalid_expression_class_init (ValaInvalidExpressionClass * klass,
                                    gpointer klass_data)
{
	vala_invalid_expression_parent_class = g_type_class_peek_parent (klass);
	((ValaExpressionClass *) klass)->is_pure = (gboolean (*) (ValaExpression*)) vala_invalid_expression_real_is_pure;
	((ValaCodeNodeClass *) klass)->check = (gboolean (*) (ValaCodeNode*, ValaCodeContext*)) vala_invalid_expression_real_check;
}

static void
vala_invalid_expression_instance_init (ValaInvalidExpression * self,
                                       gpointer klass)
{
}

/**
 * An invalid expression.
 */
static GType
vala_invalid_expression_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (ValaInvalidExpressionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_invalid_expression_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaInvalidExpression), 0, (GInstanceInitFunc) vala_invalid_expression_instance_init, NULL };
	GType vala_invalid_expression_type_id;
	vala_invalid_expression_type_id = g_type_register_static (VALA_TYPE_EXPRESSION, "ValaInvalidExpression", &g_define_type_info, 0);
	return vala_invalid_expression_type_id;
}

GType
vala_invalid_expression_get_type (void)
{
	static volatile gsize vala_invalid_expression_type_id__once = 0;
	if (g_once_init_enter (&vala_invalid_expression_type_id__once)) {
		GType vala_invalid_expression_type_id;
		vala_invalid_expression_type_id = vala_invalid_expression_get_type_once ();
		g_once_init_leave (&vala_invalid_expression_type_id__once, vala_invalid_expression_type_id);
	}
	return vala_invalid_expression_type_id__once;
}

