/* tablecell.c generated by valac, the Vala compiler
 * generated from tablecell.vala, do not modify */

/* tablecell.vala
 *
 * Copyright (C) 2008-2009 Didier Villevalois
 * Copyright (C) 2008-2012 Florian Brosch
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Didier 'Ptitjes Villevalois <ptitjes@free.fr>
 */

#include "valadoc.h"
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <glib-object.h>
#include <valagee.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

enum  {
	VALADOC_CONTENT_TABLE_CELL_0_PROPERTY,
	VALADOC_CONTENT_TABLE_CELL_HORIZONTAL_ALIGN_PROPERTY,
	VALADOC_CONTENT_TABLE_CELL_VERTICAL_ALIGN_PROPERTY,
	VALADOC_CONTENT_TABLE_CELL_STYLE_PROPERTY,
	VALADOC_CONTENT_TABLE_CELL_COLSPAN_PROPERTY,
	VALADOC_CONTENT_TABLE_CELL_ROWSPAN_PROPERTY,
	VALADOC_CONTENT_TABLE_CELL_NUM_PROPERTIES
};
static GParamSpec* valadoc_content_table_cell_properties[VALADOC_CONTENT_TABLE_CELL_NUM_PROPERTIES];
#define _g_free0(var) (var = (g_free (var), NULL))
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

struct _ValadocContentTableCellPrivate {
	ValadocContentHorizontalAlign _horizontal_align;
	ValadocContentVerticalAlign _vertical_align;
	gchar* _style;
	gint _colspan;
	gint _rowspan;
};

static gint ValadocContentTableCell_private_offset;
static gpointer valadoc_content_table_cell_parent_class = NULL;
static ValadocContentStyleAttributesIface * valadoc_content_table_cell_valadoc_content_style_attributes_parent_iface = NULL;

 G_GNUC_INTERNAL ValadocContentTableCell* valadoc_content_table_cell_new (void);
 G_GNUC_INTERNAL ValadocContentTableCell* valadoc_content_table_cell_construct (GType object_type);
 G_GNUC_INTERNAL ValadocContentInlineContent* valadoc_content_inline_content_construct (GType object_type);
static void valadoc_content_table_cell_real_check (ValadocContentContentElement* base,
                                            ValadocApiTree* api_root,
                                            ValadocApiNode* container,
                                            const gchar* file_path,
                                            ValadocErrorReporter* reporter,
                                            ValadocSettings* settings);
static void valadoc_content_table_cell_real_accept (ValadocContentContentElement* base,
                                             ValadocContentContentVisitor* visitor);
static gboolean valadoc_content_table_cell_real_is_empty (ValadocContentContentElement* base);
static ValadocContentContentElement* valadoc_content_table_cell_real_copy (ValadocContentContentElement* base,
                                                                    ValadocContentContentElement* new_parent);
 G_GNUC_INTERNAL void valadoc_content_content_element_set_parent (ValadocContentContentElement* self,
                                                 ValadocContentContentElement* value);
static void valadoc_content_table_cell_finalize (GObject * obj);
static GType valadoc_content_table_cell_get_type_once (void);
static void _vala_valadoc_content_table_cell_get_property (GObject * object,
                                                    guint property_id,
                                                    GValue * value,
                                                    GParamSpec * pspec);
static void _vala_valadoc_content_table_cell_set_property (GObject * object,
                                                    guint property_id,
                                                    const GValue * value,
                                                    GParamSpec * pspec);

static inline gpointer
valadoc_content_table_cell_get_instance_private (ValadocContentTableCell* self)
{
	return G_STRUCT_MEMBER_P (self, ValadocContentTableCell_private_offset);
}

static ValadocContentHorizontalAlign
valadoc_content_table_cell_real_get_horizontal_align (ValadocContentStyleAttributes* base)
{
	ValadocContentHorizontalAlign result;
	ValadocContentTableCell* self;
	self = (ValadocContentTableCell*) base;
	result = self->priv->_horizontal_align;
	return result;
}

static void
valadoc_content_table_cell_real_set_horizontal_align (ValadocContentStyleAttributes* base,
                                                      ValadocContentHorizontalAlign value)
{
	ValadocContentTableCell* self;
	ValadocContentHorizontalAlign old_value;
	self = (ValadocContentTableCell*) base;
	old_value = valadoc_content_table_cell_real_get_horizontal_align (base);
	if (old_value != value) {
		self->priv->_horizontal_align = value;
		g_object_notify_by_pspec ((GObject *) self, valadoc_content_table_cell_properties[VALADOC_CONTENT_TABLE_CELL_HORIZONTAL_ALIGN_PROPERTY]);
	}
}

static ValadocContentVerticalAlign
valadoc_content_table_cell_real_get_vertical_align (ValadocContentStyleAttributes* base)
{
	ValadocContentVerticalAlign result;
	ValadocContentTableCell* self;
	self = (ValadocContentTableCell*) base;
	result = self->priv->_vertical_align;
	return result;
}

static void
valadoc_content_table_cell_real_set_vertical_align (ValadocContentStyleAttributes* base,
                                                    ValadocContentVerticalAlign value)
{
	ValadocContentTableCell* self;
	ValadocContentVerticalAlign old_value;
	self = (ValadocContentTableCell*) base;
	old_value = valadoc_content_table_cell_real_get_vertical_align (base);
	if (old_value != value) {
		self->priv->_vertical_align = value;
		g_object_notify_by_pspec ((GObject *) self, valadoc_content_table_cell_properties[VALADOC_CONTENT_TABLE_CELL_VERTICAL_ALIGN_PROPERTY]);
	}
}

static const gchar*
valadoc_content_table_cell_real_get_style (ValadocContentStyleAttributes* base)
{
	const gchar* result;
	ValadocContentTableCell* self;
	const gchar* _tmp0_;
	self = (ValadocContentTableCell*) base;
	_tmp0_ = self->priv->_style;
	result = _tmp0_;
	return result;
}

static void
valadoc_content_table_cell_real_set_style (ValadocContentStyleAttributes* base,
                                           const gchar* value)
{
	ValadocContentTableCell* self;
	gchar* old_value;
	self = (ValadocContentTableCell*) base;
	old_value = valadoc_content_table_cell_real_get_style (base);
	if (g_strcmp0 (value, old_value) != 0) {
		gchar* _tmp0_;
		_tmp0_ = g_strdup (value);
		_g_free0 (self->priv->_style);
		self->priv->_style = _tmp0_;
		g_object_notify_by_pspec ((GObject *) self, valadoc_content_table_cell_properties[VALADOC_CONTENT_TABLE_CELL_STYLE_PROPERTY]);
	}
}

gint
valadoc_content_table_cell_get_colspan (ValadocContentTableCell* self)
{
	gint result;
	g_return_val_if_fail (self != NULL, 0);
	result = self->priv->_colspan;
	return result;
}

void
valadoc_content_table_cell_set_colspan (ValadocContentTableCell* self,
                                        gint value)
{
	gint old_value;
	g_return_if_fail (self != NULL);
	old_value = valadoc_content_table_cell_get_colspan (self);
	if (old_value != value) {
		self->priv->_colspan = value;
		g_object_notify_by_pspec ((GObject *) self, valadoc_content_table_cell_properties[VALADOC_CONTENT_TABLE_CELL_COLSPAN_PROPERTY]);
	}
}

gint
valadoc_content_table_cell_get_rowspan (ValadocContentTableCell* self)
{
	gint result;
	g_return_val_if_fail (self != NULL, 0);
	result = self->priv->_rowspan;
	return result;
}

void
valadoc_content_table_cell_set_rowspan (ValadocContentTableCell* self,
                                        gint value)
{
	gint old_value;
	g_return_if_fail (self != NULL);
	old_value = valadoc_content_table_cell_get_rowspan (self);
	if (old_value != value) {
		self->priv->_rowspan = value;
		g_object_notify_by_pspec ((GObject *) self, valadoc_content_table_cell_properties[VALADOC_CONTENT_TABLE_CELL_ROWSPAN_PROPERTY]);
	}
}

 G_GNUC_INTERNAL ValadocContentTableCell*
valadoc_content_table_cell_construct (GType object_type)
{
	ValadocContentTableCell * self = NULL;
	self = (ValadocContentTableCell*) valadoc_content_inline_content_construct (object_type);
	self->priv->_colspan = 1;
	self->priv->_rowspan = 1;
	return self;
}

 G_GNUC_INTERNAL ValadocContentTableCell*
valadoc_content_table_cell_new (void)
{
	return valadoc_content_table_cell_construct (VALADOC_CONTENT_TYPE_TABLE_CELL);
}

static void
valadoc_content_table_cell_real_check (ValadocContentContentElement* base,
                                       ValadocApiTree* api_root,
                                       ValadocApiNode* container,
                                       const gchar* file_path,
                                       ValadocErrorReporter* reporter,
                                       ValadocSettings* settings)
{
	ValadocContentTableCell * self;
	self = (ValadocContentTableCell*) base;
	g_return_if_fail (api_root != NULL);
	g_return_if_fail (container != NULL);
	g_return_if_fail (file_path != NULL);
	g_return_if_fail (reporter != NULL);
	g_return_if_fail (settings != NULL);
	VALADOC_CONTENT_CONTENT_ELEMENT_CLASS (valadoc_content_table_cell_parent_class)->check ((ValadocContentContentElement*) G_TYPE_CHECK_INSTANCE_CAST (self, VALADOC_CONTENT_TYPE_INLINE_CONTENT, ValadocContentInlineContent), api_root, container, file_path, reporter, settings);
}

static void
valadoc_content_table_cell_real_accept (ValadocContentContentElement* base,
                                        ValadocContentContentVisitor* visitor)
{
	ValadocContentTableCell * self;
	self = (ValadocContentTableCell*) base;
	g_return_if_fail (visitor != NULL);
	valadoc_content_content_visitor_visit_table_cell (visitor, self);
}

static gboolean
valadoc_content_table_cell_real_is_empty (ValadocContentContentElement* base)
{
	ValadocContentTableCell * self;
	gboolean result;
	self = (ValadocContentTableCell*) base;
	result = FALSE;
	return result;
}

static ValadocContentContentElement*
valadoc_content_table_cell_real_copy (ValadocContentContentElement* base,
                                      ValadocContentContentElement* new_parent)
{
	ValadocContentTableCell * self;
	ValadocContentTableCell* cell = NULL;
	ValadocContentTableCell* _tmp0_;
	ValadocContentTableCell* _tmp1_;
	ValadocContentTableCell* _tmp2_;
	ValadocContentHorizontalAlign _tmp3_;
	ValadocContentHorizontalAlign _tmp4_;
	ValadocContentTableCell* _tmp5_;
	ValadocContentVerticalAlign _tmp6_;
	ValadocContentVerticalAlign _tmp7_;
	ValadocContentTableCell* _tmp8_;
	gint _tmp9_;
	ValadocContentTableCell* _tmp10_;
	gint _tmp11_;
	ValadocContentTableCell* _tmp12_;
	const gchar* _tmp13_;
	const gchar* _tmp14_;
	ValadocContentContentElement* result;
	self = (ValadocContentTableCell*) base;
	_tmp0_ = valadoc_content_table_cell_new ();
	cell = _tmp0_;
	_tmp1_ = cell;
	valadoc_content_content_element_set_parent ((ValadocContentContentElement*) _tmp1_, new_parent);
	_tmp2_ = cell;
	_tmp3_ = valadoc_content_style_attributes_get_horizontal_align ((ValadocContentStyleAttributes*) self);
	_tmp4_ = _tmp3_;
	valadoc_content_style_attributes_set_horizontal_align ((ValadocContentStyleAttributes*) _tmp2_, _tmp4_);
	_tmp5_ = cell;
	_tmp6_ = valadoc_content_style_attributes_get_vertical_align ((ValadocContentStyleAttributes*) self);
	_tmp7_ = _tmp6_;
	valadoc_content_style_attributes_set_vertical_align ((ValadocContentStyleAttributes*) _tmp5_, _tmp7_);
	_tmp8_ = cell;
	_tmp9_ = self->priv->_colspan;
	valadoc_content_table_cell_set_colspan (_tmp8_, _tmp9_);
	_tmp10_ = cell;
	_tmp11_ = self->priv->_rowspan;
	valadoc_content_table_cell_set_rowspan (_tmp10_, _tmp11_);
	_tmp12_ = cell;
	_tmp13_ = valadoc_content_style_attributes_get_style ((ValadocContentStyleAttributes*) self);
	_tmp14_ = _tmp13_;
	valadoc_content_style_attributes_set_style ((ValadocContentStyleAttributes*) _tmp12_, _tmp14_);
	{
		ValaList* _element_list = NULL;
		ValaList* _tmp15_;
		ValaList* _tmp16_;
		gint _element_size = 0;
		ValaList* _tmp17_;
		gint _tmp18_;
		gint _tmp19_;
		gint _element_index = 0;
		_tmp15_ = valadoc_content_inline_content_get_content ((ValadocContentInlineContent*) self);
		_tmp16_ = _tmp15_;
		_element_list = _tmp16_;
		_tmp17_ = _element_list;
		_tmp18_ = vala_collection_get_size ((ValaCollection*) _tmp17_);
		_tmp19_ = _tmp18_;
		_element_size = _tmp19_;
		_element_index = -1;
		while (TRUE) {
			gint _tmp20_;
			gint _tmp21_;
			ValadocContentInline* element = NULL;
			ValaList* _tmp22_;
			gpointer _tmp23_;
			ValadocContentInline* copy = NULL;
			ValadocContentInline* _tmp24_;
			ValadocContentTableCell* _tmp25_;
			ValadocContentContentElement* _tmp26_;
			ValadocContentInline* _tmp27_;
			ValadocContentTableCell* _tmp28_;
			ValaList* _tmp29_;
			ValaList* _tmp30_;
			ValadocContentInline* _tmp31_;
			_element_index = _element_index + 1;
			_tmp20_ = _element_index;
			_tmp21_ = _element_size;
			if (!(_tmp20_ < _tmp21_)) {
				break;
			}
			_tmp22_ = _element_list;
			_tmp23_ = vala_list_get (_tmp22_, _element_index);
			element = (ValadocContentInline*) _tmp23_;
			_tmp24_ = element;
			_tmp25_ = cell;
			_tmp26_ = valadoc_content_content_element_copy ((ValadocContentContentElement*) _tmp24_, (ValadocContentContentElement*) _tmp25_);
			_tmp27_ = VALADOC_CONTENT_IS_INLINE (_tmp26_) ? ((ValadocContentInline*) _tmp26_) : NULL;
			if (_tmp27_ == NULL) {
				_g_object_unref0 (_tmp26_);
			}
			copy = _tmp27_;
			_tmp28_ = cell;
			_tmp29_ = valadoc_content_inline_content_get_content ((ValadocContentInlineContent*) _tmp28_);
			_tmp30_ = _tmp29_;
			_tmp31_ = copy;
			vala_collection_add ((ValaCollection*) _tmp30_, _tmp31_);
			_g_object_unref0 (copy);
			_g_object_unref0 (element);
		}
	}
	result = (ValadocContentContentElement*) cell;
	return result;
}

static void
valadoc_content_table_cell_class_init (ValadocContentTableCellClass * klass,
                                       gpointer klass_data)
{
	valadoc_content_table_cell_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &ValadocContentTableCell_private_offset);
	((ValadocContentContentElementClass *) klass)->check = (void (*) (ValadocContentContentElement*, ValadocApiTree*, ValadocApiNode*, const gchar*, ValadocErrorReporter*, ValadocSettings*)) valadoc_content_table_cell_real_check;
	((ValadocContentContentElementClass *) klass)->accept = (void (*) (ValadocContentContentElement*, ValadocContentContentVisitor*)) valadoc_content_table_cell_real_accept;
	((ValadocContentContentElementClass *) klass)->is_empty = (gboolean (*) (ValadocContentContentElement*)) valadoc_content_table_cell_real_is_empty;
	((ValadocContentContentElementClass *) klass)->copy = (ValadocContentContentElement* (*) (ValadocContentContentElement*, ValadocContentContentElement*)) valadoc_content_table_cell_real_copy;
	G_OBJECT_CLASS (klass)->get_property = _vala_valadoc_content_table_cell_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_valadoc_content_table_cell_set_property;
	G_OBJECT_CLASS (klass)->finalize = valadoc_content_table_cell_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALADOC_CONTENT_TABLE_CELL_HORIZONTAL_ALIGN_PROPERTY, valadoc_content_table_cell_properties[VALADOC_CONTENT_TABLE_CELL_HORIZONTAL_ALIGN_PROPERTY] = g_param_spec_enum ("horizontal-align", "horizontal-align", "horizontal-align", VALADOC_CONTENT_TYPE_HORIZONTAL_ALIGN, 0, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALADOC_CONTENT_TABLE_CELL_VERTICAL_ALIGN_PROPERTY, valadoc_content_table_cell_properties[VALADOC_CONTENT_TABLE_CELL_VERTICAL_ALIGN_PROPERTY] = g_param_spec_enum ("vertical-align", "vertical-align", "vertical-align", VALADOC_CONTENT_TYPE_VERTICAL_ALIGN, 0, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALADOC_CONTENT_TABLE_CELL_STYLE_PROPERTY, valadoc_content_table_cell_properties[VALADOC_CONTENT_TABLE_CELL_STYLE_PROPERTY] = g_param_spec_string ("style", "style", "style", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALADOC_CONTENT_TABLE_CELL_COLSPAN_PROPERTY, valadoc_content_table_cell_properties[VALADOC_CONTENT_TABLE_CELL_COLSPAN_PROPERTY] = g_param_spec_int ("colspan", "colspan", "colspan", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALADOC_CONTENT_TABLE_CELL_ROWSPAN_PROPERTY, valadoc_content_table_cell_properties[VALADOC_CONTENT_TABLE_CELL_ROWSPAN_PROPERTY] = g_param_spec_int ("rowspan", "rowspan", "rowspan", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
}

static void
valadoc_content_table_cell_valadoc_content_style_attributes_interface_init (ValadocContentStyleAttributesIface * iface,
                                                                            gpointer iface_data)
{
	valadoc_content_table_cell_valadoc_content_style_attributes_parent_iface = g_type_interface_peek_parent (iface);
	iface->get_horizontal_align = valadoc_content_table_cell_real_get_horizontal_align;
	iface->set_horizontal_align = valadoc_content_table_cell_real_set_horizontal_align;
	iface->get_vertical_align = valadoc_content_table_cell_real_get_vertical_align;
	iface->set_vertical_align = valadoc_content_table_cell_real_set_vertical_align;
	iface->get_style = valadoc_content_table_cell_real_get_style;
	iface->set_style = valadoc_content_table_cell_real_set_style;
}

static void
valadoc_content_table_cell_instance_init (ValadocContentTableCell * self,
                                          gpointer klass)
{
	self->priv = valadoc_content_table_cell_get_instance_private (self);
}

static void
valadoc_content_table_cell_finalize (GObject * obj)
{
	ValadocContentTableCell * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, VALADOC_CONTENT_TYPE_TABLE_CELL, ValadocContentTableCell);
	_g_free0 (self->priv->_style);
	G_OBJECT_CLASS (valadoc_content_table_cell_parent_class)->finalize (obj);
}

static GType
valadoc_content_table_cell_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (ValadocContentTableCellClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valadoc_content_table_cell_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValadocContentTableCell), 0, (GInstanceInitFunc) valadoc_content_table_cell_instance_init, NULL };
	static const GInterfaceInfo valadoc_content_style_attributes_info = { (GInterfaceInitFunc) valadoc_content_table_cell_valadoc_content_style_attributes_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	GType valadoc_content_table_cell_type_id;
	valadoc_content_table_cell_type_id = g_type_register_static (VALADOC_CONTENT_TYPE_INLINE_CONTENT, "ValadocContentTableCell", &g_define_type_info, 0);
	g_type_add_interface_static (valadoc_content_table_cell_type_id, VALADOC_CONTENT_TYPE_STYLE_ATTRIBUTES, &valadoc_content_style_attributes_info);
	ValadocContentTableCell_private_offset = g_type_add_instance_private (valadoc_content_table_cell_type_id, sizeof (ValadocContentTableCellPrivate));
	return valadoc_content_table_cell_type_id;
}

GType
valadoc_content_table_cell_get_type (void)
{
	static volatile gsize valadoc_content_table_cell_type_id__once = 0;
	if (g_once_init_enter (&valadoc_content_table_cell_type_id__once)) {
		GType valadoc_content_table_cell_type_id;
		valadoc_content_table_cell_type_id = valadoc_content_table_cell_get_type_once ();
		g_once_init_leave (&valadoc_content_table_cell_type_id__once, valadoc_content_table_cell_type_id);
	}
	return valadoc_content_table_cell_type_id__once;
}

static void
_vala_valadoc_content_table_cell_get_property (GObject * object,
                                               guint property_id,
                                               GValue * value,
                                               GParamSpec * pspec)
{
	ValadocContentTableCell * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, VALADOC_CONTENT_TYPE_TABLE_CELL, ValadocContentTableCell);
	switch (property_id) {
		case VALADOC_CONTENT_TABLE_CELL_HORIZONTAL_ALIGN_PROPERTY:
		g_value_set_enum (value, valadoc_content_style_attributes_get_horizontal_align ((ValadocContentStyleAttributes*) self));
		break;
		case VALADOC_CONTENT_TABLE_CELL_VERTICAL_ALIGN_PROPERTY:
		g_value_set_enum (value, valadoc_content_style_attributes_get_vertical_align ((ValadocContentStyleAttributes*) self));
		break;
		case VALADOC_CONTENT_TABLE_CELL_STYLE_PROPERTY:
		g_value_set_string (value, valadoc_content_style_attributes_get_style ((ValadocContentStyleAttributes*) self));
		break;
		case VALADOC_CONTENT_TABLE_CELL_COLSPAN_PROPERTY:
		g_value_set_int (value, valadoc_content_table_cell_get_colspan (self));
		break;
		case VALADOC_CONTENT_TABLE_CELL_ROWSPAN_PROPERTY:
		g_value_set_int (value, valadoc_content_table_cell_get_rowspan (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_valadoc_content_table_cell_set_property (GObject * object,
                                               guint property_id,
                                               const GValue * value,
                                               GParamSpec * pspec)
{
	ValadocContentTableCell * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, VALADOC_CONTENT_TYPE_TABLE_CELL, ValadocContentTableCell);
	switch (property_id) {
		case VALADOC_CONTENT_TABLE_CELL_HORIZONTAL_ALIGN_PROPERTY:
		valadoc_content_style_attributes_set_horizontal_align ((ValadocContentStyleAttributes*) self, g_value_get_enum (value));
		break;
		case VALADOC_CONTENT_TABLE_CELL_VERTICAL_ALIGN_PROPERTY:
		valadoc_content_style_attributes_set_vertical_align ((ValadocContentStyleAttributes*) self, g_value_get_enum (value));
		break;
		case VALADOC_CONTENT_TABLE_CELL_STYLE_PROPERTY:
		valadoc_content_style_attributes_set_style ((ValadocContentStyleAttributes*) self, g_value_get_string (value));
		break;
		case VALADOC_CONTENT_TABLE_CELL_COLSPAN_PROPERTY:
		valadoc_content_table_cell_set_colspan (self, g_value_get_int (value));
		break;
		case VALADOC_CONTENT_TABLE_CELL_ROWSPAN_PROPERTY:
		valadoc_content_table_cell_set_rowspan (self, g_value_get_int (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

