/* symbollink.c generated by valac, the Vala compiler
 * generated from symbollink.vala, do not modify */

/* symbollink.vala
 *
 * Copyright (C) 2008-2009 Didier Villevalois
 * Copyright (C) 2008-2014 Florian Brosch
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Didier 'Ptitjes Villevalois <ptitjes@free.fr>
 */

#include "valadoc.h"
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <glib-object.h>
#include <valagee.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

enum  {
	VALADOC_CONTENT_SYMBOL_LINK_0_PROPERTY,
	VALADOC_CONTENT_SYMBOL_LINK_SYMBOL_PROPERTY,
	VALADOC_CONTENT_SYMBOL_LINK_GIVEN_SYMBOL_NAME_PROPERTY,
	VALADOC_CONTENT_SYMBOL_LINK_NUM_PROPERTIES
};
static GParamSpec* valadoc_content_symbol_link_properties[VALADOC_CONTENT_SYMBOL_LINK_NUM_PROPERTIES];
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))

struct _ValadocContentSymbolLinkPrivate {
	ValadocApiNode* _symbol;
	gchar* _given_symbol_name;
};

static gint ValadocContentSymbolLink_private_offset;
static gpointer valadoc_content_symbol_link_parent_class = NULL;
static ValadocContentInlineIface * valadoc_content_symbol_link_valadoc_content_inline_parent_iface = NULL;

 G_GNUC_INTERNAL ValadocContentSymbolLink* valadoc_content_symbol_link_new (ValadocApiNode* symbol,
                                                           const gchar* given_symbol_name);
 G_GNUC_INTERNAL ValadocContentSymbolLink* valadoc_content_symbol_link_construct (GType object_type,
                                                                 ValadocApiNode* symbol,
                                                                 const gchar* given_symbol_name);
 G_GNUC_INTERNAL ValadocContentInlineContent* valadoc_content_inline_content_construct (GType object_type);
static void valadoc_content_symbol_link_real_configure (ValadocContentContentElement* base,
                                                 ValadocSettings* settings,
                                                 ValadocResourceLocator* locator);
static void valadoc_content_symbol_link_real_check (ValadocContentContentElement* base,
                                             ValadocApiTree* api_root,
                                             ValadocApiNode* container,
                                             const gchar* file_path,
                                             ValadocErrorReporter* reporter,
                                             ValadocSettings* settings);
static void valadoc_content_symbol_link_real_accept (ValadocContentContentElement* base,
                                              ValadocContentContentVisitor* visitor);
static gboolean valadoc_content_symbol_link_real_is_empty (ValadocContentContentElement* base);
static ValadocContentContentElement* valadoc_content_symbol_link_real_copy (ValadocContentContentElement* base,
                                                                     ValadocContentContentElement* new_parent);
 G_GNUC_INTERNAL void valadoc_content_content_element_set_parent (ValadocContentContentElement* self,
                                                 ValadocContentContentElement* value);
static void valadoc_content_symbol_link_finalize (GObject * obj);
static GType valadoc_content_symbol_link_get_type_once (void);
static void _vala_valadoc_content_symbol_link_get_property (GObject * object,
                                                     guint property_id,
                                                     GValue * value,
                                                     GParamSpec * pspec);
static void _vala_valadoc_content_symbol_link_set_property (GObject * object,
                                                     guint property_id,
                                                     const GValue * value,
                                                     GParamSpec * pspec);

static inline gpointer
valadoc_content_symbol_link_get_instance_private (ValadocContentSymbolLink* self)
{
	return G_STRUCT_MEMBER_P (self, ValadocContentSymbolLink_private_offset);
}

ValadocApiNode*
valadoc_content_symbol_link_get_symbol (ValadocContentSymbolLink* self)
{
	ValadocApiNode* result;
	ValadocApiNode* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_symbol;
	result = _tmp0_;
	return result;
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

void
valadoc_content_symbol_link_set_symbol (ValadocContentSymbolLink* self,
                                        ValadocApiNode* value)
{
	ValadocApiNode* old_value;
	g_return_if_fail (self != NULL);
	old_value = valadoc_content_symbol_link_get_symbol (self);
	if (old_value != value) {
		ValadocApiNode* _tmp0_;
		_tmp0_ = _g_object_ref0 (value);
		_g_object_unref0 (self->priv->_symbol);
		self->priv->_symbol = _tmp0_;
		g_object_notify_by_pspec ((GObject *) self, valadoc_content_symbol_link_properties[VALADOC_CONTENT_SYMBOL_LINK_SYMBOL_PROPERTY]);
	}
}

const gchar*
valadoc_content_symbol_link_get_given_symbol_name (ValadocContentSymbolLink* self)
{
	const gchar* result;
	const gchar* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_given_symbol_name;
	result = _tmp0_;
	return result;
}

void
valadoc_content_symbol_link_set_given_symbol_name (ValadocContentSymbolLink* self,
                                                   const gchar* value)
{
	gchar* old_value;
	g_return_if_fail (self != NULL);
	old_value = valadoc_content_symbol_link_get_given_symbol_name (self);
	if (g_strcmp0 (value, old_value) != 0) {
		gchar* _tmp0_;
		_tmp0_ = g_strdup (value);
		_g_free0 (self->priv->_given_symbol_name);
		self->priv->_given_symbol_name = _tmp0_;
		g_object_notify_by_pspec ((GObject *) self, valadoc_content_symbol_link_properties[VALADOC_CONTENT_SYMBOL_LINK_GIVEN_SYMBOL_NAME_PROPERTY]);
	}
}

 G_GNUC_INTERNAL ValadocContentSymbolLink*
valadoc_content_symbol_link_construct (GType object_type,
                                       ValadocApiNode* symbol,
                                       const gchar* given_symbol_name)
{
	ValadocContentSymbolLink * self = NULL;
	ValadocApiNode* _tmp0_;
	gchar* _tmp1_;
	self = (ValadocContentSymbolLink*) valadoc_content_inline_content_construct (object_type);
	_tmp0_ = _g_object_ref0 (symbol);
	_g_object_unref0 (self->priv->_symbol);
	self->priv->_symbol = _tmp0_;
	_tmp1_ = g_strdup (given_symbol_name);
	_g_free0 (self->priv->_given_symbol_name);
	self->priv->_given_symbol_name = _tmp1_;
	return self;
}

 G_GNUC_INTERNAL ValadocContentSymbolLink*
valadoc_content_symbol_link_new (ValadocApiNode* symbol,
                                 const gchar* given_symbol_name)
{
	return valadoc_content_symbol_link_construct (VALADOC_CONTENT_TYPE_SYMBOL_LINK, symbol, given_symbol_name);
}

static void
valadoc_content_symbol_link_real_configure (ValadocContentContentElement* base,
                                            ValadocSettings* settings,
                                            ValadocResourceLocator* locator)
{
	ValadocContentSymbolLink * self;
	self = (ValadocContentSymbolLink*) base;
	g_return_if_fail (settings != NULL);
	g_return_if_fail (locator != NULL);
}

static void
valadoc_content_symbol_link_real_check (ValadocContentContentElement* base,
                                        ValadocApiTree* api_root,
                                        ValadocApiNode* container,
                                        const gchar* file_path,
                                        ValadocErrorReporter* reporter,
                                        ValadocSettings* settings)
{
	ValadocContentSymbolLink * self;
	self = (ValadocContentSymbolLink*) base;
	g_return_if_fail (api_root != NULL);
	g_return_if_fail (container != NULL);
	g_return_if_fail (file_path != NULL);
	g_return_if_fail (reporter != NULL);
	g_return_if_fail (settings != NULL);
}

static void
valadoc_content_symbol_link_real_accept (ValadocContentContentElement* base,
                                         ValadocContentContentVisitor* visitor)
{
	ValadocContentSymbolLink * self;
	self = (ValadocContentSymbolLink*) base;
	g_return_if_fail (visitor != NULL);
	valadoc_content_content_visitor_visit_symbol_link (visitor, self);
}

static gboolean
valadoc_content_symbol_link_real_is_empty (ValadocContentContentElement* base)
{
	ValadocContentSymbolLink * self;
	gboolean result;
	self = (ValadocContentSymbolLink*) base;
	result = FALSE;
	return result;
}

static ValadocContentContentElement*
valadoc_content_symbol_link_real_copy (ValadocContentContentElement* base,
                                       ValadocContentContentElement* new_parent)
{
	ValadocContentSymbolLink * self;
	ValadocContentSymbolLink* link = NULL;
	ValadocApiNode* _tmp0_;
	const gchar* _tmp1_;
	ValadocContentSymbolLink* _tmp2_;
	ValadocContentSymbolLink* _tmp3_;
	ValadocContentContentElement* result;
	self = (ValadocContentSymbolLink*) base;
	_tmp0_ = self->priv->_symbol;
	_tmp1_ = self->priv->_given_symbol_name;
	_tmp2_ = valadoc_content_symbol_link_new (_tmp0_, _tmp1_);
	link = _tmp2_;
	_tmp3_ = link;
	valadoc_content_content_element_set_parent ((ValadocContentContentElement*) _tmp3_, new_parent);
	{
		ValaList* _element_list = NULL;
		ValaList* _tmp4_;
		ValaList* _tmp5_;
		gint _element_size = 0;
		ValaList* _tmp6_;
		gint _tmp7_;
		gint _tmp8_;
		gint _element_index = 0;
		_tmp4_ = valadoc_content_inline_content_get_content ((ValadocContentInlineContent*) self);
		_tmp5_ = _tmp4_;
		_element_list = _tmp5_;
		_tmp6_ = _element_list;
		_tmp7_ = vala_collection_get_size ((ValaCollection*) _tmp6_);
		_tmp8_ = _tmp7_;
		_element_size = _tmp8_;
		_element_index = -1;
		while (TRUE) {
			gint _tmp9_;
			gint _tmp10_;
			ValadocContentInline* element = NULL;
			ValaList* _tmp11_;
			gpointer _tmp12_;
			ValadocContentInline* copy = NULL;
			ValadocContentInline* _tmp13_;
			ValadocContentSymbolLink* _tmp14_;
			ValadocContentContentElement* _tmp15_;
			ValadocContentInline* _tmp16_;
			ValadocContentSymbolLink* _tmp17_;
			ValaList* _tmp18_;
			ValaList* _tmp19_;
			ValadocContentInline* _tmp20_;
			_element_index = _element_index + 1;
			_tmp9_ = _element_index;
			_tmp10_ = _element_size;
			if (!(_tmp9_ < _tmp10_)) {
				break;
			}
			_tmp11_ = _element_list;
			_tmp12_ = vala_list_get (_tmp11_, _element_index);
			element = (ValadocContentInline*) _tmp12_;
			_tmp13_ = element;
			_tmp14_ = link;
			_tmp15_ = valadoc_content_content_element_copy ((ValadocContentContentElement*) _tmp13_, (ValadocContentContentElement*) _tmp14_);
			_tmp16_ = VALADOC_CONTENT_IS_INLINE (_tmp15_) ? ((ValadocContentInline*) _tmp15_) : NULL;
			if (_tmp16_ == NULL) {
				_g_object_unref0 (_tmp15_);
			}
			copy = _tmp16_;
			_tmp17_ = link;
			_tmp18_ = valadoc_content_inline_content_get_content ((ValadocContentInlineContent*) _tmp17_);
			_tmp19_ = _tmp18_;
			_tmp20_ = copy;
			vala_collection_add ((ValaCollection*) _tmp19_, _tmp20_);
			_g_object_unref0 (copy);
			_g_object_unref0 (element);
		}
	}
	result = (ValadocContentContentElement*) link;
	return result;
}

static void
valadoc_content_symbol_link_class_init (ValadocContentSymbolLinkClass * klass,
                                        gpointer klass_data)
{
	valadoc_content_symbol_link_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &ValadocContentSymbolLink_private_offset);
	((ValadocContentContentElementClass *) klass)->configure = (void (*) (ValadocContentContentElement*, ValadocSettings*, ValadocResourceLocator*)) valadoc_content_symbol_link_real_configure;
	((ValadocContentContentElementClass *) klass)->check = (void (*) (ValadocContentContentElement*, ValadocApiTree*, ValadocApiNode*, const gchar*, ValadocErrorReporter*, ValadocSettings*)) valadoc_content_symbol_link_real_check;
	((ValadocContentContentElementClass *) klass)->accept = (void (*) (ValadocContentContentElement*, ValadocContentContentVisitor*)) valadoc_content_symbol_link_real_accept;
	((ValadocContentContentElementClass *) klass)->is_empty = (gboolean (*) (ValadocContentContentElement*)) valadoc_content_symbol_link_real_is_empty;
	((ValadocContentContentElementClass *) klass)->copy = (ValadocContentContentElement* (*) (ValadocContentContentElement*, ValadocContentContentElement*)) valadoc_content_symbol_link_real_copy;
	G_OBJECT_CLASS (klass)->get_property = _vala_valadoc_content_symbol_link_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_valadoc_content_symbol_link_set_property;
	G_OBJECT_CLASS (klass)->finalize = valadoc_content_symbol_link_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALADOC_CONTENT_SYMBOL_LINK_SYMBOL_PROPERTY, valadoc_content_symbol_link_properties[VALADOC_CONTENT_SYMBOL_LINK_SYMBOL_PROPERTY] = g_param_spec_object ("symbol", "symbol", "symbol", VALADOC_API_TYPE_NODE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALADOC_CONTENT_SYMBOL_LINK_GIVEN_SYMBOL_NAME_PROPERTY, valadoc_content_symbol_link_properties[VALADOC_CONTENT_SYMBOL_LINK_GIVEN_SYMBOL_NAME_PROPERTY] = g_param_spec_string ("given-symbol-name", "given-symbol-name", "given-symbol-name", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
}

static void
valadoc_content_symbol_link_valadoc_content_inline_interface_init (ValadocContentInlineIface * iface,
                                                                   gpointer iface_data)
{
	valadoc_content_symbol_link_valadoc_content_inline_parent_iface = g_type_interface_peek_parent (iface);
}

static void
valadoc_content_symbol_link_instance_init (ValadocContentSymbolLink * self,
                                           gpointer klass)
{
	self->priv = valadoc_content_symbol_link_get_instance_private (self);
}

static void
valadoc_content_symbol_link_finalize (GObject * obj)
{
	ValadocContentSymbolLink * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, VALADOC_CONTENT_TYPE_SYMBOL_LINK, ValadocContentSymbolLink);
	_g_object_unref0 (self->priv->_symbol);
	_g_free0 (self->priv->_given_symbol_name);
	G_OBJECT_CLASS (valadoc_content_symbol_link_parent_class)->finalize (obj);
}

static GType
valadoc_content_symbol_link_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (ValadocContentSymbolLinkClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valadoc_content_symbol_link_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValadocContentSymbolLink), 0, (GInstanceInitFunc) valadoc_content_symbol_link_instance_init, NULL };
	static const GInterfaceInfo valadoc_content_inline_info = { (GInterfaceInitFunc) valadoc_content_symbol_link_valadoc_content_inline_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	GType valadoc_content_symbol_link_type_id;
	valadoc_content_symbol_link_type_id = g_type_register_static (VALADOC_CONTENT_TYPE_INLINE_CONTENT, "ValadocContentSymbolLink", &g_define_type_info, 0);
	g_type_add_interface_static (valadoc_content_symbol_link_type_id, VALADOC_CONTENT_TYPE_INLINE, &valadoc_content_inline_info);
	ValadocContentSymbolLink_private_offset = g_type_add_instance_private (valadoc_content_symbol_link_type_id, sizeof (ValadocContentSymbolLinkPrivate));
	return valadoc_content_symbol_link_type_id;
}

GType
valadoc_content_symbol_link_get_type (void)
{
	static volatile gsize valadoc_content_symbol_link_type_id__once = 0;
	if (g_once_init_enter (&valadoc_content_symbol_link_type_id__once)) {
		GType valadoc_content_symbol_link_type_id;
		valadoc_content_symbol_link_type_id = valadoc_content_symbol_link_get_type_once ();
		g_once_init_leave (&valadoc_content_symbol_link_type_id__once, valadoc_content_symbol_link_type_id);
	}
	return valadoc_content_symbol_link_type_id__once;
}

static void
_vala_valadoc_content_symbol_link_get_property (GObject * object,
                                                guint property_id,
                                                GValue * value,
                                                GParamSpec * pspec)
{
	ValadocContentSymbolLink * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, VALADOC_CONTENT_TYPE_SYMBOL_LINK, ValadocContentSymbolLink);
	switch (property_id) {
		case VALADOC_CONTENT_SYMBOL_LINK_SYMBOL_PROPERTY:
		g_value_set_object (value, valadoc_content_symbol_link_get_symbol (self));
		break;
		case VALADOC_CONTENT_SYMBOL_LINK_GIVEN_SYMBOL_NAME_PROPERTY:
		g_value_set_string (value, valadoc_content_symbol_link_get_given_symbol_name (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_valadoc_content_symbol_link_set_property (GObject * object,
                                                guint property_id,
                                                const GValue * value,
                                                GParamSpec * pspec)
{
	ValadocContentSymbolLink * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, VALADOC_CONTENT_TYPE_SYMBOL_LINK, ValadocContentSymbolLink);
	switch (property_id) {
		case VALADOC_CONTENT_SYMBOL_LINK_SYMBOL_PROPERTY:
		valadoc_content_symbol_link_set_symbol (self, g_value_get_object (value));
		break;
		case VALADOC_CONTENT_SYMBOL_LINK_GIVEN_SYMBOL_NAME_PROPERTY:
		valadoc_content_symbol_link_set_given_symbol_name (self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

