/* visitor.c generated by valac, the Vala compiler
 * generated from visitor.vala, do not modify */

/* visitor.vala
 *
 * Copyright (C) 2008-2009 Florian Brosch, Didier Villevalois
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Didier 'Ptitjes Villevalois <ptitjes@free.fr>
 */

#include "valadoc.h"
#include <glib.h>
#include <glib-object.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

enum  {
	VALADOC_API_VISITOR_0_PROPERTY,
	VALADOC_API_VISITOR_NUM_PROPERTIES
};
static GParamSpec* valadoc_api_visitor_properties[VALADOC_API_VISITOR_NUM_PROPERTIES];

static gpointer valadoc_api_visitor_parent_class = NULL;

static void valadoc_api_visitor_real_visit_tree (ValadocApiVisitor* self,
                                          ValadocApiTree* item);
static void valadoc_api_visitor_real_visit_package (ValadocApiVisitor* self,
                                             ValadocApiPackage* item);
static void valadoc_api_visitor_real_visit_namespace (ValadocApiVisitor* self,
                                               ValadocApiNamespace* item);
static void valadoc_api_visitor_real_visit_interface (ValadocApiVisitor* self,
                                               ValadocApiInterface* item);
static void valadoc_api_visitor_real_visit_class (ValadocApiVisitor* self,
                                           ValadocApiClass* item);
static void valadoc_api_visitor_real_visit_struct (ValadocApiVisitor* self,
                                            ValadocApiStruct* item);
static void valadoc_api_visitor_real_visit_property (ValadocApiVisitor* self,
                                              ValadocApiProperty* item);
static void valadoc_api_visitor_real_visit_field (ValadocApiVisitor* self,
                                           ValadocApiField* item);
static void valadoc_api_visitor_real_visit_constant (ValadocApiVisitor* self,
                                              ValadocApiConstant* item);
static void valadoc_api_visitor_real_visit_delegate (ValadocApiVisitor* self,
                                              ValadocApiDelegate* item);
static void valadoc_api_visitor_real_visit_signal (ValadocApiVisitor* self,
                                            ValadocApiSignal* item);
static void valadoc_api_visitor_real_visit_method (ValadocApiVisitor* self,
                                            ValadocApiMethod* item);
static void valadoc_api_visitor_real_visit_type_parameter (ValadocApiVisitor* self,
                                                    ValadocApiTypeParameter* item);
static void valadoc_api_visitor_real_visit_formal_parameter (ValadocApiVisitor* self,
                                                      ValadocApiParameter* item);
static void valadoc_api_visitor_real_visit_error_domain (ValadocApiVisitor* self,
                                                  ValadocApiErrorDomain* item);
static void valadoc_api_visitor_real_visit_error_code (ValadocApiVisitor* self,
                                                ValadocApiErrorCode* item);
static void valadoc_api_visitor_real_visit_enum (ValadocApiVisitor* self,
                                          ValadocApiEnum* item);
static void valadoc_api_visitor_real_visit_enum_value (ValadocApiVisitor* self,
                                                ValadocApiEnumValue* item);
static GType valadoc_api_visitor_get_type_once (void);

/**
 * Visit operation called for api trees.
 *
 * @param item a tree
 */
static void
valadoc_api_visitor_real_visit_tree (ValadocApiVisitor* self,
                                     ValadocApiTree* item)
{
	g_return_if_fail (item != NULL);
}

void
valadoc_api_visitor_visit_tree (ValadocApiVisitor* self,
                                ValadocApiTree* item)
{
	ValadocApiVisitorClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = VALADOC_API_VISITOR_GET_CLASS (self);
	if (_klass_->visit_tree) {
		_klass_->visit_tree (self, item);
	}
}

/**
 * Visit operation called for packages like gir-files and vapi-files.
 *
 * @param item a package
 */
static void
valadoc_api_visitor_real_visit_package (ValadocApiVisitor* self,
                                        ValadocApiPackage* item)
{
	g_return_if_fail (item != NULL);
}

void
valadoc_api_visitor_visit_package (ValadocApiVisitor* self,
                                   ValadocApiPackage* item)
{
	ValadocApiVisitorClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = VALADOC_API_VISITOR_GET_CLASS (self);
	if (_klass_->visit_package) {
		_klass_->visit_package (self, item);
	}
}

/**
 * Visit operation called for namespaces
 *
 * @param item a namespace
 */
static void
valadoc_api_visitor_real_visit_namespace (ValadocApiVisitor* self,
                                          ValadocApiNamespace* item)
{
	g_return_if_fail (item != NULL);
}

void
valadoc_api_visitor_visit_namespace (ValadocApiVisitor* self,
                                     ValadocApiNamespace* item)
{
	ValadocApiVisitorClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = VALADOC_API_VISITOR_GET_CLASS (self);
	if (_klass_->visit_namespace) {
		_klass_->visit_namespace (self, item);
	}
}

/**
 * Visit operation called for interfaces.
 *
 * @param item a interface
 */
static void
valadoc_api_visitor_real_visit_interface (ValadocApiVisitor* self,
                                          ValadocApiInterface* item)
{
	g_return_if_fail (item != NULL);
}

void
valadoc_api_visitor_visit_interface (ValadocApiVisitor* self,
                                     ValadocApiInterface* item)
{
	ValadocApiVisitorClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = VALADOC_API_VISITOR_GET_CLASS (self);
	if (_klass_->visit_interface) {
		_klass_->visit_interface (self, item);
	}
}

/**
 * Visit operation called for classes.
 *
 * @param item a class
 */
static void
valadoc_api_visitor_real_visit_class (ValadocApiVisitor* self,
                                      ValadocApiClass* item)
{
	g_return_if_fail (item != NULL);
}

void
valadoc_api_visitor_visit_class (ValadocApiVisitor* self,
                                 ValadocApiClass* item)
{
	ValadocApiVisitorClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = VALADOC_API_VISITOR_GET_CLASS (self);
	if (_klass_->visit_class) {
		_klass_->visit_class (self, item);
	}
}

/**
 * Visit operation called for structs.
 *
 * @param item a struct
 */
static void
valadoc_api_visitor_real_visit_struct (ValadocApiVisitor* self,
                                       ValadocApiStruct* item)
{
	g_return_if_fail (item != NULL);
}

void
valadoc_api_visitor_visit_struct (ValadocApiVisitor* self,
                                  ValadocApiStruct* item)
{
	ValadocApiVisitorClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = VALADOC_API_VISITOR_GET_CLASS (self);
	if (_klass_->visit_struct) {
		_klass_->visit_struct (self, item);
	}
}

/**
 * Visit operation called for properties.
 *
 * @param item a property
 */
static void
valadoc_api_visitor_real_visit_property (ValadocApiVisitor* self,
                                         ValadocApiProperty* item)
{
	g_return_if_fail (item != NULL);
}

void
valadoc_api_visitor_visit_property (ValadocApiVisitor* self,
                                    ValadocApiProperty* item)
{
	ValadocApiVisitorClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = VALADOC_API_VISITOR_GET_CLASS (self);
	if (_klass_->visit_property) {
		_klass_->visit_property (self, item);
	}
}

/**
 * Visit operation called for fields.
 *
 * @param item a field
 */
static void
valadoc_api_visitor_real_visit_field (ValadocApiVisitor* self,
                                      ValadocApiField* item)
{
	g_return_if_fail (item != NULL);
}

void
valadoc_api_visitor_visit_field (ValadocApiVisitor* self,
                                 ValadocApiField* item)
{
	ValadocApiVisitorClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = VALADOC_API_VISITOR_GET_CLASS (self);
	if (_klass_->visit_field) {
		_klass_->visit_field (self, item);
	}
}

/**
 * Visit operation called for constants.
 *
 * @param item a constant
 */
static void
valadoc_api_visitor_real_visit_constant (ValadocApiVisitor* self,
                                         ValadocApiConstant* item)
{
	g_return_if_fail (item != NULL);
}

void
valadoc_api_visitor_visit_constant (ValadocApiVisitor* self,
                                    ValadocApiConstant* item)
{
	ValadocApiVisitorClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = VALADOC_API_VISITOR_GET_CLASS (self);
	if (_klass_->visit_constant) {
		_klass_->visit_constant (self, item);
	}
}

/**
 * Visit operation called for delegates.
 *
 * @param item a delegate
 */
static void
valadoc_api_visitor_real_visit_delegate (ValadocApiVisitor* self,
                                         ValadocApiDelegate* item)
{
	g_return_if_fail (item != NULL);
}

void
valadoc_api_visitor_visit_delegate (ValadocApiVisitor* self,
                                    ValadocApiDelegate* item)
{
	ValadocApiVisitorClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = VALADOC_API_VISITOR_GET_CLASS (self);
	if (_klass_->visit_delegate) {
		_klass_->visit_delegate (self, item);
	}
}

/**
 * Visit operation called for signals.
 *
 * @param item a signal
 */
static void
valadoc_api_visitor_real_visit_signal (ValadocApiVisitor* self,
                                       ValadocApiSignal* item)
{
	g_return_if_fail (item != NULL);
}

void
valadoc_api_visitor_visit_signal (ValadocApiVisitor* self,
                                  ValadocApiSignal* item)
{
	ValadocApiVisitorClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = VALADOC_API_VISITOR_GET_CLASS (self);
	if (_klass_->visit_signal) {
		_klass_->visit_signal (self, item);
	}
}

/**
 * Visit operation called for methods.
 *
 * @param item a method
 */
static void
valadoc_api_visitor_real_visit_method (ValadocApiVisitor* self,
                                       ValadocApiMethod* item)
{
	g_return_if_fail (item != NULL);
}

void
valadoc_api_visitor_visit_method (ValadocApiVisitor* self,
                                  ValadocApiMethod* item)
{
	ValadocApiVisitorClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = VALADOC_API_VISITOR_GET_CLASS (self);
	if (_klass_->visit_method) {
		_klass_->visit_method (self, item);
	}
}

/**
 * Visit operation called for type parameters.
 *
 * @param item a type parameter
 */
static void
valadoc_api_visitor_real_visit_type_parameter (ValadocApiVisitor* self,
                                               ValadocApiTypeParameter* item)
{
	g_return_if_fail (item != NULL);
}

void
valadoc_api_visitor_visit_type_parameter (ValadocApiVisitor* self,
                                          ValadocApiTypeParameter* item)
{
	ValadocApiVisitorClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = VALADOC_API_VISITOR_GET_CLASS (self);
	if (_klass_->visit_type_parameter) {
		_klass_->visit_type_parameter (self, item);
	}
}

/**
 * Visit operation called for parameters.
 *
 * @param item a parameter
 */
static void
valadoc_api_visitor_real_visit_formal_parameter (ValadocApiVisitor* self,
                                                 ValadocApiParameter* item)
{
	g_return_if_fail (item != NULL);
}

void
valadoc_api_visitor_visit_formal_parameter (ValadocApiVisitor* self,
                                            ValadocApiParameter* item)
{
	ValadocApiVisitorClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = VALADOC_API_VISITOR_GET_CLASS (self);
	if (_klass_->visit_formal_parameter) {
		_klass_->visit_formal_parameter (self, item);
	}
}

/**
 * Visit operation called for error domains.
 *
 * @param item a error domain
 */
static void
valadoc_api_visitor_real_visit_error_domain (ValadocApiVisitor* self,
                                             ValadocApiErrorDomain* item)
{
	g_return_if_fail (item != NULL);
}

void
valadoc_api_visitor_visit_error_domain (ValadocApiVisitor* self,
                                        ValadocApiErrorDomain* item)
{
	ValadocApiVisitorClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = VALADOC_API_VISITOR_GET_CLASS (self);
	if (_klass_->visit_error_domain) {
		_klass_->visit_error_domain (self, item);
	}
}

/**
 * Visit operation called for error codes.
 *
 * @param item a error code
 */
static void
valadoc_api_visitor_real_visit_error_code (ValadocApiVisitor* self,
                                           ValadocApiErrorCode* item)
{
	g_return_if_fail (item != NULL);
}

void
valadoc_api_visitor_visit_error_code (ValadocApiVisitor* self,
                                      ValadocApiErrorCode* item)
{
	ValadocApiVisitorClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = VALADOC_API_VISITOR_GET_CLASS (self);
	if (_klass_->visit_error_code) {
		_klass_->visit_error_code (self, item);
	}
}

/**
 * Visit operation called for enums.
 *
 * @param item a enum
 */
static void
valadoc_api_visitor_real_visit_enum (ValadocApiVisitor* self,
                                     ValadocApiEnum* item)
{
	g_return_if_fail (item != NULL);
}

void
valadoc_api_visitor_visit_enum (ValadocApiVisitor* self,
                                ValadocApiEnum* item)
{
	ValadocApiVisitorClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = VALADOC_API_VISITOR_GET_CLASS (self);
	if (_klass_->visit_enum) {
		_klass_->visit_enum (self, item);
	}
}

/**
 * Visit operation called for enum values.
 *
 * @param item a enum value
 */
static void
valadoc_api_visitor_real_visit_enum_value (ValadocApiVisitor* self,
                                           ValadocApiEnumValue* item)
{
	g_return_if_fail (item != NULL);
}

void
valadoc_api_visitor_visit_enum_value (ValadocApiVisitor* self,
                                      ValadocApiEnumValue* item)
{
	ValadocApiVisitorClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = VALADOC_API_VISITOR_GET_CLASS (self);
	if (_klass_->visit_enum_value) {
		_klass_->visit_enum_value (self, item);
	}
}

ValadocApiVisitor*
valadoc_api_visitor_construct (GType object_type)
{
	ValadocApiVisitor * self = NULL;
	self = (ValadocApiVisitor*) g_object_new (object_type, NULL);
	return self;
}

static void
valadoc_api_visitor_class_init (ValadocApiVisitorClass * klass,
                                gpointer klass_data)
{
	valadoc_api_visitor_parent_class = g_type_class_peek_parent (klass);
	((ValadocApiVisitorClass *) klass)->visit_tree = (void (*) (ValadocApiVisitor*, ValadocApiTree*)) valadoc_api_visitor_real_visit_tree;
	((ValadocApiVisitorClass *) klass)->visit_package = (void (*) (ValadocApiVisitor*, ValadocApiPackage*)) valadoc_api_visitor_real_visit_package;
	((ValadocApiVisitorClass *) klass)->visit_namespace = (void (*) (ValadocApiVisitor*, ValadocApiNamespace*)) valadoc_api_visitor_real_visit_namespace;
	((ValadocApiVisitorClass *) klass)->visit_interface = (void (*) (ValadocApiVisitor*, ValadocApiInterface*)) valadoc_api_visitor_real_visit_interface;
	((ValadocApiVisitorClass *) klass)->visit_class = (void (*) (ValadocApiVisitor*, ValadocApiClass*)) valadoc_api_visitor_real_visit_class;
	((ValadocApiVisitorClass *) klass)->visit_struct = (void (*) (ValadocApiVisitor*, ValadocApiStruct*)) valadoc_api_visitor_real_visit_struct;
	((ValadocApiVisitorClass *) klass)->visit_property = (void (*) (ValadocApiVisitor*, ValadocApiProperty*)) valadoc_api_visitor_real_visit_property;
	((ValadocApiVisitorClass *) klass)->visit_field = (void (*) (ValadocApiVisitor*, ValadocApiField*)) valadoc_api_visitor_real_visit_field;
	((ValadocApiVisitorClass *) klass)->visit_constant = (void (*) (ValadocApiVisitor*, ValadocApiConstant*)) valadoc_api_visitor_real_visit_constant;
	((ValadocApiVisitorClass *) klass)->visit_delegate = (void (*) (ValadocApiVisitor*, ValadocApiDelegate*)) valadoc_api_visitor_real_visit_delegate;
	((ValadocApiVisitorClass *) klass)->visit_signal = (void (*) (ValadocApiVisitor*, ValadocApiSignal*)) valadoc_api_visitor_real_visit_signal;
	((ValadocApiVisitorClass *) klass)->visit_method = (void (*) (ValadocApiVisitor*, ValadocApiMethod*)) valadoc_api_visitor_real_visit_method;
	((ValadocApiVisitorClass *) klass)->visit_type_parameter = (void (*) (ValadocApiVisitor*, ValadocApiTypeParameter*)) valadoc_api_visitor_real_visit_type_parameter;
	((ValadocApiVisitorClass *) klass)->visit_formal_parameter = (void (*) (ValadocApiVisitor*, ValadocApiParameter*)) valadoc_api_visitor_real_visit_formal_parameter;
	((ValadocApiVisitorClass *) klass)->visit_error_domain = (void (*) (ValadocApiVisitor*, ValadocApiErrorDomain*)) valadoc_api_visitor_real_visit_error_domain;
	((ValadocApiVisitorClass *) klass)->visit_error_code = (void (*) (ValadocApiVisitor*, ValadocApiErrorCode*)) valadoc_api_visitor_real_visit_error_code;
	((ValadocApiVisitorClass *) klass)->visit_enum = (void (*) (ValadocApiVisitor*, ValadocApiEnum*)) valadoc_api_visitor_real_visit_enum;
	((ValadocApiVisitorClass *) klass)->visit_enum_value = (void (*) (ValadocApiVisitor*, ValadocApiEnumValue*)) valadoc_api_visitor_real_visit_enum_value;
}

static void
valadoc_api_visitor_instance_init (ValadocApiVisitor * self,
                                   gpointer klass)
{
}

/**
 * Abstract visitor for traversing API.
 */
static GType
valadoc_api_visitor_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (ValadocApiVisitorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valadoc_api_visitor_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValadocApiVisitor), 0, (GInstanceInitFunc) valadoc_api_visitor_instance_init, NULL };
	GType valadoc_api_visitor_type_id;
	valadoc_api_visitor_type_id = g_type_register_static (G_TYPE_OBJECT, "ValadocApiVisitor", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
	return valadoc_api_visitor_type_id;
}

GType
valadoc_api_visitor_get_type (void)
{
	static volatile gsize valadoc_api_visitor_type_id__once = 0;
	if (g_once_init_enter (&valadoc_api_visitor_type_id__once)) {
		GType valadoc_api_visitor_type_id;
		valadoc_api_visitor_type_id = valadoc_api_visitor_get_type_once ();
		g_once_init_leave (&valadoc_api_visitor_type_id__once, valadoc_api_visitor_type_id);
	}
	return valadoc_api_visitor_type_id__once;
}

