/* scanner.c generated by valac, the Vala compiler
 * generated from scanner.vala, do not modify */

/* scanner.vala
 *
 * Copyright (C) 2008-2009 Florian Brosch, Didier Villevalois
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Didier 'Ptitjes Villevalois <ptitjes@free.fr>
 */

#include "valadoc.h"
#include <glib.h>
#include <stdlib.h>
#include <string.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

static GType valadoc_scanner_get_type_once (void);

void
valadoc_scanner_set_parser (ValadocScanner* self,
                            ValadocParser* parser)
{
	ValadocScannerIface* _iface_;
	g_return_if_fail (self != NULL);
	_iface_ = VALADOC_SCANNER_GET_INTERFACE (self);
	if (_iface_->set_parser) {
		_iface_->set_parser (self, parser);
	}
}

void
valadoc_scanner_reset (ValadocScanner* self)
{
	ValadocScannerIface* _iface_;
	g_return_if_fail (self != NULL);
	_iface_ = VALADOC_SCANNER_GET_INTERFACE (self);
	if (_iface_->reset) {
		_iface_->reset (self);
	}
}

void
valadoc_scanner_scan (ValadocScanner* self,
                      const gchar* content,
                      GError** error)
{
	ValadocScannerIface* _iface_;
	g_return_if_fail (self != NULL);
	_iface_ = VALADOC_SCANNER_GET_INTERFACE (self);
	if (_iface_->scan) {
		_iface_->scan (self, content, error);
	}
}

void
valadoc_scanner_end (ValadocScanner* self,
                     GError** error)
{
	ValadocScannerIface* _iface_;
	g_return_if_fail (self != NULL);
	_iface_ = VALADOC_SCANNER_GET_INTERFACE (self);
	if (_iface_->end) {
		_iface_->end (self, error);
	}
}

void
valadoc_scanner_stop (ValadocScanner* self)
{
	ValadocScannerIface* _iface_;
	g_return_if_fail (self != NULL);
	_iface_ = VALADOC_SCANNER_GET_INTERFACE (self);
	if (_iface_->stop) {
		_iface_->stop (self);
	}
}

gchar*
valadoc_scanner_get_line_content (ValadocScanner* self)
{
	ValadocScannerIface* _iface_;
	g_return_val_if_fail (self != NULL, NULL);
	_iface_ = VALADOC_SCANNER_GET_INTERFACE (self);
	if (_iface_->get_line_content) {
		return _iface_->get_line_content (self);
	}
	return NULL;
}

static void
valadoc_scanner_default_init (ValadocScannerIface * iface,
                              gpointer iface_data)
{
}

static GType
valadoc_scanner_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (ValadocScannerIface), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valadoc_scanner_default_init, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
	GType valadoc_scanner_type_id;
	valadoc_scanner_type_id = g_type_register_static (G_TYPE_INTERFACE, "ValadocScanner", &g_define_type_info, 0);
	g_type_interface_add_prerequisite (valadoc_scanner_type_id, G_TYPE_OBJECT);
	return valadoc_scanner_type_id;
}

GType
valadoc_scanner_get_type (void)
{
	static volatile gsize valadoc_scanner_type_id__once = 0;
	if (g_once_init_enter (&valadoc_scanner_type_id__once)) {
		GType valadoc_scanner_type_id;
		valadoc_scanner_type_id = valadoc_scanner_get_type_once ();
		g_once_init_leave (&valadoc_scanner_type_id__once, valadoc_scanner_type_id);
	}
	return valadoc_scanner_type_id__once;
}

