/* valactype.c generated by valac, the Vala compiler
 * generated from valactype.vala, do not modify */

/* valactype.vala
 *
 * Copyright (C) 2009  Mark Lee
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 *	Mark Lee <marklee@src.gnome.org>
 */

#include "valacodegen.h"
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <vala.h>
#include <glib-object.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

#define _g_free0(var) (var = (g_free (var), NULL))

struct _ValaCTypePrivate {
	gchar* _ctype_name;
	gchar* _cdefault_value;
};

static gint ValaCType_private_offset;
static gpointer vala_ctype_parent_class = NULL;

static ValaDataType* vala_ctype_real_copy (ValaDataType* base);
static void vala_ctype_finalize (ValaCodeNode * obj);
static GType vala_ctype_get_type_once (void);

static inline gpointer
vala_ctype_get_instance_private (ValaCType* self)
{
	return G_STRUCT_MEMBER_P (self, ValaCType_private_offset);
}

const gchar*
vala_ctype_get_ctype_name (ValaCType* self)
{
	const gchar* result;
	const gchar* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_ctype_name;
	result = _tmp0_;
	return result;
}

void
vala_ctype_set_ctype_name (ValaCType* self,
                           const gchar* value)
{
	gchar* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp0_ = g_strdup (value);
	_g_free0 (self->priv->_ctype_name);
	self->priv->_ctype_name = _tmp0_;
}

const gchar*
vala_ctype_get_cdefault_value (ValaCType* self)
{
	const gchar* result;
	const gchar* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_cdefault_value;
	result = _tmp0_;
	return result;
}

void
vala_ctype_set_cdefault_value (ValaCType* self,
                               const gchar* value)
{
	gchar* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp0_ = g_strdup (value);
	_g_free0 (self->priv->_cdefault_value);
	self->priv->_cdefault_value = _tmp0_;
}

ValaCType*
vala_ctype_construct (GType object_type,
                      const gchar* ctype_name,
                      const gchar* cdefault_value)
{
	ValaCType* self = NULL;
	g_return_val_if_fail (ctype_name != NULL, NULL);
	g_return_val_if_fail (cdefault_value != NULL, NULL);
	self = (ValaCType*) vala_data_type_construct (object_type);
	vala_ctype_set_ctype_name (self, ctype_name);
	vala_ctype_set_cdefault_value (self, cdefault_value);
	return self;
}

ValaCType*
vala_ctype_new (const gchar* ctype_name,
                const gchar* cdefault_value)
{
	return vala_ctype_construct (VALA_TYPE_CTYPE, ctype_name, cdefault_value);
}

static ValaDataType*
vala_ctype_real_copy (ValaDataType* base)
{
	ValaCType * self;
	const gchar* _tmp0_;
	const gchar* _tmp1_;
	ValaCType* _tmp2_;
	ValaDataType* result;
	self = (ValaCType*) base;
	_tmp0_ = self->priv->_ctype_name;
	_tmp1_ = self->priv->_cdefault_value;
	_tmp2_ = vala_ctype_new (_tmp0_, _tmp1_);
	result = (ValaDataType*) _tmp2_;
	return result;
}

static void
vala_ctype_class_init (ValaCTypeClass * klass,
                       gpointer klass_data)
{
	vala_ctype_parent_class = g_type_class_peek_parent (klass);
	((ValaCodeNodeClass *) klass)->finalize = vala_ctype_finalize;
	g_type_class_adjust_private_offset (klass, &ValaCType_private_offset);
	((ValaDataTypeClass *) klass)->copy = (ValaDataType* (*) (ValaDataType*)) vala_ctype_real_copy;
}

static void
vala_ctype_instance_init (ValaCType * self,
                          gpointer klass)
{
	self->priv = vala_ctype_get_instance_private (self);
}

static void
vala_ctype_finalize (ValaCodeNode * obj)
{
	ValaCType * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, VALA_TYPE_CTYPE, ValaCType);
	_g_free0 (self->priv->_ctype_name);
	_g_free0 (self->priv->_cdefault_value);
	VALA_CODE_NODE_CLASS (vala_ctype_parent_class)->finalize (obj);
}

/**
 * A C type, used only for code generation purposes.
 */
static GType
vala_ctype_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (ValaCTypeClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_ctype_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCType), 0, (GInstanceInitFunc) vala_ctype_instance_init, NULL };
	GType vala_ctype_type_id;
	vala_ctype_type_id = g_type_register_static (VALA_TYPE_DATA_TYPE, "ValaCType", &g_define_type_info, 0);
	ValaCType_private_offset = g_type_add_instance_private (vala_ctype_type_id, sizeof (ValaCTypePrivate));
	return vala_ctype_type_id;
}

GType
vala_ctype_get_type (void)
{
	static volatile gsize vala_ctype_type_id__once = 0;
	if (g_once_init_enter (&vala_ctype_type_id__once)) {
		GType vala_ctype_type_id;
		vala_ctype_type_id = vala_ctype_get_type_once ();
		g_once_init_leave (&vala_ctype_type_id__once, vala_ctype_type_id);
	}
	return vala_ctype_type_id__once;
}

