/* comment.c generated by valac, the Vala compiler
 * generated from comment.vala, do not modify */

/* comment.vala
 *
 * Copyright (C) 2008-2009 Didier Villevalois
 * Copyright (C) 2008-2012 Florian Brosch
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Didier 'Ptitjes Villevalois <ptitjes@free.fr>
 */

#include "valadoc.h"
#include <valagee.h>
#include <glib-object.h>
#include <glib.h>
#include <stdlib.h>
#include <string.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

enum  {
	VALADOC_CONTENT_COMMENT_0_PROPERTY,
	VALADOC_CONTENT_COMMENT_TAGLETS_PROPERTY,
	VALADOC_CONTENT_COMMENT_NUM_PROPERTIES
};
static GParamSpec* valadoc_content_comment_properties[VALADOC_CONTENT_COMMENT_NUM_PROPERTIES];
#define _vala_iterable_unref0(var) ((var == NULL) ? NULL : (var = (vala_iterable_unref (var), NULL)))
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _vala_assert(expr, msg) if G_LIKELY (expr) ; else g_assertion_message_expr (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);
#define _vala_return_if_fail(expr, msg) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return; }
#define _vala_return_val_if_fail(expr, msg, val) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return val; }
#define _vala_warn_if_fail(expr, msg) if G_LIKELY (expr) ; else g_warn_message (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);

struct _ValadocContentCommentPrivate {
	ValaList* _taglets;
	gboolean checked;
};

static gint ValadocContentComment_private_offset;
static gpointer valadoc_content_comment_parent_class = NULL;

static void valadoc_content_comment_set_taglets (ValadocContentComment* self,
                                          ValaList* value);
 G_GNUC_INTERNAL ValadocContentComment* valadoc_content_comment_new (void);
 G_GNUC_INTERNAL ValadocContentComment* valadoc_content_comment_construct (GType object_type);
 G_GNUC_INTERNAL ValadocContentBlockContent* valadoc_content_block_content_construct (GType object_type);
static void valadoc_content_comment_real_configure (ValadocContentContentElement* base,
                                             ValadocSettings* settings,
                                             ValadocResourceLocator* locator);
static void valadoc_content_comment_real_check (ValadocContentContentElement* base,
                                         ValadocApiTree* api_root,
                                         ValadocApiNode* container,
                                         const gchar* file_path,
                                         ValadocErrorReporter* reporter,
                                         ValadocSettings* settings);
 G_GNUC_INTERNAL void valadoc_content_content_element_set_parent (ValadocContentContentElement* self,
                                                 ValadocContentContentElement* value);
static void valadoc_content_comment_real_accept (ValadocContentContentElement* base,
                                          ValadocContentContentVisitor* visitor);
static void valadoc_content_comment_real_accept_children (ValadocContentContentElement* base,
                                                   ValadocContentContentVisitor* visitor);
static ValadocContentContentElement* valadoc_content_comment_real_copy (ValadocContentContentElement* base,
                                                                 ValadocContentContentElement* new_parent);
static void valadoc_content_comment_finalize (GObject * obj);
static GType valadoc_content_comment_get_type_once (void);
static void _vala_valadoc_content_comment_get_property (GObject * object,
                                                 guint property_id,
                                                 GValue * value,
                                                 GParamSpec * pspec);
static void _vala_valadoc_content_comment_set_property (GObject * object,
                                                 guint property_id,
                                                 const GValue * value,
                                                 GParamSpec * pspec);

static inline gpointer
valadoc_content_comment_get_instance_private (ValadocContentComment* self)
{
	return G_STRUCT_MEMBER_P (self, ValadocContentComment_private_offset);
}

ValaList*
valadoc_content_comment_get_taglets (ValadocContentComment* self)
{
	ValaList* result;
	ValaList* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_taglets;
	result = _tmp0_;
	return result;
}

static gpointer
_vala_iterable_ref0 (gpointer self)
{
	return self ? vala_iterable_ref (self) : NULL;
}

static void
valadoc_content_comment_set_taglets (ValadocContentComment* self,
                                     ValaList* value)
{
	ValaList* old_value;
	g_return_if_fail (self != NULL);
	old_value = valadoc_content_comment_get_taglets (self);
	if (old_value != value) {
		ValaList* _tmp0_;
		_tmp0_ = _vala_iterable_ref0 (value);
		_vala_iterable_unref0 (self->priv->_taglets);
		self->priv->_taglets = _tmp0_;
		g_object_notify_by_pspec ((GObject *) self, valadoc_content_comment_properties[VALADOC_CONTENT_COMMENT_TAGLETS_PROPERTY]);
	}
}

 G_GNUC_INTERNAL ValadocContentComment*
valadoc_content_comment_construct (GType object_type)
{
	ValadocContentComment * self = NULL;
	GEqualFunc _tmp0_;
	ValaArrayList* _tmp1_;
	ValaArrayList* _tmp2_;
	self = (ValadocContentComment*) valadoc_content_block_content_construct (object_type);
	_tmp0_ = g_direct_equal;
	_tmp1_ = vala_array_list_new (VALADOC_CONTENT_TYPE_TAGLET, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, _tmp0_);
	_tmp2_ = _tmp1_;
	valadoc_content_comment_set_taglets (self, (ValaList*) _tmp2_);
	_vala_iterable_unref0 (_tmp2_);
	return self;
}

 G_GNUC_INTERNAL ValadocContentComment*
valadoc_content_comment_new (void)
{
	return valadoc_content_comment_construct (VALADOC_CONTENT_TYPE_COMMENT);
}

static void
valadoc_content_comment_real_configure (ValadocContentContentElement* base,
                                        ValadocSettings* settings,
                                        ValadocResourceLocator* locator)
{
	ValadocContentComment * self;
	self = (ValadocContentComment*) base;
	g_return_if_fail (settings != NULL);
	g_return_if_fail (locator != NULL);
}

static void
valadoc_content_comment_real_check (ValadocContentContentElement* base,
                                    ValadocApiTree* api_root,
                                    ValadocApiNode* container,
                                    const gchar* file_path,
                                    ValadocErrorReporter* reporter,
                                    ValadocSettings* settings)
{
	ValadocContentComment * self;
	self = (ValadocContentComment*) base;
	g_return_if_fail (api_root != NULL);
	g_return_if_fail (container != NULL);
	g_return_if_fail (file_path != NULL);
	g_return_if_fail (reporter != NULL);
	g_return_if_fail (settings != NULL);
	if (self->priv->checked == TRUE) {
		return;
	}
	self->priv->checked = TRUE;
	VALADOC_CONTENT_CONTENT_ELEMENT_CLASS (valadoc_content_comment_parent_class)->check ((ValadocContentContentElement*) G_TYPE_CHECK_INSTANCE_CAST (self, VALADOC_CONTENT_TYPE_BLOCK_CONTENT, ValadocContentBlockContent), api_root, container, file_path, reporter, settings);
	{
		ValaList* _element_list = NULL;
		ValaList* _tmp0_;
		gint _element_size = 0;
		ValaList* _tmp1_;
		gint _tmp2_;
		gint _tmp3_;
		gint _element_index = 0;
		_tmp0_ = self->priv->_taglets;
		_element_list = _tmp0_;
		_tmp1_ = _element_list;
		_tmp2_ = vala_collection_get_size ((ValaCollection*) _tmp1_);
		_tmp3_ = _tmp2_;
		_element_size = _tmp3_;
		_element_index = -1;
		while (TRUE) {
			gint _tmp4_;
			gint _tmp5_;
			ValadocContentTaglet* element = NULL;
			ValaList* _tmp6_;
			gpointer _tmp7_;
			ValadocContentTaglet* _tmp8_;
			ValadocContentTaglet* _tmp9_;
			_element_index = _element_index + 1;
			_tmp4_ = _element_index;
			_tmp5_ = _element_size;
			if (!(_tmp4_ < _tmp5_)) {
				break;
			}
			_tmp6_ = _element_list;
			_tmp7_ = vala_list_get (_tmp6_, _element_index);
			element = (ValadocContentTaglet*) _tmp7_;
			_tmp8_ = element;
			valadoc_content_content_element_set_parent ((ValadocContentContentElement*) _tmp8_, (ValadocContentContentElement*) self);
			_tmp9_ = element;
			valadoc_content_content_element_check ((ValadocContentContentElement*) _tmp9_, api_root, container, file_path, reporter, settings);
			_g_object_unref0 (element);
		}
	}
}

static void
valadoc_content_comment_real_accept (ValadocContentContentElement* base,
                                     ValadocContentContentVisitor* visitor)
{
	ValadocContentComment * self;
	self = (ValadocContentComment*) base;
	g_return_if_fail (visitor != NULL);
	valadoc_content_content_visitor_visit_comment (visitor, self);
}

static void
valadoc_content_comment_real_accept_children (ValadocContentContentElement* base,
                                              ValadocContentContentVisitor* visitor)
{
	ValadocContentComment * self;
	self = (ValadocContentComment*) base;
	g_return_if_fail (visitor != NULL);
	VALADOC_CONTENT_CONTENT_ELEMENT_CLASS (valadoc_content_comment_parent_class)->accept_children ((ValadocContentContentElement*) G_TYPE_CHECK_INSTANCE_CAST (self, VALADOC_CONTENT_TYPE_BLOCK_CONTENT, ValadocContentBlockContent), visitor);
	{
		ValaList* _element_list = NULL;
		ValaList* _tmp0_;
		gint _element_size = 0;
		ValaList* _tmp1_;
		gint _tmp2_;
		gint _tmp3_;
		gint _element_index = 0;
		_tmp0_ = self->priv->_taglets;
		_element_list = _tmp0_;
		_tmp1_ = _element_list;
		_tmp2_ = vala_collection_get_size ((ValaCollection*) _tmp1_);
		_tmp3_ = _tmp2_;
		_element_size = _tmp3_;
		_element_index = -1;
		while (TRUE) {
			gint _tmp4_;
			gint _tmp5_;
			ValadocContentTaglet* element = NULL;
			ValaList* _tmp6_;
			gpointer _tmp7_;
			ValadocContentTaglet* _tmp8_;
			_element_index = _element_index + 1;
			_tmp4_ = _element_index;
			_tmp5_ = _element_size;
			if (!(_tmp4_ < _tmp5_)) {
				break;
			}
			_tmp6_ = _element_list;
			_tmp7_ = vala_list_get (_tmp6_, _element_index);
			element = (ValadocContentTaglet*) _tmp7_;
			_tmp8_ = element;
			valadoc_content_content_element_accept ((ValadocContentContentElement*) _tmp8_, visitor);
			_g_object_unref0 (element);
		}
	}
}

ValaList*
valadoc_content_comment_find_taglets (ValadocContentComment* self,
                                      ValadocApiNode* container,
                                      GType taglet_type)
{
	ValaList* selected_taglets = NULL;
	GEqualFunc _tmp0_;
	ValaArrayList* _tmp1_;
	ValaList* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = g_direct_equal;
	_tmp1_ = vala_array_list_new (VALADOC_CONTENT_TYPE_TAGLET, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, _tmp0_);
	selected_taglets = (ValaList*) _tmp1_;
	{
		ValaList* _taglet_list = NULL;
		ValaList* _tmp2_;
		gint _taglet_size = 0;
		ValaList* _tmp3_;
		gint _tmp4_;
		gint _tmp5_;
		gint _taglet_index = 0;
		_tmp2_ = self->priv->_taglets;
		_taglet_list = _tmp2_;
		_tmp3_ = _taglet_list;
		_tmp4_ = vala_collection_get_size ((ValaCollection*) _tmp3_);
		_tmp5_ = _tmp4_;
		_taglet_size = _tmp5_;
		_taglet_index = -1;
		while (TRUE) {
			gint _tmp6_;
			gint _tmp7_;
			ValadocContentTaglet* taglet = NULL;
			ValaList* _tmp8_;
			gpointer _tmp9_;
			ValadocContentTaglet* _tmp10_;
			_taglet_index = _taglet_index + 1;
			_tmp6_ = _taglet_index;
			_tmp7_ = _taglet_size;
			if (!(_tmp6_ < _tmp7_)) {
				break;
			}
			_tmp8_ = _taglet_list;
			_tmp9_ = vala_list_get (_tmp8_, _taglet_index);
			taglet = (ValadocContentTaglet*) _tmp9_;
			_tmp10_ = taglet;
			if (G_TYPE_FROM_INSTANCE ((GObject*) _tmp10_) == taglet_type) {
				ValaList* _tmp11_;
				ValadocContentTaglet* _tmp12_;
				_tmp11_ = selected_taglets;
				_tmp12_ = taglet;
				vala_collection_add ((ValaCollection*) _tmp11_, _tmp12_);
			}
			_g_object_unref0 (taglet);
		}
	}
	result = selected_taglets;
	return result;
}

static ValadocContentContentElement*
valadoc_content_comment_real_copy (ValadocContentContentElement* base,
                                   ValadocContentContentElement* new_parent)
{
	ValadocContentComment * self;
	ValadocContentComment* comment = NULL;
	ValadocContentComment* _tmp0_;
	ValadocContentComment* _tmp1_;
	ValadocContentContentElement* result;
	self = (ValadocContentComment*) base;
	_vala_assert (new_parent == NULL, "new_parent == null");
	_tmp0_ = valadoc_content_comment_new ();
	comment = _tmp0_;
	_tmp1_ = comment;
	valadoc_content_content_element_set_parent ((ValadocContentContentElement*) _tmp1_, new_parent);
	{
		ValaList* _element_list = NULL;
		ValaList* _tmp2_;
		ValaList* _tmp3_;
		gint _element_size = 0;
		ValaList* _tmp4_;
		gint _tmp5_;
		gint _tmp6_;
		gint _element_index = 0;
		_tmp2_ = valadoc_content_block_content_get_content ((ValadocContentBlockContent*) self);
		_tmp3_ = _tmp2_;
		_element_list = _tmp3_;
		_tmp4_ = _element_list;
		_tmp5_ = vala_collection_get_size ((ValaCollection*) _tmp4_);
		_tmp6_ = _tmp5_;
		_element_size = _tmp6_;
		_element_index = -1;
		while (TRUE) {
			gint _tmp7_;
			gint _tmp8_;
			ValadocContentBlock* element = NULL;
			ValaList* _tmp9_;
			gpointer _tmp10_;
			ValadocContentBlock* copy = NULL;
			ValadocContentBlock* _tmp11_;
			ValadocContentComment* _tmp12_;
			ValadocContentContentElement* _tmp13_;
			ValadocContentBlock* _tmp14_;
			ValadocContentComment* _tmp15_;
			ValaList* _tmp16_;
			ValaList* _tmp17_;
			ValadocContentBlock* _tmp18_;
			_element_index = _element_index + 1;
			_tmp7_ = _element_index;
			_tmp8_ = _element_size;
			if (!(_tmp7_ < _tmp8_)) {
				break;
			}
			_tmp9_ = _element_list;
			_tmp10_ = vala_list_get (_tmp9_, _element_index);
			element = (ValadocContentBlock*) _tmp10_;
			_tmp11_ = element;
			_tmp12_ = comment;
			_tmp13_ = valadoc_content_content_element_copy ((ValadocContentContentElement*) _tmp11_, (ValadocContentContentElement*) _tmp12_);
			_tmp14_ = VALADOC_CONTENT_IS_BLOCK (_tmp13_) ? ((ValadocContentBlock*) _tmp13_) : NULL;
			if (_tmp14_ == NULL) {
				_g_object_unref0 (_tmp13_);
			}
			copy = _tmp14_;
			_tmp15_ = comment;
			_tmp16_ = valadoc_content_block_content_get_content ((ValadocContentBlockContent*) _tmp15_);
			_tmp17_ = _tmp16_;
			_tmp18_ = copy;
			vala_collection_add ((ValaCollection*) _tmp17_, _tmp18_);
			_g_object_unref0 (copy);
			_g_object_unref0 (element);
		}
	}
	{
		ValaList* _taglet_list = NULL;
		ValaList* _tmp19_;
		gint _taglet_size = 0;
		ValaList* _tmp20_;
		gint _tmp21_;
		gint _tmp22_;
		gint _taglet_index = 0;
		_tmp19_ = self->priv->_taglets;
		_taglet_list = _tmp19_;
		_tmp20_ = _taglet_list;
		_tmp21_ = vala_collection_get_size ((ValaCollection*) _tmp20_);
		_tmp22_ = _tmp21_;
		_taglet_size = _tmp22_;
		_taglet_index = -1;
		while (TRUE) {
			gint _tmp23_;
			gint _tmp24_;
			ValadocContentTaglet* taglet = NULL;
			ValaList* _tmp25_;
			gpointer _tmp26_;
			ValadocContentTaglet* copy = NULL;
			ValadocContentTaglet* _tmp27_;
			ValadocContentComment* _tmp28_;
			ValadocContentContentElement* _tmp29_;
			ValadocContentTaglet* _tmp30_;
			ValadocContentComment* _tmp31_;
			ValaList* _tmp32_;
			ValadocContentTaglet* _tmp33_;
			_taglet_index = _taglet_index + 1;
			_tmp23_ = _taglet_index;
			_tmp24_ = _taglet_size;
			if (!(_tmp23_ < _tmp24_)) {
				break;
			}
			_tmp25_ = _taglet_list;
			_tmp26_ = vala_list_get (_tmp25_, _taglet_index);
			taglet = (ValadocContentTaglet*) _tmp26_;
			_tmp27_ = taglet;
			_tmp28_ = comment;
			_tmp29_ = valadoc_content_content_element_copy ((ValadocContentContentElement*) _tmp27_, (ValadocContentContentElement*) _tmp28_);
			_tmp30_ = VALADOC_CONTENT_IS_TAGLET (_tmp29_) ? ((ValadocContentTaglet*) _tmp29_) : NULL;
			if (_tmp30_ == NULL) {
				_g_object_unref0 (_tmp29_);
			}
			copy = _tmp30_;
			_tmp31_ = comment;
			_tmp32_ = _tmp31_->priv->_taglets;
			_tmp33_ = copy;
			vala_collection_add ((ValaCollection*) _tmp32_, _tmp33_);
			_g_object_unref0 (copy);
			_g_object_unref0 (taglet);
		}
	}
	result = (ValadocContentContentElement*) comment;
	return result;
}

static void
valadoc_content_comment_class_init (ValadocContentCommentClass * klass,
                                    gpointer klass_data)
{
	valadoc_content_comment_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &ValadocContentComment_private_offset);
	((ValadocContentContentElementClass *) klass)->configure = (void (*) (ValadocContentContentElement*, ValadocSettings*, ValadocResourceLocator*)) valadoc_content_comment_real_configure;
	((ValadocContentContentElementClass *) klass)->check = (void (*) (ValadocContentContentElement*, ValadocApiTree*, ValadocApiNode*, const gchar*, ValadocErrorReporter*, ValadocSettings*)) valadoc_content_comment_real_check;
	((ValadocContentContentElementClass *) klass)->accept = (void (*) (ValadocContentContentElement*, ValadocContentContentVisitor*)) valadoc_content_comment_real_accept;
	((ValadocContentContentElementClass *) klass)->accept_children = (void (*) (ValadocContentContentElement*, ValadocContentContentVisitor*)) valadoc_content_comment_real_accept_children;
	((ValadocContentContentElementClass *) klass)->copy = (ValadocContentContentElement* (*) (ValadocContentContentElement*, ValadocContentContentElement*)) valadoc_content_comment_real_copy;
	G_OBJECT_CLASS (klass)->get_property = _vala_valadoc_content_comment_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_valadoc_content_comment_set_property;
	G_OBJECT_CLASS (klass)->finalize = valadoc_content_comment_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALADOC_CONTENT_COMMENT_TAGLETS_PROPERTY, valadoc_content_comment_properties[VALADOC_CONTENT_COMMENT_TAGLETS_PROPERTY] = vala_param_spec_iterable ("taglets", "taglets", "taglets", VALA_TYPE_LIST, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
}

static void
valadoc_content_comment_instance_init (ValadocContentComment * self,
                                       gpointer klass)
{
	self->priv = valadoc_content_comment_get_instance_private (self);
	self->priv->checked = FALSE;
}

static void
valadoc_content_comment_finalize (GObject * obj)
{
	ValadocContentComment * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, VALADOC_CONTENT_TYPE_COMMENT, ValadocContentComment);
	_vala_iterable_unref0 (self->priv->_taglets);
	G_OBJECT_CLASS (valadoc_content_comment_parent_class)->finalize (obj);
}

static GType
valadoc_content_comment_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (ValadocContentCommentClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valadoc_content_comment_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValadocContentComment), 0, (GInstanceInitFunc) valadoc_content_comment_instance_init, NULL };
	GType valadoc_content_comment_type_id;
	valadoc_content_comment_type_id = g_type_register_static (VALADOC_CONTENT_TYPE_BLOCK_CONTENT, "ValadocContentComment", &g_define_type_info, 0);
	ValadocContentComment_private_offset = g_type_add_instance_private (valadoc_content_comment_type_id, sizeof (ValadocContentCommentPrivate));
	return valadoc_content_comment_type_id;
}

GType
valadoc_content_comment_get_type (void)
{
	static volatile gsize valadoc_content_comment_type_id__once = 0;
	if (g_once_init_enter (&valadoc_content_comment_type_id__once)) {
		GType valadoc_content_comment_type_id;
		valadoc_content_comment_type_id = valadoc_content_comment_get_type_once ();
		g_once_init_leave (&valadoc_content_comment_type_id__once, valadoc_content_comment_type_id);
	}
	return valadoc_content_comment_type_id__once;
}

static void
_vala_valadoc_content_comment_get_property (GObject * object,
                                            guint property_id,
                                            GValue * value,
                                            GParamSpec * pspec)
{
	ValadocContentComment * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, VALADOC_CONTENT_TYPE_COMMENT, ValadocContentComment);
	switch (property_id) {
		case VALADOC_CONTENT_COMMENT_TAGLETS_PROPERTY:
		vala_value_set_iterable (value, valadoc_content_comment_get_taglets (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_valadoc_content_comment_set_property (GObject * object,
                                            guint property_id,
                                            const GValue * value,
                                            GParamSpec * pspec)
{
	ValadocContentComment * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, VALADOC_CONTENT_TYPE_COMMENT, ValadocContentComment);
	switch (property_id) {
		case VALADOC_CONTENT_COMMENT_TAGLETS_PROPERTY:
		valadoc_content_comment_set_taglets (self, vala_value_get_iterable (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

