/* tagletinit.c generated by valac, the Vala compiler
 * generated from tagletinit.vala, do not modify */

/* tagletinit.vala
 *
 * Copyright (C) 2008-2009 Florian Brosch
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Florian Brosch <flo.brosch@gmail.com>
 */

#include "valadoc.h"
#include <glib.h>
#include <glib-object.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

void
valadoc_taglets_init (ValadocModuleLoader* loader)
{
	g_return_if_fail (loader != NULL);
	valadoc_module_loader_register_taglet (loader, "see", VALADOC_TAGLETS_TYPE_SEE);
	valadoc_module_loader_register_taglet (loader, "since", VALADOC_TAGLETS_TYPE_SINCE);
	valadoc_module_loader_register_taglet (loader, "link", VALADOC_TAGLETS_TYPE_LINK);
	valadoc_module_loader_register_taglet (loader, "throws", VALADOC_TAGLETS_TYPE_THROWS);
	valadoc_module_loader_register_taglet (loader, "return", VALADOC_TAGLETS_TYPE_RETURN);
	valadoc_module_loader_register_taglet (loader, "param", VALADOC_TAGLETS_TYPE_PARAM);
	valadoc_module_loader_register_taglet (loader, "deprecated", VALADOC_TAGLETS_TYPE_DEPRECATED);
	valadoc_module_loader_register_taglet (loader, "inheritDoc", VALADOC_TAGLETS_TYPE_INHERIT_DOC);
}

