/* contentfactory.c generated by valac, the Vala compiler
 * generated from contentfactory.vala, do not modify */

/* contentfactory.vala
 *
 * Copyright (C) 2008-2009 Didier Villevalois
 * Copyright (C) 2008-2012 Florian Brosch
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Didier 'Ptitjes Villevalois <ptitjes@free.fr>
 */

#include "valadoc.h"
#include <glib-object.h>
#include <glib.h>
#include <stdlib.h>
#include <string.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

enum  {
	VALADOC_CONTENT_CONTENT_FACTORY_0_PROPERTY,
	VALADOC_CONTENT_CONTENT_FACTORY_NUM_PROPERTIES
};
static GParamSpec* valadoc_content_content_factory_properties[VALADOC_CONTENT_CONTENT_FACTORY_NUM_PROPERTIES];
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

struct _ValadocContentContentFactoryPrivate {
	ValadocSettings* _settings;
	ValadocResourceLocator* _locator;
	ValadocModuleLoader* _modules;
};

static gint ValadocContentContentFactory_private_offset;
static gpointer valadoc_content_content_factory_parent_class = NULL;

static inline ValadocContentContentElement* valadoc_content_content_factory_configure (ValadocContentContentFactory* self,
                                                                         ValadocContentContentElement* element);
 G_GNUC_INTERNAL ValadocContentComment* valadoc_content_comment_new (void);
 G_GNUC_INTERNAL ValadocContentComment* valadoc_content_comment_construct (GType object_type);
 G_GNUC_INTERNAL ValadocContentEmbedded* valadoc_content_embedded_new (void);
 G_GNUC_INTERNAL ValadocContentEmbedded* valadoc_content_embedded_construct (GType object_type);
 G_GNUC_INTERNAL ValadocContentHeadline* valadoc_content_headline_new (void);
 G_GNUC_INTERNAL ValadocContentHeadline* valadoc_content_headline_construct (GType object_type);
 G_GNUC_INTERNAL ValadocContentLink* valadoc_content_link_new (void);
 G_GNUC_INTERNAL ValadocContentLink* valadoc_content_link_construct (GType object_type);
 G_GNUC_INTERNAL ValadocContentWikiLink* valadoc_content_wiki_link_new (void);
 G_GNUC_INTERNAL ValadocContentWikiLink* valadoc_content_wiki_link_construct (GType object_type);
 G_GNUC_INTERNAL ValadocContentList* valadoc_content_list_new (void);
 G_GNUC_INTERNAL ValadocContentList* valadoc_content_list_construct (GType object_type);
 G_GNUC_INTERNAL ValadocContentListItem* valadoc_content_list_item_new (void);
 G_GNUC_INTERNAL ValadocContentListItem* valadoc_content_list_item_construct (GType object_type);
 G_GNUC_INTERNAL ValadocContentPage* valadoc_content_page_new (void);
 G_GNUC_INTERNAL ValadocContentPage* valadoc_content_page_construct (GType object_type);
 G_GNUC_INTERNAL ValadocContentParagraph* valadoc_content_paragraph_new (void);
 G_GNUC_INTERNAL ValadocContentParagraph* valadoc_content_paragraph_construct (GType object_type);
 G_GNUC_INTERNAL ValadocContentWarning* valadoc_content_warning_new (void);
 G_GNUC_INTERNAL ValadocContentWarning* valadoc_content_warning_construct (GType object_type);
 G_GNUC_INTERNAL ValadocContentNote* valadoc_content_note_new (void);
 G_GNUC_INTERNAL ValadocContentNote* valadoc_content_note_construct (GType object_type);
 G_GNUC_INTERNAL ValadocContentRun* valadoc_content_run_new (ValadocContentRunStyle style);
 G_GNUC_INTERNAL ValadocContentRun* valadoc_content_run_construct (GType object_type,
                                                  ValadocContentRunStyle style);
 G_GNUC_INTERNAL ValadocContentSourceCode* valadoc_content_source_code_new (void);
 G_GNUC_INTERNAL ValadocContentSourceCode* valadoc_content_source_code_construct (GType object_type);
 G_GNUC_INTERNAL ValadocContentTable* valadoc_content_table_new (void);
 G_GNUC_INTERNAL ValadocContentTable* valadoc_content_table_construct (GType object_type);
 G_GNUC_INTERNAL ValadocContentTableCell* valadoc_content_table_cell_new (void);
 G_GNUC_INTERNAL ValadocContentTableCell* valadoc_content_table_cell_construct (GType object_type);
 G_GNUC_INTERNAL ValadocContentTableRow* valadoc_content_table_row_new (void);
 G_GNUC_INTERNAL ValadocContentTableRow* valadoc_content_table_row_construct (GType object_type);
 G_GNUC_INTERNAL ValadocContentText* valadoc_content_text_new (const gchar* text);
 G_GNUC_INTERNAL ValadocContentText* valadoc_content_text_construct (GType object_type,
                                                    const gchar* text);
static void valadoc_content_content_factory_finalize (GObject * obj);
static GType valadoc_content_content_factory_get_type_once (void);

static inline gpointer
valadoc_content_content_factory_get_instance_private (ValadocContentContentFactory* self)
{
	return G_STRUCT_MEMBER_P (self, ValadocContentContentFactory_private_offset);
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

ValadocContentContentFactory*
valadoc_content_content_factory_construct (GType object_type,
                                           ValadocSettings* settings,
                                           ValadocResourceLocator* locator,
                                           ValadocModuleLoader* modules)
{
	ValadocContentContentFactory * self = NULL;
	ValadocSettings* _tmp0_;
	ValadocResourceLocator* _tmp1_;
	ValadocModuleLoader* _tmp2_;
	g_return_val_if_fail (settings != NULL, NULL);
	g_return_val_if_fail (locator != NULL, NULL);
	g_return_val_if_fail (modules != NULL, NULL);
	self = (ValadocContentContentFactory*) g_object_new (object_type, NULL);
	_tmp0_ = _g_object_ref0 (settings);
	_g_object_unref0 (self->priv->_settings);
	self->priv->_settings = _tmp0_;
	_tmp1_ = _g_object_ref0 (locator);
	_g_object_unref0 (self->priv->_locator);
	self->priv->_locator = _tmp1_;
	_tmp2_ = _g_object_ref0 (modules);
	_g_object_unref0 (self->priv->_modules);
	self->priv->_modules = _tmp2_;
	return self;
}

ValadocContentContentFactory*
valadoc_content_content_factory_new (ValadocSettings* settings,
                                     ValadocResourceLocator* locator,
                                     ValadocModuleLoader* modules)
{
	return valadoc_content_content_factory_construct (VALADOC_CONTENT_TYPE_CONTENT_FACTORY, settings, locator, modules);
}

static inline ValadocContentContentElement*
valadoc_content_content_factory_configure (ValadocContentContentFactory* self,
                                           ValadocContentContentElement* element)
{
	ValadocSettings* _tmp0_;
	ValadocResourceLocator* _tmp1_;
	ValadocContentContentElement* _tmp2_;
	ValadocContentContentElement* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (element != NULL, NULL);
	_tmp0_ = self->priv->_settings;
	_tmp1_ = self->priv->_locator;
	valadoc_content_content_element_configure (element, _tmp0_, _tmp1_);
	_tmp2_ = _g_object_ref0 (element);
	result = _tmp2_;
	return result;
}

ValadocContentComment*
valadoc_content_content_factory_create_comment (ValadocContentContentFactory* self)
{
	ValadocContentComment* _tmp0_;
	ValadocContentComment* _tmp1_;
	ValadocContentContentElement* _tmp2_;
	ValadocContentComment* _tmp3_;
	ValadocContentComment* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = valadoc_content_comment_new ();
	_tmp1_ = _tmp0_;
	_tmp2_ = valadoc_content_content_factory_configure (self, (ValadocContentContentElement*) _tmp1_);
	_tmp3_ = G_TYPE_CHECK_INSTANCE_CAST (_tmp2_, VALADOC_CONTENT_TYPE_COMMENT, ValadocContentComment);
	_g_object_unref0 (_tmp1_);
	result = _tmp3_;
	return result;
}

ValadocContentEmbedded*
valadoc_content_content_factory_create_embedded (ValadocContentContentFactory* self)
{
	ValadocContentEmbedded* _tmp0_;
	ValadocContentEmbedded* _tmp1_;
	ValadocContentContentElement* _tmp2_;
	ValadocContentEmbedded* _tmp3_;
	ValadocContentEmbedded* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = valadoc_content_embedded_new ();
	_tmp1_ = _tmp0_;
	_tmp2_ = valadoc_content_content_factory_configure (self, (ValadocContentContentElement*) _tmp1_);
	_tmp3_ = G_TYPE_CHECK_INSTANCE_CAST (_tmp2_, VALADOC_CONTENT_TYPE_EMBEDDED, ValadocContentEmbedded);
	_g_object_unref0 (_tmp1_);
	result = _tmp3_;
	return result;
}

ValadocContentHeadline*
valadoc_content_content_factory_create_headline (ValadocContentContentFactory* self)
{
	ValadocContentHeadline* _tmp0_;
	ValadocContentHeadline* _tmp1_;
	ValadocContentContentElement* _tmp2_;
	ValadocContentHeadline* _tmp3_;
	ValadocContentHeadline* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = valadoc_content_headline_new ();
	_tmp1_ = _tmp0_;
	_tmp2_ = valadoc_content_content_factory_configure (self, (ValadocContentContentElement*) _tmp1_);
	_tmp3_ = G_TYPE_CHECK_INSTANCE_CAST (_tmp2_, VALADOC_CONTENT_TYPE_HEADLINE, ValadocContentHeadline);
	_g_object_unref0 (_tmp1_);
	result = _tmp3_;
	return result;
}

ValadocContentLink*
valadoc_content_content_factory_create_link (ValadocContentContentFactory* self)
{
	ValadocContentLink* _tmp0_;
	ValadocContentLink* _tmp1_;
	ValadocContentContentElement* _tmp2_;
	ValadocContentLink* _tmp3_;
	ValadocContentLink* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = valadoc_content_link_new ();
	_tmp1_ = _tmp0_;
	_tmp2_ = valadoc_content_content_factory_configure (self, (ValadocContentContentElement*) _tmp1_);
	_tmp3_ = G_TYPE_CHECK_INSTANCE_CAST (_tmp2_, VALADOC_CONTENT_TYPE_LINK, ValadocContentLink);
	_g_object_unref0 (_tmp1_);
	result = _tmp3_;
	return result;
}

ValadocContentWikiLink*
valadoc_content_content_factory_create_wiki_link (ValadocContentContentFactory* self)
{
	ValadocContentWikiLink* _tmp0_;
	ValadocContentWikiLink* _tmp1_;
	ValadocContentContentElement* _tmp2_;
	ValadocContentWikiLink* _tmp3_;
	ValadocContentWikiLink* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = valadoc_content_wiki_link_new ();
	_tmp1_ = _tmp0_;
	_tmp2_ = valadoc_content_content_factory_configure (self, (ValadocContentContentElement*) _tmp1_);
	_tmp3_ = G_TYPE_CHECK_INSTANCE_CAST (_tmp2_, VALADOC_CONTENT_TYPE_WIKI_LINK, ValadocContentWikiLink);
	_g_object_unref0 (_tmp1_);
	result = _tmp3_;
	return result;
}

ValadocContentList*
valadoc_content_content_factory_create_list (ValadocContentContentFactory* self)
{
	ValadocContentList* _tmp0_;
	ValadocContentList* _tmp1_;
	ValadocContentContentElement* _tmp2_;
	ValadocContentList* _tmp3_;
	ValadocContentList* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = valadoc_content_list_new ();
	_tmp1_ = _tmp0_;
	_tmp2_ = valadoc_content_content_factory_configure (self, (ValadocContentContentElement*) _tmp1_);
	_tmp3_ = G_TYPE_CHECK_INSTANCE_CAST (_tmp2_, VALADOC_CONTENT_TYPE_LIST, ValadocContentList);
	_g_object_unref0 (_tmp1_);
	result = _tmp3_;
	return result;
}

ValadocContentListItem*
valadoc_content_content_factory_create_list_item (ValadocContentContentFactory* self)
{
	ValadocContentListItem* _tmp0_;
	ValadocContentListItem* _tmp1_;
	ValadocContentContentElement* _tmp2_;
	ValadocContentListItem* _tmp3_;
	ValadocContentListItem* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = valadoc_content_list_item_new ();
	_tmp1_ = _tmp0_;
	_tmp2_ = valadoc_content_content_factory_configure (self, (ValadocContentContentElement*) _tmp1_);
	_tmp3_ = G_TYPE_CHECK_INSTANCE_CAST (_tmp2_, VALADOC_CONTENT_TYPE_LIST_ITEM, ValadocContentListItem);
	_g_object_unref0 (_tmp1_);
	result = _tmp3_;
	return result;
}

ValadocContentPage*
valadoc_content_content_factory_create_page (ValadocContentContentFactory* self)
{
	ValadocContentPage* _tmp0_;
	ValadocContentPage* _tmp1_;
	ValadocContentContentElement* _tmp2_;
	ValadocContentPage* _tmp3_;
	ValadocContentPage* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = valadoc_content_page_new ();
	_tmp1_ = _tmp0_;
	_tmp2_ = valadoc_content_content_factory_configure (self, (ValadocContentContentElement*) _tmp1_);
	_tmp3_ = G_TYPE_CHECK_INSTANCE_CAST (_tmp2_, VALADOC_CONTENT_TYPE_PAGE, ValadocContentPage);
	_g_object_unref0 (_tmp1_);
	result = _tmp3_;
	return result;
}

ValadocContentParagraph*
valadoc_content_content_factory_create_paragraph (ValadocContentContentFactory* self)
{
	ValadocContentParagraph* _tmp0_;
	ValadocContentParagraph* _tmp1_;
	ValadocContentContentElement* _tmp2_;
	ValadocContentParagraph* _tmp3_;
	ValadocContentParagraph* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = valadoc_content_paragraph_new ();
	_tmp1_ = _tmp0_;
	_tmp2_ = valadoc_content_content_factory_configure (self, (ValadocContentContentElement*) _tmp1_);
	_tmp3_ = G_TYPE_CHECK_INSTANCE_CAST (_tmp2_, VALADOC_CONTENT_TYPE_PARAGRAPH, ValadocContentParagraph);
	_g_object_unref0 (_tmp1_);
	result = _tmp3_;
	return result;
}

ValadocContentWarning*
valadoc_content_content_factory_create_warning (ValadocContentContentFactory* self)
{
	ValadocContentWarning* _tmp0_;
	ValadocContentWarning* _tmp1_;
	ValadocContentContentElement* _tmp2_;
	ValadocContentWarning* _tmp3_;
	ValadocContentWarning* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = valadoc_content_warning_new ();
	_tmp1_ = _tmp0_;
	_tmp2_ = valadoc_content_content_factory_configure (self, (ValadocContentContentElement*) _tmp1_);
	_tmp3_ = G_TYPE_CHECK_INSTANCE_CAST (_tmp2_, VALADOC_CONTENT_TYPE_WARNING, ValadocContentWarning);
	_g_object_unref0 (_tmp1_);
	result = _tmp3_;
	return result;
}

ValadocContentNote*
valadoc_content_content_factory_create_note (ValadocContentContentFactory* self)
{
	ValadocContentNote* _tmp0_;
	ValadocContentNote* _tmp1_;
	ValadocContentContentElement* _tmp2_;
	ValadocContentNote* _tmp3_;
	ValadocContentNote* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = valadoc_content_note_new ();
	_tmp1_ = _tmp0_;
	_tmp2_ = valadoc_content_content_factory_configure (self, (ValadocContentContentElement*) _tmp1_);
	_tmp3_ = G_TYPE_CHECK_INSTANCE_CAST (_tmp2_, VALADOC_CONTENT_TYPE_NOTE, ValadocContentNote);
	_g_object_unref0 (_tmp1_);
	result = _tmp3_;
	return result;
}

ValadocContentRun*
valadoc_content_content_factory_create_run (ValadocContentContentFactory* self,
                                            ValadocContentRunStyle style)
{
	ValadocContentRun* _tmp0_;
	ValadocContentRun* _tmp1_;
	ValadocContentContentElement* _tmp2_;
	ValadocContentRun* _tmp3_;
	ValadocContentRun* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = valadoc_content_run_new (style);
	_tmp1_ = _tmp0_;
	_tmp2_ = valadoc_content_content_factory_configure (self, (ValadocContentContentElement*) _tmp1_);
	_tmp3_ = G_TYPE_CHECK_INSTANCE_CAST (_tmp2_, VALADOC_CONTENT_TYPE_RUN, ValadocContentRun);
	_g_object_unref0 (_tmp1_);
	result = _tmp3_;
	return result;
}

ValadocContentSourceCode*
valadoc_content_content_factory_create_source_code (ValadocContentContentFactory* self)
{
	ValadocContentSourceCode* _tmp0_;
	ValadocContentSourceCode* _tmp1_;
	ValadocContentContentElement* _tmp2_;
	ValadocContentSourceCode* _tmp3_;
	ValadocContentSourceCode* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = valadoc_content_source_code_new ();
	_tmp1_ = _tmp0_;
	_tmp2_ = valadoc_content_content_factory_configure (self, (ValadocContentContentElement*) _tmp1_);
	_tmp3_ = G_TYPE_CHECK_INSTANCE_CAST (_tmp2_, VALADOC_CONTENT_TYPE_SOURCE_CODE, ValadocContentSourceCode);
	_g_object_unref0 (_tmp1_);
	result = _tmp3_;
	return result;
}

ValadocContentTable*
valadoc_content_content_factory_create_table (ValadocContentContentFactory* self)
{
	ValadocContentTable* _tmp0_;
	ValadocContentTable* _tmp1_;
	ValadocContentContentElement* _tmp2_;
	ValadocContentTable* _tmp3_;
	ValadocContentTable* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = valadoc_content_table_new ();
	_tmp1_ = _tmp0_;
	_tmp2_ = valadoc_content_content_factory_configure (self, (ValadocContentContentElement*) _tmp1_);
	_tmp3_ = G_TYPE_CHECK_INSTANCE_CAST (_tmp2_, VALADOC_CONTENT_TYPE_TABLE, ValadocContentTable);
	_g_object_unref0 (_tmp1_);
	result = _tmp3_;
	return result;
}

ValadocContentTableCell*
valadoc_content_content_factory_create_table_cell (ValadocContentContentFactory* self)
{
	ValadocContentTableCell* _tmp0_;
	ValadocContentTableCell* _tmp1_;
	ValadocContentContentElement* _tmp2_;
	ValadocContentTableCell* _tmp3_;
	ValadocContentTableCell* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = valadoc_content_table_cell_new ();
	_tmp1_ = _tmp0_;
	_tmp2_ = valadoc_content_content_factory_configure (self, (ValadocContentContentElement*) _tmp1_);
	_tmp3_ = G_TYPE_CHECK_INSTANCE_CAST (_tmp2_, VALADOC_CONTENT_TYPE_TABLE_CELL, ValadocContentTableCell);
	_g_object_unref0 (_tmp1_);
	result = _tmp3_;
	return result;
}

ValadocContentTableRow*
valadoc_content_content_factory_create_table_row (ValadocContentContentFactory* self)
{
	ValadocContentTableRow* _tmp0_;
	ValadocContentTableRow* _tmp1_;
	ValadocContentContentElement* _tmp2_;
	ValadocContentTableRow* _tmp3_;
	ValadocContentTableRow* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = valadoc_content_table_row_new ();
	_tmp1_ = _tmp0_;
	_tmp2_ = valadoc_content_content_factory_configure (self, (ValadocContentContentElement*) _tmp1_);
	_tmp3_ = G_TYPE_CHECK_INSTANCE_CAST (_tmp2_, VALADOC_CONTENT_TYPE_TABLE_ROW, ValadocContentTableRow);
	_g_object_unref0 (_tmp1_);
	result = _tmp3_;
	return result;
}

ValadocContentTaglet*
valadoc_content_content_factory_create_taglet (ValadocContentContentFactory* self,
                                               const gchar* name)
{
	ValadocModuleLoader* _tmp0_;
	ValadocContentTaglet* _tmp1_;
	ValadocContentTaglet* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (name != NULL, NULL);
	_tmp0_ = self->priv->_modules;
	_tmp1_ = valadoc_module_loader_create_taglet (_tmp0_, name);
	result = _tmp1_;
	return result;
}

ValadocContentText*
valadoc_content_content_factory_create_text (ValadocContentContentFactory* self,
                                             const gchar* text)
{
	ValadocContentText* _tmp0_;
	ValadocContentText* _tmp1_;
	ValadocContentContentElement* _tmp2_;
	ValadocContentText* _tmp3_;
	ValadocContentText* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = valadoc_content_text_new (text);
	_tmp1_ = _tmp0_;
	_tmp2_ = valadoc_content_content_factory_configure (self, (ValadocContentContentElement*) _tmp1_);
	_tmp3_ = G_TYPE_CHECK_INSTANCE_CAST (_tmp2_, VALADOC_CONTENT_TYPE_TEXT, ValadocContentText);
	_g_object_unref0 (_tmp1_);
	result = _tmp3_;
	return result;
}

ValadocContentContentElement*
valadoc_content_content_factory_set_style_attributes (ValadocContentContentFactory* self,
                                                      ValadocContentStyleAttributes* element,
                                                      ValadocContentVerticalAlign* valign,
                                                      ValadocContentHorizontalAlign* halign,
                                                      const gchar* style)
{
	ValadocContentContentElement* _tmp0_;
	ValadocContentContentElement* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (element != NULL, NULL);
	valadoc_content_style_attributes_set_vertical_align (element, *valign);
	valadoc_content_style_attributes_set_horizontal_align (element, *halign);
	valadoc_content_style_attributes_set_style (element, style);
	_tmp0_ = _g_object_ref0 ((ValadocContentContentElement*) element);
	result = _tmp0_;
	return result;
}

static void
valadoc_content_content_factory_class_init (ValadocContentContentFactoryClass * klass,
                                            gpointer klass_data)
{
	valadoc_content_content_factory_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &ValadocContentContentFactory_private_offset);
	G_OBJECT_CLASS (klass)->finalize = valadoc_content_content_factory_finalize;
}

static void
valadoc_content_content_factory_instance_init (ValadocContentContentFactory * self,
                                               gpointer klass)
{
	self->priv = valadoc_content_content_factory_get_instance_private (self);
}

static void
valadoc_content_content_factory_finalize (GObject * obj)
{
	ValadocContentContentFactory * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, VALADOC_CONTENT_TYPE_CONTENT_FACTORY, ValadocContentContentFactory);
	_g_object_unref0 (self->priv->_settings);
	_g_object_unref0 (self->priv->_locator);
	_g_object_unref0 (self->priv->_modules);
	G_OBJECT_CLASS (valadoc_content_content_factory_parent_class)->finalize (obj);
}

static GType
valadoc_content_content_factory_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (ValadocContentContentFactoryClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valadoc_content_content_factory_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValadocContentContentFactory), 0, (GInstanceInitFunc) valadoc_content_content_factory_instance_init, NULL };
	GType valadoc_content_content_factory_type_id;
	valadoc_content_content_factory_type_id = g_type_register_static (G_TYPE_OBJECT, "ValadocContentContentFactory", &g_define_type_info, 0);
	ValadocContentContentFactory_private_offset = g_type_add_instance_private (valadoc_content_content_factory_type_id, sizeof (ValadocContentContentFactoryPrivate));
	return valadoc_content_content_factory_type_id;
}

GType
valadoc_content_content_factory_get_type (void)
{
	static volatile gsize valadoc_content_content_factory_type_id__once = 0;
	if (g_once_init_enter (&valadoc_content_content_factory_type_id__once)) {
		GType valadoc_content_content_factory_type_id;
		valadoc_content_content_factory_type_id = valadoc_content_content_factory_get_type_once ();
		g_once_init_leave (&valadoc_content_content_factory_type_id__once, valadoc_content_content_factory_type_id);
	}
	return valadoc_content_content_factory_type_id__once;
}

