/* valaparser.c generated by valac, the Vala compiler
 * generated from valaparser.vala, do not modify */

/* valaparser.vala
 *
 * Copyright (C) 2006-2013  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include "vala.h"
#include <glib-object.h>
#include <glib.h>
#include <valagee.h>
#include <stdlib.h>
#include <string.h>

#define VALA_PARSER_BUFFER_SIZE 32
#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

#define VALA_PARSER_TYPE_TOKEN_INFO (vala_parser_token_info_get_type ())
typedef struct _ValaParserTokenInfo ValaParserTokenInfo;

#define VALA_PARSER_TYPE_PARTIAL_INFO (vala_parser_partial_info_get_type ())
typedef struct _ValaParserPartialInfo ValaParserPartialInfo;
typedef enum  {
	VALA_PARSER_MODIFIER_FLAGS_NONE = 0,
	VALA_PARSER_MODIFIER_FLAGS_ABSTRACT = 1 << 0,
	VALA_PARSER_MODIFIER_FLAGS_CLASS = 1 << 1,
	VALA_PARSER_MODIFIER_FLAGS_EXTERN = 1 << 2,
	VALA_PARSER_MODIFIER_FLAGS_INLINE = 1 << 3,
	VALA_PARSER_MODIFIER_FLAGS_NEW = 1 << 4,
	VALA_PARSER_MODIFIER_FLAGS_OVERRIDE = 1 << 5,
	VALA_PARSER_MODIFIER_FLAGS_STATIC = 1 << 6,
	VALA_PARSER_MODIFIER_FLAGS_VIRTUAL = 1 << 7,
	VALA_PARSER_MODIFIER_FLAGS_ASYNC = 1 << 8,
	VALA_PARSER_MODIFIER_FLAGS_SEALED = 1 << 9,
	VALA_PARSER_MODIFIER_FLAGS_PARTIAL = 1 << 10
} ValaParserModifierFlags;

#define VALA_PARSER_TYPE_MODIFIER_FLAGS (vala_parser_modifier_flags_get_type ())
typedef enum  {
	VALA_PARSER_RECOVERY_STATE_EOF,
	VALA_PARSER_RECOVERY_STATE_DECLARATION_BEGIN,
	VALA_PARSER_RECOVERY_STATE_STATEMENT_BEGIN
} ValaParserRecoveryState;

#define VALA_PARSER_TYPE_RECOVERY_STATE (vala_parser_recovery_state_get_type ())
#define _vala_scanner_unref0(var) ((var == NULL) ? NULL : (var = (vala_scanner_unref (var), NULL)))
#define _vala_code_context_unref0(var) ((var == NULL) ? NULL : (var = (vala_code_context_unref (var), NULL)))
#define _vala_comment_unref0(var) ((var == NULL) ? NULL : (var = (vala_comment_unref (var), NULL)))
#define _vala_source_reference_unref0(var) ((var == NULL) ? NULL : (var = (vala_source_reference_unref (var), NULL)))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))
#define _vala_code_node_unref0(var) ((var == NULL) ? NULL : (var = (vala_code_node_unref (var), NULL)))
#define _vala_iterable_unref0(var) ((var == NULL) ? NULL : (var = (vala_iterable_unref (var), NULL)))
#define _g_type_class_unref0(var) ((var == NULL) ? NULL : (var = (g_type_class_unref (var), NULL)))
#define _vala_assert(expr, msg) if G_LIKELY (expr) ; else g_assertion_message_expr (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);
#define _vala_return_if_fail(expr, msg) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return; }
#define _vala_return_val_if_fail(expr, msg, val) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return val; }
#define _vala_warn_if_fail(expr, msg) if G_LIKELY (expr) ; else g_warn_message (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);

struct _ValaParserTokenInfo {
	ValaTokenType type;
	ValaSourceLocation begin;
	ValaSourceLocation end;
};

struct _ValaParserPartialInfo {
	ValaSymbol* parent;
	ValaSourceLocation begin;
	ValaList* attributes;
};

struct _ValaParserPrivate {
	ValaScanner* scanner;
	ValaCodeContext* context;
	ValaParserTokenInfo* tokens;
	gint tokens_length1;
	gint _tokens_size_;
	gint index;
	gint size;
	ValaComment* comment;
	ValaParserPartialInfo* partials;
	gint partials_length1;
	gint _partials_size_;
};

static gint ValaParser_private_offset;
static gpointer vala_parser_parent_class = NULL;
static ValaList* vala_parser__empty_type_parameter_list;
static ValaList* vala_parser__empty_type_parameter_list = NULL;
static gint vala_parser_next_local_func_id;
static gint vala_parser_next_local_func_id = 0;

static GType vala_parser_token_info_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
static ValaParserTokenInfo* vala_parser_token_info_dup (const ValaParserTokenInfo* self);
static void vala_parser_token_info_free (ValaParserTokenInfo* self);
static GType vala_parser_partial_info_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
static ValaParserPartialInfo* vala_parser_partial_info_dup (const ValaParserPartialInfo* self);
static void vala_parser_partial_info_free (ValaParserPartialInfo* self);
static void vala_parser_partial_info_copy (const ValaParserPartialInfo* self,
                                    ValaParserPartialInfo* dest);
static void vala_parser_partial_info_destroy (ValaParserPartialInfo* self);
static GType vala_parser_modifier_flags_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
static GType vala_parser_recovery_state_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
static void _vala_ValaParserPartialInfo_array_free (ValaParserPartialInfo * array,
                                             gssize array_length);
static void vala_parser_real_visit_source_file (ValaCodeVisitor* base,
                                         ValaSourceFile* source_file);
static inline gboolean vala_parser_next (ValaParser* self);
static inline void vala_parser_prev (ValaParser* self);
static inline void vala_parser_safe_prev (ValaParser* self);
static inline ValaTokenType vala_parser_previous (ValaParser* self);
static inline ValaTokenType vala_parser_current (ValaParser* self);
static inline gboolean vala_parser_accept (ValaParser* self,
                             ValaTokenType type);
static void vala_parser_report_parse_error (ValaParser* self,
                                     GError* e);
static inline void vala_parser_get_location (ValaParser* self,
                               ValaSourceLocation* result);
static ValaSourceReference* vala_parser_get_src (ValaParser* self,
                                          ValaSourceLocation* begin);
static inline gboolean vala_parser_expect (ValaParser* self,
                             ValaTokenType type,
                             GError** error);
static gchar* vala_parser_get_location_string (ValaParser* self);
static gchar* vala_parser_get_current_string (ValaParser* self);
static gchar* vala_parser_get_last_string (ValaParser* self);
static ValaSourceReference* vala_parser_get_current_src (ValaParser* self);
static ValaSourceReference* vala_parser_get_last_src (ValaParser* self);
static void vala_parser_rollback (ValaParser* self,
                           ValaSourceLocation* location);
static void vala_parser_jump (ValaParser* self,
                       ValaSourceLocation* location);
static void vala_parser_skip_identifier (ValaParser* self,
                                  GError** error);
static gchar* vala_parser_parse_identifier (ValaParser* self,
                                     gboolean required,
                                     GError** error);
static ValaExpression* vala_parser_parse_literal (ValaParser* self,
                                           GError** error);
static void vala_parser_parse_file_comments (ValaParser* self);
static void vala_parser_parse_using_directives (ValaParser* self,
                                         ValaNamespace* ns,
                                         GError** error);
static void vala_parser_parse_declarations (ValaParser* self,
                                     ValaSymbol* parent,
                                     gboolean root,
                                     GError** error);
static void vala_parser_parse_class_declaration (ValaParser* self,
                                          ValaSymbol* parent,
                                          ValaList* attrs,
                                          gboolean partial_reparse,
                                          GError** error);
static void vala_parser_skip_symbol_name (ValaParser* self,
                                   GError** error);
static ValaUnresolvedSymbol* vala_parser_parse_symbol_name (ValaParser* self,
                                                     GError** error);
static void vala_parser_skip_type (ValaParser* self,
                            GError** error);
static gboolean vala_parser_is_inner_array_type (ValaParser* self);
static void vala_parser_skip_type_argument_list (ValaParser* self,
                                          GError** error);
static ValaExpression* vala_parser_parse_expression (ValaParser* self,
                                              GError** error);
static ValaDataType* vala_parser_parse_type (ValaParser* self,
                                      gboolean owned_by_default,
                                      gboolean can_weak_ref,
                                      gboolean require_unowned,
                                      GError** error);
static ValaList* vala_parser_parse_type_argument_list (ValaParser* self,
                                                gboolean maybe_expression,
                                                GError** error);
static ValaDataType* vala_parser_parse_inline_array_type (ValaParser* self,
                                                   ValaDataType* type,
                                                   GError** error);
static ValaList* vala_parser_parse_argument_list (ValaParser* self,
                                           GError** error);
static ValaExpression* vala_parser_parse_argument (ValaParser* self,
                                            GError** error);
static ValaExpression* vala_parser_parse_primary_expression (ValaParser* self,
                                                      GError** error);
static ValaInitializerList* vala_parser_parse_initializer (ValaParser* self,
                                                    GError** error);
static ValaExpression* vala_parser_parse_simple_name (ValaParser* self,
                                               GError** error);
static ValaExpression* vala_parser_parse_tuple (ValaParser* self,
                                         GError** error);
static ValaExpression* vala_parser_parse_template (ValaParser* self,
                                            GError** error);
static ValaExpression* vala_parser_parse_regex_literal (ValaParser* self,
                                                 GError** error);
static ValaExpression* vala_parser_parse_this_access (ValaParser* self,
                                               GError** error);
static ValaExpression* vala_parser_parse_base_access (ValaParser* self,
                                               GError** error);
static ValaExpression* vala_parser_parse_object_or_array_creation_expression (ValaParser* self,
                                                                       GError** error);
static ValaExpression* vala_parser_parse_yield_expression (ValaParser* self,
                                                    GError** error);
static ValaExpression* vala_parser_parse_sizeof_expression (ValaParser* self,
                                                     GError** error);
static ValaExpression* vala_parser_parse_typeof_expression (ValaParser* self,
                                                     GError** error);
static ValaExpression* vala_parser_parse_member_access (ValaParser* self,
                                                 ValaSourceLocation* begin,
                                                 ValaExpression* inner,
                                                 GError** error);
static ValaExpression* vala_parser_parse_element_access (ValaParser* self,
                                                  ValaSourceLocation* begin,
                                                  ValaExpression* inner,
                                                  GError** error);
static ValaExpression* vala_parser_parse_pointer_member_access (ValaParser* self,
                                                         ValaSourceLocation* begin,
                                                         ValaExpression* inner,
                                                         GError** error);
static ValaExpression* vala_parser_parse_method_call (ValaParser* self,
                                               ValaSourceLocation* begin,
                                               ValaExpression* inner,
                                               GError** error);
static ValaExpression* vala_parser_parse_post_increment_expression (ValaParser* self,
                                                             ValaSourceLocation* begin,
                                                             ValaExpression* inner,
                                                             GError** error);
static ValaExpression* vala_parser_parse_post_decrement_expression (ValaParser* self,
                                                             ValaSourceLocation* begin,
                                                             ValaExpression* inner,
                                                             GError** error);
static ValaList* vala_parser_parse_object_initializer (ValaParser* self,
                                                GError** error);
static ValaList* vala_parser_parse_expression_list (ValaParser* self,
                                             GError** error);
static ValaExpression* vala_parser_parse_array_creation_expression (ValaParser* self,
                                                             GError** error);
static ValaMemberAccess* vala_parser_parse_member_name (ValaParser* self,
                                                 ValaExpression* base_expr,
                                                 GError** error);
static ValaExpression* vala_parser_parse_object_creation_expression (ValaParser* self,
                                                              ValaSourceLocation* begin,
                                                              ValaMemberAccess* member,
                                                              GError** error);
static ValaMemberInitializer* vala_parser_parse_member_initializer (ValaParser* self,
                                                             gboolean chained,
                                                             GError** error);
static ValaUnaryOperator vala_parser_get_unary_operator (ValaParser* self,
                                                  ValaTokenType token_type);
static ValaExpression* vala_parser_parse_unary_expression (ValaParser* self,
                                                    GError** error);
static ValaBinaryOperator vala_parser_get_binary_operator (ValaParser* self,
                                                    ValaTokenType token_type);
static ValaExpression* vala_parser_parse_multiplicative_expression (ValaParser* self,
                                                             GError** error);
static ValaExpression* vala_parser_parse_additive_expression (ValaParser* self,
                                                       GError** error);
static ValaExpression* vala_parser_parse_shift_expression (ValaParser* self,
                                                    GError** error);
static ValaExpression* vala_parser_parse_type_check_expression (ValaParser* self,
                                                         GError** error);
static ValaExpression* vala_parser_parse_relational_expression (ValaParser* self,
                                                         GError** error);
static ValaExpression* vala_parser_parse_and_expression (ValaParser* self,
                                                  GError** error);
static ValaExpression* vala_parser_parse_exclusive_or_expression (ValaParser* self,
                                                           GError** error);
static ValaExpression* vala_parser_parse_inclusive_or_expression (ValaParser* self,
                                                           GError** error);
static ValaExpression* vala_parser_parse_in_expression (ValaParser* self,
                                                 GError** error);
static ValaExpression* vala_parser_parse_conditional_and_expression (ValaParser* self,
                                                              GError** error);
static ValaExpression* vala_parser_parse_conditional_or_expression (ValaParser* self,
                                                             GError** error);
static ValaExpression* vala_parser_parse_coalescing_expression (ValaParser* self,
                                                         GError** error);
static ValaExpression* vala_parser_parse_conditional_expression (ValaParser* self,
                                                          GError** error);
static ValaParameter* vala_parser_parse_lambda_parameter (ValaParser* self,
                                                   GError** error);
static ValaExpression* vala_parser_parse_lambda_expression (ValaParser* self,
                                                     GError** error);
static ValaBlock* vala_parser_parse_block (ValaParser* self,
                                    GError** error);
static ValaAssignmentOperator vala_parser_get_assignment_operator (ValaParser* self,
                                                            ValaTokenType token_type);
static gboolean vala_parser_is_lambda_expression (ValaParser* self);
static ValaStatement* vala_parser_parse_statement (ValaParser* self,
                                            ValaTokenType type,
                                            GError** error);
static ValaStatement* vala_parser_parse_if_statement (ValaParser* self,
                                               GError** error);
static ValaStatement* vala_parser_parse_switch_statement (ValaParser* self,
                                                   GError** error);
static ValaStatement* vala_parser_parse_switch_section_statement (ValaParser* self,
                                                           GError** error);
static ValaStatement* vala_parser_parse_while_statement (ValaParser* self,
                                                  GError** error);
static ValaStatement* vala_parser_parse_do_statement (ValaParser* self,
                                               GError** error);
static ValaStatement* vala_parser_parse_for_statement (ValaParser* self,
                                                GError** error);
static ValaStatement* vala_parser_parse_foreach_statement (ValaParser* self,
                                                    GError** error);
static ValaStatement* vala_parser_parse_break_statement (ValaParser* self,
                                                  GError** error);
static ValaStatement* vala_parser_parse_continue_statement (ValaParser* self,
                                                     GError** error);
static ValaStatement* vala_parser_parse_return_statement (ValaParser* self,
                                                   GError** error);
static ValaStatement* vala_parser_parse_yield_statement (ValaParser* self,
                                                  GError** error);
static ValaStatement* vala_parser_parse_throw_statement (ValaParser* self,
                                                  GError** error);
static ValaStatement* vala_parser_parse_try_statement (ValaParser* self,
                                                GError** error);
static ValaStatement* vala_parser_parse_lock_statement (ValaParser* self,
                                                 GError** error);
static ValaStatement* vala_parser_parse_unlock_statement (ValaParser* self,
                                                   GError** error);
static ValaStatement* vala_parser_parse_delete_statement (ValaParser* self,
                                                   GError** error);
static ValaStatement* vala_parser_parse_with_statement (ValaParser* self,
                                                 GError** error);
static gboolean vala_parser_is_expression (ValaParser* self,
                                    GError** error);
static ValaStatement* vala_parser_parse_expression_statement (ValaParser* self,
                                                       GError** error);
static void vala_parser_parse_statements (ValaParser* self,
                                   ValaBlock* block,
                                   GError** error);
static ValaStatement* vala_parser_parse_empty_statement (ValaParser* self,
                                                  GError** error);
static void vala_parser_parse_local_variable_declarations (ValaParser* self,
                                                    ValaBlock* block,
                                                    GError** error);
static void vala_parser_parse_local_constant_declarations (ValaParser* self,
                                                    ValaBlock* block,
                                                    GError** error);
static gboolean vala_parser_is_local_function (ValaParser* self);
static void vala_parser_parse_local_function_declaration (ValaParser* self,
                                                   ValaBlock* block,
                                                   GError** error);
static ValaParserRecoveryState vala_parser_recover (ValaParser* self);
static ValaBlock* vala_parser_parse_embedded_statement (ValaParser* self,
                                                 const gchar* statement_name,
                                                 gboolean accept_empty_body,
                                                 GError** error);
static ValaStatement* vala_parser_parse_embedded_statement_without_block (ValaParser* self,
                                                                   const gchar* statement_name,
                                                                   gboolean accept_empty_body,
                                                                   GError** error);
static void _vala_array_add13 (gchar** * array,
                        gint* length,
                        gint* size,
                        gchar* value);
static ValaLocalVariable* vala_parser_parse_local_variable (ValaParser* self,
                                                     ValaDataType* variable_type,
                                                     gboolean expect_initializer,
                                                     GError** error);
static ValaConstant* vala_parser_parse_local_constant (ValaParser* self,
                                                ValaDataType* constant_type,
                                                GError** error);
static ValaParameter* vala_parser_parse_parameter (ValaParser* self,
                                            GError** error);
static ValaExpression* vala_parser_parse_statement_expression (ValaParser* self,
                                                        GError** error);
static void vala_parser_parse_catch_clauses (ValaParser* self,
                                      ValaList* catch_clauses,
                                      GError** error);
static ValaBlock* vala_parser_parse_finally_clause (ValaParser* self,
                                             GError** error);
static gchar* vala_parser_parse_attribute_value (ValaParser* self,
                                          GError** error);
static ValaList* vala_parser_parse_attributes (ValaParser* self,
                                        GError** error);
static void vala_parser_set_attributes (ValaParser* self,
                                 ValaCodeNode* node,
                                 ValaList* attributes);
static void vala_parser_parse_main_block (ValaParser* self,
                                   ValaSymbol* parent,
                                   GError** error);
static void vala_parser_parse_declaration (ValaParser* self,
                                    ValaSymbol* parent,
                                    gboolean root,
                                    GError** error);
static gboolean vala_parser_is_declaration_keyword (ValaParser* self,
                                             ValaTokenType type);
static void vala_parser_parse_constructor_declaration (ValaParser* self,
                                                ValaSymbol* parent,
                                                ValaList* attrs,
                                                GError** error);
static void vala_parser_parse_destructor_declaration (ValaParser* self,
                                               ValaSymbol* parent,
                                               ValaList* attrs,
                                               GError** error);
static void vala_parser_parse_enum_declaration (ValaParser* self,
                                         ValaSymbol* parent,
                                         ValaList* attrs,
                                         GError** error);
static void vala_parser_parse_errordomain_declaration (ValaParser* self,
                                                ValaSymbol* parent,
                                                ValaList* attrs,
                                                GError** error);
static void vala_parser_parse_interface_declaration (ValaParser* self,
                                              ValaSymbol* parent,
                                              ValaList* attrs,
                                              GError** error);
static void vala_parser_parse_namespace_declaration (ValaParser* self,
                                              ValaSymbol* parent,
                                              ValaList* attrs,
                                              GError** error);
static void vala_parser_parse_struct_declaration (ValaParser* self,
                                           ValaSymbol* parent,
                                           ValaList* attrs,
                                           GError** error);
static void vala_parser_parse_creation_method_declaration (ValaParser* self,
                                                    ValaSymbol* parent,
                                                    ValaList* attrs,
                                                    GError** error);
static void vala_parser_parse_delegate_declaration (ValaParser* self,
                                             ValaSymbol* parent,
                                             ValaList* attrs,
                                             GError** error);
static void vala_parser_parse_signal_declaration (ValaParser* self,
                                           ValaSymbol* parent,
                                           ValaList* attrs,
                                           GError** error);
static void vala_parser_parse_method_declaration (ValaParser* self,
                                           ValaSymbol* parent,
                                           ValaList* attrs,
                                           GError** error);
static void vala_parser_parse_constant_declaration (ValaParser* self,
                                             ValaSymbol* parent,
                                             ValaList* attrs,
                                             GError** error);
static void vala_parser_parse_field_declaration (ValaParser* self,
                                          ValaSymbol* parent,
                                          ValaList* attrs,
                                          GError** error);
static void vala_parser_parse_property_declaration (ValaParser* self,
                                             ValaSymbol* parent,
                                             ValaList* attrs,
                                             GError** error);
static ValaSymbolAccessibility vala_parser_parse_access_modifier (ValaParser* self,
                                                           ValaSymbolAccessibility default_access);
static ValaParserModifierFlags vala_parser_parse_type_declaration_modifiers (ValaParser* self);
static ValaList* vala_parser_parse_type_parameter_list (ValaParser* self,
                                                 GError** error);
static void _vala_array_add14 (ValaParserPartialInfo* * array,
                        gint* length,
                        gint* size,
                        const ValaParserPartialInfo* value);
static ValaParserModifierFlags vala_parser_parse_member_declaration_modifiers (ValaParser* self);
static void vala_parser_finalize (ValaCodeVisitor * obj);
static GType vala_parser_get_type_once (void);
static void _vala_array_destroy (gpointer array,
                          gssize array_length,
                          GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array,
                       gssize array_length,
                       GDestroyNotify destroy_func);

static inline gpointer
vala_parser_get_instance_private (ValaParser* self)
{
	return G_STRUCT_MEMBER_P (self, ValaParser_private_offset);
}

static GType
vala_parser_modifier_flags_get_type_once (void)
{
	static const GFlagsValue values[] = {{VALA_PARSER_MODIFIER_FLAGS_NONE, "VALA_PARSER_MODIFIER_FLAGS_NONE", "none"}, {VALA_PARSER_MODIFIER_FLAGS_ABSTRACT, "VALA_PARSER_MODIFIER_FLAGS_ABSTRACT", "abstract"}, {VALA_PARSER_MODIFIER_FLAGS_CLASS, "VALA_PARSER_MODIFIER_FLAGS_CLASS", "class"}, {VALA_PARSER_MODIFIER_FLAGS_EXTERN, "VALA_PARSER_MODIFIER_FLAGS_EXTERN", "extern"}, {VALA_PARSER_MODIFIER_FLAGS_INLINE, "VALA_PARSER_MODIFIER_FLAGS_INLINE", "inline"}, {VALA_PARSER_MODIFIER_FLAGS_NEW, "VALA_PARSER_MODIFIER_FLAGS_NEW", "new"}, {VALA_PARSER_MODIFIER_FLAGS_OVERRIDE, "VALA_PARSER_MODIFIER_FLAGS_OVERRIDE", "override"}, {VALA_PARSER_MODIFIER_FLAGS_STATIC, "VALA_PARSER_MODIFIER_FLAGS_STATIC", "static"}, {VALA_PARSER_MODIFIER_FLAGS_VIRTUAL, "VALA_PARSER_MODIFIER_FLAGS_VIRTUAL", "virtual"}, {VALA_PARSER_MODIFIER_FLAGS_ASYNC, "VALA_PARSER_MODIFIER_FLAGS_ASYNC", "async"}, {VALA_PARSER_MODIFIER_FLAGS_SEALED, "VALA_PARSER_MODIFIER_FLAGS_SEALED", "sealed"}, {VALA_PARSER_MODIFIER_FLAGS_PARTIAL, "VALA_PARSER_MODIFIER_FLAGS_PARTIAL", "partial"}, {0, NULL, NULL}};
	GType vala_parser_modifier_flags_type_id;
	vala_parser_modifier_flags_type_id = g_flags_register_static ("ValaParserModifierFlags", values);
	return vala_parser_modifier_flags_type_id;
}

static GType
vala_parser_modifier_flags_get_type (void)
{
	static volatile gsize vala_parser_modifier_flags_type_id__once = 0;
	if (g_once_init_enter (&vala_parser_modifier_flags_type_id__once)) {
		GType vala_parser_modifier_flags_type_id;
		vala_parser_modifier_flags_type_id = vala_parser_modifier_flags_get_type_once ();
		g_once_init_leave (&vala_parser_modifier_flags_type_id__once, vala_parser_modifier_flags_type_id);
	}
	return vala_parser_modifier_flags_type_id__once;
}

static GType
vala_parser_recovery_state_get_type_once (void)
{
	static const GEnumValue values[] = {{VALA_PARSER_RECOVERY_STATE_EOF, "VALA_PARSER_RECOVERY_STATE_EOF", "eof"}, {VALA_PARSER_RECOVERY_STATE_DECLARATION_BEGIN, "VALA_PARSER_RECOVERY_STATE_DECLARATION_BEGIN", "declaration-begin"}, {VALA_PARSER_RECOVERY_STATE_STATEMENT_BEGIN, "VALA_PARSER_RECOVERY_STATE_STATEMENT_BEGIN", "statement-begin"}, {0, NULL, NULL}};
	GType vala_parser_recovery_state_type_id;
	vala_parser_recovery_state_type_id = g_enum_register_static ("ValaParserRecoveryState", values);
	return vala_parser_recovery_state_type_id;
}

static GType
vala_parser_recovery_state_get_type (void)
{
	static volatile gsize vala_parser_recovery_state_type_id__once = 0;
	if (g_once_init_enter (&vala_parser_recovery_state_type_id__once)) {
		GType vala_parser_recovery_state_type_id;
		vala_parser_recovery_state_type_id = vala_parser_recovery_state_get_type_once ();
		g_once_init_leave (&vala_parser_recovery_state_type_id__once, vala_parser_recovery_state_type_id);
	}
	return vala_parser_recovery_state_type_id__once;
}

static void
_vala_ValaParserPartialInfo_array_free (ValaParserPartialInfo * array,
                                        gssize array_length)
{
	if (array != NULL) {
		gssize i;
		for (i = 0; i < array_length; i = i + 1) {
			vala_parser_partial_info_destroy (&array[i]);
		}
	}
	g_free (array);
}

ValaParser*
vala_parser_construct (GType object_type)
{
	ValaParser* self = NULL;
	ValaParserTokenInfo* _tmp0_;
	self = (ValaParser*) vala_code_visitor_construct (object_type);
	_tmp0_ = g_new0 (ValaParserTokenInfo, VALA_PARSER_BUFFER_SIZE);
	self->priv->tokens = (g_free (self->priv->tokens), NULL);
	self->priv->tokens = _tmp0_;
	self->priv->tokens_length1 = VALA_PARSER_BUFFER_SIZE;
	self->priv->_tokens_size_ = self->priv->tokens_length1;
	return self;
}

ValaParser*
vala_parser_new (void)
{
	return vala_parser_construct (VALA_TYPE_PARSER);
}

/**
 * Parses all .vala and .vapi source files in the specified code
 * context and builds a code tree.
 *
 * @param context a code context
 */
static gpointer
_vala_code_context_ref0 (gpointer self)
{
	return self ? vala_code_context_ref (self) : NULL;
}

void
vala_parser_parse (ValaParser* self,
                   ValaCodeContext* context)
{
	ValaCodeContext* _tmp0_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (context != NULL);
	_tmp0_ = _vala_code_context_ref0 (context);
	_vala_code_context_unref0 (self->priv->context);
	self->priv->context = _tmp0_;
	vala_code_context_accept (context, (ValaCodeVisitor*) self);
	_vala_code_context_unref0 (self->priv->context);
	self->priv->context = NULL;
}

static void
vala_parser_real_visit_source_file (ValaCodeVisitor* base,
                                    ValaSourceFile* source_file)
{
	ValaParser * self;
	gboolean _tmp0_ = FALSE;
	gboolean _tmp1_ = FALSE;
	gboolean _tmp2_ = FALSE;
	ValaCodeContext* _tmp3_;
	self = (ValaParser*) base;
	g_return_if_fail (source_file != NULL);
	_tmp3_ = self->priv->context;
	if (_tmp3_ != NULL) {
		ValaCodeContext* _tmp4_;
		gboolean _tmp5_;
		gboolean _tmp6_;
		_tmp4_ = self->priv->context;
		_tmp5_ = vala_code_context_get_run_output (_tmp4_);
		_tmp6_ = _tmp5_;
		_tmp2_ = _tmp6_;
	} else {
		_tmp2_ = FALSE;
	}
	if (_tmp2_) {
		_tmp1_ = TRUE;
	} else {
		const gchar* _tmp7_;
		const gchar* _tmp8_;
		_tmp7_ = vala_source_file_get_filename (source_file);
		_tmp8_ = _tmp7_;
		_tmp1_ = g_str_has_suffix (_tmp8_, ".vala");
	}
	if (_tmp1_) {
		_tmp0_ = TRUE;
	} else {
		const gchar* _tmp9_;
		const gchar* _tmp10_;
		_tmp9_ = vala_source_file_get_filename (source_file);
		_tmp10_ = _tmp9_;
		_tmp0_ = g_str_has_suffix (_tmp10_, ".vapi");
	}
	if (_tmp0_) {
		vala_parser_parse_file (self, source_file);
	}
}

static inline gboolean
vala_parser_next (ValaParser* self)
{
	gint _tmp0_;
	ValaParserTokenInfo* _tmp9_;
	gint _tmp9__length1;
	ValaParserTokenInfo _tmp10_;
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	self->priv->index = (self->priv->index + 1) % VALA_PARSER_BUFFER_SIZE;
	_tmp0_ = self->priv->size;
	self->priv->size = _tmp0_ - 1;
	if (self->priv->size <= 0) {
		ValaSourceLocation begin = {0};
		ValaSourceLocation end = {0};
		ValaTokenType type = 0;
		ValaScanner* _tmp1_;
		ValaSourceLocation _tmp2_ = {0};
		ValaSourceLocation _tmp3_ = {0};
		ValaTokenType _tmp4_;
		ValaParserTokenInfo* _tmp5_;
		gint _tmp5__length1;
		ValaSourceLocation _tmp6_;
		ValaSourceLocation _tmp7_;
		ValaParserTokenInfo _tmp8_ = {0};
		_tmp1_ = self->priv->scanner;
		_tmp4_ = vala_scanner_read_token (_tmp1_, &_tmp2_, &_tmp3_);
		begin = _tmp2_;
		end = _tmp3_;
		type = _tmp4_;
		_tmp5_ = self->priv->tokens;
		_tmp5__length1 = self->priv->tokens_length1;
		_tmp6_ = begin;
		_tmp7_ = end;
		_tmp8_.type = type;
		_tmp8_.begin = _tmp6_;
		_tmp8_.end = _tmp7_;
		_tmp5_[self->priv->index] = _tmp8_;
		self->priv->size = 1;
	}
	_tmp9_ = self->priv->tokens;
	_tmp9__length1 = self->priv->tokens_length1;
	_tmp10_ = _tmp9_[self->priv->index];
	result = _tmp10_.type != VALA_TOKEN_TYPE_EOF;
	return result;
}

static inline void
vala_parser_prev (ValaParser* self)
{
	gint _tmp0_;
	g_return_if_fail (self != NULL);
	self->priv->index = ((self->priv->index - 1) + VALA_PARSER_BUFFER_SIZE) % VALA_PARSER_BUFFER_SIZE;
	_tmp0_ = self->priv->size;
	self->priv->size = _tmp0_ + 1;
	_vala_assert (self->priv->size <= VALA_PARSER_BUFFER_SIZE, "size <= BUFFER_SIZE");
}

static inline void
vala_parser_safe_prev (ValaParser* self)
{
	g_return_if_fail (self != NULL);
	switch (vala_parser_previous (self)) {
		case VALA_TOKEN_TYPE_DOT:
		case VALA_TOKEN_TYPE_DOUBLE_COLON:
		{
			break;
		}
		default:
		{
			vala_parser_prev (self);
			break;
		}
	}
}

static inline ValaTokenType
vala_parser_current (ValaParser* self)
{
	ValaParserTokenInfo* _tmp0_;
	gint _tmp0__length1;
	ValaParserTokenInfo _tmp1_;
	ValaTokenType result;
	g_return_val_if_fail (self != NULL, 0);
	_tmp0_ = self->priv->tokens;
	_tmp0__length1 = self->priv->tokens_length1;
	_tmp1_ = _tmp0_[self->priv->index];
	result = _tmp1_.type;
	return result;
}

static inline ValaTokenType
vala_parser_previous (ValaParser* self)
{
	ValaParserTokenInfo* _tmp0_;
	gint _tmp0__length1;
	ValaParserTokenInfo _tmp1_;
	ValaTokenType result;
	g_return_val_if_fail (self != NULL, 0);
	_tmp0_ = self->priv->tokens;
	_tmp0__length1 = self->priv->tokens_length1;
	_tmp1_ = _tmp0_[((self->priv->index - 1) + VALA_PARSER_BUFFER_SIZE) % VALA_PARSER_BUFFER_SIZE];
	result = _tmp1_.type;
	return result;
}

static inline gboolean
vala_parser_accept (ValaParser* self,
                    ValaTokenType type)
{
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	if (vala_parser_current (self) == type) {
		vala_parser_next (self);
		result = TRUE;
		return result;
	}
	result = FALSE;
	return result;
}

static void
vala_parser_report_parse_error (ValaParser* self,
                                GError* e)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaSourceLocation _tmp1_;
	ValaSourceReference* _tmp2_;
	ValaSourceReference* _tmp3_;
	const gchar* _tmp4_;
	g_return_if_fail (self != NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	vala_parser_next (self);
	_tmp1_ = begin;
	_tmp2_ = vala_parser_get_src (self, &_tmp1_);
	_tmp3_ = _tmp2_;
	_tmp4_ = e->message;
	vala_report_error (_tmp3_, "syntax error, %s", _tmp4_);
	_vala_source_reference_unref0 (_tmp3_);
}

static inline gboolean
vala_parser_expect (ValaParser* self,
                    ValaTokenType type,
                    GError** error)
{
	GError* _inner_error0_ = NULL;
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	if (vala_parser_accept (self, type)) {
		result = TRUE;
		return result;
	}
	switch (type) {
		case VALA_TOKEN_TYPE_CLOSE_BRACE:
		{
			const gchar* _tmp0_;
			GError* _tmp1_;
			GError* _tmp2_;
			vala_parser_safe_prev (self);
			_tmp0_ = vala_token_type_to_string (type);
			_tmp1_ = g_error_new (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "following block delimiter %s missing", _tmp0_);
			_tmp2_ = _tmp1_;
			vala_parser_report_parse_error (self, _tmp2_);
			_g_error_free0 (_tmp2_);
			result = TRUE;
			return result;
		}
		case VALA_TOKEN_TYPE_CLOSE_BRACKET:
		case VALA_TOKEN_TYPE_CLOSE_PARENS:
		case VALA_TOKEN_TYPE_SEMICOLON:
		{
			const gchar* _tmp3_;
			GError* _tmp4_;
			GError* _tmp5_;
			vala_parser_safe_prev (self);
			_tmp3_ = vala_token_type_to_string (type);
			_tmp4_ = g_error_new (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "following expression/statement delimiter %s missing", _tmp3_);
			_tmp5_ = _tmp4_;
			vala_parser_report_parse_error (self, _tmp5_);
			_g_error_free0 (_tmp5_);
			result = TRUE;
			return result;
		}
		default:
		{
			const gchar* _tmp6_;
			GError* _tmp7_;
			_tmp6_ = vala_token_type_to_string (type);
			_tmp7_ = g_error_new (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "expected %s", _tmp6_);
			_inner_error0_ = _tmp7_;
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				gboolean _tmp8_ = FALSE;
				g_propagate_error (error, _inner_error0_);
				return _tmp8_;
			} else {
				gboolean _tmp9_ = FALSE;
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return _tmp9_;
			}
		}
	}
}

static inline void
vala_parser_get_location (ValaParser* self,
                          ValaSourceLocation* result)
{
	ValaParserTokenInfo* _tmp0_;
	gint _tmp0__length1;
	ValaParserTokenInfo _tmp1_;
	ValaSourceLocation _tmp2_;
	g_return_if_fail (self != NULL);
	_tmp0_ = self->priv->tokens;
	_tmp0__length1 = self->priv->tokens_length1;
	_tmp1_ = _tmp0_[self->priv->index];
	_tmp2_ = _tmp1_.begin;
	*result = _tmp2_;
	return;
}

static gchar*
vala_parser_get_location_string (ValaParser* self)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaSourceLocation _tmp1_;
	ValaSourceLocation _tmp2_;
	gchar* _tmp3_;
	gchar* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	_tmp1_ = begin;
	_tmp2_ = begin;
	_tmp3_ = g_strdup_printf ("__VALA_L%d_C%d__", _tmp1_.line, _tmp2_.column);
	result = _tmp3_;
	return result;
}

static glong
string_strnlen (gchar* str,
                glong maxlen)
{
	gchar* end = NULL;
	gchar* _tmp0_;
	gchar* _tmp1_;
	glong result;
	_tmp0_ = memchr (str, 0, (gsize) maxlen);
	end = _tmp0_;
	_tmp1_ = end;
	if (_tmp1_ == NULL) {
		result = maxlen;
		return result;
	} else {
		gchar* _tmp2_;
		_tmp2_ = end;
		result = (glong) (_tmp2_ - str);
		return result;
	}
}

static gchar*
string_substring (const gchar* self,
                  glong offset,
                  glong len)
{
	glong string_length = 0L;
	gboolean _tmp0_ = FALSE;
	gchar* _tmp3_;
	gchar* result;
	g_return_val_if_fail (self != NULL, NULL);
	if (offset >= ((glong) 0)) {
		_tmp0_ = len >= ((glong) 0);
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		string_length = string_strnlen ((gchar*) self, offset + len);
	} else {
		gint _tmp1_;
		gint _tmp2_;
		_tmp1_ = strlen (self);
		_tmp2_ = _tmp1_;
		string_length = (glong) _tmp2_;
	}
	if (offset < ((glong) 0)) {
		offset = string_length + offset;
		g_return_val_if_fail (offset >= ((glong) 0), NULL);
	} else {
		g_return_val_if_fail (offset <= string_length, NULL);
	}
	if (len < ((glong) 0)) {
		len = string_length - offset;
	}
	g_return_val_if_fail ((offset + len) <= string_length, NULL);
	_tmp3_ = g_strndup (((gchar*) self) + offset, (gsize) len);
	result = _tmp3_;
	return result;
}

static gchar*
vala_parser_get_current_string (ValaParser* self)
{
	ValaParserTokenInfo token = {0};
	ValaParserTokenInfo* _tmp0_;
	gint _tmp0__length1;
	ValaParserTokenInfo _tmp1_;
	ValaParserTokenInfo _tmp2_;
	ValaSourceLocation _tmp3_;
	gchar* _tmp4_;
	ValaParserTokenInfo _tmp5_;
	ValaSourceLocation _tmp6_;
	gchar* _tmp7_;
	ValaParserTokenInfo _tmp8_;
	ValaSourceLocation _tmp9_;
	gchar* _tmp10_;
	gchar* _tmp11_;
	gchar* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->tokens;
	_tmp0__length1 = self->priv->tokens_length1;
	_tmp1_ = _tmp0_[self->priv->index];
	token = _tmp1_;
	_tmp2_ = token;
	_tmp3_ = _tmp2_.begin;
	_tmp4_ = _tmp3_.pos;
	_tmp5_ = token;
	_tmp6_ = _tmp5_.end;
	_tmp7_ = _tmp6_.pos;
	_tmp8_ = token;
	_tmp9_ = _tmp8_.begin;
	_tmp10_ = _tmp9_.pos;
	_tmp11_ = string_substring ((const gchar*) _tmp4_, (glong) 0, (glong) ((gint) (_tmp7_ - _tmp10_)));
	result = _tmp11_;
	return result;
}

static gchar*
vala_parser_get_last_string (ValaParser* self)
{
	gint last_index = 0;
	ValaParserTokenInfo token = {0};
	ValaParserTokenInfo* _tmp0_;
	gint _tmp0__length1;
	ValaParserTokenInfo _tmp1_;
	ValaParserTokenInfo _tmp2_;
	ValaSourceLocation _tmp3_;
	gchar* _tmp4_;
	ValaParserTokenInfo _tmp5_;
	ValaSourceLocation _tmp6_;
	gchar* _tmp7_;
	ValaParserTokenInfo _tmp8_;
	ValaSourceLocation _tmp9_;
	gchar* _tmp10_;
	gchar* _tmp11_;
	gchar* result;
	g_return_val_if_fail (self != NULL, NULL);
	last_index = ((self->priv->index + VALA_PARSER_BUFFER_SIZE) - 1) % VALA_PARSER_BUFFER_SIZE;
	_tmp0_ = self->priv->tokens;
	_tmp0__length1 = self->priv->tokens_length1;
	_tmp1_ = _tmp0_[last_index];
	token = _tmp1_;
	_tmp2_ = token;
	_tmp3_ = _tmp2_.begin;
	_tmp4_ = _tmp3_.pos;
	_tmp5_ = token;
	_tmp6_ = _tmp5_.end;
	_tmp7_ = _tmp6_.pos;
	_tmp8_ = token;
	_tmp9_ = _tmp8_.begin;
	_tmp10_ = _tmp9_.pos;
	_tmp11_ = string_substring ((const gchar*) _tmp4_, (glong) 0, (glong) ((gint) (_tmp7_ - _tmp10_)));
	result = _tmp11_;
	return result;
}

static ValaSourceReference*
vala_parser_get_src (ValaParser* self,
                     ValaSourceLocation* begin)
{
	gint last_index = 0;
	ValaScanner* _tmp0_;
	ValaSourceFile* _tmp1_;
	ValaSourceFile* _tmp2_;
	ValaSourceLocation _tmp3_;
	ValaParserTokenInfo* _tmp4_;
	gint _tmp4__length1;
	ValaParserTokenInfo _tmp5_;
	ValaSourceLocation _tmp6_;
	ValaSourceReference* _tmp7_;
	ValaSourceReference* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (begin != NULL, NULL);
	last_index = ((self->priv->index + VALA_PARSER_BUFFER_SIZE) - 1) % VALA_PARSER_BUFFER_SIZE;
	_tmp0_ = self->priv->scanner;
	_tmp1_ = vala_scanner_get_source_file (_tmp0_);
	_tmp2_ = _tmp1_;
	_tmp3_ = *begin;
	_tmp4_ = self->priv->tokens;
	_tmp4__length1 = self->priv->tokens_length1;
	_tmp5_ = _tmp4_[last_index];
	_tmp6_ = _tmp5_.end;
	_tmp7_ = vala_source_reference_new (_tmp2_, &_tmp3_, &_tmp6_);
	result = _tmp7_;
	return result;
}

static ValaSourceReference*
vala_parser_get_current_src (ValaParser* self)
{
	ValaParserTokenInfo token = {0};
	ValaParserTokenInfo* _tmp0_;
	gint _tmp0__length1;
	ValaParserTokenInfo _tmp1_;
	ValaScanner* _tmp2_;
	ValaSourceFile* _tmp3_;
	ValaSourceFile* _tmp4_;
	ValaParserTokenInfo _tmp5_;
	ValaSourceLocation _tmp6_;
	ValaParserTokenInfo _tmp7_;
	ValaSourceLocation _tmp8_;
	ValaSourceReference* _tmp9_;
	ValaSourceReference* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->tokens;
	_tmp0__length1 = self->priv->tokens_length1;
	_tmp1_ = _tmp0_[self->priv->index];
	token = _tmp1_;
	_tmp2_ = self->priv->scanner;
	_tmp3_ = vala_scanner_get_source_file (_tmp2_);
	_tmp4_ = _tmp3_;
	_tmp5_ = token;
	_tmp6_ = _tmp5_.begin;
	_tmp7_ = token;
	_tmp8_ = _tmp7_.end;
	_tmp9_ = vala_source_reference_new (_tmp4_, &_tmp6_, &_tmp8_);
	result = _tmp9_;
	return result;
}

static ValaSourceReference*
vala_parser_get_last_src (ValaParser* self)
{
	gint last_index = 0;
	ValaParserTokenInfo token = {0};
	ValaParserTokenInfo* _tmp0_;
	gint _tmp0__length1;
	ValaParserTokenInfo _tmp1_;
	ValaScanner* _tmp2_;
	ValaSourceFile* _tmp3_;
	ValaSourceFile* _tmp4_;
	ValaParserTokenInfo _tmp5_;
	ValaSourceLocation _tmp6_;
	ValaParserTokenInfo _tmp7_;
	ValaSourceLocation _tmp8_;
	ValaSourceReference* _tmp9_;
	ValaSourceReference* result;
	g_return_val_if_fail (self != NULL, NULL);
	last_index = ((self->priv->index + VALA_PARSER_BUFFER_SIZE) - 1) % VALA_PARSER_BUFFER_SIZE;
	_tmp0_ = self->priv->tokens;
	_tmp0__length1 = self->priv->tokens_length1;
	_tmp1_ = _tmp0_[last_index];
	token = _tmp1_;
	_tmp2_ = self->priv->scanner;
	_tmp3_ = vala_scanner_get_source_file (_tmp2_);
	_tmp4_ = _tmp3_;
	_tmp5_ = token;
	_tmp6_ = _tmp5_.begin;
	_tmp7_ = token;
	_tmp8_ = _tmp7_.end;
	_tmp9_ = vala_source_reference_new (_tmp4_, &_tmp6_, &_tmp8_);
	result = _tmp9_;
	return result;
}

static void
vala_parser_rollback (ValaParser* self,
                      ValaSourceLocation* location)
{
	g_return_if_fail (self != NULL);
	g_return_if_fail (location != NULL);
	while (TRUE) {
		ValaParserTokenInfo* _tmp0_;
		gint _tmp0__length1;
		ValaParserTokenInfo _tmp1_;
		ValaSourceLocation _tmp2_;
		gchar* _tmp3_;
		ValaSourceLocation _tmp4_;
		gchar* _tmp5_;
		gint _tmp6_;
		_tmp0_ = self->priv->tokens;
		_tmp0__length1 = self->priv->tokens_length1;
		_tmp1_ = _tmp0_[self->priv->index];
		_tmp2_ = _tmp1_.begin;
		_tmp3_ = _tmp2_.pos;
		_tmp4_ = *location;
		_tmp5_ = _tmp4_.pos;
		if (!(_tmp3_ != _tmp5_)) {
			break;
		}
		self->priv->index = ((self->priv->index - 1) + VALA_PARSER_BUFFER_SIZE) % VALA_PARSER_BUFFER_SIZE;
		_tmp6_ = self->priv->size;
		self->priv->size = _tmp6_ + 1;
		if (self->priv->size > VALA_PARSER_BUFFER_SIZE) {
			ValaScanner* _tmp7_;
			ValaSourceLocation _tmp8_;
			_tmp7_ = self->priv->scanner;
			_tmp8_ = *location;
			vala_scanner_seek (_tmp7_, &_tmp8_);
			self->priv->size = 0;
			self->priv->index = 0;
			vala_parser_next (self);
		}
	}
}

static void
vala_parser_jump (ValaParser* self,
                  ValaSourceLocation* location)
{
	g_return_if_fail (self != NULL);
	g_return_if_fail (location != NULL);
	while (TRUE) {
		ValaParserTokenInfo* _tmp0_;
		gint _tmp0__length1;
		ValaParserTokenInfo _tmp1_;
		ValaSourceLocation _tmp2_;
		gchar* _tmp3_;
		ValaSourceLocation _tmp4_;
		gchar* _tmp5_;
		gint _tmp6_;
		_tmp0_ = self->priv->tokens;
		_tmp0__length1 = self->priv->tokens_length1;
		_tmp1_ = _tmp0_[self->priv->index];
		_tmp2_ = _tmp1_.begin;
		_tmp3_ = _tmp2_.pos;
		_tmp4_ = *location;
		_tmp5_ = _tmp4_.pos;
		if (!(_tmp3_ != _tmp5_)) {
			break;
		}
		self->priv->index = (self->priv->index + 1) % VALA_PARSER_BUFFER_SIZE;
		_tmp6_ = self->priv->size;
		self->priv->size = _tmp6_ - 1;
		if (self->priv->size <= 0) {
			ValaScanner* _tmp7_;
			ValaSourceLocation _tmp8_;
			_tmp7_ = self->priv->scanner;
			_tmp8_ = *location;
			vala_scanner_seek (_tmp7_, &_tmp8_);
			self->priv->size = 0;
			self->priv->index = 0;
			vala_parser_next (self);
		}
	}
}

static gchar
string_get (const gchar* self,
            glong index)
{
	gchar _tmp0_;
	gchar result;
	g_return_val_if_fail (self != NULL, '\0');
	_tmp0_ = ((gchar*) self)[index];
	result = _tmp0_;
	return result;
}

static gboolean
string_contains (const gchar* self,
                 const gchar* needle)
{
	gchar* _tmp0_;
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (needle != NULL, FALSE);
	_tmp0_ = strstr ((gchar*) self, (gchar*) needle);
	result = _tmp0_ != NULL;
	return result;
}

static void
vala_parser_skip_identifier (ValaParser* self,
                             GError** error)
{
	GError* _inner_error0_ = NULL;
	g_return_if_fail (self != NULL);
	switch (vala_parser_current (self)) {
		case VALA_TOKEN_TYPE_ABSTRACT:
		case VALA_TOKEN_TYPE_AS:
		case VALA_TOKEN_TYPE_ASYNC:
		case VALA_TOKEN_TYPE_BASE:
		case VALA_TOKEN_TYPE_BREAK:
		case VALA_TOKEN_TYPE_CASE:
		case VALA_TOKEN_TYPE_CATCH:
		case VALA_TOKEN_TYPE_CLASS:
		case VALA_TOKEN_TYPE_CONST:
		case VALA_TOKEN_TYPE_CONSTRUCT:
		case VALA_TOKEN_TYPE_CONTINUE:
		case VALA_TOKEN_TYPE_DEFAULT:
		case VALA_TOKEN_TYPE_DELEGATE:
		case VALA_TOKEN_TYPE_DELETE:
		case VALA_TOKEN_TYPE_DO:
		case VALA_TOKEN_TYPE_DYNAMIC:
		case VALA_TOKEN_TYPE_ELSE:
		case VALA_TOKEN_TYPE_ENUM:
		case VALA_TOKEN_TYPE_ENSURES:
		case VALA_TOKEN_TYPE_ERRORDOMAIN:
		case VALA_TOKEN_TYPE_EXTERN:
		case VALA_TOKEN_TYPE_FALSE:
		case VALA_TOKEN_TYPE_FINALLY:
		case VALA_TOKEN_TYPE_FOR:
		case VALA_TOKEN_TYPE_FOREACH:
		case VALA_TOKEN_TYPE_GET:
		case VALA_TOKEN_TYPE_IDENTIFIER:
		case VALA_TOKEN_TYPE_IF:
		case VALA_TOKEN_TYPE_IN:
		case VALA_TOKEN_TYPE_INLINE:
		case VALA_TOKEN_TYPE_INTERFACE:
		case VALA_TOKEN_TYPE_INTERNAL:
		case VALA_TOKEN_TYPE_IS:
		case VALA_TOKEN_TYPE_LOCK:
		case VALA_TOKEN_TYPE_NAMESPACE:
		case VALA_TOKEN_TYPE_NEW:
		case VALA_TOKEN_TYPE_NULL:
		case VALA_TOKEN_TYPE_OUT:
		case VALA_TOKEN_TYPE_OVERRIDE:
		case VALA_TOKEN_TYPE_OWNED:
		case VALA_TOKEN_TYPE_PARAMS:
		case VALA_TOKEN_TYPE_PARTIAL:
		case VALA_TOKEN_TYPE_PRIVATE:
		case VALA_TOKEN_TYPE_PROTECTED:
		case VALA_TOKEN_TYPE_PUBLIC:
		case VALA_TOKEN_TYPE_REF:
		case VALA_TOKEN_TYPE_REQUIRES:
		case VALA_TOKEN_TYPE_RETURN:
		case VALA_TOKEN_TYPE_SEALED:
		case VALA_TOKEN_TYPE_SET:
		case VALA_TOKEN_TYPE_SIGNAL:
		case VALA_TOKEN_TYPE_SIZEOF:
		case VALA_TOKEN_TYPE_STATIC:
		case VALA_TOKEN_TYPE_STRUCT:
		case VALA_TOKEN_TYPE_SWITCH:
		case VALA_TOKEN_TYPE_THIS:
		case VALA_TOKEN_TYPE_THROW:
		case VALA_TOKEN_TYPE_THROWS:
		case VALA_TOKEN_TYPE_TRUE:
		case VALA_TOKEN_TYPE_TRY:
		case VALA_TOKEN_TYPE_TYPEOF:
		case VALA_TOKEN_TYPE_UNLOCK:
		case VALA_TOKEN_TYPE_UNOWNED:
		case VALA_TOKEN_TYPE_USING:
		case VALA_TOKEN_TYPE_VAR:
		case VALA_TOKEN_TYPE_VIRTUAL:
		case VALA_TOKEN_TYPE_VOID:
		case VALA_TOKEN_TYPE_VOLATILE:
		case VALA_TOKEN_TYPE_WEAK:
		case VALA_TOKEN_TYPE_WHILE:
		case VALA_TOKEN_TYPE_WITH:
		case VALA_TOKEN_TYPE_YIELD:
		{
			vala_parser_next (self);
			return;
		}
		case VALA_TOKEN_TYPE_INTEGER_LITERAL:
		case VALA_TOKEN_TYPE_REAL_LITERAL:
		{
			gchar* id = NULL;
			gchar* _tmp0_;
			gboolean _tmp1_ = FALSE;
			const gchar* _tmp2_;
			const gchar* _tmp3_;
			gint _tmp4_;
			gint _tmp5_;
			_tmp0_ = vala_parser_get_current_string (self);
			id = _tmp0_;
			_tmp2_ = id;
			_tmp3_ = id;
			_tmp4_ = strlen (_tmp3_);
			_tmp5_ = _tmp4_;
			if (g_ascii_isalpha (string_get (_tmp2_, (glong) (_tmp5_ - 1)))) {
				const gchar* _tmp6_;
				_tmp6_ = id;
				_tmp1_ = !string_contains (_tmp6_, ".");
			} else {
				_tmp1_ = FALSE;
			}
			if (_tmp1_) {
				vala_parser_next (self);
				_g_free0 (id);
				return;
			}
			_g_free0 (id);
			break;
		}
		default:
		{
			GError* _tmp7_;
			_tmp7_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "expected identifier");
			_inner_error0_ = _tmp7_;
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				return;
			} else {
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
	}
}

static gpointer
_g_error_copy0 (gpointer self)
{
	return self ? g_error_copy (self) : NULL;
}

static gchar*
vala_parser_parse_identifier (ValaParser* self,
                              gboolean required,
                              GError** error)
{
	gchar* _tmp8_;
	GError* _inner_error0_ = NULL;
	gchar* result;
	g_return_val_if_fail (self != NULL, NULL);
	{
		vala_parser_skip_identifier (self, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				goto __catch0_vala_parse_error;
			}
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	goto __finally0;
	__catch0_vala_parse_error:
	{
		GError* e = NULL;
		gboolean _tmp0_ = FALSE;
		e = _inner_error0_;
		_inner_error0_ = NULL;
		if (!required) {
			ValaCodeContext* _tmp1_;
			gboolean _tmp2_;
			gboolean _tmp3_;
			_tmp1_ = self->priv->context;
			_tmp2_ = vala_code_context_get_keep_going (_tmp1_);
			_tmp3_ = _tmp2_;
			_tmp0_ = _tmp3_;
		} else {
			_tmp0_ = FALSE;
		}
		if (_tmp0_) {
			GError* _tmp4_;
			gchar* _tmp5_;
			_tmp4_ = e;
			vala_parser_report_parse_error (self, _tmp4_);
			vala_parser_prev (self);
			_tmp5_ = vala_parser_get_location_string (self);
			result = _tmp5_;
			_g_error_free0 (e);
			return result;
		} else {
			GError* _tmp6_;
			GError* _tmp7_;
			_tmp6_ = e;
			_tmp7_ = _g_error_copy0 (_tmp6_);
			_inner_error0_ = _tmp7_;
			_g_error_free0 (e);
			goto __finally0;
		}
		_g_error_free0 (e);
	}
	__finally0:
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp8_ = vala_parser_get_last_string (self);
	result = _tmp8_;
	return result;
}

static ValaExpression*
vala_parser_parse_literal (ValaParser* self,
                           GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	switch (vala_parser_current (self)) {
		case VALA_TOKEN_TYPE_TRUE:
		{
			ValaSourceLocation _tmp1_;
			ValaSourceReference* _tmp2_;
			ValaSourceReference* _tmp3_;
			ValaBooleanLiteral* _tmp4_;
			ValaExpression* _tmp5_;
			vala_parser_next (self);
			_tmp1_ = begin;
			_tmp2_ = vala_parser_get_src (self, &_tmp1_);
			_tmp3_ = _tmp2_;
			_tmp4_ = vala_boolean_literal_new (TRUE, _tmp3_);
			_tmp5_ = (ValaExpression*) _tmp4_;
			_vala_source_reference_unref0 (_tmp3_);
			result = _tmp5_;
			return result;
		}
		case VALA_TOKEN_TYPE_FALSE:
		{
			ValaSourceLocation _tmp6_;
			ValaSourceReference* _tmp7_;
			ValaSourceReference* _tmp8_;
			ValaBooleanLiteral* _tmp9_;
			ValaExpression* _tmp10_;
			vala_parser_next (self);
			_tmp6_ = begin;
			_tmp7_ = vala_parser_get_src (self, &_tmp6_);
			_tmp8_ = _tmp7_;
			_tmp9_ = vala_boolean_literal_new (FALSE, _tmp8_);
			_tmp10_ = (ValaExpression*) _tmp9_;
			_vala_source_reference_unref0 (_tmp8_);
			result = _tmp10_;
			return result;
		}
		case VALA_TOKEN_TYPE_INTEGER_LITERAL:
		{
			gchar* _tmp11_;
			gchar* _tmp12_;
			ValaSourceLocation _tmp13_;
			ValaSourceReference* _tmp14_;
			ValaSourceReference* _tmp15_;
			ValaIntegerLiteral* _tmp16_;
			ValaExpression* _tmp17_;
			vala_parser_next (self);
			_tmp11_ = vala_parser_get_last_string (self);
			_tmp12_ = _tmp11_;
			_tmp13_ = begin;
			_tmp14_ = vala_parser_get_src (self, &_tmp13_);
			_tmp15_ = _tmp14_;
			_tmp16_ = vala_integer_literal_new (_tmp12_, _tmp15_);
			_tmp17_ = (ValaExpression*) _tmp16_;
			_vala_source_reference_unref0 (_tmp15_);
			_g_free0 (_tmp12_);
			result = _tmp17_;
			return result;
		}
		case VALA_TOKEN_TYPE_REAL_LITERAL:
		{
			gchar* _tmp18_;
			gchar* _tmp19_;
			ValaSourceLocation _tmp20_;
			ValaSourceReference* _tmp21_;
			ValaSourceReference* _tmp22_;
			ValaRealLiteral* _tmp23_;
			ValaExpression* _tmp24_;
			vala_parser_next (self);
			_tmp18_ = vala_parser_get_last_string (self);
			_tmp19_ = _tmp18_;
			_tmp20_ = begin;
			_tmp21_ = vala_parser_get_src (self, &_tmp20_);
			_tmp22_ = _tmp21_;
			_tmp23_ = vala_real_literal_new (_tmp19_, _tmp22_);
			_tmp24_ = (ValaExpression*) _tmp23_;
			_vala_source_reference_unref0 (_tmp22_);
			_g_free0 (_tmp19_);
			result = _tmp24_;
			return result;
		}
		case VALA_TOKEN_TYPE_CHARACTER_LITERAL:
		{
			ValaCharacterLiteral* lit = NULL;
			gchar* _tmp25_;
			gchar* _tmp26_;
			ValaSourceLocation _tmp27_;
			ValaSourceReference* _tmp28_;
			ValaSourceReference* _tmp29_;
			ValaCharacterLiteral* _tmp30_;
			ValaCharacterLiteral* _tmp31_;
			ValaCharacterLiteral* _tmp32_;
			gboolean _tmp33_;
			gboolean _tmp34_;
			vala_parser_next (self);
			_tmp25_ = vala_parser_get_last_string (self);
			_tmp26_ = _tmp25_;
			_tmp27_ = begin;
			_tmp28_ = vala_parser_get_src (self, &_tmp27_);
			_tmp29_ = _tmp28_;
			_tmp30_ = vala_character_literal_new (_tmp26_, _tmp29_);
			_tmp31_ = _tmp30_;
			_vala_source_reference_unref0 (_tmp29_);
			_g_free0 (_tmp26_);
			lit = _tmp31_;
			_tmp32_ = lit;
			_tmp33_ = vala_code_node_get_error ((ValaCodeNode*) _tmp32_);
			_tmp34_ = _tmp33_;
			if (_tmp34_) {
				ValaCharacterLiteral* _tmp35_;
				ValaSourceReference* _tmp36_;
				ValaSourceReference* _tmp37_;
				_tmp35_ = lit;
				_tmp36_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp35_);
				_tmp37_ = _tmp36_;
				vala_report_error (_tmp37_, "invalid character literal");
			}
			result = (ValaExpression*) lit;
			return result;
		}
		case VALA_TOKEN_TYPE_REGEX_LITERAL:
		{
			gchar* match_part = NULL;
			gchar* _tmp38_;
			ValaSourceReference* src_begin = NULL;
			ValaSourceLocation _tmp39_;
			ValaSourceReference* _tmp40_;
			gchar* close_token = NULL;
			gchar* _tmp41_;
			const gchar* _tmp42_;
			const gchar* _tmp43_;
			gchar* _tmp44_;
			gchar* _tmp45_;
			ValaSourceReference* _tmp46_;
			ValaRegexLiteral* _tmp47_;
			ValaExpression* _tmp48_;
			vala_parser_next (self);
			_tmp38_ = vala_parser_get_last_string (self);
			match_part = _tmp38_;
			_tmp39_ = begin;
			_tmp40_ = vala_parser_get_src (self, &_tmp39_);
			src_begin = _tmp40_;
			vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_REGEX_LITERAL, &_inner_error0_);
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_source_reference_unref0 (src_begin);
					_g_free0 (match_part);
					return NULL;
				} else {
					_vala_source_reference_unref0 (src_begin);
					_g_free0 (match_part);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp41_ = vala_parser_get_last_string (self);
			close_token = _tmp41_;
			_tmp42_ = close_token;
			_tmp43_ = match_part;
			_tmp44_ = g_strdup_printf ("%s/%s", _tmp42_, _tmp43_);
			_tmp45_ = _tmp44_;
			_tmp46_ = src_begin;
			_tmp47_ = vala_regex_literal_new (_tmp45_, _tmp46_);
			_tmp48_ = (ValaExpression*) _tmp47_;
			_g_free0 (_tmp45_);
			result = _tmp48_;
			_g_free0 (close_token);
			_vala_source_reference_unref0 (src_begin);
			_g_free0 (match_part);
			return result;
		}
		case VALA_TOKEN_TYPE_STRING_LITERAL:
		{
			gchar* _tmp49_;
			gchar* _tmp50_;
			ValaSourceLocation _tmp51_;
			ValaSourceReference* _tmp52_;
			ValaSourceReference* _tmp53_;
			ValaStringLiteral* _tmp54_;
			ValaExpression* _tmp55_;
			vala_parser_next (self);
			_tmp49_ = vala_parser_get_last_string (self);
			_tmp50_ = _tmp49_;
			_tmp51_ = begin;
			_tmp52_ = vala_parser_get_src (self, &_tmp51_);
			_tmp53_ = _tmp52_;
			_tmp54_ = vala_string_literal_new (_tmp50_, _tmp53_);
			_tmp55_ = (ValaExpression*) _tmp54_;
			_vala_source_reference_unref0 (_tmp53_);
			_g_free0 (_tmp50_);
			result = _tmp55_;
			return result;
		}
		case VALA_TOKEN_TYPE_TEMPLATE_STRING_LITERAL:
		{
			gchar* _tmp56_;
			gchar* _tmp57_;
			gchar* _tmp58_;
			gchar* _tmp59_;
			ValaSourceLocation _tmp60_;
			ValaSourceReference* _tmp61_;
			ValaSourceReference* _tmp62_;
			ValaStringLiteral* _tmp63_;
			ValaExpression* _tmp64_;
			vala_parser_next (self);
			_tmp56_ = vala_parser_get_last_string (self);
			_tmp57_ = _tmp56_;
			_tmp58_ = g_strdup_printf ("\"%s\"", _tmp57_);
			_tmp59_ = _tmp58_;
			_tmp60_ = begin;
			_tmp61_ = vala_parser_get_src (self, &_tmp60_);
			_tmp62_ = _tmp61_;
			_tmp63_ = vala_string_literal_new (_tmp59_, _tmp62_);
			_tmp64_ = (ValaExpression*) _tmp63_;
			_vala_source_reference_unref0 (_tmp62_);
			_g_free0 (_tmp59_);
			_g_free0 (_tmp57_);
			result = _tmp64_;
			return result;
		}
		case VALA_TOKEN_TYPE_VERBATIM_STRING_LITERAL:
		{
			gchar* raw_string = NULL;
			gchar* _tmp65_;
			gchar* escaped_string = NULL;
			const gchar* _tmp66_;
			const gchar* _tmp67_;
			gint _tmp68_;
			gint _tmp69_;
			gchar* _tmp70_;
			gchar* _tmp71_;
			gchar* _tmp72_;
			gchar* _tmp73_;
			const gchar* _tmp74_;
			gchar* _tmp75_;
			gchar* _tmp76_;
			ValaSourceLocation _tmp77_;
			ValaSourceReference* _tmp78_;
			ValaSourceReference* _tmp79_;
			ValaStringLiteral* _tmp80_;
			ValaExpression* _tmp81_;
			vala_parser_next (self);
			_tmp65_ = vala_parser_get_last_string (self);
			raw_string = _tmp65_;
			_tmp66_ = raw_string;
			_tmp67_ = raw_string;
			_tmp68_ = strlen (_tmp67_);
			_tmp69_ = _tmp68_;
			_tmp70_ = string_substring (_tmp66_, (glong) 3, (glong) (_tmp69_ - 6));
			_tmp71_ = _tmp70_;
			_tmp72_ = g_strescape (_tmp71_, "");
			_tmp73_ = _tmp72_;
			_g_free0 (_tmp71_);
			escaped_string = _tmp73_;
			_tmp74_ = escaped_string;
			_tmp75_ = g_strdup_printf ("\"%s\"", _tmp74_);
			_tmp76_ = _tmp75_;
			_tmp77_ = begin;
			_tmp78_ = vala_parser_get_src (self, &_tmp77_);
			_tmp79_ = _tmp78_;
			_tmp80_ = vala_string_literal_new (_tmp76_, _tmp79_);
			_tmp81_ = (ValaExpression*) _tmp80_;
			_vala_source_reference_unref0 (_tmp79_);
			_g_free0 (_tmp76_);
			result = _tmp81_;
			_g_free0 (escaped_string);
			_g_free0 (raw_string);
			return result;
		}
		case VALA_TOKEN_TYPE_NULL:
		{
			ValaSourceLocation _tmp82_;
			ValaSourceReference* _tmp83_;
			ValaSourceReference* _tmp84_;
			ValaNullLiteral* _tmp85_;
			ValaExpression* _tmp86_;
			vala_parser_next (self);
			_tmp82_ = begin;
			_tmp83_ = vala_parser_get_src (self, &_tmp82_);
			_tmp84_ = _tmp83_;
			_tmp85_ = vala_null_literal_new (_tmp84_);
			_tmp86_ = (ValaExpression*) _tmp85_;
			_vala_source_reference_unref0 (_tmp84_);
			result = _tmp86_;
			return result;
		}
		default:
		{
			GError* _tmp87_;
			_tmp87_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "expected literal");
			_inner_error0_ = _tmp87_;
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				return NULL;
			} else {
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
	}
}

void
vala_parser_parse_file (ValaParser* self,
                        ValaSourceFile* source_file)
{
	gboolean has_global_context = FALSE;
	ValaCodeContext* _tmp0_;
	ValaScanner* _tmp4_;
	ValaParserTokenInfo* _tmp5_;
	GError* _inner_error0_ = NULL;
	g_return_if_fail (self != NULL);
	g_return_if_fail (source_file != NULL);
	_tmp0_ = self->priv->context;
	has_global_context = _tmp0_ != NULL;
	if (!has_global_context) {
		ValaCodeContext* _tmp1_;
		ValaCodeContext* _tmp2_;
		ValaCodeContext* _tmp3_;
		_tmp1_ = vala_source_file_get_context (source_file);
		_tmp2_ = _tmp1_;
		_tmp3_ = _vala_code_context_ref0 (_tmp2_);
		_vala_code_context_unref0 (self->priv->context);
		self->priv->context = _tmp3_;
	}
	_tmp4_ = vala_scanner_new (source_file);
	_vala_scanner_unref0 (self->priv->scanner);
	self->priv->scanner = _tmp4_;
	vala_parser_parse_file_comments (self);
	_tmp5_ = g_new0 (ValaParserTokenInfo, VALA_PARSER_BUFFER_SIZE);
	self->priv->tokens = (g_free (self->priv->tokens), NULL);
	self->priv->tokens = _tmp5_;
	self->priv->tokens_length1 = VALA_PARSER_BUFFER_SIZE;
	self->priv->_tokens_size_ = self->priv->tokens_length1;
	self->priv->index = -1;
	self->priv->size = 0;
	vala_parser_next (self);
	{
		ValaParserPartialInfo* _tmp6_;
		ValaSourceLocation begin = {0};
		ValaSourceLocation _tmp7_ = {0};
		ValaCodeContext* _tmp8_;
		ValaNamespace* _tmp9_;
		ValaNamespace* _tmp10_;
		ValaCodeContext* _tmp11_;
		ValaNamespace* _tmp12_;
		ValaNamespace* _tmp13_;
		ValaParserPartialInfo* _tmp19_;
		gint _tmp19__length1;
		_tmp6_ = g_new0 (ValaParserPartialInfo, 0);
		self->priv->partials = (_vala_ValaParserPartialInfo_array_free (self->priv->partials, self->priv->partials_length1), NULL);
		self->priv->partials = _tmp6_;
		self->priv->partials_length1 = 0;
		self->priv->_partials_size_ = self->priv->partials_length1;
		vala_parser_get_location (self, &_tmp7_);
		begin = _tmp7_;
		_tmp8_ = self->priv->context;
		_tmp9_ = vala_code_context_get_root (_tmp8_);
		_tmp10_ = _tmp9_;
		vala_parser_parse_using_directives (self, _tmp10_, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				goto __catch0_vala_parse_error;
			}
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
		_tmp11_ = self->priv->context;
		_tmp12_ = vala_code_context_get_root (_tmp11_);
		_tmp13_ = _tmp12_;
		vala_parser_parse_declarations (self, (ValaSymbol*) _tmp13_, TRUE, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				goto __catch0_vala_parse_error;
			}
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
		if (vala_parser_accept (self, VALA_TOKEN_TYPE_CLOSE_BRACE)) {
			ValaCodeContext* _tmp14_;
			ValaReport* _tmp15_;
			ValaReport* _tmp16_;
			_tmp14_ = self->priv->context;
			_tmp15_ = vala_code_context_get_report (_tmp14_);
			_tmp16_ = _tmp15_;
			if (vala_report_get_errors (_tmp16_) == 0) {
				ValaSourceReference* _tmp17_;
				ValaSourceReference* _tmp18_;
				_tmp17_ = vala_parser_get_last_src (self);
				_tmp18_ = _tmp17_;
				vala_report_error (_tmp18_, "unexpected `}'");
				_vala_source_reference_unref0 (_tmp18_);
			}
		}
		_tmp19_ = self->priv->partials;
		_tmp19__length1 = self->priv->partials_length1;
		if (_tmp19__length1 > 0) {
			ValaSourceLocation _tmp20_;
			ValaParserPartialInfo* _tmp21_;
			gint _tmp21__length1;
			_tmp20_ = begin;
			vala_parser_rollback (self, &_tmp20_);
			_tmp21_ = self->priv->partials;
			_tmp21__length1 = self->priv->partials_length1;
			{
				ValaParserPartialInfo* info_collection = NULL;
				gint info_collection_length1 = 0;
				gint _info_collection_size_ = 0;
				gint info_it = 0;
				info_collection = _tmp21_;
				info_collection_length1 = _tmp21__length1;
				for (info_it = 0; info_it < info_collection_length1; info_it = info_it + 1) {
					ValaParserPartialInfo _tmp22_;
					ValaParserPartialInfo _tmp23_ = {0};
					ValaParserPartialInfo info = {0};
					_tmp22_ = info_collection[info_it];
					vala_parser_partial_info_copy (&_tmp22_, &_tmp23_);
					info = _tmp23_;
					{
						ValaParserPartialInfo _tmp24_;
						ValaSourceLocation _tmp25_;
						ValaParserPartialInfo _tmp26_;
						ValaSymbol* _tmp27_;
						ValaParserPartialInfo _tmp28_;
						ValaList* _tmp29_;
						_tmp24_ = info;
						_tmp25_ = _tmp24_.begin;
						vala_parser_jump (self, &_tmp25_);
						_tmp26_ = info;
						_tmp27_ = _tmp26_.parent;
						_tmp28_ = info;
						_tmp29_ = _tmp28_.attributes;
						vala_parser_parse_class_declaration (self, _tmp27_, _tmp29_, TRUE, &_inner_error0_);
						if (G_UNLIKELY (_inner_error0_ != NULL)) {
							vala_parser_partial_info_destroy (&info);
							if (_inner_error0_->domain == VALA_PARSE_ERROR) {
								goto __catch0_vala_parse_error;
							}
							g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
							g_clear_error (&_inner_error0_);
							return;
						}
						vala_parser_partial_info_destroy (&info);
					}
				}
			}
		}
		self->priv->partials = (_vala_ValaParserPartialInfo_array_free (self->priv->partials, self->priv->partials_length1), NULL);
		self->priv->partials = NULL;
		self->priv->partials_length1 = 0;
		self->priv->_partials_size_ = self->priv->partials_length1;
	}
	goto __finally0;
	__catch0_vala_parse_error:
	{
		GError* e = NULL;
		GError* _tmp30_;
		e = _inner_error0_;
		_inner_error0_ = NULL;
		_tmp30_ = e;
		vala_parser_report_parse_error (self, _tmp30_);
		_g_error_free0 (e);
	}
	__finally0:
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
		g_clear_error (&_inner_error0_);
		return;
	}
	_vala_scanner_unref0 (self->priv->scanner);
	self->priv->scanner = NULL;
	if (!has_global_context) {
		_vala_code_context_unref0 (self->priv->context);
		self->priv->context = NULL;
	}
}

static void
vala_parser_parse_file_comments (ValaParser* self)
{
	ValaScanner* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp0_ = self->priv->scanner;
	vala_scanner_parse_file_comments (_tmp0_);
}

static void
vala_parser_skip_symbol_name (ValaParser* self,
                              GError** error)
{
	GError* _inner_error0_ = NULL;
	g_return_if_fail (self != NULL);
	{
		gboolean _tmp0_ = FALSE;
		_tmp0_ = TRUE;
		while (TRUE) {
			if (!_tmp0_) {
				gboolean _tmp1_ = FALSE;
				if (vala_parser_accept (self, VALA_TOKEN_TYPE_DOT)) {
					_tmp1_ = TRUE;
				} else {
					_tmp1_ = vala_parser_accept (self, VALA_TOKEN_TYPE_DOUBLE_COLON);
				}
				if (!_tmp1_) {
					break;
				}
			}
			_tmp0_ = FALSE;
			vala_parser_skip_identifier (self, &_inner_error0_);
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					return;
				} else {
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return;
				}
			}
		}
	}
}

static ValaUnresolvedSymbol*
vala_parser_parse_symbol_name (ValaParser* self,
                               GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaUnresolvedSymbol* sym = NULL;
	GError* _inner_error0_ = NULL;
	ValaUnresolvedSymbol* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	sym = NULL;
	{
		gboolean _tmp1_ = FALSE;
		_tmp1_ = TRUE;
		while (TRUE) {
			gchar* name = NULL;
			gchar* _tmp2_;
			gboolean _tmp3_ = FALSE;
			const gchar* _tmp4_;
			ValaUnresolvedSymbol* _tmp15_;
			const gchar* _tmp16_;
			ValaSourceLocation _tmp17_;
			ValaSourceReference* _tmp18_;
			ValaSourceReference* _tmp19_;
			ValaUnresolvedSymbol* _tmp20_;
			if (!_tmp1_) {
				if (!vala_parser_accept (self, VALA_TOKEN_TYPE_DOT)) {
					break;
				}
			}
			_tmp1_ = FALSE;
			_tmp2_ = vala_parser_parse_identifier (self, FALSE, &_inner_error0_);
			name = _tmp2_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (sym);
					return NULL;
				} else {
					_vala_code_node_unref0 (sym);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp4_ = name;
			if (g_strcmp0 (_tmp4_, "global") == 0) {
				_tmp3_ = vala_parser_accept (self, VALA_TOKEN_TYPE_DOUBLE_COLON);
			} else {
				_tmp3_ = FALSE;
			}
			if (_tmp3_) {
				gchar* _tmp5_ = NULL;
				gchar* _tmp6_;
				gchar* _tmp7_;
				ValaUnresolvedSymbol* _tmp8_;
				const gchar* _tmp9_;
				ValaSourceLocation _tmp10_;
				ValaSourceReference* _tmp11_;
				ValaSourceReference* _tmp12_;
				ValaUnresolvedSymbol* _tmp13_;
				ValaUnresolvedSymbol* _tmp14_;
				_tmp6_ = vala_parser_parse_identifier (self, FALSE, &_inner_error0_);
				_tmp5_ = _tmp6_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_g_free0 (name);
						_vala_code_node_unref0 (sym);
						return NULL;
					} else {
						_g_free0 (name);
						_vala_code_node_unref0 (sym);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return NULL;
					}
				}
				_tmp7_ = _tmp5_;
				_tmp5_ = NULL;
				_g_free0 (name);
				name = _tmp7_;
				_tmp8_ = sym;
				_tmp9_ = name;
				_tmp10_ = begin;
				_tmp11_ = vala_parser_get_src (self, &_tmp10_);
				_tmp12_ = _tmp11_;
				_tmp13_ = vala_unresolved_symbol_new (_tmp8_, _tmp9_, _tmp12_);
				_vala_code_node_unref0 (sym);
				sym = _tmp13_;
				_vala_source_reference_unref0 (_tmp12_);
				_tmp14_ = sym;
				vala_unresolved_symbol_set_qualified (_tmp14_, TRUE);
				_g_free0 (_tmp5_);
				_g_free0 (name);
				continue;
			}
			_tmp15_ = sym;
			_tmp16_ = name;
			_tmp17_ = begin;
			_tmp18_ = vala_parser_get_src (self, &_tmp17_);
			_tmp19_ = _tmp18_;
			_tmp20_ = vala_unresolved_symbol_new (_tmp15_, _tmp16_, _tmp19_);
			_vala_code_node_unref0 (sym);
			sym = _tmp20_;
			_vala_source_reference_unref0 (_tmp19_);
			_g_free0 (name);
		}
	}
	result = sym;
	return result;
}

static void
vala_parser_skip_type (ValaParser* self,
                       GError** error)
{
	GError* _inner_error0_ = NULL;
	g_return_if_fail (self != NULL);
	vala_parser_accept (self, VALA_TOKEN_TYPE_DYNAMIC);
	vala_parser_accept (self, VALA_TOKEN_TYPE_OWNED);
	vala_parser_accept (self, VALA_TOKEN_TYPE_UNOWNED);
	vala_parser_accept (self, VALA_TOKEN_TYPE_WEAK);
	if (vala_parser_is_inner_array_type (self)) {
		vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_PARENS, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				return;
			} else {
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
		vala_parser_expect (self, VALA_TOKEN_TYPE_UNOWNED, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				return;
			} else {
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
		vala_parser_skip_type (self, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				return;
			} else {
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
		vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_PARENS, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				return;
			} else {
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
		vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_BRACKET, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				return;
			} else {
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
		vala_parser_prev (self);
	} else {
		if (vala_parser_accept (self, VALA_TOKEN_TYPE_VOID)) {
		} else {
			vala_parser_skip_symbol_name (self, &_inner_error0_);
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					return;
				} else {
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return;
				}
			}
			vala_parser_skip_type_argument_list (self, &_inner_error0_);
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					return;
				} else {
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return;
				}
			}
		}
		while (TRUE) {
			if (!vala_parser_accept (self, VALA_TOKEN_TYPE_STAR)) {
				break;
			}
		}
		vala_parser_accept (self, VALA_TOKEN_TYPE_INTERR);
	}
	while (TRUE) {
		if (!vala_parser_accept (self, VALA_TOKEN_TYPE_OPEN_BRACKET)) {
			break;
		}
		{
			gboolean _tmp0_ = FALSE;
			_tmp0_ = TRUE;
			while (TRUE) {
				gboolean _tmp1_ = FALSE;
				gboolean _tmp2_ = FALSE;
				if (!_tmp0_) {
					if (!vala_parser_accept (self, VALA_TOKEN_TYPE_COMMA)) {
						break;
					}
				}
				_tmp0_ = FALSE;
				if (vala_parser_current (self) != VALA_TOKEN_TYPE_COMMA) {
					_tmp2_ = vala_parser_current (self) != VALA_TOKEN_TYPE_CLOSE_BRACKET;
				} else {
					_tmp2_ = FALSE;
				}
				if (_tmp2_) {
					_tmp1_ = vala_parser_current (self) != VALA_TOKEN_TYPE_COLON;
				} else {
					_tmp1_ = FALSE;
				}
				if (_tmp1_) {
					ValaExpression* _tmp3_;
					ValaExpression* _tmp4_;
					_tmp3_ = vala_parser_parse_expression (self, &_inner_error0_);
					_tmp4_ = _tmp3_;
					_vala_code_node_unref0 (_tmp4_);
					if (G_UNLIKELY (_inner_error0_ != NULL)) {
						if (_inner_error0_->domain == VALA_PARSE_ERROR) {
							g_propagate_error (error, _inner_error0_);
							return;
						} else {
							g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
							g_clear_error (&_inner_error0_);
							return;
						}
					}
				}
			}
		}
		if (vala_parser_accept (self, VALA_TOKEN_TYPE_COLON)) {
			vala_parser_skip_symbol_name (self, &_inner_error0_);
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					return;
				} else {
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return;
				}
			}
		}
		vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_BRACKET, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				return;
			} else {
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
		vala_parser_accept (self, VALA_TOKEN_TYPE_INTERR);
	}
	vala_parser_accept (self, VALA_TOKEN_TYPE_OP_NEG);
}

static gboolean
vala_parser_is_inner_array_type (ValaParser* self)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	gboolean _tmp1_ = FALSE;
	gboolean _tmp2_ = FALSE;
	gboolean _result_ = FALSE;
	ValaSourceLocation _tmp3_;
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	if (vala_parser_accept (self, VALA_TOKEN_TYPE_OPEN_PARENS)) {
		_tmp2_ = vala_parser_accept (self, VALA_TOKEN_TYPE_UNOWNED);
	} else {
		_tmp2_ = FALSE;
	}
	if (_tmp2_) {
		_tmp1_ = vala_parser_current (self) != VALA_TOKEN_TYPE_CLOSE_PARENS;
	} else {
		_tmp1_ = FALSE;
	}
	_result_ = _tmp1_;
	_tmp3_ = begin;
	vala_parser_rollback (self, &_tmp3_);
	result = _result_;
	return result;
}

static gpointer
_vala_code_node_ref0 (gpointer self)
{
	return self ? vala_code_node_ref (self) : NULL;
}

static ValaDataType*
vala_parser_parse_type (ValaParser* self,
                        gboolean owned_by_default,
                        gboolean can_weak_ref,
                        gboolean require_unowned,
                        GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	gboolean is_dynamic = FALSE;
	gboolean value_owned = FALSE;
	ValaDataType* type = NULL;
	gboolean inner_type_owned = FALSE;
	ValaDataType* array_length_type = NULL;
	ValaDataType* _tmp76_;
	ValaDataType* _tmp77_;
	ValaDataType* _tmp78_;
	GError* _inner_error0_ = NULL;
	ValaDataType* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	is_dynamic = vala_parser_accept (self, VALA_TOKEN_TYPE_DYNAMIC);
	value_owned = owned_by_default;
	if (require_unowned) {
		vala_parser_expect (self, VALA_TOKEN_TYPE_UNOWNED, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				return NULL;
			} else {
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
	} else {
		if (owned_by_default) {
			if (vala_parser_accept (self, VALA_TOKEN_TYPE_UNOWNED)) {
				value_owned = FALSE;
			} else {
				if (vala_parser_accept (self, VALA_TOKEN_TYPE_WEAK)) {
					gboolean _tmp1_ = FALSE;
					if (!can_weak_ref) {
						ValaCodeContext* _tmp2_;
						gboolean _tmp3_;
						gboolean _tmp4_;
						_tmp2_ = self->priv->context;
						_tmp3_ = vala_code_context_get_deprecated (_tmp2_);
						_tmp4_ = _tmp3_;
						_tmp1_ = !_tmp4_;
					} else {
						_tmp1_ = FALSE;
					}
					if (_tmp1_) {
						ValaSourceReference* _tmp5_;
						ValaSourceReference* _tmp6_;
						_tmp5_ = vala_parser_get_last_src (self);
						_tmp6_ = _tmp5_;
						vala_report_warning (_tmp6_, "deprecated syntax, use `unowned` modifier");
						_vala_source_reference_unref0 (_tmp6_);
					}
					value_owned = FALSE;
				} else {
					if (vala_parser_accept (self, VALA_TOKEN_TYPE_OWNED)) {
						ValaSourceReference* _tmp7_;
						ValaSourceReference* _tmp8_;
						_tmp7_ = vala_parser_get_last_src (self);
						_tmp8_ = _tmp7_;
						vala_report_warning (_tmp8_, "`owned' is default in this context");
						_vala_source_reference_unref0 (_tmp8_);
					}
				}
			}
		} else {
			if (vala_parser_accept (self, VALA_TOKEN_TYPE_OWNED)) {
				value_owned = TRUE;
			} else {
				value_owned = FALSE;
				if (vala_parser_accept (self, VALA_TOKEN_TYPE_UNOWNED)) {
					ValaSourceReference* _tmp9_;
					ValaSourceReference* _tmp10_;
					_tmp9_ = vala_parser_get_last_src (self);
					_tmp10_ = _tmp9_;
					vala_report_warning (_tmp10_, "`unowned' is default in this context");
					_vala_source_reference_unref0 (_tmp10_);
				}
			}
		}
	}
	inner_type_owned = TRUE;
	if (vala_parser_accept (self, VALA_TOKEN_TYPE_OPEN_PARENS)) {
		ValaDataType* _tmp11_ = NULL;
		ValaDataType* _tmp12_;
		ValaDataType* _tmp13_;
		_tmp12_ = vala_parser_parse_type (self, FALSE, FALSE, TRUE, &_inner_error0_);
		_tmp11_ = _tmp12_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (type);
				return NULL;
			} else {
				_vala_code_node_unref0 (type);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp13_ = _tmp11_;
		_tmp11_ = NULL;
		_vala_code_node_unref0 (type);
		type = _tmp13_;
		vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_PARENS, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (_tmp11_);
				_vala_code_node_unref0 (type);
				return NULL;
			} else {
				_vala_code_node_unref0 (_tmp11_);
				_vala_code_node_unref0 (type);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		inner_type_owned = FALSE;
		vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_BRACKET, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (_tmp11_);
				_vala_code_node_unref0 (type);
				return NULL;
			} else {
				_vala_code_node_unref0 (_tmp11_);
				_vala_code_node_unref0 (type);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		vala_parser_prev (self);
		_vala_code_node_unref0 (_tmp11_);
	} else {
		gboolean _tmp14_ = FALSE;
		gboolean _tmp15_ = FALSE;
		ValaDataType* _tmp43_;
		if (!is_dynamic) {
			_tmp15_ = value_owned == owned_by_default;
		} else {
			_tmp15_ = FALSE;
		}
		if (_tmp15_) {
			_tmp14_ = vala_parser_accept (self, VALA_TOKEN_TYPE_VOID);
		} else {
			_tmp14_ = FALSE;
		}
		if (_tmp14_) {
			ValaSourceLocation _tmp16_;
			ValaSourceReference* _tmp17_;
			ValaSourceReference* _tmp18_;
			ValaVoidType* _tmp19_;
			_tmp16_ = begin;
			_tmp17_ = vala_parser_get_src (self, &_tmp16_);
			_tmp18_ = _tmp17_;
			_tmp19_ = vala_void_type_new (_tmp18_);
			_vala_code_node_unref0 (type);
			type = (ValaDataType*) _tmp19_;
			_vala_source_reference_unref0 (_tmp18_);
		} else {
			ValaUnresolvedSymbol* sym = NULL;
			ValaUnresolvedSymbol* _tmp20_;
			ValaList* type_arg_list = NULL;
			ValaList* _tmp21_;
			ValaUnresolvedSymbol* _tmp22_;
			ValaSourceLocation _tmp23_;
			ValaSourceReference* _tmp24_;
			ValaSourceReference* _tmp25_;
			ValaUnresolvedType* _tmp26_;
			ValaList* _tmp27_;
			_tmp20_ = vala_parser_parse_symbol_name (self, &_inner_error0_);
			sym = _tmp20_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (type);
					return NULL;
				} else {
					_vala_code_node_unref0 (type);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp21_ = vala_parser_parse_type_argument_list (self, FALSE, &_inner_error0_);
			type_arg_list = _tmp21_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (sym);
					_vala_code_node_unref0 (type);
					return NULL;
				} else {
					_vala_code_node_unref0 (sym);
					_vala_code_node_unref0 (type);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp22_ = sym;
			_tmp23_ = begin;
			_tmp24_ = vala_parser_get_src (self, &_tmp23_);
			_tmp25_ = _tmp24_;
			_tmp26_ = vala_unresolved_type_new_from_symbol (_tmp22_, _tmp25_);
			_vala_code_node_unref0 (type);
			type = (ValaDataType*) _tmp26_;
			_vala_source_reference_unref0 (_tmp25_);
			_tmp27_ = type_arg_list;
			if (_tmp27_ != NULL) {
				{
					ValaList* _type_arg_list = NULL;
					ValaList* _tmp28_;
					gint _type_arg_size = 0;
					ValaList* _tmp29_;
					gint _tmp30_;
					gint _tmp31_;
					gint _type_arg_index = 0;
					_tmp28_ = type_arg_list;
					_type_arg_list = _tmp28_;
					_tmp29_ = _type_arg_list;
					_tmp30_ = vala_collection_get_size ((ValaCollection*) _tmp29_);
					_tmp31_ = _tmp30_;
					_type_arg_size = _tmp31_;
					_type_arg_index = -1;
					while (TRUE) {
						gint _tmp32_;
						gint _tmp33_;
						ValaDataType* type_arg = NULL;
						ValaList* _tmp34_;
						gpointer _tmp35_;
						ValaDataType* _tmp36_;
						ValaDataType* _tmp37_;
						_type_arg_index = _type_arg_index + 1;
						_tmp32_ = _type_arg_index;
						_tmp33_ = _type_arg_size;
						if (!(_tmp32_ < _tmp33_)) {
							break;
						}
						_tmp34_ = _type_arg_list;
						_tmp35_ = vala_list_get (_tmp34_, _type_arg_index);
						type_arg = (ValaDataType*) _tmp35_;
						_tmp36_ = type;
						_tmp37_ = type_arg;
						vala_data_type_add_type_argument (_tmp36_, _tmp37_);
						_vala_code_node_unref0 (type_arg);
					}
				}
			}
			_vala_iterable_unref0 (type_arg_list);
			_vala_code_node_unref0 (sym);
		}
		while (TRUE) {
			ValaDataType* _tmp38_;
			ValaSourceLocation _tmp39_;
			ValaSourceReference* _tmp40_;
			ValaSourceReference* _tmp41_;
			ValaPointerType* _tmp42_;
			if (!vala_parser_accept (self, VALA_TOKEN_TYPE_STAR)) {
				break;
			}
			_tmp38_ = type;
			_tmp39_ = begin;
			_tmp40_ = vala_parser_get_src (self, &_tmp39_);
			_tmp41_ = _tmp40_;
			_tmp42_ = vala_pointer_type_new (_tmp38_, _tmp41_);
			_vala_code_node_unref0 (type);
			type = (ValaDataType*) _tmp42_;
			_vala_source_reference_unref0 (_tmp41_);
		}
		_tmp43_ = type;
		if (!VALA_IS_POINTER_TYPE (_tmp43_)) {
			ValaDataType* _tmp44_;
			ValaDataType* _tmp45_;
			ValaSourceLocation _tmp46_;
			ValaSourceReference* _tmp47_;
			ValaSourceReference* _tmp48_;
			_tmp44_ = type;
			vala_data_type_set_nullable (_tmp44_, vala_parser_accept (self, VALA_TOKEN_TYPE_INTERR));
			_tmp45_ = type;
			_tmp46_ = begin;
			_tmp47_ = vala_parser_get_src (self, &_tmp46_);
			_tmp48_ = _tmp47_;
			vala_code_node_set_source_reference ((ValaCodeNode*) _tmp45_, _tmp48_);
			_vala_source_reference_unref0 (_tmp48_);
		}
	}
	array_length_type = NULL;
	while (TRUE) {
		gboolean invalid_array = FALSE;
		gint array_rank = 0;
		ValaDataType* _tmp58_;
		ValaArrayType* array_type = NULL;
		ValaDataType* _tmp59_;
		ValaSourceLocation _tmp60_;
		ValaSourceReference* _tmp61_;
		ValaSourceReference* _tmp62_;
		ValaArrayType* _tmp63_;
		ValaArrayType* _tmp64_;
		ValaArrayType* _tmp65_;
		ValaArrayType* _tmp66_;
		ValaDataType* _tmp67_;
		ValaArrayType* _tmp72_;
		ValaDataType* _tmp73_;
		if (!vala_parser_accept (self, VALA_TOKEN_TYPE_OPEN_BRACKET)) {
			break;
		}
		invalid_array = FALSE;
		array_rank = 0;
		{
			gboolean _tmp49_ = FALSE;
			_tmp49_ = TRUE;
			while (TRUE) {
				gint _tmp50_;
				gboolean _tmp51_ = FALSE;
				gboolean _tmp52_ = FALSE;
				if (!_tmp49_) {
					if (!vala_parser_accept (self, VALA_TOKEN_TYPE_COMMA)) {
						break;
					}
				}
				_tmp49_ = FALSE;
				_tmp50_ = array_rank;
				array_rank = _tmp50_ + 1;
				if (vala_parser_current (self) != VALA_TOKEN_TYPE_COMMA) {
					_tmp52_ = vala_parser_current (self) != VALA_TOKEN_TYPE_CLOSE_BRACKET;
				} else {
					_tmp52_ = FALSE;
				}
				if (_tmp52_) {
					_tmp51_ = vala_parser_current (self) != VALA_TOKEN_TYPE_COLON;
				} else {
					_tmp51_ = FALSE;
				}
				if (_tmp51_) {
					ValaExpression* _tmp53_;
					ValaExpression* _tmp54_;
					_tmp53_ = vala_parser_parse_expression (self, &_inner_error0_);
					_tmp54_ = _tmp53_;
					_vala_code_node_unref0 (_tmp54_);
					if (G_UNLIKELY (_inner_error0_ != NULL)) {
						if (_inner_error0_->domain == VALA_PARSE_ERROR) {
							g_propagate_error (error, _inner_error0_);
							_vala_code_node_unref0 (array_length_type);
							_vala_code_node_unref0 (type);
							return NULL;
						} else {
							_vala_code_node_unref0 (array_length_type);
							_vala_code_node_unref0 (type);
							g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
							g_clear_error (&_inner_error0_);
							return NULL;
						}
					}
					invalid_array = TRUE;
				}
			}
		}
		if (vala_parser_accept (self, VALA_TOKEN_TYPE_COLON)) {
			ValaDataType* _tmp55_ = NULL;
			ValaDataType* _tmp56_;
			ValaDataType* _tmp57_;
			_tmp56_ = vala_parser_parse_type (self, TRUE, FALSE, FALSE, &_inner_error0_);
			_tmp55_ = _tmp56_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (array_length_type);
					_vala_code_node_unref0 (type);
					return NULL;
				} else {
					_vala_code_node_unref0 (array_length_type);
					_vala_code_node_unref0 (type);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp57_ = _tmp55_;
			_tmp55_ = NULL;
			_vala_code_node_unref0 (array_length_type);
			array_length_type = _tmp57_;
			_vala_code_node_unref0 (_tmp55_);
		}
		vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_BRACKET, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (array_length_type);
				_vala_code_node_unref0 (type);
				return NULL;
			} else {
				_vala_code_node_unref0 (array_length_type);
				_vala_code_node_unref0 (type);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp58_ = type;
		vala_data_type_set_value_owned (_tmp58_, inner_type_owned);
		_tmp59_ = type;
		_tmp60_ = begin;
		_tmp61_ = vala_parser_get_src (self, &_tmp60_);
		_tmp62_ = _tmp61_;
		_tmp63_ = vala_array_type_new (_tmp59_, array_rank, _tmp62_);
		_tmp64_ = _tmp63_;
		_vala_source_reference_unref0 (_tmp62_);
		array_type = _tmp64_;
		_tmp65_ = array_type;
		vala_data_type_set_nullable ((ValaDataType*) _tmp65_, vala_parser_accept (self, VALA_TOKEN_TYPE_INTERR));
		_tmp66_ = array_type;
		vala_array_type_set_invalid_syntax (_tmp66_, invalid_array);
		_tmp67_ = array_length_type;
		if (_tmp67_ != NULL) {
			ValaArrayType* _tmp68_;
			ValaDataType* _tmp69_;
			ValaDataType* _tmp70_;
			ValaDataType* _tmp71_;
			_tmp68_ = array_type;
			_tmp69_ = array_length_type;
			_tmp70_ = vala_data_type_copy (_tmp69_);
			_tmp71_ = _tmp70_;
			vala_array_type_set_length_type (_tmp68_, _tmp71_);
			_vala_code_node_unref0 (_tmp71_);
		}
		_tmp72_ = array_type;
		_tmp73_ = _vala_code_node_ref0 ((ValaDataType*) _tmp72_);
		_vala_code_node_unref0 (type);
		type = _tmp73_;
		_vala_code_node_unref0 (array_type);
	}
	if (vala_parser_accept (self, VALA_TOKEN_TYPE_OP_NEG)) {
		ValaSourceReference* _tmp74_;
		ValaSourceReference* _tmp75_;
		_tmp74_ = vala_parser_get_last_src (self);
		_tmp75_ = _tmp74_;
		vala_report_warning (_tmp75_, "obsolete syntax, types are non-null by default");
		_vala_source_reference_unref0 (_tmp75_);
	}
	_tmp76_ = type;
	if (VALA_IS_POINTER_TYPE (_tmp76_)) {
		value_owned = FALSE;
	}
	_tmp77_ = type;
	vala_data_type_set_is_dynamic (_tmp77_, is_dynamic);
	_tmp78_ = type;
	vala_data_type_set_value_owned (_tmp78_, value_owned);
	result = type;
	_vala_code_node_unref0 (array_length_type);
	return result;
}

static ValaDataType*
vala_parser_parse_inline_array_type (ValaParser* self,
                                     ValaDataType* type,
                                     GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	gboolean _tmp1_ = FALSE;
	ValaDataType* _tmp24_;
	GError* _inner_error0_ = NULL;
	ValaDataType* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	if (type != NULL) {
		_tmp1_ = vala_parser_accept (self, VALA_TOKEN_TYPE_OPEN_BRACKET);
	} else {
		_tmp1_ = FALSE;
	}
	if (_tmp1_) {
		ValaExpression* array_length = NULL;
		ValaDataType* array_length_type = NULL;
		ValaArrayType* array_type = NULL;
		ValaSourceLocation _tmp8_;
		ValaSourceReference* _tmp9_;
		ValaSourceReference* _tmp10_;
		ValaArrayType* _tmp11_;
		ValaArrayType* _tmp12_;
		ValaArrayType* _tmp13_;
		ValaExpression* _tmp14_;
		ValaDataType* _tmp18_;
		ValaArrayType* _tmp21_;
		gboolean _tmp22_;
		gboolean _tmp23_;
		array_length = NULL;
		array_length_type = NULL;
		if (vala_parser_current (self) != VALA_TOKEN_TYPE_CLOSE_BRACKET) {
			ValaExpression* _tmp2_ = NULL;
			ValaExpression* _tmp3_;
			ValaExpression* _tmp4_;
			_tmp3_ = vala_parser_parse_expression (self, &_inner_error0_);
			_tmp2_ = _tmp3_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (array_length_type);
					_vala_code_node_unref0 (array_length);
					return NULL;
				} else {
					_vala_code_node_unref0 (array_length_type);
					_vala_code_node_unref0 (array_length);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp4_ = _tmp2_;
			_tmp2_ = NULL;
			_vala_code_node_unref0 (array_length);
			array_length = _tmp4_;
			_vala_code_node_unref0 (_tmp2_);
		}
		if (vala_parser_accept (self, VALA_TOKEN_TYPE_COLON)) {
			ValaDataType* _tmp5_ = NULL;
			ValaDataType* _tmp6_;
			ValaDataType* _tmp7_;
			_tmp6_ = vala_parser_parse_type (self, TRUE, FALSE, FALSE, &_inner_error0_);
			_tmp5_ = _tmp6_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (array_length_type);
					_vala_code_node_unref0 (array_length);
					return NULL;
				} else {
					_vala_code_node_unref0 (array_length_type);
					_vala_code_node_unref0 (array_length);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp7_ = _tmp5_;
			_tmp5_ = NULL;
			_vala_code_node_unref0 (array_length_type);
			array_length_type = _tmp7_;
			_vala_code_node_unref0 (_tmp5_);
		}
		vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_BRACKET, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (array_length_type);
				_vala_code_node_unref0 (array_length);
				return NULL;
			} else {
				_vala_code_node_unref0 (array_length_type);
				_vala_code_node_unref0 (array_length);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp8_ = begin;
		_tmp9_ = vala_parser_get_src (self, &_tmp8_);
		_tmp10_ = _tmp9_;
		_tmp11_ = vala_array_type_new (type, 1, _tmp10_);
		_tmp12_ = _tmp11_;
		_vala_source_reference_unref0 (_tmp10_);
		array_type = _tmp12_;
		_tmp13_ = array_type;
		vala_array_type_set_inline_allocated (_tmp13_, TRUE);
		_tmp14_ = array_length;
		if (_tmp14_ != NULL) {
			ValaArrayType* _tmp15_;
			ValaArrayType* _tmp16_;
			ValaExpression* _tmp17_;
			_tmp15_ = array_type;
			vala_array_type_set_fixed_length (_tmp15_, TRUE);
			_tmp16_ = array_type;
			_tmp17_ = array_length;
			vala_array_type_set_length (_tmp16_, _tmp17_);
		}
		_tmp18_ = array_length_type;
		if (_tmp18_ != NULL) {
			ValaArrayType* _tmp19_;
			ValaDataType* _tmp20_;
			_tmp19_ = array_type;
			_tmp20_ = array_length_type;
			vala_array_type_set_length_type (_tmp19_, _tmp20_);
		}
		_tmp21_ = array_type;
		_tmp22_ = vala_data_type_get_value_owned (type);
		_tmp23_ = _tmp22_;
		vala_data_type_set_value_owned ((ValaDataType*) _tmp21_, _tmp23_);
		result = (ValaDataType*) array_type;
		_vala_code_node_unref0 (array_length_type);
		_vala_code_node_unref0 (array_length);
		return result;
	}
	_tmp24_ = _vala_code_node_ref0 (type);
	result = _tmp24_;
	return result;
}

static ValaList*
vala_parser_parse_argument_list (ValaParser* self,
                                 GError** error)
{
	ValaArrayList* list = NULL;
	GEqualFunc _tmp0_;
	ValaArrayList* _tmp1_;
	GError* _inner_error0_ = NULL;
	ValaList* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp0_ = g_direct_equal;
	_tmp1_ = vala_array_list_new (VALA_TYPE_EXPRESSION, (GBoxedCopyFunc) vala_code_node_ref, (GDestroyNotify) vala_code_node_unref, _tmp0_);
	list = _tmp1_;
	if (vala_parser_current (self) != VALA_TOKEN_TYPE_CLOSE_PARENS) {
		{
			gboolean _tmp2_ = FALSE;
			_tmp2_ = TRUE;
			while (TRUE) {
				if (!_tmp2_) {
					if (!vala_parser_accept (self, VALA_TOKEN_TYPE_COMMA)) {
						break;
					}
				}
				_tmp2_ = FALSE;
				{
					ValaExpression* _tmp3_ = NULL;
					ValaExpression* _tmp4_;
					ValaArrayList* _tmp5_;
					_tmp4_ = vala_parser_parse_argument (self, &_inner_error0_);
					_tmp3_ = _tmp4_;
					if (G_UNLIKELY (_inner_error0_ != NULL)) {
						if (_inner_error0_->domain == VALA_PARSE_ERROR) {
							goto __catch0_vala_parse_error;
						}
						_vala_iterable_unref0 (list);
						g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return NULL;
					}
					_tmp5_ = list;
					vala_collection_add ((ValaCollection*) _tmp5_, _tmp3_);
					_vala_code_node_unref0 (_tmp3_);
				}
				goto __finally0;
				__catch0_vala_parse_error:
				{
					GError* e = NULL;
					e = _inner_error0_;
					_inner_error0_ = NULL;
					if (vala_parser_current (self) == VALA_TOKEN_TYPE_CLOSE_PARENS) {
						GError* _tmp6_;
						GError* _tmp7_;
						vala_parser_prev (self);
						_tmp6_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "incomplete argument list");
						_tmp7_ = _tmp6_;
						vala_parser_report_parse_error (self, _tmp7_);
						_g_error_free0 (_tmp7_);
					} else {
						ValaCodeContext* _tmp8_;
						gboolean _tmp9_;
						gboolean _tmp10_;
						_tmp8_ = self->priv->context;
						_tmp9_ = vala_code_context_get_keep_going (_tmp8_);
						_tmp10_ = _tmp9_;
						if (_tmp10_) {
							GError* _tmp11_;
							_tmp11_ = e;
							vala_parser_report_parse_error (self, _tmp11_);
						} else {
							GError* _tmp12_;
							GError* _tmp13_;
							_tmp12_ = e;
							_tmp13_ = _g_error_copy0 (_tmp12_);
							_inner_error0_ = _tmp13_;
							_g_error_free0 (e);
							goto __finally0;
						}
					}
					_g_error_free0 (e);
				}
				__finally0:
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_iterable_unref0 (list);
						return NULL;
					} else {
						_vala_iterable_unref0 (list);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return NULL;
					}
				}
			}
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_iterable_unref0 (list);
			return NULL;
		} else {
			_vala_iterable_unref0 (list);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	result = (ValaList*) list;
	return result;
}

static ValaExpression*
vala_parser_parse_argument (ValaParser* self,
                            GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	if (vala_parser_accept (self, VALA_TOKEN_TYPE_REF)) {
		ValaExpression* inner = NULL;
		ValaExpression* _tmp1_;
		ValaExpression* _tmp2_;
		ValaSourceLocation _tmp3_;
		ValaSourceReference* _tmp4_;
		ValaSourceReference* _tmp5_;
		ValaUnaryExpression* _tmp6_;
		ValaExpression* _tmp7_;
		_tmp1_ = vala_parser_parse_expression (self, &_inner_error0_);
		inner = _tmp1_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				return NULL;
			} else {
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp2_ = inner;
		_tmp3_ = begin;
		_tmp4_ = vala_parser_get_src (self, &_tmp3_);
		_tmp5_ = _tmp4_;
		_tmp6_ = vala_unary_expression_new (VALA_UNARY_OPERATOR_REF, _tmp2_, _tmp5_);
		_tmp7_ = (ValaExpression*) _tmp6_;
		_vala_source_reference_unref0 (_tmp5_);
		result = _tmp7_;
		_vala_code_node_unref0 (inner);
		return result;
	} else {
		if (vala_parser_accept (self, VALA_TOKEN_TYPE_OUT)) {
			ValaExpression* inner = NULL;
			ValaExpression* _tmp8_;
			ValaExpression* _tmp9_;
			ValaSourceLocation _tmp10_;
			ValaSourceReference* _tmp11_;
			ValaSourceReference* _tmp12_;
			ValaUnaryExpression* _tmp13_;
			ValaExpression* _tmp14_;
			_tmp8_ = vala_parser_parse_expression (self, &_inner_error0_);
			inner = _tmp8_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					return NULL;
				} else {
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp9_ = inner;
			_tmp10_ = begin;
			_tmp11_ = vala_parser_get_src (self, &_tmp10_);
			_tmp12_ = _tmp11_;
			_tmp13_ = vala_unary_expression_new (VALA_UNARY_OPERATOR_OUT, _tmp9_, _tmp12_);
			_tmp14_ = (ValaExpression*) _tmp13_;
			_vala_source_reference_unref0 (_tmp12_);
			result = _tmp14_;
			_vala_code_node_unref0 (inner);
			return result;
		} else {
			ValaExpression* expr = NULL;
			ValaExpression* _tmp15_;
			ValaMemberAccess* ma = NULL;
			ValaExpression* _tmp16_;
			ValaMemberAccess* _tmp17_;
			gboolean _tmp18_ = FALSE;
			gboolean _tmp19_ = FALSE;
			ValaMemberAccess* _tmp20_;
			_tmp15_ = vala_parser_parse_expression (self, &_inner_error0_);
			expr = _tmp15_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					return NULL;
				} else {
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp16_ = expr;
			_tmp17_ = _vala_code_node_ref0 (VALA_IS_MEMBER_ACCESS (_tmp16_) ? ((ValaMemberAccess*) _tmp16_) : NULL);
			ma = _tmp17_;
			_tmp20_ = ma;
			if (_tmp20_ != NULL) {
				ValaMemberAccess* _tmp21_;
				ValaExpression* _tmp22_;
				ValaExpression* _tmp23_;
				_tmp21_ = ma;
				_tmp22_ = vala_member_access_get_inner (_tmp21_);
				_tmp23_ = _tmp22_;
				_tmp19_ = _tmp23_ == NULL;
			} else {
				_tmp19_ = FALSE;
			}
			if (_tmp19_) {
				_tmp18_ = vala_parser_accept (self, VALA_TOKEN_TYPE_COLON);
			} else {
				_tmp18_ = FALSE;
			}
			if (_tmp18_) {
				ValaExpression* _tmp24_ = NULL;
				ValaExpression* _tmp25_;
				ValaExpression* _tmp26_;
				ValaMemberAccess* _tmp27_;
				const gchar* _tmp28_;
				const gchar* _tmp29_;
				ValaExpression* _tmp30_;
				ValaSourceLocation _tmp31_;
				ValaSourceReference* _tmp32_;
				ValaSourceReference* _tmp33_;
				ValaNamedArgument* _tmp34_;
				ValaExpression* _tmp35_;
				_tmp25_ = vala_parser_parse_expression (self, &_inner_error0_);
				_tmp24_ = _tmp25_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_code_node_unref0 (ma);
						_vala_code_node_unref0 (expr);
						return NULL;
					} else {
						_vala_code_node_unref0 (ma);
						_vala_code_node_unref0 (expr);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return NULL;
					}
				}
				_tmp26_ = _tmp24_;
				_tmp24_ = NULL;
				_vala_code_node_unref0 (expr);
				expr = _tmp26_;
				_tmp27_ = ma;
				_tmp28_ = vala_member_access_get_member_name (_tmp27_);
				_tmp29_ = _tmp28_;
				_tmp30_ = expr;
				_tmp31_ = begin;
				_tmp32_ = vala_parser_get_src (self, &_tmp31_);
				_tmp33_ = _tmp32_;
				_tmp34_ = vala_named_argument_new (_tmp29_, _tmp30_, _tmp33_);
				_tmp35_ = (ValaExpression*) _tmp34_;
				_vala_source_reference_unref0 (_tmp33_);
				result = _tmp35_;
				_vala_code_node_unref0 (_tmp24_);
				_vala_code_node_unref0 (ma);
				_vala_code_node_unref0 (expr);
				return result;
			} else {
				result = expr;
				_vala_code_node_unref0 (ma);
				return result;
			}
			_vala_code_node_unref0 (ma);
			_vala_code_node_unref0 (expr);
		}
	}
}

static ValaExpression*
vala_parser_parse_primary_expression (ValaParser* self,
                                      GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaExpression* expr = NULL;
	gboolean found = FALSE;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	switch (vala_parser_current (self)) {
		case VALA_TOKEN_TYPE_TRUE:
		case VALA_TOKEN_TYPE_FALSE:
		case VALA_TOKEN_TYPE_INTEGER_LITERAL:
		case VALA_TOKEN_TYPE_REAL_LITERAL:
		case VALA_TOKEN_TYPE_CHARACTER_LITERAL:
		case VALA_TOKEN_TYPE_STRING_LITERAL:
		case VALA_TOKEN_TYPE_REGEX_LITERAL:
		case VALA_TOKEN_TYPE_TEMPLATE_STRING_LITERAL:
		case VALA_TOKEN_TYPE_VERBATIM_STRING_LITERAL:
		case VALA_TOKEN_TYPE_NULL:
		{
			ValaExpression* _tmp1_ = NULL;
			ValaExpression* _tmp2_;
			ValaExpression* _tmp3_;
			_tmp2_ = vala_parser_parse_literal (self, &_inner_error0_);
			_tmp1_ = _tmp2_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (expr);
					return NULL;
				} else {
					_vala_code_node_unref0 (expr);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp3_ = _tmp1_;
			_tmp1_ = NULL;
			_vala_code_node_unref0 (expr);
			expr = _tmp3_;
			_vala_code_node_unref0 (_tmp1_);
			break;
		}
		case VALA_TOKEN_TYPE_OPEN_BRACE:
		{
			ValaInitializerList* _tmp4_ = NULL;
			ValaInitializerList* _tmp5_;
			ValaInitializerList* _tmp6_;
			_tmp5_ = vala_parser_parse_initializer (self, &_inner_error0_);
			_tmp4_ = _tmp5_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (expr);
					return NULL;
				} else {
					_vala_code_node_unref0 (expr);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp6_ = _tmp4_;
			_tmp4_ = NULL;
			_vala_code_node_unref0 (expr);
			expr = (ValaExpression*) _tmp6_;
			_vala_code_node_unref0 (_tmp4_);
			break;
		}
		case VALA_TOKEN_TYPE_OPEN_BRACKET:
		{
			ValaExpression* _tmp7_ = NULL;
			ValaExpression* _tmp8_;
			ValaExpression* _tmp9_;
			_tmp8_ = vala_parser_parse_simple_name (self, &_inner_error0_);
			_tmp7_ = _tmp8_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (expr);
					return NULL;
				} else {
					_vala_code_node_unref0 (expr);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp9_ = _tmp7_;
			_tmp7_ = NULL;
			_vala_code_node_unref0 (expr);
			expr = _tmp9_;
			_vala_code_node_unref0 (_tmp7_);
			break;
		}
		case VALA_TOKEN_TYPE_OPEN_PARENS:
		{
			ValaExpression* _tmp10_ = NULL;
			ValaExpression* _tmp11_;
			ValaExpression* _tmp12_;
			_tmp11_ = vala_parser_parse_tuple (self, &_inner_error0_);
			_tmp10_ = _tmp11_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (expr);
					return NULL;
				} else {
					_vala_code_node_unref0 (expr);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp12_ = _tmp10_;
			_tmp10_ = NULL;
			_vala_code_node_unref0 (expr);
			expr = _tmp12_;
			_vala_code_node_unref0 (_tmp10_);
			break;
		}
		case VALA_TOKEN_TYPE_OPEN_TEMPLATE:
		{
			ValaExpression* _tmp13_ = NULL;
			ValaExpression* _tmp14_;
			ValaExpression* _tmp15_;
			_tmp14_ = vala_parser_parse_template (self, &_inner_error0_);
			_tmp13_ = _tmp14_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (expr);
					return NULL;
				} else {
					_vala_code_node_unref0 (expr);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp15_ = _tmp13_;
			_tmp13_ = NULL;
			_vala_code_node_unref0 (expr);
			expr = _tmp15_;
			_vala_code_node_unref0 (_tmp13_);
			break;
		}
		case VALA_TOKEN_TYPE_OPEN_REGEX_LITERAL:
		{
			ValaExpression* _tmp16_ = NULL;
			ValaExpression* _tmp17_;
			ValaExpression* _tmp18_;
			_tmp17_ = vala_parser_parse_regex_literal (self, &_inner_error0_);
			_tmp16_ = _tmp17_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (expr);
					return NULL;
				} else {
					_vala_code_node_unref0 (expr);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp18_ = _tmp16_;
			_tmp16_ = NULL;
			_vala_code_node_unref0 (expr);
			expr = _tmp18_;
			_vala_code_node_unref0 (_tmp16_);
			break;
		}
		case VALA_TOKEN_TYPE_THIS:
		{
			ValaExpression* _tmp19_ = NULL;
			ValaExpression* _tmp20_;
			ValaExpression* _tmp21_;
			_tmp20_ = vala_parser_parse_this_access (self, &_inner_error0_);
			_tmp19_ = _tmp20_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (expr);
					return NULL;
				} else {
					_vala_code_node_unref0 (expr);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp21_ = _tmp19_;
			_tmp19_ = NULL;
			_vala_code_node_unref0 (expr);
			expr = _tmp21_;
			_vala_code_node_unref0 (_tmp19_);
			break;
		}
		case VALA_TOKEN_TYPE_BASE:
		{
			ValaExpression* _tmp22_ = NULL;
			ValaExpression* _tmp23_;
			ValaExpression* _tmp24_;
			_tmp23_ = vala_parser_parse_base_access (self, &_inner_error0_);
			_tmp22_ = _tmp23_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (expr);
					return NULL;
				} else {
					_vala_code_node_unref0 (expr);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp24_ = _tmp22_;
			_tmp22_ = NULL;
			_vala_code_node_unref0 (expr);
			expr = _tmp24_;
			_vala_code_node_unref0 (_tmp22_);
			break;
		}
		case VALA_TOKEN_TYPE_NEW:
		{
			ValaExpression* _tmp25_ = NULL;
			ValaExpression* _tmp26_;
			ValaExpression* _tmp27_;
			_tmp26_ = vala_parser_parse_object_or_array_creation_expression (self, &_inner_error0_);
			_tmp25_ = _tmp26_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (expr);
					return NULL;
				} else {
					_vala_code_node_unref0 (expr);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp27_ = _tmp25_;
			_tmp25_ = NULL;
			_vala_code_node_unref0 (expr);
			expr = _tmp27_;
			_vala_code_node_unref0 (_tmp25_);
			break;
		}
		case VALA_TOKEN_TYPE_YIELD:
		{
			ValaExpression* _tmp28_ = NULL;
			ValaExpression* _tmp29_;
			ValaExpression* _tmp30_;
			_tmp29_ = vala_parser_parse_yield_expression (self, &_inner_error0_);
			_tmp28_ = _tmp29_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (expr);
					return NULL;
				} else {
					_vala_code_node_unref0 (expr);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp30_ = _tmp28_;
			_tmp28_ = NULL;
			_vala_code_node_unref0 (expr);
			expr = _tmp30_;
			_vala_code_node_unref0 (_tmp28_);
			break;
		}
		case VALA_TOKEN_TYPE_SIZEOF:
		{
			ValaExpression* _tmp31_ = NULL;
			ValaExpression* _tmp32_;
			ValaExpression* _tmp33_;
			_tmp32_ = vala_parser_parse_sizeof_expression (self, &_inner_error0_);
			_tmp31_ = _tmp32_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (expr);
					return NULL;
				} else {
					_vala_code_node_unref0 (expr);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp33_ = _tmp31_;
			_tmp31_ = NULL;
			_vala_code_node_unref0 (expr);
			expr = _tmp33_;
			_vala_code_node_unref0 (_tmp31_);
			break;
		}
		case VALA_TOKEN_TYPE_TYPEOF:
		{
			ValaExpression* _tmp34_ = NULL;
			ValaExpression* _tmp35_;
			ValaExpression* _tmp36_;
			_tmp35_ = vala_parser_parse_typeof_expression (self, &_inner_error0_);
			_tmp34_ = _tmp35_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (expr);
					return NULL;
				} else {
					_vala_code_node_unref0 (expr);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp36_ = _tmp34_;
			_tmp34_ = NULL;
			_vala_code_node_unref0 (expr);
			expr = _tmp36_;
			_vala_code_node_unref0 (_tmp34_);
			break;
		}
		default:
		{
			ValaExpression* _tmp37_ = NULL;
			ValaExpression* _tmp38_;
			ValaExpression* _tmp39_;
			_tmp38_ = vala_parser_parse_simple_name (self, &_inner_error0_);
			_tmp37_ = _tmp38_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (expr);
					return NULL;
				} else {
					_vala_code_node_unref0 (expr);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp39_ = _tmp37_;
			_tmp37_ = NULL;
			_vala_code_node_unref0 (expr);
			expr = _tmp39_;
			_vala_code_node_unref0 (_tmp37_);
			break;
		}
	}
	found = TRUE;
	while (TRUE) {
		if (!found) {
			break;
		}
		switch (vala_parser_current (self)) {
			case VALA_TOKEN_TYPE_INTERR:
			{
				vala_parser_next (self);
				switch (vala_parser_current (self)) {
					case VALA_TOKEN_TYPE_DOT:
					{
						ValaExpression* _tmp40_ = NULL;
						ValaSourceLocation _tmp41_;
						ValaExpression* _tmp42_;
						ValaExpression* _tmp43_;
						ValaExpression* _tmp44_;
						_tmp41_ = begin;
						_tmp42_ = expr;
						_tmp43_ = vala_parser_parse_member_access (self, &_tmp41_, _tmp42_, &_inner_error0_);
						_tmp40_ = _tmp43_;
						if (G_UNLIKELY (_inner_error0_ != NULL)) {
							if (_inner_error0_->domain == VALA_PARSE_ERROR) {
								g_propagate_error (error, _inner_error0_);
								_vala_code_node_unref0 (expr);
								return NULL;
							} else {
								_vala_code_node_unref0 (expr);
								g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
								g_clear_error (&_inner_error0_);
								return NULL;
							}
						}
						_tmp44_ = _tmp40_;
						_tmp40_ = NULL;
						_vala_code_node_unref0 (expr);
						expr = _tmp44_;
						_vala_code_node_unref0 (_tmp40_);
						break;
					}
					case VALA_TOKEN_TYPE_OPEN_BRACKET:
					{
						ValaExpression* _tmp45_ = NULL;
						ValaSourceLocation _tmp46_;
						ValaExpression* _tmp47_;
						ValaExpression* _tmp48_;
						ValaExpression* _tmp49_;
						_tmp46_ = begin;
						_tmp47_ = expr;
						_tmp48_ = vala_parser_parse_element_access (self, &_tmp46_, _tmp47_, &_inner_error0_);
						_tmp45_ = _tmp48_;
						if (G_UNLIKELY (_inner_error0_ != NULL)) {
							if (_inner_error0_->domain == VALA_PARSE_ERROR) {
								g_propagate_error (error, _inner_error0_);
								_vala_code_node_unref0 (expr);
								return NULL;
							} else {
								_vala_code_node_unref0 (expr);
								g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
								g_clear_error (&_inner_error0_);
								return NULL;
							}
						}
						_tmp49_ = _tmp45_;
						_tmp45_ = NULL;
						_vala_code_node_unref0 (expr);
						expr = _tmp49_;
						_vala_code_node_unref0 (_tmp45_);
						break;
					}
					default:
					{
						vala_parser_prev (self);
						found = FALSE;
						break;
					}
				}
				break;
			}
			case VALA_TOKEN_TYPE_DOT:
			{
				ValaExpression* _tmp50_ = NULL;
				ValaSourceLocation _tmp51_;
				ValaExpression* _tmp52_;
				ValaExpression* _tmp53_;
				ValaExpression* _tmp54_;
				_tmp51_ = begin;
				_tmp52_ = expr;
				_tmp53_ = vala_parser_parse_member_access (self, &_tmp51_, _tmp52_, &_inner_error0_);
				_tmp50_ = _tmp53_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_code_node_unref0 (expr);
						return NULL;
					} else {
						_vala_code_node_unref0 (expr);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return NULL;
					}
				}
				_tmp54_ = _tmp50_;
				_tmp50_ = NULL;
				_vala_code_node_unref0 (expr);
				expr = _tmp54_;
				_vala_code_node_unref0 (_tmp50_);
				break;
			}
			case VALA_TOKEN_TYPE_OP_PTR:
			{
				ValaExpression* _tmp55_ = NULL;
				ValaSourceLocation _tmp56_;
				ValaExpression* _tmp57_;
				ValaExpression* _tmp58_;
				ValaExpression* _tmp59_;
				_tmp56_ = begin;
				_tmp57_ = expr;
				_tmp58_ = vala_parser_parse_pointer_member_access (self, &_tmp56_, _tmp57_, &_inner_error0_);
				_tmp55_ = _tmp58_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_code_node_unref0 (expr);
						return NULL;
					} else {
						_vala_code_node_unref0 (expr);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return NULL;
					}
				}
				_tmp59_ = _tmp55_;
				_tmp55_ = NULL;
				_vala_code_node_unref0 (expr);
				expr = _tmp59_;
				_vala_code_node_unref0 (_tmp55_);
				break;
			}
			case VALA_TOKEN_TYPE_OPEN_PARENS:
			{
				ValaExpression* _tmp60_ = NULL;
				ValaSourceLocation _tmp61_;
				ValaExpression* _tmp62_;
				ValaExpression* _tmp63_;
				ValaExpression* _tmp64_;
				_tmp61_ = begin;
				_tmp62_ = expr;
				_tmp63_ = vala_parser_parse_method_call (self, &_tmp61_, _tmp62_, &_inner_error0_);
				_tmp60_ = _tmp63_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_code_node_unref0 (expr);
						return NULL;
					} else {
						_vala_code_node_unref0 (expr);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return NULL;
					}
				}
				_tmp64_ = _tmp60_;
				_tmp60_ = NULL;
				_vala_code_node_unref0 (expr);
				expr = _tmp64_;
				_vala_code_node_unref0 (_tmp60_);
				break;
			}
			case VALA_TOKEN_TYPE_OPEN_BRACKET:
			{
				ValaExpression* _tmp65_ = NULL;
				ValaSourceLocation _tmp66_;
				ValaExpression* _tmp67_;
				ValaExpression* _tmp68_;
				ValaExpression* _tmp69_;
				_tmp66_ = begin;
				_tmp67_ = expr;
				_tmp68_ = vala_parser_parse_element_access (self, &_tmp66_, _tmp67_, &_inner_error0_);
				_tmp65_ = _tmp68_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_code_node_unref0 (expr);
						return NULL;
					} else {
						_vala_code_node_unref0 (expr);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return NULL;
					}
				}
				_tmp69_ = _tmp65_;
				_tmp65_ = NULL;
				_vala_code_node_unref0 (expr);
				expr = _tmp69_;
				_vala_code_node_unref0 (_tmp65_);
				break;
			}
			case VALA_TOKEN_TYPE_OP_INC:
			{
				ValaExpression* _tmp70_ = NULL;
				ValaSourceLocation _tmp71_;
				ValaExpression* _tmp72_;
				ValaExpression* _tmp73_;
				ValaExpression* _tmp74_;
				_tmp71_ = begin;
				_tmp72_ = expr;
				_tmp73_ = vala_parser_parse_post_increment_expression (self, &_tmp71_, _tmp72_, &_inner_error0_);
				_tmp70_ = _tmp73_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_code_node_unref0 (expr);
						return NULL;
					} else {
						_vala_code_node_unref0 (expr);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return NULL;
					}
				}
				_tmp74_ = _tmp70_;
				_tmp70_ = NULL;
				_vala_code_node_unref0 (expr);
				expr = _tmp74_;
				_vala_code_node_unref0 (_tmp70_);
				break;
			}
			case VALA_TOKEN_TYPE_OP_DEC:
			{
				ValaExpression* _tmp75_ = NULL;
				ValaSourceLocation _tmp76_;
				ValaExpression* _tmp77_;
				ValaExpression* _tmp78_;
				ValaExpression* _tmp79_;
				_tmp76_ = begin;
				_tmp77_ = expr;
				_tmp78_ = vala_parser_parse_post_decrement_expression (self, &_tmp76_, _tmp77_, &_inner_error0_);
				_tmp75_ = _tmp78_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_code_node_unref0 (expr);
						return NULL;
					} else {
						_vala_code_node_unref0 (expr);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return NULL;
					}
				}
				_tmp79_ = _tmp75_;
				_tmp75_ = NULL;
				_vala_code_node_unref0 (expr);
				expr = _tmp79_;
				_vala_code_node_unref0 (_tmp75_);
				break;
			}
			default:
			{
				found = FALSE;
				break;
			}
		}
	}
	result = expr;
	return result;
}

static ValaExpression*
vala_parser_parse_simple_name (ValaParser* self,
                               GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	gchar* id = NULL;
	gchar* _tmp1_;
	gboolean qualified = FALSE;
	gboolean _tmp2_ = FALSE;
	const gchar* _tmp3_;
	ValaList* type_arg_list = NULL;
	ValaList* _tmp7_;
	ValaMemberAccess* expr = NULL;
	const gchar* _tmp8_;
	ValaSourceLocation _tmp9_;
	ValaSourceReference* _tmp10_;
	ValaSourceReference* _tmp11_;
	ValaMemberAccess* _tmp12_;
	ValaMemberAccess* _tmp13_;
	ValaMemberAccess* _tmp14_;
	ValaList* _tmp15_;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	_tmp1_ = vala_parser_parse_identifier (self, FALSE, &_inner_error0_);
	id = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	qualified = FALSE;
	_tmp3_ = id;
	if (g_strcmp0 (_tmp3_, "global") == 0) {
		_tmp2_ = vala_parser_accept (self, VALA_TOKEN_TYPE_DOUBLE_COLON);
	} else {
		_tmp2_ = FALSE;
	}
	if (_tmp2_) {
		gchar* _tmp4_ = NULL;
		gchar* _tmp5_;
		gchar* _tmp6_;
		_tmp5_ = vala_parser_parse_identifier (self, FALSE, &_inner_error0_);
		_tmp4_ = _tmp5_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_g_free0 (id);
				return NULL;
			} else {
				_g_free0 (id);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp6_ = _tmp4_;
		_tmp4_ = NULL;
		_g_free0 (id);
		id = _tmp6_;
		qualified = TRUE;
		_g_free0 (_tmp4_);
	}
	_tmp7_ = vala_parser_parse_type_argument_list (self, TRUE, &_inner_error0_);
	type_arg_list = _tmp7_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_g_free0 (id);
			return NULL;
		} else {
			_g_free0 (id);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp8_ = id;
	_tmp9_ = begin;
	_tmp10_ = vala_parser_get_src (self, &_tmp9_);
	_tmp11_ = _tmp10_;
	_tmp12_ = vala_member_access_new (NULL, _tmp8_, _tmp11_);
	_tmp13_ = _tmp12_;
	_vala_source_reference_unref0 (_tmp11_);
	expr = _tmp13_;
	_tmp14_ = expr;
	vala_member_access_set_qualified (_tmp14_, qualified);
	_tmp15_ = type_arg_list;
	if (_tmp15_ != NULL) {
		{
			ValaList* _type_arg_list = NULL;
			ValaList* _tmp16_;
			gint _type_arg_size = 0;
			ValaList* _tmp17_;
			gint _tmp18_;
			gint _tmp19_;
			gint _type_arg_index = 0;
			_tmp16_ = type_arg_list;
			_type_arg_list = _tmp16_;
			_tmp17_ = _type_arg_list;
			_tmp18_ = vala_collection_get_size ((ValaCollection*) _tmp17_);
			_tmp19_ = _tmp18_;
			_type_arg_size = _tmp19_;
			_type_arg_index = -1;
			while (TRUE) {
				gint _tmp20_;
				gint _tmp21_;
				ValaDataType* type_arg = NULL;
				ValaList* _tmp22_;
				gpointer _tmp23_;
				ValaMemberAccess* _tmp24_;
				ValaDataType* _tmp25_;
				_type_arg_index = _type_arg_index + 1;
				_tmp20_ = _type_arg_index;
				_tmp21_ = _type_arg_size;
				if (!(_tmp20_ < _tmp21_)) {
					break;
				}
				_tmp22_ = _type_arg_list;
				_tmp23_ = vala_list_get (_tmp22_, _type_arg_index);
				type_arg = (ValaDataType*) _tmp23_;
				_tmp24_ = expr;
				_tmp25_ = type_arg;
				vala_member_access_add_type_argument (_tmp24_, _tmp25_);
				_vala_code_node_unref0 (type_arg);
			}
		}
	}
	result = (ValaExpression*) expr;
	_vala_iterable_unref0 (type_arg_list);
	_g_free0 (id);
	return result;
}

static ValaExpression*
vala_parser_parse_tuple (ValaParser* self,
                         GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaArrayList* expr_list = NULL;
	GEqualFunc _tmp1_;
	ValaArrayList* _tmp2_;
	ValaArrayList* _tmp7_;
	gint _tmp8_;
	gint _tmp9_;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp1_ = g_direct_equal;
	_tmp2_ = vala_array_list_new (VALA_TYPE_EXPRESSION, (GBoxedCopyFunc) vala_code_node_ref, (GDestroyNotify) vala_code_node_unref, _tmp1_);
	expr_list = _tmp2_;
	if (vala_parser_current (self) != VALA_TOKEN_TYPE_CLOSE_PARENS) {
		{
			gboolean _tmp3_ = FALSE;
			_tmp3_ = TRUE;
			while (TRUE) {
				ValaExpression* _tmp4_ = NULL;
				ValaExpression* _tmp5_;
				ValaArrayList* _tmp6_;
				if (!_tmp3_) {
					if (!vala_parser_accept (self, VALA_TOKEN_TYPE_COMMA)) {
						break;
					}
				}
				_tmp3_ = FALSE;
				_tmp5_ = vala_parser_parse_expression (self, &_inner_error0_);
				_tmp4_ = _tmp5_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_iterable_unref0 (expr_list);
						return NULL;
					} else {
						_vala_iterable_unref0 (expr_list);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return NULL;
					}
				}
				_tmp6_ = expr_list;
				vala_collection_add ((ValaCollection*) _tmp6_, _tmp4_);
				_vala_code_node_unref0 (_tmp4_);
			}
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_iterable_unref0 (expr_list);
			return NULL;
		} else {
			_vala_iterable_unref0 (expr_list);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp7_ = expr_list;
	_tmp8_ = vala_collection_get_size ((ValaCollection*) _tmp7_);
	_tmp9_ = _tmp8_;
	if (_tmp9_ != 1) {
		ValaTuple* tuple = NULL;
		ValaSourceLocation _tmp10_;
		ValaSourceReference* _tmp11_;
		ValaSourceReference* _tmp12_;
		ValaTuple* _tmp13_;
		ValaTuple* _tmp14_;
		_tmp10_ = begin;
		_tmp11_ = vala_parser_get_src (self, &_tmp10_);
		_tmp12_ = _tmp11_;
		_tmp13_ = vala_tuple_new (_tmp12_);
		_tmp14_ = _tmp13_;
		_vala_source_reference_unref0 (_tmp12_);
		tuple = _tmp14_;
		{
			ValaArrayList* _expr_list = NULL;
			ValaArrayList* _tmp15_;
			gint _expr_size = 0;
			ValaArrayList* _tmp16_;
			gint _tmp17_;
			gint _tmp18_;
			gint _expr_index = 0;
			_tmp15_ = expr_list;
			_expr_list = _tmp15_;
			_tmp16_ = _expr_list;
			_tmp17_ = vala_collection_get_size ((ValaCollection*) _tmp16_);
			_tmp18_ = _tmp17_;
			_expr_size = _tmp18_;
			_expr_index = -1;
			while (TRUE) {
				gint _tmp19_;
				gint _tmp20_;
				ValaExpression* expr = NULL;
				ValaArrayList* _tmp21_;
				gpointer _tmp22_;
				ValaTuple* _tmp23_;
				ValaExpression* _tmp24_;
				_expr_index = _expr_index + 1;
				_tmp19_ = _expr_index;
				_tmp20_ = _expr_size;
				if (!(_tmp19_ < _tmp20_)) {
					break;
				}
				_tmp21_ = _expr_list;
				_tmp22_ = vala_list_get ((ValaList*) _tmp21_, _expr_index);
				expr = (ValaExpression*) _tmp22_;
				_tmp23_ = tuple;
				_tmp24_ = expr;
				vala_tuple_add_expression (_tmp23_, _tmp24_);
				_vala_code_node_unref0 (expr);
			}
		}
		result = (ValaExpression*) tuple;
		_vala_iterable_unref0 (expr_list);
		return result;
	} else {
		ValaExpression* expr = NULL;
		ValaArrayList* _tmp25_;
		gpointer _tmp26_;
		ValaExpression* _tmp27_;
		ValaSourceLocation _tmp28_;
		ValaSourceReference* _tmp29_;
		ValaSourceReference* _tmp30_;
		_tmp25_ = expr_list;
		_tmp26_ = vala_list_get ((ValaList*) _tmp25_, 0);
		expr = (ValaExpression*) _tmp26_;
		_tmp27_ = expr;
		_tmp28_ = begin;
		_tmp29_ = vala_parser_get_src (self, &_tmp28_);
		_tmp30_ = _tmp29_;
		vala_code_node_set_source_reference ((ValaCodeNode*) _tmp27_, _tmp30_);
		_vala_source_reference_unref0 (_tmp30_);
		result = expr;
		_vala_iterable_unref0 (expr_list);
		return result;
	}
}

static ValaExpression*
vala_parser_parse_template (ValaParser* self,
                            GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaTemplate* template = NULL;
	ValaTemplate* _tmp1_;
	ValaTemplate* _tmp5_;
	ValaSourceLocation _tmp6_;
	ValaSourceReference* _tmp7_;
	ValaSourceReference* _tmp8_;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	_tmp1_ = vala_template_new (NULL);
	template = _tmp1_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_TEMPLATE, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (template);
			return NULL;
		} else {
			_vala_code_node_unref0 (template);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	while (TRUE) {
		ValaExpression* _tmp2_ = NULL;
		ValaExpression* _tmp3_;
		ValaTemplate* _tmp4_;
		if (!(vala_parser_current (self) != VALA_TOKEN_TYPE_CLOSE_TEMPLATE)) {
			break;
		}
		_tmp3_ = vala_parser_parse_expression (self, &_inner_error0_);
		_tmp2_ = _tmp3_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (template);
				return NULL;
			} else {
				_vala_code_node_unref0 (template);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp4_ = template;
		vala_template_add_expression (_tmp4_, _tmp2_);
		vala_parser_expect (self, VALA_TOKEN_TYPE_COMMA, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (_tmp2_);
				_vala_code_node_unref0 (template);
				return NULL;
			} else {
				_vala_code_node_unref0 (_tmp2_);
				_vala_code_node_unref0 (template);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_vala_code_node_unref0 (_tmp2_);
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_TEMPLATE, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (template);
			return NULL;
		} else {
			_vala_code_node_unref0 (template);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp5_ = template;
	_tmp6_ = begin;
	_tmp7_ = vala_parser_get_src (self, &_tmp6_);
	_tmp8_ = _tmp7_;
	vala_code_node_set_source_reference ((ValaCodeNode*) _tmp5_, _tmp8_);
	_vala_source_reference_unref0 (_tmp8_);
	result = (ValaExpression*) template;
	return result;
}

static ValaExpression*
vala_parser_parse_regex_literal (ValaParser* self,
                                 GError** error)
{
	ValaExpression* expr = NULL;
	ValaExpression* _tmp0_;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_REGEX_LITERAL, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp0_ = vala_parser_parse_literal (self, &_inner_error0_);
	expr = _tmp0_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	result = expr;
	return result;
}

static ValaExpression*
vala_parser_parse_member_access (ValaParser* self,
                                 ValaSourceLocation* begin,
                                 ValaExpression* inner,
                                 GError** error)
{
	gboolean null_safe = FALSE;
	gchar* id = NULL;
	gchar* _tmp0_;
	ValaList* type_arg_list = NULL;
	ValaList* _tmp1_;
	ValaMemberAccess* expr = NULL;
	const gchar* _tmp2_;
	ValaSourceLocation _tmp3_;
	ValaSourceReference* _tmp4_;
	ValaSourceReference* _tmp5_;
	ValaMemberAccess* _tmp6_;
	ValaMemberAccess* _tmp7_;
	ValaMemberAccess* _tmp8_;
	ValaList* _tmp9_;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (begin != NULL, NULL);
	g_return_val_if_fail (inner != NULL, NULL);
	null_safe = vala_parser_previous (self) == VALA_TOKEN_TYPE_INTERR;
	vala_parser_expect (self, VALA_TOKEN_TYPE_DOT, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp0_ = vala_parser_parse_identifier (self, FALSE, &_inner_error0_);
	id = _tmp0_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp1_ = vala_parser_parse_type_argument_list (self, TRUE, &_inner_error0_);
	type_arg_list = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_g_free0 (id);
			return NULL;
		} else {
			_g_free0 (id);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp2_ = id;
	_tmp3_ = *begin;
	_tmp4_ = vala_parser_get_src (self, &_tmp3_);
	_tmp5_ = _tmp4_;
	_tmp6_ = vala_member_access_new (inner, _tmp2_, _tmp5_);
	_tmp7_ = _tmp6_;
	_vala_source_reference_unref0 (_tmp5_);
	expr = _tmp7_;
	_tmp8_ = expr;
	vala_member_access_set_null_safe_access (_tmp8_, null_safe);
	_tmp9_ = type_arg_list;
	if (_tmp9_ != NULL) {
		{
			ValaList* _type_arg_list = NULL;
			ValaList* _tmp10_;
			gint _type_arg_size = 0;
			ValaList* _tmp11_;
			gint _tmp12_;
			gint _tmp13_;
			gint _type_arg_index = 0;
			_tmp10_ = type_arg_list;
			_type_arg_list = _tmp10_;
			_tmp11_ = _type_arg_list;
			_tmp12_ = vala_collection_get_size ((ValaCollection*) _tmp11_);
			_tmp13_ = _tmp12_;
			_type_arg_size = _tmp13_;
			_type_arg_index = -1;
			while (TRUE) {
				gint _tmp14_;
				gint _tmp15_;
				ValaDataType* type_arg = NULL;
				ValaList* _tmp16_;
				gpointer _tmp17_;
				ValaMemberAccess* _tmp18_;
				ValaDataType* _tmp19_;
				_type_arg_index = _type_arg_index + 1;
				_tmp14_ = _type_arg_index;
				_tmp15_ = _type_arg_size;
				if (!(_tmp14_ < _tmp15_)) {
					break;
				}
				_tmp16_ = _type_arg_list;
				_tmp17_ = vala_list_get (_tmp16_, _type_arg_index);
				type_arg = (ValaDataType*) _tmp17_;
				_tmp18_ = expr;
				_tmp19_ = type_arg;
				vala_member_access_add_type_argument (_tmp18_, _tmp19_);
				_vala_code_node_unref0 (type_arg);
			}
		}
	}
	result = (ValaExpression*) expr;
	_vala_iterable_unref0 (type_arg_list);
	_g_free0 (id);
	return result;
}

static ValaExpression*
vala_parser_parse_pointer_member_access (ValaParser* self,
                                         ValaSourceLocation* begin,
                                         ValaExpression* inner,
                                         GError** error)
{
	gchar* id = NULL;
	gchar* _tmp0_;
	ValaList* type_arg_list = NULL;
	ValaList* _tmp1_;
	ValaMemberAccess* expr = NULL;
	const gchar* _tmp2_;
	ValaSourceLocation _tmp3_;
	ValaSourceReference* _tmp4_;
	ValaSourceReference* _tmp5_;
	ValaMemberAccess* _tmp6_;
	ValaMemberAccess* _tmp7_;
	ValaList* _tmp8_;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (begin != NULL, NULL);
	g_return_val_if_fail (inner != NULL, NULL);
	vala_parser_expect (self, VALA_TOKEN_TYPE_OP_PTR, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp0_ = vala_parser_parse_identifier (self, FALSE, &_inner_error0_);
	id = _tmp0_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp1_ = vala_parser_parse_type_argument_list (self, TRUE, &_inner_error0_);
	type_arg_list = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_g_free0 (id);
			return NULL;
		} else {
			_g_free0 (id);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp2_ = id;
	_tmp3_ = *begin;
	_tmp4_ = vala_parser_get_src (self, &_tmp3_);
	_tmp5_ = _tmp4_;
	_tmp6_ = vala_member_access_new_pointer (inner, _tmp2_, _tmp5_);
	_tmp7_ = _tmp6_;
	_vala_source_reference_unref0 (_tmp5_);
	expr = _tmp7_;
	_tmp8_ = type_arg_list;
	if (_tmp8_ != NULL) {
		{
			ValaList* _type_arg_list = NULL;
			ValaList* _tmp9_;
			gint _type_arg_size = 0;
			ValaList* _tmp10_;
			gint _tmp11_;
			gint _tmp12_;
			gint _type_arg_index = 0;
			_tmp9_ = type_arg_list;
			_type_arg_list = _tmp9_;
			_tmp10_ = _type_arg_list;
			_tmp11_ = vala_collection_get_size ((ValaCollection*) _tmp10_);
			_tmp12_ = _tmp11_;
			_type_arg_size = _tmp12_;
			_type_arg_index = -1;
			while (TRUE) {
				gint _tmp13_;
				gint _tmp14_;
				ValaDataType* type_arg = NULL;
				ValaList* _tmp15_;
				gpointer _tmp16_;
				ValaMemberAccess* _tmp17_;
				ValaDataType* _tmp18_;
				_type_arg_index = _type_arg_index + 1;
				_tmp13_ = _type_arg_index;
				_tmp14_ = _type_arg_size;
				if (!(_tmp13_ < _tmp14_)) {
					break;
				}
				_tmp15_ = _type_arg_list;
				_tmp16_ = vala_list_get (_tmp15_, _type_arg_index);
				type_arg = (ValaDataType*) _tmp16_;
				_tmp17_ = expr;
				_tmp18_ = type_arg;
				vala_member_access_add_type_argument (_tmp17_, _tmp18_);
				_vala_code_node_unref0 (type_arg);
			}
		}
	}
	result = (ValaExpression*) expr;
	_vala_iterable_unref0 (type_arg_list);
	_g_free0 (id);
	return result;
}

static ValaExpression*
vala_parser_parse_method_call (ValaParser* self,
                               ValaSourceLocation* begin,
                               ValaExpression* inner,
                               GError** error)
{
	ValaList* arg_list = NULL;
	ValaList* _tmp0_;
	ValaSourceReference* src = NULL;
	ValaSourceLocation _tmp1_;
	ValaSourceReference* _tmp2_;
	ValaList* init_list = NULL;
	ValaList* _tmp3_;
	gboolean _tmp4_ = FALSE;
	ValaList* _tmp5_;
	gint _tmp6_;
	gint _tmp7_;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (begin != NULL, NULL);
	g_return_val_if_fail (inner != NULL, NULL);
	_tmp0_ = vala_parser_parse_argument_list (self, &_inner_error0_);
	arg_list = _tmp0_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp1_ = *begin;
	_tmp2_ = vala_parser_get_src (self, &_tmp1_);
	src = _tmp2_;
	_tmp3_ = vala_parser_parse_object_initializer (self, &_inner_error0_);
	init_list = _tmp3_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_source_reference_unref0 (src);
			_vala_iterable_unref0 (arg_list);
			return NULL;
		} else {
			_vala_source_reference_unref0 (src);
			_vala_iterable_unref0 (arg_list);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp5_ = init_list;
	_tmp6_ = vala_collection_get_size ((ValaCollection*) _tmp5_);
	_tmp7_ = _tmp6_;
	if (_tmp7_ > 0) {
		_tmp4_ = VALA_IS_MEMBER_ACCESS (inner);
	} else {
		_tmp4_ = FALSE;
	}
	if (_tmp4_) {
		ValaMemberAccess* member = NULL;
		ValaObjectCreationExpression* expr = NULL;
		ValaMemberAccess* _tmp8_;
		ValaSourceReference* _tmp9_;
		ValaObjectCreationExpression* _tmp10_;
		ValaObjectCreationExpression* _tmp11_;
		member = G_TYPE_CHECK_INSTANCE_CAST (inner, VALA_TYPE_MEMBER_ACCESS, ValaMemberAccess);
		_tmp8_ = member;
		_tmp9_ = src;
		_tmp10_ = vala_object_creation_expression_new (_tmp8_, _tmp9_);
		expr = _tmp10_;
		_tmp11_ = expr;
		vala_object_creation_expression_set_struct_creation (_tmp11_, TRUE);
		{
			ValaList* _arg_list = NULL;
			ValaList* _tmp12_;
			gint _arg_size = 0;
			ValaList* _tmp13_;
			gint _tmp14_;
			gint _tmp15_;
			gint _arg_index = 0;
			_tmp12_ = arg_list;
			_arg_list = _tmp12_;
			_tmp13_ = _arg_list;
			_tmp14_ = vala_collection_get_size ((ValaCollection*) _tmp13_);
			_tmp15_ = _tmp14_;
			_arg_size = _tmp15_;
			_arg_index = -1;
			while (TRUE) {
				gint _tmp16_;
				gint _tmp17_;
				ValaExpression* arg = NULL;
				ValaList* _tmp18_;
				gpointer _tmp19_;
				ValaObjectCreationExpression* _tmp20_;
				ValaExpression* _tmp21_;
				_arg_index = _arg_index + 1;
				_tmp16_ = _arg_index;
				_tmp17_ = _arg_size;
				if (!(_tmp16_ < _tmp17_)) {
					break;
				}
				_tmp18_ = _arg_list;
				_tmp19_ = vala_list_get (_tmp18_, _arg_index);
				arg = (ValaExpression*) _tmp19_;
				_tmp20_ = expr;
				_tmp21_ = arg;
				vala_callable_expression_add_argument ((ValaCallableExpression*) _tmp20_, _tmp21_);
				_vala_code_node_unref0 (arg);
			}
		}
		{
			ValaList* _initializer_list = NULL;
			ValaList* _tmp22_;
			gint _initializer_size = 0;
			ValaList* _tmp23_;
			gint _tmp24_;
			gint _tmp25_;
			gint _initializer_index = 0;
			_tmp22_ = init_list;
			_initializer_list = _tmp22_;
			_tmp23_ = _initializer_list;
			_tmp24_ = vala_collection_get_size ((ValaCollection*) _tmp23_);
			_tmp25_ = _tmp24_;
			_initializer_size = _tmp25_;
			_initializer_index = -1;
			while (TRUE) {
				gint _tmp26_;
				gint _tmp27_;
				ValaMemberInitializer* initializer = NULL;
				ValaList* _tmp28_;
				gpointer _tmp29_;
				ValaObjectCreationExpression* _tmp30_;
				ValaMemberInitializer* _tmp31_;
				_initializer_index = _initializer_index + 1;
				_tmp26_ = _initializer_index;
				_tmp27_ = _initializer_size;
				if (!(_tmp26_ < _tmp27_)) {
					break;
				}
				_tmp28_ = _initializer_list;
				_tmp29_ = vala_list_get (_tmp28_, _initializer_index);
				initializer = (ValaMemberInitializer*) _tmp29_;
				_tmp30_ = expr;
				_tmp31_ = initializer;
				vala_object_creation_expression_add_member_initializer (_tmp30_, _tmp31_);
				_vala_code_node_unref0 (initializer);
			}
		}
		result = (ValaExpression*) expr;
		_vala_iterable_unref0 (init_list);
		_vala_source_reference_unref0 (src);
		_vala_iterable_unref0 (arg_list);
		return result;
	} else {
		ValaMethodCall* expr = NULL;
		ValaSourceReference* _tmp32_;
		ValaMethodCall* _tmp33_;
		_tmp32_ = src;
		_tmp33_ = vala_method_call_new (inner, _tmp32_);
		expr = _tmp33_;
		{
			ValaList* _arg_list = NULL;
			ValaList* _tmp34_;
			gint _arg_size = 0;
			ValaList* _tmp35_;
			gint _tmp36_;
			gint _tmp37_;
			gint _arg_index = 0;
			_tmp34_ = arg_list;
			_arg_list = _tmp34_;
			_tmp35_ = _arg_list;
			_tmp36_ = vala_collection_get_size ((ValaCollection*) _tmp35_);
			_tmp37_ = _tmp36_;
			_arg_size = _tmp37_;
			_arg_index = -1;
			while (TRUE) {
				gint _tmp38_;
				gint _tmp39_;
				ValaExpression* arg = NULL;
				ValaList* _tmp40_;
				gpointer _tmp41_;
				ValaMethodCall* _tmp42_;
				ValaExpression* _tmp43_;
				_arg_index = _arg_index + 1;
				_tmp38_ = _arg_index;
				_tmp39_ = _arg_size;
				if (!(_tmp38_ < _tmp39_)) {
					break;
				}
				_tmp40_ = _arg_list;
				_tmp41_ = vala_list_get (_tmp40_, _arg_index);
				arg = (ValaExpression*) _tmp41_;
				_tmp42_ = expr;
				_tmp43_ = arg;
				vala_callable_expression_add_argument ((ValaCallableExpression*) _tmp42_, _tmp43_);
				_vala_code_node_unref0 (arg);
			}
		}
		result = (ValaExpression*) expr;
		_vala_iterable_unref0 (init_list);
		_vala_source_reference_unref0 (src);
		_vala_iterable_unref0 (arg_list);
		return result;
	}
}

static ValaExpression*
vala_parser_parse_element_access (ValaParser* self,
                                  ValaSourceLocation* begin,
                                  ValaExpression* inner,
                                  GError** error)
{
	ValaExpression* stop = NULL;
	ValaList* index_list = NULL;
	gboolean null_safe = FALSE;
	gboolean _tmp11_ = FALSE;
	ValaList* _tmp12_;
	gint _tmp13_;
	gint _tmp14_;
	ValaExpression* _tmp22_;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (begin != NULL, NULL);
	g_return_val_if_fail (inner != NULL, NULL);
	stop = NULL;
	null_safe = vala_parser_previous (self) == VALA_TOKEN_TYPE_INTERR;
	vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_BRACKET, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_iterable_unref0 (index_list);
			_vala_code_node_unref0 (stop);
			return NULL;
		} else {
			_vala_iterable_unref0 (index_list);
			_vala_code_node_unref0 (stop);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	if (vala_parser_current (self) == VALA_TOKEN_TYPE_COLON) {
		GEqualFunc _tmp0_;
		ValaArrayList* _tmp1_;
		ValaList* _tmp2_;
		ValaSourceLocation _tmp3_;
		ValaSourceReference* _tmp4_;
		ValaSourceReference* _tmp5_;
		ValaIntegerLiteral* _tmp6_;
		ValaIntegerLiteral* _tmp7_;
		_tmp0_ = g_direct_equal;
		_tmp1_ = vala_array_list_new (VALA_TYPE_EXPRESSION, (GBoxedCopyFunc) vala_code_node_ref, (GDestroyNotify) vala_code_node_unref, _tmp0_);
		_vala_iterable_unref0 (index_list);
		index_list = (ValaList*) _tmp1_;
		_tmp2_ = index_list;
		_tmp3_ = *begin;
		_tmp4_ = vala_parser_get_src (self, &_tmp3_);
		_tmp5_ = _tmp4_;
		_tmp6_ = vala_integer_literal_new ("0", _tmp5_);
		_tmp7_ = _tmp6_;
		vala_collection_add ((ValaCollection*) _tmp2_, (ValaExpression*) _tmp7_);
		_vala_code_node_unref0 (_tmp7_);
		_vala_source_reference_unref0 (_tmp5_);
	} else {
		ValaList* _tmp8_ = NULL;
		ValaList* _tmp9_;
		ValaList* _tmp10_;
		_tmp9_ = vala_parser_parse_expression_list (self, &_inner_error0_);
		_tmp8_ = _tmp9_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_iterable_unref0 (index_list);
				_vala_code_node_unref0 (stop);
				return NULL;
			} else {
				_vala_iterable_unref0 (index_list);
				_vala_code_node_unref0 (stop);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp10_ = _tmp8_;
		_tmp8_ = NULL;
		_vala_iterable_unref0 (index_list);
		index_list = _tmp10_;
		_vala_iterable_unref0 (_tmp8_);
	}
	_tmp12_ = index_list;
	_tmp13_ = vala_collection_get_size ((ValaCollection*) _tmp12_);
	_tmp14_ = _tmp13_;
	if (_tmp14_ == 1) {
		_tmp11_ = vala_parser_accept (self, VALA_TOKEN_TYPE_COLON);
	} else {
		_tmp11_ = FALSE;
	}
	if (_tmp11_) {
		if (vala_parser_current (self) == VALA_TOKEN_TYPE_CLOSE_BRACKET) {
			ValaSourceLocation _tmp15_;
			ValaSourceReference* _tmp16_;
			ValaSourceReference* _tmp17_;
			ValaMemberAccess* _tmp18_;
			_tmp15_ = *begin;
			_tmp16_ = vala_parser_get_src (self, &_tmp15_);
			_tmp17_ = _tmp16_;
			_tmp18_ = vala_member_access_new (inner, "length", _tmp17_);
			_vala_code_node_unref0 (stop);
			stop = (ValaExpression*) _tmp18_;
			_vala_source_reference_unref0 (_tmp17_);
		} else {
			ValaExpression* _tmp19_ = NULL;
			ValaExpression* _tmp20_;
			ValaExpression* _tmp21_;
			_tmp20_ = vala_parser_parse_expression (self, &_inner_error0_);
			_tmp19_ = _tmp20_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_iterable_unref0 (index_list);
					_vala_code_node_unref0 (stop);
					return NULL;
				} else {
					_vala_iterable_unref0 (index_list);
					_vala_code_node_unref0 (stop);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp21_ = _tmp19_;
			_tmp19_ = NULL;
			_vala_code_node_unref0 (stop);
			stop = _tmp21_;
			_vala_code_node_unref0 (_tmp19_);
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_BRACKET, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_iterable_unref0 (index_list);
			_vala_code_node_unref0 (stop);
			return NULL;
		} else {
			_vala_iterable_unref0 (index_list);
			_vala_code_node_unref0 (stop);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp22_ = stop;
	if (_tmp22_ == NULL) {
		ValaElementAccess* expr = NULL;
		ValaSourceLocation _tmp23_;
		ValaSourceReference* _tmp24_;
		ValaSourceReference* _tmp25_;
		ValaElementAccess* _tmp26_;
		ValaElementAccess* _tmp27_;
		ValaElementAccess* _tmp28_;
		_tmp23_ = *begin;
		_tmp24_ = vala_parser_get_src (self, &_tmp23_);
		_tmp25_ = _tmp24_;
		_tmp26_ = vala_element_access_new (inner, _tmp25_);
		_tmp27_ = _tmp26_;
		_vala_source_reference_unref0 (_tmp25_);
		expr = _tmp27_;
		_tmp28_ = expr;
		vala_element_access_set_null_safe_access (_tmp28_, null_safe);
		{
			ValaList* _index_list = NULL;
			ValaList* _tmp29_;
			gint _index_size = 0;
			ValaList* _tmp30_;
			gint _tmp31_;
			gint _tmp32_;
			gint _index_index = 0;
			_tmp29_ = index_list;
			_index_list = _tmp29_;
			_tmp30_ = _index_list;
			_tmp31_ = vala_collection_get_size ((ValaCollection*) _tmp30_);
			_tmp32_ = _tmp31_;
			_index_size = _tmp32_;
			_index_index = -1;
			while (TRUE) {
				gint _tmp33_;
				gint _tmp34_;
				ValaExpression* index = NULL;
				ValaList* _tmp35_;
				gpointer _tmp36_;
				ValaElementAccess* _tmp37_;
				ValaExpression* _tmp38_;
				_index_index = _index_index + 1;
				_tmp33_ = _index_index;
				_tmp34_ = _index_size;
				if (!(_tmp33_ < _tmp34_)) {
					break;
				}
				_tmp35_ = _index_list;
				_tmp36_ = vala_list_get (_tmp35_, _index_index);
				index = (ValaExpression*) _tmp36_;
				_tmp37_ = expr;
				_tmp38_ = index;
				vala_element_access_append_index (_tmp37_, _tmp38_);
				_vala_code_node_unref0 (index);
			}
		}
		result = (ValaExpression*) expr;
		_vala_iterable_unref0 (index_list);
		_vala_code_node_unref0 (stop);
		return result;
	} else {
		ValaSliceExpression* expr = NULL;
		ValaList* _tmp39_;
		gpointer _tmp40_;
		ValaExpression* _tmp41_;
		ValaExpression* _tmp42_;
		ValaSourceLocation _tmp43_;
		ValaSourceReference* _tmp44_;
		ValaSourceReference* _tmp45_;
		ValaSliceExpression* _tmp46_;
		ValaSliceExpression* _tmp47_;
		ValaSliceExpression* _tmp48_;
		_tmp39_ = index_list;
		_tmp40_ = vala_list_get (_tmp39_, 0);
		_tmp41_ = (ValaExpression*) _tmp40_;
		_tmp42_ = stop;
		_tmp43_ = *begin;
		_tmp44_ = vala_parser_get_src (self, &_tmp43_);
		_tmp45_ = _tmp44_;
		_tmp46_ = vala_slice_expression_new (inner, _tmp41_, _tmp42_, _tmp45_);
		_tmp47_ = _tmp46_;
		_vala_source_reference_unref0 (_tmp45_);
		_vala_code_node_unref0 (_tmp41_);
		expr = _tmp47_;
		_tmp48_ = expr;
		vala_slice_expression_set_null_safe_access (_tmp48_, null_safe);
		result = (ValaExpression*) expr;
		_vala_iterable_unref0 (index_list);
		_vala_code_node_unref0 (stop);
		return result;
	}
}

static ValaList*
vala_parser_parse_expression_list (ValaParser* self,
                                   GError** error)
{
	ValaArrayList* list = NULL;
	GEqualFunc _tmp0_;
	ValaArrayList* _tmp1_;
	GError* _inner_error0_ = NULL;
	ValaList* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = g_direct_equal;
	_tmp1_ = vala_array_list_new (VALA_TYPE_EXPRESSION, (GBoxedCopyFunc) vala_code_node_ref, (GDestroyNotify) vala_code_node_unref, _tmp0_);
	list = _tmp1_;
	{
		gboolean _tmp2_ = FALSE;
		_tmp2_ = TRUE;
		while (TRUE) {
			ValaExpression* _tmp3_ = NULL;
			ValaExpression* _tmp4_;
			ValaArrayList* _tmp5_;
			if (!_tmp2_) {
				if (!vala_parser_accept (self, VALA_TOKEN_TYPE_COMMA)) {
					break;
				}
			}
			_tmp2_ = FALSE;
			_tmp4_ = vala_parser_parse_expression (self, &_inner_error0_);
			_tmp3_ = _tmp4_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_iterable_unref0 (list);
					return NULL;
				} else {
					_vala_iterable_unref0 (list);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp5_ = list;
			vala_collection_add ((ValaCollection*) _tmp5_, _tmp3_);
			_vala_code_node_unref0 (_tmp3_);
		}
	}
	result = (ValaList*) list;
	return result;
}

static ValaExpression*
vala_parser_parse_this_access (ValaParser* self,
                               GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaSourceLocation _tmp1_;
	ValaSourceReference* _tmp2_;
	ValaSourceReference* _tmp3_;
	ValaMemberAccess* _tmp4_;
	ValaExpression* _tmp5_;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_THIS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp1_ = begin;
	_tmp2_ = vala_parser_get_src (self, &_tmp1_);
	_tmp3_ = _tmp2_;
	_tmp4_ = vala_member_access_new (NULL, "this", _tmp3_);
	_tmp5_ = (ValaExpression*) _tmp4_;
	_vala_source_reference_unref0 (_tmp3_);
	result = _tmp5_;
	return result;
}

static ValaExpression*
vala_parser_parse_base_access (ValaParser* self,
                               GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaSourceLocation _tmp1_;
	ValaSourceReference* _tmp2_;
	ValaSourceReference* _tmp3_;
	ValaBaseAccess* _tmp4_;
	ValaExpression* _tmp5_;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_BASE, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp1_ = begin;
	_tmp2_ = vala_parser_get_src (self, &_tmp1_);
	_tmp3_ = _tmp2_;
	_tmp4_ = vala_base_access_new (_tmp3_);
	_tmp5_ = (ValaExpression*) _tmp4_;
	_vala_source_reference_unref0 (_tmp3_);
	result = _tmp5_;
	return result;
}

static ValaExpression*
vala_parser_parse_post_increment_expression (ValaParser* self,
                                             ValaSourceLocation* begin,
                                             ValaExpression* inner,
                                             GError** error)
{
	ValaSourceLocation _tmp0_;
	ValaSourceReference* _tmp1_;
	ValaSourceReference* _tmp2_;
	ValaPostfixExpression* _tmp3_;
	ValaExpression* _tmp4_;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (begin != NULL, NULL);
	g_return_val_if_fail (inner != NULL, NULL);
	vala_parser_expect (self, VALA_TOKEN_TYPE_OP_INC, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp0_ = *begin;
	_tmp1_ = vala_parser_get_src (self, &_tmp0_);
	_tmp2_ = _tmp1_;
	_tmp3_ = vala_postfix_expression_new (inner, TRUE, _tmp2_);
	_tmp4_ = (ValaExpression*) _tmp3_;
	_vala_source_reference_unref0 (_tmp2_);
	result = _tmp4_;
	return result;
}

static ValaExpression*
vala_parser_parse_post_decrement_expression (ValaParser* self,
                                             ValaSourceLocation* begin,
                                             ValaExpression* inner,
                                             GError** error)
{
	ValaSourceLocation _tmp0_;
	ValaSourceReference* _tmp1_;
	ValaSourceReference* _tmp2_;
	ValaPostfixExpression* _tmp3_;
	ValaExpression* _tmp4_;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (begin != NULL, NULL);
	g_return_val_if_fail (inner != NULL, NULL);
	vala_parser_expect (self, VALA_TOKEN_TYPE_OP_DEC, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp0_ = *begin;
	_tmp1_ = vala_parser_get_src (self, &_tmp0_);
	_tmp2_ = _tmp1_;
	_tmp3_ = vala_postfix_expression_new (inner, FALSE, _tmp2_);
	_tmp4_ = (ValaExpression*) _tmp3_;
	_vala_source_reference_unref0 (_tmp2_);
	result = _tmp4_;
	return result;
}

static ValaExpression*
vala_parser_parse_object_or_array_creation_expression (ValaParser* self,
                                                       GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaMemberAccess* member = NULL;
	ValaMemberAccess* _tmp5_;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_NEW, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	if (vala_parser_is_inner_array_type (self)) {
		ValaSourceLocation _tmp1_;
		ValaExpression* _tmp2_ = NULL;
		ValaExpression* _tmp3_;
		ValaExpression* _tmp4_;
		_tmp1_ = begin;
		vala_parser_rollback (self, &_tmp1_);
		_tmp3_ = vala_parser_parse_array_creation_expression (self, &_inner_error0_);
		_tmp2_ = _tmp3_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				return NULL;
			} else {
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp4_ = _tmp2_;
		_tmp2_ = NULL;
		result = _tmp4_;
		_vala_code_node_unref0 (_tmp2_);
		return result;
	}
	_tmp5_ = vala_parser_parse_member_name (self, NULL, &_inner_error0_);
	member = _tmp5_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	if (vala_parser_current (self) == VALA_TOKEN_TYPE_OPEN_PARENS) {
		ValaExpression* expr = NULL;
		ValaSourceLocation _tmp6_;
		ValaMemberAccess* _tmp7_;
		ValaExpression* _tmp8_;
		_tmp6_ = begin;
		_tmp7_ = member;
		_tmp8_ = vala_parser_parse_object_creation_expression (self, &_tmp6_, _tmp7_, &_inner_error0_);
		expr = _tmp8_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (member);
				return NULL;
			} else {
				_vala_code_node_unref0 (member);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		result = expr;
		_vala_code_node_unref0 (member);
		return result;
	} else {
		gboolean is_pointer_type = FALSE;
		is_pointer_type = FALSE;
		while (TRUE) {
			if (!vala_parser_accept (self, VALA_TOKEN_TYPE_STAR)) {
				break;
			}
			is_pointer_type = TRUE;
		}
		if (!is_pointer_type) {
			vala_parser_accept (self, VALA_TOKEN_TYPE_INTERR);
		}
		if (vala_parser_accept (self, VALA_TOKEN_TYPE_OPEN_BRACKET)) {
			ValaSourceLocation _tmp9_;
			ValaExpression* expr = NULL;
			ValaExpression* _tmp10_;
			_tmp9_ = begin;
			vala_parser_rollback (self, &_tmp9_);
			_tmp10_ = vala_parser_parse_array_creation_expression (self, &_inner_error0_);
			expr = _tmp10_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (member);
					return NULL;
				} else {
					_vala_code_node_unref0 (member);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			result = expr;
			_vala_code_node_unref0 (member);
			return result;
		} else {
			GError* _tmp11_;
			_tmp11_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "expected ( or [");
			_inner_error0_ = _tmp11_;
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (member);
				return NULL;
			} else {
				_vala_code_node_unref0 (member);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
	}
}

static ValaExpression*
vala_parser_parse_object_creation_expression (ValaParser* self,
                                              ValaSourceLocation* begin,
                                              ValaMemberAccess* member,
                                              GError** error)
{
	ValaList* arg_list = NULL;
	ValaList* _tmp0_;
	ValaSourceReference* src = NULL;
	ValaSourceLocation _tmp1_;
	ValaSourceReference* _tmp2_;
	ValaList* init_list = NULL;
	ValaList* _tmp3_;
	ValaObjectCreationExpression* expr = NULL;
	ValaSourceReference* _tmp4_;
	ValaObjectCreationExpression* _tmp5_;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (begin != NULL, NULL);
	g_return_val_if_fail (member != NULL, NULL);
	_tmp0_ = vala_parser_parse_argument_list (self, &_inner_error0_);
	arg_list = _tmp0_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp1_ = *begin;
	_tmp2_ = vala_parser_get_src (self, &_tmp1_);
	src = _tmp2_;
	_tmp3_ = vala_parser_parse_object_initializer (self, &_inner_error0_);
	init_list = _tmp3_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_source_reference_unref0 (src);
			_vala_iterable_unref0 (arg_list);
			return NULL;
		} else {
			_vala_source_reference_unref0 (src);
			_vala_iterable_unref0 (arg_list);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp4_ = src;
	_tmp5_ = vala_object_creation_expression_new (member, _tmp4_);
	expr = _tmp5_;
	{
		ValaList* _arg_list = NULL;
		ValaList* _tmp6_;
		gint _arg_size = 0;
		ValaList* _tmp7_;
		gint _tmp8_;
		gint _tmp9_;
		gint _arg_index = 0;
		_tmp6_ = arg_list;
		_arg_list = _tmp6_;
		_tmp7_ = _arg_list;
		_tmp8_ = vala_collection_get_size ((ValaCollection*) _tmp7_);
		_tmp9_ = _tmp8_;
		_arg_size = _tmp9_;
		_arg_index = -1;
		while (TRUE) {
			gint _tmp10_;
			gint _tmp11_;
			ValaExpression* arg = NULL;
			ValaList* _tmp12_;
			gpointer _tmp13_;
			ValaObjectCreationExpression* _tmp14_;
			ValaExpression* _tmp15_;
			_arg_index = _arg_index + 1;
			_tmp10_ = _arg_index;
			_tmp11_ = _arg_size;
			if (!(_tmp10_ < _tmp11_)) {
				break;
			}
			_tmp12_ = _arg_list;
			_tmp13_ = vala_list_get (_tmp12_, _arg_index);
			arg = (ValaExpression*) _tmp13_;
			_tmp14_ = expr;
			_tmp15_ = arg;
			vala_callable_expression_add_argument ((ValaCallableExpression*) _tmp14_, _tmp15_);
			_vala_code_node_unref0 (arg);
		}
	}
	{
		ValaList* _initializer_list = NULL;
		ValaList* _tmp16_;
		gint _initializer_size = 0;
		ValaList* _tmp17_;
		gint _tmp18_;
		gint _tmp19_;
		gint _initializer_index = 0;
		_tmp16_ = init_list;
		_initializer_list = _tmp16_;
		_tmp17_ = _initializer_list;
		_tmp18_ = vala_collection_get_size ((ValaCollection*) _tmp17_);
		_tmp19_ = _tmp18_;
		_initializer_size = _tmp19_;
		_initializer_index = -1;
		while (TRUE) {
			gint _tmp20_;
			gint _tmp21_;
			ValaMemberInitializer* initializer = NULL;
			ValaList* _tmp22_;
			gpointer _tmp23_;
			ValaObjectCreationExpression* _tmp24_;
			ValaMemberInitializer* _tmp25_;
			_initializer_index = _initializer_index + 1;
			_tmp20_ = _initializer_index;
			_tmp21_ = _initializer_size;
			if (!(_tmp20_ < _tmp21_)) {
				break;
			}
			_tmp22_ = _initializer_list;
			_tmp23_ = vala_list_get (_tmp22_, _initializer_index);
			initializer = (ValaMemberInitializer*) _tmp23_;
			_tmp24_ = expr;
			_tmp25_ = initializer;
			vala_object_creation_expression_add_member_initializer (_tmp24_, _tmp25_);
			_vala_code_node_unref0 (initializer);
		}
	}
	result = (ValaExpression*) expr;
	_vala_iterable_unref0 (init_list);
	_vala_source_reference_unref0 (src);
	_vala_iterable_unref0 (arg_list);
	return result;
}

static ValaExpression*
vala_parser_parse_array_creation_expression (ValaParser* self,
                                             GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	gboolean inner_array_type = FALSE;
	ValaMemberAccess* member = NULL;
	ValaMemberAccess* _tmp1_;
	ValaDataType* element_type = NULL;
	ValaMemberAccess* _tmp2_;
	ValaUnresolvedType* _tmp3_;
	gboolean is_pointer_type = FALSE;
	gboolean size_specified = FALSE;
	ValaList* size_specifier_list = NULL;
	gboolean first = FALSE;
	ValaDataType* array_length_type = NULL;
	ValaSourceReference* src = NULL;
	ValaSourceLocation _tmp40_;
	ValaSourceReference* _tmp41_;
	ValaInitializerList* initializer = NULL;
	ValaArrayCreationExpression* expr = NULL;
	ValaDataType* _tmp45_;
	ValaList* _tmp46_;
	gint _tmp47_;
	gint _tmp48_;
	ValaInitializerList* _tmp49_;
	ValaSourceReference* _tmp50_;
	ValaArrayCreationExpression* _tmp51_;
	ValaDataType* _tmp52_;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_NEW, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	inner_array_type = vala_parser_is_inner_array_type (self);
	if (inner_array_type) {
		vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_PARENS, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				return NULL;
			} else {
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		vala_parser_expect (self, VALA_TOKEN_TYPE_UNOWNED, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				return NULL;
			} else {
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
	}
	_tmp1_ = vala_parser_parse_member_name (self, NULL, &_inner_error0_);
	member = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp2_ = member;
	_tmp3_ = vala_unresolved_type_new_from_expression (_tmp2_);
	element_type = (ValaDataType*) _tmp3_;
	is_pointer_type = FALSE;
	while (TRUE) {
		ValaDataType* _tmp4_;
		ValaSourceLocation _tmp5_;
		ValaSourceReference* _tmp6_;
		ValaSourceReference* _tmp7_;
		ValaPointerType* _tmp8_;
		if (!vala_parser_accept (self, VALA_TOKEN_TYPE_STAR)) {
			break;
		}
		_tmp4_ = element_type;
		_tmp5_ = begin;
		_tmp6_ = vala_parser_get_src (self, &_tmp5_);
		_tmp7_ = _tmp6_;
		_tmp8_ = vala_pointer_type_new (_tmp4_, _tmp7_);
		_vala_code_node_unref0 (element_type);
		element_type = (ValaDataType*) _tmp8_;
		_vala_source_reference_unref0 (_tmp7_);
		is_pointer_type = TRUE;
	}
	if (!is_pointer_type) {
		if (vala_parser_accept (self, VALA_TOKEN_TYPE_INTERR)) {
			ValaDataType* _tmp9_;
			_tmp9_ = element_type;
			vala_data_type_set_nullable (_tmp9_, TRUE);
		}
	}
	if (inner_array_type) {
		ValaDataType* _tmp10_;
		vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_PARENS, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (element_type);
				_vala_code_node_unref0 (member);
				return NULL;
			} else {
				_vala_code_node_unref0 (element_type);
				_vala_code_node_unref0 (member);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp10_ = element_type;
		vala_data_type_set_value_owned (_tmp10_, FALSE);
	} else {
		ValaDataType* _tmp11_;
		_tmp11_ = element_type;
		vala_data_type_set_value_owned (_tmp11_, TRUE);
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_BRACKET, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (element_type);
			_vala_code_node_unref0 (member);
			return NULL;
		} else {
			_vala_code_node_unref0 (element_type);
			_vala_code_node_unref0 (member);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	size_specified = FALSE;
	size_specifier_list = NULL;
	first = TRUE;
	array_length_type = NULL;
	{
		gboolean _tmp12_ = FALSE;
		_tmp12_ = TRUE;
		while (TRUE) {
			GEqualFunc _tmp27_;
			ValaArrayList* _tmp28_;
			if (!_tmp12_) {
				if (!vala_parser_accept (self, VALA_TOKEN_TYPE_OPEN_BRACKET)) {
					break;
				}
			}
			_tmp12_ = FALSE;
			if (!first) {
				ValaDataType* _tmp14_;
				ValaList* _tmp15_;
				gint _tmp16_;
				gint _tmp17_;
				ValaDataType* _tmp18_;
				ValaSourceReference* _tmp19_;
				ValaSourceReference* _tmp20_;
				ValaArrayType* _tmp21_;
				ValaDataType* _tmp22_;
				if (size_specified) {
					GError* _tmp13_;
					_tmp13_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "size of inner arrays must not be specified in array creation expressio" \
"n");
					_inner_error0_ = _tmp13_;
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_code_node_unref0 (array_length_type);
						_vala_iterable_unref0 (size_specifier_list);
						_vala_code_node_unref0 (element_type);
						_vala_code_node_unref0 (member);
						return NULL;
					} else {
						_vala_code_node_unref0 (array_length_type);
						_vala_iterable_unref0 (size_specifier_list);
						_vala_code_node_unref0 (element_type);
						_vala_code_node_unref0 (member);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return NULL;
					}
				}
				_tmp14_ = element_type;
				_tmp15_ = size_specifier_list;
				_tmp16_ = vala_collection_get_size ((ValaCollection*) _tmp15_);
				_tmp17_ = _tmp16_;
				_tmp18_ = element_type;
				_tmp19_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp18_);
				_tmp20_ = _tmp19_;
				_tmp21_ = vala_array_type_new (_tmp14_, _tmp17_, _tmp20_);
				_vala_code_node_unref0 (element_type);
				element_type = (ValaDataType*) _tmp21_;
				_tmp22_ = array_length_type;
				if (_tmp22_ != NULL) {
					ValaDataType* _tmp23_;
					ValaDataType* _tmp24_;
					ValaDataType* _tmp25_;
					ValaDataType* _tmp26_;
					_tmp23_ = element_type;
					_tmp24_ = array_length_type;
					_tmp25_ = vala_data_type_copy (_tmp24_);
					_tmp26_ = _tmp25_;
					vala_array_type_set_length_type (G_TYPE_CHECK_INSTANCE_CAST (_tmp23_, VALA_TYPE_ARRAY_TYPE, ValaArrayType), _tmp26_);
					_vala_code_node_unref0 (_tmp26_);
				}
			} else {
				first = FALSE;
			}
			_tmp27_ = g_direct_equal;
			_tmp28_ = vala_array_list_new (VALA_TYPE_EXPRESSION, (GBoxedCopyFunc) vala_code_node_ref, (GDestroyNotify) vala_code_node_unref, _tmp27_);
			_vala_iterable_unref0 (size_specifier_list);
			size_specifier_list = (ValaList*) _tmp28_;
			{
				gboolean _tmp29_ = FALSE;
				_tmp29_ = TRUE;
				while (TRUE) {
					ValaExpression* size = NULL;
					gboolean _tmp30_ = FALSE;
					gboolean _tmp31_ = FALSE;
					ValaList* _tmp35_;
					ValaExpression* _tmp36_;
					if (!_tmp29_) {
						if (!vala_parser_accept (self, VALA_TOKEN_TYPE_COMMA)) {
							break;
						}
					}
					_tmp29_ = FALSE;
					size = NULL;
					if (vala_parser_current (self) != VALA_TOKEN_TYPE_CLOSE_BRACKET) {
						_tmp31_ = vala_parser_current (self) != VALA_TOKEN_TYPE_COMMA;
					} else {
						_tmp31_ = FALSE;
					}
					if (_tmp31_) {
						_tmp30_ = vala_parser_current (self) != VALA_TOKEN_TYPE_COLON;
					} else {
						_tmp30_ = FALSE;
					}
					if (_tmp30_) {
						ValaExpression* _tmp32_ = NULL;
						ValaExpression* _tmp33_;
						ValaExpression* _tmp34_;
						_tmp33_ = vala_parser_parse_expression (self, &_inner_error0_);
						_tmp32_ = _tmp33_;
						if (G_UNLIKELY (_inner_error0_ != NULL)) {
							if (_inner_error0_->domain == VALA_PARSE_ERROR) {
								g_propagate_error (error, _inner_error0_);
								_vala_code_node_unref0 (size);
								_vala_code_node_unref0 (array_length_type);
								_vala_iterable_unref0 (size_specifier_list);
								_vala_code_node_unref0 (element_type);
								_vala_code_node_unref0 (member);
								return NULL;
							} else {
								_vala_code_node_unref0 (size);
								_vala_code_node_unref0 (array_length_type);
								_vala_iterable_unref0 (size_specifier_list);
								_vala_code_node_unref0 (element_type);
								_vala_code_node_unref0 (member);
								g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
								g_clear_error (&_inner_error0_);
								return NULL;
							}
						}
						_tmp34_ = _tmp32_;
						_tmp32_ = NULL;
						_vala_code_node_unref0 (size);
						size = _tmp34_;
						size_specified = TRUE;
						_vala_code_node_unref0 (_tmp32_);
					}
					_tmp35_ = size_specifier_list;
					_tmp36_ = size;
					vala_collection_add ((ValaCollection*) _tmp35_, _tmp36_);
					_vala_code_node_unref0 (size);
				}
			}
			if (vala_parser_accept (self, VALA_TOKEN_TYPE_COLON)) {
				ValaDataType* _tmp37_ = NULL;
				ValaDataType* _tmp38_;
				ValaDataType* _tmp39_;
				_tmp38_ = vala_parser_parse_type (self, TRUE, FALSE, FALSE, &_inner_error0_);
				_tmp37_ = _tmp38_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_code_node_unref0 (array_length_type);
						_vala_iterable_unref0 (size_specifier_list);
						_vala_code_node_unref0 (element_type);
						_vala_code_node_unref0 (member);
						return NULL;
					} else {
						_vala_code_node_unref0 (array_length_type);
						_vala_iterable_unref0 (size_specifier_list);
						_vala_code_node_unref0 (element_type);
						_vala_code_node_unref0 (member);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return NULL;
					}
				}
				_tmp39_ = _tmp37_;
				_tmp37_ = NULL;
				_vala_code_node_unref0 (array_length_type);
				array_length_type = _tmp39_;
				_vala_code_node_unref0 (_tmp37_);
			}
			vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_BRACKET, &_inner_error0_);
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (array_length_type);
					_vala_iterable_unref0 (size_specifier_list);
					_vala_code_node_unref0 (element_type);
					_vala_code_node_unref0 (member);
					return NULL;
				} else {
					_vala_code_node_unref0 (array_length_type);
					_vala_iterable_unref0 (size_specifier_list);
					_vala_code_node_unref0 (element_type);
					_vala_code_node_unref0 (member);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
		}
	}
	_tmp40_ = begin;
	_tmp41_ = vala_parser_get_src (self, &_tmp40_);
	src = _tmp41_;
	initializer = NULL;
	if (vala_parser_current (self) == VALA_TOKEN_TYPE_OPEN_BRACE) {
		ValaInitializerList* _tmp42_ = NULL;
		ValaInitializerList* _tmp43_;
		ValaInitializerList* _tmp44_;
		_tmp43_ = vala_parser_parse_initializer (self, &_inner_error0_);
		_tmp42_ = _tmp43_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (initializer);
				_vala_source_reference_unref0 (src);
				_vala_code_node_unref0 (array_length_type);
				_vala_iterable_unref0 (size_specifier_list);
				_vala_code_node_unref0 (element_type);
				_vala_code_node_unref0 (member);
				return NULL;
			} else {
				_vala_code_node_unref0 (initializer);
				_vala_source_reference_unref0 (src);
				_vala_code_node_unref0 (array_length_type);
				_vala_iterable_unref0 (size_specifier_list);
				_vala_code_node_unref0 (element_type);
				_vala_code_node_unref0 (member);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp44_ = _tmp42_;
		_tmp42_ = NULL;
		_vala_code_node_unref0 (initializer);
		initializer = _tmp44_;
		_vala_code_node_unref0 (_tmp42_);
	}
	_tmp45_ = element_type;
	_tmp46_ = size_specifier_list;
	_tmp47_ = vala_collection_get_size ((ValaCollection*) _tmp46_);
	_tmp48_ = _tmp47_;
	_tmp49_ = initializer;
	_tmp50_ = src;
	_tmp51_ = vala_array_creation_expression_new (_tmp45_, _tmp48_, _tmp49_, _tmp50_);
	expr = _tmp51_;
	_tmp52_ = array_length_type;
	if (_tmp52_ != NULL) {
		ValaArrayCreationExpression* _tmp53_;
		ValaDataType* _tmp54_;
		ValaDataType* _tmp55_;
		ValaDataType* _tmp56_;
		_tmp53_ = expr;
		_tmp54_ = array_length_type;
		_tmp55_ = vala_data_type_copy (_tmp54_);
		_tmp56_ = _tmp55_;
		vala_array_creation_expression_set_length_type (_tmp53_, _tmp56_);
		_vala_code_node_unref0 (_tmp56_);
	}
	if (size_specified) {
		{
			ValaList* _size_list = NULL;
			ValaList* _tmp57_;
			gint _size_size = 0;
			ValaList* _tmp58_;
			gint _tmp59_;
			gint _tmp60_;
			gint _size_index = 0;
			_tmp57_ = size_specifier_list;
			_size_list = _tmp57_;
			_tmp58_ = _size_list;
			_tmp59_ = vala_collection_get_size ((ValaCollection*) _tmp58_);
			_tmp60_ = _tmp59_;
			_size_size = _tmp60_;
			_size_index = -1;
			while (TRUE) {
				gint _tmp61_;
				gint _tmp62_;
				ValaExpression* size = NULL;
				ValaList* _tmp63_;
				gpointer _tmp64_;
				ValaArrayCreationExpression* _tmp65_;
				ValaExpression* _tmp66_;
				_size_index = _size_index + 1;
				_tmp61_ = _size_index;
				_tmp62_ = _size_size;
				if (!(_tmp61_ < _tmp62_)) {
					break;
				}
				_tmp63_ = _size_list;
				_tmp64_ = vala_list_get (_tmp63_, _size_index);
				size = (ValaExpression*) _tmp64_;
				_tmp65_ = expr;
				_tmp66_ = size;
				vala_array_creation_expression_append_size (_tmp65_, _tmp66_);
				_vala_code_node_unref0 (size);
			}
		}
	} else {
		ValaInitializerList* _tmp67_;
		_tmp67_ = initializer;
		if (_tmp67_ == NULL) {
			ValaSourceReference* _tmp68_;
			GError* _tmp69_;
			_tmp68_ = src;
			vala_report_warning (_tmp68_, "possibly missing array size");
			_tmp69_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "expected array initializer list");
			_inner_error0_ = _tmp69_;
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (expr);
				_vala_code_node_unref0 (initializer);
				_vala_source_reference_unref0 (src);
				_vala_code_node_unref0 (array_length_type);
				_vala_iterable_unref0 (size_specifier_list);
				_vala_code_node_unref0 (element_type);
				_vala_code_node_unref0 (member);
				return NULL;
			} else {
				_vala_code_node_unref0 (expr);
				_vala_code_node_unref0 (initializer);
				_vala_source_reference_unref0 (src);
				_vala_code_node_unref0 (array_length_type);
				_vala_iterable_unref0 (size_specifier_list);
				_vala_code_node_unref0 (element_type);
				_vala_code_node_unref0 (member);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
	}
	result = (ValaExpression*) expr;
	_vala_code_node_unref0 (initializer);
	_vala_source_reference_unref0 (src);
	_vala_code_node_unref0 (array_length_type);
	_vala_iterable_unref0 (size_specifier_list);
	_vala_code_node_unref0 (element_type);
	_vala_code_node_unref0 (member);
	return result;
}

static ValaList*
vala_parser_parse_object_initializer (ValaParser* self,
                                      GError** error)
{
	ValaArrayList* list = NULL;
	GEqualFunc _tmp0_;
	ValaArrayList* _tmp1_;
	GError* _inner_error0_ = NULL;
	ValaList* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = g_direct_equal;
	_tmp1_ = vala_array_list_new (VALA_TYPE_MEMBER_INITIALIZER, (GBoxedCopyFunc) vala_code_node_ref, (GDestroyNotify) vala_code_node_unref, _tmp0_);
	list = _tmp1_;
	if (vala_parser_accept (self, VALA_TOKEN_TYPE_OPEN_BRACE)) {
		while (TRUE) {
			ValaMemberInitializer* _tmp2_ = NULL;
			ValaMemberInitializer* _tmp3_;
			ValaArrayList* _tmp4_;
			if (!(vala_parser_current (self) != VALA_TOKEN_TYPE_CLOSE_BRACE)) {
				break;
			}
			_tmp3_ = vala_parser_parse_member_initializer (self, FALSE, &_inner_error0_);
			_tmp2_ = _tmp3_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_iterable_unref0 (list);
					return NULL;
				} else {
					_vala_iterable_unref0 (list);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp4_ = list;
			vala_collection_add ((ValaCollection*) _tmp4_, _tmp2_);
			if (!vala_parser_accept (self, VALA_TOKEN_TYPE_COMMA)) {
				_vala_code_node_unref0 (_tmp2_);
				break;
			}
			_vala_code_node_unref0 (_tmp2_);
		}
		vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_BRACE, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_iterable_unref0 (list);
				return NULL;
			} else {
				_vala_iterable_unref0 (list);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
	}
	result = (ValaList*) list;
	return result;
}

static ValaMemberInitializer*
vala_parser_parse_member_initializer (ValaParser* self,
                                      gboolean chained,
                                      GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	gchar* id = NULL;
	gchar* _tmp1_;
	ValaSourceLocation inner = {0};
	ValaSourceLocation _tmp2_ = {0};
	ValaExpression* expr = NULL;
	const gchar* _tmp10_;
	ValaExpression* _tmp11_;
	ValaSourceLocation _tmp12_;
	ValaSourceReference* _tmp13_;
	ValaSourceReference* _tmp14_;
	ValaMemberInitializer* _tmp15_;
	ValaMemberInitializer* _tmp16_;
	GError* _inner_error0_ = NULL;
	ValaMemberInitializer* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	_tmp1_ = vala_parser_parse_identifier (self, chained, &_inner_error0_);
	id = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_ASSIGN, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_g_free0 (id);
			return NULL;
		} else {
			_g_free0 (id);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	vala_parser_get_location (self, &_tmp2_);
	inner = _tmp2_;
	{
		ValaMemberInitializer* _tmp3_ = NULL;
		ValaMemberInitializer* _tmp4_;
		ValaMemberInitializer* _tmp5_;
		_tmp4_ = vala_parser_parse_member_initializer (self, TRUE, &_inner_error0_);
		_tmp3_ = _tmp4_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			goto __catch0_g_error;
		}
		_tmp5_ = _tmp3_;
		_tmp3_ = NULL;
		_vala_code_node_unref0 (expr);
		expr = (ValaExpression*) _tmp5_;
		_vala_code_node_unref0 (_tmp3_);
	}
	goto __finally0;
	__catch0_g_error:
	{
		ValaSourceLocation _tmp6_;
		ValaExpression* _tmp7_ = NULL;
		ValaExpression* _tmp8_;
		ValaExpression* _tmp9_;
		g_clear_error (&_inner_error0_);
		_tmp6_ = inner;
		vala_parser_rollback (self, &_tmp6_);
		_tmp8_ = vala_parser_parse_expression (self, &_inner_error0_);
		_tmp7_ = _tmp8_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			goto __finally0;
		}
		_tmp9_ = _tmp7_;
		_tmp7_ = NULL;
		_vala_code_node_unref0 (expr);
		expr = _tmp9_;
		_vala_code_node_unref0 (_tmp7_);
	}
	__finally0:
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (expr);
			_g_free0 (id);
			return NULL;
		} else {
			_vala_code_node_unref0 (expr);
			_g_free0 (id);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp10_ = id;
	_tmp11_ = expr;
	_tmp12_ = begin;
	_tmp13_ = vala_parser_get_src (self, &_tmp12_);
	_tmp14_ = _tmp13_;
	_tmp15_ = vala_member_initializer_new (_tmp10_, _tmp11_, _tmp14_);
	_tmp16_ = _tmp15_;
	_vala_source_reference_unref0 (_tmp14_);
	result = _tmp16_;
	_vala_code_node_unref0 (expr);
	_g_free0 (id);
	return result;
}

static ValaExpression*
vala_parser_parse_yield_expression (ValaParser* self,
                                    GError** error)
{
	ValaExpression* expr = NULL;
	ValaExpression* _tmp0_;
	ValaMethodCall* call = NULL;
	ValaExpression* _tmp1_;
	ValaObjectCreationExpression* object_creation = NULL;
	ValaExpression* _tmp2_;
	gboolean _tmp3_ = FALSE;
	ValaMethodCall* _tmp4_;
	ValaMethodCall* _tmp10_;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_expect (self, VALA_TOKEN_TYPE_YIELD, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp0_ = vala_parser_parse_expression (self, &_inner_error0_);
	expr = _tmp0_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp1_ = expr;
	call = VALA_IS_METHOD_CALL (_tmp1_) ? ((ValaMethodCall*) _tmp1_) : NULL;
	_tmp2_ = expr;
	object_creation = VALA_IS_OBJECT_CREATION_EXPRESSION (_tmp2_) ? ((ValaObjectCreationExpression*) _tmp2_) : NULL;
	_tmp4_ = call;
	if (_tmp4_ == NULL) {
		ValaObjectCreationExpression* _tmp5_;
		_tmp5_ = object_creation;
		_tmp3_ = _tmp5_ == NULL;
	} else {
		_tmp3_ = FALSE;
	}
	if (_tmp3_) {
		ValaExpression* _tmp6_;
		ValaSourceReference* _tmp7_;
		ValaSourceReference* _tmp8_;
		ValaExpression* _tmp9_;
		_tmp6_ = expr;
		_tmp7_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp6_);
		_tmp8_ = _tmp7_;
		vala_report_error (_tmp8_, "syntax error, expected method call");
		_tmp9_ = expr;
		vala_code_node_set_error ((ValaCodeNode*) _tmp9_, TRUE);
	}
	_tmp10_ = call;
	if (_tmp10_ != NULL) {
		ValaMethodCall* _tmp11_;
		_tmp11_ = call;
		vala_callable_expression_set_is_yield_expression ((ValaCallableExpression*) _tmp11_, TRUE);
	} else {
		ValaObjectCreationExpression* _tmp12_;
		_tmp12_ = object_creation;
		if (_tmp12_ != NULL) {
			ValaObjectCreationExpression* _tmp13_;
			_tmp13_ = object_creation;
			vala_callable_expression_set_is_yield_expression ((ValaCallableExpression*) _tmp13_, TRUE);
		}
	}
	result = expr;
	return result;
}

static ValaExpression*
vala_parser_parse_sizeof_expression (ValaParser* self,
                                     GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaDataType* type = NULL;
	ValaDataType* _tmp1_;
	ValaDataType* _tmp2_;
	ValaSourceLocation _tmp3_;
	ValaSourceReference* _tmp4_;
	ValaSourceReference* _tmp5_;
	ValaSizeofExpression* _tmp6_;
	ValaExpression* _tmp7_;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_SIZEOF, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp1_ = vala_parser_parse_type (self, TRUE, FALSE, FALSE, &_inner_error0_);
	type = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (type);
			return NULL;
		} else {
			_vala_code_node_unref0 (type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp2_ = type;
	_tmp3_ = begin;
	_tmp4_ = vala_parser_get_src (self, &_tmp3_);
	_tmp5_ = _tmp4_;
	_tmp6_ = vala_sizeof_expression_new (_tmp2_, _tmp5_);
	_tmp7_ = (ValaExpression*) _tmp6_;
	_vala_source_reference_unref0 (_tmp5_);
	result = _tmp7_;
	_vala_code_node_unref0 (type);
	return result;
}

static ValaExpression*
vala_parser_parse_typeof_expression (ValaParser* self,
                                     GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaDataType* type = NULL;
	ValaDataType* _tmp1_;
	ValaDataType* _tmp2_;
	ValaSourceLocation _tmp3_;
	ValaSourceReference* _tmp4_;
	ValaSourceReference* _tmp5_;
	ValaTypeofExpression* _tmp6_;
	ValaExpression* _tmp7_;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_TYPEOF, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp1_ = vala_parser_parse_type (self, TRUE, FALSE, FALSE, &_inner_error0_);
	type = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (type);
			return NULL;
		} else {
			_vala_code_node_unref0 (type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp2_ = type;
	_tmp3_ = begin;
	_tmp4_ = vala_parser_get_src (self, &_tmp3_);
	_tmp5_ = _tmp4_;
	_tmp6_ = vala_typeof_expression_new (_tmp2_, _tmp5_);
	_tmp7_ = (ValaExpression*) _tmp6_;
	_vala_source_reference_unref0 (_tmp5_);
	result = _tmp7_;
	_vala_code_node_unref0 (type);
	return result;
}

static ValaUnaryOperator
vala_parser_get_unary_operator (ValaParser* self,
                                ValaTokenType token_type)
{
	ValaUnaryOperator result;
	g_return_val_if_fail (self != NULL, 0);
	switch (token_type) {
		case VALA_TOKEN_TYPE_PLUS:
		{
			result = VALA_UNARY_OPERATOR_PLUS;
			return result;
		}
		case VALA_TOKEN_TYPE_MINUS:
		{
			result = VALA_UNARY_OPERATOR_MINUS;
			return result;
		}
		case VALA_TOKEN_TYPE_OP_NEG:
		{
			result = VALA_UNARY_OPERATOR_LOGICAL_NEGATION;
			return result;
		}
		case VALA_TOKEN_TYPE_TILDE:
		{
			result = VALA_UNARY_OPERATOR_BITWISE_COMPLEMENT;
			return result;
		}
		case VALA_TOKEN_TYPE_OP_INC:
		{
			result = VALA_UNARY_OPERATOR_INCREMENT;
			return result;
		}
		case VALA_TOKEN_TYPE_OP_DEC:
		{
			result = VALA_UNARY_OPERATOR_DECREMENT;
			return result;
		}
		default:
		{
			result = VALA_UNARY_OPERATOR_NONE;
			return result;
		}
	}
}

static ValaExpression*
vala_parser_parse_unary_expression (ValaParser* self,
                                    GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaUnaryOperator operator = 0;
	ValaExpression* expr = NULL;
	ValaExpression* _tmp89_;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	operator = vala_parser_get_unary_operator (self, vala_parser_current (self));
	if (operator != VALA_UNARY_OPERATOR_NONE) {
		ValaExpression* op = NULL;
		ValaExpression* _tmp1_;
		ValaIntegerLiteral* lit = NULL;
		ValaExpression* _tmp2_;
		ValaIntegerLiteral* _tmp3_;
		ValaExpression* _tmp16_;
		ValaSourceLocation _tmp17_;
		ValaSourceReference* _tmp18_;
		ValaSourceReference* _tmp19_;
		ValaUnaryExpression* _tmp20_;
		ValaExpression* _tmp21_;
		vala_parser_next (self);
		_tmp1_ = vala_parser_parse_unary_expression (self, &_inner_error0_);
		op = _tmp1_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				return NULL;
			} else {
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp2_ = op;
		lit = VALA_IS_INTEGER_LITERAL (_tmp2_) ? ((ValaIntegerLiteral*) _tmp2_) : NULL;
		_tmp3_ = lit;
		if (_tmp3_ != NULL) {
			if (operator == VALA_UNARY_OPERATOR_PLUS) {
				ValaIntegerLiteral* _tmp4_;
				ValaExpression* _tmp5_;
				_tmp4_ = lit;
				_tmp5_ = _vala_code_node_ref0 ((ValaExpression*) _tmp4_);
				result = _tmp5_;
				_vala_code_node_unref0 (op);
				return result;
			} else {
				if (operator == VALA_UNARY_OPERATOR_MINUS) {
					ValaIntegerLiteral* _tmp6_;
					const gchar* _tmp7_;
					const gchar* _tmp8_;
					gchar* _tmp9_;
					gchar* _tmp10_;
					ValaSourceLocation _tmp11_;
					ValaSourceReference* _tmp12_;
					ValaSourceReference* _tmp13_;
					ValaIntegerLiteral* _tmp14_;
					ValaExpression* _tmp15_;
					_tmp6_ = lit;
					_tmp7_ = vala_integer_literal_get_value (_tmp6_);
					_tmp8_ = _tmp7_;
					_tmp9_ = g_strconcat ("-", _tmp8_, NULL);
					_tmp10_ = _tmp9_;
					_tmp11_ = begin;
					_tmp12_ = vala_parser_get_src (self, &_tmp11_);
					_tmp13_ = _tmp12_;
					_tmp14_ = vala_integer_literal_new (_tmp10_, _tmp13_);
					_tmp15_ = (ValaExpression*) _tmp14_;
					_vala_source_reference_unref0 (_tmp13_);
					_g_free0 (_tmp10_);
					result = _tmp15_;
					_vala_code_node_unref0 (op);
					return result;
				}
			}
		}
		_tmp16_ = op;
		_tmp17_ = begin;
		_tmp18_ = vala_parser_get_src (self, &_tmp17_);
		_tmp19_ = _tmp18_;
		_tmp20_ = vala_unary_expression_new (operator, _tmp16_, _tmp19_);
		_tmp21_ = (ValaExpression*) _tmp20_;
		_vala_source_reference_unref0 (_tmp19_);
		result = _tmp21_;
		_vala_code_node_unref0 (op);
		return result;
	}
	switch (vala_parser_current (self)) {
		case VALA_TOKEN_TYPE_OPEN_PARENS:
		{
			ValaSourceLocation _tmp74_;
			vala_parser_next (self);
			switch (vala_parser_current (self)) {
				case VALA_TOKEN_TYPE_UNOWNED:
				{
					break;
				}
				case VALA_TOKEN_TYPE_OWNED:
				{
					vala_parser_next (self);
					if (vala_parser_accept (self, VALA_TOKEN_TYPE_CLOSE_PARENS)) {
						ValaExpression* op = NULL;
						ValaExpression* _tmp22_;
						ValaExpression* _tmp23_;
						ValaSourceLocation _tmp24_;
						ValaSourceReference* _tmp25_;
						ValaSourceReference* _tmp26_;
						ValaReferenceTransferExpression* _tmp27_;
						ValaExpression* _tmp28_;
						_tmp22_ = vala_parser_parse_unary_expression (self, &_inner_error0_);
						op = _tmp22_;
						if (G_UNLIKELY (_inner_error0_ != NULL)) {
							if (_inner_error0_->domain == VALA_PARSE_ERROR) {
								g_propagate_error (error, _inner_error0_);
								return NULL;
							} else {
								g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
								g_clear_error (&_inner_error0_);
								return NULL;
							}
						}
						_tmp23_ = op;
						_tmp24_ = begin;
						_tmp25_ = vala_parser_get_src (self, &_tmp24_);
						_tmp26_ = _tmp25_;
						_tmp27_ = vala_reference_transfer_expression_new (_tmp23_, _tmp26_);
						_tmp28_ = (ValaExpression*) _tmp27_;
						_vala_source_reference_unref0 (_tmp26_);
						result = _tmp28_;
						_vala_code_node_unref0 (op);
						return result;
					}
					break;
				}
				case VALA_TOKEN_TYPE_VOID:
				case VALA_TOKEN_TYPE_DYNAMIC:
				case VALA_TOKEN_TYPE_OPEN_PARENS:
				case VALA_TOKEN_TYPE_IDENTIFIER:
				{
					gboolean _tmp29_ = FALSE;
					if (vala_parser_current (self) != VALA_TOKEN_TYPE_OPEN_PARENS) {
						_tmp29_ = TRUE;
					} else {
						_tmp29_ = vala_parser_is_inner_array_type (self);
					}
					if (_tmp29_) {
						ValaDataType* type = NULL;
						ValaDataType* _tmp30_;
						_tmp30_ = vala_parser_parse_type (self, TRUE, FALSE, FALSE, &_inner_error0_);
						type = _tmp30_;
						if (G_UNLIKELY (_inner_error0_ != NULL)) {
							if (_inner_error0_->domain == VALA_PARSE_ERROR) {
								g_propagate_error (error, _inner_error0_);
								return NULL;
							} else {
								g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
								g_clear_error (&_inner_error0_);
								return NULL;
							}
						}
						if (vala_parser_accept (self, VALA_TOKEN_TYPE_CLOSE_PARENS)) {
							switch (vala_parser_current (self)) {
								case VALA_TOKEN_TYPE_OP_NEG:
								case VALA_TOKEN_TYPE_TILDE:
								case VALA_TOKEN_TYPE_OPEN_PARENS:
								case VALA_TOKEN_TYPE_TRUE:
								case VALA_TOKEN_TYPE_FALSE:
								case VALA_TOKEN_TYPE_INTEGER_LITERAL:
								case VALA_TOKEN_TYPE_REAL_LITERAL:
								case VALA_TOKEN_TYPE_CHARACTER_LITERAL:
								case VALA_TOKEN_TYPE_STRING_LITERAL:
								case VALA_TOKEN_TYPE_TEMPLATE_STRING_LITERAL:
								case VALA_TOKEN_TYPE_VERBATIM_STRING_LITERAL:
								case VALA_TOKEN_TYPE_REGEX_LITERAL:
								case VALA_TOKEN_TYPE_NULL:
								case VALA_TOKEN_TYPE_THIS:
								case VALA_TOKEN_TYPE_BASE:
								case VALA_TOKEN_TYPE_NEW:
								case VALA_TOKEN_TYPE_YIELD:
								case VALA_TOKEN_TYPE_SIZEOF:
								case VALA_TOKEN_TYPE_TYPEOF:
								case VALA_TOKEN_TYPE_IDENTIFIER:
								case VALA_TOKEN_TYPE_PARAMS:
								{
									ValaExpression* inner = NULL;
									ValaExpression* _tmp31_;
									ValaExpression* _tmp32_;
									ValaDataType* _tmp33_;
									ValaSourceLocation _tmp34_;
									ValaSourceReference* _tmp35_;
									ValaSourceReference* _tmp36_;
									ValaCastExpression* _tmp37_;
									ValaExpression* _tmp38_;
									_tmp31_ = vala_parser_parse_unary_expression (self, &_inner_error0_);
									inner = _tmp31_;
									if (G_UNLIKELY (_inner_error0_ != NULL)) {
										if (_inner_error0_->domain == VALA_PARSE_ERROR) {
											g_propagate_error (error, _inner_error0_);
											_vala_code_node_unref0 (type);
											return NULL;
										} else {
											_vala_code_node_unref0 (type);
											g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
											g_clear_error (&_inner_error0_);
											return NULL;
										}
									}
									_tmp32_ = inner;
									_tmp33_ = type;
									_tmp34_ = begin;
									_tmp35_ = vala_parser_get_src (self, &_tmp34_);
									_tmp36_ = _tmp35_;
									_tmp37_ = vala_cast_expression_new (_tmp32_, _tmp33_, _tmp36_);
									_tmp38_ = (ValaExpression*) _tmp37_;
									_vala_source_reference_unref0 (_tmp36_);
									result = _tmp38_;
									_vala_code_node_unref0 (inner);
									_vala_code_node_unref0 (type);
									return result;
								}
								case VALA_TOKEN_TYPE_STAR:
								{
									ValaExpression* op = NULL;
									ValaExpression* _tmp39_;
									ValaPointerIndirection* inner = NULL;
									ValaExpression* _tmp40_;
									ValaSourceLocation _tmp41_;
									ValaSourceReference* _tmp42_;
									ValaSourceReference* _tmp43_;
									ValaPointerIndirection* _tmp44_;
									ValaPointerIndirection* _tmp45_;
									ValaPointerIndirection* _tmp46_;
									ValaDataType* _tmp47_;
									ValaSourceLocation _tmp48_;
									ValaSourceReference* _tmp49_;
									ValaSourceReference* _tmp50_;
									ValaCastExpression* _tmp51_;
									ValaExpression* _tmp52_;
									vala_parser_next (self);
									_tmp39_ = vala_parser_parse_unary_expression (self, &_inner_error0_);
									op = _tmp39_;
									if (G_UNLIKELY (_inner_error0_ != NULL)) {
										if (_inner_error0_->domain == VALA_PARSE_ERROR) {
											g_propagate_error (error, _inner_error0_);
											_vala_code_node_unref0 (type);
											return NULL;
										} else {
											_vala_code_node_unref0 (type);
											g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
											g_clear_error (&_inner_error0_);
											return NULL;
										}
									}
									_tmp40_ = op;
									_tmp41_ = begin;
									_tmp42_ = vala_parser_get_src (self, &_tmp41_);
									_tmp43_ = _tmp42_;
									_tmp44_ = vala_pointer_indirection_new (_tmp40_, _tmp43_);
									_tmp45_ = _tmp44_;
									_vala_source_reference_unref0 (_tmp43_);
									inner = _tmp45_;
									_tmp46_ = inner;
									_tmp47_ = type;
									_tmp48_ = begin;
									_tmp49_ = vala_parser_get_src (self, &_tmp48_);
									_tmp50_ = _tmp49_;
									_tmp51_ = vala_cast_expression_new ((ValaExpression*) _tmp46_, _tmp47_, _tmp50_);
									_tmp52_ = (ValaExpression*) _tmp51_;
									_vala_source_reference_unref0 (_tmp50_);
									result = _tmp52_;
									_vala_code_node_unref0 (inner);
									_vala_code_node_unref0 (op);
									_vala_code_node_unref0 (type);
									return result;
								}
								case VALA_TOKEN_TYPE_BITWISE_AND:
								{
									ValaExpression* op = NULL;
									ValaExpression* _tmp53_;
									ValaAddressofExpression* inner = NULL;
									ValaExpression* _tmp54_;
									ValaSourceLocation _tmp55_;
									ValaSourceReference* _tmp56_;
									ValaSourceReference* _tmp57_;
									ValaAddressofExpression* _tmp58_;
									ValaAddressofExpression* _tmp59_;
									ValaAddressofExpression* _tmp60_;
									ValaDataType* _tmp61_;
									ValaSourceLocation _tmp62_;
									ValaSourceReference* _tmp63_;
									ValaSourceReference* _tmp64_;
									ValaCastExpression* _tmp65_;
									ValaExpression* _tmp66_;
									vala_parser_next (self);
									_tmp53_ = vala_parser_parse_unary_expression (self, &_inner_error0_);
									op = _tmp53_;
									if (G_UNLIKELY (_inner_error0_ != NULL)) {
										if (_inner_error0_->domain == VALA_PARSE_ERROR) {
											g_propagate_error (error, _inner_error0_);
											_vala_code_node_unref0 (type);
											return NULL;
										} else {
											_vala_code_node_unref0 (type);
											g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
											g_clear_error (&_inner_error0_);
											return NULL;
										}
									}
									_tmp54_ = op;
									_tmp55_ = begin;
									_tmp56_ = vala_parser_get_src (self, &_tmp55_);
									_tmp57_ = _tmp56_;
									_tmp58_ = vala_addressof_expression_new (_tmp54_, _tmp57_);
									_tmp59_ = _tmp58_;
									_vala_source_reference_unref0 (_tmp57_);
									inner = _tmp59_;
									_tmp60_ = inner;
									_tmp61_ = type;
									_tmp62_ = begin;
									_tmp63_ = vala_parser_get_src (self, &_tmp62_);
									_tmp64_ = _tmp63_;
									_tmp65_ = vala_cast_expression_new ((ValaExpression*) _tmp60_, _tmp61_, _tmp64_);
									_tmp66_ = (ValaExpression*) _tmp65_;
									_vala_source_reference_unref0 (_tmp64_);
									result = _tmp66_;
									_vala_code_node_unref0 (inner);
									_vala_code_node_unref0 (op);
									_vala_code_node_unref0 (type);
									return result;
								}
								default:
								{
									break;
								}
							}
						}
						_vala_code_node_unref0 (type);
					}
					break;
				}
				case VALA_TOKEN_TYPE_OP_NEG:
				{
					vala_parser_next (self);
					if (vala_parser_accept (self, VALA_TOKEN_TYPE_CLOSE_PARENS)) {
						ValaExpression* inner = NULL;
						ValaExpression* _tmp67_;
						ValaExpression* _tmp68_;
						ValaSourceLocation _tmp69_;
						ValaSourceReference* _tmp70_;
						ValaSourceReference* _tmp71_;
						ValaCastExpression* _tmp72_;
						ValaExpression* _tmp73_;
						_tmp67_ = vala_parser_parse_unary_expression (self, &_inner_error0_);
						inner = _tmp67_;
						if (G_UNLIKELY (_inner_error0_ != NULL)) {
							if (_inner_error0_->domain == VALA_PARSE_ERROR) {
								g_propagate_error (error, _inner_error0_);
								return NULL;
							} else {
								g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
								g_clear_error (&_inner_error0_);
								return NULL;
							}
						}
						_tmp68_ = inner;
						_tmp69_ = begin;
						_tmp70_ = vala_parser_get_src (self, &_tmp69_);
						_tmp71_ = _tmp70_;
						_tmp72_ = vala_cast_expression_new_non_null (_tmp68_, _tmp71_);
						_tmp73_ = (ValaExpression*) _tmp72_;
						_vala_source_reference_unref0 (_tmp71_);
						result = _tmp73_;
						_vala_code_node_unref0 (inner);
						return result;
					}
					break;
				}
				default:
				{
					break;
				}
			}
			_tmp74_ = begin;
			vala_parser_rollback (self, &_tmp74_);
			break;
		}
		case VALA_TOKEN_TYPE_STAR:
		{
			ValaExpression* op = NULL;
			ValaExpression* _tmp75_;
			ValaExpression* _tmp76_;
			ValaSourceLocation _tmp77_;
			ValaSourceReference* _tmp78_;
			ValaSourceReference* _tmp79_;
			ValaPointerIndirection* _tmp80_;
			ValaExpression* _tmp81_;
			vala_parser_next (self);
			_tmp75_ = vala_parser_parse_unary_expression (self, &_inner_error0_);
			op = _tmp75_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					return NULL;
				} else {
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp76_ = op;
			_tmp77_ = begin;
			_tmp78_ = vala_parser_get_src (self, &_tmp77_);
			_tmp79_ = _tmp78_;
			_tmp80_ = vala_pointer_indirection_new (_tmp76_, _tmp79_);
			_tmp81_ = (ValaExpression*) _tmp80_;
			_vala_source_reference_unref0 (_tmp79_);
			result = _tmp81_;
			_vala_code_node_unref0 (op);
			return result;
		}
		case VALA_TOKEN_TYPE_BITWISE_AND:
		{
			ValaExpression* op = NULL;
			ValaExpression* _tmp82_;
			ValaExpression* _tmp83_;
			ValaSourceLocation _tmp84_;
			ValaSourceReference* _tmp85_;
			ValaSourceReference* _tmp86_;
			ValaAddressofExpression* _tmp87_;
			ValaExpression* _tmp88_;
			vala_parser_next (self);
			_tmp82_ = vala_parser_parse_unary_expression (self, &_inner_error0_);
			op = _tmp82_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					return NULL;
				} else {
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp83_ = op;
			_tmp84_ = begin;
			_tmp85_ = vala_parser_get_src (self, &_tmp84_);
			_tmp86_ = _tmp85_;
			_tmp87_ = vala_addressof_expression_new (_tmp83_, _tmp86_);
			_tmp88_ = (ValaExpression*) _tmp87_;
			_vala_source_reference_unref0 (_tmp86_);
			result = _tmp88_;
			_vala_code_node_unref0 (op);
			return result;
		}
		default:
		{
			break;
		}
	}
	_tmp89_ = vala_parser_parse_primary_expression (self, &_inner_error0_);
	expr = _tmp89_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	result = expr;
	return result;
}

static ValaBinaryOperator
vala_parser_get_binary_operator (ValaParser* self,
                                 ValaTokenType token_type)
{
	ValaBinaryOperator result;
	g_return_val_if_fail (self != NULL, 0);
	switch (token_type) {
		case VALA_TOKEN_TYPE_STAR:
		{
			result = VALA_BINARY_OPERATOR_MUL;
			return result;
		}
		case VALA_TOKEN_TYPE_DIV:
		{
			result = VALA_BINARY_OPERATOR_DIV;
			return result;
		}
		case VALA_TOKEN_TYPE_PERCENT:
		{
			result = VALA_BINARY_OPERATOR_MOD;
			return result;
		}
		case VALA_TOKEN_TYPE_PLUS:
		{
			result = VALA_BINARY_OPERATOR_PLUS;
			return result;
		}
		case VALA_TOKEN_TYPE_MINUS:
		{
			result = VALA_BINARY_OPERATOR_MINUS;
			return result;
		}
		case VALA_TOKEN_TYPE_OP_LT:
		{
			result = VALA_BINARY_OPERATOR_LESS_THAN;
			return result;
		}
		case VALA_TOKEN_TYPE_OP_GT:
		{
			result = VALA_BINARY_OPERATOR_GREATER_THAN;
			return result;
		}
		case VALA_TOKEN_TYPE_OP_LE:
		{
			result = VALA_BINARY_OPERATOR_LESS_THAN_OR_EQUAL;
			return result;
		}
		case VALA_TOKEN_TYPE_OP_GE:
		{
			result = VALA_BINARY_OPERATOR_GREATER_THAN_OR_EQUAL;
			return result;
		}
		case VALA_TOKEN_TYPE_OP_EQ:
		{
			result = VALA_BINARY_OPERATOR_EQUALITY;
			return result;
		}
		case VALA_TOKEN_TYPE_OP_NE:
		{
			result = VALA_BINARY_OPERATOR_INEQUALITY;
			return result;
		}
		default:
		{
			result = VALA_BINARY_OPERATOR_NONE;
			return result;
		}
	}
}

static ValaExpression*
vala_parser_parse_multiplicative_expression (ValaParser* self,
                                             GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaExpression* left = NULL;
	ValaExpression* _tmp1_;
	gboolean found = FALSE;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	_tmp1_ = vala_parser_parse_unary_expression (self, &_inner_error0_);
	left = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	found = TRUE;
	while (TRUE) {
		ValaBinaryOperator operator = 0;
		if (!found) {
			break;
		}
		operator = vala_parser_get_binary_operator (self, vala_parser_current (self));
		switch (operator) {
			case VALA_BINARY_OPERATOR_MUL:
			case VALA_BINARY_OPERATOR_DIV:
			case VALA_BINARY_OPERATOR_MOD:
			{
				ValaExpression* right = NULL;
				ValaExpression* _tmp2_;
				ValaExpression* _tmp3_;
				ValaExpression* _tmp4_;
				ValaSourceLocation _tmp5_;
				ValaSourceReference* _tmp6_;
				ValaSourceReference* _tmp7_;
				ValaBinaryExpression* _tmp8_;
				vala_parser_next (self);
				_tmp2_ = vala_parser_parse_unary_expression (self, &_inner_error0_);
				right = _tmp2_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_code_node_unref0 (left);
						return NULL;
					} else {
						_vala_code_node_unref0 (left);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return NULL;
					}
				}
				_tmp3_ = left;
				_tmp4_ = right;
				_tmp5_ = begin;
				_tmp6_ = vala_parser_get_src (self, &_tmp5_);
				_tmp7_ = _tmp6_;
				_tmp8_ = vala_binary_expression_new (operator, _tmp3_, _tmp4_, _tmp7_);
				_vala_code_node_unref0 (left);
				left = (ValaExpression*) _tmp8_;
				_vala_source_reference_unref0 (_tmp7_);
				_vala_code_node_unref0 (right);
				break;
			}
			default:
			{
				found = FALSE;
				break;
			}
		}
	}
	result = left;
	return result;
}

static ValaExpression*
vala_parser_parse_additive_expression (ValaParser* self,
                                       GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaExpression* left = NULL;
	ValaExpression* _tmp1_;
	gboolean found = FALSE;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	_tmp1_ = vala_parser_parse_multiplicative_expression (self, &_inner_error0_);
	left = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	found = TRUE;
	while (TRUE) {
		ValaBinaryOperator operator = 0;
		if (!found) {
			break;
		}
		operator = vala_parser_get_binary_operator (self, vala_parser_current (self));
		switch (operator) {
			case VALA_BINARY_OPERATOR_PLUS:
			case VALA_BINARY_OPERATOR_MINUS:
			{
				ValaExpression* right = NULL;
				ValaExpression* _tmp2_;
				ValaExpression* _tmp3_;
				ValaExpression* _tmp4_;
				ValaSourceLocation _tmp5_;
				ValaSourceReference* _tmp6_;
				ValaSourceReference* _tmp7_;
				ValaBinaryExpression* _tmp8_;
				vala_parser_next (self);
				_tmp2_ = vala_parser_parse_multiplicative_expression (self, &_inner_error0_);
				right = _tmp2_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_code_node_unref0 (left);
						return NULL;
					} else {
						_vala_code_node_unref0 (left);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return NULL;
					}
				}
				_tmp3_ = left;
				_tmp4_ = right;
				_tmp5_ = begin;
				_tmp6_ = vala_parser_get_src (self, &_tmp5_);
				_tmp7_ = _tmp6_;
				_tmp8_ = vala_binary_expression_new (operator, _tmp3_, _tmp4_, _tmp7_);
				_vala_code_node_unref0 (left);
				left = (ValaExpression*) _tmp8_;
				_vala_source_reference_unref0 (_tmp7_);
				_vala_code_node_unref0 (right);
				break;
			}
			default:
			{
				found = FALSE;
				break;
			}
		}
	}
	result = left;
	return result;
}

static ValaExpression*
vala_parser_parse_shift_expression (ValaParser* self,
                                    GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaExpression* left = NULL;
	ValaExpression* _tmp1_;
	gboolean found = FALSE;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	_tmp1_ = vala_parser_parse_additive_expression (self, &_inner_error0_);
	left = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	found = TRUE;
	while (TRUE) {
		if (!found) {
			break;
		}
		switch (vala_parser_current (self)) {
			case VALA_TOKEN_TYPE_OP_SHIFT_LEFT:
			{
				ValaExpression* right = NULL;
				ValaExpression* _tmp2_;
				ValaExpression* _tmp3_;
				ValaExpression* _tmp4_;
				ValaSourceLocation _tmp5_;
				ValaSourceReference* _tmp6_;
				ValaSourceReference* _tmp7_;
				ValaBinaryExpression* _tmp8_;
				vala_parser_next (self);
				_tmp2_ = vala_parser_parse_additive_expression (self, &_inner_error0_);
				right = _tmp2_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_code_node_unref0 (left);
						return NULL;
					} else {
						_vala_code_node_unref0 (left);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return NULL;
					}
				}
				_tmp3_ = left;
				_tmp4_ = right;
				_tmp5_ = begin;
				_tmp6_ = vala_parser_get_src (self, &_tmp5_);
				_tmp7_ = _tmp6_;
				_tmp8_ = vala_binary_expression_new (VALA_BINARY_OPERATOR_SHIFT_LEFT, _tmp3_, _tmp4_, _tmp7_);
				_vala_code_node_unref0 (left);
				left = (ValaExpression*) _tmp8_;
				_vala_source_reference_unref0 (_tmp7_);
				_vala_code_node_unref0 (right);
				break;
			}
			case VALA_TOKEN_TYPE_OP_GT:
			{
				gchar* first_gt_pos = NULL;
				ValaParserTokenInfo* _tmp9_;
				gint _tmp9__length1;
				ValaParserTokenInfo _tmp10_;
				ValaSourceLocation _tmp11_;
				gchar* _tmp12_;
				gboolean _tmp13_ = FALSE;
				_tmp9_ = self->priv->tokens;
				_tmp9__length1 = self->priv->tokens_length1;
				_tmp10_ = _tmp9_[self->priv->index];
				_tmp11_ = _tmp10_.begin;
				_tmp12_ = _tmp11_.pos;
				first_gt_pos = _tmp12_;
				vala_parser_next (self);
				if (vala_parser_current (self) == VALA_TOKEN_TYPE_OP_GT) {
					ValaParserTokenInfo* _tmp14_;
					gint _tmp14__length1;
					ValaParserTokenInfo _tmp15_;
					ValaSourceLocation _tmp16_;
					gchar* _tmp17_;
					gchar* _tmp18_;
					_tmp14_ = self->priv->tokens;
					_tmp14__length1 = self->priv->tokens_length1;
					_tmp15_ = _tmp14_[self->priv->index];
					_tmp16_ = _tmp15_.begin;
					_tmp17_ = _tmp16_.pos;
					_tmp18_ = first_gt_pos;
					_tmp13_ = _tmp17_ == (_tmp18_ + 1);
				} else {
					_tmp13_ = FALSE;
				}
				if (_tmp13_) {
					ValaExpression* right = NULL;
					ValaExpression* _tmp19_;
					ValaExpression* _tmp20_;
					ValaExpression* _tmp21_;
					ValaSourceLocation _tmp22_;
					ValaSourceReference* _tmp23_;
					ValaSourceReference* _tmp24_;
					ValaBinaryExpression* _tmp25_;
					vala_parser_next (self);
					_tmp19_ = vala_parser_parse_additive_expression (self, &_inner_error0_);
					right = _tmp19_;
					if (G_UNLIKELY (_inner_error0_ != NULL)) {
						if (_inner_error0_->domain == VALA_PARSE_ERROR) {
							g_propagate_error (error, _inner_error0_);
							_vala_code_node_unref0 (left);
							return NULL;
						} else {
							_vala_code_node_unref0 (left);
							g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
							g_clear_error (&_inner_error0_);
							return NULL;
						}
					}
					_tmp20_ = left;
					_tmp21_ = right;
					_tmp22_ = begin;
					_tmp23_ = vala_parser_get_src (self, &_tmp22_);
					_tmp24_ = _tmp23_;
					_tmp25_ = vala_binary_expression_new (VALA_BINARY_OPERATOR_SHIFT_RIGHT, _tmp20_, _tmp21_, _tmp24_);
					_vala_code_node_unref0 (left);
					left = (ValaExpression*) _tmp25_;
					_vala_source_reference_unref0 (_tmp24_);
					_vala_code_node_unref0 (right);
				} else {
					vala_parser_prev (self);
					found = FALSE;
				}
				break;
			}
			default:
			{
				found = FALSE;
				break;
			}
		}
	}
	result = left;
	return result;
}

static ValaExpression*
vala_parser_parse_type_check_expression (ValaParser* self,
                                         GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaExpression* left = NULL;
	ValaExpression* _tmp1_;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	_tmp1_ = vala_parser_parse_shift_expression (self, &_inner_error0_);
	left = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	if (vala_parser_accept (self, VALA_TOKEN_TYPE_IS)) {
		ValaDataType* type = NULL;
		ValaDataType* _tmp2_;
		ValaExpression* _tmp3_;
		ValaDataType* _tmp4_;
		ValaSourceLocation _tmp5_;
		ValaSourceReference* _tmp6_;
		ValaSourceReference* _tmp7_;
		ValaTypeCheck* _tmp8_;
		_tmp2_ = vala_parser_parse_type (self, TRUE, FALSE, FALSE, &_inner_error0_);
		type = _tmp2_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (left);
				return NULL;
			} else {
				_vala_code_node_unref0 (left);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp3_ = left;
		_tmp4_ = type;
		_tmp5_ = begin;
		_tmp6_ = vala_parser_get_src (self, &_tmp5_);
		_tmp7_ = _tmp6_;
		_tmp8_ = vala_typecheck_new (_tmp3_, _tmp4_, _tmp7_);
		_vala_code_node_unref0 (left);
		left = (ValaExpression*) _tmp8_;
		_vala_source_reference_unref0 (_tmp7_);
		_vala_code_node_unref0 (type);
	} else {
		if (vala_parser_accept (self, VALA_TOKEN_TYPE_AS)) {
			ValaDataType* type = NULL;
			ValaDataType* _tmp9_;
			ValaExpression* _tmp10_;
			ValaDataType* _tmp11_;
			ValaSourceLocation _tmp12_;
			ValaSourceReference* _tmp13_;
			ValaSourceReference* _tmp14_;
			ValaCastExpression* _tmp15_;
			_tmp9_ = vala_parser_parse_type (self, TRUE, FALSE, FALSE, &_inner_error0_);
			type = _tmp9_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (left);
					return NULL;
				} else {
					_vala_code_node_unref0 (left);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp10_ = left;
			_tmp11_ = type;
			_tmp12_ = begin;
			_tmp13_ = vala_parser_get_src (self, &_tmp12_);
			_tmp14_ = _tmp13_;
			_tmp15_ = vala_cast_expression_new_silent (_tmp10_, _tmp11_, _tmp14_);
			_vala_code_node_unref0 (left);
			left = (ValaExpression*) _tmp15_;
			_vala_source_reference_unref0 (_tmp14_);
			_vala_code_node_unref0 (type);
		}
	}
	result = left;
	return result;
}

static ValaExpression*
vala_parser_parse_relational_expression (ValaParser* self,
                                         GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaExpression* left = NULL;
	ValaExpression* _tmp1_;
	gboolean first = FALSE;
	gboolean found = FALSE;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	_tmp1_ = vala_parser_parse_type_check_expression (self, &_inner_error0_);
	left = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	first = TRUE;
	found = TRUE;
	while (TRUE) {
		ValaBinaryOperator operator = 0;
		if (!found) {
			break;
		}
		operator = vala_parser_get_binary_operator (self, vala_parser_current (self));
		switch (operator) {
			case VALA_BINARY_OPERATOR_LESS_THAN:
			case VALA_BINARY_OPERATOR_LESS_THAN_OR_EQUAL:
			case VALA_BINARY_OPERATOR_GREATER_THAN_OR_EQUAL:
			case VALA_BINARY_OPERATOR_EQUALITY:
			case VALA_BINARY_OPERATOR_INEQUALITY:
			{
				ValaExpression* right = NULL;
				ValaExpression* _tmp2_;
				vala_parser_next (self);
				_tmp2_ = vala_parser_parse_type_check_expression (self, &_inner_error0_);
				right = _tmp2_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_code_node_unref0 (left);
						return NULL;
					} else {
						_vala_code_node_unref0 (left);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return NULL;
					}
				}
				if (first) {
					ValaExpression* _tmp3_;
					ValaExpression* _tmp4_;
					ValaSourceLocation _tmp5_;
					ValaSourceReference* _tmp6_;
					ValaSourceReference* _tmp7_;
					ValaBinaryExpression* _tmp8_;
					_tmp3_ = left;
					_tmp4_ = right;
					_tmp5_ = begin;
					_tmp6_ = vala_parser_get_src (self, &_tmp5_);
					_tmp7_ = _tmp6_;
					_tmp8_ = vala_binary_expression_new (operator, _tmp3_, _tmp4_, _tmp7_);
					_vala_code_node_unref0 (left);
					left = (ValaExpression*) _tmp8_;
					_vala_source_reference_unref0 (_tmp7_);
				} else {
					ValaExpression* _tmp9_;
					ValaExpression* _tmp10_;
					ValaSourceLocation _tmp11_;
					ValaSourceReference* _tmp12_;
					ValaSourceReference* _tmp13_;
					ValaBinaryExpression* _tmp14_;
					_tmp9_ = left;
					_tmp10_ = right;
					_tmp11_ = begin;
					_tmp12_ = vala_parser_get_src (self, &_tmp11_);
					_tmp13_ = _tmp12_;
					_tmp14_ = vala_binary_expression_new_chained (operator, _tmp9_, _tmp10_, _tmp13_);
					_vala_code_node_unref0 (left);
					left = (ValaExpression*) _tmp14_;
					_vala_source_reference_unref0 (_tmp13_);
				}
				first = FALSE;
				_vala_code_node_unref0 (right);
				break;
			}
			case VALA_BINARY_OPERATOR_GREATER_THAN:
			{
				gboolean _tmp15_ = FALSE;
				vala_parser_next (self);
				if (vala_parser_current (self) != VALA_TOKEN_TYPE_OP_GT) {
					_tmp15_ = vala_parser_current (self) != VALA_TOKEN_TYPE_OP_GE;
				} else {
					_tmp15_ = FALSE;
				}
				if (_tmp15_) {
					ValaExpression* right = NULL;
					ValaExpression* _tmp16_;
					_tmp16_ = vala_parser_parse_type_check_expression (self, &_inner_error0_);
					right = _tmp16_;
					if (G_UNLIKELY (_inner_error0_ != NULL)) {
						if (_inner_error0_->domain == VALA_PARSE_ERROR) {
							g_propagate_error (error, _inner_error0_);
							_vala_code_node_unref0 (left);
							return NULL;
						} else {
							_vala_code_node_unref0 (left);
							g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
							g_clear_error (&_inner_error0_);
							return NULL;
						}
					}
					if (first) {
						ValaExpression* _tmp17_;
						ValaExpression* _tmp18_;
						ValaSourceLocation _tmp19_;
						ValaSourceReference* _tmp20_;
						ValaSourceReference* _tmp21_;
						ValaBinaryExpression* _tmp22_;
						_tmp17_ = left;
						_tmp18_ = right;
						_tmp19_ = begin;
						_tmp20_ = vala_parser_get_src (self, &_tmp19_);
						_tmp21_ = _tmp20_;
						_tmp22_ = vala_binary_expression_new (operator, _tmp17_, _tmp18_, _tmp21_);
						_vala_code_node_unref0 (left);
						left = (ValaExpression*) _tmp22_;
						_vala_source_reference_unref0 (_tmp21_);
					} else {
						ValaExpression* _tmp23_;
						ValaExpression* _tmp24_;
						ValaSourceLocation _tmp25_;
						ValaSourceReference* _tmp26_;
						ValaSourceReference* _tmp27_;
						ValaBinaryExpression* _tmp28_;
						_tmp23_ = left;
						_tmp24_ = right;
						_tmp25_ = begin;
						_tmp26_ = vala_parser_get_src (self, &_tmp25_);
						_tmp27_ = _tmp26_;
						_tmp28_ = vala_binary_expression_new_chained (operator, _tmp23_, _tmp24_, _tmp27_);
						_vala_code_node_unref0 (left);
						left = (ValaExpression*) _tmp28_;
						_vala_source_reference_unref0 (_tmp27_);
					}
					first = FALSE;
					_vala_code_node_unref0 (right);
				} else {
					vala_parser_prev (self);
					found = FALSE;
				}
				break;
			}
			default:
			{
				found = FALSE;
				break;
			}
		}
	}
	result = left;
	return result;
}

static ValaExpression*
vala_parser_parse_and_expression (ValaParser* self,
                                  GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaExpression* left = NULL;
	ValaExpression* _tmp1_;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	_tmp1_ = vala_parser_parse_relational_expression (self, &_inner_error0_);
	left = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	while (TRUE) {
		ValaExpression* right = NULL;
		ValaExpression* _tmp2_;
		ValaExpression* _tmp3_;
		ValaExpression* _tmp4_;
		ValaSourceLocation _tmp5_;
		ValaSourceReference* _tmp6_;
		ValaSourceReference* _tmp7_;
		ValaBinaryExpression* _tmp8_;
		if (!vala_parser_accept (self, VALA_TOKEN_TYPE_BITWISE_AND)) {
			break;
		}
		_tmp2_ = vala_parser_parse_relational_expression (self, &_inner_error0_);
		right = _tmp2_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (left);
				return NULL;
			} else {
				_vala_code_node_unref0 (left);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp3_ = left;
		_tmp4_ = right;
		_tmp5_ = begin;
		_tmp6_ = vala_parser_get_src (self, &_tmp5_);
		_tmp7_ = _tmp6_;
		_tmp8_ = vala_binary_expression_new (VALA_BINARY_OPERATOR_BITWISE_AND, _tmp3_, _tmp4_, _tmp7_);
		_vala_code_node_unref0 (left);
		left = (ValaExpression*) _tmp8_;
		_vala_source_reference_unref0 (_tmp7_);
		_vala_code_node_unref0 (right);
	}
	result = left;
	return result;
}

static ValaExpression*
vala_parser_parse_exclusive_or_expression (ValaParser* self,
                                           GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaExpression* left = NULL;
	ValaExpression* _tmp1_;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	_tmp1_ = vala_parser_parse_and_expression (self, &_inner_error0_);
	left = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	while (TRUE) {
		ValaExpression* right = NULL;
		ValaExpression* _tmp2_;
		ValaExpression* _tmp3_;
		ValaExpression* _tmp4_;
		ValaSourceLocation _tmp5_;
		ValaSourceReference* _tmp6_;
		ValaSourceReference* _tmp7_;
		ValaBinaryExpression* _tmp8_;
		if (!vala_parser_accept (self, VALA_TOKEN_TYPE_CARRET)) {
			break;
		}
		_tmp2_ = vala_parser_parse_and_expression (self, &_inner_error0_);
		right = _tmp2_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (left);
				return NULL;
			} else {
				_vala_code_node_unref0 (left);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp3_ = left;
		_tmp4_ = right;
		_tmp5_ = begin;
		_tmp6_ = vala_parser_get_src (self, &_tmp5_);
		_tmp7_ = _tmp6_;
		_tmp8_ = vala_binary_expression_new (VALA_BINARY_OPERATOR_BITWISE_XOR, _tmp3_, _tmp4_, _tmp7_);
		_vala_code_node_unref0 (left);
		left = (ValaExpression*) _tmp8_;
		_vala_source_reference_unref0 (_tmp7_);
		_vala_code_node_unref0 (right);
	}
	result = left;
	return result;
}

static ValaExpression*
vala_parser_parse_inclusive_or_expression (ValaParser* self,
                                           GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaExpression* left = NULL;
	ValaExpression* _tmp1_;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	_tmp1_ = vala_parser_parse_exclusive_or_expression (self, &_inner_error0_);
	left = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	while (TRUE) {
		ValaExpression* right = NULL;
		ValaExpression* _tmp2_;
		ValaExpression* _tmp3_;
		ValaExpression* _tmp4_;
		ValaSourceLocation _tmp5_;
		ValaSourceReference* _tmp6_;
		ValaSourceReference* _tmp7_;
		ValaBinaryExpression* _tmp8_;
		if (!vala_parser_accept (self, VALA_TOKEN_TYPE_BITWISE_OR)) {
			break;
		}
		_tmp2_ = vala_parser_parse_exclusive_or_expression (self, &_inner_error0_);
		right = _tmp2_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (left);
				return NULL;
			} else {
				_vala_code_node_unref0 (left);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp3_ = left;
		_tmp4_ = right;
		_tmp5_ = begin;
		_tmp6_ = vala_parser_get_src (self, &_tmp5_);
		_tmp7_ = _tmp6_;
		_tmp8_ = vala_binary_expression_new (VALA_BINARY_OPERATOR_BITWISE_OR, _tmp3_, _tmp4_, _tmp7_);
		_vala_code_node_unref0 (left);
		left = (ValaExpression*) _tmp8_;
		_vala_source_reference_unref0 (_tmp7_);
		_vala_code_node_unref0 (right);
	}
	result = left;
	return result;
}

static ValaExpression*
vala_parser_parse_in_expression (ValaParser* self,
                                 GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaExpression* left = NULL;
	ValaExpression* _tmp1_;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	_tmp1_ = vala_parser_parse_inclusive_or_expression (self, &_inner_error0_);
	left = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	while (TRUE) {
		ValaExpression* right = NULL;
		ValaExpression* _tmp2_;
		ValaExpression* _tmp3_;
		ValaExpression* _tmp4_;
		ValaSourceLocation _tmp5_;
		ValaSourceReference* _tmp6_;
		ValaSourceReference* _tmp7_;
		ValaBinaryExpression* _tmp8_;
		if (!vala_parser_accept (self, VALA_TOKEN_TYPE_IN)) {
			break;
		}
		_tmp2_ = vala_parser_parse_inclusive_or_expression (self, &_inner_error0_);
		right = _tmp2_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (left);
				return NULL;
			} else {
				_vala_code_node_unref0 (left);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp3_ = left;
		_tmp4_ = right;
		_tmp5_ = begin;
		_tmp6_ = vala_parser_get_src (self, &_tmp5_);
		_tmp7_ = _tmp6_;
		_tmp8_ = vala_binary_expression_new (VALA_BINARY_OPERATOR_IN, _tmp3_, _tmp4_, _tmp7_);
		_vala_code_node_unref0 (left);
		left = (ValaExpression*) _tmp8_;
		_vala_source_reference_unref0 (_tmp7_);
		_vala_code_node_unref0 (right);
	}
	result = left;
	return result;
}

static ValaExpression*
vala_parser_parse_conditional_and_expression (ValaParser* self,
                                              GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaExpression* left = NULL;
	ValaExpression* _tmp1_;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	_tmp1_ = vala_parser_parse_in_expression (self, &_inner_error0_);
	left = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	while (TRUE) {
		ValaExpression* right = NULL;
		ValaExpression* _tmp2_;
		ValaExpression* _tmp3_;
		ValaExpression* _tmp4_;
		ValaSourceLocation _tmp5_;
		ValaSourceReference* _tmp6_;
		ValaSourceReference* _tmp7_;
		ValaBinaryExpression* _tmp8_;
		if (!vala_parser_accept (self, VALA_TOKEN_TYPE_OP_AND)) {
			break;
		}
		_tmp2_ = vala_parser_parse_in_expression (self, &_inner_error0_);
		right = _tmp2_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (left);
				return NULL;
			} else {
				_vala_code_node_unref0 (left);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp3_ = left;
		_tmp4_ = right;
		_tmp5_ = begin;
		_tmp6_ = vala_parser_get_src (self, &_tmp5_);
		_tmp7_ = _tmp6_;
		_tmp8_ = vala_binary_expression_new (VALA_BINARY_OPERATOR_AND, _tmp3_, _tmp4_, _tmp7_);
		_vala_code_node_unref0 (left);
		left = (ValaExpression*) _tmp8_;
		_vala_source_reference_unref0 (_tmp7_);
		_vala_code_node_unref0 (right);
	}
	result = left;
	return result;
}

static ValaExpression*
vala_parser_parse_conditional_or_expression (ValaParser* self,
                                             GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaExpression* left = NULL;
	ValaExpression* _tmp1_;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	_tmp1_ = vala_parser_parse_conditional_and_expression (self, &_inner_error0_);
	left = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	while (TRUE) {
		ValaExpression* right = NULL;
		ValaExpression* _tmp2_;
		ValaExpression* _tmp3_;
		ValaExpression* _tmp4_;
		ValaSourceLocation _tmp5_;
		ValaSourceReference* _tmp6_;
		ValaSourceReference* _tmp7_;
		ValaBinaryExpression* _tmp8_;
		if (!vala_parser_accept (self, VALA_TOKEN_TYPE_OP_OR)) {
			break;
		}
		_tmp2_ = vala_parser_parse_conditional_and_expression (self, &_inner_error0_);
		right = _tmp2_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (left);
				return NULL;
			} else {
				_vala_code_node_unref0 (left);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp3_ = left;
		_tmp4_ = right;
		_tmp5_ = begin;
		_tmp6_ = vala_parser_get_src (self, &_tmp5_);
		_tmp7_ = _tmp6_;
		_tmp8_ = vala_binary_expression_new (VALA_BINARY_OPERATOR_OR, _tmp3_, _tmp4_, _tmp7_);
		_vala_code_node_unref0 (left);
		left = (ValaExpression*) _tmp8_;
		_vala_source_reference_unref0 (_tmp7_);
		_vala_code_node_unref0 (right);
	}
	result = left;
	return result;
}

static ValaExpression*
vala_parser_parse_coalescing_expression (ValaParser* self,
                                         GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaExpression* left = NULL;
	ValaExpression* _tmp1_;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	_tmp1_ = vala_parser_parse_conditional_or_expression (self, &_inner_error0_);
	left = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	if (vala_parser_accept (self, VALA_TOKEN_TYPE_OP_COALESCING)) {
		ValaExpression* right = NULL;
		ValaExpression* _tmp2_;
		ValaExpression* _tmp3_;
		ValaExpression* _tmp4_;
		ValaSourceLocation _tmp5_;
		ValaSourceReference* _tmp6_;
		ValaSourceReference* _tmp7_;
		ValaBinaryExpression* _tmp8_;
		ValaExpression* _tmp9_;
		_tmp2_ = vala_parser_parse_coalescing_expression (self, &_inner_error0_);
		right = _tmp2_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (left);
				return NULL;
			} else {
				_vala_code_node_unref0 (left);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp3_ = left;
		_tmp4_ = right;
		_tmp5_ = begin;
		_tmp6_ = vala_parser_get_src (self, &_tmp5_);
		_tmp7_ = _tmp6_;
		_tmp8_ = vala_binary_expression_new (VALA_BINARY_OPERATOR_COALESCE, _tmp3_, _tmp4_, _tmp7_);
		_tmp9_ = (ValaExpression*) _tmp8_;
		_vala_source_reference_unref0 (_tmp7_);
		result = _tmp9_;
		_vala_code_node_unref0 (right);
		_vala_code_node_unref0 (left);
		return result;
	} else {
		result = left;
		return result;
	}
}

static ValaExpression*
vala_parser_parse_conditional_expression (ValaParser* self,
                                          GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaExpression* condition = NULL;
	ValaExpression* _tmp1_;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	_tmp1_ = vala_parser_parse_coalescing_expression (self, &_inner_error0_);
	condition = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	if (vala_parser_accept (self, VALA_TOKEN_TYPE_INTERR)) {
		ValaExpression* true_expr = NULL;
		ValaExpression* _tmp2_;
		ValaExpression* false_expr = NULL;
		ValaExpression* _tmp3_;
		ValaExpression* _tmp4_;
		ValaExpression* _tmp5_;
		ValaExpression* _tmp6_;
		ValaSourceLocation _tmp7_;
		ValaSourceReference* _tmp8_;
		ValaSourceReference* _tmp9_;
		ValaConditionalExpression* _tmp10_;
		ValaExpression* _tmp11_;
		_tmp2_ = vala_parser_parse_expression (self, &_inner_error0_);
		true_expr = _tmp2_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (condition);
				return NULL;
			} else {
				_vala_code_node_unref0 (condition);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		vala_parser_expect (self, VALA_TOKEN_TYPE_COLON, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (true_expr);
				_vala_code_node_unref0 (condition);
				return NULL;
			} else {
				_vala_code_node_unref0 (true_expr);
				_vala_code_node_unref0 (condition);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp3_ = vala_parser_parse_expression (self, &_inner_error0_);
		false_expr = _tmp3_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (true_expr);
				_vala_code_node_unref0 (condition);
				return NULL;
			} else {
				_vala_code_node_unref0 (true_expr);
				_vala_code_node_unref0 (condition);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp4_ = condition;
		_tmp5_ = true_expr;
		_tmp6_ = false_expr;
		_tmp7_ = begin;
		_tmp8_ = vala_parser_get_src (self, &_tmp7_);
		_tmp9_ = _tmp8_;
		_tmp10_ = vala_conditional_expression_new (_tmp4_, _tmp5_, _tmp6_, _tmp9_);
		_tmp11_ = (ValaExpression*) _tmp10_;
		_vala_source_reference_unref0 (_tmp9_);
		result = _tmp11_;
		_vala_code_node_unref0 (false_expr);
		_vala_code_node_unref0 (true_expr);
		_vala_code_node_unref0 (condition);
		return result;
	} else {
		result = condition;
		return result;
	}
}

static ValaParameter*
vala_parser_parse_lambda_parameter (ValaParser* self,
                                    GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaParameterDirection direction = 0;
	gchar* id = NULL;
	gchar* _tmp1_;
	ValaParameter* param = NULL;
	ValaSourceLocation _tmp2_;
	ValaSourceReference* _tmp3_;
	ValaSourceReference* _tmp4_;
	ValaParameter* _tmp5_;
	ValaParameter* _tmp6_;
	ValaParameter* _tmp7_;
	GError* _inner_error0_ = NULL;
	ValaParameter* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	direction = VALA_PARAMETER_DIRECTION_IN;
	if (vala_parser_accept (self, VALA_TOKEN_TYPE_OUT)) {
		direction = VALA_PARAMETER_DIRECTION_OUT;
	} else {
		if (vala_parser_accept (self, VALA_TOKEN_TYPE_REF)) {
			direction = VALA_PARAMETER_DIRECTION_REF;
		}
	}
	_tmp1_ = vala_parser_parse_identifier (self, FALSE, &_inner_error0_);
	id = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp2_ = begin;
	_tmp3_ = vala_parser_get_src (self, &_tmp2_);
	_tmp4_ = _tmp3_;
	_tmp5_ = vala_parameter_new (id, NULL, _tmp4_);
	_tmp6_ = _tmp5_;
	_vala_source_reference_unref0 (_tmp4_);
	param = _tmp6_;
	_tmp7_ = param;
	vala_parameter_set_direction (_tmp7_, direction);
	result = param;
	_g_free0 (id);
	return result;
}

static ValaExpression*
vala_parser_parse_lambda_expression (ValaParser* self,
                                     GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaList* params = NULL;
	GEqualFunc _tmp1_;
	ValaArrayList* _tmp2_;
	ValaLambdaExpression* lambda = NULL;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	_tmp1_ = g_direct_equal;
	_tmp2_ = vala_array_list_new (VALA_TYPE_PARAMETER, (GBoxedCopyFunc) vala_code_node_ref, (GDestroyNotify) vala_code_node_unref, _tmp1_);
	params = (ValaList*) _tmp2_;
	if (vala_parser_accept (self, VALA_TOKEN_TYPE_OPEN_PARENS)) {
		if (vala_parser_current (self) != VALA_TOKEN_TYPE_CLOSE_PARENS) {
			{
				gboolean _tmp3_ = FALSE;
				_tmp3_ = TRUE;
				while (TRUE) {
					ValaParameter* _tmp4_ = NULL;
					ValaParameter* _tmp5_;
					ValaList* _tmp6_;
					if (!_tmp3_) {
						if (!vala_parser_accept (self, VALA_TOKEN_TYPE_COMMA)) {
							break;
						}
					}
					_tmp3_ = FALSE;
					_tmp5_ = vala_parser_parse_lambda_parameter (self, &_inner_error0_);
					_tmp4_ = _tmp5_;
					if (G_UNLIKELY (_inner_error0_ != NULL)) {
						if (_inner_error0_->domain == VALA_PARSE_ERROR) {
							g_propagate_error (error, _inner_error0_);
							_vala_iterable_unref0 (params);
							return NULL;
						} else {
							_vala_iterable_unref0 (params);
							g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
							g_clear_error (&_inner_error0_);
							return NULL;
						}
					}
					_tmp6_ = params;
					vala_collection_add ((ValaCollection*) _tmp6_, _tmp4_);
					_vala_code_node_unref0 (_tmp4_);
				}
			}
		}
		vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_PARENS, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_iterable_unref0 (params);
				return NULL;
			} else {
				_vala_iterable_unref0 (params);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
	} else {
		ValaParameter* _tmp7_ = NULL;
		ValaParameter* _tmp8_;
		ValaList* _tmp9_;
		_tmp8_ = vala_parser_parse_lambda_parameter (self, &_inner_error0_);
		_tmp7_ = _tmp8_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_iterable_unref0 (params);
				return NULL;
			} else {
				_vala_iterable_unref0 (params);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp9_ = params;
		vala_collection_add ((ValaCollection*) _tmp9_, _tmp7_);
		_vala_code_node_unref0 (_tmp7_);
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_LAMBDA, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_iterable_unref0 (params);
			return NULL;
		} else {
			_vala_iterable_unref0 (params);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	if (vala_parser_current (self) == VALA_TOKEN_TYPE_OPEN_BRACE) {
		ValaBlock* block = NULL;
		ValaBlock* _tmp10_;
		ValaBlock* _tmp11_;
		ValaSourceLocation _tmp12_;
		ValaSourceReference* _tmp13_;
		ValaSourceReference* _tmp14_;
		ValaLambdaExpression* _tmp15_;
		_tmp10_ = vala_parser_parse_block (self, &_inner_error0_);
		block = _tmp10_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (lambda);
				_vala_iterable_unref0 (params);
				return NULL;
			} else {
				_vala_code_node_unref0 (lambda);
				_vala_iterable_unref0 (params);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp11_ = block;
		_tmp12_ = begin;
		_tmp13_ = vala_parser_get_src (self, &_tmp12_);
		_tmp14_ = _tmp13_;
		_tmp15_ = vala_lambda_expression_new_with_statement_body (_tmp11_, _tmp14_);
		_vala_code_node_unref0 (lambda);
		lambda = _tmp15_;
		_vala_source_reference_unref0 (_tmp14_);
		_vala_code_node_unref0 (block);
	} else {
		ValaExpression* expr = NULL;
		ValaExpression* _tmp16_;
		ValaExpression* _tmp17_;
		ValaSourceLocation _tmp18_;
		ValaSourceReference* _tmp19_;
		ValaSourceReference* _tmp20_;
		ValaLambdaExpression* _tmp21_;
		_tmp16_ = vala_parser_parse_expression (self, &_inner_error0_);
		expr = _tmp16_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (lambda);
				_vala_iterable_unref0 (params);
				return NULL;
			} else {
				_vala_code_node_unref0 (lambda);
				_vala_iterable_unref0 (params);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp17_ = expr;
		_tmp18_ = begin;
		_tmp19_ = vala_parser_get_src (self, &_tmp18_);
		_tmp20_ = _tmp19_;
		_tmp21_ = vala_lambda_expression_new (_tmp17_, _tmp20_);
		_vala_code_node_unref0 (lambda);
		lambda = _tmp21_;
		_vala_source_reference_unref0 (_tmp20_);
		_vala_code_node_unref0 (expr);
	}
	{
		ValaList* _param_list = NULL;
		ValaList* _tmp22_;
		gint _param_size = 0;
		ValaList* _tmp23_;
		gint _tmp24_;
		gint _tmp25_;
		gint _param_index = 0;
		_tmp22_ = params;
		_param_list = _tmp22_;
		_tmp23_ = _param_list;
		_tmp24_ = vala_collection_get_size ((ValaCollection*) _tmp23_);
		_tmp25_ = _tmp24_;
		_param_size = _tmp25_;
		_param_index = -1;
		while (TRUE) {
			gint _tmp26_;
			gint _tmp27_;
			ValaParameter* param = NULL;
			ValaList* _tmp28_;
			gpointer _tmp29_;
			ValaLambdaExpression* _tmp30_;
			ValaParameter* _tmp31_;
			_param_index = _param_index + 1;
			_tmp26_ = _param_index;
			_tmp27_ = _param_size;
			if (!(_tmp26_ < _tmp27_)) {
				break;
			}
			_tmp28_ = _param_list;
			_tmp29_ = vala_list_get (_tmp28_, _param_index);
			param = (ValaParameter*) _tmp29_;
			_tmp30_ = lambda;
			_tmp31_ = param;
			vala_lambda_expression_add_parameter (_tmp30_, _tmp31_);
			_vala_code_node_unref0 (param);
		}
	}
	result = (ValaExpression*) lambda;
	_vala_iterable_unref0 (params);
	return result;
}

static ValaAssignmentOperator
vala_parser_get_assignment_operator (ValaParser* self,
                                     ValaTokenType token_type)
{
	ValaAssignmentOperator result;
	g_return_val_if_fail (self != NULL, 0);
	switch (token_type) {
		case VALA_TOKEN_TYPE_ASSIGN:
		{
			result = VALA_ASSIGNMENT_OPERATOR_SIMPLE;
			return result;
		}
		case VALA_TOKEN_TYPE_ASSIGN_ADD:
		{
			result = VALA_ASSIGNMENT_OPERATOR_ADD;
			return result;
		}
		case VALA_TOKEN_TYPE_ASSIGN_SUB:
		{
			result = VALA_ASSIGNMENT_OPERATOR_SUB;
			return result;
		}
		case VALA_TOKEN_TYPE_ASSIGN_BITWISE_OR:
		{
			result = VALA_ASSIGNMENT_OPERATOR_BITWISE_OR;
			return result;
		}
		case VALA_TOKEN_TYPE_ASSIGN_BITWISE_AND:
		{
			result = VALA_ASSIGNMENT_OPERATOR_BITWISE_AND;
			return result;
		}
		case VALA_TOKEN_TYPE_ASSIGN_BITWISE_XOR:
		{
			result = VALA_ASSIGNMENT_OPERATOR_BITWISE_XOR;
			return result;
		}
		case VALA_TOKEN_TYPE_ASSIGN_DIV:
		{
			result = VALA_ASSIGNMENT_OPERATOR_DIV;
			return result;
		}
		case VALA_TOKEN_TYPE_ASSIGN_MUL:
		{
			result = VALA_ASSIGNMENT_OPERATOR_MUL;
			return result;
		}
		case VALA_TOKEN_TYPE_ASSIGN_PERCENT:
		{
			result = VALA_ASSIGNMENT_OPERATOR_PERCENT;
			return result;
		}
		case VALA_TOKEN_TYPE_ASSIGN_SHIFT_LEFT:
		{
			result = VALA_ASSIGNMENT_OPERATOR_SHIFT_LEFT;
			return result;
		}
		default:
		{
			result = VALA_ASSIGNMENT_OPERATOR_NONE;
			return result;
		}
	}
}

static ValaExpression*
vala_parser_parse_expression (ValaParser* self,
                              GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp3_ = {0};
	ValaExpression* expr = NULL;
	ValaExpression* _tmp4_;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	if (vala_parser_is_lambda_expression (self)) {
		ValaExpression* _tmp0_ = NULL;
		ValaExpression* _tmp1_;
		ValaExpression* _tmp2_;
		_tmp1_ = vala_parser_parse_lambda_expression (self, &_inner_error0_);
		_tmp0_ = _tmp1_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				return NULL;
			} else {
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp2_ = _tmp0_;
		_tmp0_ = NULL;
		result = _tmp2_;
		_vala_code_node_unref0 (_tmp0_);
		return result;
	}
	vala_parser_get_location (self, &_tmp3_);
	begin = _tmp3_;
	_tmp4_ = vala_parser_parse_conditional_expression (self, &_inner_error0_);
	expr = _tmp4_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	while (TRUE) {
		ValaAssignmentOperator operator = 0;
		operator = vala_parser_get_assignment_operator (self, vala_parser_current (self));
		if (operator != VALA_ASSIGNMENT_OPERATOR_NONE) {
			ValaExpression* rhs = NULL;
			ValaExpression* _tmp5_;
			ValaExpression* _tmp6_;
			ValaExpression* _tmp7_;
			ValaSourceLocation _tmp8_;
			ValaSourceReference* _tmp9_;
			ValaSourceReference* _tmp10_;
			ValaAssignment* _tmp11_;
			vala_parser_next (self);
			_tmp5_ = vala_parser_parse_expression (self, &_inner_error0_);
			rhs = _tmp5_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (expr);
					return NULL;
				} else {
					_vala_code_node_unref0 (expr);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp6_ = expr;
			_tmp7_ = rhs;
			_tmp8_ = begin;
			_tmp9_ = vala_parser_get_src (self, &_tmp8_);
			_tmp10_ = _tmp9_;
			_tmp11_ = vala_assignment_new (_tmp6_, _tmp7_, operator, _tmp10_);
			_vala_code_node_unref0 (expr);
			expr = (ValaExpression*) _tmp11_;
			_vala_source_reference_unref0 (_tmp10_);
			_vala_code_node_unref0 (rhs);
		} else {
			if (vala_parser_current (self) == VALA_TOKEN_TYPE_OP_GT) {
				gchar* first_gt_pos = NULL;
				ValaParserTokenInfo* _tmp12_;
				gint _tmp12__length1;
				ValaParserTokenInfo _tmp13_;
				ValaSourceLocation _tmp14_;
				gchar* _tmp15_;
				gboolean _tmp16_ = FALSE;
				_tmp12_ = self->priv->tokens;
				_tmp12__length1 = self->priv->tokens_length1;
				_tmp13_ = _tmp12_[self->priv->index];
				_tmp14_ = _tmp13_.begin;
				_tmp15_ = _tmp14_.pos;
				first_gt_pos = _tmp15_;
				vala_parser_next (self);
				if (vala_parser_current (self) == VALA_TOKEN_TYPE_OP_GE) {
					ValaParserTokenInfo* _tmp17_;
					gint _tmp17__length1;
					ValaParserTokenInfo _tmp18_;
					ValaSourceLocation _tmp19_;
					gchar* _tmp20_;
					gchar* _tmp21_;
					_tmp17_ = self->priv->tokens;
					_tmp17__length1 = self->priv->tokens_length1;
					_tmp18_ = _tmp17_[self->priv->index];
					_tmp19_ = _tmp18_.begin;
					_tmp20_ = _tmp19_.pos;
					_tmp21_ = first_gt_pos;
					_tmp16_ = _tmp20_ == (_tmp21_ + 1);
				} else {
					_tmp16_ = FALSE;
				}
				if (_tmp16_) {
					ValaExpression* rhs = NULL;
					ValaExpression* _tmp22_;
					ValaExpression* _tmp23_;
					ValaExpression* _tmp24_;
					ValaSourceLocation _tmp25_;
					ValaSourceReference* _tmp26_;
					ValaSourceReference* _tmp27_;
					ValaAssignment* _tmp28_;
					vala_parser_next (self);
					_tmp22_ = vala_parser_parse_expression (self, &_inner_error0_);
					rhs = _tmp22_;
					if (G_UNLIKELY (_inner_error0_ != NULL)) {
						if (_inner_error0_->domain == VALA_PARSE_ERROR) {
							g_propagate_error (error, _inner_error0_);
							_vala_code_node_unref0 (expr);
							return NULL;
						} else {
							_vala_code_node_unref0 (expr);
							g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
							g_clear_error (&_inner_error0_);
							return NULL;
						}
					}
					_tmp23_ = expr;
					_tmp24_ = rhs;
					_tmp25_ = begin;
					_tmp26_ = vala_parser_get_src (self, &_tmp25_);
					_tmp27_ = _tmp26_;
					_tmp28_ = vala_assignment_new (_tmp23_, _tmp24_, VALA_ASSIGNMENT_OPERATOR_SHIFT_RIGHT, _tmp27_);
					_vala_code_node_unref0 (expr);
					expr = (ValaExpression*) _tmp28_;
					_vala_source_reference_unref0 (_tmp27_);
					_vala_code_node_unref0 (rhs);
				} else {
					vala_parser_prev (self);
					break;
				}
			} else {
				break;
			}
		}
	}
	result = expr;
	return result;
}

static ValaStatement*
vala_parser_parse_statement (ValaParser* self,
                             ValaTokenType type,
                             GError** error)
{
	ValaStatement* stmt = NULL;
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	GError* _inner_error0_ = NULL;
	ValaStatement* result;
	g_return_val_if_fail (self != NULL, NULL);
	stmt = NULL;
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	{
		switch (type) {
			case VALA_TOKEN_TYPE_IF:
			{
				ValaStatement* _tmp1_ = NULL;
				ValaStatement* _tmp2_;
				ValaStatement* _tmp3_;
				_tmp2_ = vala_parser_parse_if_statement (self, &_inner_error0_);
				_tmp1_ = _tmp2_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						goto __catch0_vala_parse_error;
					}
					_vala_code_node_unref0 (stmt);
					g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
				_tmp3_ = _tmp1_;
				_tmp1_ = NULL;
				_vala_code_node_unref0 (stmt);
				stmt = _tmp3_;
				_vala_code_node_unref0 (_tmp1_);
				break;
			}
			case VALA_TOKEN_TYPE_SWITCH:
			{
				ValaStatement* _tmp4_ = NULL;
				ValaStatement* _tmp5_;
				ValaStatement* _tmp6_;
				_tmp5_ = vala_parser_parse_switch_statement (self, &_inner_error0_);
				_tmp4_ = _tmp5_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						goto __catch0_vala_parse_error;
					}
					_vala_code_node_unref0 (stmt);
					g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
				_tmp6_ = _tmp4_;
				_tmp4_ = NULL;
				_vala_code_node_unref0 (stmt);
				stmt = _tmp6_;
				_vala_code_node_unref0 (_tmp4_);
				break;
			}
			case VALA_TOKEN_TYPE_CASE:
			case VALA_TOKEN_TYPE_DEFAULT:
			{
				ValaStatement* _tmp7_ = NULL;
				ValaStatement* _tmp8_;
				ValaStatement* _tmp9_;
				_tmp8_ = vala_parser_parse_switch_section_statement (self, &_inner_error0_);
				_tmp7_ = _tmp8_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						goto __catch0_vala_parse_error;
					}
					_vala_code_node_unref0 (stmt);
					g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
				_tmp9_ = _tmp7_;
				_tmp7_ = NULL;
				_vala_code_node_unref0 (stmt);
				stmt = _tmp9_;
				_vala_code_node_unref0 (_tmp7_);
				break;
			}
			case VALA_TOKEN_TYPE_WHILE:
			{
				ValaStatement* _tmp10_ = NULL;
				ValaStatement* _tmp11_;
				ValaStatement* _tmp12_;
				_tmp11_ = vala_parser_parse_while_statement (self, &_inner_error0_);
				_tmp10_ = _tmp11_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						goto __catch0_vala_parse_error;
					}
					_vala_code_node_unref0 (stmt);
					g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
				_tmp12_ = _tmp10_;
				_tmp10_ = NULL;
				_vala_code_node_unref0 (stmt);
				stmt = _tmp12_;
				_vala_code_node_unref0 (_tmp10_);
				break;
			}
			case VALA_TOKEN_TYPE_DO:
			{
				ValaStatement* _tmp13_ = NULL;
				ValaStatement* _tmp14_;
				ValaStatement* _tmp15_;
				_tmp14_ = vala_parser_parse_do_statement (self, &_inner_error0_);
				_tmp13_ = _tmp14_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						goto __catch0_vala_parse_error;
					}
					_vala_code_node_unref0 (stmt);
					g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
				_tmp15_ = _tmp13_;
				_tmp13_ = NULL;
				_vala_code_node_unref0 (stmt);
				stmt = _tmp15_;
				_vala_code_node_unref0 (_tmp13_);
				break;
			}
			case VALA_TOKEN_TYPE_FOR:
			{
				ValaStatement* _tmp16_ = NULL;
				ValaStatement* _tmp17_;
				ValaStatement* _tmp18_;
				_tmp17_ = vala_parser_parse_for_statement (self, &_inner_error0_);
				_tmp16_ = _tmp17_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						goto __catch0_vala_parse_error;
					}
					_vala_code_node_unref0 (stmt);
					g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
				_tmp18_ = _tmp16_;
				_tmp16_ = NULL;
				_vala_code_node_unref0 (stmt);
				stmt = _tmp18_;
				_vala_code_node_unref0 (_tmp16_);
				break;
			}
			case VALA_TOKEN_TYPE_FOREACH:
			{
				ValaStatement* _tmp19_ = NULL;
				ValaStatement* _tmp20_;
				ValaStatement* _tmp21_;
				_tmp20_ = vala_parser_parse_foreach_statement (self, &_inner_error0_);
				_tmp19_ = _tmp20_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						goto __catch0_vala_parse_error;
					}
					_vala_code_node_unref0 (stmt);
					g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
				_tmp21_ = _tmp19_;
				_tmp19_ = NULL;
				_vala_code_node_unref0 (stmt);
				stmt = _tmp21_;
				_vala_code_node_unref0 (_tmp19_);
				break;
			}
			case VALA_TOKEN_TYPE_BREAK:
			{
				ValaStatement* _tmp22_ = NULL;
				ValaStatement* _tmp23_;
				ValaStatement* _tmp24_;
				_tmp23_ = vala_parser_parse_break_statement (self, &_inner_error0_);
				_tmp22_ = _tmp23_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						goto __catch0_vala_parse_error;
					}
					_vala_code_node_unref0 (stmt);
					g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
				_tmp24_ = _tmp22_;
				_tmp22_ = NULL;
				_vala_code_node_unref0 (stmt);
				stmt = _tmp24_;
				_vala_code_node_unref0 (_tmp22_);
				break;
			}
			case VALA_TOKEN_TYPE_CONTINUE:
			{
				ValaStatement* _tmp25_ = NULL;
				ValaStatement* _tmp26_;
				ValaStatement* _tmp27_;
				_tmp26_ = vala_parser_parse_continue_statement (self, &_inner_error0_);
				_tmp25_ = _tmp26_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						goto __catch0_vala_parse_error;
					}
					_vala_code_node_unref0 (stmt);
					g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
				_tmp27_ = _tmp25_;
				_tmp25_ = NULL;
				_vala_code_node_unref0 (stmt);
				stmt = _tmp27_;
				_vala_code_node_unref0 (_tmp25_);
				break;
			}
			case VALA_TOKEN_TYPE_RETURN:
			{
				ValaStatement* _tmp28_ = NULL;
				ValaStatement* _tmp29_;
				ValaStatement* _tmp30_;
				_tmp29_ = vala_parser_parse_return_statement (self, &_inner_error0_);
				_tmp28_ = _tmp29_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						goto __catch0_vala_parse_error;
					}
					_vala_code_node_unref0 (stmt);
					g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
				_tmp30_ = _tmp28_;
				_tmp28_ = NULL;
				_vala_code_node_unref0 (stmt);
				stmt = _tmp30_;
				_vala_code_node_unref0 (_tmp28_);
				break;
			}
			case VALA_TOKEN_TYPE_YIELD:
			{
				ValaStatement* _tmp31_ = NULL;
				ValaStatement* _tmp32_;
				ValaStatement* _tmp33_;
				_tmp32_ = vala_parser_parse_yield_statement (self, &_inner_error0_);
				_tmp31_ = _tmp32_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						goto __catch0_vala_parse_error;
					}
					_vala_code_node_unref0 (stmt);
					g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
				_tmp33_ = _tmp31_;
				_tmp31_ = NULL;
				_vala_code_node_unref0 (stmt);
				stmt = _tmp33_;
				_vala_code_node_unref0 (_tmp31_);
				break;
			}
			case VALA_TOKEN_TYPE_THROW:
			{
				ValaStatement* _tmp34_ = NULL;
				ValaStatement* _tmp35_;
				ValaStatement* _tmp36_;
				_tmp35_ = vala_parser_parse_throw_statement (self, &_inner_error0_);
				_tmp34_ = _tmp35_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						goto __catch0_vala_parse_error;
					}
					_vala_code_node_unref0 (stmt);
					g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
				_tmp36_ = _tmp34_;
				_tmp34_ = NULL;
				_vala_code_node_unref0 (stmt);
				stmt = _tmp36_;
				_vala_code_node_unref0 (_tmp34_);
				break;
			}
			case VALA_TOKEN_TYPE_TRY:
			{
				ValaStatement* _tmp37_ = NULL;
				ValaStatement* _tmp38_;
				ValaStatement* _tmp39_;
				_tmp38_ = vala_parser_parse_try_statement (self, &_inner_error0_);
				_tmp37_ = _tmp38_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						goto __catch0_vala_parse_error;
					}
					_vala_code_node_unref0 (stmt);
					g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
				_tmp39_ = _tmp37_;
				_tmp37_ = NULL;
				_vala_code_node_unref0 (stmt);
				stmt = _tmp39_;
				_vala_code_node_unref0 (_tmp37_);
				break;
			}
			case VALA_TOKEN_TYPE_LOCK:
			{
				ValaStatement* _tmp40_ = NULL;
				ValaStatement* _tmp41_;
				ValaStatement* _tmp42_;
				_tmp41_ = vala_parser_parse_lock_statement (self, &_inner_error0_);
				_tmp40_ = _tmp41_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						goto __catch0_vala_parse_error;
					}
					_vala_code_node_unref0 (stmt);
					g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
				_tmp42_ = _tmp40_;
				_tmp40_ = NULL;
				_vala_code_node_unref0 (stmt);
				stmt = _tmp42_;
				_vala_code_node_unref0 (_tmp40_);
				break;
			}
			case VALA_TOKEN_TYPE_UNLOCK:
			{
				ValaStatement* _tmp43_ = NULL;
				ValaStatement* _tmp44_;
				ValaStatement* _tmp45_;
				_tmp44_ = vala_parser_parse_unlock_statement (self, &_inner_error0_);
				_tmp43_ = _tmp44_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						goto __catch0_vala_parse_error;
					}
					_vala_code_node_unref0 (stmt);
					g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
				_tmp45_ = _tmp43_;
				_tmp43_ = NULL;
				_vala_code_node_unref0 (stmt);
				stmt = _tmp45_;
				_vala_code_node_unref0 (_tmp43_);
				break;
			}
			case VALA_TOKEN_TYPE_DELETE:
			{
				ValaStatement* _tmp46_ = NULL;
				ValaStatement* _tmp47_;
				ValaStatement* _tmp48_;
				_tmp47_ = vala_parser_parse_delete_statement (self, &_inner_error0_);
				_tmp46_ = _tmp47_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						goto __catch0_vala_parse_error;
					}
					_vala_code_node_unref0 (stmt);
					g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
				_tmp48_ = _tmp46_;
				_tmp46_ = NULL;
				_vala_code_node_unref0 (stmt);
				stmt = _tmp48_;
				_vala_code_node_unref0 (_tmp46_);
				break;
			}
			case VALA_TOKEN_TYPE_WITH:
			{
				ValaStatement* _tmp49_ = NULL;
				ValaStatement* _tmp50_;
				ValaStatement* _tmp51_;
				_tmp50_ = vala_parser_parse_with_statement (self, &_inner_error0_);
				_tmp49_ = _tmp50_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						goto __catch0_vala_parse_error;
					}
					_vala_code_node_unref0 (stmt);
					g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
				_tmp51_ = _tmp49_;
				_tmp49_ = NULL;
				_vala_code_node_unref0 (stmt);
				stmt = _tmp51_;
				_vala_code_node_unref0 (_tmp49_);
				break;
			}
			default:
			{
				g_assert_not_reached ();
			}
		}
	}
	goto __finally0;
	__catch0_vala_parse_error:
	{
		GError* e = NULL;
		ValaSourceLocation e_begin = {0};
		ValaSourceLocation _tmp52_ = {0};
		gchar* token = NULL;
		GTypeClass* _tmp53_;
		GEnumClass* _tmp54_;
		const gchar* _tmp55_;
		gchar* _tmp56_;
		gchar* _tmp57_;
		ValaSourceLocation _tmp58_;
		gboolean _tmp59_ = FALSE;
		e = _inner_error0_;
		_inner_error0_ = NULL;
		vala_parser_get_location (self, &_tmp52_);
		e_begin = _tmp52_;
		_tmp53_ = g_type_class_ref (VALA_TYPE_TOKEN_TYPE);
		_tmp54_ = (GEnumClass*) _tmp53_;
		_tmp55_ = (*g_enum_get_value (_tmp54_, (gint) type)).value_nick;
		_tmp56_ = g_strdup (_tmp55_);
		_tmp57_ = _tmp56_;
		_g_type_class_unref0 (_tmp54_);
		token = _tmp57_;
		_tmp58_ = begin;
		vala_parser_rollback (self, &_tmp58_);
		_tmp59_ = vala_parser_is_expression (self, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			_g_free0 (token);
			_g_error_free0 (e);
			goto __finally0;
		}
		if (!_tmp59_) {
			ValaSourceLocation _tmp60_;
			GError* _tmp61_;
			GError* _tmp62_;
			_tmp60_ = e_begin;
			vala_parser_rollback (self, &_tmp60_);
			_tmp61_ = e;
			_tmp62_ = _g_error_copy0 (_tmp61_);
			_inner_error0_ = _tmp62_;
			_g_free0 (token);
			_g_error_free0 (e);
			goto __finally0;
		}
		{
			ValaStatement* _tmp63_ = NULL;
			ValaStatement* _tmp64_;
			ValaStatement* _tmp65_;
			ValaSourceLocation _tmp66_;
			ValaSourceReference* _tmp67_;
			ValaSourceReference* _tmp68_;
			const gchar* _tmp69_;
			const gchar* _tmp70_;
			_tmp64_ = vala_parser_parse_expression_statement (self, &_inner_error0_);
			_tmp63_ = _tmp64_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					goto __catch1_vala_parse_error;
				}
				_g_free0 (token);
				_g_error_free0 (e);
				_vala_code_node_unref0 (stmt);
				g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
			_tmp65_ = _tmp63_;
			_tmp63_ = NULL;
			_vala_code_node_unref0 (stmt);
			stmt = _tmp65_;
			_tmp66_ = begin;
			_tmp67_ = vala_parser_get_src (self, &_tmp66_);
			_tmp68_ = _tmp67_;
			_tmp69_ = token;
			_tmp70_ = token;
			vala_report_warning (_tmp68_, "`%s' is a syntax keyword, replace with `@%s'", _tmp69_, _tmp70_);
			_vala_source_reference_unref0 (_tmp68_);
			_vala_code_node_unref0 (_tmp63_);
		}
		goto __finally1;
		__catch1_vala_parse_error:
		{
			GError* e2 = NULL;
			ValaSourceLocation e2_begin = {0};
			ValaSourceLocation _tmp71_ = {0};
			ValaSourceLocation _tmp72_;
			ValaSourceLocation _tmp73_;
			ValaSourceReference* _tmp74_;
			ValaSourceReference* _tmp75_;
			const gchar* _tmp76_;
			GError* _tmp77_;
			const gchar* _tmp78_;
			ValaSourceLocation _tmp79_;
			GError* _tmp80_;
			GError* _tmp81_;
			e2 = _inner_error0_;
			_inner_error0_ = NULL;
			vala_parser_get_location (self, &_tmp71_);
			e2_begin = _tmp71_;
			_tmp72_ = e_begin;
			vala_parser_rollback (self, &_tmp72_);
			vala_parser_next (self);
			_tmp73_ = e_begin;
			_tmp74_ = vala_parser_get_src (self, &_tmp73_);
			_tmp75_ = _tmp74_;
			_tmp76_ = token;
			_tmp77_ = e;
			_tmp78_ = _tmp77_->message;
			vala_report_error (_tmp75_, "Possible `%s-statement' syntax error, %s", _tmp76_, _tmp78_);
			_vala_source_reference_unref0 (_tmp75_);
			_tmp79_ = e2_begin;
			vala_parser_rollback (self, &_tmp79_);
			_tmp80_ = e2;
			_tmp81_ = _g_error_copy0 (_tmp80_);
			_inner_error0_ = _tmp81_;
			_g_error_free0 (e2);
			goto __finally1;
		}
		__finally1:
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			_g_free0 (token);
			_g_error_free0 (e);
			goto __finally0;
		}
		_g_free0 (token);
		_g_error_free0 (e);
	}
	__finally0:
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (stmt);
			return NULL;
		} else {
			_vala_code_node_unref0 (stmt);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	result = stmt;
	return result;
}

static void
vala_parser_parse_statements (ValaParser* self,
                              ValaBlock* block,
                              GError** error)
{
	GError* _inner_error0_ = NULL;
	g_return_if_fail (self != NULL);
	g_return_if_fail (block != NULL);
	while (TRUE) {
		gboolean _tmp0_ = FALSE;
		if (vala_parser_current (self) != VALA_TOKEN_TYPE_CLOSE_BRACE) {
			_tmp0_ = vala_parser_current (self) != VALA_TOKEN_TYPE_EOF;
		} else {
			_tmp0_ = FALSE;
		}
		if (!_tmp0_) {
			break;
		}
		{
			ValaStatement* stmt = NULL;
			gboolean is_decl = FALSE;
			ValaScanner* _tmp1_;
			ValaComment* _tmp2_;
			stmt = NULL;
			is_decl = FALSE;
			_tmp1_ = self->priv->scanner;
			_tmp2_ = vala_scanner_pop_comment (_tmp1_);
			_vala_comment_unref0 (self->priv->comment);
			self->priv->comment = _tmp2_;
			switch (vala_parser_current (self)) {
				case VALA_TOKEN_TYPE_OPEN_BRACE:
				{
					ValaBlock* _tmp3_ = NULL;
					ValaBlock* _tmp4_;
					ValaBlock* _tmp5_;
					_tmp4_ = vala_parser_parse_block (self, &_inner_error0_);
					_tmp3_ = _tmp4_;
					if (G_UNLIKELY (_inner_error0_ != NULL)) {
						_vala_code_node_unref0 (stmt);
						if (_inner_error0_->domain == VALA_PARSE_ERROR) {
							goto __catch0_vala_parse_error;
						}
						g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return;
					}
					_tmp5_ = _tmp3_;
					_tmp3_ = NULL;
					_vala_code_node_unref0 (stmt);
					stmt = (ValaStatement*) _tmp5_;
					_vala_code_node_unref0 (_tmp3_);
					break;
				}
				case VALA_TOKEN_TYPE_SEMICOLON:
				{
					ValaStatement* _tmp6_ = NULL;
					ValaStatement* _tmp7_;
					ValaStatement* _tmp8_;
					_tmp7_ = vala_parser_parse_empty_statement (self, &_inner_error0_);
					_tmp6_ = _tmp7_;
					if (G_UNLIKELY (_inner_error0_ != NULL)) {
						_vala_code_node_unref0 (stmt);
						if (_inner_error0_->domain == VALA_PARSE_ERROR) {
							goto __catch0_vala_parse_error;
						}
						g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return;
					}
					_tmp8_ = _tmp6_;
					_tmp6_ = NULL;
					_vala_code_node_unref0 (stmt);
					stmt = _tmp8_;
					_vala_code_node_unref0 (_tmp6_);
					break;
				}
				case VALA_TOKEN_TYPE_IF:
				case VALA_TOKEN_TYPE_SWITCH:
				case VALA_TOKEN_TYPE_WHILE:
				case VALA_TOKEN_TYPE_DO:
				case VALA_TOKEN_TYPE_FOR:
				case VALA_TOKEN_TYPE_FOREACH:
				case VALA_TOKEN_TYPE_BREAK:
				case VALA_TOKEN_TYPE_CONTINUE:
				case VALA_TOKEN_TYPE_RETURN:
				case VALA_TOKEN_TYPE_YIELD:
				case VALA_TOKEN_TYPE_THROW:
				case VALA_TOKEN_TYPE_TRY:
				case VALA_TOKEN_TYPE_LOCK:
				case VALA_TOKEN_TYPE_UNLOCK:
				case VALA_TOKEN_TYPE_DELETE:
				case VALA_TOKEN_TYPE_WITH:
				{
					ValaStatement* _tmp9_ = NULL;
					ValaStatement* _tmp10_;
					ValaStatement* _tmp11_;
					_tmp10_ = vala_parser_parse_statement (self, vala_parser_current (self), &_inner_error0_);
					_tmp9_ = _tmp10_;
					if (G_UNLIKELY (_inner_error0_ != NULL)) {
						_vala_code_node_unref0 (stmt);
						if (_inner_error0_->domain == VALA_PARSE_ERROR) {
							goto __catch0_vala_parse_error;
						}
						g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return;
					}
					_tmp11_ = _tmp9_;
					_tmp9_ = NULL;
					_vala_code_node_unref0 (stmt);
					stmt = _tmp11_;
					_vala_code_node_unref0 (_tmp9_);
					break;
				}
				case VALA_TOKEN_TYPE_CASE:
				case VALA_TOKEN_TYPE_DEFAULT:
				{
					if (VALA_IS_SWITCH_SECTION (block)) {
						_vala_code_node_unref0 (stmt);
						return;
					} else {
						ValaStatement* _tmp12_ = NULL;
						ValaStatement* _tmp13_;
						ValaStatement* _tmp14_;
						_tmp13_ = vala_parser_parse_statement (self, vala_parser_current (self), &_inner_error0_);
						_tmp12_ = _tmp13_;
						if (G_UNLIKELY (_inner_error0_ != NULL)) {
							_vala_code_node_unref0 (stmt);
							if (_inner_error0_->domain == VALA_PARSE_ERROR) {
								goto __catch0_vala_parse_error;
							}
							g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
							g_clear_error (&_inner_error0_);
							return;
						}
						_tmp14_ = _tmp12_;
						_tmp12_ = NULL;
						_vala_code_node_unref0 (stmt);
						stmt = _tmp14_;
						_vala_code_node_unref0 (_tmp12_);
					}
					break;
				}
				case VALA_TOKEN_TYPE_VAR:
				{
					is_decl = TRUE;
					vala_parser_parse_local_variable_declarations (self, block, &_inner_error0_);
					if (G_UNLIKELY (_inner_error0_ != NULL)) {
						_vala_code_node_unref0 (stmt);
						if (_inner_error0_->domain == VALA_PARSE_ERROR) {
							goto __catch0_vala_parse_error;
						}
						g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return;
					}
					break;
				}
				case VALA_TOKEN_TYPE_CONST:
				{
					is_decl = TRUE;
					vala_parser_parse_local_constant_declarations (self, block, &_inner_error0_);
					if (G_UNLIKELY (_inner_error0_ != NULL)) {
						_vala_code_node_unref0 (stmt);
						if (_inner_error0_->domain == VALA_PARSE_ERROR) {
							goto __catch0_vala_parse_error;
						}
						g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return;
					}
					break;
				}
				case VALA_TOKEN_TYPE_OP_INC:
				case VALA_TOKEN_TYPE_OP_DEC:
				case VALA_TOKEN_TYPE_BASE:
				case VALA_TOKEN_TYPE_THIS:
				case VALA_TOKEN_TYPE_STAR:
				case VALA_TOKEN_TYPE_NEW:
				{
					ValaStatement* _tmp15_ = NULL;
					ValaStatement* _tmp16_;
					ValaStatement* _tmp17_;
					_tmp16_ = vala_parser_parse_expression_statement (self, &_inner_error0_);
					_tmp15_ = _tmp16_;
					if (G_UNLIKELY (_inner_error0_ != NULL)) {
						_vala_code_node_unref0 (stmt);
						if (_inner_error0_->domain == VALA_PARSE_ERROR) {
							goto __catch0_vala_parse_error;
						}
						g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return;
					}
					_tmp17_ = _tmp15_;
					_tmp15_ = NULL;
					_vala_code_node_unref0 (stmt);
					stmt = _tmp17_;
					_vala_code_node_unref0 (_tmp15_);
					break;
				}
				default:
				{
					gboolean is_expr = FALSE;
					is_expr = vala_parser_is_expression (self, &_inner_error0_);
					if (G_UNLIKELY (_inner_error0_ != NULL)) {
						_vala_code_node_unref0 (stmt);
						if (_inner_error0_->domain == VALA_PARSE_ERROR) {
							goto __catch0_vala_parse_error;
						}
						g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return;
					}
					if (is_expr) {
						ValaStatement* _tmp18_ = NULL;
						ValaStatement* _tmp19_;
						ValaStatement* _tmp20_;
						_tmp19_ = vala_parser_parse_expression_statement (self, &_inner_error0_);
						_tmp18_ = _tmp19_;
						if (G_UNLIKELY (_inner_error0_ != NULL)) {
							_vala_code_node_unref0 (stmt);
							if (_inner_error0_->domain == VALA_PARSE_ERROR) {
								goto __catch0_vala_parse_error;
							}
							g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
							g_clear_error (&_inner_error0_);
							return;
						}
						_tmp20_ = _tmp18_;
						_tmp18_ = NULL;
						_vala_code_node_unref0 (stmt);
						stmt = _tmp20_;
						_vala_code_node_unref0 (_tmp18_);
					} else {
						gboolean is_local_func = FALSE;
						is_decl = TRUE;
						is_local_func = vala_parser_is_local_function (self);
						if (is_local_func) {
							vala_parser_parse_local_function_declaration (self, block, &_inner_error0_);
							if (G_UNLIKELY (_inner_error0_ != NULL)) {
								_vala_code_node_unref0 (stmt);
								if (_inner_error0_->domain == VALA_PARSE_ERROR) {
									goto __catch0_vala_parse_error;
								}
								g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
								g_clear_error (&_inner_error0_);
								return;
							}
						} else {
							vala_parser_parse_local_variable_declarations (self, block, &_inner_error0_);
							if (G_UNLIKELY (_inner_error0_ != NULL)) {
								_vala_code_node_unref0 (stmt);
								if (_inner_error0_->domain == VALA_PARSE_ERROR) {
									goto __catch0_vala_parse_error;
								}
								g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
								g_clear_error (&_inner_error0_);
								return;
							}
						}
					}
					break;
				}
			}
			if (!is_decl) {
				ValaStatement* _tmp21_;
				_tmp21_ = stmt;
				vala_block_add_statement (block, _tmp21_);
			}
			_vala_code_node_unref0 (stmt);
		}
		goto __finally0;
		__catch0_vala_parse_error:
		{
			GError* e = NULL;
			GError* _tmp22_;
			e = _inner_error0_;
			_inner_error0_ = NULL;
			_tmp22_ = e;
			vala_parser_report_parse_error (self, _tmp22_);
			if (vala_parser_recover (self) != VALA_PARSER_RECOVERY_STATE_STATEMENT_BEGIN) {
				_g_error_free0 (e);
				break;
			}
			_g_error_free0 (e);
		}
		__finally0:
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				return;
			} else {
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
	}
}

static gboolean
vala_parser_is_expression (ValaParser* self,
                           GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	GError* _inner_error0_ = NULL;
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	if (vala_parser_current (self) == VALA_TOKEN_TYPE_OPEN_PARENS) {
		result = !vala_parser_is_inner_array_type (self);
		return result;
	}
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	{
		vala_parser_skip_type (self, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			gboolean _tmp1_ = FALSE;
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				goto __catch0_vala_parse_error;
			}
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return _tmp1_;
		}
	}
	goto __finally0;
	__catch0_vala_parse_error:
	{
		GError* e = NULL;
		ValaTokenType token = 0;
		gboolean _tmp2_ = FALSE;
		e = _inner_error0_;
		_inner_error0_ = NULL;
		token = vala_parser_previous (self);
		if (token == VALA_TOKEN_TYPE_DOT) {
			_tmp2_ = TRUE;
		} else {
			_tmp2_ = token == VALA_TOKEN_TYPE_DOUBLE_COLON;
		}
		if (_tmp2_) {
			ValaSourceLocation _tmp3_;
			_tmp3_ = begin;
			vala_parser_rollback (self, &_tmp3_);
			result = TRUE;
			_g_error_free0 (e);
			return result;
		} else {
			GError* _tmp4_;
			GError* _tmp5_;
			_tmp4_ = e;
			_tmp5_ = _g_error_copy0 (_tmp4_);
			_inner_error0_ = _tmp5_;
			_g_error_free0 (e);
			goto __finally0;
		}
		_g_error_free0 (e);
	}
	__finally0:
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			gboolean _tmp6_ = FALSE;
			g_propagate_error (error, _inner_error0_);
			return _tmp6_;
		} else {
			gboolean _tmp7_ = FALSE;
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return _tmp7_;
		}
	}
	switch (vala_parser_current (self)) {
		case VALA_TOKEN_TYPE_OPEN_PARENS:
		case VALA_TOKEN_TYPE_OP_INC:
		case VALA_TOKEN_TYPE_OP_DEC:
		case VALA_TOKEN_TYPE_ASSIGN:
		case VALA_TOKEN_TYPE_ASSIGN_ADD:
		case VALA_TOKEN_TYPE_ASSIGN_BITWISE_AND:
		case VALA_TOKEN_TYPE_ASSIGN_BITWISE_OR:
		case VALA_TOKEN_TYPE_ASSIGN_BITWISE_XOR:
		case VALA_TOKEN_TYPE_ASSIGN_DIV:
		case VALA_TOKEN_TYPE_ASSIGN_MUL:
		case VALA_TOKEN_TYPE_ASSIGN_PERCENT:
		case VALA_TOKEN_TYPE_ASSIGN_SHIFT_LEFT:
		case VALA_TOKEN_TYPE_ASSIGN_SUB:
		case VALA_TOKEN_TYPE_OP_GT:
		case VALA_TOKEN_TYPE_DOT:
		case VALA_TOKEN_TYPE_OP_PTR:
		case VALA_TOKEN_TYPE_INTEGER_LITERAL:
		case VALA_TOKEN_TYPE_REAL_LITERAL:
		{
			ValaSourceLocation _tmp8_;
			_tmp8_ = begin;
			vala_parser_rollback (self, &_tmp8_);
			result = TRUE;
			return result;
		}
		default:
		{
			ValaSourceLocation _tmp9_;
			_tmp9_ = begin;
			vala_parser_rollback (self, &_tmp9_);
			result = FALSE;
			return result;
		}
	}
}

static gboolean
vala_parser_is_lambda_expression (ValaParser* self)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaSourceLocation _tmp9_;
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	switch (vala_parser_current (self)) {
		case VALA_TOKEN_TYPE_OUT:
		case VALA_TOKEN_TYPE_REF:
		{
			gboolean _tmp1_ = FALSE;
			vala_parser_next (self);
			if (vala_parser_accept (self, VALA_TOKEN_TYPE_IDENTIFIER)) {
				_tmp1_ = vala_parser_accept (self, VALA_TOKEN_TYPE_LAMBDA);
			} else {
				_tmp1_ = FALSE;
			}
			if (_tmp1_) {
				ValaSourceLocation _tmp2_;
				_tmp2_ = begin;
				vala_parser_rollback (self, &_tmp2_);
				result = TRUE;
				return result;
			}
			break;
		}
		case VALA_TOKEN_TYPE_IDENTIFIER:
		{
			vala_parser_next (self);
			if (vala_parser_accept (self, VALA_TOKEN_TYPE_LAMBDA)) {
				ValaSourceLocation _tmp3_;
				_tmp3_ = begin;
				vala_parser_rollback (self, &_tmp3_);
				result = TRUE;
				return result;
			}
			break;
		}
		case VALA_TOKEN_TYPE_OPEN_PARENS:
		{
			gboolean _tmp7_ = FALSE;
			vala_parser_next (self);
			if (vala_parser_current (self) != VALA_TOKEN_TYPE_CLOSE_PARENS) {
				{
					gboolean _tmp4_ = FALSE;
					_tmp4_ = TRUE;
					while (TRUE) {
						gboolean _tmp5_ = FALSE;
						if (!_tmp4_) {
							if (!vala_parser_accept (self, VALA_TOKEN_TYPE_COMMA)) {
								break;
							}
						}
						_tmp4_ = FALSE;
						if (vala_parser_current (self) == VALA_TOKEN_TYPE_OUT) {
							_tmp5_ = TRUE;
						} else {
							_tmp5_ = vala_parser_current (self) == VALA_TOKEN_TYPE_REF;
						}
						if (_tmp5_) {
							vala_parser_next (self);
						}
						if (!vala_parser_accept (self, VALA_TOKEN_TYPE_IDENTIFIER)) {
							ValaSourceLocation _tmp6_;
							_tmp6_ = begin;
							vala_parser_rollback (self, &_tmp6_);
							result = FALSE;
							return result;
						}
					}
				}
			}
			if (vala_parser_accept (self, VALA_TOKEN_TYPE_CLOSE_PARENS)) {
				_tmp7_ = vala_parser_accept (self, VALA_TOKEN_TYPE_LAMBDA);
			} else {
				_tmp7_ = FALSE;
			}
			if (_tmp7_) {
				ValaSourceLocation _tmp8_;
				_tmp8_ = begin;
				vala_parser_rollback (self, &_tmp8_);
				result = TRUE;
				return result;
			}
			break;
		}
		default:
		{
			break;
		}
	}
	_tmp9_ = begin;
	vala_parser_rollback (self, &_tmp9_);
	result = FALSE;
	return result;
}

static gboolean
vala_parser_is_local_function (ValaParser* self)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaSourceLocation _tmp4_;
	GError* _inner_error0_ = NULL;
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	{
		gboolean _tmp1_ = FALSE;
		vala_parser_accept (self, VALA_TOKEN_TYPE_STATIC);
		vala_parser_skip_type (self, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			goto __catch0_g_error;
		}
		if (vala_parser_accept (self, VALA_TOKEN_TYPE_IDENTIFIER)) {
			_tmp1_ = vala_parser_accept (self, VALA_TOKEN_TYPE_OPEN_PARENS);
		} else {
			_tmp1_ = FALSE;
		}
		if (_tmp1_) {
			ValaSourceLocation _tmp2_;
			_tmp2_ = begin;
			vala_parser_rollback (self, &_tmp2_);
			result = TRUE;
			return result;
		}
	}
	goto __finally0;
	__catch0_g_error:
	{
		g_clear_error (&_inner_error0_);
	}
	__finally0:
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		gboolean _tmp3_ = FALSE;
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
		g_clear_error (&_inner_error0_);
		return _tmp3_;
	}
	_tmp4_ = begin;
	vala_parser_rollback (self, &_tmp4_);
	result = FALSE;
	return result;
}

static ValaBlock*
vala_parser_parse_embedded_statement (ValaParser* self,
                                      const gchar* statement_name,
                                      gboolean accept_empty_body,
                                      GError** error)
{
	ValaScanner* _tmp1_;
	ValaComment* _tmp2_;
	ValaBlock* block = NULL;
	ValaSourceLocation _tmp3_ = {0};
	ValaSourceReference* _tmp4_;
	ValaSourceReference* _tmp5_;
	ValaBlock* _tmp6_;
	ValaBlock* _tmp7_;
	ValaBlock* _tmp17_;
	ValaSourceReference* _tmp18_;
	ValaSourceReference* _tmp19_;
	ValaSourceReference* _tmp20_;
	ValaSourceReference* _tmp21_;
	ValaSourceLocation _tmp22_ = {0};
	ValaSourceLocation _tmp23_;
	GError* _inner_error0_ = NULL;
	ValaBlock* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (statement_name != NULL, NULL);
	if (vala_parser_current (self) == VALA_TOKEN_TYPE_OPEN_BRACE) {
		ValaBlock* block = NULL;
		ValaBlock* _tmp0_;
		_tmp0_ = vala_parser_parse_block (self, &_inner_error0_);
		block = _tmp0_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				return NULL;
			} else {
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		result = block;
		return result;
	}
	_tmp1_ = self->priv->scanner;
	_tmp2_ = vala_scanner_pop_comment (_tmp1_);
	_vala_comment_unref0 (self->priv->comment);
	self->priv->comment = _tmp2_;
	vala_parser_get_location (self, &_tmp3_);
	_tmp4_ = vala_parser_get_src (self, &_tmp3_);
	_tmp5_ = _tmp4_;
	_tmp6_ = vala_block_new (_tmp5_);
	_tmp7_ = _tmp6_;
	_vala_source_reference_unref0 (_tmp5_);
	block = _tmp7_;
	{
		ValaStatement* stmt = NULL;
		ValaStatement* _tmp8_;
		ValaBlock* _tmp9_;
		ValaStatement* _tmp10_;
		_tmp8_ = vala_parser_parse_embedded_statement_without_block (self, statement_name, accept_empty_body, &_inner_error0_);
		stmt = _tmp8_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				goto __catch0_vala_parse_error;
			}
			_vala_code_node_unref0 (block);
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
		_tmp9_ = block;
		_tmp10_ = stmt;
		vala_block_add_statement (_tmp9_, _tmp10_);
		_vala_code_node_unref0 (stmt);
	}
	goto __finally0;
	__catch0_vala_parse_error:
	{
		GError* e = NULL;
		ValaCodeContext* _tmp11_;
		gboolean _tmp12_;
		gboolean _tmp13_;
		e = _inner_error0_;
		_inner_error0_ = NULL;
		_tmp11_ = self->priv->context;
		_tmp12_ = vala_code_context_get_keep_going (_tmp11_);
		_tmp13_ = _tmp12_;
		if (_tmp13_) {
			GError* _tmp14_;
			_tmp14_ = e;
			vala_parser_report_parse_error (self, _tmp14_);
		} else {
			GError* _tmp15_;
			GError* _tmp16_;
			_tmp15_ = e;
			_tmp16_ = _g_error_copy0 (_tmp15_);
			_inner_error0_ = _tmp16_;
			_g_error_free0 (e);
			goto __finally0;
		}
		_g_error_free0 (e);
	}
	__finally0:
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (block);
			return NULL;
		} else {
			_vala_code_node_unref0 (block);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp17_ = block;
	_tmp18_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp17_);
	_tmp19_ = _tmp18_;
	_tmp20_ = vala_parser_get_last_src (self);
	_tmp21_ = _tmp20_;
	vala_source_reference_get_end (_tmp21_, &_tmp22_);
	_tmp23_ = _tmp22_;
	vala_source_reference_set_end (_tmp19_, &_tmp23_);
	_vala_source_reference_unref0 (_tmp21_);
	result = block;
	return result;
}

static ValaStatement*
vala_parser_parse_embedded_statement_without_block (ValaParser* self,
                                                    const gchar* statement_name,
                                                    gboolean accept_empty_body,
                                                    GError** error)
{
	GError* _inner_error0_ = NULL;
	ValaStatement* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (statement_name != NULL, NULL);
	switch (vala_parser_current (self)) {
		case VALA_TOKEN_TYPE_SEMICOLON:
		{
			ValaStatement* _tmp2_ = NULL;
			ValaStatement* _tmp3_;
			ValaStatement* _tmp4_;
			if (!accept_empty_body) {
				ValaSourceReference* _tmp0_;
				ValaSourceReference* _tmp1_;
				_tmp0_ = vala_parser_get_current_src (self);
				_tmp1_ = _tmp0_;
				vala_report_warning (_tmp1_, "%s-statement without body", statement_name);
				_vala_source_reference_unref0 (_tmp1_);
			}
			_tmp3_ = vala_parser_parse_empty_statement (self, &_inner_error0_);
			_tmp2_ = _tmp3_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					return NULL;
				} else {
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp4_ = _tmp2_;
			_tmp2_ = NULL;
			result = _tmp4_;
			_vala_code_node_unref0 (_tmp2_);
			return result;
		}
		case VALA_TOKEN_TYPE_IF:
		case VALA_TOKEN_TYPE_SWITCH:
		case VALA_TOKEN_TYPE_WHILE:
		case VALA_TOKEN_TYPE_DO:
		case VALA_TOKEN_TYPE_FOR:
		case VALA_TOKEN_TYPE_FOREACH:
		case VALA_TOKEN_TYPE_BREAK:
		case VALA_TOKEN_TYPE_CONTINUE:
		case VALA_TOKEN_TYPE_RETURN:
		case VALA_TOKEN_TYPE_YIELD:
		case VALA_TOKEN_TYPE_THROW:
		case VALA_TOKEN_TYPE_TRY:
		case VALA_TOKEN_TYPE_LOCK:
		case VALA_TOKEN_TYPE_UNLOCK:
		case VALA_TOKEN_TYPE_DELETE:
		case VALA_TOKEN_TYPE_WITH:
		{
			ValaStatement* _tmp5_ = NULL;
			ValaStatement* _tmp6_;
			ValaStatement* _tmp7_;
			_tmp6_ = vala_parser_parse_statement (self, vala_parser_current (self), &_inner_error0_);
			_tmp5_ = _tmp6_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					return NULL;
				} else {
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp7_ = _tmp5_;
			_tmp5_ = NULL;
			result = _tmp7_;
			_vala_code_node_unref0 (_tmp5_);
			return result;
		}
		case VALA_TOKEN_TYPE_VAR:
		case VALA_TOKEN_TYPE_CONST:
		{
			GError* _tmp8_;
			_tmp8_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "embedded statement cannot be declaration ");
			_inner_error0_ = _tmp8_;
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				return NULL;
			} else {
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		case VALA_TOKEN_TYPE_OP_INC:
		case VALA_TOKEN_TYPE_OP_DEC:
		case VALA_TOKEN_TYPE_BASE:
		case VALA_TOKEN_TYPE_THIS:
		case VALA_TOKEN_TYPE_OPEN_PARENS:
		case VALA_TOKEN_TYPE_STAR:
		case VALA_TOKEN_TYPE_NEW:
		{
			ValaStatement* _tmp9_ = NULL;
			ValaStatement* _tmp10_;
			ValaStatement* _tmp11_;
			_tmp10_ = vala_parser_parse_expression_statement (self, &_inner_error0_);
			_tmp9_ = _tmp10_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					return NULL;
				} else {
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp11_ = _tmp9_;
			_tmp9_ = NULL;
			result = _tmp11_;
			_vala_code_node_unref0 (_tmp9_);
			return result;
		}
		default:
		{
			gboolean _tmp12_ = FALSE;
			_tmp12_ = vala_parser_is_expression (self, &_inner_error0_);
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					return NULL;
				} else {
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			if (_tmp12_) {
				ValaStatement* _tmp13_ = NULL;
				ValaStatement* _tmp14_;
				ValaStatement* _tmp15_;
				_tmp14_ = vala_parser_parse_expression_statement (self, &_inner_error0_);
				_tmp13_ = _tmp14_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						return NULL;
					} else {
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return NULL;
					}
				}
				_tmp15_ = _tmp13_;
				_tmp13_ = NULL;
				result = _tmp15_;
				_vala_code_node_unref0 (_tmp13_);
				return result;
			} else {
				GError* _tmp16_;
				_tmp16_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "embedded statement cannot be declaration");
				_inner_error0_ = _tmp16_;
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					return NULL;
				} else {
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
		}
	}
}

static ValaBlock*
vala_parser_parse_block (ValaParser* self,
                         GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaBlock* block = NULL;
	ValaSourceLocation _tmp1_;
	ValaSourceReference* _tmp2_;
	ValaSourceReference* _tmp3_;
	ValaBlock* _tmp4_;
	ValaBlock* _tmp5_;
	ValaBlock* _tmp6_;
	ValaBlock* _tmp12_;
	ValaSourceReference* _tmp13_;
	ValaSourceReference* _tmp14_;
	ValaSourceReference* _tmp15_;
	ValaSourceReference* _tmp16_;
	ValaSourceLocation _tmp17_ = {0};
	ValaSourceLocation _tmp18_;
	GError* _inner_error0_ = NULL;
	ValaBlock* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_BRACE, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp1_ = begin;
	_tmp2_ = vala_parser_get_src (self, &_tmp1_);
	_tmp3_ = _tmp2_;
	_tmp4_ = vala_block_new (_tmp3_);
	_tmp5_ = _tmp4_;
	_vala_source_reference_unref0 (_tmp3_);
	block = _tmp5_;
	_tmp6_ = block;
	vala_parser_parse_statements (self, _tmp6_, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (block);
			return NULL;
		} else {
			_vala_code_node_unref0 (block);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	if (!vala_parser_accept (self, VALA_TOKEN_TYPE_CLOSE_BRACE)) {
		ValaCodeContext* _tmp7_;
		ValaReport* _tmp8_;
		ValaReport* _tmp9_;
		_tmp7_ = self->priv->context;
		_tmp8_ = vala_code_context_get_report (_tmp7_);
		_tmp9_ = _tmp8_;
		if (vala_report_get_errors (_tmp9_) == 0) {
			ValaSourceReference* _tmp10_;
			ValaSourceReference* _tmp11_;
			_tmp10_ = vala_parser_get_current_src (self);
			_tmp11_ = _tmp10_;
			vala_report_error (_tmp11_, "expected `}'");
			_vala_source_reference_unref0 (_tmp11_);
		}
	}
	_tmp12_ = block;
	_tmp13_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp12_);
	_tmp14_ = _tmp13_;
	_tmp15_ = vala_parser_get_last_src (self);
	_tmp16_ = _tmp15_;
	vala_source_reference_get_end (_tmp16_, &_tmp17_);
	_tmp18_ = _tmp17_;
	vala_source_reference_set_end (_tmp14_, &_tmp18_);
	_vala_source_reference_unref0 (_tmp16_);
	result = block;
	return result;
}

static ValaStatement*
vala_parser_parse_empty_statement (ValaParser* self,
                                   GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaSourceLocation _tmp1_;
	ValaSourceReference* _tmp2_;
	ValaSourceReference* _tmp3_;
	ValaEmptyStatement* _tmp4_;
	ValaStatement* _tmp5_;
	GError* _inner_error0_ = NULL;
	ValaStatement* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_SEMICOLON, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp1_ = begin;
	_tmp2_ = vala_parser_get_src (self, &_tmp1_);
	_tmp3_ = _tmp2_;
	_tmp4_ = vala_empty_statement_new (_tmp3_);
	_tmp5_ = (ValaStatement*) _tmp4_;
	_vala_source_reference_unref0 (_tmp3_);
	result = _tmp5_;
	return result;
}

static void
_vala_array_add13 (gchar** * array,
                   gint* length,
                   gint* size,
                   gchar* value)
{
	if ((*length) == (*size)) {
		*size = (*size) ? (2 * (*size)) : 4;
		*array = g_renew (gchar*, *array, (*size) + 1);
	}
	(*array)[(*length)++] = value;
	(*array)[*length] = NULL;
}

static void
vala_parser_parse_local_variable_declarations (ValaParser* self,
                                               ValaBlock* block,
                                               GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaDataType* variable_type = NULL;
	gboolean is_dynamic = FALSE;
	gboolean _tmp1_ = FALSE;
	gboolean is_first = FALSE;
	GError* _inner_error0_ = NULL;
	g_return_if_fail (self != NULL);
	g_return_if_fail (block != NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	is_dynamic = vala_parser_accept (self, VALA_TOKEN_TYPE_DYNAMIC);
	if (vala_parser_accept (self, VALA_TOKEN_TYPE_UNOWNED)) {
		_tmp1_ = vala_parser_accept (self, VALA_TOKEN_TYPE_VAR);
	} else {
		_tmp1_ = FALSE;
	}
	if (_tmp1_) {
		ValaVarType* _tmp2_;
		ValaDataType* _tmp3_;
		ValaDataType* _tmp4_;
		_tmp2_ = vala_var_type_new (FALSE);
		_vala_code_node_unref0 (variable_type);
		variable_type = (ValaDataType*) _tmp2_;
		_tmp3_ = variable_type;
		vala_data_type_set_nullable (_tmp3_, vala_parser_accept (self, VALA_TOKEN_TYPE_INTERR));
		_tmp4_ = variable_type;
		vala_data_type_set_is_dynamic (_tmp4_, is_dynamic);
	} else {
		ValaSourceLocation _tmp5_;
		_tmp5_ = begin;
		vala_parser_rollback (self, &_tmp5_);
		is_dynamic = vala_parser_accept (self, VALA_TOKEN_TYPE_DYNAMIC);
		if (vala_parser_accept (self, VALA_TOKEN_TYPE_VAR)) {
			ValaVarType* _tmp6_;
			ValaDataType* _tmp7_;
			ValaDataType* _tmp8_;
			_tmp6_ = vala_var_type_new (TRUE);
			_vala_code_node_unref0 (variable_type);
			variable_type = (ValaDataType*) _tmp6_;
			_tmp7_ = variable_type;
			vala_data_type_set_nullable (_tmp7_, vala_parser_accept (self, VALA_TOKEN_TYPE_INTERR));
			_tmp8_ = variable_type;
			vala_data_type_set_is_dynamic (_tmp8_, is_dynamic);
		} else {
			ValaSourceLocation _tmp9_;
			ValaDataType* _tmp10_ = NULL;
			ValaDataType* _tmp11_;
			ValaDataType* _tmp12_;
			_tmp9_ = begin;
			vala_parser_rollback (self, &_tmp9_);
			_tmp11_ = vala_parser_parse_type (self, TRUE, TRUE, FALSE, &_inner_error0_);
			_tmp10_ = _tmp11_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (variable_type);
					return;
				} else {
					_vala_code_node_unref0 (variable_type);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return;
				}
			}
			_tmp12_ = _tmp10_;
			_tmp10_ = NULL;
			_vala_code_node_unref0 (variable_type);
			variable_type = _tmp12_;
			_vala_code_node_unref0 (_tmp10_);
		}
	}
	is_first = TRUE;
	{
		gboolean _tmp13_ = FALSE;
		_tmp13_ = TRUE;
		while (TRUE) {
			gboolean _tmp14_ = FALSE;
			gboolean _tmp15_ = FALSE;
			ValaDataType* _tmp16_;
			ValaDataType* type_copy = NULL;
			ValaDataType* _tmp75_;
			ValaLocalVariable* local = NULL;
			ValaDataType* _tmp78_;
			ValaLocalVariable* _tmp79_;
			ValaLocalVariable* _tmp80_;
			ValaSourceLocation _tmp81_;
			ValaSourceReference* _tmp82_;
			ValaSourceReference* _tmp83_;
			ValaDeclarationStatement* _tmp84_;
			ValaDeclarationStatement* _tmp85_;
			if (!_tmp13_) {
				if (!vala_parser_accept (self, VALA_TOKEN_TYPE_COMMA)) {
					break;
				}
			}
			_tmp13_ = FALSE;
			_tmp16_ = variable_type;
			if (VALA_IS_VAR_TYPE (_tmp16_)) {
				ValaDataType* _tmp17_;
				gboolean _tmp18_;
				gboolean _tmp19_;
				_tmp17_ = variable_type;
				_tmp18_ = vala_data_type_get_value_owned (_tmp17_);
				_tmp19_ = _tmp18_;
				_tmp15_ = _tmp19_;
			} else {
				_tmp15_ = FALSE;
			}
			if (_tmp15_) {
				_tmp14_ = vala_parser_accept (self, VALA_TOKEN_TYPE_OPEN_PARENS);
			} else {
				_tmp14_ = FALSE;
			}
			if (_tmp14_) {
				ValaSourceLocation _tmp20_ = {0};
				gchar** identifiers = NULL;
				gchar** _tmp21_;
				gint identifiers_length1;
				gint _identifiers_size_;
				ValaExpression* tuple = NULL;
				ValaExpression* _tmp26_;
				ValaLocalVariable* tuple_local = NULL;
				gchar* _tmp27_;
				gchar* _tmp28_;
				ValaExpression* _tmp29_;
				ValaSourceLocation _tmp30_;
				ValaSourceReference* _tmp31_;
				ValaSourceReference* _tmp32_;
				ValaLocalVariable* _tmp33_;
				ValaLocalVariable* _tmp34_;
				ValaLocalVariable* _tmp35_;
				ValaLocalVariable* _tmp36_;
				ValaSourceReference* _tmp37_;
				ValaSourceReference* _tmp38_;
				ValaDeclarationStatement* _tmp39_;
				ValaDeclarationStatement* _tmp40_;
				vala_parser_get_location (self, &_tmp20_);
				begin = _tmp20_;
				_tmp21_ = g_new0 (gchar*, 0 + 1);
				identifiers = _tmp21_;
				identifiers_length1 = 0;
				_identifiers_size_ = identifiers_length1;
				{
					gboolean _tmp22_ = FALSE;
					_tmp22_ = TRUE;
					while (TRUE) {
						gchar* _tmp23_ = NULL;
						gchar* _tmp24_;
						gchar* _tmp25_;
						if (!_tmp22_) {
							if (!vala_parser_accept (self, VALA_TOKEN_TYPE_COMMA)) {
								break;
							}
						}
						_tmp22_ = FALSE;
						_tmp24_ = vala_parser_parse_identifier (self, FALSE, &_inner_error0_);
						_tmp23_ = _tmp24_;
						if (G_UNLIKELY (_inner_error0_ != NULL)) {
							if (_inner_error0_->domain == VALA_PARSE_ERROR) {
								g_propagate_error (error, _inner_error0_);
								identifiers = (_vala_array_free (identifiers, identifiers_length1, (GDestroyNotify) g_free), NULL);
								_vala_code_node_unref0 (variable_type);
								return;
							} else {
								identifiers = (_vala_array_free (identifiers, identifiers_length1, (GDestroyNotify) g_free), NULL);
								_vala_code_node_unref0 (variable_type);
								g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
								g_clear_error (&_inner_error0_);
								return;
							}
						}
						_tmp25_ = _tmp23_;
						_tmp23_ = NULL;
						_vala_array_add13 (&identifiers, &identifiers_length1, &_identifiers_size_, _tmp25_);
						_g_free0 (_tmp23_);
					}
				}
				vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_PARENS, &_inner_error0_);
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						identifiers = (_vala_array_free (identifiers, identifiers_length1, (GDestroyNotify) g_free), NULL);
						_vala_code_node_unref0 (variable_type);
						return;
					} else {
						identifiers = (_vala_array_free (identifiers, identifiers_length1, (GDestroyNotify) g_free), NULL);
						_vala_code_node_unref0 (variable_type);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return;
					}
				}
				vala_parser_expect (self, VALA_TOKEN_TYPE_ASSIGN, &_inner_error0_);
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						identifiers = (_vala_array_free (identifiers, identifiers_length1, (GDestroyNotify) g_free), NULL);
						_vala_code_node_unref0 (variable_type);
						return;
					} else {
						identifiers = (_vala_array_free (identifiers, identifiers_length1, (GDestroyNotify) g_free), NULL);
						_vala_code_node_unref0 (variable_type);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return;
					}
				}
				_tmp26_ = vala_parser_parse_expression (self, &_inner_error0_);
				tuple = _tmp26_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						identifiers = (_vala_array_free (identifiers, identifiers_length1, (GDestroyNotify) g_free), NULL);
						_vala_code_node_unref0 (variable_type);
						return;
					} else {
						identifiers = (_vala_array_free (identifiers, identifiers_length1, (GDestroyNotify) g_free), NULL);
						_vala_code_node_unref0 (variable_type);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return;
					}
				}
				_tmp27_ = vala_code_node_get_temp_name ();
				_tmp28_ = _tmp27_;
				_tmp29_ = tuple;
				_tmp30_ = begin;
				_tmp31_ = vala_parser_get_src (self, &_tmp30_);
				_tmp32_ = _tmp31_;
				_tmp33_ = vala_local_variable_new (NULL, _tmp28_, _tmp29_, _tmp32_);
				_tmp34_ = _tmp33_;
				_vala_source_reference_unref0 (_tmp32_);
				_g_free0 (_tmp28_);
				tuple_local = _tmp34_;
				_tmp35_ = tuple_local;
				_tmp36_ = tuple_local;
				_tmp37_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp36_);
				_tmp38_ = _tmp37_;
				_tmp39_ = vala_declaration_statement_new ((ValaSymbol*) _tmp35_, _tmp38_);
				_tmp40_ = _tmp39_;
				vala_block_add_statement (block, (ValaStatement*) _tmp40_);
				_vala_code_node_unref0 (_tmp40_);
				{
					gint i = 0;
					i = 0;
					{
						gboolean _tmp41_ = FALSE;
						_tmp41_ = TRUE;
						while (TRUE) {
							gchar** _tmp43_;
							gint _tmp43__length1;
							ValaMemberAccess* temp_access = NULL;
							ValaLocalVariable* _tmp44_;
							const gchar* _tmp45_;
							const gchar* _tmp46_;
							ValaLocalVariable* _tmp47_;
							ValaSourceReference* _tmp48_;
							ValaSourceReference* _tmp49_;
							ValaMemberAccess* _tmp50_;
							ValaElementAccess* ea = NULL;
							ValaMemberAccess* _tmp51_;
							ValaLocalVariable* _tmp52_;
							ValaSourceReference* _tmp53_;
							ValaSourceReference* _tmp54_;
							ValaElementAccess* _tmp55_;
							ValaElementAccess* _tmp56_;
							gchar* _tmp57_;
							gchar* _tmp58_;
							ValaIntegerLiteral* _tmp59_;
							ValaIntegerLiteral* _tmp60_;
							ValaLocalVariable* local = NULL;
							gchar** _tmp61_;
							gint _tmp61__length1;
							const gchar* _tmp62_;
							ValaElementAccess* _tmp63_;
							ValaLocalVariable* _tmp64_;
							ValaSourceReference* _tmp65_;
							ValaSourceReference* _tmp66_;
							ValaLocalVariable* _tmp67_;
							ValaLocalVariable* _tmp68_;
							ValaLocalVariable* _tmp69_;
							ValaSourceReference* _tmp70_;
							ValaSourceReference* _tmp71_;
							ValaDeclarationStatement* _tmp72_;
							ValaDeclarationStatement* _tmp73_;
							if (!_tmp41_) {
								gint _tmp42_;
								_tmp42_ = i;
								i = _tmp42_ + 1;
							}
							_tmp41_ = FALSE;
							_tmp43_ = identifiers;
							_tmp43__length1 = identifiers_length1;
							if (!(i < _tmp43__length1)) {
								break;
							}
							_tmp44_ = tuple_local;
							_tmp45_ = vala_symbol_get_name ((ValaSymbol*) _tmp44_);
							_tmp46_ = _tmp45_;
							_tmp47_ = tuple_local;
							_tmp48_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp47_);
							_tmp49_ = _tmp48_;
							_tmp50_ = vala_member_access_new_simple (_tmp46_, _tmp49_);
							temp_access = _tmp50_;
							_tmp51_ = temp_access;
							_tmp52_ = tuple_local;
							_tmp53_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp52_);
							_tmp54_ = _tmp53_;
							_tmp55_ = vala_element_access_new ((ValaExpression*) _tmp51_, _tmp54_);
							ea = _tmp55_;
							_tmp56_ = ea;
							_tmp57_ = g_strdup_printf ("%i", i);
							_tmp58_ = _tmp57_;
							_tmp59_ = vala_integer_literal_new (_tmp58_, NULL);
							_tmp60_ = _tmp59_;
							vala_element_access_append_index (_tmp56_, (ValaExpression*) _tmp60_);
							_vala_code_node_unref0 (_tmp60_);
							_g_free0 (_tmp58_);
							_tmp61_ = identifiers;
							_tmp61__length1 = identifiers_length1;
							_tmp62_ = _tmp61_[i];
							_tmp63_ = ea;
							_tmp64_ = tuple_local;
							_tmp65_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp64_);
							_tmp66_ = _tmp65_;
							_tmp67_ = vala_local_variable_new (NULL, _tmp62_, (ValaExpression*) _tmp63_, _tmp66_);
							local = _tmp67_;
							_tmp68_ = local;
							_tmp69_ = local;
							_tmp70_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp69_);
							_tmp71_ = _tmp70_;
							_tmp72_ = vala_declaration_statement_new ((ValaSymbol*) _tmp68_, _tmp71_);
							_tmp73_ = _tmp72_;
							vala_block_add_statement (block, (ValaStatement*) _tmp73_);
							_vala_code_node_unref0 (_tmp73_);
							_vala_code_node_unref0 (local);
							_vala_code_node_unref0 (ea);
							_vala_code_node_unref0 (temp_access);
						}
					}
				}
				_vala_code_node_unref0 (tuple_local);
				_vala_code_node_unref0 (tuple);
				identifiers = (_vala_array_free (identifiers, identifiers_length1, (GDestroyNotify) g_free), NULL);
				continue;
			}
			if (!is_first) {
				ValaSourceLocation _tmp74_ = {0};
				vala_parser_get_location (self, &_tmp74_);
				begin = _tmp74_;
			} else {
				is_first = FALSE;
			}
			type_copy = NULL;
			_tmp75_ = variable_type;
			if (_tmp75_ != NULL) {
				ValaDataType* _tmp76_;
				ValaDataType* _tmp77_;
				_tmp76_ = variable_type;
				_tmp77_ = vala_data_type_copy (_tmp76_);
				_vala_code_node_unref0 (type_copy);
				type_copy = _tmp77_;
			}
			_tmp78_ = type_copy;
			_tmp79_ = vala_parser_parse_local_variable (self, _tmp78_, FALSE, &_inner_error0_);
			local = _tmp79_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (type_copy);
					_vala_code_node_unref0 (variable_type);
					return;
				} else {
					_vala_code_node_unref0 (type_copy);
					_vala_code_node_unref0 (variable_type);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return;
				}
			}
			_tmp80_ = local;
			_tmp81_ = begin;
			_tmp82_ = vala_parser_get_src (self, &_tmp81_);
			_tmp83_ = _tmp82_;
			_tmp84_ = vala_declaration_statement_new ((ValaSymbol*) _tmp80_, _tmp83_);
			_tmp85_ = _tmp84_;
			vala_block_add_statement (block, (ValaStatement*) _tmp85_);
			_vala_code_node_unref0 (_tmp85_);
			_vala_source_reference_unref0 (_tmp83_);
			_vala_code_node_unref0 (local);
			_vala_code_node_unref0 (type_copy);
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_SEMICOLON, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (variable_type);
			return;
		} else {
			_vala_code_node_unref0 (variable_type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_vala_code_node_unref0 (variable_type);
}

static ValaLocalVariable*
vala_parser_parse_local_variable (ValaParser* self,
                                  ValaDataType* variable_type,
                                  gboolean expect_initializer,
                                  GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	gchar* id = NULL;
	gchar* _tmp1_;
	ValaDataType* type = NULL;
	ValaDataType* _tmp2_;
	ValaSourceReference* src = NULL;
	ValaSourceLocation _tmp3_;
	ValaSourceReference* _tmp4_;
	ValaExpression* initializer = NULL;
	ValaDataType* _tmp11_;
	const gchar* _tmp12_;
	ValaExpression* _tmp13_;
	ValaSourceReference* _tmp14_;
	ValaLocalVariable* _tmp15_;
	GError* _inner_error0_ = NULL;
	ValaLocalVariable* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	_tmp1_ = vala_parser_parse_identifier (self, FALSE, &_inner_error0_);
	id = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp2_ = vala_parser_parse_inline_array_type (self, variable_type, &_inner_error0_);
	type = _tmp2_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_g_free0 (id);
			return NULL;
		} else {
			_g_free0 (id);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp3_ = begin;
	_tmp4_ = vala_parser_get_src (self, &_tmp3_);
	src = _tmp4_;
	initializer = NULL;
	if (vala_parser_accept (self, VALA_TOKEN_TYPE_ASSIGN)) {
		ValaExpression* _tmp5_ = NULL;
		ValaExpression* _tmp6_;
		ValaExpression* _tmp7_;
		_tmp6_ = vala_parser_parse_expression (self, &_inner_error0_);
		_tmp5_ = _tmp6_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (initializer);
				_vala_source_reference_unref0 (src);
				_vala_code_node_unref0 (type);
				_g_free0 (id);
				return NULL;
			} else {
				_vala_code_node_unref0 (initializer);
				_vala_source_reference_unref0 (src);
				_vala_code_node_unref0 (type);
				_g_free0 (id);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp7_ = _tmp5_;
		_tmp5_ = NULL;
		_vala_code_node_unref0 (initializer);
		initializer = _tmp7_;
		_vala_code_node_unref0 (_tmp5_);
	} else {
		if (expect_initializer) {
			GError* _tmp8_;
			GError* _tmp9_;
			ValaInvalidExpression* _tmp10_;
			_tmp8_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "expected initializer");
			_tmp9_ = _tmp8_;
			vala_parser_report_parse_error (self, _tmp9_);
			_g_error_free0 (_tmp9_);
			vala_parser_prev (self);
			_tmp10_ = vala_invalid_expression_new ();
			_vala_code_node_unref0 (initializer);
			initializer = (ValaExpression*) _tmp10_;
		}
	}
	_tmp11_ = type;
	_tmp12_ = id;
	_tmp13_ = initializer;
	_tmp14_ = src;
	_tmp15_ = vala_local_variable_new (_tmp11_, _tmp12_, _tmp13_, _tmp14_);
	result = _tmp15_;
	_vala_code_node_unref0 (initializer);
	_vala_source_reference_unref0 (src);
	_vala_code_node_unref0 (type);
	_g_free0 (id);
	return result;
}

static void
vala_parser_parse_local_constant_declarations (ValaParser* self,
                                               ValaBlock* block,
                                               GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaDataType* constant_type = NULL;
	ValaDataType* _tmp1_;
	ValaArrayType* array_type = NULL;
	ValaDataType* _tmp2_;
	ValaArrayType* _tmp3_;
	gboolean is_first = FALSE;
	GError* _inner_error0_ = NULL;
	g_return_if_fail (self != NULL);
	g_return_if_fail (block != NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_CONST, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp1_ = vala_parser_parse_type (self, FALSE, FALSE, FALSE, &_inner_error0_);
	constant_type = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp2_ = constant_type;
	array_type = VALA_IS_ARRAY_TYPE (_tmp2_) ? ((ValaArrayType*) _tmp2_) : NULL;
	_tmp3_ = array_type;
	if (_tmp3_ != NULL) {
		ValaArrayType* _tmp4_;
		ValaDataType* _tmp5_;
		ValaDataType* _tmp6_;
		_tmp4_ = array_type;
		_tmp5_ = vala_array_type_get_element_type (_tmp4_);
		_tmp6_ = _tmp5_;
		vala_data_type_set_value_owned (_tmp6_, FALSE);
	}
	is_first = TRUE;
	{
		gboolean _tmp7_ = FALSE;
		_tmp7_ = TRUE;
		while (TRUE) {
			ValaDataType* type_copy = NULL;
			ValaDataType* _tmp9_;
			ValaDataType* _tmp10_;
			ValaConstant* local = NULL;
			ValaDataType* _tmp11_;
			ValaConstant* _tmp12_;
			ValaConstant* _tmp13_;
			ValaSourceLocation _tmp14_;
			ValaSourceReference* _tmp15_;
			ValaSourceReference* _tmp16_;
			ValaDeclarationStatement* _tmp17_;
			ValaDeclarationStatement* _tmp18_;
			ValaConstant* _tmp19_;
			ValaConstant* _tmp20_;
			if (!_tmp7_) {
				if (!vala_parser_accept (self, VALA_TOKEN_TYPE_COMMA)) {
					break;
				}
			}
			_tmp7_ = FALSE;
			if (!is_first) {
				ValaSourceLocation _tmp8_ = {0};
				vala_parser_get_location (self, &_tmp8_);
				begin = _tmp8_;
			} else {
				is_first = FALSE;
			}
			_tmp9_ = constant_type;
			_tmp10_ = vala_data_type_copy (_tmp9_);
			type_copy = _tmp10_;
			_tmp11_ = type_copy;
			_tmp12_ = vala_parser_parse_local_constant (self, _tmp11_, &_inner_error0_);
			local = _tmp12_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (type_copy);
					_vala_code_node_unref0 (constant_type);
					return;
				} else {
					_vala_code_node_unref0 (type_copy);
					_vala_code_node_unref0 (constant_type);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return;
				}
			}
			_tmp13_ = local;
			_tmp14_ = begin;
			_tmp15_ = vala_parser_get_src (self, &_tmp14_);
			_tmp16_ = _tmp15_;
			_tmp17_ = vala_declaration_statement_new ((ValaSymbol*) _tmp13_, _tmp16_);
			_tmp18_ = _tmp17_;
			vala_block_add_statement (block, (ValaStatement*) _tmp18_);
			_vala_code_node_unref0 (_tmp18_);
			_vala_source_reference_unref0 (_tmp16_);
			_tmp19_ = local;
			vala_block_add_local_constant (block, _tmp19_);
			_tmp20_ = local;
			vala_symbol_set_active ((ValaSymbol*) _tmp20_, FALSE);
			_vala_code_node_unref0 (local);
			_vala_code_node_unref0 (type_copy);
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_SEMICOLON, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (constant_type);
			return;
		} else {
			_vala_code_node_unref0 (constant_type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_vala_code_node_unref0 (constant_type);
}

static ValaConstant*
vala_parser_parse_local_constant (ValaParser* self,
                                  ValaDataType* constant_type,
                                  GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	gchar* id = NULL;
	gchar* _tmp1_;
	ValaDataType* type = NULL;
	ValaDataType* _tmp2_;
	ValaSourceReference* src = NULL;
	ValaSourceLocation _tmp3_;
	ValaSourceReference* _tmp4_;
	ValaExpression* initializer = NULL;
	ValaExpression* _tmp5_;
	ValaDataType* _tmp6_;
	gboolean _tmp7_;
	gboolean _tmp8_;
	const gchar* _tmp10_;
	ValaDataType* _tmp11_;
	ValaExpression* _tmp12_;
	ValaSourceReference* _tmp13_;
	ValaConstant* _tmp14_;
	GError* _inner_error0_ = NULL;
	ValaConstant* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (constant_type != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	_tmp1_ = vala_parser_parse_identifier (self, FALSE, &_inner_error0_);
	id = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp2_ = vala_parser_parse_inline_array_type (self, constant_type, &_inner_error0_);
	type = _tmp2_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_g_free0 (id);
			return NULL;
		} else {
			_g_free0 (id);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp3_ = begin;
	_tmp4_ = vala_parser_get_src (self, &_tmp3_);
	src = _tmp4_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_ASSIGN, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_source_reference_unref0 (src);
			_vala_code_node_unref0 (type);
			_g_free0 (id);
			return NULL;
		} else {
			_vala_source_reference_unref0 (src);
			_vala_code_node_unref0 (type);
			_g_free0 (id);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp5_ = vala_parser_parse_expression (self, &_inner_error0_);
	initializer = _tmp5_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_source_reference_unref0 (src);
			_vala_code_node_unref0 (type);
			_g_free0 (id);
			return NULL;
		} else {
			_vala_source_reference_unref0 (src);
			_vala_code_node_unref0 (type);
			_g_free0 (id);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp6_ = type;
	_tmp7_ = vala_data_type_get_value_owned (_tmp6_);
	_tmp8_ = _tmp7_;
	if (_tmp8_) {
		ValaSourceReference* _tmp9_;
		_tmp9_ = src;
		vala_report_error (_tmp9_, "`owned' is not allowed on constants");
	}
	_tmp10_ = id;
	_tmp11_ = type;
	_tmp12_ = initializer;
	_tmp13_ = src;
	_tmp14_ = vala_constant_new (_tmp10_, _tmp11_, _tmp12_, _tmp13_, NULL);
	result = _tmp14_;
	_vala_code_node_unref0 (initializer);
	_vala_source_reference_unref0 (src);
	_vala_code_node_unref0 (type);
	_g_free0 (id);
	return result;
}

static void
vala_parser_parse_local_function_declaration (ValaParser* self,
                                              ValaBlock* block,
                                              GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	gboolean is_static = FALSE;
	ValaDataType* type = NULL;
	ValaDataType* _tmp1_;
	ValaUnresolvedSymbol* sym = NULL;
	ValaUnresolvedSymbol* _tmp2_;
	ValaList* params = NULL;
	GEqualFunc _tmp3_;
	ValaArrayList* _tmp4_;
	ValaSourceReference* src = NULL;
	ValaSourceLocation _tmp9_;
	ValaSourceReference* _tmp10_;
	ValaDelegate* d = NULL;
	gint _tmp11_;
	gchar* _tmp12_;
	gchar* _tmp13_;
	ValaDataType* _tmp14_;
	ValaSourceReference* _tmp15_;
	ValaDelegate* _tmp16_;
	ValaDelegate* _tmp17_;
	ValaDelegate* _tmp18_;
	ValaCodeContext* _tmp29_;
	ValaNamespace* _tmp30_;
	ValaNamespace* _tmp31_;
	ValaDelegate* _tmp32_;
	ValaBlock* _tmp33_ = NULL;
	ValaBlock* _tmp34_;
	ValaLambdaExpression* lambda = NULL;
	ValaSourceReference* _tmp35_;
	ValaLambdaExpression* _tmp36_;
	ValaCodeContext* _tmp58_;
	gboolean _tmp59_;
	gboolean _tmp60_;
	ValaLocalVariable* local = NULL;
	ValaDelegate* _tmp62_;
	ValaDelegateType* _tmp63_;
	ValaDelegateType* _tmp64_;
	ValaUnresolvedSymbol* _tmp65_;
	const gchar* _tmp66_;
	const gchar* _tmp67_;
	ValaLambdaExpression* _tmp68_;
	ValaSourceReference* _tmp69_;
	ValaLocalVariable* _tmp70_;
	ValaLocalVariable* _tmp71_;
	ValaLocalVariable* _tmp72_;
	ValaSourceReference* _tmp73_;
	ValaDeclarationStatement* _tmp74_;
	ValaDeclarationStatement* _tmp75_;
	GError* _inner_error0_ = NULL;
	g_return_if_fail (self != NULL);
	g_return_if_fail (block != NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	is_static = vala_parser_accept (self, VALA_TOKEN_TYPE_STATIC);
	_tmp1_ = vala_parser_parse_type (self, TRUE, FALSE, FALSE, &_inner_error0_);
	type = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp2_ = vala_parser_parse_symbol_name (self, &_inner_error0_);
	sym = _tmp2_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (type);
			return;
		} else {
			_vala_code_node_unref0 (type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (sym);
			_vala_code_node_unref0 (type);
			return;
		} else {
			_vala_code_node_unref0 (sym);
			_vala_code_node_unref0 (type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp3_ = g_direct_equal;
	_tmp4_ = vala_array_list_new (VALA_TYPE_PARAMETER, (GBoxedCopyFunc) vala_code_node_ref, (GDestroyNotify) vala_code_node_unref, _tmp3_);
	params = (ValaList*) _tmp4_;
	if (vala_parser_current (self) != VALA_TOKEN_TYPE_CLOSE_PARENS) {
		{
			gboolean _tmp5_ = FALSE;
			_tmp5_ = TRUE;
			while (TRUE) {
				ValaParameter* _tmp6_ = NULL;
				ValaParameter* _tmp7_;
				ValaList* _tmp8_;
				if (!_tmp5_) {
					if (!vala_parser_accept (self, VALA_TOKEN_TYPE_COMMA)) {
						break;
					}
				}
				_tmp5_ = FALSE;
				_tmp7_ = vala_parser_parse_parameter (self, &_inner_error0_);
				_tmp6_ = _tmp7_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_iterable_unref0 (params);
						_vala_code_node_unref0 (sym);
						_vala_code_node_unref0 (type);
						return;
					} else {
						_vala_iterable_unref0 (params);
						_vala_code_node_unref0 (sym);
						_vala_code_node_unref0 (type);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return;
					}
				}
				_tmp8_ = params;
				vala_collection_add ((ValaCollection*) _tmp8_, _tmp6_);
				_vala_code_node_unref0 (_tmp6_);
			}
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_iterable_unref0 (params);
			_vala_code_node_unref0 (sym);
			_vala_code_node_unref0 (type);
			return;
		} else {
			_vala_iterable_unref0 (params);
			_vala_code_node_unref0 (sym);
			_vala_code_node_unref0 (type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp9_ = begin;
	_tmp10_ = vala_parser_get_src (self, &_tmp9_);
	src = _tmp10_;
	_tmp11_ = vala_parser_next_local_func_id;
	vala_parser_next_local_func_id = _tmp11_ + 1;
	_tmp12_ = g_strdup_printf ("_LocalFunc%i_", _tmp11_);
	_tmp13_ = _tmp12_;
	_tmp14_ = type;
	_tmp15_ = src;
	_tmp16_ = vala_delegate_new (_tmp13_, _tmp14_, _tmp15_, NULL);
	_tmp17_ = _tmp16_;
	_g_free0 (_tmp13_);
	d = _tmp17_;
	_tmp18_ = d;
	vala_delegate_set_has_target (_tmp18_, !is_static);
	{
		ValaList* _param_list = NULL;
		ValaList* _tmp19_;
		gint _param_size = 0;
		ValaList* _tmp20_;
		gint _tmp21_;
		gint _tmp22_;
		gint _param_index = 0;
		_tmp19_ = params;
		_param_list = _tmp19_;
		_tmp20_ = _param_list;
		_tmp21_ = vala_collection_get_size ((ValaCollection*) _tmp20_);
		_tmp22_ = _tmp21_;
		_param_size = _tmp22_;
		_param_index = -1;
		while (TRUE) {
			gint _tmp23_;
			gint _tmp24_;
			ValaParameter* param = NULL;
			ValaList* _tmp25_;
			gpointer _tmp26_;
			ValaDelegate* _tmp27_;
			ValaParameter* _tmp28_;
			_param_index = _param_index + 1;
			_tmp23_ = _param_index;
			_tmp24_ = _param_size;
			if (!(_tmp23_ < _tmp24_)) {
				break;
			}
			_tmp25_ = _param_list;
			_tmp26_ = vala_list_get (_tmp25_, _param_index);
			param = (ValaParameter*) _tmp26_;
			_tmp27_ = d;
			_tmp28_ = param;
			vala_callable_add_parameter ((ValaCallable*) _tmp27_, _tmp28_);
			_vala_code_node_unref0 (param);
		}
	}
	_tmp29_ = self->priv->context;
	_tmp30_ = vala_code_context_get_root (_tmp29_);
	_tmp31_ = _tmp30_;
	_tmp32_ = d;
	vala_symbol_add_delegate ((ValaSymbol*) _tmp31_, _tmp32_);
	_tmp34_ = vala_parser_parse_block (self, &_inner_error0_);
	_tmp33_ = _tmp34_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (d);
			_vala_source_reference_unref0 (src);
			_vala_iterable_unref0 (params);
			_vala_code_node_unref0 (sym);
			_vala_code_node_unref0 (type);
			return;
		} else {
			_vala_code_node_unref0 (d);
			_vala_source_reference_unref0 (src);
			_vala_iterable_unref0 (params);
			_vala_code_node_unref0 (sym);
			_vala_code_node_unref0 (type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp35_ = src;
	_tmp36_ = vala_lambda_expression_new_with_statement_body (_tmp33_, _tmp35_);
	lambda = _tmp36_;
	{
		ValaList* _p_list = NULL;
		ValaList* _tmp37_;
		gint _p_size = 0;
		ValaList* _tmp38_;
		gint _tmp39_;
		gint _tmp40_;
		gint _p_index = 0;
		_tmp37_ = params;
		_p_list = _tmp37_;
		_tmp38_ = _p_list;
		_tmp39_ = vala_collection_get_size ((ValaCollection*) _tmp38_);
		_tmp40_ = _tmp39_;
		_p_size = _tmp40_;
		_p_index = -1;
		while (TRUE) {
			gint _tmp41_;
			gint _tmp42_;
			ValaParameter* p = NULL;
			ValaList* _tmp43_;
			gpointer _tmp44_;
			ValaParameter* param = NULL;
			ValaParameter* _tmp45_;
			const gchar* _tmp46_;
			const gchar* _tmp47_;
			ValaParameter* _tmp48_;
			ValaSourceReference* _tmp49_;
			ValaSourceReference* _tmp50_;
			ValaParameter* _tmp51_;
			ValaParameter* _tmp52_;
			ValaParameter* _tmp53_;
			ValaParameterDirection _tmp54_;
			ValaParameterDirection _tmp55_;
			ValaLambdaExpression* _tmp56_;
			ValaParameter* _tmp57_;
			_p_index = _p_index + 1;
			_tmp41_ = _p_index;
			_tmp42_ = _p_size;
			if (!(_tmp41_ < _tmp42_)) {
				break;
			}
			_tmp43_ = _p_list;
			_tmp44_ = vala_list_get (_tmp43_, _p_index);
			p = (ValaParameter*) _tmp44_;
			_tmp45_ = p;
			_tmp46_ = vala_symbol_get_name ((ValaSymbol*) _tmp45_);
			_tmp47_ = _tmp46_;
			_tmp48_ = p;
			_tmp49_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp48_);
			_tmp50_ = _tmp49_;
			_tmp51_ = vala_parameter_new (_tmp47_, NULL, _tmp50_);
			param = _tmp51_;
			_tmp52_ = param;
			_tmp53_ = p;
			_tmp54_ = vala_parameter_get_direction (_tmp53_);
			_tmp55_ = _tmp54_;
			vala_parameter_set_direction (_tmp52_, _tmp55_);
			_tmp56_ = lambda;
			_tmp57_ = param;
			vala_lambda_expression_add_parameter (_tmp56_, _tmp57_);
			_vala_code_node_unref0 (param);
			_vala_code_node_unref0 (p);
		}
	}
	_tmp58_ = self->priv->context;
	_tmp59_ = vala_code_context_get_experimental (_tmp58_);
	_tmp60_ = _tmp59_;
	if (!_tmp60_) {
		ValaSourceReference* _tmp61_;
		_tmp61_ = src;
		vala_report_warning (_tmp61_, "local functions are experimental");
	}
	_tmp62_ = d;
	_tmp63_ = vala_delegate_type_new (_tmp62_, NULL);
	_tmp64_ = _tmp63_;
	_tmp65_ = sym;
	_tmp66_ = vala_symbol_get_name ((ValaSymbol*) _tmp65_);
	_tmp67_ = _tmp66_;
	_tmp68_ = lambda;
	_tmp69_ = src;
	_tmp70_ = vala_local_variable_new ((ValaDataType*) _tmp64_, _tmp67_, (ValaExpression*) _tmp68_, _tmp69_);
	_tmp71_ = _tmp70_;
	_vala_code_node_unref0 (_tmp64_);
	local = _tmp71_;
	_tmp72_ = local;
	_tmp73_ = src;
	_tmp74_ = vala_declaration_statement_new ((ValaSymbol*) _tmp72_, _tmp73_);
	_tmp75_ = _tmp74_;
	vala_block_add_statement (block, (ValaStatement*) _tmp75_);
	_vala_code_node_unref0 (_tmp75_);
	_vala_code_node_unref0 (local);
	_vala_code_node_unref0 (lambda);
	_vala_code_node_unref0 (_tmp33_);
	_vala_code_node_unref0 (d);
	_vala_source_reference_unref0 (src);
	_vala_iterable_unref0 (params);
	_vala_code_node_unref0 (sym);
	_vala_code_node_unref0 (type);
}

static ValaStatement*
vala_parser_parse_expression_statement (ValaParser* self,
                                        GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaExpression* expr = NULL;
	ValaExpression* _tmp1_;
	ValaSourceReference* src = NULL;
	ValaSourceLocation _tmp2_;
	ValaSourceReference* _tmp3_;
	ValaExpression* _tmp4_;
	ValaSourceReference* _tmp5_;
	ValaExpressionStatement* _tmp6_;
	GError* _inner_error0_ = NULL;
	ValaStatement* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	_tmp1_ = vala_parser_parse_statement_expression (self, &_inner_error0_);
	expr = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp2_ = begin;
	_tmp3_ = vala_parser_get_src (self, &_tmp2_);
	src = _tmp3_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_SEMICOLON, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_source_reference_unref0 (src);
			_vala_code_node_unref0 (expr);
			return NULL;
		} else {
			_vala_source_reference_unref0 (src);
			_vala_code_node_unref0 (expr);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp4_ = expr;
	_tmp5_ = src;
	_tmp6_ = vala_expression_statement_new (_tmp4_, _tmp5_);
	result = (ValaStatement*) _tmp6_;
	_vala_source_reference_unref0 (src);
	_vala_code_node_unref0 (expr);
	return result;
}

static ValaExpression*
vala_parser_parse_statement_expression (ValaParser* self,
                                        GError** error)
{
	ValaExpression* expr = NULL;
	ValaExpression* _tmp0_;
	GError* _inner_error0_ = NULL;
	ValaExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = vala_parser_parse_expression (self, &_inner_error0_);
	expr = _tmp0_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	result = expr;
	return result;
}

static ValaStatement*
vala_parser_parse_if_statement (ValaParser* self,
                                GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaExpression* condition = NULL;
	ValaExpression* _tmp1_;
	ValaSourceReference* src = NULL;
	ValaSourceLocation _tmp2_;
	ValaSourceReference* _tmp3_;
	ValaBlock* true_stmt = NULL;
	ValaBlock* _tmp4_;
	ValaBlock* false_stmt = NULL;
	ValaExpression* _tmp8_;
	ValaBlock* _tmp9_;
	ValaBlock* _tmp10_;
	ValaSourceReference* _tmp11_;
	ValaIfStatement* _tmp12_;
	GError* _inner_error0_ = NULL;
	ValaStatement* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_IF, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp1_ = vala_parser_parse_expression (self, &_inner_error0_);
	condition = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (condition);
			return NULL;
		} else {
			_vala_code_node_unref0 (condition);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp2_ = begin;
	_tmp3_ = vala_parser_get_src (self, &_tmp2_);
	src = _tmp3_;
	_tmp4_ = vala_parser_parse_embedded_statement (self, "if", FALSE, &_inner_error0_);
	true_stmt = _tmp4_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_source_reference_unref0 (src);
			_vala_code_node_unref0 (condition);
			return NULL;
		} else {
			_vala_source_reference_unref0 (src);
			_vala_code_node_unref0 (condition);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	false_stmt = NULL;
	if (vala_parser_accept (self, VALA_TOKEN_TYPE_ELSE)) {
		ValaBlock* _tmp5_ = NULL;
		ValaBlock* _tmp6_;
		ValaBlock* _tmp7_;
		_tmp6_ = vala_parser_parse_embedded_statement (self, "else", FALSE, &_inner_error0_);
		_tmp5_ = _tmp6_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (false_stmt);
				_vala_code_node_unref0 (true_stmt);
				_vala_source_reference_unref0 (src);
				_vala_code_node_unref0 (condition);
				return NULL;
			} else {
				_vala_code_node_unref0 (false_stmt);
				_vala_code_node_unref0 (true_stmt);
				_vala_source_reference_unref0 (src);
				_vala_code_node_unref0 (condition);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp7_ = _tmp5_;
		_tmp5_ = NULL;
		_vala_code_node_unref0 (false_stmt);
		false_stmt = _tmp7_;
		_vala_code_node_unref0 (_tmp5_);
	}
	_tmp8_ = condition;
	_tmp9_ = true_stmt;
	_tmp10_ = false_stmt;
	_tmp11_ = src;
	_tmp12_ = vala_if_statement_new (_tmp8_, _tmp9_, _tmp10_, _tmp11_);
	result = (ValaStatement*) _tmp12_;
	_vala_code_node_unref0 (false_stmt);
	_vala_code_node_unref0 (true_stmt);
	_vala_source_reference_unref0 (src);
	_vala_code_node_unref0 (condition);
	return result;
}

static ValaStatement*
vala_parser_parse_switch_statement (ValaParser* self,
                                    GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaExpression* condition = NULL;
	ValaExpression* _tmp1_;
	ValaSwitchStatement* stmt = NULL;
	ValaExpression* _tmp2_;
	ValaSourceLocation _tmp3_;
	ValaSourceReference* _tmp4_;
	ValaSourceReference* _tmp5_;
	ValaSwitchStatement* _tmp6_;
	ValaSwitchStatement* _tmp7_;
	GError* _inner_error0_ = NULL;
	ValaStatement* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_SWITCH, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp1_ = vala_parser_parse_expression (self, &_inner_error0_);
	condition = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (condition);
			return NULL;
		} else {
			_vala_code_node_unref0 (condition);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp2_ = condition;
	_tmp3_ = begin;
	_tmp4_ = vala_parser_get_src (self, &_tmp3_);
	_tmp5_ = _tmp4_;
	_tmp6_ = vala_switch_statement_new (_tmp2_, _tmp5_);
	_tmp7_ = _tmp6_;
	_vala_source_reference_unref0 (_tmp5_);
	stmt = _tmp7_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_BRACE, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (stmt);
			_vala_code_node_unref0 (condition);
			return NULL;
		} else {
			_vala_code_node_unref0 (stmt);
			_vala_code_node_unref0 (condition);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	while (TRUE) {
		ValaStatement* _tmp8_ = NULL;
		ValaStatement* _tmp9_;
		ValaSwitchStatement* _tmp10_;
		if (!(vala_parser_current (self) != VALA_TOKEN_TYPE_CLOSE_BRACE)) {
			break;
		}
		_tmp9_ = vala_parser_parse_switch_section_statement (self, &_inner_error0_);
		_tmp8_ = _tmp9_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (stmt);
				_vala_code_node_unref0 (condition);
				return NULL;
			} else {
				_vala_code_node_unref0 (stmt);
				_vala_code_node_unref0 (condition);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp10_ = stmt;
		vala_switch_statement_add_section (_tmp10_, G_TYPE_CHECK_INSTANCE_CAST (_tmp8_, VALA_TYPE_SWITCH_SECTION, ValaSwitchSection));
		_vala_code_node_unref0 (_tmp8_);
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_BRACE, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (stmt);
			_vala_code_node_unref0 (condition);
			return NULL;
		} else {
			_vala_code_node_unref0 (stmt);
			_vala_code_node_unref0 (condition);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	result = (ValaStatement*) stmt;
	_vala_code_node_unref0 (condition);
	return result;
}

static ValaStatement*
vala_parser_parse_switch_section_statement (ValaParser* self,
                                            GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaSwitchSection* section = NULL;
	ValaSourceLocation _tmp1_;
	ValaSourceReference* _tmp2_;
	ValaSourceReference* _tmp3_;
	ValaSwitchSection* _tmp4_;
	ValaSwitchSection* _tmp5_;
	ValaSwitchSection* _tmp31_;
	GError* _inner_error0_ = NULL;
	ValaStatement* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	_tmp1_ = begin;
	_tmp2_ = vala_parser_get_src (self, &_tmp1_);
	_tmp3_ = _tmp2_;
	_tmp4_ = vala_switch_section_new (_tmp3_);
	_tmp5_ = _tmp4_;
	_vala_source_reference_unref0 (_tmp3_);
	section = _tmp5_;
	{
		gboolean _tmp6_ = FALSE;
		_tmp6_ = TRUE;
		while (TRUE) {
			if (!_tmp6_) {
				gboolean _tmp7_ = FALSE;
				if (vala_parser_current (self) == VALA_TOKEN_TYPE_CASE) {
					_tmp7_ = TRUE;
				} else {
					_tmp7_ = vala_parser_current (self) == VALA_TOKEN_TYPE_DEFAULT;
				}
				if (!_tmp7_) {
					break;
				}
			}
			_tmp6_ = FALSE;
			if (vala_parser_accept (self, VALA_TOKEN_TYPE_CASE)) {
				ValaExpression* _tmp8_ = NULL;
				ValaExpression* _tmp9_;
				ValaSwitchSection* _tmp10_;
				ValaSourceLocation _tmp11_;
				ValaSourceReference* _tmp12_;
				ValaSourceReference* _tmp13_;
				ValaSwitchLabel* _tmp14_;
				ValaSwitchLabel* _tmp15_;
				_tmp9_ = vala_parser_parse_expression (self, &_inner_error0_);
				_tmp8_ = _tmp9_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_code_node_unref0 (section);
						return NULL;
					} else {
						_vala_code_node_unref0 (section);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return NULL;
					}
				}
				_tmp10_ = section;
				_tmp11_ = begin;
				_tmp12_ = vala_parser_get_src (self, &_tmp11_);
				_tmp13_ = _tmp12_;
				_tmp14_ = vala_switch_label_new (_tmp8_, _tmp13_);
				_tmp15_ = _tmp14_;
				vala_switch_section_add_label (_tmp10_, _tmp15_);
				_vala_code_node_unref0 (_tmp15_);
				_vala_source_reference_unref0 (_tmp13_);
				while (TRUE) {
					ValaExpression* _tmp16_ = NULL;
					ValaExpression* _tmp17_;
					ValaSwitchSection* _tmp18_;
					ValaSourceLocation _tmp19_;
					ValaSourceReference* _tmp20_;
					ValaSourceReference* _tmp21_;
					ValaSwitchLabel* _tmp22_;
					ValaSwitchLabel* _tmp23_;
					if (!(vala_parser_current (self) == VALA_TOKEN_TYPE_COMMA)) {
						break;
					}
					vala_parser_expect (self, VALA_TOKEN_TYPE_COMMA, &_inner_error0_);
					if (G_UNLIKELY (_inner_error0_ != NULL)) {
						if (_inner_error0_->domain == VALA_PARSE_ERROR) {
							g_propagate_error (error, _inner_error0_);
							_vala_code_node_unref0 (_tmp8_);
							_vala_code_node_unref0 (section);
							return NULL;
						} else {
							_vala_code_node_unref0 (_tmp8_);
							_vala_code_node_unref0 (section);
							g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
							g_clear_error (&_inner_error0_);
							return NULL;
						}
					}
					_tmp17_ = vala_parser_parse_expression (self, &_inner_error0_);
					_tmp16_ = _tmp17_;
					if (G_UNLIKELY (_inner_error0_ != NULL)) {
						if (_inner_error0_->domain == VALA_PARSE_ERROR) {
							g_propagate_error (error, _inner_error0_);
							_vala_code_node_unref0 (_tmp8_);
							_vala_code_node_unref0 (section);
							return NULL;
						} else {
							_vala_code_node_unref0 (_tmp8_);
							_vala_code_node_unref0 (section);
							g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
							g_clear_error (&_inner_error0_);
							return NULL;
						}
					}
					_tmp18_ = section;
					_tmp19_ = begin;
					_tmp20_ = vala_parser_get_src (self, &_tmp19_);
					_tmp21_ = _tmp20_;
					_tmp22_ = vala_switch_label_new (_tmp16_, _tmp21_);
					_tmp23_ = _tmp22_;
					vala_switch_section_add_label (_tmp18_, _tmp23_);
					_vala_code_node_unref0 (_tmp23_);
					_vala_source_reference_unref0 (_tmp21_);
					_vala_code_node_unref0 (_tmp16_);
				}
				vala_parser_expect (self, VALA_TOKEN_TYPE_COLON, &_inner_error0_);
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_code_node_unref0 (_tmp8_);
						_vala_code_node_unref0 (section);
						return NULL;
					} else {
						_vala_code_node_unref0 (_tmp8_);
						_vala_code_node_unref0 (section);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return NULL;
					}
				}
				_vala_code_node_unref0 (_tmp8_);
			} else {
				if (vala_parser_accept (self, VALA_TOKEN_TYPE_DEFAULT)) {
					ValaSwitchSection* _tmp24_;
					ValaSourceLocation _tmp25_;
					ValaSourceReference* _tmp26_;
					ValaSourceReference* _tmp27_;
					ValaSwitchLabel* _tmp28_;
					ValaSwitchLabel* _tmp29_;
					_tmp24_ = section;
					_tmp25_ = begin;
					_tmp26_ = vala_parser_get_src (self, &_tmp25_);
					_tmp27_ = _tmp26_;
					_tmp28_ = vala_switch_label_new_with_default (_tmp27_);
					_tmp29_ = _tmp28_;
					vala_switch_section_add_label (_tmp24_, _tmp29_);
					_vala_code_node_unref0 (_tmp29_);
					_vala_source_reference_unref0 (_tmp27_);
					vala_parser_expect (self, VALA_TOKEN_TYPE_COLON, &_inner_error0_);
					if (G_UNLIKELY (_inner_error0_ != NULL)) {
						if (_inner_error0_->domain == VALA_PARSE_ERROR) {
							g_propagate_error (error, _inner_error0_);
							_vala_code_node_unref0 (section);
							return NULL;
						} else {
							_vala_code_node_unref0 (section);
							g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
							g_clear_error (&_inner_error0_);
							return NULL;
						}
					}
				} else {
					GError* _tmp30_;
					_tmp30_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "expected `case' or `default' switch label");
					_inner_error0_ = _tmp30_;
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_code_node_unref0 (section);
						return NULL;
					} else {
						_vala_code_node_unref0 (section);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return NULL;
					}
				}
			}
		}
	}
	_tmp31_ = section;
	vala_parser_parse_statements (self, (ValaBlock*) _tmp31_, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (section);
			return NULL;
		} else {
			_vala_code_node_unref0 (section);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	result = (ValaStatement*) section;
	return result;
}

static ValaStatement*
vala_parser_parse_while_statement (ValaParser* self,
                                   GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaExpression* condition = NULL;
	ValaExpression* _tmp1_;
	ValaBlock* body = NULL;
	ValaBlock* _tmp2_;
	ValaExpression* _tmp3_;
	ValaBlock* _tmp4_;
	ValaSourceLocation _tmp5_;
	ValaSourceReference* _tmp6_;
	ValaSourceReference* _tmp7_;
	ValaWhileStatement* _tmp8_;
	ValaStatement* _tmp9_;
	GError* _inner_error0_ = NULL;
	ValaStatement* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_WHILE, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp1_ = vala_parser_parse_expression (self, &_inner_error0_);
	condition = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (condition);
			return NULL;
		} else {
			_vala_code_node_unref0 (condition);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp2_ = vala_parser_parse_embedded_statement (self, "while", TRUE, &_inner_error0_);
	body = _tmp2_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (condition);
			return NULL;
		} else {
			_vala_code_node_unref0 (condition);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp3_ = condition;
	_tmp4_ = body;
	_tmp5_ = begin;
	_tmp6_ = vala_parser_get_src (self, &_tmp5_);
	_tmp7_ = _tmp6_;
	_tmp8_ = vala_while_statement_new (_tmp3_, _tmp4_, _tmp7_);
	_tmp9_ = (ValaStatement*) _tmp8_;
	_vala_source_reference_unref0 (_tmp7_);
	result = _tmp9_;
	_vala_code_node_unref0 (body);
	_vala_code_node_unref0 (condition);
	return result;
}

static ValaStatement*
vala_parser_parse_do_statement (ValaParser* self,
                                GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaBlock* body = NULL;
	ValaBlock* _tmp1_;
	ValaExpression* condition = NULL;
	ValaExpression* _tmp2_;
	ValaExpression* _tmp3_;
	ValaBlock* _tmp4_;
	ValaSourceLocation _tmp5_;
	ValaSourceReference* _tmp6_;
	ValaSourceReference* _tmp7_;
	ValaDoStatement* _tmp8_;
	ValaStatement* _tmp9_;
	GError* _inner_error0_ = NULL;
	ValaStatement* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_DO, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp1_ = vala_parser_parse_embedded_statement (self, "do", TRUE, &_inner_error0_);
	body = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_WHILE, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (body);
			return NULL;
		} else {
			_vala_code_node_unref0 (body);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (body);
			return NULL;
		} else {
			_vala_code_node_unref0 (body);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp2_ = vala_parser_parse_expression (self, &_inner_error0_);
	condition = _tmp2_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (body);
			return NULL;
		} else {
			_vala_code_node_unref0 (body);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (condition);
			_vala_code_node_unref0 (body);
			return NULL;
		} else {
			_vala_code_node_unref0 (condition);
			_vala_code_node_unref0 (body);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_SEMICOLON, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (condition);
			_vala_code_node_unref0 (body);
			return NULL;
		} else {
			_vala_code_node_unref0 (condition);
			_vala_code_node_unref0 (body);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp3_ = condition;
	_tmp4_ = body;
	_tmp5_ = begin;
	_tmp6_ = vala_parser_get_src (self, &_tmp5_);
	_tmp7_ = _tmp6_;
	_tmp8_ = vala_do_statement_new (_tmp3_, _tmp4_, _tmp7_);
	_tmp9_ = (ValaStatement*) _tmp8_;
	_vala_source_reference_unref0 (_tmp7_);
	result = _tmp9_;
	_vala_code_node_unref0 (condition);
	_vala_code_node_unref0 (body);
	return result;
}

static ValaStatement*
vala_parser_parse_for_statement (ValaParser* self,
                                 GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaBlock* block = NULL;
	ValaArrayList* initializer_list = NULL;
	GEqualFunc _tmp1_;
	ValaArrayList* _tmp2_;
	ValaExpression* condition = NULL;
	ValaArrayList* iterator_list = NULL;
	GEqualFunc _tmp24_;
	ValaArrayList* _tmp25_;
	ValaSourceReference* src = NULL;
	ValaSourceLocation _tmp30_;
	ValaSourceReference* _tmp31_;
	ValaBlock* body = NULL;
	ValaBlock* _tmp32_;
	ValaForStatement* stmt = NULL;
	ValaExpression* _tmp33_;
	ValaBlock* _tmp34_;
	ValaSourceReference* _tmp35_;
	ValaForStatement* _tmp36_;
	ValaBlock* _tmp57_;
	GError* _inner_error0_ = NULL;
	ValaStatement* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	block = NULL;
	vala_parser_expect (self, VALA_TOKEN_TYPE_FOR, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (block);
			return NULL;
		} else {
			_vala_code_node_unref0 (block);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (block);
			return NULL;
		} else {
			_vala_code_node_unref0 (block);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp1_ = g_direct_equal;
	_tmp2_ = vala_array_list_new (VALA_TYPE_EXPRESSION, (GBoxedCopyFunc) vala_code_node_ref, (GDestroyNotify) vala_code_node_unref, _tmp1_);
	initializer_list = _tmp2_;
	if (!vala_parser_accept (self, VALA_TOKEN_TYPE_SEMICOLON)) {
		gboolean is_expr = FALSE;
		switch (vala_parser_current (self)) {
			case VALA_TOKEN_TYPE_VAR:
			{
				is_expr = FALSE;
				break;
			}
			case VALA_TOKEN_TYPE_OP_INC:
			case VALA_TOKEN_TYPE_OP_DEC:
			{
				is_expr = TRUE;
				break;
			}
			default:
			{
				gboolean _tmp3_ = FALSE;
				_tmp3_ = vala_parser_is_expression (self, &_inner_error0_);
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_iterable_unref0 (initializer_list);
						_vala_code_node_unref0 (block);
						return NULL;
					} else {
						_vala_iterable_unref0 (initializer_list);
						_vala_code_node_unref0 (block);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return NULL;
					}
				}
				is_expr = _tmp3_;
				break;
			}
		}
		if (is_expr) {
			{
				gboolean _tmp4_ = FALSE;
				_tmp4_ = TRUE;
				while (TRUE) {
					ValaExpression* _tmp5_ = NULL;
					ValaExpression* _tmp6_;
					ValaArrayList* _tmp7_;
					if (!_tmp4_) {
						if (!vala_parser_accept (self, VALA_TOKEN_TYPE_COMMA)) {
							break;
						}
					}
					_tmp4_ = FALSE;
					_tmp6_ = vala_parser_parse_statement_expression (self, &_inner_error0_);
					_tmp5_ = _tmp6_;
					if (G_UNLIKELY (_inner_error0_ != NULL)) {
						if (_inner_error0_->domain == VALA_PARSE_ERROR) {
							g_propagate_error (error, _inner_error0_);
							_vala_iterable_unref0 (initializer_list);
							_vala_code_node_unref0 (block);
							return NULL;
						} else {
							_vala_iterable_unref0 (initializer_list);
							_vala_code_node_unref0 (block);
							g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
							g_clear_error (&_inner_error0_);
							return NULL;
						}
					}
					_tmp7_ = initializer_list;
					vala_collection_add ((ValaCollection*) _tmp7_, _tmp5_);
					_vala_code_node_unref0 (_tmp5_);
				}
			}
			vala_parser_expect (self, VALA_TOKEN_TYPE_SEMICOLON, &_inner_error0_);
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_iterable_unref0 (initializer_list);
					_vala_code_node_unref0 (block);
					return NULL;
				} else {
					_vala_iterable_unref0 (initializer_list);
					_vala_code_node_unref0 (block);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
		} else {
			ValaSourceLocation decl_begin = {0};
			ValaSourceLocation _tmp8_ = {0};
			ValaSourceLocation _tmp9_;
			ValaSourceReference* _tmp10_;
			ValaSourceReference* _tmp11_;
			ValaBlock* _tmp12_;
			ValaBlock* _tmp13_;
			ValaBlock* _tmp14_;
			ValaSourceReference* _tmp15_;
			ValaSourceReference* _tmp16_;
			ValaSourceReference* _tmp17_;
			ValaSourceReference* _tmp18_;
			ValaSourceLocation _tmp19_ = {0};
			ValaSourceLocation _tmp20_;
			vala_parser_get_location (self, &_tmp8_);
			decl_begin = _tmp8_;
			_tmp9_ = decl_begin;
			_tmp10_ = vala_parser_get_src (self, &_tmp9_);
			_tmp11_ = _tmp10_;
			_tmp12_ = vala_block_new (_tmp11_);
			_vala_code_node_unref0 (block);
			block = _tmp12_;
			_vala_source_reference_unref0 (_tmp11_);
			_tmp13_ = block;
			vala_parser_parse_local_variable_declarations (self, _tmp13_, &_inner_error0_);
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_iterable_unref0 (initializer_list);
					_vala_code_node_unref0 (block);
					return NULL;
				} else {
					_vala_iterable_unref0 (initializer_list);
					_vala_code_node_unref0 (block);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp14_ = block;
			_tmp15_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp14_);
			_tmp16_ = _tmp15_;
			_tmp17_ = vala_parser_get_last_src (self);
			_tmp18_ = _tmp17_;
			vala_source_reference_get_end (_tmp18_, &_tmp19_);
			_tmp20_ = _tmp19_;
			vala_source_reference_set_end (_tmp16_, &_tmp20_);
			_vala_source_reference_unref0 (_tmp18_);
		}
	}
	condition = NULL;
	if (vala_parser_current (self) != VALA_TOKEN_TYPE_SEMICOLON) {
		ValaExpression* _tmp21_ = NULL;
		ValaExpression* _tmp22_;
		ValaExpression* _tmp23_;
		_tmp22_ = vala_parser_parse_expression (self, &_inner_error0_);
		_tmp21_ = _tmp22_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (condition);
				_vala_iterable_unref0 (initializer_list);
				_vala_code_node_unref0 (block);
				return NULL;
			} else {
				_vala_code_node_unref0 (condition);
				_vala_iterable_unref0 (initializer_list);
				_vala_code_node_unref0 (block);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp23_ = _tmp21_;
		_tmp21_ = NULL;
		_vala_code_node_unref0 (condition);
		condition = _tmp23_;
		_vala_code_node_unref0 (_tmp21_);
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_SEMICOLON, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (condition);
			_vala_iterable_unref0 (initializer_list);
			_vala_code_node_unref0 (block);
			return NULL;
		} else {
			_vala_code_node_unref0 (condition);
			_vala_iterable_unref0 (initializer_list);
			_vala_code_node_unref0 (block);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp24_ = g_direct_equal;
	_tmp25_ = vala_array_list_new (VALA_TYPE_EXPRESSION, (GBoxedCopyFunc) vala_code_node_ref, (GDestroyNotify) vala_code_node_unref, _tmp24_);
	iterator_list = _tmp25_;
	if (vala_parser_current (self) != VALA_TOKEN_TYPE_CLOSE_PARENS) {
		{
			gboolean _tmp26_ = FALSE;
			_tmp26_ = TRUE;
			while (TRUE) {
				ValaExpression* _tmp27_ = NULL;
				ValaExpression* _tmp28_;
				ValaArrayList* _tmp29_;
				if (!_tmp26_) {
					if (!vala_parser_accept (self, VALA_TOKEN_TYPE_COMMA)) {
						break;
					}
				}
				_tmp26_ = FALSE;
				_tmp28_ = vala_parser_parse_statement_expression (self, &_inner_error0_);
				_tmp27_ = _tmp28_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_iterable_unref0 (iterator_list);
						_vala_code_node_unref0 (condition);
						_vala_iterable_unref0 (initializer_list);
						_vala_code_node_unref0 (block);
						return NULL;
					} else {
						_vala_iterable_unref0 (iterator_list);
						_vala_code_node_unref0 (condition);
						_vala_iterable_unref0 (initializer_list);
						_vala_code_node_unref0 (block);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return NULL;
					}
				}
				_tmp29_ = iterator_list;
				vala_collection_add ((ValaCollection*) _tmp29_, _tmp27_);
				_vala_code_node_unref0 (_tmp27_);
			}
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_iterable_unref0 (iterator_list);
			_vala_code_node_unref0 (condition);
			_vala_iterable_unref0 (initializer_list);
			_vala_code_node_unref0 (block);
			return NULL;
		} else {
			_vala_iterable_unref0 (iterator_list);
			_vala_code_node_unref0 (condition);
			_vala_iterable_unref0 (initializer_list);
			_vala_code_node_unref0 (block);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp30_ = begin;
	_tmp31_ = vala_parser_get_src (self, &_tmp30_);
	src = _tmp31_;
	_tmp32_ = vala_parser_parse_embedded_statement (self, "for", TRUE, &_inner_error0_);
	body = _tmp32_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_source_reference_unref0 (src);
			_vala_iterable_unref0 (iterator_list);
			_vala_code_node_unref0 (condition);
			_vala_iterable_unref0 (initializer_list);
			_vala_code_node_unref0 (block);
			return NULL;
		} else {
			_vala_source_reference_unref0 (src);
			_vala_iterable_unref0 (iterator_list);
			_vala_code_node_unref0 (condition);
			_vala_iterable_unref0 (initializer_list);
			_vala_code_node_unref0 (block);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp33_ = condition;
	_tmp34_ = body;
	_tmp35_ = src;
	_tmp36_ = vala_for_statement_new (_tmp33_, _tmp34_, _tmp35_);
	stmt = _tmp36_;
	{
		ValaArrayList* _init_list = NULL;
		ValaArrayList* _tmp37_;
		gint _init_size = 0;
		ValaArrayList* _tmp38_;
		gint _tmp39_;
		gint _tmp40_;
		gint _init_index = 0;
		_tmp37_ = initializer_list;
		_init_list = _tmp37_;
		_tmp38_ = _init_list;
		_tmp39_ = vala_collection_get_size ((ValaCollection*) _tmp38_);
		_tmp40_ = _tmp39_;
		_init_size = _tmp40_;
		_init_index = -1;
		while (TRUE) {
			gint _tmp41_;
			gint _tmp42_;
			ValaExpression* init = NULL;
			ValaArrayList* _tmp43_;
			gpointer _tmp44_;
			ValaForStatement* _tmp45_;
			ValaExpression* _tmp46_;
			_init_index = _init_index + 1;
			_tmp41_ = _init_index;
			_tmp42_ = _init_size;
			if (!(_tmp41_ < _tmp42_)) {
				break;
			}
			_tmp43_ = _init_list;
			_tmp44_ = vala_list_get ((ValaList*) _tmp43_, _init_index);
			init = (ValaExpression*) _tmp44_;
			_tmp45_ = stmt;
			_tmp46_ = init;
			vala_for_statement_add_initializer (_tmp45_, _tmp46_);
			_vala_code_node_unref0 (init);
		}
	}
	{
		ValaArrayList* _iter_list = NULL;
		ValaArrayList* _tmp47_;
		gint _iter_size = 0;
		ValaArrayList* _tmp48_;
		gint _tmp49_;
		gint _tmp50_;
		gint _iter_index = 0;
		_tmp47_ = iterator_list;
		_iter_list = _tmp47_;
		_tmp48_ = _iter_list;
		_tmp49_ = vala_collection_get_size ((ValaCollection*) _tmp48_);
		_tmp50_ = _tmp49_;
		_iter_size = _tmp50_;
		_iter_index = -1;
		while (TRUE) {
			gint _tmp51_;
			gint _tmp52_;
			ValaExpression* iter = NULL;
			ValaArrayList* _tmp53_;
			gpointer _tmp54_;
			ValaForStatement* _tmp55_;
			ValaExpression* _tmp56_;
			_iter_index = _iter_index + 1;
			_tmp51_ = _iter_index;
			_tmp52_ = _iter_size;
			if (!(_tmp51_ < _tmp52_)) {
				break;
			}
			_tmp53_ = _iter_list;
			_tmp54_ = vala_list_get ((ValaList*) _tmp53_, _iter_index);
			iter = (ValaExpression*) _tmp54_;
			_tmp55_ = stmt;
			_tmp56_ = iter;
			vala_for_statement_add_iterator (_tmp55_, _tmp56_);
			_vala_code_node_unref0 (iter);
		}
	}
	_tmp57_ = block;
	if (_tmp57_ != NULL) {
		ValaBlock* _tmp58_;
		ValaForStatement* _tmp59_;
		_tmp58_ = block;
		_tmp59_ = stmt;
		vala_block_add_statement (_tmp58_, (ValaStatement*) _tmp59_);
		result = (ValaStatement*) block;
		_vala_code_node_unref0 (stmt);
		_vala_code_node_unref0 (body);
		_vala_source_reference_unref0 (src);
		_vala_iterable_unref0 (iterator_list);
		_vala_code_node_unref0 (condition);
		_vala_iterable_unref0 (initializer_list);
		return result;
	} else {
		result = (ValaStatement*) stmt;
		_vala_code_node_unref0 (body);
		_vala_source_reference_unref0 (src);
		_vala_iterable_unref0 (iterator_list);
		_vala_code_node_unref0 (condition);
		_vala_iterable_unref0 (initializer_list);
		_vala_code_node_unref0 (block);
		return result;
	}
}

static ValaStatement*
vala_parser_parse_foreach_statement (ValaParser* self,
                                     GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaSourceLocation var_or_type = {0};
	ValaSourceLocation _tmp1_ = {0};
	ValaDataType* type = NULL;
	gboolean is_dynamic = FALSE;
	gboolean _tmp2_ = FALSE;
	gchar* id = NULL;
	gchar* _tmp18_;
	ValaExpression* collection = NULL;
	ValaExpression* _tmp19_;
	ValaSourceReference* src = NULL;
	ValaSourceLocation _tmp20_;
	ValaSourceReference* _tmp21_;
	ValaBlock* body = NULL;
	ValaBlock* _tmp22_;
	ValaDataType* _tmp23_;
	const gchar* _tmp24_;
	ValaExpression* _tmp25_;
	ValaBlock* _tmp26_;
	ValaSourceReference* _tmp27_;
	ValaForeachStatement* _tmp28_;
	GError* _inner_error0_ = NULL;
	ValaStatement* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_FOREACH, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	vala_parser_get_location (self, &_tmp1_);
	var_or_type = _tmp1_;
	is_dynamic = vala_parser_accept (self, VALA_TOKEN_TYPE_DYNAMIC);
	if (vala_parser_accept (self, VALA_TOKEN_TYPE_UNOWNED)) {
		_tmp2_ = vala_parser_accept (self, VALA_TOKEN_TYPE_VAR);
	} else {
		_tmp2_ = FALSE;
	}
	if (_tmp2_) {
		ValaVarType* _tmp3_;
		ValaDataType* _tmp4_;
		ValaDataType* _tmp5_;
		_tmp3_ = vala_var_type_new (FALSE);
		_vala_code_node_unref0 (type);
		type = (ValaDataType*) _tmp3_;
		_tmp4_ = type;
		vala_data_type_set_nullable (_tmp4_, vala_parser_accept (self, VALA_TOKEN_TYPE_INTERR));
		_tmp5_ = type;
		vala_data_type_set_is_dynamic (_tmp5_, is_dynamic);
	} else {
		ValaSourceLocation _tmp6_;
		_tmp6_ = var_or_type;
		vala_parser_rollback (self, &_tmp6_);
		is_dynamic = vala_parser_accept (self, VALA_TOKEN_TYPE_DYNAMIC);
		if (vala_parser_accept (self, VALA_TOKEN_TYPE_VAR)) {
			ValaVarType* _tmp7_;
			ValaDataType* _tmp8_;
			ValaDataType* _tmp9_;
			_tmp7_ = vala_var_type_new (TRUE);
			_vala_code_node_unref0 (type);
			type = (ValaDataType*) _tmp7_;
			_tmp8_ = type;
			vala_data_type_set_nullable (_tmp8_, vala_parser_accept (self, VALA_TOKEN_TYPE_INTERR));
			_tmp9_ = type;
			vala_data_type_set_is_dynamic (_tmp9_, is_dynamic);
		} else {
			ValaSourceLocation _tmp10_;
			ValaDataType* _tmp11_ = NULL;
			ValaDataType* _tmp12_;
			ValaDataType* _tmp13_;
			_tmp10_ = var_or_type;
			vala_parser_rollback (self, &_tmp10_);
			_tmp12_ = vala_parser_parse_type (self, TRUE, TRUE, FALSE, &_inner_error0_);
			_tmp11_ = _tmp12_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (type);
					return NULL;
				} else {
					_vala_code_node_unref0 (type);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp13_ = _tmp11_;
			_tmp11_ = NULL;
			_vala_code_node_unref0 (type);
			type = _tmp13_;
			if (vala_parser_accept (self, VALA_TOKEN_TYPE_IN)) {
				ValaDataType* _tmp14_;
				ValaSourceReference* _tmp15_;
				ValaSourceReference* _tmp16_;
				GError* _tmp17_;
				_tmp14_ = type;
				_tmp15_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp14_);
				_tmp16_ = _tmp15_;
				vala_report_error (_tmp16_, "syntax error, expected `unowned var', `var' or type");
				_tmp17_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "expected `unowned var', `var' or type");
				_inner_error0_ = _tmp17_;
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (_tmp11_);
					_vala_code_node_unref0 (type);
					return NULL;
				} else {
					_vala_code_node_unref0 (_tmp11_);
					_vala_code_node_unref0 (type);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_vala_code_node_unref0 (_tmp11_);
		}
	}
	_tmp18_ = vala_parser_parse_identifier (self, FALSE, &_inner_error0_);
	id = _tmp18_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (type);
			return NULL;
		} else {
			_vala_code_node_unref0 (type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_IN, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_g_free0 (id);
			_vala_code_node_unref0 (type);
			return NULL;
		} else {
			_g_free0 (id);
			_vala_code_node_unref0 (type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp19_ = vala_parser_parse_expression (self, &_inner_error0_);
	collection = _tmp19_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_g_free0 (id);
			_vala_code_node_unref0 (type);
			return NULL;
		} else {
			_g_free0 (id);
			_vala_code_node_unref0 (type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (collection);
			_g_free0 (id);
			_vala_code_node_unref0 (type);
			return NULL;
		} else {
			_vala_code_node_unref0 (collection);
			_g_free0 (id);
			_vala_code_node_unref0 (type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp20_ = begin;
	_tmp21_ = vala_parser_get_src (self, &_tmp20_);
	src = _tmp21_;
	_tmp22_ = vala_parser_parse_embedded_statement (self, "foreach", TRUE, &_inner_error0_);
	body = _tmp22_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_source_reference_unref0 (src);
			_vala_code_node_unref0 (collection);
			_g_free0 (id);
			_vala_code_node_unref0 (type);
			return NULL;
		} else {
			_vala_source_reference_unref0 (src);
			_vala_code_node_unref0 (collection);
			_g_free0 (id);
			_vala_code_node_unref0 (type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp23_ = type;
	_tmp24_ = id;
	_tmp25_ = collection;
	_tmp26_ = body;
	_tmp27_ = src;
	_tmp28_ = vala_foreach_statement_new (_tmp23_, _tmp24_, _tmp25_, _tmp26_, _tmp27_);
	result = (ValaStatement*) _tmp28_;
	_vala_code_node_unref0 (body);
	_vala_source_reference_unref0 (src);
	_vala_code_node_unref0 (collection);
	_g_free0 (id);
	_vala_code_node_unref0 (type);
	return result;
}

static ValaStatement*
vala_parser_parse_break_statement (ValaParser* self,
                                   GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaSourceReference* src = NULL;
	ValaSourceLocation _tmp1_;
	ValaSourceReference* _tmp2_;
	ValaSourceReference* _tmp3_;
	ValaBreakStatement* _tmp4_;
	GError* _inner_error0_ = NULL;
	ValaStatement* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_BREAK, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp1_ = begin;
	_tmp2_ = vala_parser_get_src (self, &_tmp1_);
	src = _tmp2_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_SEMICOLON, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_source_reference_unref0 (src);
			return NULL;
		} else {
			_vala_source_reference_unref0 (src);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp3_ = src;
	_tmp4_ = vala_break_statement_new (_tmp3_);
	result = (ValaStatement*) _tmp4_;
	_vala_source_reference_unref0 (src);
	return result;
}

static ValaStatement*
vala_parser_parse_continue_statement (ValaParser* self,
                                      GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaSourceReference* src = NULL;
	ValaSourceLocation _tmp1_;
	ValaSourceReference* _tmp2_;
	ValaSourceReference* _tmp3_;
	ValaContinueStatement* _tmp4_;
	GError* _inner_error0_ = NULL;
	ValaStatement* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_CONTINUE, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp1_ = begin;
	_tmp2_ = vala_parser_get_src (self, &_tmp1_);
	src = _tmp2_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_SEMICOLON, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_source_reference_unref0 (src);
			return NULL;
		} else {
			_vala_source_reference_unref0 (src);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp3_ = src;
	_tmp4_ = vala_continue_statement_new (_tmp3_);
	result = (ValaStatement*) _tmp4_;
	_vala_source_reference_unref0 (src);
	return result;
}

static ValaStatement*
vala_parser_parse_return_statement (ValaParser* self,
                                    GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaExpression* expr = NULL;
	ValaSourceReference* src = NULL;
	ValaSourceLocation _tmp4_;
	ValaSourceReference* _tmp5_;
	ValaExpression* _tmp6_;
	ValaSourceReference* _tmp7_;
	ValaReturnStatement* _tmp8_;
	GError* _inner_error0_ = NULL;
	ValaStatement* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_RETURN, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	expr = NULL;
	if (vala_parser_current (self) != VALA_TOKEN_TYPE_SEMICOLON) {
		ValaExpression* _tmp1_ = NULL;
		ValaExpression* _tmp2_;
		ValaExpression* _tmp3_;
		_tmp2_ = vala_parser_parse_expression (self, &_inner_error0_);
		_tmp1_ = _tmp2_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (expr);
				return NULL;
			} else {
				_vala_code_node_unref0 (expr);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp3_ = _tmp1_;
		_tmp1_ = NULL;
		_vala_code_node_unref0 (expr);
		expr = _tmp3_;
		_vala_code_node_unref0 (_tmp1_);
	}
	_tmp4_ = begin;
	_tmp5_ = vala_parser_get_src (self, &_tmp4_);
	src = _tmp5_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_SEMICOLON, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_source_reference_unref0 (src);
			_vala_code_node_unref0 (expr);
			return NULL;
		} else {
			_vala_source_reference_unref0 (src);
			_vala_code_node_unref0 (expr);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp6_ = expr;
	_tmp7_ = src;
	_tmp8_ = vala_return_statement_new (_tmp6_, _tmp7_);
	result = (ValaStatement*) _tmp8_;
	_vala_source_reference_unref0 (src);
	_vala_code_node_unref0 (expr);
	return result;
}

static ValaStatement*
vala_parser_parse_yield_statement (ValaParser* self,
                                   GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaTokenType token = 0;
	ValaSourceReference* src = NULL;
	ValaSourceLocation _tmp5_;
	ValaSourceReference* _tmp6_;
	ValaSourceReference* _tmp7_;
	ValaYieldStatement* _tmp8_;
	GError* _inner_error0_ = NULL;
	ValaStatement* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_YIELD, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	token = vala_parser_current (self);
	if (token != VALA_TOKEN_TYPE_SEMICOLON) {
		ValaStatement* _tmp2_ = NULL;
		ValaStatement* _tmp3_;
		ValaStatement* _tmp4_;
		vala_parser_prev (self);
		if (token == VALA_TOKEN_TYPE_RETURN) {
			GError* _tmp1_;
			_tmp1_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "expected `return yield'");
			_inner_error0_ = _tmp1_;
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				return NULL;
			} else {
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp3_ = vala_parser_parse_expression_statement (self, &_inner_error0_);
		_tmp2_ = _tmp3_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				return NULL;
			} else {
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp4_ = _tmp2_;
		_tmp2_ = NULL;
		result = _tmp4_;
		_vala_code_node_unref0 (_tmp2_);
		return result;
	}
	_tmp5_ = begin;
	_tmp6_ = vala_parser_get_src (self, &_tmp5_);
	src = _tmp6_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_SEMICOLON, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_source_reference_unref0 (src);
			return NULL;
		} else {
			_vala_source_reference_unref0 (src);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp7_ = src;
	_tmp8_ = vala_yield_statement_new (_tmp7_);
	result = (ValaStatement*) _tmp8_;
	_vala_source_reference_unref0 (src);
	return result;
}

static ValaStatement*
vala_parser_parse_throw_statement (ValaParser* self,
                                   GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaExpression* expr = NULL;
	ValaExpression* _tmp1_;
	ValaSourceReference* src = NULL;
	ValaSourceLocation _tmp2_;
	ValaSourceReference* _tmp3_;
	ValaExpression* _tmp4_;
	ValaSourceReference* _tmp5_;
	ValaThrowStatement* _tmp6_;
	GError* _inner_error0_ = NULL;
	ValaStatement* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_THROW, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp1_ = vala_parser_parse_expression (self, &_inner_error0_);
	expr = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp2_ = begin;
	_tmp3_ = vala_parser_get_src (self, &_tmp2_);
	src = _tmp3_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_SEMICOLON, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_source_reference_unref0 (src);
			_vala_code_node_unref0 (expr);
			return NULL;
		} else {
			_vala_source_reference_unref0 (src);
			_vala_code_node_unref0 (expr);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp4_ = expr;
	_tmp5_ = src;
	_tmp6_ = vala_throw_statement_new (_tmp4_, _tmp5_);
	result = (ValaStatement*) _tmp6_;
	_vala_source_reference_unref0 (src);
	_vala_code_node_unref0 (expr);
	return result;
}

static ValaStatement*
vala_parser_parse_try_statement (ValaParser* self,
                                 GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaBlock* try_block = NULL;
	ValaBlock* _tmp1_;
	ValaBlock* finally_clause = NULL;
	ValaArrayList* catch_clauses = NULL;
	GEqualFunc _tmp2_;
	ValaArrayList* _tmp3_;
	ValaTryStatement* stmt = NULL;
	ValaBlock* _tmp13_;
	ValaBlock* _tmp14_;
	ValaSourceLocation _tmp15_;
	ValaSourceReference* _tmp16_;
	ValaSourceReference* _tmp17_;
	ValaTryStatement* _tmp18_;
	ValaTryStatement* _tmp19_;
	GError* _inner_error0_ = NULL;
	ValaStatement* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_TRY, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp1_ = vala_parser_parse_block (self, &_inner_error0_);
	try_block = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	finally_clause = NULL;
	_tmp2_ = g_direct_equal;
	_tmp3_ = vala_array_list_new (VALA_TYPE_CATCH_CLAUSE, (GBoxedCopyFunc) vala_code_node_ref, (GDestroyNotify) vala_code_node_unref, _tmp2_);
	catch_clauses = _tmp3_;
	if (vala_parser_current (self) == VALA_TOKEN_TYPE_CATCH) {
		ValaArrayList* _tmp4_;
		_tmp4_ = catch_clauses;
		vala_parser_parse_catch_clauses (self, (ValaList*) _tmp4_, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_iterable_unref0 (catch_clauses);
				_vala_code_node_unref0 (finally_clause);
				_vala_code_node_unref0 (try_block);
				return NULL;
			} else {
				_vala_iterable_unref0 (catch_clauses);
				_vala_code_node_unref0 (finally_clause);
				_vala_code_node_unref0 (try_block);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		if (vala_parser_current (self) == VALA_TOKEN_TYPE_FINALLY) {
			ValaBlock* _tmp5_ = NULL;
			ValaBlock* _tmp6_;
			ValaBlock* _tmp7_;
			_tmp6_ = vala_parser_parse_finally_clause (self, &_inner_error0_);
			_tmp5_ = _tmp6_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_iterable_unref0 (catch_clauses);
					_vala_code_node_unref0 (finally_clause);
					_vala_code_node_unref0 (try_block);
					return NULL;
				} else {
					_vala_iterable_unref0 (catch_clauses);
					_vala_code_node_unref0 (finally_clause);
					_vala_code_node_unref0 (try_block);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp7_ = _tmp5_;
			_tmp5_ = NULL;
			_vala_code_node_unref0 (finally_clause);
			finally_clause = _tmp7_;
			_vala_code_node_unref0 (_tmp5_);
		}
	} else {
		if (vala_parser_current (self) == VALA_TOKEN_TYPE_FINALLY) {
			ValaBlock* _tmp8_ = NULL;
			ValaBlock* _tmp9_;
			ValaBlock* _tmp10_;
			_tmp9_ = vala_parser_parse_finally_clause (self, &_inner_error0_);
			_tmp8_ = _tmp9_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_iterable_unref0 (catch_clauses);
					_vala_code_node_unref0 (finally_clause);
					_vala_code_node_unref0 (try_block);
					return NULL;
				} else {
					_vala_iterable_unref0 (catch_clauses);
					_vala_code_node_unref0 (finally_clause);
					_vala_code_node_unref0 (try_block);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp10_ = _tmp8_;
			_tmp8_ = NULL;
			_vala_code_node_unref0 (finally_clause);
			finally_clause = _tmp10_;
			_vala_code_node_unref0 (_tmp8_);
		} else {
			GError* _tmp11_;
			GError* _tmp12_;
			_tmp11_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "expected `catch' or `finally'");
			_tmp12_ = _tmp11_;
			vala_parser_report_parse_error (self, _tmp12_);
			_g_error_free0 (_tmp12_);
		}
	}
	_tmp13_ = try_block;
	_tmp14_ = finally_clause;
	_tmp15_ = begin;
	_tmp16_ = vala_parser_get_src (self, &_tmp15_);
	_tmp17_ = _tmp16_;
	_tmp18_ = vala_try_statement_new (_tmp13_, _tmp14_, _tmp17_);
	_tmp19_ = _tmp18_;
	_vala_source_reference_unref0 (_tmp17_);
	stmt = _tmp19_;
	{
		ValaArrayList* _clause_list = NULL;
		ValaArrayList* _tmp20_;
		gint _clause_size = 0;
		ValaArrayList* _tmp21_;
		gint _tmp22_;
		gint _tmp23_;
		gint _clause_index = 0;
		_tmp20_ = catch_clauses;
		_clause_list = _tmp20_;
		_tmp21_ = _clause_list;
		_tmp22_ = vala_collection_get_size ((ValaCollection*) _tmp21_);
		_tmp23_ = _tmp22_;
		_clause_size = _tmp23_;
		_clause_index = -1;
		while (TRUE) {
			gint _tmp24_;
			gint _tmp25_;
			ValaCatchClause* clause = NULL;
			ValaArrayList* _tmp26_;
			gpointer _tmp27_;
			ValaTryStatement* _tmp28_;
			ValaCatchClause* _tmp29_;
			_clause_index = _clause_index + 1;
			_tmp24_ = _clause_index;
			_tmp25_ = _clause_size;
			if (!(_tmp24_ < _tmp25_)) {
				break;
			}
			_tmp26_ = _clause_list;
			_tmp27_ = vala_list_get ((ValaList*) _tmp26_, _clause_index);
			clause = (ValaCatchClause*) _tmp27_;
			_tmp28_ = stmt;
			_tmp29_ = clause;
			vala_try_statement_add_catch_clause (_tmp28_, _tmp29_);
			_vala_code_node_unref0 (clause);
		}
	}
	result = (ValaStatement*) stmt;
	_vala_iterable_unref0 (catch_clauses);
	_vala_code_node_unref0 (finally_clause);
	_vala_code_node_unref0 (try_block);
	return result;
}

static void
vala_parser_parse_catch_clauses (ValaParser* self,
                                 ValaList* catch_clauses,
                                 GError** error)
{
	GError* _inner_error0_ = NULL;
	g_return_if_fail (self != NULL);
	g_return_if_fail (catch_clauses != NULL);
	while (TRUE) {
		ValaSourceLocation begin = {0};
		ValaSourceLocation _tmp0_ = {0};
		ValaDataType* type = NULL;
		gchar* id = NULL;
		ValaSourceReference* src = NULL;
		ValaSourceLocation _tmp7_;
		ValaSourceReference* _tmp8_;
		ValaBlock* block = NULL;
		ValaBlock* _tmp9_;
		ValaDataType* _tmp10_;
		const gchar* _tmp11_;
		ValaBlock* _tmp12_;
		ValaSourceReference* _tmp13_;
		ValaCatchClause* _tmp14_;
		ValaCatchClause* _tmp15_;
		if (!(vala_parser_current (self) == VALA_TOKEN_TYPE_CATCH)) {
			break;
		}
		vala_parser_get_location (self, &_tmp0_);
		begin = _tmp0_;
		vala_parser_expect (self, VALA_TOKEN_TYPE_CATCH, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				return;
			} else {
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
		type = NULL;
		id = NULL;
		if (vala_parser_accept (self, VALA_TOKEN_TYPE_OPEN_PARENS)) {
			ValaDataType* _tmp1_ = NULL;
			ValaDataType* _tmp2_;
			ValaDataType* _tmp3_;
			gchar* _tmp4_ = NULL;
			gchar* _tmp5_;
			gchar* _tmp6_;
			_tmp2_ = vala_parser_parse_type (self, TRUE, TRUE, FALSE, &_inner_error0_);
			_tmp1_ = _tmp2_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_g_free0 (id);
					_vala_code_node_unref0 (type);
					return;
				} else {
					_g_free0 (id);
					_vala_code_node_unref0 (type);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return;
				}
			}
			_tmp3_ = _tmp1_;
			_tmp1_ = NULL;
			_vala_code_node_unref0 (type);
			type = _tmp3_;
			_tmp5_ = vala_parser_parse_identifier (self, FALSE, &_inner_error0_);
			_tmp4_ = _tmp5_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (_tmp1_);
					_g_free0 (id);
					_vala_code_node_unref0 (type);
					return;
				} else {
					_vala_code_node_unref0 (_tmp1_);
					_g_free0 (id);
					_vala_code_node_unref0 (type);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return;
				}
			}
			_tmp6_ = _tmp4_;
			_tmp4_ = NULL;
			_g_free0 (id);
			id = _tmp6_;
			vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_PARENS, &_inner_error0_);
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_g_free0 (_tmp4_);
					_vala_code_node_unref0 (_tmp1_);
					_g_free0 (id);
					_vala_code_node_unref0 (type);
					return;
				} else {
					_g_free0 (_tmp4_);
					_vala_code_node_unref0 (_tmp1_);
					_g_free0 (id);
					_vala_code_node_unref0 (type);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return;
				}
			}
			_g_free0 (_tmp4_);
			_vala_code_node_unref0 (_tmp1_);
		}
		_tmp7_ = begin;
		_tmp8_ = vala_parser_get_src (self, &_tmp7_);
		src = _tmp8_;
		_tmp9_ = vala_parser_parse_block (self, &_inner_error0_);
		block = _tmp9_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_source_reference_unref0 (src);
				_g_free0 (id);
				_vala_code_node_unref0 (type);
				return;
			} else {
				_vala_source_reference_unref0 (src);
				_g_free0 (id);
				_vala_code_node_unref0 (type);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
		_tmp10_ = type;
		_tmp11_ = id;
		_tmp12_ = block;
		_tmp13_ = src;
		_tmp14_ = vala_catch_clause_new (_tmp10_, _tmp11_, _tmp12_, _tmp13_);
		_tmp15_ = _tmp14_;
		vala_collection_add ((ValaCollection*) catch_clauses, _tmp15_);
		_vala_code_node_unref0 (_tmp15_);
		_vala_code_node_unref0 (block);
		_vala_source_reference_unref0 (src);
		_g_free0 (id);
		_vala_code_node_unref0 (type);
	}
}

static ValaBlock*
vala_parser_parse_finally_clause (ValaParser* self,
                                  GError** error)
{
	ValaBlock* block = NULL;
	ValaBlock* _tmp0_;
	GError* _inner_error0_ = NULL;
	ValaBlock* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_expect (self, VALA_TOKEN_TYPE_FINALLY, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp0_ = vala_parser_parse_block (self, &_inner_error0_);
	block = _tmp0_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	result = block;
	return result;
}

static ValaStatement*
vala_parser_parse_lock_statement (ValaParser* self,
                                  GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaExpression* expr = NULL;
	ValaExpression* _tmp1_;
	ValaSourceReference* src = NULL;
	ValaSourceLocation _tmp2_;
	ValaSourceReference* _tmp3_;
	ValaBlock* stmt = NULL;
	ValaExpression* _tmp7_;
	ValaBlock* _tmp8_;
	ValaSourceReference* _tmp9_;
	ValaLockStatement* _tmp10_;
	GError* _inner_error0_ = NULL;
	ValaStatement* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_LOCK, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp1_ = vala_parser_parse_expression (self, &_inner_error0_);
	expr = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (expr);
			return NULL;
		} else {
			_vala_code_node_unref0 (expr);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp2_ = begin;
	_tmp3_ = vala_parser_get_src (self, &_tmp2_);
	src = _tmp3_;
	stmt = NULL;
	if (vala_parser_current (self) != VALA_TOKEN_TYPE_SEMICOLON) {
		ValaBlock* _tmp4_ = NULL;
		ValaBlock* _tmp5_;
		ValaBlock* _tmp6_;
		_tmp5_ = vala_parser_parse_embedded_statement (self, "lock", FALSE, &_inner_error0_);
		_tmp4_ = _tmp5_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (stmt);
				_vala_source_reference_unref0 (src);
				_vala_code_node_unref0 (expr);
				return NULL;
			} else {
				_vala_code_node_unref0 (stmt);
				_vala_source_reference_unref0 (src);
				_vala_code_node_unref0 (expr);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp6_ = _tmp4_;
		_tmp4_ = NULL;
		_vala_code_node_unref0 (stmt);
		stmt = _tmp6_;
		_vala_code_node_unref0 (_tmp4_);
	}
	_tmp7_ = expr;
	_tmp8_ = stmt;
	_tmp9_ = src;
	_tmp10_ = vala_lock_statement_new (_tmp7_, _tmp8_, _tmp9_);
	result = (ValaStatement*) _tmp10_;
	_vala_code_node_unref0 (stmt);
	_vala_source_reference_unref0 (src);
	_vala_code_node_unref0 (expr);
	return result;
}

static ValaStatement*
vala_parser_parse_unlock_statement (ValaParser* self,
                                    GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaExpression* expr = NULL;
	ValaExpression* _tmp1_;
	ValaSourceReference* src = NULL;
	ValaSourceLocation _tmp2_;
	ValaSourceReference* _tmp3_;
	ValaExpression* _tmp4_;
	ValaSourceReference* _tmp5_;
	ValaUnlockStatement* _tmp6_;
	GError* _inner_error0_ = NULL;
	ValaStatement* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_UNLOCK, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp1_ = vala_parser_parse_expression (self, &_inner_error0_);
	expr = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (expr);
			return NULL;
		} else {
			_vala_code_node_unref0 (expr);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp2_ = begin;
	_tmp3_ = vala_parser_get_src (self, &_tmp2_);
	src = _tmp3_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_SEMICOLON, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_source_reference_unref0 (src);
			_vala_code_node_unref0 (expr);
			return NULL;
		} else {
			_vala_source_reference_unref0 (src);
			_vala_code_node_unref0 (expr);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp4_ = expr;
	_tmp5_ = src;
	_tmp6_ = vala_unlock_statement_new (_tmp4_, _tmp5_);
	result = (ValaStatement*) _tmp6_;
	_vala_source_reference_unref0 (src);
	_vala_code_node_unref0 (expr);
	return result;
}

static ValaStatement*
vala_parser_parse_delete_statement (ValaParser* self,
                                    GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaExpression* expr = NULL;
	ValaExpression* _tmp1_;
	ValaSourceReference* src = NULL;
	ValaSourceLocation _tmp2_;
	ValaSourceReference* _tmp3_;
	ValaExpression* _tmp4_;
	ValaSourceReference* _tmp5_;
	ValaDeleteStatement* _tmp6_;
	GError* _inner_error0_ = NULL;
	ValaStatement* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_DELETE, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp1_ = vala_parser_parse_expression (self, &_inner_error0_);
	expr = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp2_ = begin;
	_tmp3_ = vala_parser_get_src (self, &_tmp2_);
	src = _tmp3_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_SEMICOLON, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_source_reference_unref0 (src);
			_vala_code_node_unref0 (expr);
			return NULL;
		} else {
			_vala_source_reference_unref0 (src);
			_vala_code_node_unref0 (expr);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp4_ = expr;
	_tmp5_ = src;
	_tmp6_ = vala_delete_statement_new (_tmp4_, _tmp5_);
	result = (ValaStatement*) _tmp6_;
	_vala_source_reference_unref0 (src);
	_vala_code_node_unref0 (expr);
	return result;
}

static ValaStatement*
vala_parser_parse_with_statement (ValaParser* self,
                                  GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaSourceLocation expr_or_decl = {0};
	ValaSourceLocation _tmp1_ = {0};
	ValaLocalVariable* local = NULL;
	ValaExpression* expr = NULL;
	gboolean _tmp5_ = FALSE;
	ValaExpression* _tmp6_;
	ValaSourceReference* src = NULL;
	ValaSourceLocation _tmp27_;
	ValaSourceReference* _tmp28_;
	ValaCodeContext* _tmp29_;
	gboolean _tmp30_;
	gboolean _tmp31_;
	ValaBlock* body = NULL;
	ValaBlock* _tmp33_;
	ValaLocalVariable* _tmp34_;
	ValaExpression* _tmp35_;
	ValaBlock* _tmp36_;
	ValaSourceReference* _tmp37_;
	ValaWithStatement* _tmp38_;
	GError* _inner_error0_ = NULL;
	ValaStatement* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_WITH, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	vala_parser_get_location (self, &_tmp1_);
	expr_or_decl = _tmp1_;
	local = NULL;
	{
		ValaExpression* _tmp2_ = NULL;
		ValaExpression* _tmp3_;
		ValaExpression* _tmp4_;
		_tmp3_ = vala_parser_parse_expression (self, &_inner_error0_);
		_tmp2_ = _tmp3_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			goto __catch0_g_error;
		}
		_tmp4_ = _tmp2_;
		_tmp2_ = NULL;
		_vala_code_node_unref0 (expr);
		expr = _tmp4_;
		_vala_code_node_unref0 (_tmp2_);
	}
	goto __finally0;
	__catch0_g_error:
	{
		g_clear_error (&_inner_error0_);
		_vala_code_node_unref0 (expr);
		expr = NULL;
	}
	__finally0:
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (expr);
			_vala_code_node_unref0 (local);
			return NULL;
		} else {
			_vala_code_node_unref0 (expr);
			_vala_code_node_unref0 (local);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp6_ = expr;
	if (_tmp6_ == NULL) {
		_tmp5_ = TRUE;
	} else {
		_tmp5_ = !vala_parser_accept (self, VALA_TOKEN_TYPE_CLOSE_PARENS);
	}
	if (_tmp5_) {
		ValaSourceLocation _tmp7_;
		ValaDataType* variable_type = NULL;
		gboolean is_dynamic = FALSE;
		gboolean _tmp8_ = FALSE;
		ValaLocalVariable* _tmp19_ = NULL;
		ValaDataType* _tmp20_;
		ValaLocalVariable* _tmp21_;
		ValaLocalVariable* _tmp22_;
		ValaLocalVariable* _tmp23_;
		ValaExpression* _tmp24_;
		ValaExpression* _tmp25_;
		ValaExpression* _tmp26_;
		_tmp7_ = expr_or_decl;
		vala_parser_rollback (self, &_tmp7_);
		is_dynamic = vala_parser_accept (self, VALA_TOKEN_TYPE_DYNAMIC);
		if (vala_parser_accept (self, VALA_TOKEN_TYPE_UNOWNED)) {
			_tmp8_ = vala_parser_accept (self, VALA_TOKEN_TYPE_VAR);
		} else {
			_tmp8_ = FALSE;
		}
		if (_tmp8_) {
			ValaVarType* _tmp9_;
			ValaDataType* _tmp10_;
			ValaDataType* _tmp11_;
			_tmp9_ = vala_var_type_new (FALSE);
			_vala_code_node_unref0 (variable_type);
			variable_type = (ValaDataType*) _tmp9_;
			_tmp10_ = variable_type;
			vala_data_type_set_nullable (_tmp10_, vala_parser_accept (self, VALA_TOKEN_TYPE_INTERR));
			_tmp11_ = variable_type;
			vala_data_type_set_is_dynamic (_tmp11_, is_dynamic);
		} else {
			ValaSourceLocation _tmp12_;
			_tmp12_ = expr_or_decl;
			vala_parser_rollback (self, &_tmp12_);
			is_dynamic = vala_parser_accept (self, VALA_TOKEN_TYPE_DYNAMIC);
			if (vala_parser_accept (self, VALA_TOKEN_TYPE_VAR)) {
				ValaVarType* _tmp13_;
				ValaDataType* _tmp14_;
				ValaDataType* _tmp15_;
				_tmp13_ = vala_var_type_new (TRUE);
				_vala_code_node_unref0 (variable_type);
				variable_type = (ValaDataType*) _tmp13_;
				_tmp14_ = variable_type;
				vala_data_type_set_nullable (_tmp14_, vala_parser_accept (self, VALA_TOKEN_TYPE_INTERR));
				_tmp15_ = variable_type;
				vala_data_type_set_is_dynamic (_tmp15_, is_dynamic);
			} else {
				ValaDataType* _tmp16_ = NULL;
				ValaDataType* _tmp17_;
				ValaDataType* _tmp18_;
				_tmp17_ = vala_parser_parse_type (self, TRUE, TRUE, FALSE, &_inner_error0_);
				_tmp16_ = _tmp17_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_code_node_unref0 (variable_type);
						_vala_code_node_unref0 (expr);
						_vala_code_node_unref0 (local);
						return NULL;
					} else {
						_vala_code_node_unref0 (variable_type);
						_vala_code_node_unref0 (expr);
						_vala_code_node_unref0 (local);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return NULL;
					}
				}
				_tmp18_ = _tmp16_;
				_tmp16_ = NULL;
				_vala_code_node_unref0 (variable_type);
				variable_type = _tmp18_;
				_vala_code_node_unref0 (_tmp16_);
			}
		}
		_tmp20_ = variable_type;
		_tmp21_ = vala_parser_parse_local_variable (self, _tmp20_, TRUE, &_inner_error0_);
		_tmp19_ = _tmp21_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (variable_type);
				_vala_code_node_unref0 (expr);
				_vala_code_node_unref0 (local);
				return NULL;
			} else {
				_vala_code_node_unref0 (variable_type);
				_vala_code_node_unref0 (expr);
				_vala_code_node_unref0 (local);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp22_ = _tmp19_;
		_tmp19_ = NULL;
		_vala_code_node_unref0 (local);
		local = _tmp22_;
		_tmp23_ = local;
		_tmp24_ = vala_variable_get_initializer ((ValaVariable*) _tmp23_);
		_tmp25_ = _tmp24_;
		_tmp26_ = _vala_code_node_ref0 (_tmp25_);
		_vala_code_node_unref0 (expr);
		expr = _tmp26_;
		vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_PARENS, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (_tmp19_);
				_vala_code_node_unref0 (variable_type);
				_vala_code_node_unref0 (expr);
				_vala_code_node_unref0 (local);
				return NULL;
			} else {
				_vala_code_node_unref0 (_tmp19_);
				_vala_code_node_unref0 (variable_type);
				_vala_code_node_unref0 (expr);
				_vala_code_node_unref0 (local);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_vala_code_node_unref0 (_tmp19_);
		_vala_code_node_unref0 (variable_type);
	}
	_tmp27_ = begin;
	_tmp28_ = vala_parser_get_src (self, &_tmp27_);
	src = _tmp28_;
	_tmp29_ = self->priv->context;
	_tmp30_ = vala_code_context_get_experimental (_tmp29_);
	_tmp31_ = _tmp30_;
	if (!_tmp31_) {
		ValaSourceReference* _tmp32_;
		_tmp32_ = src;
		vala_report_warning (_tmp32_, "`with' statements are experimental");
	}
	_tmp33_ = vala_parser_parse_embedded_statement (self, "with", FALSE, &_inner_error0_);
	body = _tmp33_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_source_reference_unref0 (src);
			_vala_code_node_unref0 (expr);
			_vala_code_node_unref0 (local);
			return NULL;
		} else {
			_vala_source_reference_unref0 (src);
			_vala_code_node_unref0 (expr);
			_vala_code_node_unref0 (local);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp34_ = local;
	_tmp35_ = expr;
	_tmp36_ = body;
	_tmp37_ = src;
	_tmp38_ = vala_with_statement_new (_tmp34_, _tmp35_, _tmp36_, _tmp37_);
	result = (ValaStatement*) _tmp38_;
	_vala_code_node_unref0 (body);
	_vala_source_reference_unref0 (src);
	_vala_code_node_unref0 (expr);
	_vala_code_node_unref0 (local);
	return result;
}

static gchar*
vala_parser_parse_attribute_value (ValaParser* self,
                                   GError** error)
{
	GError* _inner_error0_ = NULL;
	gchar* result;
	g_return_val_if_fail (self != NULL, NULL);
	switch (vala_parser_current (self)) {
		case VALA_TOKEN_TYPE_NULL:
		case VALA_TOKEN_TYPE_TRUE:
		case VALA_TOKEN_TYPE_FALSE:
		case VALA_TOKEN_TYPE_INTEGER_LITERAL:
		case VALA_TOKEN_TYPE_REAL_LITERAL:
		case VALA_TOKEN_TYPE_STRING_LITERAL:
		{
			gchar* _tmp0_;
			vala_parser_next (self);
			_tmp0_ = vala_parser_get_last_string (self);
			result = _tmp0_;
			return result;
		}
		case VALA_TOKEN_TYPE_MINUS:
		{
			vala_parser_next (self);
			switch (vala_parser_current (self)) {
				case VALA_TOKEN_TYPE_INTEGER_LITERAL:
				case VALA_TOKEN_TYPE_REAL_LITERAL:
				{
					gchar* _tmp1_;
					gchar* _tmp2_;
					gchar* _tmp3_;
					gchar* _tmp4_;
					vala_parser_next (self);
					_tmp1_ = vala_parser_get_last_string (self);
					_tmp2_ = _tmp1_;
					_tmp3_ = g_strconcat ("-", _tmp2_, NULL);
					_tmp4_ = _tmp3_;
					_g_free0 (_tmp2_);
					result = _tmp4_;
					return result;
				}
				default:
				{
					GError* _tmp5_;
					_tmp5_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "expected number");
					_inner_error0_ = _tmp5_;
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						return NULL;
					} else {
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return NULL;
					}
				}
			}
		}
		default:
		{
			GError* _tmp6_;
			_tmp6_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "expected literal");
			_inner_error0_ = _tmp6_;
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				return NULL;
			} else {
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
	}
}

static ValaList*
vala_parser_parse_attributes (ValaParser* self,
                              GError** error)
{
	ValaArrayList* attrs = NULL;
	GEqualFunc _tmp0_;
	ValaArrayList* _tmp1_;
	GError* _inner_error0_ = NULL;
	ValaList* result;
	g_return_val_if_fail (self != NULL, NULL);
	if (vala_parser_current (self) != VALA_TOKEN_TYPE_OPEN_BRACKET) {
		result = NULL;
		return result;
	}
	_tmp0_ = g_direct_equal;
	_tmp1_ = vala_array_list_new (VALA_TYPE_ATTRIBUTE, (GBoxedCopyFunc) vala_code_node_ref, (GDestroyNotify) vala_code_node_unref, _tmp0_);
	attrs = _tmp1_;
	while (TRUE) {
		if (!vala_parser_accept (self, VALA_TOKEN_TYPE_OPEN_BRACKET)) {
			break;
		}
		{
			gboolean _tmp2_ = FALSE;
			_tmp2_ = TRUE;
			while (TRUE) {
				ValaSourceLocation begin = {0};
				ValaSourceLocation _tmp3_ = {0};
				gchar* id = NULL;
				gchar* _tmp4_;
				ValaAttribute* attr = NULL;
				const gchar* _tmp5_;
				ValaSourceLocation _tmp6_;
				ValaSourceReference* _tmp7_;
				ValaSourceReference* _tmp8_;
				ValaAttribute* _tmp9_;
				ValaAttribute* _tmp10_;
				ValaArrayList* _tmp19_;
				ValaAttribute* _tmp20_;
				if (!_tmp2_) {
					if (!vala_parser_accept (self, VALA_TOKEN_TYPE_COMMA)) {
						break;
					}
				}
				_tmp2_ = FALSE;
				vala_parser_get_location (self, &_tmp3_);
				begin = _tmp3_;
				_tmp4_ = vala_parser_parse_identifier (self, FALSE, &_inner_error0_);
				id = _tmp4_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_iterable_unref0 (attrs);
						return NULL;
					} else {
						_vala_iterable_unref0 (attrs);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return NULL;
					}
				}
				_tmp5_ = id;
				_tmp6_ = begin;
				_tmp7_ = vala_parser_get_src (self, &_tmp6_);
				_tmp8_ = _tmp7_;
				_tmp9_ = vala_attribute_new (_tmp5_, _tmp8_);
				_tmp10_ = _tmp9_;
				_vala_source_reference_unref0 (_tmp8_);
				attr = _tmp10_;
				if (vala_parser_accept (self, VALA_TOKEN_TYPE_OPEN_PARENS)) {
					if (vala_parser_current (self) != VALA_TOKEN_TYPE_CLOSE_PARENS) {
						{
							gboolean _tmp11_ = FALSE;
							_tmp11_ = TRUE;
							while (TRUE) {
								gchar* _tmp12_ = NULL;
								gchar* _tmp13_;
								gchar* _tmp14_;
								gchar* _tmp15_ = NULL;
								gchar* _tmp16_;
								ValaAttribute* _tmp17_;
								const gchar* _tmp18_;
								if (!_tmp11_) {
									if (!vala_parser_accept (self, VALA_TOKEN_TYPE_COMMA)) {
										break;
									}
								}
								_tmp11_ = FALSE;
								_tmp13_ = vala_parser_parse_identifier (self, FALSE, &_inner_error0_);
								_tmp12_ = _tmp13_;
								if (G_UNLIKELY (_inner_error0_ != NULL)) {
									if (_inner_error0_->domain == VALA_PARSE_ERROR) {
										g_propagate_error (error, _inner_error0_);
										_vala_code_node_unref0 (attr);
										_g_free0 (id);
										_vala_iterable_unref0 (attrs);
										return NULL;
									} else {
										_vala_code_node_unref0 (attr);
										_g_free0 (id);
										_vala_iterable_unref0 (attrs);
										g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
										g_clear_error (&_inner_error0_);
										return NULL;
									}
								}
								_tmp14_ = _tmp12_;
								_tmp12_ = NULL;
								_g_free0 (id);
								id = _tmp14_;
								vala_parser_expect (self, VALA_TOKEN_TYPE_ASSIGN, &_inner_error0_);
								if (G_UNLIKELY (_inner_error0_ != NULL)) {
									if (_inner_error0_->domain == VALA_PARSE_ERROR) {
										g_propagate_error (error, _inner_error0_);
										_g_free0 (_tmp12_);
										_vala_code_node_unref0 (attr);
										_g_free0 (id);
										_vala_iterable_unref0 (attrs);
										return NULL;
									} else {
										_g_free0 (_tmp12_);
										_vala_code_node_unref0 (attr);
										_g_free0 (id);
										_vala_iterable_unref0 (attrs);
										g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
										g_clear_error (&_inner_error0_);
										return NULL;
									}
								}
								_tmp16_ = vala_parser_parse_attribute_value (self, &_inner_error0_);
								_tmp15_ = _tmp16_;
								if (G_UNLIKELY (_inner_error0_ != NULL)) {
									if (_inner_error0_->domain == VALA_PARSE_ERROR) {
										g_propagate_error (error, _inner_error0_);
										_g_free0 (_tmp12_);
										_vala_code_node_unref0 (attr);
										_g_free0 (id);
										_vala_iterable_unref0 (attrs);
										return NULL;
									} else {
										_g_free0 (_tmp12_);
										_vala_code_node_unref0 (attr);
										_g_free0 (id);
										_vala_iterable_unref0 (attrs);
										g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
										g_clear_error (&_inner_error0_);
										return NULL;
									}
								}
								_tmp17_ = attr;
								_tmp18_ = id;
								vala_attribute_add_argument (_tmp17_, _tmp18_, _tmp15_);
								_g_free0 (_tmp15_);
								_g_free0 (_tmp12_);
							}
						}
					}
					vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_PARENS, &_inner_error0_);
					if (G_UNLIKELY (_inner_error0_ != NULL)) {
						if (_inner_error0_->domain == VALA_PARSE_ERROR) {
							g_propagate_error (error, _inner_error0_);
							_vala_code_node_unref0 (attr);
							_g_free0 (id);
							_vala_iterable_unref0 (attrs);
							return NULL;
						} else {
							_vala_code_node_unref0 (attr);
							_g_free0 (id);
							_vala_iterable_unref0 (attrs);
							g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
							g_clear_error (&_inner_error0_);
							return NULL;
						}
					}
				}
				_tmp19_ = attrs;
				_tmp20_ = attr;
				vala_collection_add ((ValaCollection*) _tmp19_, _tmp20_);
				_vala_code_node_unref0 (attr);
				_g_free0 (id);
			}
		}
		vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_BRACKET, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_iterable_unref0 (attrs);
				return NULL;
			} else {
				_vala_iterable_unref0 (attrs);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
	}
	result = (ValaList*) attrs;
	return result;
}

static void
vala_parser_set_attributes (ValaParser* self,
                            ValaCodeNode* node,
                            ValaList* attributes)
{
	g_return_if_fail (self != NULL);
	g_return_if_fail (node != NULL);
	if (attributes != NULL) {
		{
			ValaList* _attr_list = NULL;
			gint _attr_size = 0;
			ValaList* _tmp0_;
			gint _tmp1_;
			gint _tmp2_;
			gint _attr_index = 0;
			_attr_list = G_TYPE_CHECK_INSTANCE_CAST (attributes, VALA_TYPE_LIST, ValaList);
			_tmp0_ = _attr_list;
			_tmp1_ = vala_collection_get_size ((ValaCollection*) _tmp0_);
			_tmp2_ = _tmp1_;
			_attr_size = _tmp2_;
			_attr_index = -1;
			while (TRUE) {
				gint _tmp3_;
				gint _tmp4_;
				ValaAttribute* attr = NULL;
				ValaList* _tmp5_;
				gpointer _tmp6_;
				ValaAttribute* _tmp7_;
				const gchar* _tmp8_;
				const gchar* _tmp9_;
				ValaAttribute* _tmp16_;
				ValaAttribute* _tmp17_;
				_attr_index = _attr_index + 1;
				_tmp3_ = _attr_index;
				_tmp4_ = _attr_size;
				if (!(_tmp3_ < _tmp4_)) {
					break;
				}
				_tmp5_ = _attr_list;
				_tmp6_ = vala_list_get (_tmp5_, _attr_index);
				attr = (ValaAttribute*) _tmp6_;
				_tmp7_ = attr;
				_tmp8_ = vala_attribute_get_name (_tmp7_);
				_tmp9_ = _tmp8_;
				if (vala_code_node_has_attribute (node, _tmp9_)) {
					ValaAttribute* _tmp10_;
					ValaSourceReference* _tmp11_;
					ValaSourceReference* _tmp12_;
					ValaAttribute* _tmp13_;
					const gchar* _tmp14_;
					const gchar* _tmp15_;
					_tmp10_ = attr;
					_tmp11_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp10_);
					_tmp12_ = _tmp11_;
					_tmp13_ = attr;
					_tmp14_ = vala_attribute_get_name (_tmp13_);
					_tmp15_ = _tmp14_;
					vala_report_error (_tmp12_, "duplicate attribute `%s'", _tmp15_);
				}
				_tmp16_ = attr;
				_tmp17_ = _vala_code_node_ref0 (_tmp16_);
				node->attributes = g_list_append (node->attributes, _tmp17_);
				_vala_code_node_unref0 (attr);
			}
		}
	}
}

static void
vala_parser_parse_main_block (ValaParser* self,
                              ValaSymbol* parent,
                              GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaMethod* method = NULL;
	ValaScanner* _tmp1_;
	ValaSourceFile* _tmp2_;
	ValaSourceFile* _tmp3_;
	ValaSourceLocation _tmp4_;
	ValaSourceLocation _tmp5_;
	ValaSourceReference* _tmp6_;
	ValaSourceReference* _tmp7_;
	ValaMethod* _tmp8_;
	ValaMethod* _tmp9_;
	ValaSourceLocation _tmp10_;
	ValaSourceReference* _tmp11_;
	ValaSourceReference* _tmp12_;
	ValaBlock* _tmp13_;
	ValaBlock* _tmp14_;
	ValaBlock* _tmp15_;
	ValaBlock* _tmp16_;
	ValaBlock* _tmp19_;
	ValaBlock* _tmp20_;
	ValaSourceReference* _tmp21_;
	ValaSourceReference* _tmp22_;
	ValaSourceReference* _tmp23_;
	ValaSourceReference* _tmp24_;
	ValaSourceLocation _tmp25_ = {0};
	ValaSourceLocation _tmp26_;
	ValaCodeContext* _tmp27_;
	gboolean _tmp28_;
	gboolean _tmp29_;
	GError* _inner_error0_ = NULL;
	g_return_if_fail (self != NULL);
	g_return_if_fail (parent != NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	_tmp1_ = self->priv->scanner;
	_tmp2_ = vala_scanner_get_source_file (_tmp1_);
	_tmp3_ = _tmp2_;
	_tmp4_ = begin;
	_tmp5_ = begin;
	_tmp6_ = vala_source_reference_new (_tmp3_, &_tmp4_, &_tmp5_);
	_tmp7_ = _tmp6_;
	_tmp8_ = vala_method_new_main_block (_tmp7_);
	_tmp9_ = _tmp8_;
	_vala_source_reference_unref0 (_tmp7_);
	method = _tmp9_;
	vala_symbol_set_access ((ValaSymbol*) method, VALA_SYMBOL_ACCESSIBILITY_PUBLIC);
	vala_method_set_binding (method, VALA_MEMBER_BINDING_STATIC);
	_tmp10_ = begin;
	_tmp11_ = vala_parser_get_src (self, &_tmp10_);
	_tmp12_ = _tmp11_;
	_tmp13_ = vala_block_new (_tmp12_);
	_tmp14_ = _tmp13_;
	vala_subroutine_set_body ((ValaSubroutine*) method, _tmp14_);
	_vala_code_node_unref0 (_tmp14_);
	_vala_source_reference_unref0 (_tmp12_);
	_tmp15_ = vala_subroutine_get_body ((ValaSubroutine*) method);
	_tmp16_ = _tmp15_;
	vala_parser_parse_statements (self, _tmp16_, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (method);
			return;
		} else {
			_vala_code_node_unref0 (method);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	if (vala_parser_current (self) != VALA_TOKEN_TYPE_EOF) {
		ValaSourceReference* _tmp17_;
		ValaSourceReference* _tmp18_;
		_tmp17_ = vala_parser_get_current_src (self);
		_tmp18_ = _tmp17_;
		vala_report_error (_tmp18_, "expected end of file");
		_vala_source_reference_unref0 (_tmp18_);
	}
	_tmp19_ = vala_subroutine_get_body ((ValaSubroutine*) method);
	_tmp20_ = _tmp19_;
	_tmp21_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp20_);
	_tmp22_ = _tmp21_;
	_tmp23_ = vala_parser_get_current_src (self);
	_tmp24_ = _tmp23_;
	vala_source_reference_get_end (_tmp24_, &_tmp25_);
	_tmp26_ = _tmp25_;
	vala_source_reference_set_end (_tmp22_, &_tmp26_);
	_vala_source_reference_unref0 (_tmp24_);
	_tmp27_ = self->priv->context;
	_tmp28_ = vala_code_context_get_experimental (_tmp27_);
	_tmp29_ = _tmp28_;
	if (!_tmp29_) {
		ValaSourceReference* _tmp30_;
		ValaSourceReference* _tmp31_;
		_tmp30_ = vala_code_node_get_source_reference ((ValaCodeNode*) method);
		_tmp31_ = _tmp30_;
		vala_report_warning (_tmp31_, "main blocks are experimental");
	}
	vala_symbol_add_method (parent, method);
	_vala_code_node_unref0 (method);
}

static void
vala_parser_parse_declaration (ValaParser* self,
                               ValaSymbol* parent,
                               gboolean root,
                               GError** error)
{
	ValaScanner* _tmp0_;
	ValaComment* _tmp1_;
	ValaList* attrs = NULL;
	ValaList* _tmp2_;
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp3_ = {0};
	ValaTokenType last_keyword = 0;
	ValaSourceLocation _tmp39_;
	GError* _tmp40_;
	GError* _inner_error0_ = NULL;
	g_return_if_fail (self != NULL);
	g_return_if_fail (parent != NULL);
	_tmp0_ = self->priv->scanner;
	_tmp1_ = vala_scanner_pop_comment (_tmp0_);
	_vala_comment_unref0 (self->priv->comment);
	self->priv->comment = _tmp1_;
	_tmp2_ = vala_parser_parse_attributes (self, &_inner_error0_);
	attrs = _tmp2_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	vala_parser_get_location (self, &_tmp3_);
	begin = _tmp3_;
	last_keyword = vala_parser_current (self);
	while (TRUE) {
		if (!vala_parser_is_declaration_keyword (self, vala_parser_current (self))) {
			break;
		}
		last_keyword = vala_parser_current (self);
		vala_parser_next (self);
	}
	switch (vala_parser_current (self)) {
		case VALA_TOKEN_TYPE_CONSTRUCT:
		{
			ValaCodeContext* _tmp4_;
			ValaProfile _tmp5_;
			ValaProfile _tmp6_;
			_tmp4_ = self->priv->context;
			_tmp5_ = vala_code_context_get_profile (_tmp4_);
			_tmp6_ = _tmp5_;
			if (_tmp6_ == VALA_PROFILE_GOBJECT) {
				ValaSourceLocation _tmp7_;
				ValaList* _tmp9_;
				_tmp7_ = begin;
				vala_parser_rollback (self, &_tmp7_);
				if (!VALA_IS_TYPESYMBOL (parent)) {
					GError* _tmp8_;
					_tmp8_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "unexpected `construct' declaration");
					_inner_error0_ = _tmp8_;
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_iterable_unref0 (attrs);
						return;
					} else {
						_vala_iterable_unref0 (attrs);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return;
					}
				}
				_tmp9_ = attrs;
				vala_parser_parse_constructor_declaration (self, parent, _tmp9_, &_inner_error0_);
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_iterable_unref0 (attrs);
						return;
					} else {
						_vala_iterable_unref0 (attrs);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return;
					}
				}
				_vala_iterable_unref0 (attrs);
				return;
			}
			break;
		}
		case VALA_TOKEN_TYPE_TILDE:
		{
			ValaSourceLocation _tmp10_;
			ValaList* _tmp12_;
			_tmp10_ = begin;
			vala_parser_rollback (self, &_tmp10_);
			if (!VALA_IS_TYPESYMBOL (parent)) {
				GError* _tmp11_;
				_tmp11_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "unexpected `destructor' declaration");
				_inner_error0_ = _tmp11_;
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_iterable_unref0 (attrs);
					return;
				} else {
					_vala_iterable_unref0 (attrs);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return;
				}
			}
			_tmp12_ = attrs;
			vala_parser_parse_destructor_declaration (self, parent, _tmp12_, &_inner_error0_);
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_iterable_unref0 (attrs);
					return;
				} else {
					_vala_iterable_unref0 (attrs);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return;
				}
			}
			_vala_iterable_unref0 (attrs);
			return;
		}
		case VALA_TOKEN_TYPE_OPEN_BRACE:
		case VALA_TOKEN_TYPE_SEMICOLON:
		case VALA_TOKEN_TYPE_IF:
		case VALA_TOKEN_TYPE_SWITCH:
		case VALA_TOKEN_TYPE_WHILE:
		case VALA_TOKEN_TYPE_DO:
		case VALA_TOKEN_TYPE_FOR:
		case VALA_TOKEN_TYPE_FOREACH:
		case VALA_TOKEN_TYPE_BREAK:
		case VALA_TOKEN_TYPE_CONTINUE:
		case VALA_TOKEN_TYPE_RETURN:
		case VALA_TOKEN_TYPE_YIELD:
		case VALA_TOKEN_TYPE_THROW:
		case VALA_TOKEN_TYPE_TRY:
		case VALA_TOKEN_TYPE_LOCK:
		case VALA_TOKEN_TYPE_UNLOCK:
		case VALA_TOKEN_TYPE_DELETE:
		case VALA_TOKEN_TYPE_VAR:
		case VALA_TOKEN_TYPE_OP_INC:
		case VALA_TOKEN_TYPE_OP_DEC:
		case VALA_TOKEN_TYPE_BASE:
		case VALA_TOKEN_TYPE_THIS:
		case VALA_TOKEN_TYPE_STAR:
		case VALA_TOKEN_TYPE_NEW:
		{
			ValaList* _tmp13_;
			ValaSourceLocation _tmp16_;
			_tmp13_ = attrs;
			if (_tmp13_ != NULL) {
				GError* _tmp14_;
				_tmp14_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "expected statement");
				_inner_error0_ = _tmp14_;
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_iterable_unref0 (attrs);
					return;
				} else {
					_vala_iterable_unref0 (attrs);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return;
				}
			}
			if (!root) {
				GError* _tmp15_;
				_tmp15_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "statements outside blocks allowed only in root namespace");
				_inner_error0_ = _tmp15_;
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_iterable_unref0 (attrs);
					return;
				} else {
					_vala_iterable_unref0 (attrs);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return;
				}
			}
			_tmp16_ = begin;
			vala_parser_rollback (self, &_tmp16_);
			vala_parser_parse_main_block (self, parent, &_inner_error0_);
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_iterable_unref0 (attrs);
					return;
				} else {
					_vala_iterable_unref0 (attrs);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return;
				}
			}
			_vala_iterable_unref0 (attrs);
			return;
		}
		default:
		{
			if (root) {
				gboolean is_expr = FALSE;
				is_expr = vala_parser_is_expression (self, &_inner_error0_);
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_iterable_unref0 (attrs);
						return;
					} else {
						_vala_iterable_unref0 (attrs);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return;
					}
				}
				if (is_expr) {
					ValaSourceLocation _tmp17_;
					_tmp17_ = begin;
					vala_parser_rollback (self, &_tmp17_);
					vala_parser_parse_main_block (self, parent, &_inner_error0_);
					if (G_UNLIKELY (_inner_error0_ != NULL)) {
						if (_inner_error0_->domain == VALA_PARSE_ERROR) {
							g_propagate_error (error, _inner_error0_);
							_vala_iterable_unref0 (attrs);
							return;
						} else {
							_vala_iterable_unref0 (attrs);
							g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
							g_clear_error (&_inner_error0_);
							return;
						}
					}
					_vala_iterable_unref0 (attrs);
					return;
				}
			}
			vala_parser_skip_type (self, &_inner_error0_);
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_iterable_unref0 (attrs);
					return;
				} else {
					_vala_iterable_unref0 (attrs);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return;
				}
			}
			switch (vala_parser_current (self)) {
				case VALA_TOKEN_TYPE_OPEN_BRACE:
				case VALA_TOKEN_TYPE_SEMICOLON:
				case VALA_TOKEN_TYPE_COLON:
				{
					ValaSourceLocation _tmp18_;
					_tmp18_ = begin;
					vala_parser_rollback (self, &_tmp18_);
					switch (last_keyword) {
						case VALA_TOKEN_TYPE_CLASS:
						{
							ValaList* _tmp19_;
							_tmp19_ = attrs;
							vala_parser_parse_class_declaration (self, parent, _tmp19_, FALSE, &_inner_error0_);
							if (G_UNLIKELY (_inner_error0_ != NULL)) {
								if (_inner_error0_->domain == VALA_PARSE_ERROR) {
									g_propagate_error (error, _inner_error0_);
									_vala_iterable_unref0 (attrs);
									return;
								} else {
									_vala_iterable_unref0 (attrs);
									g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
									g_clear_error (&_inner_error0_);
									return;
								}
							}
							_vala_iterable_unref0 (attrs);
							return;
						}
						case VALA_TOKEN_TYPE_ENUM:
						{
							ValaList* _tmp20_;
							_tmp20_ = attrs;
							vala_parser_parse_enum_declaration (self, parent, _tmp20_, &_inner_error0_);
							if (G_UNLIKELY (_inner_error0_ != NULL)) {
								if (_inner_error0_->domain == VALA_PARSE_ERROR) {
									g_propagate_error (error, _inner_error0_);
									_vala_iterable_unref0 (attrs);
									return;
								} else {
									_vala_iterable_unref0 (attrs);
									g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
									g_clear_error (&_inner_error0_);
									return;
								}
							}
							_vala_iterable_unref0 (attrs);
							return;
						}
						case VALA_TOKEN_TYPE_ERRORDOMAIN:
						{
							ValaList* _tmp21_;
							_tmp21_ = attrs;
							vala_parser_parse_errordomain_declaration (self, parent, _tmp21_, &_inner_error0_);
							if (G_UNLIKELY (_inner_error0_ != NULL)) {
								if (_inner_error0_->domain == VALA_PARSE_ERROR) {
									g_propagate_error (error, _inner_error0_);
									_vala_iterable_unref0 (attrs);
									return;
								} else {
									_vala_iterable_unref0 (attrs);
									g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
									g_clear_error (&_inner_error0_);
									return;
								}
							}
							_vala_iterable_unref0 (attrs);
							return;
						}
						case VALA_TOKEN_TYPE_INTERFACE:
						{
							ValaList* _tmp22_;
							_tmp22_ = attrs;
							vala_parser_parse_interface_declaration (self, parent, _tmp22_, &_inner_error0_);
							if (G_UNLIKELY (_inner_error0_ != NULL)) {
								if (_inner_error0_->domain == VALA_PARSE_ERROR) {
									g_propagate_error (error, _inner_error0_);
									_vala_iterable_unref0 (attrs);
									return;
								} else {
									_vala_iterable_unref0 (attrs);
									g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
									g_clear_error (&_inner_error0_);
									return;
								}
							}
							_vala_iterable_unref0 (attrs);
							return;
						}
						case VALA_TOKEN_TYPE_NAMESPACE:
						{
							ValaList* _tmp23_;
							_tmp23_ = attrs;
							vala_parser_parse_namespace_declaration (self, parent, _tmp23_, &_inner_error0_);
							if (G_UNLIKELY (_inner_error0_ != NULL)) {
								if (_inner_error0_->domain == VALA_PARSE_ERROR) {
									g_propagate_error (error, _inner_error0_);
									_vala_iterable_unref0 (attrs);
									return;
								} else {
									_vala_iterable_unref0 (attrs);
									g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
									g_clear_error (&_inner_error0_);
									return;
								}
							}
							_vala_iterable_unref0 (attrs);
							return;
						}
						case VALA_TOKEN_TYPE_STRUCT:
						{
							ValaList* _tmp24_;
							_tmp24_ = attrs;
							vala_parser_parse_struct_declaration (self, parent, _tmp24_, &_inner_error0_);
							if (G_UNLIKELY (_inner_error0_ != NULL)) {
								if (_inner_error0_->domain == VALA_PARSE_ERROR) {
									g_propagate_error (error, _inner_error0_);
									_vala_iterable_unref0 (attrs);
									return;
								} else {
									_vala_iterable_unref0 (attrs);
									g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
									g_clear_error (&_inner_error0_);
									return;
								}
							}
							_vala_iterable_unref0 (attrs);
							return;
						}
						default:
						{
							break;
						}
					}
					break;
				}
				case VALA_TOKEN_TYPE_OPEN_PARENS:
				{
					ValaSourceLocation _tmp25_;
					ValaList* _tmp27_;
					_tmp25_ = begin;
					vala_parser_rollback (self, &_tmp25_);
					if (!VALA_IS_TYPESYMBOL (parent)) {
						GError* _tmp26_;
						_tmp26_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "unexpected `constructor' declaration");
						_inner_error0_ = _tmp26_;
						if (_inner_error0_->domain == VALA_PARSE_ERROR) {
							g_propagate_error (error, _inner_error0_);
							_vala_iterable_unref0 (attrs);
							return;
						} else {
							_vala_iterable_unref0 (attrs);
							g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
							g_clear_error (&_inner_error0_);
							return;
						}
					}
					_tmp27_ = attrs;
					vala_parser_parse_creation_method_declaration (self, parent, _tmp27_, &_inner_error0_);
					if (G_UNLIKELY (_inner_error0_ != NULL)) {
						if (_inner_error0_->domain == VALA_PARSE_ERROR) {
							g_propagate_error (error, _inner_error0_);
							_vala_iterable_unref0 (attrs);
							return;
						} else {
							_vala_iterable_unref0 (attrs);
							g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
							g_clear_error (&_inner_error0_);
							return;
						}
					}
					_vala_iterable_unref0 (attrs);
					return;
				}
				default:
				{
					vala_parser_skip_type (self, &_inner_error0_);
					if (G_UNLIKELY (_inner_error0_ != NULL)) {
						if (_inner_error0_->domain == VALA_PARSE_ERROR) {
							g_propagate_error (error, _inner_error0_);
							_vala_iterable_unref0 (attrs);
							return;
						} else {
							_vala_iterable_unref0 (attrs);
							g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
							g_clear_error (&_inner_error0_);
							return;
						}
					}
					switch (vala_parser_current (self)) {
						case VALA_TOKEN_TYPE_OPEN_PARENS:
						{
							ValaSourceLocation _tmp28_;
							_tmp28_ = begin;
							vala_parser_rollback (self, &_tmp28_);
							switch (last_keyword) {
								case VALA_TOKEN_TYPE_DELEGATE:
								{
									ValaList* _tmp29_;
									_tmp29_ = attrs;
									vala_parser_parse_delegate_declaration (self, parent, _tmp29_, &_inner_error0_);
									if (G_UNLIKELY (_inner_error0_ != NULL)) {
										if (_inner_error0_->domain == VALA_PARSE_ERROR) {
											g_propagate_error (error, _inner_error0_);
											_vala_iterable_unref0 (attrs);
											return;
										} else {
											_vala_iterable_unref0 (attrs);
											g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
											g_clear_error (&_inner_error0_);
											return;
										}
									}
									_vala_iterable_unref0 (attrs);
									return;
								}
								case VALA_TOKEN_TYPE_SIGNAL:
								{
									ValaList* _tmp31_;
									if (!VALA_IS_OBJECT_TYPE_SYMBOL (parent)) {
										GError* _tmp30_;
										_tmp30_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "unexpected `signal' declaration");
										_inner_error0_ = _tmp30_;
										if (_inner_error0_->domain == VALA_PARSE_ERROR) {
											g_propagate_error (error, _inner_error0_);
											_vala_iterable_unref0 (attrs);
											return;
										} else {
											_vala_iterable_unref0 (attrs);
											g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
											g_clear_error (&_inner_error0_);
											return;
										}
									}
									_tmp31_ = attrs;
									vala_parser_parse_signal_declaration (self, parent, _tmp31_, &_inner_error0_);
									if (G_UNLIKELY (_inner_error0_ != NULL)) {
										if (_inner_error0_->domain == VALA_PARSE_ERROR) {
											g_propagate_error (error, _inner_error0_);
											_vala_iterable_unref0 (attrs);
											return;
										} else {
											_vala_iterable_unref0 (attrs);
											g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
											g_clear_error (&_inner_error0_);
											return;
										}
									}
									_vala_iterable_unref0 (attrs);
									return;
								}
								default:
								{
									ValaList* _tmp32_;
									_tmp32_ = attrs;
									vala_parser_parse_method_declaration (self, parent, _tmp32_, &_inner_error0_);
									if (G_UNLIKELY (_inner_error0_ != NULL)) {
										if (_inner_error0_->domain == VALA_PARSE_ERROR) {
											g_propagate_error (error, _inner_error0_);
											_vala_iterable_unref0 (attrs);
											return;
										} else {
											_vala_iterable_unref0 (attrs);
											g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
											g_clear_error (&_inner_error0_);
											return;
										}
									}
									_vala_iterable_unref0 (attrs);
									return;
								}
							}
						}
						case VALA_TOKEN_TYPE_ASSIGN:
						case VALA_TOKEN_TYPE_SEMICOLON:
						{
							ValaSourceLocation _tmp33_;
							_tmp33_ = begin;
							vala_parser_rollback (self, &_tmp33_);
							switch (last_keyword) {
								case VALA_TOKEN_TYPE_CONST:
								{
									ValaList* _tmp34_;
									_tmp34_ = attrs;
									vala_parser_parse_constant_declaration (self, parent, _tmp34_, &_inner_error0_);
									if (G_UNLIKELY (_inner_error0_ != NULL)) {
										if (_inner_error0_->domain == VALA_PARSE_ERROR) {
											g_propagate_error (error, _inner_error0_);
											_vala_iterable_unref0 (attrs);
											return;
										} else {
											_vala_iterable_unref0 (attrs);
											g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
											g_clear_error (&_inner_error0_);
											return;
										}
									}
									_vala_iterable_unref0 (attrs);
									return;
								}
								default:
								{
									ValaList* _tmp35_;
									_tmp35_ = attrs;
									vala_parser_parse_field_declaration (self, parent, _tmp35_, &_inner_error0_);
									if (G_UNLIKELY (_inner_error0_ != NULL)) {
										if (_inner_error0_->domain == VALA_PARSE_ERROR) {
											g_propagate_error (error, _inner_error0_);
											_vala_iterable_unref0 (attrs);
											return;
										} else {
											_vala_iterable_unref0 (attrs);
											g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
											g_clear_error (&_inner_error0_);
											return;
										}
									}
									_vala_iterable_unref0 (attrs);
									return;
								}
							}
						}
						case VALA_TOKEN_TYPE_OPEN_BRACE:
						case VALA_TOKEN_TYPE_THROWS:
						{
							ValaSourceLocation _tmp36_;
							ValaList* _tmp38_;
							_tmp36_ = begin;
							vala_parser_rollback (self, &_tmp36_);
							if (!VALA_IS_TYPESYMBOL (parent)) {
								GError* _tmp37_;
								_tmp37_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "unexpected `property' declaration");
								_inner_error0_ = _tmp37_;
								if (_inner_error0_->domain == VALA_PARSE_ERROR) {
									g_propagate_error (error, _inner_error0_);
									_vala_iterable_unref0 (attrs);
									return;
								} else {
									_vala_iterable_unref0 (attrs);
									g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
									g_clear_error (&_inner_error0_);
									return;
								}
							}
							_tmp38_ = attrs;
							vala_parser_parse_property_declaration (self, parent, _tmp38_, &_inner_error0_);
							if (G_UNLIKELY (_inner_error0_ != NULL)) {
								if (_inner_error0_->domain == VALA_PARSE_ERROR) {
									g_propagate_error (error, _inner_error0_);
									_vala_iterable_unref0 (attrs);
									return;
								} else {
									_vala_iterable_unref0 (attrs);
									g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
									g_clear_error (&_inner_error0_);
									return;
								}
							}
							_vala_iterable_unref0 (attrs);
							return;
						}
						default:
						{
							break;
						}
					}
					break;
				}
			}
			break;
		}
	}
	_tmp39_ = begin;
	vala_parser_rollback (self, &_tmp39_);
	_tmp40_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "expected declaration");
	_inner_error0_ = _tmp40_;
	if (_inner_error0_->domain == VALA_PARSE_ERROR) {
		g_propagate_error (error, _inner_error0_);
		_vala_iterable_unref0 (attrs);
		return;
	} else {
		_vala_iterable_unref0 (attrs);
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
		g_clear_error (&_inner_error0_);
		return;
	}
}

static void
vala_parser_parse_declarations (ValaParser* self,
                                ValaSymbol* parent,
                                gboolean root,
                                GError** error)
{
	GError* _inner_error0_ = NULL;
	g_return_if_fail (self != NULL);
	g_return_if_fail (parent != NULL);
	if (!root) {
		vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_BRACE, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				return;
			} else {
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
	}
	while (TRUE) {
		gboolean _tmp0_ = FALSE;
		if (vala_parser_current (self) != VALA_TOKEN_TYPE_CLOSE_BRACE) {
			_tmp0_ = vala_parser_current (self) != VALA_TOKEN_TYPE_EOF;
		} else {
			_tmp0_ = FALSE;
		}
		if (!_tmp0_) {
			break;
		}
		{
			ValaCodeContext* _tmp1_;
			ValaNamespace* _tmp2_;
			ValaNamespace* _tmp3_;
			_tmp1_ = self->priv->context;
			_tmp2_ = vala_code_context_get_root (_tmp1_);
			_tmp3_ = _tmp2_;
			vala_parser_parse_declaration (self, parent, parent == G_TYPE_CHECK_INSTANCE_CAST (_tmp3_, VALA_TYPE_SYMBOL, ValaSymbol), &_inner_error0_);
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					goto __catch0_vala_parse_error;
				}
				g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
		goto __finally0;
		__catch0_vala_parse_error:
		{
			GError* e = NULL;
			GError* _tmp4_;
			gint r = 0;
			e = _inner_error0_;
			_inner_error0_ = NULL;
			_tmp4_ = e;
			vala_parser_report_parse_error (self, _tmp4_);
			while (TRUE) {
				r = (gint) vala_parser_recover (self);
				if (r == ((gint) VALA_PARSER_RECOVERY_STATE_STATEMENT_BEGIN)) {
					vala_parser_next (self);
				} else {
					break;
				}
			}
			if (r == ((gint) VALA_PARSER_RECOVERY_STATE_EOF)) {
				_g_error_free0 (e);
				return;
			}
			_g_error_free0 (e);
		}
		__finally0:
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				return;
			} else {
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
	}
	if (!root) {
		if (!vala_parser_accept (self, VALA_TOKEN_TYPE_CLOSE_BRACE)) {
			ValaCodeContext* _tmp5_;
			ValaReport* _tmp6_;
			ValaReport* _tmp7_;
			_tmp5_ = self->priv->context;
			_tmp6_ = vala_code_context_get_report (_tmp5_);
			_tmp7_ = _tmp6_;
			if (vala_report_get_errors (_tmp7_) == 0) {
				ValaSourceReference* _tmp8_;
				ValaSourceReference* _tmp9_;
				_tmp8_ = vala_parser_get_current_src (self);
				_tmp9_ = _tmp8_;
				vala_report_error (_tmp9_, "expected `}'");
				_vala_source_reference_unref0 (_tmp9_);
			}
		}
	}
}

static ValaParserRecoveryState
vala_parser_recover (ValaParser* self)
{
	ValaParserRecoveryState result;
	g_return_val_if_fail (self != NULL, 0);
	while (TRUE) {
		if (!(vala_parser_current (self) != VALA_TOKEN_TYPE_EOF)) {
			break;
		}
		switch (vala_parser_current (self)) {
			case VALA_TOKEN_TYPE_ABSTRACT:
			case VALA_TOKEN_TYPE_CLASS:
			case VALA_TOKEN_TYPE_CONST:
			case VALA_TOKEN_TYPE_CONSTRUCT:
			case VALA_TOKEN_TYPE_DELEGATE:
			case VALA_TOKEN_TYPE_ENUM:
			case VALA_TOKEN_TYPE_ERRORDOMAIN:
			case VALA_TOKEN_TYPE_EXTERN:
			case VALA_TOKEN_TYPE_INLINE:
			case VALA_TOKEN_TYPE_INTERFACE:
			case VALA_TOKEN_TYPE_INTERNAL:
			case VALA_TOKEN_TYPE_NAMESPACE:
			case VALA_TOKEN_TYPE_NEW:
			case VALA_TOKEN_TYPE_OVERRIDE:
			case VALA_TOKEN_TYPE_PARTIAL:
			case VALA_TOKEN_TYPE_PRIVATE:
			case VALA_TOKEN_TYPE_PROTECTED:
			case VALA_TOKEN_TYPE_PUBLIC:
			case VALA_TOKEN_TYPE_SEALED:
			case VALA_TOKEN_TYPE_SIGNAL:
			case VALA_TOKEN_TYPE_STATIC:
			case VALA_TOKEN_TYPE_STRUCT:
			case VALA_TOKEN_TYPE_VIRTUAL:
			case VALA_TOKEN_TYPE_VOLATILE:
			{
				result = VALA_PARSER_RECOVERY_STATE_DECLARATION_BEGIN;
				return result;
			}
			case VALA_TOKEN_TYPE_BREAK:
			case VALA_TOKEN_TYPE_CONTINUE:
			case VALA_TOKEN_TYPE_DELETE:
			case VALA_TOKEN_TYPE_DO:
			case VALA_TOKEN_TYPE_FOR:
			case VALA_TOKEN_TYPE_FOREACH:
			case VALA_TOKEN_TYPE_IF:
			case VALA_TOKEN_TYPE_LOCK:
			case VALA_TOKEN_TYPE_RETURN:
			case VALA_TOKEN_TYPE_SWITCH:
			case VALA_TOKEN_TYPE_THROW:
			case VALA_TOKEN_TYPE_TRY:
			case VALA_TOKEN_TYPE_UNLOCK:
			case VALA_TOKEN_TYPE_VAR:
			case VALA_TOKEN_TYPE_WHILE:
			case VALA_TOKEN_TYPE_WITH:
			case VALA_TOKEN_TYPE_YIELD:
			{
				result = VALA_PARSER_RECOVERY_STATE_STATEMENT_BEGIN;
				return result;
			}
			default:
			{
				vala_parser_next (self);
				break;
			}
		}
	}
	result = VALA_PARSER_RECOVERY_STATE_EOF;
	return result;
}

static gpointer
_vala_iterable_ref0 (gpointer self)
{
	return self ? vala_iterable_ref (self) : NULL;
}

static void
vala_parser_parse_namespace_declaration (ValaParser* self,
                                         ValaSymbol* parent,
                                         ValaList* attrs,
                                         GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaUnresolvedSymbol* sym = NULL;
	ValaUnresolvedSymbol* _tmp1_;
	ValaNamespace* ns = NULL;
	ValaUnresolvedSymbol* _tmp2_;
	const gchar* _tmp3_;
	const gchar* _tmp4_;
	ValaSourceLocation _tmp5_;
	ValaSourceReference* _tmp6_;
	ValaSourceReference* _tmp7_;
	ValaNamespace* _tmp8_;
	ValaNamespace* _tmp9_;
	ValaComment* _tmp10_;
	ValaNamespace* _tmp13_;
	ValaList* old_using_directives = NULL;
	ValaScanner* _tmp14_;
	ValaSourceFile* _tmp15_;
	ValaSourceFile* _tmp16_;
	ValaList* _tmp17_;
	ValaList* _tmp18_;
	ValaList* _tmp19_;
	ValaNamespace* _tmp20_;
	ValaNamespace* _tmp21_;
	ValaScanner* _tmp22_;
	ValaSourceFile* _tmp23_;
	ValaSourceFile* _tmp24_;
	ValaList* _tmp25_;
	ValaSymbol* _result_ = NULL;
	ValaNamespace* _tmp31_;
	ValaSymbol* _tmp32_;
	GError* _inner_error0_ = NULL;
	g_return_if_fail (self != NULL);
	g_return_if_fail (parent != NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_NAMESPACE, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp1_ = vala_parser_parse_symbol_name (self, &_inner_error0_);
	sym = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp2_ = sym;
	_tmp3_ = vala_symbol_get_name ((ValaSymbol*) _tmp2_);
	_tmp4_ = _tmp3_;
	_tmp5_ = begin;
	_tmp6_ = vala_parser_get_src (self, &_tmp5_);
	_tmp7_ = _tmp6_;
	_tmp8_ = vala_namespace_new (_tmp4_, _tmp7_);
	_tmp9_ = _tmp8_;
	_vala_source_reference_unref0 (_tmp7_);
	ns = _tmp9_;
	_tmp10_ = self->priv->comment;
	if (_tmp10_ != NULL) {
		ValaNamespace* _tmp11_;
		ValaComment* _tmp12_;
		_tmp11_ = ns;
		_tmp12_ = self->priv->comment;
		vala_namespace_add_comment (_tmp11_, _tmp12_);
		_vala_comment_unref0 (self->priv->comment);
		self->priv->comment = NULL;
	}
	_tmp13_ = ns;
	vala_parser_set_attributes (self, (ValaCodeNode*) _tmp13_, attrs);
	vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_BRACE, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (ns);
			_vala_code_node_unref0 (sym);
			return;
		} else {
			_vala_code_node_unref0 (ns);
			_vala_code_node_unref0 (sym);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp14_ = self->priv->scanner;
	_tmp15_ = vala_scanner_get_source_file (_tmp14_);
	_tmp16_ = _tmp15_;
	_tmp17_ = vala_source_file_get_current_using_directives (_tmp16_);
	_tmp18_ = _tmp17_;
	_tmp19_ = _vala_iterable_ref0 (_tmp18_);
	old_using_directives = _tmp19_;
	_tmp20_ = ns;
	vala_parser_parse_using_directives (self, _tmp20_, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_iterable_unref0 (old_using_directives);
			_vala_code_node_unref0 (ns);
			_vala_code_node_unref0 (sym);
			return;
		} else {
			_vala_iterable_unref0 (old_using_directives);
			_vala_code_node_unref0 (ns);
			_vala_code_node_unref0 (sym);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp21_ = ns;
	vala_parser_parse_declarations (self, (ValaSymbol*) _tmp21_, TRUE, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_iterable_unref0 (old_using_directives);
			_vala_code_node_unref0 (ns);
			_vala_code_node_unref0 (sym);
			return;
		} else {
			_vala_iterable_unref0 (old_using_directives);
			_vala_code_node_unref0 (ns);
			_vala_code_node_unref0 (sym);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp22_ = self->priv->scanner;
	_tmp23_ = vala_scanner_get_source_file (_tmp22_);
	_tmp24_ = _tmp23_;
	_tmp25_ = old_using_directives;
	vala_source_file_set_current_using_directives (_tmp24_, _tmp25_);
	if (!vala_parser_accept (self, VALA_TOKEN_TYPE_CLOSE_BRACE)) {
		ValaCodeContext* _tmp26_;
		ValaReport* _tmp27_;
		ValaReport* _tmp28_;
		_tmp26_ = self->priv->context;
		_tmp27_ = vala_code_context_get_report (_tmp26_);
		_tmp28_ = _tmp27_;
		if (vala_report_get_errors (_tmp28_) == 0) {
			ValaSourceReference* _tmp29_;
			ValaSourceReference* _tmp30_;
			_tmp29_ = vala_parser_get_current_src (self);
			_tmp30_ = _tmp29_;
			vala_report_error (_tmp30_, "expected `}'");
			_vala_source_reference_unref0 (_tmp30_);
		}
	}
	_tmp31_ = ns;
	_tmp32_ = _vala_code_node_ref0 ((ValaSymbol*) _tmp31_);
	_result_ = _tmp32_;
	while (TRUE) {
		ValaUnresolvedSymbol* _tmp33_;
		ValaUnresolvedSymbol* _tmp34_;
		ValaUnresolvedSymbol* _tmp35_;
		ValaUnresolvedSymbol* _tmp36_;
		ValaUnresolvedSymbol* _tmp37_;
		ValaSymbol* _tmp38_ = NULL;
		ValaUnresolvedSymbol* _tmp39_;
		ValaSymbol* next = NULL;
		ValaSymbol* _tmp48_;
		ValaSymbol* _tmp49_;
		ValaSymbol* _tmp50_;
		ValaSymbol* _tmp51_;
		ValaSymbol* _tmp52_;
		_tmp33_ = sym;
		if (!(_tmp33_ != NULL)) {
			break;
		}
		_tmp34_ = sym;
		_tmp35_ = vala_unresolved_symbol_get_inner (_tmp34_);
		_tmp36_ = _tmp35_;
		_tmp37_ = _vala_code_node_ref0 (_tmp36_);
		_vala_code_node_unref0 (sym);
		sym = _tmp37_;
		_tmp39_ = sym;
		if (_tmp39_ != NULL) {
			ValaUnresolvedSymbol* _tmp40_;
			const gchar* _tmp41_;
			const gchar* _tmp42_;
			ValaNamespace* _tmp43_;
			ValaSourceReference* _tmp44_;
			ValaSourceReference* _tmp45_;
			ValaNamespace* _tmp46_;
			_tmp40_ = sym;
			_tmp41_ = vala_symbol_get_name ((ValaSymbol*) _tmp40_);
			_tmp42_ = _tmp41_;
			_tmp43_ = ns;
			_tmp44_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp43_);
			_tmp45_ = _tmp44_;
			_tmp46_ = vala_namespace_new (_tmp42_, _tmp45_);
			_vala_code_node_unref0 (_tmp38_);
			_tmp38_ = (ValaSymbol*) _tmp46_;
		} else {
			ValaSymbol* _tmp47_;
			_tmp47_ = _vala_code_node_ref0 (parent);
			_vala_code_node_unref0 (_tmp38_);
			_tmp38_ = _tmp47_;
		}
		_tmp48_ = _vala_code_node_ref0 (_tmp38_);
		next = _tmp48_;
		_tmp49_ = next;
		_tmp50_ = _result_;
		vala_symbol_add_namespace (_tmp49_, G_TYPE_CHECK_INSTANCE_CAST (_tmp50_, VALA_TYPE_NAMESPACE, ValaNamespace));
		_tmp51_ = next;
		_tmp52_ = _vala_code_node_ref0 (_tmp51_);
		_vala_code_node_unref0 (_result_);
		_result_ = _tmp52_;
		_vala_code_node_unref0 (next);
		_vala_code_node_unref0 (_tmp38_);
	}
	_vala_code_node_unref0 (_result_);
	_vala_iterable_unref0 (old_using_directives);
	_vala_code_node_unref0 (ns);
	_vala_code_node_unref0 (sym);
}

static void
vala_parser_parse_using_directives (ValaParser* self,
                                    ValaNamespace* ns,
                                    GError** error)
{
	GError* _inner_error0_ = NULL;
	g_return_if_fail (self != NULL);
	g_return_if_fail (ns != NULL);
	while (TRUE) {
		if (!vala_parser_accept (self, VALA_TOKEN_TYPE_USING)) {
			break;
		}
		{
			gboolean _tmp0_ = FALSE;
			_tmp0_ = TRUE;
			while (TRUE) {
				ValaSourceLocation begin = {0};
				ValaSourceLocation _tmp1_ = {0};
				ValaUnresolvedSymbol* sym = NULL;
				ValaUnresolvedSymbol* _tmp2_;
				ValaUsingDirective* ns_ref = NULL;
				ValaUnresolvedSymbol* _tmp3_;
				ValaSourceLocation _tmp4_;
				ValaSourceReference* _tmp5_;
				ValaSourceReference* _tmp6_;
				ValaUsingDirective* _tmp7_;
				ValaUsingDirective* _tmp8_;
				ValaScanner* _tmp9_;
				ValaSourceFile* _tmp10_;
				ValaSourceFile* _tmp11_;
				ValaUsingDirective* _tmp12_;
				ValaUsingDirective* _tmp13_;
				if (!_tmp0_) {
					if (!vala_parser_accept (self, VALA_TOKEN_TYPE_COMMA)) {
						break;
					}
				}
				_tmp0_ = FALSE;
				vala_parser_get_location (self, &_tmp1_);
				begin = _tmp1_;
				_tmp2_ = vala_parser_parse_symbol_name (self, &_inner_error0_);
				sym = _tmp2_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						return;
					} else {
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return;
					}
				}
				_tmp3_ = sym;
				_tmp4_ = begin;
				_tmp5_ = vala_parser_get_src (self, &_tmp4_);
				_tmp6_ = _tmp5_;
				_tmp7_ = vala_using_directive_new ((ValaSymbol*) _tmp3_, _tmp6_);
				_tmp8_ = _tmp7_;
				_vala_source_reference_unref0 (_tmp6_);
				ns_ref = _tmp8_;
				_tmp9_ = self->priv->scanner;
				_tmp10_ = vala_scanner_get_source_file (_tmp9_);
				_tmp11_ = _tmp10_;
				_tmp12_ = ns_ref;
				vala_source_file_add_using_directive (_tmp11_, _tmp12_);
				_tmp13_ = ns_ref;
				vala_namespace_add_using_directive (ns, _tmp13_);
				_vala_code_node_unref0 (ns_ref);
				_vala_code_node_unref0 (sym);
			}
		}
		vala_parser_expect (self, VALA_TOKEN_TYPE_SEMICOLON, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				return;
			} else {
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
	}
}

static void
_vala_array_add14 (ValaParserPartialInfo* * array,
                   gint* length,
                   gint* size,
                   const ValaParserPartialInfo* value)
{
	if ((*length) == (*size)) {
		*size = (*size) ? (2 * (*size)) : 4;
		*array = g_renew (ValaParserPartialInfo, *array, *size);
	}
	(*array)[(*length)++] = *value;
}

static void
vala_parser_parse_class_declaration (ValaParser* self,
                                     ValaSymbol* parent,
                                     ValaList* attrs,
                                     gboolean partial_reparse,
                                     GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaSymbolAccessibility access = 0;
	ValaParserModifierFlags flags = 0U;
	ValaUnresolvedSymbol* sym = NULL;
	ValaUnresolvedSymbol* _tmp1_;
	ValaList* type_param_list = NULL;
	ValaList* _tmp2_;
	ValaArrayList* base_types = NULL;
	GEqualFunc _tmp3_;
	ValaArrayList* _tmp4_;
	ValaClass* cl = NULL;
	ValaUnresolvedSymbol* _tmp9_;
	const gchar* _tmp10_;
	const gchar* _tmp11_;
	ValaSourceLocation _tmp12_;
	ValaSourceReference* _tmp13_;
	ValaSourceReference* _tmp14_;
	ValaComment* _tmp15_;
	ValaClass* _tmp16_;
	ValaClass* _tmp17_;
	ValaClass* _tmp18_;
	ValaClass* old_cl = NULL;
	gboolean _tmp85_ = FALSE;
	ValaClass* _tmp86_;
	ValaClass* _tmp153_;
	ValaClass* _tmp174_;
	gboolean _tmp175_ = FALSE;
	ValaClass* _tmp176_;
	gboolean _tmp180_ = FALSE;
	ValaScanner* _tmp181_;
	ValaSourceFile* _tmp182_;
	ValaSourceFile* _tmp183_;
	ValaSourceFileType _tmp184_;
	ValaSourceFileType _tmp185_;
	ValaSymbol* _result_ = NULL;
	ValaClass* _tmp210_;
	ValaSymbol* _tmp211_;
	GError* _inner_error0_ = NULL;
	g_return_if_fail (self != NULL);
	g_return_if_fail (parent != NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	access = vala_parser_parse_access_modifier (self, VALA_SYMBOL_ACCESSIBILITY_PRIVATE);
	flags = vala_parser_parse_type_declaration_modifiers (self);
	vala_parser_expect (self, VALA_TOKEN_TYPE_CLASS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp1_ = vala_parser_parse_symbol_name (self, &_inner_error0_);
	sym = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp2_ = vala_parser_parse_type_parameter_list (self, &_inner_error0_);
	type_param_list = _tmp2_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (sym);
			return;
		} else {
			_vala_code_node_unref0 (sym);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp3_ = g_direct_equal;
	_tmp4_ = vala_array_list_new (VALA_TYPE_DATA_TYPE, (GBoxedCopyFunc) vala_code_node_ref, (GDestroyNotify) vala_code_node_unref, _tmp3_);
	base_types = _tmp4_;
	if (vala_parser_accept (self, VALA_TOKEN_TYPE_COLON)) {
		{
			gboolean _tmp5_ = FALSE;
			_tmp5_ = TRUE;
			while (TRUE) {
				ValaDataType* _tmp6_ = NULL;
				ValaDataType* _tmp7_;
				ValaArrayList* _tmp8_;
				if (!_tmp5_) {
					if (!vala_parser_accept (self, VALA_TOKEN_TYPE_COMMA)) {
						break;
					}
				}
				_tmp5_ = FALSE;
				_tmp7_ = vala_parser_parse_type (self, TRUE, FALSE, FALSE, &_inner_error0_);
				_tmp6_ = _tmp7_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_iterable_unref0 (base_types);
						_vala_iterable_unref0 (type_param_list);
						_vala_code_node_unref0 (sym);
						return;
					} else {
						_vala_iterable_unref0 (base_types);
						_vala_iterable_unref0 (type_param_list);
						_vala_code_node_unref0 (sym);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return;
					}
				}
				_tmp8_ = base_types;
				vala_collection_add ((ValaCollection*) _tmp8_, _tmp6_);
				_vala_code_node_unref0 (_tmp6_);
			}
		}
	}
	_tmp9_ = sym;
	_tmp10_ = vala_symbol_get_name ((ValaSymbol*) _tmp9_);
	_tmp11_ = _tmp10_;
	_tmp12_ = begin;
	_tmp13_ = vala_parser_get_src (self, &_tmp12_);
	_tmp14_ = _tmp13_;
	_tmp15_ = self->priv->comment;
	_tmp16_ = vala_class_new (_tmp11_, _tmp14_, _tmp15_);
	_tmp17_ = _tmp16_;
	_vala_source_reference_unref0 (_tmp14_);
	cl = _tmp17_;
	_tmp18_ = cl;
	vala_symbol_set_access ((ValaSymbol*) _tmp18_, access);
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_ABSTRACT) == VALA_PARSER_MODIFIER_FLAGS_ABSTRACT) {
		ValaClass* _tmp19_;
		_tmp19_ = cl;
		vala_class_set_is_abstract (_tmp19_, TRUE);
	}
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_SEALED) == VALA_PARSER_MODIFIER_FLAGS_SEALED) {
		ValaClass* _tmp20_;
		_tmp20_ = cl;
		vala_class_set_is_sealed (_tmp20_, TRUE);
	}
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_PARTIAL) == VALA_PARSER_MODIFIER_FLAGS_PARTIAL) {
		ValaCodeContext* _tmp21_;
		gboolean _tmp22_;
		gboolean _tmp23_;
		ValaClass* _tmp27_;
		_tmp21_ = self->priv->context;
		_tmp22_ = vala_code_context_get_experimental (_tmp21_);
		_tmp23_ = _tmp22_;
		if (!_tmp23_) {
			ValaClass* _tmp24_;
			ValaSourceReference* _tmp25_;
			ValaSourceReference* _tmp26_;
			_tmp24_ = cl;
			_tmp25_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp24_);
			_tmp26_ = _tmp25_;
			vala_report_warning (_tmp26_, "`partial' classes are experimental");
		}
		_tmp27_ = cl;
		vala_class_set_is_partial (_tmp27_, TRUE);
	}
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_EXTERN) == VALA_PARSER_MODIFIER_FLAGS_EXTERN) {
		ValaClass* _tmp28_;
		_tmp28_ = cl;
		vala_symbol_set_is_extern ((ValaSymbol*) _tmp28_, TRUE);
	}
	old_cl = NULL;
	if (partial_reparse) {
		ValaArrayList* names = NULL;
		GEqualFunc _tmp29_;
		ValaArrayList* _tmp30_;
		ValaSymbol* p = NULL;
		ValaSymbol* _tmp42_;
		ValaCodeContext* _tmp57_;
		ValaNamespace* _tmp58_;
		ValaNamespace* _tmp59_;
		ValaSymbol* _tmp60_;
		ValaSymbol* _tmp75_;
		_tmp29_ = g_direct_equal;
		_tmp30_ = vala_array_list_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, _tmp29_);
		names = _tmp30_;
		while (TRUE) {
			ValaUnresolvedSymbol* _tmp31_;
			ValaUnresolvedSymbol* _tmp32_;
			ValaUnresolvedSymbol* _tmp33_;
			ValaUnresolvedSymbol* _tmp34_;
			ValaUnresolvedSymbol* _tmp35_;
			ValaUnresolvedSymbol* _tmp36_;
			ValaUnresolvedSymbol* _tmp37_;
			ValaArrayList* _tmp38_;
			ValaUnresolvedSymbol* _tmp39_;
			const gchar* _tmp40_;
			const gchar* _tmp41_;
			_tmp31_ = sym;
			_tmp32_ = vala_unresolved_symbol_get_inner (_tmp31_);
			_tmp33_ = _tmp32_;
			if (!(_tmp33_ != NULL)) {
				break;
			}
			_tmp34_ = sym;
			_tmp35_ = vala_unresolved_symbol_get_inner (_tmp34_);
			_tmp36_ = _tmp35_;
			_tmp37_ = _vala_code_node_ref0 (_tmp36_);
			_vala_code_node_unref0 (sym);
			sym = _tmp37_;
			_tmp38_ = names;
			_tmp39_ = sym;
			_tmp40_ = vala_symbol_get_name ((ValaSymbol*) _tmp39_);
			_tmp41_ = _tmp40_;
			vala_list_insert ((ValaList*) _tmp38_, 0, _tmp41_);
		}
		_tmp42_ = _vala_code_node_ref0 (parent);
		p = _tmp42_;
		while (TRUE) {
			gboolean _tmp43_ = FALSE;
			ValaSymbol* _tmp44_;
			ValaArrayList* _tmp49_;
			ValaSymbol* _tmp50_;
			const gchar* _tmp51_;
			const gchar* _tmp52_;
			ValaSymbol* _tmp53_;
			ValaSymbol* _tmp54_;
			ValaSymbol* _tmp55_;
			ValaSymbol* _tmp56_;
			_tmp44_ = p;
			if (_tmp44_ != NULL) {
				ValaSymbol* _tmp45_;
				ValaCodeContext* _tmp46_;
				ValaNamespace* _tmp47_;
				ValaNamespace* _tmp48_;
				_tmp45_ = p;
				_tmp46_ = self->priv->context;
				_tmp47_ = vala_code_context_get_root (_tmp46_);
				_tmp48_ = _tmp47_;
				_tmp43_ = _tmp45_ != G_TYPE_CHECK_INSTANCE_CAST (_tmp48_, VALA_TYPE_SYMBOL, ValaSymbol);
			} else {
				_tmp43_ = FALSE;
			}
			if (!_tmp43_) {
				break;
			}
			_tmp49_ = names;
			_tmp50_ = p;
			_tmp51_ = vala_symbol_get_name (_tmp50_);
			_tmp52_ = _tmp51_;
			vala_list_insert ((ValaList*) _tmp49_, 0, _tmp52_);
			_tmp53_ = p;
			_tmp54_ = vala_symbol_get_parent_symbol (_tmp53_);
			_tmp55_ = _tmp54_;
			_tmp56_ = _vala_code_node_ref0 (_tmp55_);
			_vala_code_node_unref0 (p);
			p = _tmp56_;
		}
		_tmp57_ = self->priv->context;
		_tmp58_ = vala_code_context_get_root (_tmp57_);
		_tmp59_ = _tmp58_;
		_tmp60_ = _vala_code_node_ref0 ((ValaSymbol*) _tmp59_);
		_vala_code_node_unref0 (p);
		p = _tmp60_;
		{
			ValaArrayList* _name_list = NULL;
			ValaArrayList* _tmp61_;
			gint _name_size = 0;
			ValaArrayList* _tmp62_;
			gint _tmp63_;
			gint _tmp64_;
			gint _name_index = 0;
			_tmp61_ = names;
			_name_list = _tmp61_;
			_tmp62_ = _name_list;
			_tmp63_ = vala_collection_get_size ((ValaCollection*) _tmp62_);
			_tmp64_ = _tmp63_;
			_name_size = _tmp64_;
			_name_index = -1;
			while (TRUE) {
				gint _tmp65_;
				gint _tmp66_;
				gchar* name = NULL;
				ValaArrayList* _tmp67_;
				gpointer _tmp68_;
				ValaSymbol* _tmp69_;
				ValaScope* _tmp70_;
				ValaScope* _tmp71_;
				const gchar* _tmp72_;
				ValaSymbol* _tmp73_;
				ValaSymbol* _tmp74_;
				_name_index = _name_index + 1;
				_tmp65_ = _name_index;
				_tmp66_ = _name_size;
				if (!(_tmp65_ < _tmp66_)) {
					break;
				}
				_tmp67_ = _name_list;
				_tmp68_ = vala_list_get ((ValaList*) _tmp67_, _name_index);
				name = (gchar*) _tmp68_;
				_tmp69_ = p;
				_tmp70_ = vala_symbol_get_scope (_tmp69_);
				_tmp71_ = _tmp70_;
				_tmp72_ = name;
				_tmp73_ = vala_scope_lookup (_tmp71_, _tmp72_);
				_vala_code_node_unref0 (p);
				p = _tmp73_;
				_tmp74_ = p;
				if (_tmp74_ == NULL) {
					_g_free0 (name);
					break;
				}
				_g_free0 (name);
			}
		}
		_tmp75_ = p;
		if (_tmp75_ != NULL) {
			ValaSymbol* _tmp76_;
			ValaScope* _tmp77_;
			ValaScope* _tmp78_;
			ValaClass* _tmp79_;
			const gchar* _tmp80_;
			const gchar* _tmp81_;
			ValaSymbol* _tmp82_;
			ValaClass* _tmp83_;
			ValaSymbol* _tmp84_;
			_tmp76_ = p;
			_tmp77_ = vala_symbol_get_scope (_tmp76_);
			_tmp78_ = _tmp77_;
			_tmp79_ = cl;
			_tmp80_ = vala_symbol_get_name ((ValaSymbol*) _tmp79_);
			_tmp81_ = _tmp80_;
			_tmp82_ = vala_scope_lookup (_tmp78_, _tmp81_);
			_tmp83_ = VALA_IS_CLASS (_tmp82_) ? ((ValaClass*) _tmp82_) : NULL;
			if (_tmp83_ == NULL) {
				_vala_code_node_unref0 (_tmp82_);
			}
			_vala_code_node_unref0 (old_cl);
			old_cl = _tmp83_;
			_tmp84_ = p;
			parent = _tmp84_;
		}
		_vala_code_node_unref0 (p);
		_vala_iterable_unref0 (names);
	}
	_tmp86_ = old_cl;
	if (_tmp86_ != NULL) {
		ValaClass* _tmp87_;
		gboolean _tmp88_;
		gboolean _tmp89_;
		_tmp87_ = old_cl;
		_tmp88_ = vala_class_get_is_partial (_tmp87_);
		_tmp89_ = _tmp88_;
		_tmp85_ = _tmp89_;
	} else {
		_tmp85_ = FALSE;
	}
	if (_tmp85_) {
		ValaClass* _tmp90_;
		gboolean _tmp91_;
		gboolean _tmp92_;
		ValaClass* _tmp93_;
		gboolean _tmp94_;
		gboolean _tmp95_;
		ValaClass* _tmp103_;
		ValaSymbolAccessibility _tmp104_;
		ValaSymbolAccessibility _tmp105_;
		ValaClass* _tmp106_;
		ValaSymbolAccessibility _tmp107_;
		ValaSymbolAccessibility _tmp108_;
		ValaClass* _tmp116_;
		gboolean _tmp117_;
		gboolean _tmp118_;
		ValaClass* _tmp119_;
		gboolean _tmp120_;
		gboolean _tmp121_;
		ValaClass* _tmp129_;
		gboolean _tmp130_;
		gboolean _tmp131_;
		ValaClass* _tmp132_;
		gboolean _tmp133_;
		gboolean _tmp134_;
		ValaClass* _tmp142_;
		gboolean _tmp143_;
		gboolean _tmp144_;
		ValaClass* _tmp151_;
		ValaClass* _tmp152_;
		_tmp90_ = cl;
		_tmp91_ = vala_class_get_is_partial (_tmp90_);
		_tmp92_ = _tmp91_;
		_tmp93_ = old_cl;
		_tmp94_ = vala_class_get_is_partial (_tmp93_);
		_tmp95_ = _tmp94_;
		if (_tmp92_ != _tmp95_) {
			ValaClass* _tmp96_;
			ValaSourceReference* _tmp97_;
			ValaSourceReference* _tmp98_;
			ValaClass* _tmp99_;
			const gchar* _tmp100_;
			const gchar* _tmp101_;
			ValaClass* _tmp102_;
			_tmp96_ = cl;
			_tmp97_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp96_);
			_tmp98_ = _tmp97_;
			_tmp99_ = cl;
			_tmp100_ = vala_symbol_get_name ((ValaSymbol*) _tmp99_);
			_tmp101_ = _tmp100_;
			vala_report_error (_tmp98_, "conflicting partial and not partial declarations of `%s'", _tmp101_);
			_tmp102_ = cl;
			vala_code_node_set_error ((ValaCodeNode*) _tmp102_, TRUE);
		}
		_tmp103_ = cl;
		_tmp104_ = vala_symbol_get_access ((ValaSymbol*) _tmp103_);
		_tmp105_ = _tmp104_;
		_tmp106_ = old_cl;
		_tmp107_ = vala_symbol_get_access ((ValaSymbol*) _tmp106_);
		_tmp108_ = _tmp107_;
		if (_tmp105_ != _tmp108_) {
			ValaClass* _tmp109_;
			ValaSourceReference* _tmp110_;
			ValaSourceReference* _tmp111_;
			ValaClass* _tmp112_;
			const gchar* _tmp113_;
			const gchar* _tmp114_;
			ValaClass* _tmp115_;
			_tmp109_ = cl;
			_tmp110_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp109_);
			_tmp111_ = _tmp110_;
			_tmp112_ = cl;
			_tmp113_ = vala_symbol_get_name ((ValaSymbol*) _tmp112_);
			_tmp114_ = _tmp113_;
			vala_report_error (_tmp111_, "partial declarations of `%s' have conflicting accessiblity modifiers", _tmp114_);
			_tmp115_ = cl;
			vala_code_node_set_error ((ValaCodeNode*) _tmp115_, TRUE);
		}
		_tmp116_ = cl;
		_tmp117_ = vala_class_get_is_abstract (_tmp116_);
		_tmp118_ = _tmp117_;
		_tmp119_ = old_cl;
		_tmp120_ = vala_class_get_is_abstract (_tmp119_);
		_tmp121_ = _tmp120_;
		if (_tmp118_ != _tmp121_) {
			ValaClass* _tmp122_;
			ValaSourceReference* _tmp123_;
			ValaSourceReference* _tmp124_;
			ValaClass* _tmp125_;
			const gchar* _tmp126_;
			const gchar* _tmp127_;
			ValaClass* _tmp128_;
			_tmp122_ = cl;
			_tmp123_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp122_);
			_tmp124_ = _tmp123_;
			_tmp125_ = cl;
			_tmp126_ = vala_symbol_get_name ((ValaSymbol*) _tmp125_);
			_tmp127_ = _tmp126_;
			vala_report_error (_tmp124_, "partial declarations of `%s' have conflicting abstract modifiers", _tmp127_);
			_tmp128_ = cl;
			vala_code_node_set_error ((ValaCodeNode*) _tmp128_, TRUE);
		}
		_tmp129_ = cl;
		_tmp130_ = vala_class_get_is_sealed (_tmp129_);
		_tmp131_ = _tmp130_;
		_tmp132_ = old_cl;
		_tmp133_ = vala_class_get_is_sealed (_tmp132_);
		_tmp134_ = _tmp133_;
		if (_tmp131_ != _tmp134_) {
			ValaClass* _tmp135_;
			ValaSourceReference* _tmp136_;
			ValaSourceReference* _tmp137_;
			ValaClass* _tmp138_;
			const gchar* _tmp139_;
			const gchar* _tmp140_;
			ValaClass* _tmp141_;
			_tmp135_ = cl;
			_tmp136_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp135_);
			_tmp137_ = _tmp136_;
			_tmp138_ = cl;
			_tmp139_ = vala_symbol_get_name ((ValaSymbol*) _tmp138_);
			_tmp140_ = _tmp139_;
			vala_report_error (_tmp137_, "partial declarations of `%s' have conflicting sealed modifiers", _tmp140_);
			_tmp141_ = cl;
			vala_code_node_set_error ((ValaCodeNode*) _tmp141_, TRUE);
		}
		_tmp142_ = cl;
		_tmp143_ = vala_code_node_get_error ((ValaCodeNode*) _tmp142_);
		_tmp144_ = _tmp143_;
		if (_tmp144_) {
			ValaClass* _tmp145_;
			ValaSourceReference* _tmp146_;
			ValaSourceReference* _tmp147_;
			ValaClass* _tmp148_;
			const gchar* _tmp149_;
			const gchar* _tmp150_;
			_tmp145_ = old_cl;
			_tmp146_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp145_);
			_tmp147_ = _tmp146_;
			_tmp148_ = old_cl;
			_tmp149_ = vala_symbol_get_name ((ValaSymbol*) _tmp148_);
			_tmp150_ = _tmp149_;
			vala_report_notice (_tmp147_, "previous declaration of `%s' was here", _tmp150_);
			_vala_code_node_unref0 (old_cl);
			_vala_code_node_unref0 (cl);
			_vala_iterable_unref0 (base_types);
			_vala_iterable_unref0 (type_param_list);
			_vala_code_node_unref0 (sym);
			return;
		}
		_tmp151_ = old_cl;
		_tmp152_ = _vala_code_node_ref0 (_tmp151_);
		_vala_code_node_unref0 (cl);
		cl = _tmp152_;
	}
	_tmp153_ = cl;
	vala_parser_set_attributes (self, (ValaCodeNode*) _tmp153_, attrs);
	{
		ValaList* _type_param_list = NULL;
		ValaList* _tmp154_;
		gint _type_param_size = 0;
		ValaList* _tmp155_;
		gint _tmp156_;
		gint _tmp157_;
		gint _type_param_index = 0;
		_tmp154_ = type_param_list;
		_type_param_list = _tmp154_;
		_tmp155_ = _type_param_list;
		_tmp156_ = vala_collection_get_size ((ValaCollection*) _tmp155_);
		_tmp157_ = _tmp156_;
		_type_param_size = _tmp157_;
		_type_param_index = -1;
		while (TRUE) {
			gint _tmp158_;
			gint _tmp159_;
			ValaTypeParameter* type_param = NULL;
			ValaList* _tmp160_;
			gpointer _tmp161_;
			ValaClass* _tmp162_;
			ValaTypeParameter* _tmp163_;
			_type_param_index = _type_param_index + 1;
			_tmp158_ = _type_param_index;
			_tmp159_ = _type_param_size;
			if (!(_tmp158_ < _tmp159_)) {
				break;
			}
			_tmp160_ = _type_param_list;
			_tmp161_ = vala_list_get (_tmp160_, _type_param_index);
			type_param = (ValaTypeParameter*) _tmp161_;
			_tmp162_ = cl;
			_tmp163_ = type_param;
			vala_object_type_symbol_add_type_parameter ((ValaObjectTypeSymbol*) _tmp162_, _tmp163_);
			_vala_code_node_unref0 (type_param);
		}
	}
	{
		ValaArrayList* _base_type_list = NULL;
		ValaArrayList* _tmp164_;
		gint _base_type_size = 0;
		ValaArrayList* _tmp165_;
		gint _tmp166_;
		gint _tmp167_;
		gint _base_type_index = 0;
		_tmp164_ = base_types;
		_base_type_list = _tmp164_;
		_tmp165_ = _base_type_list;
		_tmp166_ = vala_collection_get_size ((ValaCollection*) _tmp165_);
		_tmp167_ = _tmp166_;
		_base_type_size = _tmp167_;
		_base_type_index = -1;
		while (TRUE) {
			gint _tmp168_;
			gint _tmp169_;
			ValaDataType* base_type = NULL;
			ValaArrayList* _tmp170_;
			gpointer _tmp171_;
			ValaClass* _tmp172_;
			ValaDataType* _tmp173_;
			_base_type_index = _base_type_index + 1;
			_tmp168_ = _base_type_index;
			_tmp169_ = _base_type_size;
			if (!(_tmp168_ < _tmp169_)) {
				break;
			}
			_tmp170_ = _base_type_list;
			_tmp171_ = vala_list_get ((ValaList*) _tmp170_, _base_type_index);
			base_type = (ValaDataType*) _tmp171_;
			_tmp172_ = cl;
			_tmp173_ = base_type;
			vala_class_add_base_type (_tmp172_, _tmp173_);
			_vala_code_node_unref0 (base_type);
		}
	}
	_tmp174_ = cl;
	vala_parser_parse_declarations (self, (ValaSymbol*) _tmp174_, FALSE, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (old_cl);
			_vala_code_node_unref0 (cl);
			_vala_iterable_unref0 (base_types);
			_vala_iterable_unref0 (type_param_list);
			_vala_code_node_unref0 (sym);
			return;
		} else {
			_vala_code_node_unref0 (old_cl);
			_vala_code_node_unref0 (cl);
			_vala_iterable_unref0 (base_types);
			_vala_iterable_unref0 (type_param_list);
			_vala_code_node_unref0 (sym);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp176_ = old_cl;
	if (_tmp176_ != NULL) {
		ValaClass* _tmp177_;
		gboolean _tmp178_;
		gboolean _tmp179_;
		_tmp177_ = old_cl;
		_tmp178_ = vala_class_get_is_partial (_tmp177_);
		_tmp179_ = _tmp178_;
		_tmp175_ = _tmp179_;
	} else {
		_tmp175_ = FALSE;
	}
	if (_tmp175_) {
		_vala_code_node_unref0 (old_cl);
		_vala_code_node_unref0 (cl);
		_vala_iterable_unref0 (base_types);
		_vala_iterable_unref0 (type_param_list);
		_vala_code_node_unref0 (sym);
		return;
	}
	_tmp181_ = self->priv->scanner;
	_tmp182_ = vala_scanner_get_source_file (_tmp181_);
	_tmp183_ = _tmp182_;
	_tmp184_ = vala_source_file_get_file_type (_tmp183_);
	_tmp185_ = _tmp184_;
	if (_tmp185_ == VALA_SOURCE_FILE_TYPE_SOURCE) {
		ValaClass* _tmp186_;
		ValaCreationMethod* _tmp187_;
		ValaCreationMethod* _tmp188_;
		_tmp186_ = cl;
		_tmp187_ = vala_class_get_default_construction_method (_tmp186_);
		_tmp188_ = _tmp187_;
		_tmp180_ = _tmp188_ == NULL;
	} else {
		_tmp180_ = FALSE;
	}
	if (_tmp180_) {
		ValaCreationMethod* m = NULL;
		ValaClass* _tmp189_;
		const gchar* _tmp190_;
		const gchar* _tmp191_;
		ValaClass* _tmp192_;
		ValaSourceReference* _tmp193_;
		ValaSourceReference* _tmp194_;
		ValaCreationMethod* _tmp195_;
		ValaSymbolAccessibility _tmp196_ = 0;
		ValaClass* _tmp197_;
		gboolean _tmp198_;
		gboolean _tmp199_;
		ValaCreationMethod* _tmp200_;
		ValaCreationMethod* _tmp201_;
		ValaClass* _tmp202_;
		ValaSourceReference* _tmp203_;
		ValaSourceReference* _tmp204_;
		ValaBlock* _tmp205_;
		ValaBlock* _tmp206_;
		ValaClass* _tmp207_;
		ValaCreationMethod* _tmp208_;
		_tmp189_ = cl;
		_tmp190_ = vala_symbol_get_name ((ValaSymbol*) _tmp189_);
		_tmp191_ = _tmp190_;
		_tmp192_ = cl;
		_tmp193_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp192_);
		_tmp194_ = _tmp193_;
		_tmp195_ = vala_creation_method_new (_tmp191_, NULL, _tmp194_, NULL);
		m = _tmp195_;
		_tmp197_ = cl;
		_tmp198_ = vala_class_get_is_abstract (_tmp197_);
		_tmp199_ = _tmp198_;
		if (_tmp199_) {
			_tmp196_ = VALA_SYMBOL_ACCESSIBILITY_PROTECTED;
		} else {
			_tmp196_ = VALA_SYMBOL_ACCESSIBILITY_PUBLIC;
		}
		_tmp200_ = m;
		vala_symbol_set_access ((ValaSymbol*) _tmp200_, _tmp196_);
		_tmp201_ = m;
		_tmp202_ = cl;
		_tmp203_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp202_);
		_tmp204_ = _tmp203_;
		_tmp205_ = vala_block_new (_tmp204_);
		_tmp206_ = _tmp205_;
		vala_subroutine_set_body ((ValaSubroutine*) _tmp201_, _tmp206_);
		_vala_code_node_unref0 (_tmp206_);
		_tmp207_ = cl;
		_tmp208_ = m;
		vala_symbol_add_method ((ValaSymbol*) _tmp207_, (ValaMethod*) _tmp208_);
		_vala_code_node_unref0 (m);
	}
	if (partial_reparse) {
		ValaClass* _tmp209_;
		_tmp209_ = cl;
		vala_symbol_add_class (parent, _tmp209_);
		_vala_code_node_unref0 (old_cl);
		_vala_code_node_unref0 (cl);
		_vala_iterable_unref0 (base_types);
		_vala_iterable_unref0 (type_param_list);
		_vala_code_node_unref0 (sym);
		return;
	}
	_tmp210_ = cl;
	_tmp211_ = _vala_code_node_ref0 ((ValaSymbol*) _tmp210_);
	_result_ = _tmp211_;
	while (TRUE) {
		ValaUnresolvedSymbol* _tmp212_;
		ValaUnresolvedSymbol* _tmp213_;
		ValaUnresolvedSymbol* _tmp214_;
		ValaUnresolvedSymbol* _tmp215_;
		ValaUnresolvedSymbol* _tmp216_;
		ValaSymbol* _tmp217_ = NULL;
		ValaUnresolvedSymbol* _tmp218_;
		ValaSymbol* next = NULL;
		ValaSymbol* _tmp227_;
		ValaSymbol* _tmp228_;
		ValaSymbol* _tmp247_;
		ValaSymbol* _tmp248_;
		_tmp212_ = sym;
		if (!(_tmp212_ != NULL)) {
			break;
		}
		_tmp213_ = sym;
		_tmp214_ = vala_unresolved_symbol_get_inner (_tmp213_);
		_tmp215_ = _tmp214_;
		_tmp216_ = _vala_code_node_ref0 (_tmp215_);
		_vala_code_node_unref0 (sym);
		sym = _tmp216_;
		_tmp218_ = sym;
		if (_tmp218_ != NULL) {
			ValaUnresolvedSymbol* _tmp219_;
			const gchar* _tmp220_;
			const gchar* _tmp221_;
			ValaClass* _tmp222_;
			ValaSourceReference* _tmp223_;
			ValaSourceReference* _tmp224_;
			ValaNamespace* _tmp225_;
			_tmp219_ = sym;
			_tmp220_ = vala_symbol_get_name ((ValaSymbol*) _tmp219_);
			_tmp221_ = _tmp220_;
			_tmp222_ = cl;
			_tmp223_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp222_);
			_tmp224_ = _tmp223_;
			_tmp225_ = vala_namespace_new (_tmp221_, _tmp224_);
			_vala_code_node_unref0 (_tmp217_);
			_tmp217_ = (ValaSymbol*) _tmp225_;
		} else {
			ValaSymbol* _tmp226_;
			_tmp226_ = _vala_code_node_ref0 (parent);
			_vala_code_node_unref0 (_tmp217_);
			_tmp217_ = _tmp226_;
		}
		_tmp227_ = _vala_code_node_ref0 (_tmp217_);
		next = _tmp227_;
		_tmp228_ = _result_;
		if (VALA_IS_NAMESPACE (_tmp228_)) {
			ValaSymbol* _tmp229_;
			ValaSymbol* _tmp230_;
			_tmp229_ = next;
			_tmp230_ = _result_;
			vala_symbol_add_namespace (_tmp229_, G_TYPE_CHECK_INSTANCE_CAST (_tmp230_, VALA_TYPE_NAMESPACE, ValaNamespace));
		} else {
			gboolean _tmp231_ = FALSE;
			if (!partial_reparse) {
				ValaClass* _tmp232_;
				gboolean _tmp233_;
				gboolean _tmp234_;
				_tmp232_ = cl;
				_tmp233_ = vala_class_get_is_partial (_tmp232_);
				_tmp234_ = _tmp233_;
				_tmp231_ = _tmp234_;
			} else {
				_tmp231_ = FALSE;
			}
			if (_tmp231_) {
				ValaParserPartialInfo info = {0};
				ValaSymbol* _tmp235_;
				ValaSourceLocation _tmp236_;
				ValaList* _tmp237_;
				ValaParserPartialInfo _tmp238_ = {0};
				ValaParserPartialInfo _tmp239_;
				ValaParserPartialInfo _tmp240_;
				ValaParserPartialInfo _tmp241_ = {0};
				_tmp235_ = _vala_code_node_ref0 (parent);
				_tmp236_ = begin;
				_tmp237_ = _vala_iterable_ref0 (attrs);
				_vala_code_node_unref0 (_tmp238_.parent);
				_tmp238_.parent = _tmp235_;
				_tmp238_.begin = _tmp236_;
				_vala_iterable_unref0 (_tmp238_.attributes);
				_tmp238_.attributes = _tmp237_;
				info = _tmp238_;
				_tmp239_ = info;
				_tmp240_ = _tmp239_;
				vala_parser_partial_info_copy (&_tmp240_, &_tmp241_);
				_vala_array_add14 (&self->priv->partials, &self->priv->partials_length1, &self->priv->_partials_size_, &_tmp241_);
				vala_parser_partial_info_destroy (&info);
			} else {
				ValaClass* _tmp242_;
				gboolean _tmp243_;
				gboolean _tmp244_;
				_tmp242_ = cl;
				_tmp243_ = vala_class_get_is_partial (_tmp242_);
				_tmp244_ = _tmp243_;
				if (!_tmp244_) {
					ValaSymbol* _tmp245_;
					ValaSymbol* _tmp246_;
					_tmp245_ = next;
					_tmp246_ = _result_;
					vala_symbol_add_class (_tmp245_, G_TYPE_CHECK_INSTANCE_CAST (_tmp246_, VALA_TYPE_CLASS, ValaClass));
				}
			}
		}
		_tmp247_ = next;
		_tmp248_ = _vala_code_node_ref0 (_tmp247_);
		_vala_code_node_unref0 (_result_);
		_result_ = _tmp248_;
		_vala_code_node_unref0 (next);
		_vala_code_node_unref0 (_tmp217_);
	}
	_vala_code_node_unref0 (_result_);
	_vala_code_node_unref0 (old_cl);
	_vala_code_node_unref0 (cl);
	_vala_iterable_unref0 (base_types);
	_vala_iterable_unref0 (type_param_list);
	_vala_code_node_unref0 (sym);
}

static void
vala_parser_parse_constant_declaration (ValaParser* self,
                                        ValaSymbol* parent,
                                        ValaList* attrs,
                                        GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaSymbolAccessibility access = 0;
	ValaParserModifierFlags flags = 0U;
	ValaDataType* type = NULL;
	ValaDataType* _tmp1_;
	gchar* id = NULL;
	gchar* _tmp2_;
	ValaDataType* _tmp3_ = NULL;
	ValaDataType* _tmp4_;
	ValaDataType* _tmp5_;
	ValaDataType* _tmp6_;
	ValaSourceReference* src = NULL;
	ValaSourceLocation _tmp7_;
	ValaSourceReference* _tmp8_;
	ValaArrayType* array_type = NULL;
	ValaDataType* _tmp9_;
	ValaArrayType* _tmp10_;
	ValaExpression* initializer = NULL;
	ValaConstant* c = NULL;
	const gchar* _tmp17_;
	ValaDataType* _tmp18_;
	ValaExpression* _tmp19_;
	ValaSourceReference* _tmp20_;
	ValaComment* _tmp21_;
	ValaConstant* _tmp22_;
	ValaConstant* _tmp23_;
	ValaConstant* _tmp26_;
	ValaDataType* _tmp30_;
	gboolean _tmp31_;
	gboolean _tmp32_;
	ValaConstant* _tmp36_;
	GError* _inner_error0_ = NULL;
	g_return_if_fail (self != NULL);
	g_return_if_fail (parent != NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	access = vala_parser_parse_access_modifier (self, VALA_SYMBOL_ACCESSIBILITY_PRIVATE);
	flags = vala_parser_parse_member_declaration_modifiers (self);
	vala_parser_expect (self, VALA_TOKEN_TYPE_CONST, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp1_ = vala_parser_parse_type (self, FALSE, FALSE, FALSE, &_inner_error0_);
	type = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp2_ = vala_parser_parse_identifier (self, FALSE, &_inner_error0_);
	id = _tmp2_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (type);
			return;
		} else {
			_vala_code_node_unref0 (type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp4_ = type;
	_tmp5_ = vala_parser_parse_inline_array_type (self, _tmp4_, &_inner_error0_);
	_tmp3_ = _tmp5_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_g_free0 (id);
			_vala_code_node_unref0 (type);
			return;
		} else {
			_g_free0 (id);
			_vala_code_node_unref0 (type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp6_ = _tmp3_;
	_tmp3_ = NULL;
	_vala_code_node_unref0 (type);
	type = _tmp6_;
	_tmp7_ = begin;
	_tmp8_ = vala_parser_get_src (self, &_tmp7_);
	src = _tmp8_;
	_tmp9_ = type;
	array_type = VALA_IS_ARRAY_TYPE (_tmp9_) ? ((ValaArrayType*) _tmp9_) : NULL;
	_tmp10_ = array_type;
	if (_tmp10_ != NULL) {
		ValaArrayType* _tmp11_;
		ValaDataType* _tmp12_;
		ValaDataType* _tmp13_;
		_tmp11_ = array_type;
		_tmp12_ = vala_array_type_get_element_type (_tmp11_);
		_tmp13_ = _tmp12_;
		vala_data_type_set_value_owned (_tmp13_, FALSE);
	}
	initializer = NULL;
	if (vala_parser_accept (self, VALA_TOKEN_TYPE_ASSIGN)) {
		ValaExpression* _tmp14_ = NULL;
		ValaExpression* _tmp15_;
		ValaExpression* _tmp16_;
		_tmp15_ = vala_parser_parse_expression (self, &_inner_error0_);
		_tmp14_ = _tmp15_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (initializer);
				_vala_source_reference_unref0 (src);
				_vala_code_node_unref0 (_tmp3_);
				_g_free0 (id);
				_vala_code_node_unref0 (type);
				return;
			} else {
				_vala_code_node_unref0 (initializer);
				_vala_source_reference_unref0 (src);
				_vala_code_node_unref0 (_tmp3_);
				_g_free0 (id);
				_vala_code_node_unref0 (type);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
		_tmp16_ = _tmp14_;
		_tmp14_ = NULL;
		_vala_code_node_unref0 (initializer);
		initializer = _tmp16_;
		_vala_code_node_unref0 (_tmp14_);
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_SEMICOLON, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (initializer);
			_vala_source_reference_unref0 (src);
			_vala_code_node_unref0 (_tmp3_);
			_g_free0 (id);
			_vala_code_node_unref0 (type);
			return;
		} else {
			_vala_code_node_unref0 (initializer);
			_vala_source_reference_unref0 (src);
			_vala_code_node_unref0 (_tmp3_);
			_g_free0 (id);
			_vala_code_node_unref0 (type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp17_ = id;
	_tmp18_ = type;
	_tmp19_ = initializer;
	_tmp20_ = src;
	_tmp21_ = self->priv->comment;
	_tmp22_ = vala_constant_new (_tmp17_, _tmp18_, _tmp19_, _tmp20_, _tmp21_);
	c = _tmp22_;
	_tmp23_ = c;
	vala_symbol_set_access ((ValaSymbol*) _tmp23_, access);
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_EXTERN) == VALA_PARSER_MODIFIER_FLAGS_EXTERN) {
		ValaConstant* _tmp24_;
		_tmp24_ = c;
		vala_symbol_set_is_extern ((ValaSymbol*) _tmp24_, TRUE);
	}
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_NEW) == VALA_PARSER_MODIFIER_FLAGS_NEW) {
		ValaConstant* _tmp25_;
		_tmp25_ = c;
		vala_symbol_set_hides ((ValaSymbol*) _tmp25_, TRUE);
	}
	_tmp26_ = c;
	vala_parser_set_attributes (self, (ValaCodeNode*) _tmp26_, attrs);
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_STATIC) == VALA_PARSER_MODIFIER_FLAGS_STATIC) {
		ValaConstant* _tmp27_;
		ValaSourceReference* _tmp28_;
		ValaSourceReference* _tmp29_;
		_tmp27_ = c;
		_tmp28_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp27_);
		_tmp29_ = _tmp28_;
		vala_report_warning (_tmp29_, "the modifier `static' is not applicable to constants");
	}
	_tmp30_ = type;
	_tmp31_ = vala_data_type_get_value_owned (_tmp30_);
	_tmp32_ = _tmp31_;
	if (_tmp32_) {
		ValaConstant* _tmp33_;
		ValaSourceReference* _tmp34_;
		ValaSourceReference* _tmp35_;
		_tmp33_ = c;
		_tmp34_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp33_);
		_tmp35_ = _tmp34_;
		vala_report_error (_tmp35_, "`owned' is not allowed on constants");
	}
	_tmp36_ = c;
	vala_symbol_add_constant (parent, _tmp36_);
	_vala_code_node_unref0 (c);
	_vala_code_node_unref0 (initializer);
	_vala_source_reference_unref0 (src);
	_vala_code_node_unref0 (_tmp3_);
	_g_free0 (id);
	_vala_code_node_unref0 (type);
}

static void
vala_parser_parse_field_declaration (ValaParser* self,
                                     ValaSymbol* parent,
                                     ValaList* attrs,
                                     GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaSymbolAccessibility _tmp1_ = 0;
	ValaSymbolAccessibility access = 0;
	ValaParserModifierFlags flags = 0U;
	ValaDataType* type = NULL;
	ValaDataType* _tmp2_;
	gchar* id = NULL;
	gchar* _tmp3_;
	ValaDataType* _tmp4_ = NULL;
	ValaDataType* _tmp5_;
	ValaDataType* _tmp6_;
	ValaDataType* _tmp7_;
	ValaField* f = NULL;
	const gchar* _tmp8_;
	ValaDataType* _tmp9_;
	ValaSourceLocation _tmp10_;
	ValaSourceReference* _tmp11_;
	ValaSourceReference* _tmp12_;
	ValaComment* _tmp13_;
	ValaField* _tmp14_;
	ValaField* _tmp15_;
	ValaField* _tmp16_;
	ValaField* _tmp17_;
	gboolean _tmp18_ = FALSE;
	gboolean _tmp25_ = FALSE;
	gboolean _tmp26_ = FALSE;
	gboolean _tmp27_ = FALSE;
	gboolean _tmp28_;
	gboolean _tmp29_;
	gboolean _tmp39_ = FALSE;
	gboolean _tmp40_ = FALSE;
	ValaField* _tmp49_;
	GError* _inner_error0_ = NULL;
	g_return_if_fail (self != NULL);
	g_return_if_fail (parent != NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	if (VALA_IS_STRUCT (parent)) {
		_tmp1_ = VALA_SYMBOL_ACCESSIBILITY_PUBLIC;
	} else {
		_tmp1_ = VALA_SYMBOL_ACCESSIBILITY_PRIVATE;
	}
	access = vala_parser_parse_access_modifier (self, _tmp1_);
	flags = vala_parser_parse_member_declaration_modifiers (self);
	_tmp2_ = vala_parser_parse_type (self, TRUE, TRUE, FALSE, &_inner_error0_);
	type = _tmp2_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp3_ = vala_parser_parse_identifier (self, FALSE, &_inner_error0_);
	id = _tmp3_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (type);
			return;
		} else {
			_vala_code_node_unref0 (type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp5_ = type;
	_tmp6_ = vala_parser_parse_inline_array_type (self, _tmp5_, &_inner_error0_);
	_tmp4_ = _tmp6_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_g_free0 (id);
			_vala_code_node_unref0 (type);
			return;
		} else {
			_g_free0 (id);
			_vala_code_node_unref0 (type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp7_ = _tmp4_;
	_tmp4_ = NULL;
	_vala_code_node_unref0 (type);
	type = _tmp7_;
	_tmp8_ = id;
	_tmp9_ = type;
	_tmp10_ = begin;
	_tmp11_ = vala_parser_get_src (self, &_tmp10_);
	_tmp12_ = _tmp11_;
	_tmp13_ = self->priv->comment;
	_tmp14_ = vala_field_new (_tmp8_, _tmp9_, NULL, _tmp12_, _tmp13_);
	_tmp15_ = _tmp14_;
	_vala_source_reference_unref0 (_tmp12_);
	f = _tmp15_;
	_tmp16_ = f;
	vala_symbol_set_access ((ValaSymbol*) _tmp16_, access);
	_tmp17_ = f;
	vala_parser_set_attributes (self, (ValaCodeNode*) _tmp17_, attrs);
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_STATIC) == VALA_PARSER_MODIFIER_FLAGS_STATIC) {
		_tmp18_ = (flags & VALA_PARSER_MODIFIER_FLAGS_CLASS) == VALA_PARSER_MODIFIER_FLAGS_CLASS;
	} else {
		_tmp18_ = FALSE;
	}
	if (_tmp18_) {
		ValaField* _tmp19_;
		ValaSourceReference* _tmp20_;
		ValaSourceReference* _tmp21_;
		_tmp19_ = f;
		_tmp20_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp19_);
		_tmp21_ = _tmp20_;
		vala_report_error (_tmp21_, "only one of `static' or `class' may be specified");
	} else {
		if ((flags & VALA_PARSER_MODIFIER_FLAGS_STATIC) == VALA_PARSER_MODIFIER_FLAGS_STATIC) {
			ValaField* _tmp22_;
			_tmp22_ = f;
			vala_field_set_binding (_tmp22_, VALA_MEMBER_BINDING_STATIC);
		} else {
			if ((flags & VALA_PARSER_MODIFIER_FLAGS_CLASS) == VALA_PARSER_MODIFIER_FLAGS_CLASS) {
				ValaField* _tmp23_;
				_tmp23_ = f;
				vala_field_set_binding (_tmp23_, VALA_MEMBER_BINDING_CLASS);
			} else {
				if (VALA_IS_NAMESPACE (parent)) {
					ValaField* _tmp24_;
					_tmp24_ = f;
					vala_field_set_binding (_tmp24_, VALA_MEMBER_BINDING_STATIC);
				}
			}
		}
	}
	_tmp28_ = vala_symbol_get_external_package (parent);
	_tmp29_ = _tmp28_;
	if (!_tmp29_) {
		_tmp27_ = VALA_IS_STRUCT (parent);
	} else {
		_tmp27_ = FALSE;
	}
	if (_tmp27_) {
		ValaField* _tmp30_;
		ValaSymbolAccessibility _tmp31_;
		ValaSymbolAccessibility _tmp32_;
		_tmp30_ = f;
		_tmp31_ = vala_symbol_get_access ((ValaSymbol*) _tmp30_);
		_tmp32_ = _tmp31_;
		_tmp26_ = _tmp32_ != VALA_SYMBOL_ACCESSIBILITY_PUBLIC;
	} else {
		_tmp26_ = FALSE;
	}
	if (_tmp26_) {
		ValaField* _tmp33_;
		ValaMemberBinding _tmp34_;
		ValaMemberBinding _tmp35_;
		_tmp33_ = f;
		_tmp34_ = vala_field_get_binding (_tmp33_);
		_tmp35_ = _tmp34_;
		_tmp25_ = _tmp35_ == VALA_MEMBER_BINDING_INSTANCE;
	} else {
		_tmp25_ = FALSE;
	}
	if (_tmp25_) {
		ValaField* _tmp36_;
		ValaSourceReference* _tmp37_;
		ValaSourceReference* _tmp38_;
		_tmp36_ = f;
		_tmp37_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp36_);
		_tmp38_ = _tmp37_;
		vala_report_warning (_tmp38_, "accessibility of struct fields can only be `public`");
	}
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_ABSTRACT) == VALA_PARSER_MODIFIER_FLAGS_ABSTRACT) {
		_tmp40_ = TRUE;
	} else {
		_tmp40_ = (flags & VALA_PARSER_MODIFIER_FLAGS_VIRTUAL) == VALA_PARSER_MODIFIER_FLAGS_VIRTUAL;
	}
	if (_tmp40_) {
		_tmp39_ = TRUE;
	} else {
		_tmp39_ = (flags & VALA_PARSER_MODIFIER_FLAGS_OVERRIDE) == VALA_PARSER_MODIFIER_FLAGS_OVERRIDE;
	}
	if (_tmp39_) {
		ValaField* _tmp41_;
		ValaSourceReference* _tmp42_;
		ValaSourceReference* _tmp43_;
		_tmp41_ = f;
		_tmp42_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp41_);
		_tmp43_ = _tmp42_;
		vala_report_error (_tmp43_, "abstract, virtual, and override modifiers are not applicable to fields");
	}
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_EXTERN) == VALA_PARSER_MODIFIER_FLAGS_EXTERN) {
		ValaField* _tmp44_;
		_tmp44_ = f;
		vala_symbol_set_is_extern ((ValaSymbol*) _tmp44_, TRUE);
	}
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_NEW) == VALA_PARSER_MODIFIER_FLAGS_NEW) {
		ValaField* _tmp45_;
		_tmp45_ = f;
		vala_symbol_set_hides ((ValaSymbol*) _tmp45_, TRUE);
	}
	if (vala_parser_accept (self, VALA_TOKEN_TYPE_ASSIGN)) {
		ValaExpression* _tmp46_ = NULL;
		ValaExpression* _tmp47_;
		ValaField* _tmp48_;
		_tmp47_ = vala_parser_parse_expression (self, &_inner_error0_);
		_tmp46_ = _tmp47_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (f);
				_vala_code_node_unref0 (_tmp4_);
				_g_free0 (id);
				_vala_code_node_unref0 (type);
				return;
			} else {
				_vala_code_node_unref0 (f);
				_vala_code_node_unref0 (_tmp4_);
				_g_free0 (id);
				_vala_code_node_unref0 (type);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
		_tmp48_ = f;
		vala_variable_set_initializer ((ValaVariable*) _tmp48_, _tmp46_);
		_vala_code_node_unref0 (_tmp46_);
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_SEMICOLON, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (f);
			_vala_code_node_unref0 (_tmp4_);
			_g_free0 (id);
			_vala_code_node_unref0 (type);
			return;
		} else {
			_vala_code_node_unref0 (f);
			_vala_code_node_unref0 (_tmp4_);
			_g_free0 (id);
			_vala_code_node_unref0 (type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp49_ = f;
	vala_symbol_add_field (parent, _tmp49_);
	_vala_code_node_unref0 (f);
	_vala_code_node_unref0 (_tmp4_);
	_g_free0 (id);
	_vala_code_node_unref0 (type);
}

static ValaInitializerList*
vala_parser_parse_initializer (ValaParser* self,
                               GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaInitializerList* initializer = NULL;
	ValaSourceLocation _tmp1_;
	ValaSourceReference* _tmp2_;
	ValaSourceReference* _tmp3_;
	ValaInitializerList* _tmp4_;
	ValaInitializerList* _tmp5_;
	ValaInitializerList* _tmp17_;
	ValaSourceLocation _tmp18_;
	ValaSourceReference* _tmp19_;
	ValaSourceReference* _tmp20_;
	GError* _inner_error0_ = NULL;
	ValaInitializerList* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_BRACE, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp1_ = begin;
	_tmp2_ = vala_parser_get_src (self, &_tmp1_);
	_tmp3_ = _tmp2_;
	_tmp4_ = vala_initializer_list_new (_tmp3_);
	_tmp5_ = _tmp4_;
	_vala_source_reference_unref0 (_tmp3_);
	initializer = _tmp5_;
	while (TRUE) {
		if (!(vala_parser_current (self) != VALA_TOKEN_TYPE_CLOSE_BRACE)) {
			break;
		}
		{
			ValaExpression* _tmp6_ = NULL;
			ValaExpression* _tmp7_;
			ValaInitializerList* _tmp8_;
			_tmp7_ = vala_parser_parse_argument (self, &_inner_error0_);
			_tmp6_ = _tmp7_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					goto __catch0_vala_parse_error;
				}
				_vala_code_node_unref0 (initializer);
				g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
			_tmp8_ = initializer;
			vala_initializer_list_append (_tmp8_, _tmp6_);
			_vala_code_node_unref0 (_tmp6_);
		}
		goto __finally0;
		__catch0_vala_parse_error:
		{
			GError* e = NULL;
			e = _inner_error0_;
			_inner_error0_ = NULL;
			if (vala_parser_current (self) == VALA_TOKEN_TYPE_CLOSE_BRACE) {
				GError* _tmp9_;
				GError* _tmp10_;
				vala_parser_prev (self);
				_tmp9_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "incomplete initializer");
				_tmp10_ = _tmp9_;
				vala_parser_report_parse_error (self, _tmp10_);
				_g_error_free0 (_tmp10_);
			} else {
				ValaCodeContext* _tmp11_;
				gboolean _tmp12_;
				gboolean _tmp13_;
				_tmp11_ = self->priv->context;
				_tmp12_ = vala_code_context_get_keep_going (_tmp11_);
				_tmp13_ = _tmp12_;
				if (_tmp13_) {
					GError* _tmp14_;
					_tmp14_ = e;
					vala_parser_report_parse_error (self, _tmp14_);
				} else {
					GError* _tmp15_;
					GError* _tmp16_;
					_tmp15_ = e;
					_tmp16_ = _g_error_copy0 (_tmp15_);
					_inner_error0_ = _tmp16_;
					_g_error_free0 (e);
					goto __finally0;
				}
			}
			_g_error_free0 (e);
		}
		__finally0:
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (initializer);
				return NULL;
			} else {
				_vala_code_node_unref0 (initializer);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		if (!vala_parser_accept (self, VALA_TOKEN_TYPE_COMMA)) {
			break;
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_BRACE, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (initializer);
			return NULL;
		} else {
			_vala_code_node_unref0 (initializer);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp17_ = initializer;
	_tmp18_ = begin;
	_tmp19_ = vala_parser_get_src (self, &_tmp18_);
	_tmp20_ = _tmp19_;
	vala_code_node_set_source_reference ((ValaCodeNode*) _tmp17_, _tmp20_);
	_vala_source_reference_unref0 (_tmp20_);
	result = initializer;
	return result;
}

static void
vala_parser_parse_method_declaration (ValaParser* self,
                                      ValaSymbol* parent,
                                      ValaList* attrs,
                                      GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaSymbolAccessibility access = 0;
	ValaParserModifierFlags flags = 0U;
	ValaDataType* type = NULL;
	ValaDataType* _tmp1_;
	ValaUnresolvedSymbol* sym = NULL;
	ValaUnresolvedSymbol* _tmp2_;
	ValaList* type_param_list = NULL;
	ValaList* _tmp3_;
	ValaMethod* method = NULL;
	ValaUnresolvedSymbol* _tmp4_;
	const gchar* _tmp5_;
	const gchar* _tmp6_;
	ValaDataType* _tmp7_;
	ValaSourceLocation _tmp8_;
	ValaSourceReference* _tmp9_;
	ValaSourceReference* _tmp10_;
	ValaComment* _tmp11_;
	ValaMethod* _tmp12_;
	ValaMethod* _tmp13_;
	ValaUnresolvedSymbol* _tmp14_;
	ValaUnresolvedSymbol* _tmp15_;
	ValaUnresolvedSymbol* _tmp16_;
	ValaMethod* _tmp28_;
	ValaMethod* _tmp29_;
	gboolean _tmp40_ = FALSE;
	ValaMethod* _tmp49_;
	ValaMemberBinding _tmp50_;
	ValaMemberBinding _tmp51_;
	ValaMethod* _tmp103_;
	GError* _inner_error0_ = NULL;
	g_return_if_fail (self != NULL);
	g_return_if_fail (parent != NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	access = vala_parser_parse_access_modifier (self, VALA_SYMBOL_ACCESSIBILITY_PRIVATE);
	flags = vala_parser_parse_member_declaration_modifiers (self);
	_tmp1_ = vala_parser_parse_type (self, TRUE, FALSE, FALSE, &_inner_error0_);
	type = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp2_ = vala_parser_parse_symbol_name (self, &_inner_error0_);
	sym = _tmp2_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (type);
			return;
		} else {
			_vala_code_node_unref0 (type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp3_ = vala_parser_parse_type_parameter_list (self, &_inner_error0_);
	type_param_list = _tmp3_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (sym);
			_vala_code_node_unref0 (type);
			return;
		} else {
			_vala_code_node_unref0 (sym);
			_vala_code_node_unref0 (type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp4_ = sym;
	_tmp5_ = vala_symbol_get_name ((ValaSymbol*) _tmp4_);
	_tmp6_ = _tmp5_;
	_tmp7_ = type;
	_tmp8_ = begin;
	_tmp9_ = vala_parser_get_src (self, &_tmp8_);
	_tmp10_ = _tmp9_;
	_tmp11_ = self->priv->comment;
	_tmp12_ = vala_method_new (_tmp6_, _tmp7_, _tmp10_, _tmp11_);
	_tmp13_ = _tmp12_;
	_vala_source_reference_unref0 (_tmp10_);
	method = _tmp13_;
	_tmp14_ = sym;
	_tmp15_ = vala_unresolved_symbol_get_inner (_tmp14_);
	_tmp16_ = _tmp15_;
	if (_tmp16_ != NULL) {
		ValaMethod* _tmp17_;
		ValaUnresolvedSymbol* _tmp18_;
		ValaUnresolvedSymbol* _tmp19_;
		ValaUnresolvedSymbol* _tmp20_;
		ValaUnresolvedSymbol* _tmp21_;
		ValaUnresolvedSymbol* _tmp22_;
		ValaUnresolvedSymbol* _tmp23_;
		ValaSourceReference* _tmp24_;
		ValaSourceReference* _tmp25_;
		ValaUnresolvedType* _tmp26_;
		ValaUnresolvedType* _tmp27_;
		_tmp17_ = method;
		_tmp18_ = sym;
		_tmp19_ = vala_unresolved_symbol_get_inner (_tmp18_);
		_tmp20_ = _tmp19_;
		_tmp21_ = sym;
		_tmp22_ = vala_unresolved_symbol_get_inner (_tmp21_);
		_tmp23_ = _tmp22_;
		_tmp24_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp23_);
		_tmp25_ = _tmp24_;
		_tmp26_ = vala_unresolved_type_new_from_symbol (_tmp20_, _tmp25_);
		_tmp27_ = _tmp26_;
		vala_method_set_base_interface_type (_tmp17_, (ValaDataType*) _tmp27_);
		_vala_code_node_unref0 (_tmp27_);
	}
	_tmp28_ = method;
	vala_symbol_set_access ((ValaSymbol*) _tmp28_, access);
	_tmp29_ = method;
	vala_parser_set_attributes (self, (ValaCodeNode*) _tmp29_, attrs);
	{
		ValaList* _type_param_list = NULL;
		ValaList* _tmp30_;
		gint _type_param_size = 0;
		ValaList* _tmp31_;
		gint _tmp32_;
		gint _tmp33_;
		gint _type_param_index = 0;
		_tmp30_ = type_param_list;
		_type_param_list = _tmp30_;
		_tmp31_ = _type_param_list;
		_tmp32_ = vala_collection_get_size ((ValaCollection*) _tmp31_);
		_tmp33_ = _tmp32_;
		_type_param_size = _tmp33_;
		_type_param_index = -1;
		while (TRUE) {
			gint _tmp34_;
			gint _tmp35_;
			ValaTypeParameter* type_param = NULL;
			ValaList* _tmp36_;
			gpointer _tmp37_;
			ValaMethod* _tmp38_;
			ValaTypeParameter* _tmp39_;
			_type_param_index = _type_param_index + 1;
			_tmp34_ = _type_param_index;
			_tmp35_ = _type_param_size;
			if (!(_tmp34_ < _tmp35_)) {
				break;
			}
			_tmp36_ = _type_param_list;
			_tmp37_ = vala_list_get (_tmp36_, _type_param_index);
			type_param = (ValaTypeParameter*) _tmp37_;
			_tmp38_ = method;
			_tmp39_ = type_param;
			vala_method_add_type_parameter (_tmp38_, _tmp39_);
			_vala_code_node_unref0 (type_param);
		}
	}
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_STATIC) == VALA_PARSER_MODIFIER_FLAGS_STATIC) {
		_tmp40_ = (flags & VALA_PARSER_MODIFIER_FLAGS_CLASS) == VALA_PARSER_MODIFIER_FLAGS_CLASS;
	} else {
		_tmp40_ = FALSE;
	}
	if (_tmp40_) {
		ValaMethod* _tmp41_;
		ValaSourceReference* _tmp42_;
		ValaSourceReference* _tmp43_;
		_tmp41_ = method;
		_tmp42_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp41_);
		_tmp43_ = _tmp42_;
		vala_report_error (_tmp43_, "only one of `static' or `class' may be specified");
	} else {
		if ((flags & VALA_PARSER_MODIFIER_FLAGS_STATIC) == VALA_PARSER_MODIFIER_FLAGS_STATIC) {
			ValaMethod* _tmp44_;
			_tmp44_ = method;
			vala_method_set_binding (_tmp44_, VALA_MEMBER_BINDING_STATIC);
		} else {
			if ((flags & VALA_PARSER_MODIFIER_FLAGS_CLASS) == VALA_PARSER_MODIFIER_FLAGS_CLASS) {
				ValaMethod* _tmp45_;
				_tmp45_ = method;
				vala_method_set_binding (_tmp45_, VALA_MEMBER_BINDING_CLASS);
			} else {
				if (VALA_IS_NAMESPACE (parent)) {
					ValaMethod* _tmp46_;
					_tmp46_ = method;
					vala_method_set_binding (_tmp46_, VALA_MEMBER_BINDING_STATIC);
				}
			}
		}
	}
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_ASYNC) == VALA_PARSER_MODIFIER_FLAGS_ASYNC) {
		ValaMethod* _tmp47_;
		_tmp47_ = method;
		vala_method_set_coroutine (_tmp47_, TRUE);
	}
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_NEW) == VALA_PARSER_MODIFIER_FLAGS_NEW) {
		ValaMethod* _tmp48_;
		_tmp48_ = method;
		vala_symbol_set_hides ((ValaSymbol*) _tmp48_, TRUE);
	}
	_tmp49_ = method;
	_tmp50_ = vala_method_get_binding (_tmp49_);
	_tmp51_ = _tmp50_;
	if (_tmp51_ == VALA_MEMBER_BINDING_INSTANCE) {
		gboolean _tmp55_ = FALSE;
		gboolean _tmp56_ = FALSE;
		gboolean _tmp57_ = FALSE;
		ValaMethod* _tmp58_;
		gboolean _tmp59_;
		gboolean _tmp60_;
		if ((flags & VALA_PARSER_MODIFIER_FLAGS_ABSTRACT) == VALA_PARSER_MODIFIER_FLAGS_ABSTRACT) {
			ValaMethod* _tmp52_;
			_tmp52_ = method;
			vala_method_set_is_abstract (_tmp52_, TRUE);
		}
		if ((flags & VALA_PARSER_MODIFIER_FLAGS_VIRTUAL) == VALA_PARSER_MODIFIER_FLAGS_VIRTUAL) {
			ValaMethod* _tmp53_;
			_tmp53_ = method;
			vala_method_set_is_virtual (_tmp53_, TRUE);
		}
		if ((flags & VALA_PARSER_MODIFIER_FLAGS_OVERRIDE) == VALA_PARSER_MODIFIER_FLAGS_OVERRIDE) {
			ValaMethod* _tmp54_;
			_tmp54_ = method;
			vala_method_set_overrides (_tmp54_, TRUE);
		}
		_tmp58_ = method;
		_tmp59_ = vala_method_get_is_abstract (_tmp58_);
		_tmp60_ = _tmp59_;
		if (_tmp60_) {
			ValaMethod* _tmp61_;
			gboolean _tmp62_;
			gboolean _tmp63_;
			_tmp61_ = method;
			_tmp62_ = vala_method_get_is_virtual (_tmp61_);
			_tmp63_ = _tmp62_;
			_tmp57_ = _tmp63_;
		} else {
			_tmp57_ = FALSE;
		}
		if (_tmp57_) {
			_tmp56_ = TRUE;
		} else {
			gboolean _tmp64_ = FALSE;
			ValaMethod* _tmp65_;
			gboolean _tmp66_;
			gboolean _tmp67_;
			_tmp65_ = method;
			_tmp66_ = vala_method_get_is_abstract (_tmp65_);
			_tmp67_ = _tmp66_;
			if (_tmp67_) {
				ValaMethod* _tmp68_;
				gboolean _tmp69_;
				gboolean _tmp70_;
				_tmp68_ = method;
				_tmp69_ = vala_method_get_overrides (_tmp68_);
				_tmp70_ = _tmp69_;
				_tmp64_ = _tmp70_;
			} else {
				_tmp64_ = FALSE;
			}
			_tmp56_ = _tmp64_;
		}
		if (_tmp56_) {
			_tmp55_ = TRUE;
		} else {
			gboolean _tmp71_ = FALSE;
			ValaMethod* _tmp72_;
			gboolean _tmp73_;
			gboolean _tmp74_;
			_tmp72_ = method;
			_tmp73_ = vala_method_get_is_virtual (_tmp72_);
			_tmp74_ = _tmp73_;
			if (_tmp74_) {
				ValaMethod* _tmp75_;
				gboolean _tmp76_;
				gboolean _tmp77_;
				_tmp75_ = method;
				_tmp76_ = vala_method_get_overrides (_tmp75_);
				_tmp77_ = _tmp76_;
				_tmp71_ = _tmp77_;
			} else {
				_tmp71_ = FALSE;
			}
			_tmp55_ = _tmp71_;
		}
		if (_tmp55_) {
			GError* _tmp78_;
			_tmp78_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "only one of `abstract', `virtual', or `override' may be specified");
			_inner_error0_ = _tmp78_;
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (method);
				_vala_iterable_unref0 (type_param_list);
				_vala_code_node_unref0 (sym);
				_vala_code_node_unref0 (type);
				return;
			} else {
				_vala_code_node_unref0 (method);
				_vala_iterable_unref0 (type_param_list);
				_vala_code_node_unref0 (sym);
				_vala_code_node_unref0 (type);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
	} else {
		gboolean _tmp79_ = FALSE;
		gboolean _tmp80_ = FALSE;
		if ((flags & VALA_PARSER_MODIFIER_FLAGS_ABSTRACT) == VALA_PARSER_MODIFIER_FLAGS_ABSTRACT) {
			_tmp80_ = TRUE;
		} else {
			_tmp80_ = (flags & VALA_PARSER_MODIFIER_FLAGS_VIRTUAL) == VALA_PARSER_MODIFIER_FLAGS_VIRTUAL;
		}
		if (_tmp80_) {
			_tmp79_ = TRUE;
		} else {
			_tmp79_ = (flags & VALA_PARSER_MODIFIER_FLAGS_OVERRIDE) == VALA_PARSER_MODIFIER_FLAGS_OVERRIDE;
		}
		if (_tmp79_) {
			const gchar* _tmp81_ = NULL;
			GError* _tmp82_;
			if ((flags & VALA_PARSER_MODIFIER_FLAGS_CLASS) == VALA_PARSER_MODIFIER_FLAGS_CLASS) {
				_tmp81_ = "class";
			} else {
				_tmp81_ = "static";
			}
			_tmp82_ = g_error_new (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "the modifiers `abstract', `virtual', and `override' are not valid for " \
"%s methods", _tmp81_);
			_inner_error0_ = _tmp82_;
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (method);
				_vala_iterable_unref0 (type_param_list);
				_vala_code_node_unref0 (sym);
				_vala_code_node_unref0 (type);
				return;
			} else {
				_vala_code_node_unref0 (method);
				_vala_iterable_unref0 (type_param_list);
				_vala_code_node_unref0 (sym);
				_vala_code_node_unref0 (type);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
	}
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_INLINE) == VALA_PARSER_MODIFIER_FLAGS_INLINE) {
		ValaMethod* _tmp83_;
		_tmp83_ = method;
		vala_method_set_is_inline (_tmp83_, TRUE);
	}
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_EXTERN) == VALA_PARSER_MODIFIER_FLAGS_EXTERN) {
		ValaMethod* _tmp84_;
		_tmp84_ = method;
		vala_symbol_set_is_extern ((ValaSymbol*) _tmp84_, TRUE);
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (method);
			_vala_iterable_unref0 (type_param_list);
			_vala_code_node_unref0 (sym);
			_vala_code_node_unref0 (type);
			return;
		} else {
			_vala_code_node_unref0 (method);
			_vala_iterable_unref0 (type_param_list);
			_vala_code_node_unref0 (sym);
			_vala_code_node_unref0 (type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	if (vala_parser_current (self) != VALA_TOKEN_TYPE_CLOSE_PARENS) {
		{
			gboolean _tmp85_ = FALSE;
			_tmp85_ = TRUE;
			while (TRUE) {
				ValaParameter* param = NULL;
				ValaParameter* _tmp86_;
				ValaMethod* _tmp87_;
				ValaParameter* _tmp88_;
				if (!_tmp85_) {
					if (!vala_parser_accept (self, VALA_TOKEN_TYPE_COMMA)) {
						break;
					}
				}
				_tmp85_ = FALSE;
				_tmp86_ = vala_parser_parse_parameter (self, &_inner_error0_);
				param = _tmp86_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_code_node_unref0 (method);
						_vala_iterable_unref0 (type_param_list);
						_vala_code_node_unref0 (sym);
						_vala_code_node_unref0 (type);
						return;
					} else {
						_vala_code_node_unref0 (method);
						_vala_iterable_unref0 (type_param_list);
						_vala_code_node_unref0 (sym);
						_vala_code_node_unref0 (type);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return;
					}
				}
				_tmp87_ = method;
				_tmp88_ = param;
				vala_callable_add_parameter ((ValaCallable*) _tmp87_, _tmp88_);
				_vala_code_node_unref0 (param);
			}
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (method);
			_vala_iterable_unref0 (type_param_list);
			_vala_code_node_unref0 (sym);
			_vala_code_node_unref0 (type);
			return;
		} else {
			_vala_code_node_unref0 (method);
			_vala_iterable_unref0 (type_param_list);
			_vala_code_node_unref0 (sym);
			_vala_code_node_unref0 (type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	if (vala_parser_accept (self, VALA_TOKEN_TYPE_THROWS)) {
		{
			gboolean _tmp89_ = FALSE;
			_tmp89_ = TRUE;
			while (TRUE) {
				ValaDataType* _tmp90_ = NULL;
				ValaDataType* _tmp91_;
				ValaMethod* _tmp92_;
				if (!_tmp89_) {
					if (!vala_parser_accept (self, VALA_TOKEN_TYPE_COMMA)) {
						break;
					}
				}
				_tmp89_ = FALSE;
				_tmp91_ = vala_parser_parse_type (self, TRUE, FALSE, FALSE, &_inner_error0_);
				_tmp90_ = _tmp91_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_code_node_unref0 (method);
						_vala_iterable_unref0 (type_param_list);
						_vala_code_node_unref0 (sym);
						_vala_code_node_unref0 (type);
						return;
					} else {
						_vala_code_node_unref0 (method);
						_vala_iterable_unref0 (type_param_list);
						_vala_code_node_unref0 (sym);
						_vala_code_node_unref0 (type);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return;
					}
				}
				_tmp92_ = method;
				vala_method_add_error_type (_tmp92_, _tmp90_);
				_vala_code_node_unref0 (_tmp90_);
			}
		}
	}
	while (TRUE) {
		ValaExpression* _tmp93_ = NULL;
		ValaExpression* _tmp94_;
		ValaMethod* _tmp95_;
		if (!vala_parser_accept (self, VALA_TOKEN_TYPE_REQUIRES)) {
			break;
		}
		vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_PARENS, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (method);
				_vala_iterable_unref0 (type_param_list);
				_vala_code_node_unref0 (sym);
				_vala_code_node_unref0 (type);
				return;
			} else {
				_vala_code_node_unref0 (method);
				_vala_iterable_unref0 (type_param_list);
				_vala_code_node_unref0 (sym);
				_vala_code_node_unref0 (type);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
		_tmp94_ = vala_parser_parse_expression (self, &_inner_error0_);
		_tmp93_ = _tmp94_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (method);
				_vala_iterable_unref0 (type_param_list);
				_vala_code_node_unref0 (sym);
				_vala_code_node_unref0 (type);
				return;
			} else {
				_vala_code_node_unref0 (method);
				_vala_iterable_unref0 (type_param_list);
				_vala_code_node_unref0 (sym);
				_vala_code_node_unref0 (type);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
		_tmp95_ = method;
		vala_method_add_precondition (_tmp95_, _tmp93_);
		vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_PARENS, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (_tmp93_);
				_vala_code_node_unref0 (method);
				_vala_iterable_unref0 (type_param_list);
				_vala_code_node_unref0 (sym);
				_vala_code_node_unref0 (type);
				return;
			} else {
				_vala_code_node_unref0 (_tmp93_);
				_vala_code_node_unref0 (method);
				_vala_iterable_unref0 (type_param_list);
				_vala_code_node_unref0 (sym);
				_vala_code_node_unref0 (type);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
		_vala_code_node_unref0 (_tmp93_);
	}
	while (TRUE) {
		ValaExpression* _tmp96_ = NULL;
		ValaExpression* _tmp97_;
		ValaMethod* _tmp98_;
		if (!vala_parser_accept (self, VALA_TOKEN_TYPE_ENSURES)) {
			break;
		}
		vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_PARENS, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (method);
				_vala_iterable_unref0 (type_param_list);
				_vala_code_node_unref0 (sym);
				_vala_code_node_unref0 (type);
				return;
			} else {
				_vala_code_node_unref0 (method);
				_vala_iterable_unref0 (type_param_list);
				_vala_code_node_unref0 (sym);
				_vala_code_node_unref0 (type);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
		_tmp97_ = vala_parser_parse_expression (self, &_inner_error0_);
		_tmp96_ = _tmp97_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (method);
				_vala_iterable_unref0 (type_param_list);
				_vala_code_node_unref0 (sym);
				_vala_code_node_unref0 (type);
				return;
			} else {
				_vala_code_node_unref0 (method);
				_vala_iterable_unref0 (type_param_list);
				_vala_code_node_unref0 (sym);
				_vala_code_node_unref0 (type);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
		_tmp98_ = method;
		vala_method_add_postcondition (_tmp98_, _tmp96_);
		vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_PARENS, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (_tmp96_);
				_vala_code_node_unref0 (method);
				_vala_iterable_unref0 (type_param_list);
				_vala_code_node_unref0 (sym);
				_vala_code_node_unref0 (type);
				return;
			} else {
				_vala_code_node_unref0 (_tmp96_);
				_vala_code_node_unref0 (method);
				_vala_iterable_unref0 (type_param_list);
				_vala_code_node_unref0 (sym);
				_vala_code_node_unref0 (type);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
		_vala_code_node_unref0 (_tmp96_);
	}
	if (!vala_parser_accept (self, VALA_TOKEN_TYPE_SEMICOLON)) {
		ValaBlock* _tmp99_ = NULL;
		ValaBlock* _tmp100_;
		ValaMethod* _tmp101_;
		ValaMethod* _tmp102_;
		_tmp100_ = vala_parser_parse_block (self, &_inner_error0_);
		_tmp99_ = _tmp100_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (method);
				_vala_iterable_unref0 (type_param_list);
				_vala_code_node_unref0 (sym);
				_vala_code_node_unref0 (type);
				return;
			} else {
				_vala_code_node_unref0 (method);
				_vala_iterable_unref0 (type_param_list);
				_vala_code_node_unref0 (sym);
				_vala_code_node_unref0 (type);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
		_tmp101_ = method;
		vala_subroutine_set_body ((ValaSubroutine*) _tmp101_, _tmp99_);
		_tmp102_ = method;
		vala_symbol_set_external ((ValaSymbol*) _tmp102_, FALSE);
		_vala_code_node_unref0 (_tmp99_);
	}
	_tmp103_ = method;
	vala_symbol_add_method (parent, _tmp103_);
	_vala_code_node_unref0 (method);
	_vala_iterable_unref0 (type_param_list);
	_vala_code_node_unref0 (sym);
	_vala_code_node_unref0 (type);
}

static void
vala_parser_parse_property_declaration (ValaParser* self,
                                        ValaSymbol* parent,
                                        ValaList* attrs,
                                        GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaSymbolAccessibility access = 0;
	ValaParserModifierFlags flags = 0U;
	ValaDataType* type = NULL;
	ValaDataType* _tmp1_;
	gchar* id = NULL;
	gchar* _tmp2_;
	ValaProperty* prop = NULL;
	const gchar* _tmp3_;
	ValaDataType* _tmp4_;
	ValaSourceLocation _tmp5_;
	ValaSourceReference* _tmp6_;
	ValaSourceReference* _tmp7_;
	ValaComment* _tmp8_;
	ValaProperty* _tmp9_;
	ValaProperty* _tmp10_;
	ValaProperty* _tmp11_;
	ValaProperty* _tmp12_;
	gboolean _tmp13_ = FALSE;
	gboolean _tmp27_ = FALSE;
	gboolean _tmp28_ = FALSE;
	gboolean _tmp29_ = FALSE;
	ValaProperty* _tmp30_;
	gboolean _tmp31_;
	gboolean _tmp32_;
	ValaProperty* _tmp131_;
	GError* _inner_error0_ = NULL;
	g_return_if_fail (self != NULL);
	g_return_if_fail (parent != NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	access = vala_parser_parse_access_modifier (self, VALA_SYMBOL_ACCESSIBILITY_PRIVATE);
	flags = vala_parser_parse_member_declaration_modifiers (self);
	_tmp1_ = vala_parser_parse_type (self, TRUE, TRUE, FALSE, &_inner_error0_);
	type = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp2_ = vala_parser_parse_identifier (self, FALSE, &_inner_error0_);
	id = _tmp2_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (type);
			return;
		} else {
			_vala_code_node_unref0 (type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp3_ = id;
	_tmp4_ = type;
	_tmp5_ = begin;
	_tmp6_ = vala_parser_get_src (self, &_tmp5_);
	_tmp7_ = _tmp6_;
	_tmp8_ = self->priv->comment;
	_tmp9_ = vala_property_new (_tmp3_, _tmp4_, NULL, NULL, _tmp7_, _tmp8_);
	_tmp10_ = _tmp9_;
	_vala_source_reference_unref0 (_tmp7_);
	prop = _tmp10_;
	_tmp11_ = prop;
	vala_symbol_set_access ((ValaSymbol*) _tmp11_, access);
	_tmp12_ = prop;
	vala_parser_set_attributes (self, (ValaCodeNode*) _tmp12_, attrs);
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_STATIC) == VALA_PARSER_MODIFIER_FLAGS_STATIC) {
		_tmp13_ = (flags & VALA_PARSER_MODIFIER_FLAGS_CLASS) == VALA_PARSER_MODIFIER_FLAGS_CLASS;
	} else {
		_tmp13_ = FALSE;
	}
	if (_tmp13_) {
		ValaProperty* _tmp14_;
		ValaSourceReference* _tmp15_;
		ValaSourceReference* _tmp16_;
		_tmp14_ = prop;
		_tmp15_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp14_);
		_tmp16_ = _tmp15_;
		vala_report_error (_tmp16_, "only one of `static' or `class' may be specified");
	} else {
		if ((flags & VALA_PARSER_MODIFIER_FLAGS_STATIC) == VALA_PARSER_MODIFIER_FLAGS_STATIC) {
			ValaProperty* _tmp17_;
			_tmp17_ = prop;
			vala_property_set_binding (_tmp17_, VALA_MEMBER_BINDING_STATIC);
		} else {
			if ((flags & VALA_PARSER_MODIFIER_FLAGS_CLASS) == VALA_PARSER_MODIFIER_FLAGS_CLASS) {
				ValaProperty* _tmp18_;
				_tmp18_ = prop;
				vala_property_set_binding (_tmp18_, VALA_MEMBER_BINDING_CLASS);
			}
		}
	}
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_ABSTRACT) == VALA_PARSER_MODIFIER_FLAGS_ABSTRACT) {
		ValaProperty* _tmp19_;
		_tmp19_ = prop;
		vala_property_set_is_abstract (_tmp19_, TRUE);
	}
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_VIRTUAL) == VALA_PARSER_MODIFIER_FLAGS_VIRTUAL) {
		ValaProperty* _tmp20_;
		_tmp20_ = prop;
		vala_property_set_is_virtual (_tmp20_, TRUE);
	}
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_OVERRIDE) == VALA_PARSER_MODIFIER_FLAGS_OVERRIDE) {
		ValaProperty* _tmp21_;
		_tmp21_ = prop;
		vala_property_set_overrides (_tmp21_, TRUE);
	}
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_NEW) == VALA_PARSER_MODIFIER_FLAGS_NEW) {
		ValaProperty* _tmp22_;
		_tmp22_ = prop;
		vala_symbol_set_hides ((ValaSymbol*) _tmp22_, TRUE);
	}
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_ASYNC) == VALA_PARSER_MODIFIER_FLAGS_ASYNC) {
		ValaProperty* _tmp23_;
		ValaSourceReference* _tmp24_;
		ValaSourceReference* _tmp25_;
		_tmp23_ = prop;
		_tmp24_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp23_);
		_tmp25_ = _tmp24_;
		vala_report_error (_tmp25_, "async properties are not supported yet");
	}
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_EXTERN) == VALA_PARSER_MODIFIER_FLAGS_EXTERN) {
		ValaProperty* _tmp26_;
		_tmp26_ = prop;
		vala_symbol_set_is_extern ((ValaSymbol*) _tmp26_, TRUE);
	}
	_tmp30_ = prop;
	_tmp31_ = vala_property_get_is_abstract (_tmp30_);
	_tmp32_ = _tmp31_;
	if (_tmp32_) {
		ValaProperty* _tmp33_;
		gboolean _tmp34_;
		gboolean _tmp35_;
		_tmp33_ = prop;
		_tmp34_ = vala_property_get_is_virtual (_tmp33_);
		_tmp35_ = _tmp34_;
		_tmp29_ = _tmp35_;
	} else {
		_tmp29_ = FALSE;
	}
	if (_tmp29_) {
		_tmp28_ = TRUE;
	} else {
		gboolean _tmp36_ = FALSE;
		ValaProperty* _tmp37_;
		gboolean _tmp38_;
		gboolean _tmp39_;
		_tmp37_ = prop;
		_tmp38_ = vala_property_get_is_abstract (_tmp37_);
		_tmp39_ = _tmp38_;
		if (_tmp39_) {
			ValaProperty* _tmp40_;
			gboolean _tmp41_;
			gboolean _tmp42_;
			_tmp40_ = prop;
			_tmp41_ = vala_property_get_overrides (_tmp40_);
			_tmp42_ = _tmp41_;
			_tmp36_ = _tmp42_;
		} else {
			_tmp36_ = FALSE;
		}
		_tmp28_ = _tmp36_;
	}
	if (_tmp28_) {
		_tmp27_ = TRUE;
	} else {
		gboolean _tmp43_ = FALSE;
		ValaProperty* _tmp44_;
		gboolean _tmp45_;
		gboolean _tmp46_;
		_tmp44_ = prop;
		_tmp45_ = vala_property_get_is_virtual (_tmp44_);
		_tmp46_ = _tmp45_;
		if (_tmp46_) {
			ValaProperty* _tmp47_;
			gboolean _tmp48_;
			gboolean _tmp49_;
			_tmp47_ = prop;
			_tmp48_ = vala_property_get_overrides (_tmp47_);
			_tmp49_ = _tmp48_;
			_tmp43_ = _tmp49_;
		} else {
			_tmp43_ = FALSE;
		}
		_tmp27_ = _tmp43_;
	}
	if (_tmp27_) {
		GError* _tmp50_;
		_tmp50_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "only one of `abstract', `virtual', or `override' may be specified");
		_inner_error0_ = _tmp50_;
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (prop);
			_g_free0 (id);
			_vala_code_node_unref0 (type);
			return;
		} else {
			_vala_code_node_unref0 (prop);
			_g_free0 (id);
			_vala_code_node_unref0 (type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	if (vala_parser_accept (self, VALA_TOKEN_TYPE_THROWS)) {
		ValaProperty* _tmp54_;
		ValaSourceReference* _tmp55_;
		ValaSourceReference* _tmp56_;
		{
			gboolean _tmp51_ = FALSE;
			_tmp51_ = TRUE;
			while (TRUE) {
				ValaDataType* _tmp52_;
				ValaDataType* _tmp53_;
				if (!_tmp51_) {
					if (!vala_parser_accept (self, VALA_TOKEN_TYPE_COMMA)) {
						break;
					}
				}
				_tmp51_ = FALSE;
				_tmp52_ = vala_parser_parse_type (self, TRUE, FALSE, FALSE, &_inner_error0_);
				_tmp53_ = _tmp52_;
				_vala_code_node_unref0 (_tmp53_);
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_code_node_unref0 (prop);
						_g_free0 (id);
						_vala_code_node_unref0 (type);
						return;
					} else {
						_vala_code_node_unref0 (prop);
						_g_free0 (id);
						_vala_code_node_unref0 (type);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return;
					}
				}
			}
		}
		_tmp54_ = prop;
		_tmp55_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp54_);
		_tmp56_ = _tmp55_;
		vala_report_error (_tmp56_, "properties throwing errors are not supported yet");
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_BRACE, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (prop);
			_g_free0 (id);
			_vala_code_node_unref0 (type);
			return;
		} else {
			_vala_code_node_unref0 (prop);
			_g_free0 (id);
			_vala_code_node_unref0 (type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	while (TRUE) {
		if (!(vala_parser_current (self) != VALA_TOKEN_TYPE_CLOSE_BRACE)) {
			break;
		}
		if (vala_parser_accept (self, VALA_TOKEN_TYPE_DEFAULT)) {
			ValaProperty* _tmp57_;
			ValaExpression* _tmp58_;
			ValaExpression* _tmp59_;
			ValaExpression* _tmp61_ = NULL;
			ValaExpression* _tmp62_;
			ValaProperty* _tmp63_;
			_tmp57_ = prop;
			_tmp58_ = vala_property_get_initializer (_tmp57_);
			_tmp59_ = _tmp58_;
			if (_tmp59_ != NULL) {
				GError* _tmp60_;
				_tmp60_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "property default value already defined");
				_inner_error0_ = _tmp60_;
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (prop);
					_g_free0 (id);
					_vala_code_node_unref0 (type);
					return;
				} else {
					_vala_code_node_unref0 (prop);
					_g_free0 (id);
					_vala_code_node_unref0 (type);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return;
				}
			}
			vala_parser_expect (self, VALA_TOKEN_TYPE_ASSIGN, &_inner_error0_);
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (prop);
					_g_free0 (id);
					_vala_code_node_unref0 (type);
					return;
				} else {
					_vala_code_node_unref0 (prop);
					_g_free0 (id);
					_vala_code_node_unref0 (type);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return;
				}
			}
			_tmp62_ = vala_parser_parse_expression (self, &_inner_error0_);
			_tmp61_ = _tmp62_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (prop);
					_g_free0 (id);
					_vala_code_node_unref0 (type);
					return;
				} else {
					_vala_code_node_unref0 (prop);
					_g_free0 (id);
					_vala_code_node_unref0 (type);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return;
				}
			}
			_tmp63_ = prop;
			vala_property_set_initializer (_tmp63_, _tmp61_);
			vala_parser_expect (self, VALA_TOKEN_TYPE_SEMICOLON, &_inner_error0_);
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (_tmp61_);
					_vala_code_node_unref0 (prop);
					_g_free0 (id);
					_vala_code_node_unref0 (type);
					return;
				} else {
					_vala_code_node_unref0 (_tmp61_);
					_vala_code_node_unref0 (prop);
					_g_free0 (id);
					_vala_code_node_unref0 (type);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return;
				}
			}
			_vala_code_node_unref0 (_tmp61_);
		} else {
			ValaScanner* _tmp64_;
			ValaComment* _tmp65_;
			ValaSourceLocation accessor_begin = {0};
			ValaSourceLocation _tmp66_ = {0};
			ValaList* accessor_attrs = NULL;
			ValaList* _tmp67_;
			ValaSymbolAccessibility accessor_access = 0;
			ValaDataType* value_type = NULL;
			ValaDataType* _tmp68_;
			ValaDataType* _tmp69_;
			_tmp64_ = self->priv->scanner;
			_tmp65_ = vala_scanner_pop_comment (_tmp64_);
			_vala_comment_unref0 (self->priv->comment);
			self->priv->comment = _tmp65_;
			vala_parser_get_location (self, &_tmp66_);
			accessor_begin = _tmp66_;
			_tmp67_ = vala_parser_parse_attributes (self, &_inner_error0_);
			accessor_attrs = _tmp67_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (prop);
					_g_free0 (id);
					_vala_code_node_unref0 (type);
					return;
				} else {
					_vala_code_node_unref0 (prop);
					_g_free0 (id);
					_vala_code_node_unref0 (type);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return;
				}
			}
			accessor_access = vala_parser_parse_access_modifier (self, VALA_SYMBOL_ACCESSIBILITY_PUBLIC);
			_tmp68_ = type;
			_tmp69_ = vala_data_type_copy (_tmp68_);
			value_type = _tmp69_;
			if (vala_parser_accept (self, VALA_TOKEN_TYPE_OWNED)) {
				ValaDataType* _tmp70_;
				_tmp70_ = value_type;
				vala_data_type_set_value_owned (_tmp70_, TRUE);
			} else {
				ValaDataType* _tmp71_;
				_tmp71_ = value_type;
				vala_data_type_set_value_owned (_tmp71_, FALSE);
				if (vala_parser_accept (self, VALA_TOKEN_TYPE_UNOWNED)) {
					ValaSourceReference* _tmp72_;
					ValaSourceReference* _tmp73_;
					_tmp72_ = vala_parser_get_last_src (self);
					_tmp73_ = _tmp72_;
					vala_report_warning (_tmp73_, "property getters are `unowned' by default");
					_vala_source_reference_unref0 (_tmp73_);
				}
			}
			if (vala_parser_accept (self, VALA_TOKEN_TYPE_GET)) {
				ValaProperty* _tmp74_;
				ValaPropertyAccessor* _tmp75_;
				ValaPropertyAccessor* _tmp76_;
				ValaBlock* block = NULL;
				ValaProperty* _tmp82_;
				ValaDataType* _tmp83_;
				ValaBlock* _tmp84_;
				ValaSourceLocation _tmp85_;
				ValaSourceReference* _tmp86_;
				ValaSourceReference* _tmp87_;
				ValaComment* _tmp88_;
				ValaPropertyAccessor* _tmp89_;
				ValaPropertyAccessor* _tmp90_;
				ValaProperty* _tmp91_;
				ValaPropertyAccessor* _tmp92_;
				ValaPropertyAccessor* _tmp93_;
				ValaList* _tmp94_;
				ValaProperty* _tmp95_;
				ValaPropertyAccessor* _tmp96_;
				ValaPropertyAccessor* _tmp97_;
				_tmp74_ = prop;
				_tmp75_ = vala_property_get_get_accessor (_tmp74_);
				_tmp76_ = _tmp75_;
				if (_tmp76_ != NULL) {
					GError* _tmp77_;
					_tmp77_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "property get accessor already defined");
					_inner_error0_ = _tmp77_;
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_code_node_unref0 (value_type);
						_vala_iterable_unref0 (accessor_attrs);
						_vala_code_node_unref0 (prop);
						_g_free0 (id);
						_vala_code_node_unref0 (type);
						return;
					} else {
						_vala_code_node_unref0 (value_type);
						_vala_iterable_unref0 (accessor_attrs);
						_vala_code_node_unref0 (prop);
						_g_free0 (id);
						_vala_code_node_unref0 (type);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return;
					}
				}
				block = NULL;
				if (!vala_parser_accept (self, VALA_TOKEN_TYPE_SEMICOLON)) {
					ValaBlock* _tmp78_ = NULL;
					ValaBlock* _tmp79_;
					ValaBlock* _tmp80_;
					ValaProperty* _tmp81_;
					_tmp79_ = vala_parser_parse_block (self, &_inner_error0_);
					_tmp78_ = _tmp79_;
					if (G_UNLIKELY (_inner_error0_ != NULL)) {
						if (_inner_error0_->domain == VALA_PARSE_ERROR) {
							g_propagate_error (error, _inner_error0_);
							_vala_code_node_unref0 (block);
							_vala_code_node_unref0 (value_type);
							_vala_iterable_unref0 (accessor_attrs);
							_vala_code_node_unref0 (prop);
							_g_free0 (id);
							_vala_code_node_unref0 (type);
							return;
						} else {
							_vala_code_node_unref0 (block);
							_vala_code_node_unref0 (value_type);
							_vala_iterable_unref0 (accessor_attrs);
							_vala_code_node_unref0 (prop);
							_g_free0 (id);
							_vala_code_node_unref0 (type);
							g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
							g_clear_error (&_inner_error0_);
							return;
						}
					}
					_tmp80_ = _tmp78_;
					_tmp78_ = NULL;
					_vala_code_node_unref0 (block);
					block = _tmp80_;
					_tmp81_ = prop;
					vala_symbol_set_external ((ValaSymbol*) _tmp81_, FALSE);
					_vala_code_node_unref0 (_tmp78_);
				}
				_tmp82_ = prop;
				_tmp83_ = value_type;
				_tmp84_ = block;
				_tmp85_ = accessor_begin;
				_tmp86_ = vala_parser_get_src (self, &_tmp85_);
				_tmp87_ = _tmp86_;
				_tmp88_ = self->priv->comment;
				_tmp89_ = vala_property_accessor_new (TRUE, FALSE, FALSE, _tmp83_, _tmp84_, _tmp87_, _tmp88_);
				_tmp90_ = _tmp89_;
				vala_property_set_get_accessor (_tmp82_, _tmp90_);
				_vala_code_node_unref0 (_tmp90_);
				_vala_source_reference_unref0 (_tmp87_);
				_tmp91_ = prop;
				_tmp92_ = vala_property_get_get_accessor (_tmp91_);
				_tmp93_ = _tmp92_;
				_tmp94_ = accessor_attrs;
				vala_parser_set_attributes (self, (ValaCodeNode*) _tmp93_, _tmp94_);
				_tmp95_ = prop;
				_tmp96_ = vala_property_get_get_accessor (_tmp95_);
				_tmp97_ = _tmp96_;
				vala_symbol_set_access ((ValaSymbol*) _tmp97_, accessor_access);
				_vala_code_node_unref0 (block);
			} else {
				gboolean writable = FALSE;
				gboolean _construct = FALSE;
				ValaProperty* _tmp107_;
				ValaPropertyAccessor* _tmp108_;
				ValaPropertyAccessor* _tmp109_;
				ValaBlock* block = NULL;
				ValaProperty* _tmp115_;
				ValaDataType* _tmp116_;
				ValaBlock* _tmp117_;
				ValaSourceLocation _tmp118_;
				ValaSourceReference* _tmp119_;
				ValaSourceReference* _tmp120_;
				ValaComment* _tmp121_;
				ValaPropertyAccessor* _tmp122_;
				ValaPropertyAccessor* _tmp123_;
				ValaProperty* _tmp124_;
				ValaPropertyAccessor* _tmp125_;
				ValaPropertyAccessor* _tmp126_;
				ValaList* _tmp127_;
				ValaProperty* _tmp128_;
				ValaPropertyAccessor* _tmp129_;
				ValaPropertyAccessor* _tmp130_;
				if (vala_parser_accept (self, VALA_TOKEN_TYPE_SET)) {
					gboolean _tmp98_ = FALSE;
					ValaCodeContext* _tmp99_;
					ValaProfile _tmp100_;
					ValaProfile _tmp101_;
					writable = TRUE;
					_tmp99_ = self->priv->context;
					_tmp100_ = vala_code_context_get_profile (_tmp99_);
					_tmp101_ = _tmp100_;
					if (_tmp101_ == VALA_PROFILE_GOBJECT) {
						_tmp98_ = vala_parser_accept (self, VALA_TOKEN_TYPE_CONSTRUCT);
					} else {
						_tmp98_ = FALSE;
					}
					_construct = _tmp98_;
				} else {
					gboolean _tmp102_ = FALSE;
					ValaCodeContext* _tmp103_;
					ValaProfile _tmp104_;
					ValaProfile _tmp105_;
					_tmp103_ = self->priv->context;
					_tmp104_ = vala_code_context_get_profile (_tmp103_);
					_tmp105_ = _tmp104_;
					if (_tmp105_ == VALA_PROFILE_GOBJECT) {
						_tmp102_ = vala_parser_accept (self, VALA_TOKEN_TYPE_CONSTRUCT);
					} else {
						_tmp102_ = FALSE;
					}
					if (_tmp102_) {
						_construct = TRUE;
						writable = vala_parser_accept (self, VALA_TOKEN_TYPE_SET);
					} else {
						GError* _tmp106_;
						_tmp106_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "expected get, set, or construct");
						_inner_error0_ = _tmp106_;
						if (_inner_error0_->domain == VALA_PARSE_ERROR) {
							g_propagate_error (error, _inner_error0_);
							_vala_code_node_unref0 (value_type);
							_vala_iterable_unref0 (accessor_attrs);
							_vala_code_node_unref0 (prop);
							_g_free0 (id);
							_vala_code_node_unref0 (type);
							return;
						} else {
							_vala_code_node_unref0 (value_type);
							_vala_iterable_unref0 (accessor_attrs);
							_vala_code_node_unref0 (prop);
							_g_free0 (id);
							_vala_code_node_unref0 (type);
							g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
							g_clear_error (&_inner_error0_);
							return;
						}
					}
				}
				_tmp107_ = prop;
				_tmp108_ = vala_property_get_set_accessor (_tmp107_);
				_tmp109_ = _tmp108_;
				if (_tmp109_ != NULL) {
					GError* _tmp110_;
					_tmp110_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "property set accessor already defined");
					_inner_error0_ = _tmp110_;
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_code_node_unref0 (value_type);
						_vala_iterable_unref0 (accessor_attrs);
						_vala_code_node_unref0 (prop);
						_g_free0 (id);
						_vala_code_node_unref0 (type);
						return;
					} else {
						_vala_code_node_unref0 (value_type);
						_vala_iterable_unref0 (accessor_attrs);
						_vala_code_node_unref0 (prop);
						_g_free0 (id);
						_vala_code_node_unref0 (type);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return;
					}
				}
				block = NULL;
				if (!vala_parser_accept (self, VALA_TOKEN_TYPE_SEMICOLON)) {
					ValaBlock* _tmp111_ = NULL;
					ValaBlock* _tmp112_;
					ValaBlock* _tmp113_;
					ValaProperty* _tmp114_;
					_tmp112_ = vala_parser_parse_block (self, &_inner_error0_);
					_tmp111_ = _tmp112_;
					if (G_UNLIKELY (_inner_error0_ != NULL)) {
						if (_inner_error0_->domain == VALA_PARSE_ERROR) {
							g_propagate_error (error, _inner_error0_);
							_vala_code_node_unref0 (block);
							_vala_code_node_unref0 (value_type);
							_vala_iterable_unref0 (accessor_attrs);
							_vala_code_node_unref0 (prop);
							_g_free0 (id);
							_vala_code_node_unref0 (type);
							return;
						} else {
							_vala_code_node_unref0 (block);
							_vala_code_node_unref0 (value_type);
							_vala_iterable_unref0 (accessor_attrs);
							_vala_code_node_unref0 (prop);
							_g_free0 (id);
							_vala_code_node_unref0 (type);
							g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
							g_clear_error (&_inner_error0_);
							return;
						}
					}
					_tmp113_ = _tmp111_;
					_tmp111_ = NULL;
					_vala_code_node_unref0 (block);
					block = _tmp113_;
					_tmp114_ = prop;
					vala_symbol_set_external ((ValaSymbol*) _tmp114_, FALSE);
					_vala_code_node_unref0 (_tmp111_);
				}
				_tmp115_ = prop;
				_tmp116_ = value_type;
				_tmp117_ = block;
				_tmp118_ = accessor_begin;
				_tmp119_ = vala_parser_get_src (self, &_tmp118_);
				_tmp120_ = _tmp119_;
				_tmp121_ = self->priv->comment;
				_tmp122_ = vala_property_accessor_new (FALSE, writable, _construct, _tmp116_, _tmp117_, _tmp120_, _tmp121_);
				_tmp123_ = _tmp122_;
				vala_property_set_set_accessor (_tmp115_, _tmp123_);
				_vala_code_node_unref0 (_tmp123_);
				_vala_source_reference_unref0 (_tmp120_);
				_tmp124_ = prop;
				_tmp125_ = vala_property_get_set_accessor (_tmp124_);
				_tmp126_ = _tmp125_;
				_tmp127_ = accessor_attrs;
				vala_parser_set_attributes (self, (ValaCodeNode*) _tmp126_, _tmp127_);
				_tmp128_ = prop;
				_tmp129_ = vala_property_get_set_accessor (_tmp128_);
				_tmp130_ = _tmp129_;
				vala_symbol_set_access ((ValaSymbol*) _tmp130_, accessor_access);
				_vala_code_node_unref0 (block);
			}
			_vala_code_node_unref0 (value_type);
			_vala_iterable_unref0 (accessor_attrs);
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_BRACE, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (prop);
			_g_free0 (id);
			_vala_code_node_unref0 (type);
			return;
		} else {
			_vala_code_node_unref0 (prop);
			_g_free0 (id);
			_vala_code_node_unref0 (type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp131_ = prop;
	vala_symbol_add_property (parent, _tmp131_);
	_vala_code_node_unref0 (prop);
	_g_free0 (id);
	_vala_code_node_unref0 (type);
}

static void
vala_parser_parse_signal_declaration (ValaParser* self,
                                      ValaSymbol* parent,
                                      ValaList* attrs,
                                      GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaSymbolAccessibility access = 0;
	ValaParserModifierFlags flags = 0U;
	ValaDataType* type = NULL;
	ValaDataType* _tmp1_;
	gchar* id = NULL;
	gchar* _tmp2_;
	ValaSignal* sig = NULL;
	const gchar* _tmp3_;
	ValaDataType* _tmp4_;
	ValaSourceLocation _tmp5_;
	ValaSourceReference* _tmp6_;
	ValaSourceReference* _tmp7_;
	ValaComment* _tmp8_;
	ValaSignal* _tmp9_;
	ValaSignal* _tmp10_;
	ValaSignal* _tmp11_;
	ValaSignal* _tmp12_;
	ValaSignal* _tmp24_;
	GError* _inner_error0_ = NULL;
	g_return_if_fail (self != NULL);
	g_return_if_fail (parent != NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	access = vala_parser_parse_access_modifier (self, VALA_SYMBOL_ACCESSIBILITY_PRIVATE);
	flags = vala_parser_parse_member_declaration_modifiers (self);
	vala_parser_expect (self, VALA_TOKEN_TYPE_SIGNAL, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp1_ = vala_parser_parse_type (self, TRUE, FALSE, FALSE, &_inner_error0_);
	type = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp2_ = vala_parser_parse_identifier (self, FALSE, &_inner_error0_);
	id = _tmp2_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (type);
			return;
		} else {
			_vala_code_node_unref0 (type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp3_ = id;
	_tmp4_ = type;
	_tmp5_ = begin;
	_tmp6_ = vala_parser_get_src (self, &_tmp5_);
	_tmp7_ = _tmp6_;
	_tmp8_ = self->priv->comment;
	_tmp9_ = vala_signal_new (_tmp3_, _tmp4_, _tmp7_, _tmp8_);
	_tmp10_ = _tmp9_;
	_vala_source_reference_unref0 (_tmp7_);
	sig = _tmp10_;
	_tmp11_ = sig;
	vala_symbol_set_access ((ValaSymbol*) _tmp11_, access);
	_tmp12_ = sig;
	vala_parser_set_attributes (self, (ValaCodeNode*) _tmp12_, attrs);
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_STATIC) == VALA_PARSER_MODIFIER_FLAGS_STATIC) {
		GError* _tmp13_;
		_tmp13_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "`static' modifier not allowed on signals");
		_inner_error0_ = _tmp13_;
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (sig);
			_g_free0 (id);
			_vala_code_node_unref0 (type);
			return;
		} else {
			_vala_code_node_unref0 (sig);
			_g_free0 (id);
			_vala_code_node_unref0 (type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	} else {
		if ((flags & VALA_PARSER_MODIFIER_FLAGS_CLASS) == VALA_PARSER_MODIFIER_FLAGS_CLASS) {
			GError* _tmp14_;
			_tmp14_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "`class' modifier not allowed on signals");
			_inner_error0_ = _tmp14_;
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (sig);
				_g_free0 (id);
				_vala_code_node_unref0 (type);
				return;
			} else {
				_vala_code_node_unref0 (sig);
				_g_free0 (id);
				_vala_code_node_unref0 (type);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
	}
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_VIRTUAL) == VALA_PARSER_MODIFIER_FLAGS_VIRTUAL) {
		ValaSignal* _tmp15_;
		_tmp15_ = sig;
		vala_signal_set_is_virtual (_tmp15_, TRUE);
	}
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_NEW) == VALA_PARSER_MODIFIER_FLAGS_NEW) {
		ValaSignal* _tmp16_;
		_tmp16_ = sig;
		vala_symbol_set_hides ((ValaSymbol*) _tmp16_, TRUE);
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (sig);
			_g_free0 (id);
			_vala_code_node_unref0 (type);
			return;
		} else {
			_vala_code_node_unref0 (sig);
			_g_free0 (id);
			_vala_code_node_unref0 (type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	if (vala_parser_current (self) != VALA_TOKEN_TYPE_CLOSE_PARENS) {
		{
			gboolean _tmp17_ = FALSE;
			_tmp17_ = TRUE;
			while (TRUE) {
				ValaParameter* param = NULL;
				ValaParameter* _tmp18_;
				ValaSignal* _tmp19_;
				ValaParameter* _tmp20_;
				if (!_tmp17_) {
					if (!vala_parser_accept (self, VALA_TOKEN_TYPE_COMMA)) {
						break;
					}
				}
				_tmp17_ = FALSE;
				_tmp18_ = vala_parser_parse_parameter (self, &_inner_error0_);
				param = _tmp18_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_code_node_unref0 (sig);
						_g_free0 (id);
						_vala_code_node_unref0 (type);
						return;
					} else {
						_vala_code_node_unref0 (sig);
						_g_free0 (id);
						_vala_code_node_unref0 (type);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return;
					}
				}
				_tmp19_ = sig;
				_tmp20_ = param;
				vala_callable_add_parameter ((ValaCallable*) _tmp19_, _tmp20_);
				_vala_code_node_unref0 (param);
			}
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (sig);
			_g_free0 (id);
			_vala_code_node_unref0 (type);
			return;
		} else {
			_vala_code_node_unref0 (sig);
			_g_free0 (id);
			_vala_code_node_unref0 (type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	if (!vala_parser_accept (self, VALA_TOKEN_TYPE_SEMICOLON)) {
		ValaBlock* _tmp21_ = NULL;
		ValaBlock* _tmp22_;
		ValaSignal* _tmp23_;
		_tmp22_ = vala_parser_parse_block (self, &_inner_error0_);
		_tmp21_ = _tmp22_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (sig);
				_g_free0 (id);
				_vala_code_node_unref0 (type);
				return;
			} else {
				_vala_code_node_unref0 (sig);
				_g_free0 (id);
				_vala_code_node_unref0 (type);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
		_tmp23_ = sig;
		vala_signal_set_body (_tmp23_, _tmp21_);
		_vala_code_node_unref0 (_tmp21_);
	}
	_tmp24_ = sig;
	vala_symbol_add_signal (parent, _tmp24_);
	_vala_code_node_unref0 (sig);
	_g_free0 (id);
	_vala_code_node_unref0 (type);
}

static void
vala_parser_parse_constructor_declaration (ValaParser* self,
                                           ValaSymbol* parent,
                                           ValaList* attrs,
                                           GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaParserModifierFlags flags = 0U;
	ValaConstructor* c = NULL;
	ValaSourceLocation _tmp2_;
	ValaSourceReference* _tmp3_;
	ValaSourceReference* _tmp4_;
	ValaConstructor* _tmp5_;
	ValaConstructor* _tmp6_;
	gboolean _tmp7_ = FALSE;
	ValaBlock* _tmp13_ = NULL;
	ValaBlock* _tmp14_;
	ValaConstructor* _tmp15_;
	ValaConstructor* _tmp16_;
	GError* _inner_error0_ = NULL;
	g_return_if_fail (self != NULL);
	g_return_if_fail (parent != NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	flags = vala_parser_parse_member_declaration_modifiers (self);
	vala_parser_expect (self, VALA_TOKEN_TYPE_CONSTRUCT, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_NEW) == VALA_PARSER_MODIFIER_FLAGS_NEW) {
		GError* _tmp1_;
		_tmp1_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "`new' modifier not allowed on constructor");
		_inner_error0_ = _tmp1_;
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp2_ = begin;
	_tmp3_ = vala_parser_get_src (self, &_tmp2_);
	_tmp4_ = _tmp3_;
	_tmp5_ = vala_constructor_new (_tmp4_);
	_tmp6_ = _tmp5_;
	_vala_source_reference_unref0 (_tmp4_);
	c = _tmp6_;
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_STATIC) == VALA_PARSER_MODIFIER_FLAGS_STATIC) {
		_tmp7_ = (flags & VALA_PARSER_MODIFIER_FLAGS_CLASS) == VALA_PARSER_MODIFIER_FLAGS_CLASS;
	} else {
		_tmp7_ = FALSE;
	}
	if (_tmp7_) {
		ValaConstructor* _tmp8_;
		ValaSourceReference* _tmp9_;
		ValaSourceReference* _tmp10_;
		_tmp8_ = c;
		_tmp9_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp8_);
		_tmp10_ = _tmp9_;
		vala_report_error (_tmp10_, "only one of `static' or `class' may be specified");
	} else {
		if ((flags & VALA_PARSER_MODIFIER_FLAGS_STATIC) == VALA_PARSER_MODIFIER_FLAGS_STATIC) {
			ValaConstructor* _tmp11_;
			_tmp11_ = c;
			vala_constructor_set_binding (_tmp11_, VALA_MEMBER_BINDING_STATIC);
		} else {
			if ((flags & VALA_PARSER_MODIFIER_FLAGS_CLASS) == VALA_PARSER_MODIFIER_FLAGS_CLASS) {
				ValaConstructor* _tmp12_;
				_tmp12_ = c;
				vala_constructor_set_binding (_tmp12_, VALA_MEMBER_BINDING_CLASS);
			}
		}
	}
	_tmp14_ = vala_parser_parse_block (self, &_inner_error0_);
	_tmp13_ = _tmp14_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (c);
			return;
		} else {
			_vala_code_node_unref0 (c);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp15_ = c;
	vala_subroutine_set_body ((ValaSubroutine*) _tmp15_, _tmp13_);
	_tmp16_ = c;
	vala_symbol_add_constructor (parent, _tmp16_);
	_vala_code_node_unref0 (_tmp13_);
	_vala_code_node_unref0 (c);
}

static void
vala_parser_parse_destructor_declaration (ValaParser* self,
                                          ValaSymbol* parent,
                                          ValaList* attrs,
                                          GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaParserModifierFlags flags = 0U;
	gchar* identifier = NULL;
	gchar* _tmp1_;
	ValaDestructor* d = NULL;
	ValaSourceLocation _tmp3_;
	ValaSourceReference* _tmp4_;
	ValaSourceReference* _tmp5_;
	ValaDestructor* _tmp6_;
	ValaDestructor* _tmp7_;
	const gchar* _tmp8_;
	const gchar* _tmp9_;
	const gchar* _tmp10_;
	gboolean _tmp14_ = FALSE;
	ValaBlock* _tmp20_ = NULL;
	ValaBlock* _tmp21_;
	ValaDestructor* _tmp22_;
	ValaDestructor* _tmp23_;
	GError* _inner_error0_ = NULL;
	g_return_if_fail (self != NULL);
	g_return_if_fail (parent != NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	flags = vala_parser_parse_member_declaration_modifiers (self);
	vala_parser_expect (self, VALA_TOKEN_TYPE_TILDE, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp1_ = vala_parser_parse_identifier (self, FALSE, &_inner_error0_);
	identifier = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_g_free0 (identifier);
			return;
		} else {
			_g_free0 (identifier);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_g_free0 (identifier);
			return;
		} else {
			_g_free0 (identifier);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_NEW) == VALA_PARSER_MODIFIER_FLAGS_NEW) {
		GError* _tmp2_;
		_tmp2_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "`new' modifier not allowed on destructor");
		_inner_error0_ = _tmp2_;
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_g_free0 (identifier);
			return;
		} else {
			_g_free0 (identifier);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp3_ = begin;
	_tmp4_ = vala_parser_get_src (self, &_tmp3_);
	_tmp5_ = _tmp4_;
	_tmp6_ = vala_destructor_new (_tmp5_);
	_tmp7_ = _tmp6_;
	_vala_source_reference_unref0 (_tmp5_);
	d = _tmp7_;
	_tmp8_ = identifier;
	_tmp9_ = vala_symbol_get_name (parent);
	_tmp10_ = _tmp9_;
	if (g_strcmp0 (_tmp8_, _tmp10_) != 0) {
		ValaDestructor* _tmp11_;
		ValaSourceReference* _tmp12_;
		ValaSourceReference* _tmp13_;
		_tmp11_ = d;
		_tmp12_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp11_);
		_tmp13_ = _tmp12_;
		vala_report_error (_tmp13_, "destructor and parent symbol name do not match");
	}
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_STATIC) == VALA_PARSER_MODIFIER_FLAGS_STATIC) {
		_tmp14_ = (flags & VALA_PARSER_MODIFIER_FLAGS_CLASS) == VALA_PARSER_MODIFIER_FLAGS_CLASS;
	} else {
		_tmp14_ = FALSE;
	}
	if (_tmp14_) {
		ValaDestructor* _tmp15_;
		ValaSourceReference* _tmp16_;
		ValaSourceReference* _tmp17_;
		_tmp15_ = d;
		_tmp16_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp15_);
		_tmp17_ = _tmp16_;
		vala_report_error (_tmp17_, "only one of `static' or `class' may be specified");
	} else {
		if ((flags & VALA_PARSER_MODIFIER_FLAGS_STATIC) == VALA_PARSER_MODIFIER_FLAGS_STATIC) {
			ValaDestructor* _tmp18_;
			_tmp18_ = d;
			vala_destructor_set_binding (_tmp18_, VALA_MEMBER_BINDING_STATIC);
		} else {
			if ((flags & VALA_PARSER_MODIFIER_FLAGS_CLASS) == VALA_PARSER_MODIFIER_FLAGS_CLASS) {
				ValaDestructor* _tmp19_;
				_tmp19_ = d;
				vala_destructor_set_binding (_tmp19_, VALA_MEMBER_BINDING_CLASS);
			}
		}
	}
	_tmp21_ = vala_parser_parse_block (self, &_inner_error0_);
	_tmp20_ = _tmp21_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (d);
			_g_free0 (identifier);
			return;
		} else {
			_vala_code_node_unref0 (d);
			_g_free0 (identifier);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp22_ = d;
	vala_subroutine_set_body ((ValaSubroutine*) _tmp22_, _tmp20_);
	_tmp23_ = d;
	vala_symbol_add_destructor (parent, _tmp23_);
	_vala_code_node_unref0 (_tmp20_);
	_vala_code_node_unref0 (d);
	_g_free0 (identifier);
}

static void
vala_parser_parse_struct_declaration (ValaParser* self,
                                      ValaSymbol* parent,
                                      ValaList* attrs,
                                      GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaSymbolAccessibility access = 0;
	ValaParserModifierFlags flags = 0U;
	ValaUnresolvedSymbol* sym = NULL;
	ValaUnresolvedSymbol* _tmp1_;
	ValaList* type_param_list = NULL;
	ValaList* _tmp2_;
	ValaDataType* base_type = NULL;
	ValaStruct* st = NULL;
	ValaUnresolvedSymbol* _tmp6_;
	const gchar* _tmp7_;
	const gchar* _tmp8_;
	ValaSourceLocation _tmp9_;
	ValaSourceReference* _tmp10_;
	ValaSourceReference* _tmp11_;
	ValaComment* _tmp12_;
	ValaStruct* _tmp13_;
	ValaStruct* _tmp14_;
	ValaStruct* _tmp15_;
	ValaStruct* _tmp17_;
	ValaDataType* _tmp28_;
	ValaStruct* _tmp31_;
	ValaSymbol* _result_ = NULL;
	ValaStruct* _tmp32_;
	ValaSymbol* _tmp33_;
	GError* _inner_error0_ = NULL;
	g_return_if_fail (self != NULL);
	g_return_if_fail (parent != NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	access = vala_parser_parse_access_modifier (self, VALA_SYMBOL_ACCESSIBILITY_PRIVATE);
	flags = vala_parser_parse_type_declaration_modifiers (self);
	vala_parser_expect (self, VALA_TOKEN_TYPE_STRUCT, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp1_ = vala_parser_parse_symbol_name (self, &_inner_error0_);
	sym = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp2_ = vala_parser_parse_type_parameter_list (self, &_inner_error0_);
	type_param_list = _tmp2_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (sym);
			return;
		} else {
			_vala_code_node_unref0 (sym);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	base_type = NULL;
	if (vala_parser_accept (self, VALA_TOKEN_TYPE_COLON)) {
		ValaDataType* _tmp3_ = NULL;
		ValaDataType* _tmp4_;
		ValaDataType* _tmp5_;
		_tmp4_ = vala_parser_parse_type (self, TRUE, FALSE, FALSE, &_inner_error0_);
		_tmp3_ = _tmp4_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (base_type);
				_vala_iterable_unref0 (type_param_list);
				_vala_code_node_unref0 (sym);
				return;
			} else {
				_vala_code_node_unref0 (base_type);
				_vala_iterable_unref0 (type_param_list);
				_vala_code_node_unref0 (sym);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
		_tmp5_ = _tmp3_;
		_tmp3_ = NULL;
		_vala_code_node_unref0 (base_type);
		base_type = _tmp5_;
		_vala_code_node_unref0 (_tmp3_);
	}
	_tmp6_ = sym;
	_tmp7_ = vala_symbol_get_name ((ValaSymbol*) _tmp6_);
	_tmp8_ = _tmp7_;
	_tmp9_ = begin;
	_tmp10_ = vala_parser_get_src (self, &_tmp9_);
	_tmp11_ = _tmp10_;
	_tmp12_ = self->priv->comment;
	_tmp13_ = vala_struct_new (_tmp8_, _tmp11_, _tmp12_);
	_tmp14_ = _tmp13_;
	_vala_source_reference_unref0 (_tmp11_);
	st = _tmp14_;
	_tmp15_ = st;
	vala_symbol_set_access ((ValaSymbol*) _tmp15_, access);
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_EXTERN) == VALA_PARSER_MODIFIER_FLAGS_EXTERN) {
		ValaStruct* _tmp16_;
		_tmp16_ = st;
		vala_symbol_set_is_extern ((ValaSymbol*) _tmp16_, TRUE);
	}
	_tmp17_ = st;
	vala_parser_set_attributes (self, (ValaCodeNode*) _tmp17_, attrs);
	{
		ValaList* _type_param_list = NULL;
		ValaList* _tmp18_;
		gint _type_param_size = 0;
		ValaList* _tmp19_;
		gint _tmp20_;
		gint _tmp21_;
		gint _type_param_index = 0;
		_tmp18_ = type_param_list;
		_type_param_list = _tmp18_;
		_tmp19_ = _type_param_list;
		_tmp20_ = vala_collection_get_size ((ValaCollection*) _tmp19_);
		_tmp21_ = _tmp20_;
		_type_param_size = _tmp21_;
		_type_param_index = -1;
		while (TRUE) {
			gint _tmp22_;
			gint _tmp23_;
			ValaTypeParameter* type_param = NULL;
			ValaList* _tmp24_;
			gpointer _tmp25_;
			ValaStruct* _tmp26_;
			ValaTypeParameter* _tmp27_;
			_type_param_index = _type_param_index + 1;
			_tmp22_ = _type_param_index;
			_tmp23_ = _type_param_size;
			if (!(_tmp22_ < _tmp23_)) {
				break;
			}
			_tmp24_ = _type_param_list;
			_tmp25_ = vala_list_get (_tmp24_, _type_param_index);
			type_param = (ValaTypeParameter*) _tmp25_;
			_tmp26_ = st;
			_tmp27_ = type_param;
			vala_struct_add_type_parameter (_tmp26_, _tmp27_);
			_vala_code_node_unref0 (type_param);
		}
	}
	_tmp28_ = base_type;
	if (_tmp28_ != NULL) {
		ValaStruct* _tmp29_;
		ValaDataType* _tmp30_;
		_tmp29_ = st;
		_tmp30_ = base_type;
		vala_struct_set_base_type (_tmp29_, _tmp30_);
	}
	_tmp31_ = st;
	vala_parser_parse_declarations (self, (ValaSymbol*) _tmp31_, FALSE, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (st);
			_vala_code_node_unref0 (base_type);
			_vala_iterable_unref0 (type_param_list);
			_vala_code_node_unref0 (sym);
			return;
		} else {
			_vala_code_node_unref0 (st);
			_vala_code_node_unref0 (base_type);
			_vala_iterable_unref0 (type_param_list);
			_vala_code_node_unref0 (sym);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp32_ = st;
	_tmp33_ = _vala_code_node_ref0 ((ValaSymbol*) _tmp32_);
	_result_ = _tmp33_;
	while (TRUE) {
		ValaUnresolvedSymbol* _tmp34_;
		ValaUnresolvedSymbol* _tmp35_;
		ValaUnresolvedSymbol* _tmp36_;
		ValaUnresolvedSymbol* _tmp37_;
		ValaUnresolvedSymbol* _tmp38_;
		ValaSymbol* _tmp39_ = NULL;
		ValaUnresolvedSymbol* _tmp40_;
		ValaSymbol* next = NULL;
		ValaSymbol* _tmp49_;
		ValaSymbol* _tmp50_;
		ValaSymbol* _tmp55_;
		ValaSymbol* _tmp56_;
		_tmp34_ = sym;
		if (!(_tmp34_ != NULL)) {
			break;
		}
		_tmp35_ = sym;
		_tmp36_ = vala_unresolved_symbol_get_inner (_tmp35_);
		_tmp37_ = _tmp36_;
		_tmp38_ = _vala_code_node_ref0 (_tmp37_);
		_vala_code_node_unref0 (sym);
		sym = _tmp38_;
		_tmp40_ = sym;
		if (_tmp40_ != NULL) {
			ValaUnresolvedSymbol* _tmp41_;
			const gchar* _tmp42_;
			const gchar* _tmp43_;
			ValaStruct* _tmp44_;
			ValaSourceReference* _tmp45_;
			ValaSourceReference* _tmp46_;
			ValaNamespace* _tmp47_;
			_tmp41_ = sym;
			_tmp42_ = vala_symbol_get_name ((ValaSymbol*) _tmp41_);
			_tmp43_ = _tmp42_;
			_tmp44_ = st;
			_tmp45_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp44_);
			_tmp46_ = _tmp45_;
			_tmp47_ = vala_namespace_new (_tmp43_, _tmp46_);
			_vala_code_node_unref0 (_tmp39_);
			_tmp39_ = (ValaSymbol*) _tmp47_;
		} else {
			ValaSymbol* _tmp48_;
			_tmp48_ = _vala_code_node_ref0 (parent);
			_vala_code_node_unref0 (_tmp39_);
			_tmp39_ = _tmp48_;
		}
		_tmp49_ = _vala_code_node_ref0 (_tmp39_);
		next = _tmp49_;
		_tmp50_ = _result_;
		if (VALA_IS_NAMESPACE (_tmp50_)) {
			ValaSymbol* _tmp51_;
			ValaSymbol* _tmp52_;
			_tmp51_ = next;
			_tmp52_ = _result_;
			vala_symbol_add_namespace (_tmp51_, G_TYPE_CHECK_INSTANCE_CAST (_tmp52_, VALA_TYPE_NAMESPACE, ValaNamespace));
		} else {
			ValaSymbol* _tmp53_;
			ValaSymbol* _tmp54_;
			_tmp53_ = next;
			_tmp54_ = _result_;
			vala_symbol_add_struct (_tmp53_, G_TYPE_CHECK_INSTANCE_CAST (_tmp54_, VALA_TYPE_STRUCT, ValaStruct));
		}
		_tmp55_ = next;
		_tmp56_ = _vala_code_node_ref0 (_tmp55_);
		_vala_code_node_unref0 (_result_);
		_result_ = _tmp56_;
		_vala_code_node_unref0 (next);
		_vala_code_node_unref0 (_tmp39_);
	}
	_vala_code_node_unref0 (_result_);
	_vala_code_node_unref0 (st);
	_vala_code_node_unref0 (base_type);
	_vala_iterable_unref0 (type_param_list);
	_vala_code_node_unref0 (sym);
}

static void
vala_parser_parse_interface_declaration (ValaParser* self,
                                         ValaSymbol* parent,
                                         ValaList* attrs,
                                         GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaSymbolAccessibility access = 0;
	ValaParserModifierFlags flags = 0U;
	ValaUnresolvedSymbol* sym = NULL;
	ValaUnresolvedSymbol* _tmp1_;
	ValaList* type_param_list = NULL;
	ValaList* _tmp2_;
	ValaArrayList* base_types = NULL;
	GEqualFunc _tmp3_;
	ValaArrayList* _tmp4_;
	ValaInterface* iface = NULL;
	ValaUnresolvedSymbol* _tmp9_;
	const gchar* _tmp10_;
	const gchar* _tmp11_;
	ValaSourceLocation _tmp12_;
	ValaSourceReference* _tmp13_;
	ValaSourceReference* _tmp14_;
	ValaComment* _tmp15_;
	ValaInterface* _tmp16_;
	ValaInterface* _tmp17_;
	ValaInterface* _tmp18_;
	ValaInterface* _tmp20_;
	ValaInterface* _tmp41_;
	ValaSymbol* _result_ = NULL;
	ValaInterface* _tmp42_;
	ValaSymbol* _tmp43_;
	GError* _inner_error0_ = NULL;
	g_return_if_fail (self != NULL);
	g_return_if_fail (parent != NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	access = vala_parser_parse_access_modifier (self, VALA_SYMBOL_ACCESSIBILITY_PRIVATE);
	flags = vala_parser_parse_type_declaration_modifiers (self);
	vala_parser_expect (self, VALA_TOKEN_TYPE_INTERFACE, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp1_ = vala_parser_parse_symbol_name (self, &_inner_error0_);
	sym = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp2_ = vala_parser_parse_type_parameter_list (self, &_inner_error0_);
	type_param_list = _tmp2_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (sym);
			return;
		} else {
			_vala_code_node_unref0 (sym);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp3_ = g_direct_equal;
	_tmp4_ = vala_array_list_new (VALA_TYPE_DATA_TYPE, (GBoxedCopyFunc) vala_code_node_ref, (GDestroyNotify) vala_code_node_unref, _tmp3_);
	base_types = _tmp4_;
	if (vala_parser_accept (self, VALA_TOKEN_TYPE_COLON)) {
		{
			gboolean _tmp5_ = FALSE;
			_tmp5_ = TRUE;
			while (TRUE) {
				ValaDataType* type = NULL;
				ValaDataType* _tmp6_;
				ValaArrayList* _tmp7_;
				ValaDataType* _tmp8_;
				if (!_tmp5_) {
					if (!vala_parser_accept (self, VALA_TOKEN_TYPE_COMMA)) {
						break;
					}
				}
				_tmp5_ = FALSE;
				_tmp6_ = vala_parser_parse_type (self, TRUE, FALSE, FALSE, &_inner_error0_);
				type = _tmp6_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_iterable_unref0 (base_types);
						_vala_iterable_unref0 (type_param_list);
						_vala_code_node_unref0 (sym);
						return;
					} else {
						_vala_iterable_unref0 (base_types);
						_vala_iterable_unref0 (type_param_list);
						_vala_code_node_unref0 (sym);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return;
					}
				}
				_tmp7_ = base_types;
				_tmp8_ = type;
				vala_collection_add ((ValaCollection*) _tmp7_, _tmp8_);
				_vala_code_node_unref0 (type);
			}
		}
	}
	_tmp9_ = sym;
	_tmp10_ = vala_symbol_get_name ((ValaSymbol*) _tmp9_);
	_tmp11_ = _tmp10_;
	_tmp12_ = begin;
	_tmp13_ = vala_parser_get_src (self, &_tmp12_);
	_tmp14_ = _tmp13_;
	_tmp15_ = self->priv->comment;
	_tmp16_ = vala_interface_new (_tmp11_, _tmp14_, _tmp15_);
	_tmp17_ = _tmp16_;
	_vala_source_reference_unref0 (_tmp14_);
	iface = _tmp17_;
	_tmp18_ = iface;
	vala_symbol_set_access ((ValaSymbol*) _tmp18_, access);
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_EXTERN) == VALA_PARSER_MODIFIER_FLAGS_EXTERN) {
		ValaInterface* _tmp19_;
		_tmp19_ = iface;
		vala_symbol_set_is_extern ((ValaSymbol*) _tmp19_, TRUE);
	}
	_tmp20_ = iface;
	vala_parser_set_attributes (self, (ValaCodeNode*) _tmp20_, attrs);
	{
		ValaList* _type_param_list = NULL;
		ValaList* _tmp21_;
		gint _type_param_size = 0;
		ValaList* _tmp22_;
		gint _tmp23_;
		gint _tmp24_;
		gint _type_param_index = 0;
		_tmp21_ = type_param_list;
		_type_param_list = _tmp21_;
		_tmp22_ = _type_param_list;
		_tmp23_ = vala_collection_get_size ((ValaCollection*) _tmp22_);
		_tmp24_ = _tmp23_;
		_type_param_size = _tmp24_;
		_type_param_index = -1;
		while (TRUE) {
			gint _tmp25_;
			gint _tmp26_;
			ValaTypeParameter* type_param = NULL;
			ValaList* _tmp27_;
			gpointer _tmp28_;
			ValaInterface* _tmp29_;
			ValaTypeParameter* _tmp30_;
			_type_param_index = _type_param_index + 1;
			_tmp25_ = _type_param_index;
			_tmp26_ = _type_param_size;
			if (!(_tmp25_ < _tmp26_)) {
				break;
			}
			_tmp27_ = _type_param_list;
			_tmp28_ = vala_list_get (_tmp27_, _type_param_index);
			type_param = (ValaTypeParameter*) _tmp28_;
			_tmp29_ = iface;
			_tmp30_ = type_param;
			vala_object_type_symbol_add_type_parameter ((ValaObjectTypeSymbol*) _tmp29_, _tmp30_);
			_vala_code_node_unref0 (type_param);
		}
	}
	{
		ValaArrayList* _base_type_list = NULL;
		ValaArrayList* _tmp31_;
		gint _base_type_size = 0;
		ValaArrayList* _tmp32_;
		gint _tmp33_;
		gint _tmp34_;
		gint _base_type_index = 0;
		_tmp31_ = base_types;
		_base_type_list = _tmp31_;
		_tmp32_ = _base_type_list;
		_tmp33_ = vala_collection_get_size ((ValaCollection*) _tmp32_);
		_tmp34_ = _tmp33_;
		_base_type_size = _tmp34_;
		_base_type_index = -1;
		while (TRUE) {
			gint _tmp35_;
			gint _tmp36_;
			ValaDataType* base_type = NULL;
			ValaArrayList* _tmp37_;
			gpointer _tmp38_;
			ValaInterface* _tmp39_;
			ValaDataType* _tmp40_;
			_base_type_index = _base_type_index + 1;
			_tmp35_ = _base_type_index;
			_tmp36_ = _base_type_size;
			if (!(_tmp35_ < _tmp36_)) {
				break;
			}
			_tmp37_ = _base_type_list;
			_tmp38_ = vala_list_get ((ValaList*) _tmp37_, _base_type_index);
			base_type = (ValaDataType*) _tmp38_;
			_tmp39_ = iface;
			_tmp40_ = base_type;
			vala_interface_add_prerequisite (_tmp39_, _tmp40_);
			_vala_code_node_unref0 (base_type);
		}
	}
	_tmp41_ = iface;
	vala_parser_parse_declarations (self, (ValaSymbol*) _tmp41_, FALSE, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (iface);
			_vala_iterable_unref0 (base_types);
			_vala_iterable_unref0 (type_param_list);
			_vala_code_node_unref0 (sym);
			return;
		} else {
			_vala_code_node_unref0 (iface);
			_vala_iterable_unref0 (base_types);
			_vala_iterable_unref0 (type_param_list);
			_vala_code_node_unref0 (sym);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp42_ = iface;
	_tmp43_ = _vala_code_node_ref0 ((ValaSymbol*) _tmp42_);
	_result_ = _tmp43_;
	while (TRUE) {
		ValaUnresolvedSymbol* _tmp44_;
		ValaUnresolvedSymbol* _tmp45_;
		ValaUnresolvedSymbol* _tmp46_;
		ValaUnresolvedSymbol* _tmp47_;
		ValaUnresolvedSymbol* _tmp48_;
		ValaSymbol* _tmp49_ = NULL;
		ValaUnresolvedSymbol* _tmp50_;
		ValaSymbol* next = NULL;
		ValaSymbol* _tmp59_;
		ValaSymbol* _tmp60_;
		ValaSymbol* _tmp65_;
		ValaSymbol* _tmp66_;
		_tmp44_ = sym;
		if (!(_tmp44_ != NULL)) {
			break;
		}
		_tmp45_ = sym;
		_tmp46_ = vala_unresolved_symbol_get_inner (_tmp45_);
		_tmp47_ = _tmp46_;
		_tmp48_ = _vala_code_node_ref0 (_tmp47_);
		_vala_code_node_unref0 (sym);
		sym = _tmp48_;
		_tmp50_ = sym;
		if (_tmp50_ != NULL) {
			ValaUnresolvedSymbol* _tmp51_;
			const gchar* _tmp52_;
			const gchar* _tmp53_;
			ValaInterface* _tmp54_;
			ValaSourceReference* _tmp55_;
			ValaSourceReference* _tmp56_;
			ValaNamespace* _tmp57_;
			_tmp51_ = sym;
			_tmp52_ = vala_symbol_get_name ((ValaSymbol*) _tmp51_);
			_tmp53_ = _tmp52_;
			_tmp54_ = iface;
			_tmp55_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp54_);
			_tmp56_ = _tmp55_;
			_tmp57_ = vala_namespace_new (_tmp53_, _tmp56_);
			_vala_code_node_unref0 (_tmp49_);
			_tmp49_ = (ValaSymbol*) _tmp57_;
		} else {
			ValaSymbol* _tmp58_;
			_tmp58_ = _vala_code_node_ref0 (parent);
			_vala_code_node_unref0 (_tmp49_);
			_tmp49_ = _tmp58_;
		}
		_tmp59_ = _vala_code_node_ref0 (_tmp49_);
		next = _tmp59_;
		_tmp60_ = _result_;
		if (VALA_IS_NAMESPACE (_tmp60_)) {
			ValaSymbol* _tmp61_;
			ValaSymbol* _tmp62_;
			_tmp61_ = next;
			_tmp62_ = _result_;
			vala_symbol_add_namespace (_tmp61_, G_TYPE_CHECK_INSTANCE_CAST (_tmp62_, VALA_TYPE_NAMESPACE, ValaNamespace));
		} else {
			ValaSymbol* _tmp63_;
			ValaSymbol* _tmp64_;
			_tmp63_ = next;
			_tmp64_ = _result_;
			vala_symbol_add_interface (_tmp63_, G_TYPE_CHECK_INSTANCE_CAST (_tmp64_, VALA_TYPE_INTERFACE, ValaInterface));
		}
		_tmp65_ = next;
		_tmp66_ = _vala_code_node_ref0 (_tmp65_);
		_vala_code_node_unref0 (_result_);
		_result_ = _tmp66_;
		_vala_code_node_unref0 (next);
		_vala_code_node_unref0 (_tmp49_);
	}
	_vala_code_node_unref0 (_result_);
	_vala_code_node_unref0 (iface);
	_vala_iterable_unref0 (base_types);
	_vala_iterable_unref0 (type_param_list);
	_vala_code_node_unref0 (sym);
}

static void
vala_parser_parse_enum_declaration (ValaParser* self,
                                    ValaSymbol* parent,
                                    ValaList* attrs,
                                    GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaSymbolAccessibility access = 0;
	ValaParserModifierFlags flags = 0U;
	ValaUnresolvedSymbol* sym = NULL;
	ValaUnresolvedSymbol* _tmp1_;
	ValaEnum* en = NULL;
	ValaUnresolvedSymbol* _tmp2_;
	const gchar* _tmp3_;
	const gchar* _tmp4_;
	ValaSourceLocation _tmp5_;
	ValaSourceReference* _tmp6_;
	ValaSourceReference* _tmp7_;
	ValaComment* _tmp8_;
	ValaEnum* _tmp9_;
	ValaEnum* _tmp10_;
	ValaEnum* _tmp11_;
	ValaEnum* _tmp13_;
	ValaSourceLocation inner_begin = {0};
	ValaSourceLocation _tmp14_ = {0};
	ValaSymbol* _result_ = NULL;
	ValaEnum* _tmp41_;
	ValaSymbol* _tmp42_;
	GError* _inner_error0_ = NULL;
	g_return_if_fail (self != NULL);
	g_return_if_fail (parent != NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	access = vala_parser_parse_access_modifier (self, VALA_SYMBOL_ACCESSIBILITY_PRIVATE);
	flags = vala_parser_parse_type_declaration_modifiers (self);
	vala_parser_expect (self, VALA_TOKEN_TYPE_ENUM, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp1_ = vala_parser_parse_symbol_name (self, &_inner_error0_);
	sym = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp2_ = sym;
	_tmp3_ = vala_symbol_get_name ((ValaSymbol*) _tmp2_);
	_tmp4_ = _tmp3_;
	_tmp5_ = begin;
	_tmp6_ = vala_parser_get_src (self, &_tmp5_);
	_tmp7_ = _tmp6_;
	_tmp8_ = self->priv->comment;
	_tmp9_ = vala_enum_new (_tmp4_, _tmp7_, _tmp8_);
	_tmp10_ = _tmp9_;
	_vala_source_reference_unref0 (_tmp7_);
	en = _tmp10_;
	_tmp11_ = en;
	vala_symbol_set_access ((ValaSymbol*) _tmp11_, access);
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_EXTERN) == VALA_PARSER_MODIFIER_FLAGS_EXTERN) {
		ValaEnum* _tmp12_;
		_tmp12_ = en;
		vala_symbol_set_is_extern ((ValaSymbol*) _tmp12_, TRUE);
	}
	_tmp13_ = en;
	vala_parser_set_attributes (self, (ValaCodeNode*) _tmp13_, attrs);
	vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_BRACE, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (en);
			_vala_code_node_unref0 (sym);
			return;
		} else {
			_vala_code_node_unref0 (en);
			_vala_code_node_unref0 (sym);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	vala_parser_get_location (self, &_tmp14_);
	inner_begin = _tmp14_;
	{
		while (TRUE) {
			ValaEnum* _tmp15_;
			if (!(vala_parser_current (self) != VALA_TOKEN_TYPE_CLOSE_BRACE)) {
				break;
			}
			_tmp15_ = en;
			vala_parser_parse_declaration (self, (ValaSymbol*) _tmp15_, FALSE, &_inner_error0_);
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					goto __catch0_vala_parse_error;
				}
				_vala_code_node_unref0 (en);
				_vala_code_node_unref0 (sym);
				g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
	}
	goto __finally0;
	__catch0_vala_parse_error:
	{
		ValaSourceLocation _tmp16_;
		g_clear_error (&_inner_error0_);
		_tmp16_ = inner_begin;
		vala_parser_rollback (self, &_tmp16_);
	}
	__finally0:
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (en);
			_vala_code_node_unref0 (sym);
			return;
		} else {
			_vala_code_node_unref0 (en);
			_vala_code_node_unref0 (sym);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	{
		gboolean _tmp17_ = FALSE;
		_tmp17_ = TRUE;
		while (TRUE) {
			ValaList* value_attrs = NULL;
			ValaList* _tmp18_;
			ValaSourceLocation value_begin = {0};
			ValaSourceLocation _tmp19_ = {0};
			gchar* id = NULL;
			gchar* _tmp20_;
			ValaComment* _tmp21_;
			ValaExpression* value = NULL;
			ValaEnumValue* ev = NULL;
			const gchar* _tmp27_;
			ValaExpression* _tmp28_;
			ValaSourceLocation _tmp29_;
			ValaSourceReference* _tmp30_;
			ValaSourceReference* _tmp31_;
			ValaComment* _tmp32_;
			ValaEnumValue* _tmp33_;
			ValaEnumValue* _tmp34_;
			ValaEnumValue* _tmp35_;
			ValaEnumValue* _tmp36_;
			ValaList* _tmp37_;
			ValaEnum* _tmp38_;
			ValaEnumValue* _tmp39_;
			if (!_tmp17_) {
				if (!vala_parser_accept (self, VALA_TOKEN_TYPE_COMMA)) {
					break;
				}
			}
			_tmp17_ = FALSE;
			if (vala_parser_current (self) == VALA_TOKEN_TYPE_CLOSE_BRACE) {
				break;
			}
			_tmp18_ = vala_parser_parse_attributes (self, &_inner_error0_);
			value_attrs = _tmp18_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (en);
					_vala_code_node_unref0 (sym);
					return;
				} else {
					_vala_code_node_unref0 (en);
					_vala_code_node_unref0 (sym);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return;
				}
			}
			vala_parser_get_location (self, &_tmp19_);
			value_begin = _tmp19_;
			_tmp20_ = vala_parser_parse_identifier (self, FALSE, &_inner_error0_);
			id = _tmp20_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_iterable_unref0 (value_attrs);
					_vala_code_node_unref0 (en);
					_vala_code_node_unref0 (sym);
					return;
				} else {
					_vala_iterable_unref0 (value_attrs);
					_vala_code_node_unref0 (en);
					_vala_code_node_unref0 (sym);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return;
				}
			}
			_tmp21_ = self->priv->comment;
			if (_tmp21_ == NULL) {
				ValaScanner* _tmp22_;
				ValaComment* _tmp23_;
				_tmp22_ = self->priv->scanner;
				_tmp23_ = vala_scanner_pop_comment (_tmp22_);
				_vala_comment_unref0 (self->priv->comment);
				self->priv->comment = _tmp23_;
			}
			value = NULL;
			if (vala_parser_accept (self, VALA_TOKEN_TYPE_ASSIGN)) {
				ValaExpression* _tmp24_ = NULL;
				ValaExpression* _tmp25_;
				ValaExpression* _tmp26_;
				_tmp25_ = vala_parser_parse_expression (self, &_inner_error0_);
				_tmp24_ = _tmp25_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_code_node_unref0 (value);
						_g_free0 (id);
						_vala_iterable_unref0 (value_attrs);
						_vala_code_node_unref0 (en);
						_vala_code_node_unref0 (sym);
						return;
					} else {
						_vala_code_node_unref0 (value);
						_g_free0 (id);
						_vala_iterable_unref0 (value_attrs);
						_vala_code_node_unref0 (en);
						_vala_code_node_unref0 (sym);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return;
					}
				}
				_tmp26_ = _tmp24_;
				_tmp24_ = NULL;
				_vala_code_node_unref0 (value);
				value = _tmp26_;
				_vala_code_node_unref0 (_tmp24_);
			}
			_tmp27_ = id;
			_tmp28_ = value;
			_tmp29_ = value_begin;
			_tmp30_ = vala_parser_get_src (self, &_tmp29_);
			_tmp31_ = _tmp30_;
			_tmp32_ = self->priv->comment;
			_tmp33_ = vala_enum_value_new (_tmp27_, _tmp28_, _tmp31_, _tmp32_);
			_tmp34_ = _tmp33_;
			_vala_source_reference_unref0 (_tmp31_);
			ev = _tmp34_;
			_tmp35_ = ev;
			vala_symbol_set_access ((ValaSymbol*) _tmp35_, VALA_SYMBOL_ACCESSIBILITY_PUBLIC);
			_tmp36_ = ev;
			_tmp37_ = value_attrs;
			vala_parser_set_attributes (self, (ValaCodeNode*) _tmp36_, _tmp37_);
			_tmp38_ = en;
			_tmp39_ = ev;
			vala_enum_add_value (_tmp38_, _tmp39_);
			_vala_comment_unref0 (self->priv->comment);
			self->priv->comment = NULL;
			_vala_code_node_unref0 (ev);
			_vala_code_node_unref0 (value);
			_g_free0 (id);
			_vala_iterable_unref0 (value_attrs);
		}
	}
	if (vala_parser_accept (self, VALA_TOKEN_TYPE_SEMICOLON)) {
		while (TRUE) {
			ValaEnum* _tmp40_;
			if (!(vala_parser_current (self) != VALA_TOKEN_TYPE_CLOSE_BRACE)) {
				break;
			}
			_tmp40_ = en;
			vala_parser_parse_declaration (self, (ValaSymbol*) _tmp40_, FALSE, &_inner_error0_);
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (en);
					_vala_code_node_unref0 (sym);
					return;
				} else {
					_vala_code_node_unref0 (en);
					_vala_code_node_unref0 (sym);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return;
				}
			}
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_BRACE, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (en);
			_vala_code_node_unref0 (sym);
			return;
		} else {
			_vala_code_node_unref0 (en);
			_vala_code_node_unref0 (sym);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp41_ = en;
	_tmp42_ = _vala_code_node_ref0 ((ValaSymbol*) _tmp41_);
	_result_ = _tmp42_;
	while (TRUE) {
		ValaUnresolvedSymbol* _tmp43_;
		ValaUnresolvedSymbol* _tmp44_;
		ValaUnresolvedSymbol* _tmp45_;
		ValaUnresolvedSymbol* _tmp46_;
		ValaUnresolvedSymbol* _tmp47_;
		ValaSymbol* _tmp48_ = NULL;
		ValaUnresolvedSymbol* _tmp49_;
		ValaSymbol* next = NULL;
		ValaSymbol* _tmp58_;
		ValaSymbol* _tmp59_;
		ValaSymbol* _tmp64_;
		ValaSymbol* _tmp65_;
		_tmp43_ = sym;
		if (!(_tmp43_ != NULL)) {
			break;
		}
		_tmp44_ = sym;
		_tmp45_ = vala_unresolved_symbol_get_inner (_tmp44_);
		_tmp46_ = _tmp45_;
		_tmp47_ = _vala_code_node_ref0 (_tmp46_);
		_vala_code_node_unref0 (sym);
		sym = _tmp47_;
		_tmp49_ = sym;
		if (_tmp49_ != NULL) {
			ValaUnresolvedSymbol* _tmp50_;
			const gchar* _tmp51_;
			const gchar* _tmp52_;
			ValaEnum* _tmp53_;
			ValaSourceReference* _tmp54_;
			ValaSourceReference* _tmp55_;
			ValaNamespace* _tmp56_;
			_tmp50_ = sym;
			_tmp51_ = vala_symbol_get_name ((ValaSymbol*) _tmp50_);
			_tmp52_ = _tmp51_;
			_tmp53_ = en;
			_tmp54_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp53_);
			_tmp55_ = _tmp54_;
			_tmp56_ = vala_namespace_new (_tmp52_, _tmp55_);
			_vala_code_node_unref0 (_tmp48_);
			_tmp48_ = (ValaSymbol*) _tmp56_;
		} else {
			ValaSymbol* _tmp57_;
			_tmp57_ = _vala_code_node_ref0 (parent);
			_vala_code_node_unref0 (_tmp48_);
			_tmp48_ = _tmp57_;
		}
		_tmp58_ = _vala_code_node_ref0 (_tmp48_);
		next = _tmp58_;
		_tmp59_ = _result_;
		if (VALA_IS_NAMESPACE (_tmp59_)) {
			ValaSymbol* _tmp60_;
			ValaSymbol* _tmp61_;
			_tmp60_ = next;
			_tmp61_ = _result_;
			vala_symbol_add_namespace (_tmp60_, G_TYPE_CHECK_INSTANCE_CAST (_tmp61_, VALA_TYPE_NAMESPACE, ValaNamespace));
		} else {
			ValaSymbol* _tmp62_;
			ValaSymbol* _tmp63_;
			_tmp62_ = next;
			_tmp63_ = _result_;
			vala_symbol_add_enum (_tmp62_, G_TYPE_CHECK_INSTANCE_CAST (_tmp63_, VALA_TYPE_ENUM, ValaEnum));
		}
		_tmp64_ = next;
		_tmp65_ = _vala_code_node_ref0 (_tmp64_);
		_vala_code_node_unref0 (_result_);
		_result_ = _tmp65_;
		_vala_code_node_unref0 (next);
		_vala_code_node_unref0 (_tmp48_);
	}
	_vala_code_node_unref0 (_result_);
	_vala_code_node_unref0 (en);
	_vala_code_node_unref0 (sym);
}

static void
vala_parser_parse_errordomain_declaration (ValaParser* self,
                                           ValaSymbol* parent,
                                           ValaList* attrs,
                                           GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaSymbolAccessibility access = 0;
	ValaParserModifierFlags flags = 0U;
	ValaUnresolvedSymbol* sym = NULL;
	ValaUnresolvedSymbol* _tmp1_;
	ValaErrorDomain* ed = NULL;
	ValaUnresolvedSymbol* _tmp2_;
	const gchar* _tmp3_;
	const gchar* _tmp4_;
	ValaSourceLocation _tmp5_;
	ValaSourceReference* _tmp6_;
	ValaSourceReference* _tmp7_;
	ValaComment* _tmp8_;
	ValaErrorDomain* _tmp9_;
	ValaErrorDomain* _tmp10_;
	ValaErrorDomain* _tmp11_;
	ValaErrorDomain* _tmp13_;
	ValaSourceLocation inner_begin = {0};
	ValaSourceLocation _tmp14_ = {0};
	ValaSymbol* _result_ = NULL;
	ValaErrorDomain* _tmp40_;
	ValaSymbol* _tmp41_;
	GError* _inner_error0_ = NULL;
	g_return_if_fail (self != NULL);
	g_return_if_fail (parent != NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	access = vala_parser_parse_access_modifier (self, VALA_SYMBOL_ACCESSIBILITY_PRIVATE);
	flags = vala_parser_parse_type_declaration_modifiers (self);
	vala_parser_expect (self, VALA_TOKEN_TYPE_ERRORDOMAIN, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp1_ = vala_parser_parse_symbol_name (self, &_inner_error0_);
	sym = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp2_ = sym;
	_tmp3_ = vala_symbol_get_name ((ValaSymbol*) _tmp2_);
	_tmp4_ = _tmp3_;
	_tmp5_ = begin;
	_tmp6_ = vala_parser_get_src (self, &_tmp5_);
	_tmp7_ = _tmp6_;
	_tmp8_ = self->priv->comment;
	_tmp9_ = vala_error_domain_new (_tmp4_, _tmp7_, _tmp8_);
	_tmp10_ = _tmp9_;
	_vala_source_reference_unref0 (_tmp7_);
	ed = _tmp10_;
	_tmp11_ = ed;
	vala_symbol_set_access ((ValaSymbol*) _tmp11_, access);
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_EXTERN) == VALA_PARSER_MODIFIER_FLAGS_EXTERN) {
		ValaErrorDomain* _tmp12_;
		_tmp12_ = ed;
		vala_symbol_set_is_extern ((ValaSymbol*) _tmp12_, TRUE);
	}
	_tmp13_ = ed;
	vala_parser_set_attributes (self, (ValaCodeNode*) _tmp13_, attrs);
	vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_BRACE, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (ed);
			_vala_code_node_unref0 (sym);
			return;
		} else {
			_vala_code_node_unref0 (ed);
			_vala_code_node_unref0 (sym);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	vala_parser_get_location (self, &_tmp14_);
	inner_begin = _tmp14_;
	{
		while (TRUE) {
			ValaErrorDomain* _tmp15_;
			if (!(vala_parser_current (self) != VALA_TOKEN_TYPE_CLOSE_BRACE)) {
				break;
			}
			_tmp15_ = ed;
			vala_parser_parse_declaration (self, (ValaSymbol*) _tmp15_, FALSE, &_inner_error0_);
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					goto __catch0_vala_parse_error;
				}
				_vala_code_node_unref0 (ed);
				_vala_code_node_unref0 (sym);
				g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
	}
	goto __finally0;
	__catch0_vala_parse_error:
	{
		ValaSourceLocation _tmp16_;
		g_clear_error (&_inner_error0_);
		_tmp16_ = inner_begin;
		vala_parser_rollback (self, &_tmp16_);
	}
	__finally0:
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (ed);
			_vala_code_node_unref0 (sym);
			return;
		} else {
			_vala_code_node_unref0 (ed);
			_vala_code_node_unref0 (sym);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	{
		gboolean _tmp17_ = FALSE;
		_tmp17_ = TRUE;
		while (TRUE) {
			ValaList* code_attrs = NULL;
			ValaList* _tmp18_;
			ValaSourceLocation code_begin = {0};
			ValaSourceLocation _tmp19_ = {0};
			gchar* id = NULL;
			gchar* _tmp20_;
			ValaComment* _tmp21_;
			ValaErrorCode* ec = NULL;
			const gchar* _tmp24_;
			ValaSourceLocation _tmp25_;
			ValaSourceReference* _tmp26_;
			ValaSourceReference* _tmp27_;
			ValaComment* _tmp28_;
			ValaErrorCode* _tmp29_;
			ValaErrorCode* _tmp30_;
			ValaErrorCode* _tmp31_;
			ValaErrorCode* _tmp32_;
			ValaList* _tmp33_;
			ValaErrorDomain* _tmp37_;
			ValaErrorCode* _tmp38_;
			if (!_tmp17_) {
				if (!vala_parser_accept (self, VALA_TOKEN_TYPE_COMMA)) {
					break;
				}
			}
			_tmp17_ = FALSE;
			if (vala_parser_current (self) == VALA_TOKEN_TYPE_CLOSE_BRACE) {
				break;
			}
			_tmp18_ = vala_parser_parse_attributes (self, &_inner_error0_);
			code_attrs = _tmp18_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (ed);
					_vala_code_node_unref0 (sym);
					return;
				} else {
					_vala_code_node_unref0 (ed);
					_vala_code_node_unref0 (sym);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return;
				}
			}
			vala_parser_get_location (self, &_tmp19_);
			code_begin = _tmp19_;
			_tmp20_ = vala_parser_parse_identifier (self, FALSE, &_inner_error0_);
			id = _tmp20_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_iterable_unref0 (code_attrs);
					_vala_code_node_unref0 (ed);
					_vala_code_node_unref0 (sym);
					return;
				} else {
					_vala_iterable_unref0 (code_attrs);
					_vala_code_node_unref0 (ed);
					_vala_code_node_unref0 (sym);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return;
				}
			}
			_tmp21_ = self->priv->comment;
			if (_tmp21_ == NULL) {
				ValaScanner* _tmp22_;
				ValaComment* _tmp23_;
				_tmp22_ = self->priv->scanner;
				_tmp23_ = vala_scanner_pop_comment (_tmp22_);
				_vala_comment_unref0 (self->priv->comment);
				self->priv->comment = _tmp23_;
			}
			_tmp24_ = id;
			_tmp25_ = code_begin;
			_tmp26_ = vala_parser_get_src (self, &_tmp25_);
			_tmp27_ = _tmp26_;
			_tmp28_ = self->priv->comment;
			_tmp29_ = vala_error_code_new (_tmp24_, _tmp27_, _tmp28_);
			_tmp30_ = _tmp29_;
			_vala_source_reference_unref0 (_tmp27_);
			ec = _tmp30_;
			_tmp31_ = ec;
			vala_symbol_set_access ((ValaSymbol*) _tmp31_, VALA_SYMBOL_ACCESSIBILITY_PUBLIC);
			_tmp32_ = ec;
			_tmp33_ = code_attrs;
			vala_parser_set_attributes (self, (ValaCodeNode*) _tmp32_, _tmp33_);
			if (vala_parser_accept (self, VALA_TOKEN_TYPE_ASSIGN)) {
				ValaExpression* _tmp34_ = NULL;
				ValaExpression* _tmp35_;
				ValaErrorCode* _tmp36_;
				_tmp35_ = vala_parser_parse_expression (self, &_inner_error0_);
				_tmp34_ = _tmp35_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_code_node_unref0 (ec);
						_g_free0 (id);
						_vala_iterable_unref0 (code_attrs);
						_vala_code_node_unref0 (ed);
						_vala_code_node_unref0 (sym);
						return;
					} else {
						_vala_code_node_unref0 (ec);
						_g_free0 (id);
						_vala_iterable_unref0 (code_attrs);
						_vala_code_node_unref0 (ed);
						_vala_code_node_unref0 (sym);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return;
					}
				}
				_tmp36_ = ec;
				vala_error_code_set_value (_tmp36_, _tmp34_);
				_vala_code_node_unref0 (_tmp34_);
			}
			_tmp37_ = ed;
			_tmp38_ = ec;
			vala_error_domain_add_code (_tmp37_, _tmp38_);
			_vala_comment_unref0 (self->priv->comment);
			self->priv->comment = NULL;
			_vala_code_node_unref0 (ec);
			_g_free0 (id);
			_vala_iterable_unref0 (code_attrs);
		}
	}
	if (vala_parser_accept (self, VALA_TOKEN_TYPE_SEMICOLON)) {
		while (TRUE) {
			ValaErrorDomain* _tmp39_;
			if (!(vala_parser_current (self) != VALA_TOKEN_TYPE_CLOSE_BRACE)) {
				break;
			}
			_tmp39_ = ed;
			vala_parser_parse_declaration (self, (ValaSymbol*) _tmp39_, FALSE, &_inner_error0_);
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (ed);
					_vala_code_node_unref0 (sym);
					return;
				} else {
					_vala_code_node_unref0 (ed);
					_vala_code_node_unref0 (sym);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return;
				}
			}
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_BRACE, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (ed);
			_vala_code_node_unref0 (sym);
			return;
		} else {
			_vala_code_node_unref0 (ed);
			_vala_code_node_unref0 (sym);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp40_ = ed;
	_tmp41_ = _vala_code_node_ref0 ((ValaSymbol*) _tmp40_);
	_result_ = _tmp41_;
	while (TRUE) {
		ValaUnresolvedSymbol* _tmp42_;
		ValaUnresolvedSymbol* _tmp43_;
		ValaUnresolvedSymbol* _tmp44_;
		ValaUnresolvedSymbol* _tmp45_;
		ValaUnresolvedSymbol* _tmp46_;
		ValaSymbol* _tmp47_ = NULL;
		ValaUnresolvedSymbol* _tmp48_;
		ValaSymbol* next = NULL;
		ValaSymbol* _tmp57_;
		ValaSymbol* _tmp58_;
		ValaSymbol* _tmp63_;
		ValaSymbol* _tmp64_;
		_tmp42_ = sym;
		if (!(_tmp42_ != NULL)) {
			break;
		}
		_tmp43_ = sym;
		_tmp44_ = vala_unresolved_symbol_get_inner (_tmp43_);
		_tmp45_ = _tmp44_;
		_tmp46_ = _vala_code_node_ref0 (_tmp45_);
		_vala_code_node_unref0 (sym);
		sym = _tmp46_;
		_tmp48_ = sym;
		if (_tmp48_ != NULL) {
			ValaUnresolvedSymbol* _tmp49_;
			const gchar* _tmp50_;
			const gchar* _tmp51_;
			ValaErrorDomain* _tmp52_;
			ValaSourceReference* _tmp53_;
			ValaSourceReference* _tmp54_;
			ValaNamespace* _tmp55_;
			_tmp49_ = sym;
			_tmp50_ = vala_symbol_get_name ((ValaSymbol*) _tmp49_);
			_tmp51_ = _tmp50_;
			_tmp52_ = ed;
			_tmp53_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp52_);
			_tmp54_ = _tmp53_;
			_tmp55_ = vala_namespace_new (_tmp51_, _tmp54_);
			_vala_code_node_unref0 (_tmp47_);
			_tmp47_ = (ValaSymbol*) _tmp55_;
		} else {
			ValaSymbol* _tmp56_;
			_tmp56_ = _vala_code_node_ref0 (parent);
			_vala_code_node_unref0 (_tmp47_);
			_tmp47_ = _tmp56_;
		}
		_tmp57_ = _vala_code_node_ref0 (_tmp47_);
		next = _tmp57_;
		_tmp58_ = _result_;
		if (VALA_IS_NAMESPACE (_tmp58_)) {
			ValaSymbol* _tmp59_;
			ValaSymbol* _tmp60_;
			_tmp59_ = next;
			_tmp60_ = _result_;
			vala_symbol_add_namespace (_tmp59_, G_TYPE_CHECK_INSTANCE_CAST (_tmp60_, VALA_TYPE_NAMESPACE, ValaNamespace));
		} else {
			ValaSymbol* _tmp61_;
			ValaSymbol* _tmp62_;
			_tmp61_ = next;
			_tmp62_ = _result_;
			vala_symbol_add_error_domain (_tmp61_, G_TYPE_CHECK_INSTANCE_CAST (_tmp62_, VALA_TYPE_ERROR_DOMAIN, ValaErrorDomain));
		}
		_tmp63_ = next;
		_tmp64_ = _vala_code_node_ref0 (_tmp63_);
		_vala_code_node_unref0 (_result_);
		_result_ = _tmp64_;
		_vala_code_node_unref0 (next);
		_vala_code_node_unref0 (_tmp47_);
	}
	_vala_code_node_unref0 (_result_);
	_vala_code_node_unref0 (ed);
	_vala_code_node_unref0 (sym);
}

static ValaSymbolAccessibility
vala_parser_parse_access_modifier (ValaParser* self,
                                   ValaSymbolAccessibility default_access)
{
	ValaSymbolAccessibility result;
	g_return_val_if_fail (self != NULL, 0);
	switch (vala_parser_current (self)) {
		case VALA_TOKEN_TYPE_PRIVATE:
		{
			vala_parser_next (self);
			result = VALA_SYMBOL_ACCESSIBILITY_PRIVATE;
			return result;
		}
		case VALA_TOKEN_TYPE_PROTECTED:
		{
			vala_parser_next (self);
			result = VALA_SYMBOL_ACCESSIBILITY_PROTECTED;
			return result;
		}
		case VALA_TOKEN_TYPE_INTERNAL:
		{
			vala_parser_next (self);
			result = VALA_SYMBOL_ACCESSIBILITY_INTERNAL;
			return result;
		}
		case VALA_TOKEN_TYPE_PUBLIC:
		{
			vala_parser_next (self);
			result = VALA_SYMBOL_ACCESSIBILITY_PUBLIC;
			return result;
		}
		default:
		{
			result = default_access;
			return result;
		}
	}
}

static ValaParserModifierFlags
vala_parser_parse_type_declaration_modifiers (ValaParser* self)
{
	ValaParserModifierFlags flags = 0U;
	ValaParserModifierFlags result;
	g_return_val_if_fail (self != NULL, 0U);
	flags = 0;
	while (TRUE) {
		switch (vala_parser_current (self)) {
			case VALA_TOKEN_TYPE_ABSTRACT:
			{
				vala_parser_next (self);
				flags |= VALA_PARSER_MODIFIER_FLAGS_ABSTRACT;
				break;
			}
			case VALA_TOKEN_TYPE_EXTERN:
			{
				vala_parser_next (self);
				flags |= VALA_PARSER_MODIFIER_FLAGS_EXTERN;
				break;
			}
			case VALA_TOKEN_TYPE_PARTIAL:
			{
				vala_parser_next (self);
				flags |= VALA_PARSER_MODIFIER_FLAGS_PARTIAL;
				break;
			}
			case VALA_TOKEN_TYPE_SEALED:
			{
				vala_parser_next (self);
				flags |= VALA_PARSER_MODIFIER_FLAGS_SEALED;
				break;
			}
			default:
			{
				result = flags;
				return result;
			}
		}
	}
}

static ValaParserModifierFlags
vala_parser_parse_member_declaration_modifiers (ValaParser* self)
{
	ValaParserModifierFlags flags = 0U;
	ValaParserModifierFlags result;
	g_return_val_if_fail (self != NULL, 0U);
	flags = 0;
	while (TRUE) {
		switch (vala_parser_current (self)) {
			case VALA_TOKEN_TYPE_ABSTRACT:
			{
				vala_parser_next (self);
				flags |= VALA_PARSER_MODIFIER_FLAGS_ABSTRACT;
				break;
			}
			case VALA_TOKEN_TYPE_ASYNC:
			{
				vala_parser_next (self);
				flags |= VALA_PARSER_MODIFIER_FLAGS_ASYNC;
				break;
			}
			case VALA_TOKEN_TYPE_CLASS:
			{
				vala_parser_next (self);
				flags |= VALA_PARSER_MODIFIER_FLAGS_CLASS;
				break;
			}
			case VALA_TOKEN_TYPE_EXTERN:
			{
				vala_parser_next (self);
				flags |= VALA_PARSER_MODIFIER_FLAGS_EXTERN;
				break;
			}
			case VALA_TOKEN_TYPE_INLINE:
			{
				vala_parser_next (self);
				flags |= VALA_PARSER_MODIFIER_FLAGS_INLINE;
				break;
			}
			case VALA_TOKEN_TYPE_NEW:
			{
				vala_parser_next (self);
				flags |= VALA_PARSER_MODIFIER_FLAGS_NEW;
				break;
			}
			case VALA_TOKEN_TYPE_OVERRIDE:
			{
				vala_parser_next (self);
				flags |= VALA_PARSER_MODIFIER_FLAGS_OVERRIDE;
				break;
			}
			case VALA_TOKEN_TYPE_SEALED:
			{
				vala_parser_next (self);
				flags |= VALA_PARSER_MODIFIER_FLAGS_SEALED;
				break;
			}
			case VALA_TOKEN_TYPE_STATIC:
			{
				vala_parser_next (self);
				flags |= VALA_PARSER_MODIFIER_FLAGS_STATIC;
				break;
			}
			case VALA_TOKEN_TYPE_VIRTUAL:
			{
				vala_parser_next (self);
				flags |= VALA_PARSER_MODIFIER_FLAGS_VIRTUAL;
				break;
			}
			default:
			{
				result = flags;
				return result;
			}
		}
	}
}

static ValaParameter*
vala_parser_parse_parameter (ValaParser* self,
                             GError** error)
{
	ValaList* attrs = NULL;
	ValaList* _tmp0_;
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp1_ = {0};
	gboolean params_array = FALSE;
	ValaParameterDirection direction = 0;
	ValaDataType* type = NULL;
	gchar* id = NULL;
	gchar* _tmp16_;
	ValaDataType* _tmp17_ = NULL;
	ValaDataType* _tmp18_;
	ValaDataType* _tmp19_;
	ValaDataType* _tmp20_;
	ValaParameter* param = NULL;
	const gchar* _tmp21_;
	ValaDataType* _tmp22_;
	ValaSourceLocation _tmp23_;
	ValaSourceReference* _tmp24_;
	ValaSourceReference* _tmp25_;
	ValaParameter* _tmp26_;
	ValaParameter* _tmp27_;
	ValaParameter* _tmp28_;
	ValaList* _tmp29_;
	ValaParameter* _tmp30_;
	ValaParameter* _tmp31_;
	GError* _inner_error0_ = NULL;
	ValaParameter* result;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = vala_parser_parse_attributes (self, &_inner_error0_);
	attrs = _tmp0_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return NULL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	vala_parser_get_location (self, &_tmp1_);
	begin = _tmp1_;
	if (vala_parser_accept (self, VALA_TOKEN_TYPE_ELLIPSIS)) {
		ValaSourceLocation _tmp2_;
		ValaSourceReference* _tmp3_;
		ValaSourceReference* _tmp4_;
		ValaParameter* _tmp5_;
		ValaParameter* _tmp6_;
		_tmp2_ = begin;
		_tmp3_ = vala_parser_get_src (self, &_tmp2_);
		_tmp4_ = _tmp3_;
		_tmp5_ = vala_parameter_new_with_ellipsis (_tmp4_);
		_tmp6_ = _tmp5_;
		_vala_source_reference_unref0 (_tmp4_);
		result = _tmp6_;
		_vala_iterable_unref0 (attrs);
		return result;
	}
	params_array = vala_parser_accept (self, VALA_TOKEN_TYPE_PARAMS);
	direction = VALA_PARAMETER_DIRECTION_IN;
	if (vala_parser_accept (self, VALA_TOKEN_TYPE_OUT)) {
		direction = VALA_PARAMETER_DIRECTION_OUT;
	} else {
		if (vala_parser_accept (self, VALA_TOKEN_TYPE_REF)) {
			direction = VALA_PARAMETER_DIRECTION_REF;
		}
	}
	if (direction == VALA_PARAMETER_DIRECTION_IN) {
		ValaDataType* _tmp7_ = NULL;
		ValaDataType* _tmp8_;
		ValaDataType* _tmp9_;
		_tmp8_ = vala_parser_parse_type (self, FALSE, FALSE, FALSE, &_inner_error0_);
		_tmp7_ = _tmp8_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (type);
				_vala_iterable_unref0 (attrs);
				return NULL;
			} else {
				_vala_code_node_unref0 (type);
				_vala_iterable_unref0 (attrs);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp9_ = _tmp7_;
		_tmp7_ = NULL;
		_vala_code_node_unref0 (type);
		type = _tmp9_;
		_vala_code_node_unref0 (_tmp7_);
	} else {
		if (direction == VALA_PARAMETER_DIRECTION_REF) {
			ValaDataType* _tmp10_ = NULL;
			ValaDataType* _tmp11_;
			ValaDataType* _tmp12_;
			_tmp11_ = vala_parser_parse_type (self, TRUE, TRUE, FALSE, &_inner_error0_);
			_tmp10_ = _tmp11_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (type);
					_vala_iterable_unref0 (attrs);
					return NULL;
				} else {
					_vala_code_node_unref0 (type);
					_vala_iterable_unref0 (attrs);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp12_ = _tmp10_;
			_tmp10_ = NULL;
			_vala_code_node_unref0 (type);
			type = _tmp12_;
			_vala_code_node_unref0 (_tmp10_);
		} else {
			ValaDataType* _tmp13_ = NULL;
			ValaDataType* _tmp14_;
			ValaDataType* _tmp15_;
			_tmp14_ = vala_parser_parse_type (self, TRUE, FALSE, FALSE, &_inner_error0_);
			_tmp13_ = _tmp14_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (type);
					_vala_iterable_unref0 (attrs);
					return NULL;
				} else {
					_vala_code_node_unref0 (type);
					_vala_iterable_unref0 (attrs);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp15_ = _tmp13_;
			_tmp13_ = NULL;
			_vala_code_node_unref0 (type);
			type = _tmp15_;
			_vala_code_node_unref0 (_tmp13_);
		}
	}
	_tmp16_ = vala_parser_parse_identifier (self, FALSE, &_inner_error0_);
	id = _tmp16_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (type);
			_vala_iterable_unref0 (attrs);
			return NULL;
		} else {
			_vala_code_node_unref0 (type);
			_vala_iterable_unref0 (attrs);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp18_ = type;
	_tmp19_ = vala_parser_parse_inline_array_type (self, _tmp18_, &_inner_error0_);
	_tmp17_ = _tmp19_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_g_free0 (id);
			_vala_code_node_unref0 (type);
			_vala_iterable_unref0 (attrs);
			return NULL;
		} else {
			_g_free0 (id);
			_vala_code_node_unref0 (type);
			_vala_iterable_unref0 (attrs);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return NULL;
		}
	}
	_tmp20_ = _tmp17_;
	_tmp17_ = NULL;
	_vala_code_node_unref0 (type);
	type = _tmp20_;
	_tmp21_ = id;
	_tmp22_ = type;
	_tmp23_ = begin;
	_tmp24_ = vala_parser_get_src (self, &_tmp23_);
	_tmp25_ = _tmp24_;
	_tmp26_ = vala_parameter_new (_tmp21_, _tmp22_, _tmp25_);
	_tmp27_ = _tmp26_;
	_vala_source_reference_unref0 (_tmp25_);
	param = _tmp27_;
	_tmp28_ = param;
	_tmp29_ = attrs;
	vala_parser_set_attributes (self, (ValaCodeNode*) _tmp28_, _tmp29_);
	_tmp30_ = param;
	vala_parameter_set_direction (_tmp30_, direction);
	_tmp31_ = param;
	vala_parameter_set_params_array (_tmp31_, params_array);
	if (vala_parser_accept (self, VALA_TOKEN_TYPE_ASSIGN)) {
		ValaExpression* _tmp32_ = NULL;
		ValaExpression* _tmp33_;
		ValaParameter* _tmp34_;
		_tmp33_ = vala_parser_parse_expression (self, &_inner_error0_);
		_tmp32_ = _tmp33_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (param);
				_vala_code_node_unref0 (_tmp17_);
				_g_free0 (id);
				_vala_code_node_unref0 (type);
				_vala_iterable_unref0 (attrs);
				return NULL;
			} else {
				_vala_code_node_unref0 (param);
				_vala_code_node_unref0 (_tmp17_);
				_g_free0 (id);
				_vala_code_node_unref0 (type);
				_vala_iterable_unref0 (attrs);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		_tmp34_ = param;
		vala_variable_set_initializer ((ValaVariable*) _tmp34_, _tmp32_);
		_vala_code_node_unref0 (_tmp32_);
	}
	result = param;
	_vala_code_node_unref0 (_tmp17_);
	_g_free0 (id);
	_vala_code_node_unref0 (type);
	_vala_iterable_unref0 (attrs);
	return result;
}

static void
vala_parser_parse_creation_method_declaration (ValaParser* self,
                                               ValaSymbol* parent,
                                               ValaList* attrs,
                                               GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaSymbolAccessibility access = 0;
	ValaParserModifierFlags flags = 0U;
	ValaUnresolvedSymbol* sym = NULL;
	ValaUnresolvedSymbol* _tmp1_;
	ValaCreationMethod* method = NULL;
	ValaUnresolvedSymbol* _tmp3_;
	ValaUnresolvedSymbol* _tmp4_;
	ValaUnresolvedSymbol* _tmp5_;
	gboolean _tmp28_ = FALSE;
	gboolean _tmp29_ = FALSE;
	ValaCreationMethod* _tmp48_;
	ValaCreationMethod* _tmp49_;
	ValaCreationMethod* _tmp54_;
	GError* _inner_error0_ = NULL;
	g_return_if_fail (self != NULL);
	g_return_if_fail (parent != NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	access = vala_parser_parse_access_modifier (self, VALA_SYMBOL_ACCESSIBILITY_PRIVATE);
	flags = vala_parser_parse_member_declaration_modifiers (self);
	_tmp1_ = vala_parser_parse_symbol_name (self, &_inner_error0_);
	sym = _tmp1_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_NEW) == VALA_PARSER_MODIFIER_FLAGS_NEW) {
		GError* _tmp2_;
		_tmp2_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "`new' modifier not allowed on creation method");
		_inner_error0_ = _tmp2_;
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (sym);
			return;
		} else {
			_vala_code_node_unref0 (sym);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp3_ = sym;
	_tmp4_ = vala_unresolved_symbol_get_inner (_tmp3_);
	_tmp5_ = _tmp4_;
	if (_tmp5_ == NULL) {
		ValaUnresolvedSymbol* _tmp6_;
		const gchar* _tmp7_;
		const gchar* _tmp8_;
		ValaSourceLocation _tmp9_;
		ValaSourceReference* _tmp10_;
		ValaSourceReference* _tmp11_;
		ValaComment* _tmp12_;
		ValaCreationMethod* _tmp13_;
		_tmp6_ = sym;
		_tmp7_ = vala_symbol_get_name ((ValaSymbol*) _tmp6_);
		_tmp8_ = _tmp7_;
		_tmp9_ = begin;
		_tmp10_ = vala_parser_get_src (self, &_tmp9_);
		_tmp11_ = _tmp10_;
		_tmp12_ = self->priv->comment;
		_tmp13_ = vala_creation_method_new (_tmp8_, NULL, _tmp11_, _tmp12_);
		_vala_code_node_unref0 (method);
		method = _tmp13_;
		_vala_source_reference_unref0 (_tmp11_);
	} else {
		ValaUnresolvedSymbol* _tmp14_;
		ValaUnresolvedSymbol* _tmp15_;
		ValaUnresolvedSymbol* _tmp16_;
		const gchar* _tmp17_;
		const gchar* _tmp18_;
		ValaUnresolvedSymbol* _tmp19_;
		const gchar* _tmp20_;
		const gchar* _tmp21_;
		ValaSourceLocation _tmp22_;
		ValaSourceReference* _tmp23_;
		ValaSourceReference* _tmp24_;
		ValaComment* _tmp25_;
		ValaCreationMethod* _tmp26_;
		_tmp14_ = sym;
		_tmp15_ = vala_unresolved_symbol_get_inner (_tmp14_);
		_tmp16_ = _tmp15_;
		_tmp17_ = vala_symbol_get_name ((ValaSymbol*) _tmp16_);
		_tmp18_ = _tmp17_;
		_tmp19_ = sym;
		_tmp20_ = vala_symbol_get_name ((ValaSymbol*) _tmp19_);
		_tmp21_ = _tmp20_;
		_tmp22_ = begin;
		_tmp23_ = vala_parser_get_src (self, &_tmp22_);
		_tmp24_ = _tmp23_;
		_tmp25_ = self->priv->comment;
		_tmp26_ = vala_creation_method_new (_tmp18_, _tmp21_, _tmp24_, _tmp25_);
		_vala_code_node_unref0 (method);
		method = _tmp26_;
		_vala_source_reference_unref0 (_tmp24_);
	}
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_EXTERN) == VALA_PARSER_MODIFIER_FLAGS_EXTERN) {
		ValaCreationMethod* _tmp27_;
		_tmp27_ = method;
		vala_symbol_set_is_extern ((ValaSymbol*) _tmp27_, TRUE);
	}
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_ABSTRACT) == VALA_PARSER_MODIFIER_FLAGS_ABSTRACT) {
		_tmp29_ = TRUE;
	} else {
		_tmp29_ = (flags & VALA_PARSER_MODIFIER_FLAGS_VIRTUAL) == VALA_PARSER_MODIFIER_FLAGS_VIRTUAL;
	}
	if (_tmp29_) {
		_tmp28_ = TRUE;
	} else {
		_tmp28_ = (flags & VALA_PARSER_MODIFIER_FLAGS_OVERRIDE) == VALA_PARSER_MODIFIER_FLAGS_OVERRIDE;
	}
	if (_tmp28_) {
		ValaCreationMethod* _tmp30_;
		ValaSourceReference* _tmp31_;
		ValaSourceReference* _tmp32_;
		_tmp30_ = method;
		_tmp31_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp30_);
		_tmp32_ = _tmp31_;
		vala_report_error (_tmp32_, "abstract, virtual, and override modifiers are not applicable to creati" \
"on methods");
	}
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_ASYNC) == VALA_PARSER_MODIFIER_FLAGS_ASYNC) {
		ValaCreationMethod* _tmp33_;
		_tmp33_ = method;
		vala_method_set_coroutine ((ValaMethod*) _tmp33_, TRUE);
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (method);
			_vala_code_node_unref0 (sym);
			return;
		} else {
			_vala_code_node_unref0 (method);
			_vala_code_node_unref0 (sym);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	if (vala_parser_current (self) != VALA_TOKEN_TYPE_CLOSE_PARENS) {
		{
			gboolean _tmp34_ = FALSE;
			_tmp34_ = TRUE;
			while (TRUE) {
				ValaParameter* param = NULL;
				ValaParameter* _tmp35_;
				ValaCreationMethod* _tmp36_;
				ValaParameter* _tmp37_;
				if (!_tmp34_) {
					if (!vala_parser_accept (self, VALA_TOKEN_TYPE_COMMA)) {
						break;
					}
				}
				_tmp34_ = FALSE;
				_tmp35_ = vala_parser_parse_parameter (self, &_inner_error0_);
				param = _tmp35_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_code_node_unref0 (method);
						_vala_code_node_unref0 (sym);
						return;
					} else {
						_vala_code_node_unref0 (method);
						_vala_code_node_unref0 (sym);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return;
					}
				}
				_tmp36_ = method;
				_tmp37_ = param;
				vala_callable_add_parameter ((ValaCallable*) _tmp36_, _tmp37_);
				_vala_code_node_unref0 (param);
			}
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (method);
			_vala_code_node_unref0 (sym);
			return;
		} else {
			_vala_code_node_unref0 (method);
			_vala_code_node_unref0 (sym);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	if (vala_parser_accept (self, VALA_TOKEN_TYPE_THROWS)) {
		{
			gboolean _tmp38_ = FALSE;
			_tmp38_ = TRUE;
			while (TRUE) {
				ValaDataType* _tmp39_ = NULL;
				ValaDataType* _tmp40_;
				ValaCreationMethod* _tmp41_;
				if (!_tmp38_) {
					if (!vala_parser_accept (self, VALA_TOKEN_TYPE_COMMA)) {
						break;
					}
				}
				_tmp38_ = FALSE;
				_tmp40_ = vala_parser_parse_type (self, TRUE, FALSE, FALSE, &_inner_error0_);
				_tmp39_ = _tmp40_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_code_node_unref0 (method);
						_vala_code_node_unref0 (sym);
						return;
					} else {
						_vala_code_node_unref0 (method);
						_vala_code_node_unref0 (sym);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return;
					}
				}
				_tmp41_ = method;
				vala_method_add_error_type ((ValaMethod*) _tmp41_, _tmp39_);
				_vala_code_node_unref0 (_tmp39_);
			}
		}
	}
	while (TRUE) {
		ValaExpression* _tmp42_ = NULL;
		ValaExpression* _tmp43_;
		ValaCreationMethod* _tmp44_;
		if (!vala_parser_accept (self, VALA_TOKEN_TYPE_REQUIRES)) {
			break;
		}
		vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_PARENS, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (method);
				_vala_code_node_unref0 (sym);
				return;
			} else {
				_vala_code_node_unref0 (method);
				_vala_code_node_unref0 (sym);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
		_tmp43_ = vala_parser_parse_expression (self, &_inner_error0_);
		_tmp42_ = _tmp43_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (method);
				_vala_code_node_unref0 (sym);
				return;
			} else {
				_vala_code_node_unref0 (method);
				_vala_code_node_unref0 (sym);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
		_tmp44_ = method;
		vala_method_add_precondition ((ValaMethod*) _tmp44_, _tmp42_);
		vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_PARENS, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (_tmp42_);
				_vala_code_node_unref0 (method);
				_vala_code_node_unref0 (sym);
				return;
			} else {
				_vala_code_node_unref0 (_tmp42_);
				_vala_code_node_unref0 (method);
				_vala_code_node_unref0 (sym);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
		_vala_code_node_unref0 (_tmp42_);
	}
	while (TRUE) {
		ValaExpression* _tmp45_ = NULL;
		ValaExpression* _tmp46_;
		ValaCreationMethod* _tmp47_;
		if (!vala_parser_accept (self, VALA_TOKEN_TYPE_ENSURES)) {
			break;
		}
		vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_PARENS, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (method);
				_vala_code_node_unref0 (sym);
				return;
			} else {
				_vala_code_node_unref0 (method);
				_vala_code_node_unref0 (sym);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
		_tmp46_ = vala_parser_parse_expression (self, &_inner_error0_);
		_tmp45_ = _tmp46_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (method);
				_vala_code_node_unref0 (sym);
				return;
			} else {
				_vala_code_node_unref0 (method);
				_vala_code_node_unref0 (sym);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
		_tmp47_ = method;
		vala_method_add_postcondition ((ValaMethod*) _tmp47_, _tmp45_);
		vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_PARENS, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (_tmp45_);
				_vala_code_node_unref0 (method);
				_vala_code_node_unref0 (sym);
				return;
			} else {
				_vala_code_node_unref0 (_tmp45_);
				_vala_code_node_unref0 (method);
				_vala_code_node_unref0 (sym);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
		_vala_code_node_unref0 (_tmp45_);
	}
	_tmp48_ = method;
	vala_symbol_set_access ((ValaSymbol*) _tmp48_, access);
	_tmp49_ = method;
	vala_parser_set_attributes (self, (ValaCodeNode*) _tmp49_, attrs);
	if (!vala_parser_accept (self, VALA_TOKEN_TYPE_SEMICOLON)) {
		ValaBlock* _tmp50_ = NULL;
		ValaBlock* _tmp51_;
		ValaCreationMethod* _tmp52_;
		ValaCreationMethod* _tmp53_;
		_tmp51_ = vala_parser_parse_block (self, &_inner_error0_);
		_tmp50_ = _tmp51_;
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_code_node_unref0 (method);
				_vala_code_node_unref0 (sym);
				return;
			} else {
				_vala_code_node_unref0 (method);
				_vala_code_node_unref0 (sym);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
		_tmp52_ = method;
		vala_subroutine_set_body ((ValaSubroutine*) _tmp52_, _tmp50_);
		_tmp53_ = method;
		vala_symbol_set_external ((ValaSymbol*) _tmp53_, FALSE);
		_vala_code_node_unref0 (_tmp50_);
	}
	_tmp54_ = method;
	vala_symbol_add_method (parent, (ValaMethod*) _tmp54_);
	_vala_code_node_unref0 (method);
	_vala_code_node_unref0 (sym);
}

static void
vala_parser_parse_delegate_declaration (ValaParser* self,
                                        ValaSymbol* parent,
                                        ValaList* attrs,
                                        GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaSymbolAccessibility access = 0;
	ValaParserModifierFlags flags = 0U;
	ValaDataType* type = NULL;
	ValaDataType* _tmp2_;
	ValaUnresolvedSymbol* sym = NULL;
	ValaUnresolvedSymbol* _tmp3_;
	ValaList* type_param_list = NULL;
	ValaList* _tmp4_;
	ValaDelegate* d = NULL;
	ValaUnresolvedSymbol* _tmp5_;
	const gchar* _tmp6_;
	const gchar* _tmp7_;
	ValaDataType* _tmp8_;
	ValaSourceLocation _tmp9_;
	ValaSourceReference* _tmp10_;
	ValaSourceReference* _tmp11_;
	ValaComment* _tmp12_;
	ValaDelegate* _tmp13_;
	ValaDelegate* _tmp14_;
	ValaDelegate* _tmp15_;
	ValaDelegate* _tmp16_;
	ValaDelegate* _tmp24_;
	ValaSymbol* _result_ = NULL;
	ValaDelegate* _tmp49_;
	ValaSymbol* _tmp50_;
	GError* _inner_error0_ = NULL;
	g_return_if_fail (self != NULL);
	g_return_if_fail (parent != NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	access = vala_parser_parse_access_modifier (self, VALA_SYMBOL_ACCESSIBILITY_PRIVATE);
	flags = vala_parser_parse_member_declaration_modifiers (self);
	vala_parser_expect (self, VALA_TOKEN_TYPE_DELEGATE, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_NEW) == VALA_PARSER_MODIFIER_FLAGS_NEW) {
		GError* _tmp1_;
		_tmp1_ = g_error_new_literal (VALA_PARSE_ERROR, VALA_PARSE_ERROR_SYNTAX, "`new' modifier not allowed on delegates");
		_inner_error0_ = _tmp1_;
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp2_ = vala_parser_parse_type (self, TRUE, FALSE, FALSE, &_inner_error0_);
	type = _tmp2_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp3_ = vala_parser_parse_symbol_name (self, &_inner_error0_);
	sym = _tmp3_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (type);
			return;
		} else {
			_vala_code_node_unref0 (type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp4_ = vala_parser_parse_type_parameter_list (self, &_inner_error0_);
	type_param_list = _tmp4_;
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (sym);
			_vala_code_node_unref0 (type);
			return;
		} else {
			_vala_code_node_unref0 (sym);
			_vala_code_node_unref0 (type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp5_ = sym;
	_tmp6_ = vala_symbol_get_name ((ValaSymbol*) _tmp5_);
	_tmp7_ = _tmp6_;
	_tmp8_ = type;
	_tmp9_ = begin;
	_tmp10_ = vala_parser_get_src (self, &_tmp9_);
	_tmp11_ = _tmp10_;
	_tmp12_ = self->priv->comment;
	_tmp13_ = vala_delegate_new (_tmp7_, _tmp8_, _tmp11_, _tmp12_);
	_tmp14_ = _tmp13_;
	_vala_source_reference_unref0 (_tmp11_);
	d = _tmp14_;
	_tmp15_ = d;
	vala_symbol_set_access ((ValaSymbol*) _tmp15_, access);
	_tmp16_ = d;
	vala_parser_set_attributes (self, (ValaCodeNode*) _tmp16_, attrs);
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_STATIC) == VALA_PARSER_MODIFIER_FLAGS_STATIC) {
		ValaCodeContext* _tmp17_;
		gboolean _tmp18_;
		gboolean _tmp19_;
		ValaDelegate* _tmp22_;
		_tmp17_ = self->priv->context;
		_tmp18_ = vala_code_context_get_deprecated (_tmp17_);
		_tmp19_ = _tmp18_;
		if (!_tmp19_) {
			ValaSourceReference* _tmp20_;
			ValaSourceReference* _tmp21_;
			_tmp20_ = vala_parser_get_last_src (self);
			_tmp21_ = _tmp20_;
			vala_report_warning (_tmp21_, "deprecated syntax, use [CCode (has_target = false)]");
			_vala_source_reference_unref0 (_tmp21_);
		}
		_tmp22_ = d;
		vala_delegate_set_has_target (_tmp22_, FALSE);
	}
	if ((flags & VALA_PARSER_MODIFIER_FLAGS_EXTERN) == VALA_PARSER_MODIFIER_FLAGS_EXTERN) {
		ValaDelegate* _tmp23_;
		_tmp23_ = d;
		vala_symbol_set_is_extern ((ValaSymbol*) _tmp23_, TRUE);
	}
	_tmp24_ = d;
	if (!vala_code_node_get_attribute_bool ((ValaCodeNode*) _tmp24_, "CCode", "has_typedef", TRUE)) {
		ValaDelegate* _tmp25_;
		gboolean _tmp26_;
		gboolean _tmp27_;
		ValaDelegate* _tmp30_;
		_tmp25_ = d;
		_tmp26_ = vala_symbol_get_external ((ValaSymbol*) _tmp25_);
		_tmp27_ = _tmp26_;
		if (!_tmp27_) {
			ValaSourceReference* _tmp28_;
			ValaSourceReference* _tmp29_;
			_tmp28_ = vala_parser_get_last_src (self);
			_tmp29_ = _tmp28_;
			vala_report_error (_tmp29_, "Delegates without definition must be external");
			_vala_source_reference_unref0 (_tmp29_);
		}
		_tmp30_ = d;
		vala_symbol_set_anonymous ((ValaSymbol*) _tmp30_, TRUE);
	}
	{
		ValaList* _type_param_list = NULL;
		ValaList* _tmp31_;
		gint _type_param_size = 0;
		ValaList* _tmp32_;
		gint _tmp33_;
		gint _tmp34_;
		gint _type_param_index = 0;
		_tmp31_ = type_param_list;
		_type_param_list = _tmp31_;
		_tmp32_ = _type_param_list;
		_tmp33_ = vala_collection_get_size ((ValaCollection*) _tmp32_);
		_tmp34_ = _tmp33_;
		_type_param_size = _tmp34_;
		_type_param_index = -1;
		while (TRUE) {
			gint _tmp35_;
			gint _tmp36_;
			ValaTypeParameter* type_param = NULL;
			ValaList* _tmp37_;
			gpointer _tmp38_;
			ValaDelegate* _tmp39_;
			ValaTypeParameter* _tmp40_;
			_type_param_index = _type_param_index + 1;
			_tmp35_ = _type_param_index;
			_tmp36_ = _type_param_size;
			if (!(_tmp35_ < _tmp36_)) {
				break;
			}
			_tmp37_ = _type_param_list;
			_tmp38_ = vala_list_get (_tmp37_, _type_param_index);
			type_param = (ValaTypeParameter*) _tmp38_;
			_tmp39_ = d;
			_tmp40_ = type_param;
			vala_delegate_add_type_parameter (_tmp39_, _tmp40_);
			_vala_code_node_unref0 (type_param);
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_OPEN_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (d);
			_vala_iterable_unref0 (type_param_list);
			_vala_code_node_unref0 (sym);
			_vala_code_node_unref0 (type);
			return;
		} else {
			_vala_code_node_unref0 (d);
			_vala_iterable_unref0 (type_param_list);
			_vala_code_node_unref0 (sym);
			_vala_code_node_unref0 (type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	if (vala_parser_current (self) != VALA_TOKEN_TYPE_CLOSE_PARENS) {
		{
			gboolean _tmp41_ = FALSE;
			_tmp41_ = TRUE;
			while (TRUE) {
				ValaParameter* param = NULL;
				ValaParameter* _tmp42_;
				ValaDelegate* _tmp43_;
				ValaParameter* _tmp44_;
				if (!_tmp41_) {
					if (!vala_parser_accept (self, VALA_TOKEN_TYPE_COMMA)) {
						break;
					}
				}
				_tmp41_ = FALSE;
				_tmp42_ = vala_parser_parse_parameter (self, &_inner_error0_);
				param = _tmp42_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_code_node_unref0 (d);
						_vala_iterable_unref0 (type_param_list);
						_vala_code_node_unref0 (sym);
						_vala_code_node_unref0 (type);
						return;
					} else {
						_vala_code_node_unref0 (d);
						_vala_iterable_unref0 (type_param_list);
						_vala_code_node_unref0 (sym);
						_vala_code_node_unref0 (type);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return;
					}
				}
				_tmp43_ = d;
				_tmp44_ = param;
				vala_callable_add_parameter ((ValaCallable*) _tmp43_, _tmp44_);
				_vala_code_node_unref0 (param);
			}
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_CLOSE_PARENS, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (d);
			_vala_iterable_unref0 (type_param_list);
			_vala_code_node_unref0 (sym);
			_vala_code_node_unref0 (type);
			return;
		} else {
			_vala_code_node_unref0 (d);
			_vala_iterable_unref0 (type_param_list);
			_vala_code_node_unref0 (sym);
			_vala_code_node_unref0 (type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	if (vala_parser_accept (self, VALA_TOKEN_TYPE_THROWS)) {
		{
			gboolean _tmp45_ = FALSE;
			_tmp45_ = TRUE;
			while (TRUE) {
				ValaDataType* _tmp46_ = NULL;
				ValaDataType* _tmp47_;
				ValaDelegate* _tmp48_;
				if (!_tmp45_) {
					if (!vala_parser_accept (self, VALA_TOKEN_TYPE_COMMA)) {
						break;
					}
				}
				_tmp45_ = FALSE;
				_tmp47_ = vala_parser_parse_type (self, TRUE, FALSE, FALSE, &_inner_error0_);
				_tmp46_ = _tmp47_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_code_node_unref0 (d);
						_vala_iterable_unref0 (type_param_list);
						_vala_code_node_unref0 (sym);
						_vala_code_node_unref0 (type);
						return;
					} else {
						_vala_code_node_unref0 (d);
						_vala_iterable_unref0 (type_param_list);
						_vala_code_node_unref0 (sym);
						_vala_code_node_unref0 (type);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return;
					}
				}
				_tmp48_ = d;
				vala_delegate_add_error_type (_tmp48_, _tmp46_);
				_vala_code_node_unref0 (_tmp46_);
			}
		}
	}
	vala_parser_expect (self, VALA_TOKEN_TYPE_SEMICOLON, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		if (_inner_error0_->domain == VALA_PARSE_ERROR) {
			g_propagate_error (error, _inner_error0_);
			_vala_code_node_unref0 (d);
			_vala_iterable_unref0 (type_param_list);
			_vala_code_node_unref0 (sym);
			_vala_code_node_unref0 (type);
			return;
		} else {
			_vala_code_node_unref0 (d);
			_vala_iterable_unref0 (type_param_list);
			_vala_code_node_unref0 (sym);
			_vala_code_node_unref0 (type);
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
			g_clear_error (&_inner_error0_);
			return;
		}
	}
	_tmp49_ = d;
	_tmp50_ = _vala_code_node_ref0 ((ValaSymbol*) _tmp49_);
	_result_ = _tmp50_;
	while (TRUE) {
		ValaUnresolvedSymbol* _tmp51_;
		ValaUnresolvedSymbol* _tmp52_;
		ValaUnresolvedSymbol* _tmp53_;
		ValaUnresolvedSymbol* _tmp54_;
		ValaUnresolvedSymbol* _tmp55_;
		ValaSymbol* _tmp56_ = NULL;
		ValaUnresolvedSymbol* _tmp57_;
		ValaSymbol* next = NULL;
		ValaSymbol* _tmp66_;
		ValaSymbol* _tmp67_;
		ValaSymbol* _tmp72_;
		ValaSymbol* _tmp73_;
		_tmp51_ = sym;
		if (!(_tmp51_ != NULL)) {
			break;
		}
		_tmp52_ = sym;
		_tmp53_ = vala_unresolved_symbol_get_inner (_tmp52_);
		_tmp54_ = _tmp53_;
		_tmp55_ = _vala_code_node_ref0 (_tmp54_);
		_vala_code_node_unref0 (sym);
		sym = _tmp55_;
		_tmp57_ = sym;
		if (_tmp57_ != NULL) {
			ValaUnresolvedSymbol* _tmp58_;
			const gchar* _tmp59_;
			const gchar* _tmp60_;
			ValaDelegate* _tmp61_;
			ValaSourceReference* _tmp62_;
			ValaSourceReference* _tmp63_;
			ValaNamespace* _tmp64_;
			_tmp58_ = sym;
			_tmp59_ = vala_symbol_get_name ((ValaSymbol*) _tmp58_);
			_tmp60_ = _tmp59_;
			_tmp61_ = d;
			_tmp62_ = vala_code_node_get_source_reference ((ValaCodeNode*) _tmp61_);
			_tmp63_ = _tmp62_;
			_tmp64_ = vala_namespace_new (_tmp60_, _tmp63_);
			_vala_code_node_unref0 (_tmp56_);
			_tmp56_ = (ValaSymbol*) _tmp64_;
		} else {
			ValaSymbol* _tmp65_;
			_tmp65_ = _vala_code_node_ref0 (parent);
			_vala_code_node_unref0 (_tmp56_);
			_tmp56_ = _tmp65_;
		}
		_tmp66_ = _vala_code_node_ref0 (_tmp56_);
		next = _tmp66_;
		_tmp67_ = _result_;
		if (VALA_IS_NAMESPACE (_tmp67_)) {
			ValaSymbol* _tmp68_;
			ValaSymbol* _tmp69_;
			_tmp68_ = next;
			_tmp69_ = _result_;
			vala_symbol_add_namespace (_tmp68_, G_TYPE_CHECK_INSTANCE_CAST (_tmp69_, VALA_TYPE_NAMESPACE, ValaNamespace));
		} else {
			ValaSymbol* _tmp70_;
			ValaSymbol* _tmp71_;
			_tmp70_ = next;
			_tmp71_ = _result_;
			vala_symbol_add_delegate (_tmp70_, G_TYPE_CHECK_INSTANCE_CAST (_tmp71_, VALA_TYPE_DELEGATE, ValaDelegate));
		}
		_tmp72_ = next;
		_tmp73_ = _vala_code_node_ref0 (_tmp72_);
		_vala_code_node_unref0 (_result_);
		_result_ = _tmp73_;
		_vala_code_node_unref0 (next);
		_vala_code_node_unref0 (_tmp56_);
	}
	_vala_code_node_unref0 (_result_);
	_vala_code_node_unref0 (d);
	_vala_iterable_unref0 (type_param_list);
	_vala_code_node_unref0 (sym);
	_vala_code_node_unref0 (type);
}

static ValaList*
vala_parser_parse_type_parameter_list (ValaParser* self,
                                       GError** error)
{
	GError* _inner_error0_ = NULL;
	ValaList* result;
	g_return_val_if_fail (self != NULL, NULL);
	if (vala_parser_accept (self, VALA_TOKEN_TYPE_OP_LT)) {
		ValaArrayList* list = NULL;
		GEqualFunc _tmp0_;
		ValaArrayList* _tmp1_;
		_tmp0_ = g_direct_equal;
		_tmp1_ = vala_array_list_new (VALA_TYPE_TYPEPARAMETER, (GBoxedCopyFunc) vala_code_node_ref, (GDestroyNotify) vala_code_node_unref, _tmp0_);
		list = _tmp1_;
		{
			gboolean _tmp2_ = FALSE;
			_tmp2_ = TRUE;
			while (TRUE) {
				ValaSourceLocation begin = {0};
				ValaSourceLocation _tmp3_ = {0};
				gchar* id = NULL;
				gchar* _tmp4_;
				ValaArrayList* _tmp5_;
				const gchar* _tmp6_;
				ValaSourceLocation _tmp7_;
				ValaSourceReference* _tmp8_;
				ValaSourceReference* _tmp9_;
				ValaTypeParameter* _tmp10_;
				ValaTypeParameter* _tmp11_;
				if (!_tmp2_) {
					if (!vala_parser_accept (self, VALA_TOKEN_TYPE_COMMA)) {
						break;
					}
				}
				_tmp2_ = FALSE;
				vala_parser_get_location (self, &_tmp3_);
				begin = _tmp3_;
				_tmp4_ = vala_parser_parse_identifier (self, FALSE, &_inner_error0_);
				id = _tmp4_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_vala_iterable_unref0 (list);
						return NULL;
					} else {
						_vala_iterable_unref0 (list);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return NULL;
					}
				}
				_tmp5_ = list;
				_tmp6_ = id;
				_tmp7_ = begin;
				_tmp8_ = vala_parser_get_src (self, &_tmp7_);
				_tmp9_ = _tmp8_;
				_tmp10_ = vala_typeparameter_new (_tmp6_, _tmp9_);
				_tmp11_ = _tmp10_;
				vala_collection_add ((ValaCollection*) _tmp5_, _tmp11_);
				_vala_code_node_unref0 (_tmp11_);
				_vala_source_reference_unref0 (_tmp9_);
				_g_free0 (id);
			}
		}
		vala_parser_expect (self, VALA_TOKEN_TYPE_OP_GT, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				_vala_iterable_unref0 (list);
				return NULL;
			} else {
				_vala_iterable_unref0 (list);
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return NULL;
			}
		}
		result = (ValaList*) list;
		return result;
	} else {
		ValaList* _tmp12_;
		ValaList* _tmp15_;
		ValaList* _tmp16_;
		_tmp12_ = vala_parser__empty_type_parameter_list;
		if (_tmp12_ == NULL) {
			GEqualFunc _tmp13_;
			ValaArrayList* _tmp14_;
			_tmp13_ = g_direct_equal;
			_tmp14_ = vala_array_list_new (VALA_TYPE_TYPEPARAMETER, (GBoxedCopyFunc) vala_code_node_ref, (GDestroyNotify) vala_code_node_unref, _tmp13_);
			_vala_iterable_unref0 (vala_parser__empty_type_parameter_list);
			vala_parser__empty_type_parameter_list = (ValaList*) _tmp14_;
		}
		_tmp15_ = vala_parser__empty_type_parameter_list;
		_tmp16_ = _vala_iterable_ref0 (_tmp15_);
		result = _tmp16_;
		return result;
	}
}

static void
vala_parser_skip_type_argument_list (ValaParser* self,
                                     GError** error)
{
	GError* _inner_error0_ = NULL;
	g_return_if_fail (self != NULL);
	if (vala_parser_accept (self, VALA_TOKEN_TYPE_OP_LT)) {
		{
			gboolean _tmp0_ = FALSE;
			_tmp0_ = TRUE;
			while (TRUE) {
				if (!_tmp0_) {
					if (!vala_parser_accept (self, VALA_TOKEN_TYPE_COMMA)) {
						break;
					}
				}
				_tmp0_ = FALSE;
				vala_parser_skip_type (self, &_inner_error0_);
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						return;
					} else {
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return;
					}
				}
			}
		}
		vala_parser_expect (self, VALA_TOKEN_TYPE_OP_GT, &_inner_error0_);
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
			if (_inner_error0_->domain == VALA_PARSE_ERROR) {
				g_propagate_error (error, _inner_error0_);
				return;
			} else {
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
				g_clear_error (&_inner_error0_);
				return;
			}
		}
	}
}

static ValaList*
vala_parser_parse_type_argument_list (ValaParser* self,
                                      gboolean maybe_expression,
                                      GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	GError* _inner_error0_ = NULL;
	ValaList* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	if (vala_parser_accept (self, VALA_TOKEN_TYPE_OP_LT)) {
		ValaArrayList* list = NULL;
		GEqualFunc _tmp1_;
		ValaArrayList* _tmp2_;
		_tmp1_ = g_direct_equal;
		_tmp2_ = vala_array_list_new (VALA_TYPE_DATA_TYPE, (GBoxedCopyFunc) vala_code_node_ref, (GDestroyNotify) vala_code_node_unref, _tmp1_);
		list = _tmp2_;
		{
			gboolean _tmp3_ = FALSE;
			_tmp3_ = TRUE;
			while (TRUE) {
				if (!_tmp3_) {
					if (!vala_parser_accept (self, VALA_TOKEN_TYPE_COMMA)) {
						break;
					}
				}
				_tmp3_ = FALSE;
				switch (vala_parser_current (self)) {
					case VALA_TOKEN_TYPE_VOID:
					case VALA_TOKEN_TYPE_DYNAMIC:
					case VALA_TOKEN_TYPE_UNOWNED:
					case VALA_TOKEN_TYPE_WEAK:
					case VALA_TOKEN_TYPE_IDENTIFIER:
					{
						ValaDataType* type = NULL;
						ValaDataType* _tmp4_;
						ValaArrayList* _tmp5_;
						ValaDataType* _tmp6_;
						_tmp4_ = vala_parser_parse_type (self, TRUE, TRUE, FALSE, &_inner_error0_);
						type = _tmp4_;
						if (G_UNLIKELY (_inner_error0_ != NULL)) {
							if (_inner_error0_->domain == VALA_PARSE_ERROR) {
								g_propagate_error (error, _inner_error0_);
								_vala_iterable_unref0 (list);
								return NULL;
							} else {
								_vala_iterable_unref0 (list);
								g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
								g_clear_error (&_inner_error0_);
								return NULL;
							}
						}
						_tmp5_ = list;
						_tmp6_ = type;
						vala_collection_add ((ValaCollection*) _tmp5_, _tmp6_);
						_vala_code_node_unref0 (type);
						break;
					}
					default:
					{
						ValaSourceLocation _tmp7_;
						_tmp7_ = begin;
						vala_parser_rollback (self, &_tmp7_);
						result = NULL;
						_vala_iterable_unref0 (list);
						return result;
					}
				}
			}
		}
		if (!vala_parser_accept (self, VALA_TOKEN_TYPE_OP_GT)) {
			ValaSourceLocation _tmp8_;
			_tmp8_ = begin;
			vala_parser_rollback (self, &_tmp8_);
			result = NULL;
			_vala_iterable_unref0 (list);
			return result;
		}
		if (maybe_expression) {
			switch (vala_parser_current (self)) {
				case VALA_TOKEN_TYPE_OPEN_PARENS:
				case VALA_TOKEN_TYPE_CLOSE_PARENS:
				case VALA_TOKEN_TYPE_CLOSE_BRACKET:
				case VALA_TOKEN_TYPE_OPEN_BRACE:
				case VALA_TOKEN_TYPE_COLON:
				case VALA_TOKEN_TYPE_SEMICOLON:
				case VALA_TOKEN_TYPE_COMMA:
				case VALA_TOKEN_TYPE_DOT:
				case VALA_TOKEN_TYPE_INTERR:
				case VALA_TOKEN_TYPE_OP_EQ:
				case VALA_TOKEN_TYPE_OP_NE:
				{
					break;
				}
				default:
				{
					ValaSourceLocation _tmp9_;
					_tmp9_ = begin;
					vala_parser_rollback (self, &_tmp9_);
					result = NULL;
					_vala_iterable_unref0 (list);
					return result;
				}
			}
		}
		result = (ValaList*) list;
		return result;
	}
	result = NULL;
	return result;
}

static ValaMemberAccess*
vala_parser_parse_member_name (ValaParser* self,
                               ValaExpression* base_expr,
                               GError** error)
{
	ValaSourceLocation begin = {0};
	ValaSourceLocation _tmp0_ = {0};
	ValaMemberAccess* expr = NULL;
	gboolean first = FALSE;
	GError* _inner_error0_ = NULL;
	ValaMemberAccess* result;
	g_return_val_if_fail (self != NULL, NULL);
	vala_parser_get_location (self, &_tmp0_);
	begin = _tmp0_;
	expr = NULL;
	first = TRUE;
	{
		gboolean _tmp1_ = FALSE;
		_tmp1_ = TRUE;
		while (TRUE) {
			gchar* id = NULL;
			gchar* _tmp2_;
			gboolean qualified = FALSE;
			gboolean _tmp3_ = FALSE;
			gboolean _tmp4_ = FALSE;
			ValaList* type_arg_list = NULL;
			ValaList* _tmp9_;
			ValaExpression* _tmp10_ = NULL;
			ValaMemberAccess* _tmp11_;
			const gchar* _tmp13_;
			ValaSourceLocation _tmp14_;
			ValaSourceReference* _tmp15_;
			ValaSourceReference* _tmp16_;
			ValaMemberAccess* _tmp17_;
			ValaMemberAccess* _tmp18_;
			ValaList* _tmp19_;
			if (!_tmp1_) {
				if (!vala_parser_accept (self, VALA_TOKEN_TYPE_DOT)) {
					break;
				}
			}
			_tmp1_ = FALSE;
			_tmp2_ = vala_parser_parse_identifier (self, FALSE, &_inner_error0_);
			id = _tmp2_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_vala_code_node_unref0 (expr);
					return NULL;
				} else {
					_vala_code_node_unref0 (expr);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			qualified = FALSE;
			if (first) {
				const gchar* _tmp5_;
				_tmp5_ = id;
				_tmp4_ = g_strcmp0 (_tmp5_, "global") == 0;
			} else {
				_tmp4_ = FALSE;
			}
			if (_tmp4_) {
				_tmp3_ = vala_parser_accept (self, VALA_TOKEN_TYPE_DOUBLE_COLON);
			} else {
				_tmp3_ = FALSE;
			}
			if (_tmp3_) {
				gchar* _tmp6_ = NULL;
				gchar* _tmp7_;
				gchar* _tmp8_;
				_tmp7_ = vala_parser_parse_identifier (self, FALSE, &_inner_error0_);
				_tmp6_ = _tmp7_;
				if (G_UNLIKELY (_inner_error0_ != NULL)) {
					if (_inner_error0_->domain == VALA_PARSE_ERROR) {
						g_propagate_error (error, _inner_error0_);
						_g_free0 (id);
						_vala_code_node_unref0 (expr);
						return NULL;
					} else {
						_g_free0 (id);
						_vala_code_node_unref0 (expr);
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
						g_clear_error (&_inner_error0_);
						return NULL;
					}
				}
				_tmp8_ = _tmp6_;
				_tmp6_ = NULL;
				_g_free0 (id);
				id = _tmp8_;
				qualified = TRUE;
				_g_free0 (_tmp6_);
			}
			_tmp9_ = vala_parser_parse_type_argument_list (self, FALSE, &_inner_error0_);
			type_arg_list = _tmp9_;
			if (G_UNLIKELY (_inner_error0_ != NULL)) {
				if (_inner_error0_->domain == VALA_PARSE_ERROR) {
					g_propagate_error (error, _inner_error0_);
					_g_free0 (id);
					_vala_code_node_unref0 (expr);
					return NULL;
				} else {
					_g_free0 (id);
					_vala_code_node_unref0 (expr);
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
					g_clear_error (&_inner_error0_);
					return NULL;
				}
			}
			_tmp11_ = expr;
			if (_tmp11_ != NULL) {
				ValaMemberAccess* _tmp12_;
				_tmp12_ = expr;
				_tmp10_ = (ValaExpression*) _tmp12_;
			} else {
				_tmp10_ = base_expr;
			}
			_tmp13_ = id;
			_tmp14_ = begin;
			_tmp15_ = vala_parser_get_src (self, &_tmp14_);
			_tmp16_ = _tmp15_;
			_tmp17_ = vala_member_access_new (_tmp10_, _tmp13_, _tmp16_);
			_vala_code_node_unref0 (expr);
			expr = _tmp17_;
			_vala_source_reference_unref0 (_tmp16_);
			_tmp18_ = expr;
			vala_member_access_set_qualified (_tmp18_, qualified);
			_tmp19_ = type_arg_list;
			if (_tmp19_ != NULL) {
				{
					ValaList* _type_arg_list = NULL;
					ValaList* _tmp20_;
					gint _type_arg_size = 0;
					ValaList* _tmp21_;
					gint _tmp22_;
					gint _tmp23_;
					gint _type_arg_index = 0;
					_tmp20_ = type_arg_list;
					_type_arg_list = _tmp20_;
					_tmp21_ = _type_arg_list;
					_tmp22_ = vala_collection_get_size ((ValaCollection*) _tmp21_);
					_tmp23_ = _tmp22_;
					_type_arg_size = _tmp23_;
					_type_arg_index = -1;
					while (TRUE) {
						gint _tmp24_;
						gint _tmp25_;
						ValaDataType* type_arg = NULL;
						ValaList* _tmp26_;
						gpointer _tmp27_;
						ValaMemberAccess* _tmp28_;
						ValaDataType* _tmp29_;
						_type_arg_index = _type_arg_index + 1;
						_tmp24_ = _type_arg_index;
						_tmp25_ = _type_arg_size;
						if (!(_tmp24_ < _tmp25_)) {
							break;
						}
						_tmp26_ = _type_arg_list;
						_tmp27_ = vala_list_get (_tmp26_, _type_arg_index);
						type_arg = (ValaDataType*) _tmp27_;
						_tmp28_ = expr;
						_tmp29_ = type_arg;
						vala_member_access_add_type_argument (_tmp28_, _tmp29_);
						_vala_code_node_unref0 (type_arg);
					}
				}
			}
			first = FALSE;
			_vala_iterable_unref0 (type_arg_list);
			_g_free0 (id);
		}
	}
	result = expr;
	return result;
}

static gboolean
vala_parser_is_declaration_keyword (ValaParser* self,
                                    ValaTokenType type)
{
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	switch (type) {
		case VALA_TOKEN_TYPE_ABSTRACT:
		case VALA_TOKEN_TYPE_ASYNC:
		case VALA_TOKEN_TYPE_CLASS:
		case VALA_TOKEN_TYPE_CONST:
		case VALA_TOKEN_TYPE_DELEGATE:
		case VALA_TOKEN_TYPE_ENUM:
		case VALA_TOKEN_TYPE_ERRORDOMAIN:
		case VALA_TOKEN_TYPE_EXTERN:
		case VALA_TOKEN_TYPE_INLINE:
		case VALA_TOKEN_TYPE_INTERFACE:
		case VALA_TOKEN_TYPE_INTERNAL:
		case VALA_TOKEN_TYPE_NAMESPACE:
		case VALA_TOKEN_TYPE_NEW:
		case VALA_TOKEN_TYPE_OVERRIDE:
		case VALA_TOKEN_TYPE_PARTIAL:
		case VALA_TOKEN_TYPE_PRIVATE:
		case VALA_TOKEN_TYPE_PROTECTED:
		case VALA_TOKEN_TYPE_PUBLIC:
		case VALA_TOKEN_TYPE_SEALED:
		case VALA_TOKEN_TYPE_SIGNAL:
		case VALA_TOKEN_TYPE_STATIC:
		case VALA_TOKEN_TYPE_STRUCT:
		case VALA_TOKEN_TYPE_VIRTUAL:
		case VALA_TOKEN_TYPE_VOLATILE:
		{
			result = TRUE;
			return result;
		}
		default:
		{
			result = FALSE;
			return result;
		}
	}
}

static ValaParserTokenInfo*
vala_parser_token_info_dup (const ValaParserTokenInfo* self)
{
	ValaParserTokenInfo* dup;
	dup = g_new0 (ValaParserTokenInfo, 1);
	memcpy (dup, self, sizeof (ValaParserTokenInfo));
	return dup;
}

static void
vala_parser_token_info_free (ValaParserTokenInfo* self)
{
	g_free (self);
}

static GType
vala_parser_token_info_get_type_once (void)
{
	GType vala_parser_token_info_type_id;
	vala_parser_token_info_type_id = g_boxed_type_register_static ("ValaParserTokenInfo", (GBoxedCopyFunc) vala_parser_token_info_dup, (GBoxedFreeFunc) vala_parser_token_info_free);
	return vala_parser_token_info_type_id;
}

static GType
vala_parser_token_info_get_type (void)
{
	static volatile gsize vala_parser_token_info_type_id__once = 0;
	if (g_once_init_enter (&vala_parser_token_info_type_id__once)) {
		GType vala_parser_token_info_type_id;
		vala_parser_token_info_type_id = vala_parser_token_info_get_type_once ();
		g_once_init_leave (&vala_parser_token_info_type_id__once, vala_parser_token_info_type_id);
	}
	return vala_parser_token_info_type_id__once;
}

static void
vala_parser_partial_info_copy (const ValaParserPartialInfo* self,
                               ValaParserPartialInfo* dest)
{
	ValaSymbol* _tmp0_;
	ValaSymbol* _tmp1_;
	ValaSourceLocation _tmp2_;
	ValaList* _tmp3_;
	ValaList* _tmp4_;
	_tmp0_ = (*self).parent;
	_tmp1_ = _vala_code_node_ref0 (_tmp0_);
	_vala_code_node_unref0 ((*dest).parent);
	(*dest).parent = _tmp1_;
	_tmp2_ = (*self).begin;
	(*dest).begin = _tmp2_;
	_tmp3_ = (*self).attributes;
	_tmp4_ = _vala_iterable_ref0 (_tmp3_);
	_vala_iterable_unref0 ((*dest).attributes);
	(*dest).attributes = _tmp4_;
}

static void
vala_parser_partial_info_destroy (ValaParserPartialInfo* self)
{
	_vala_code_node_unref0 ((*self).parent);
	_vala_iterable_unref0 ((*self).attributes);
}

static ValaParserPartialInfo*
vala_parser_partial_info_dup (const ValaParserPartialInfo* self)
{
	ValaParserPartialInfo* dup;
	dup = g_new0 (ValaParserPartialInfo, 1);
	vala_parser_partial_info_copy (self, dup);
	return dup;
}

static void
vala_parser_partial_info_free (ValaParserPartialInfo* self)
{
	vala_parser_partial_info_destroy (self);
	g_free (self);
}

static GType
vala_parser_partial_info_get_type_once (void)
{
	GType vala_parser_partial_info_type_id;
	vala_parser_partial_info_type_id = g_boxed_type_register_static ("ValaParserPartialInfo", (GBoxedCopyFunc) vala_parser_partial_info_dup, (GBoxedFreeFunc) vala_parser_partial_info_free);
	return vala_parser_partial_info_type_id;
}

static GType
vala_parser_partial_info_get_type (void)
{
	static volatile gsize vala_parser_partial_info_type_id__once = 0;
	if (g_once_init_enter (&vala_parser_partial_info_type_id__once)) {
		GType vala_parser_partial_info_type_id;
		vala_parser_partial_info_type_id = vala_parser_partial_info_get_type_once ();
		g_once_init_leave (&vala_parser_partial_info_type_id__once, vala_parser_partial_info_type_id);
	}
	return vala_parser_partial_info_type_id__once;
}

static void
vala_parser_class_init (ValaParserClass * klass,
                        gpointer klass_data)
{
	vala_parser_parent_class = g_type_class_peek_parent (klass);
	((ValaCodeVisitorClass *) klass)->finalize = vala_parser_finalize;
	g_type_class_adjust_private_offset (klass, &ValaParser_private_offset);
	((ValaCodeVisitorClass *) klass)->visit_source_file = (void (*) (ValaCodeVisitor*, ValaSourceFile*)) vala_parser_real_visit_source_file;
}

static void
vala_parser_instance_init (ValaParser * self,
                           gpointer klass)
{
	self->priv = vala_parser_get_instance_private (self);
}

static void
vala_parser_finalize (ValaCodeVisitor * obj)
{
	ValaParser * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, VALA_TYPE_PARSER, ValaParser);
	_vala_scanner_unref0 (self->priv->scanner);
	_vala_code_context_unref0 (self->priv->context);
	self->priv->tokens = (g_free (self->priv->tokens), NULL);
	_vala_comment_unref0 (self->priv->comment);
	self->priv->partials = (_vala_ValaParserPartialInfo_array_free (self->priv->partials, self->priv->partials_length1), NULL);
	VALA_CODE_VISITOR_CLASS (vala_parser_parent_class)->finalize (obj);
}

/**
 * Code visitor parsing all Vala source files.
 */
static GType
vala_parser_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (ValaParserClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_parser_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaParser), 0, (GInstanceInitFunc) vala_parser_instance_init, NULL };
	GType vala_parser_type_id;
	vala_parser_type_id = g_type_register_static (VALA_TYPE_CODE_VISITOR, "ValaParser", &g_define_type_info, 0);
	ValaParser_private_offset = g_type_add_instance_private (vala_parser_type_id, sizeof (ValaParserPrivate));
	return vala_parser_type_id;
}

GType
vala_parser_get_type (void)
{
	static volatile gsize vala_parser_type_id__once = 0;
	if (g_once_init_enter (&vala_parser_type_id__once)) {
		GType vala_parser_type_id;
		vala_parser_type_id = vala_parser_get_type_once ();
		g_once_init_leave (&vala_parser_type_id__once, vala_parser_type_id);
	}
	return vala_parser_type_id__once;
}

GQuark
vala_parse_error_quark (void)
{
	return g_quark_from_static_string ("vala-parse-error-quark");
}

static GType
vala_parse_error_get_type_once (void)
{
	static const GEnumValue values[] = {{VALA_PARSE_ERROR_FAILED, "VALA_PARSE_ERROR_FAILED", "failed"}, {VALA_PARSE_ERROR_SYNTAX, "VALA_PARSE_ERROR_SYNTAX", "syntax"}, {0, NULL, NULL}};
	GType vala_parse_error_type_id;
	vala_parse_error_type_id = g_enum_register_static ("ValaParseError", values);
	return vala_parse_error_type_id;
}

GType
vala_parse_error_get_type (void)
{
	static volatile gsize vala_parse_error_type_id__once = 0;
	if (g_once_init_enter (&vala_parse_error_type_id__once)) {
		GType vala_parse_error_type_id;
		vala_parse_error_type_id = vala_parse_error_get_type_once ();
		g_once_init_leave (&vala_parse_error_type_id__once, vala_parse_error_type_id);
	}
	return vala_parse_error_type_id__once;
}

static void
_vala_array_destroy (gpointer array,
                     gssize array_length,
                     GDestroyNotify destroy_func)
{
	if ((array != NULL) && (destroy_func != NULL)) {
		gssize i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}

static void
_vala_array_free (gpointer array,
                  gssize array_length,
                  GDestroyNotify destroy_func)
{
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}

